/*=============================================================================
        File: barray.hh
     Purpose:      
    Revision: $Id: barray.hh,v 1.2 1998/04/07 17:39:09 lavoie Exp $
  Created by: Philippe Lavoie          (3 Oct, 1996)
 Modified by: 

 Copyright notice:
          Copyright (C) 1996-1998 Philippe Lavoie
 
	  This library is free software; you can redistribute it and/or
	  modify it under the terms of the GNU Library General Public
	  License as published by the Free Software Foundation; either
	  version 2 of the License, or (at your option) any later version.
 
	  This library is distributed in the hope that it will be useful,
	  but WITHOUT ANY WARRANTY; without even the implied warranty of
	  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
	  Library General Public License for more details.
 
	  You should have received a copy of the GNU Library General Public
	  License along with this library; if not, write to the Free
	  Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
=============================================================================*/

#ifndef _MATRIX_barray_h_
#define _MATRIX_barray_h_

#include <iostream.h>
//#include <fstream.h>
#include "specialType.hh"
#include "list.hh"

// Predefining every friend functions
// This is required by latest ISO C++ draft

template <class T> class BasicArray ;
template <class T> int operator!=(const BasicArray<T>&,const BasicArray<T>&);
template <class T> int operator==(const BasicArray<T>&,const BasicArray<T>&);
template <class T> istream& operator>>(istream& is, BasicArray<T>& arry);
template <class T> ostream& operator<<(ostream& os, const BasicArray<T>& arry);

/**  **  **  **  **  **  **  **  **  **  **  **  **  **  **  **  **  **  **
        Class: BasicArray --- A basic templated array class
  Description: This is a basis array class, the only particularity is that 
               the resize is {\em not} destructive. 

   Created by: Philippe Lavoie (4 Oct. 1996)
  Modified by:
 **  **  **  **  **  **  **  **  **  **  **  **  **  **  **  **  **  **  **/
template<class T> class BasicArray
{
public:
  const int& n; // the perceived size of the array
  BasicArray();
  BasicArray(const int ni);
  BasicArray(const BasicArray<T>& f2);
  BasicArray(T* ap, const int size) ;  
  BasicArray(BasicList<T>& list) ;
  virtual ~BasicArray();
  
  BasicArray<T>& operator= (const BasicArray<T>& f2);
  
  virtual int size() const { return sze; } // returns the size of the array
  virtual void resize(const int nsize);
  void resize(const BasicArray<T>& A) { resize(A.n); } // resize the array with the same dimension of the vector A
  
  
  void freeze() { resize( n ); } // fix the actual size to the current size
  void trim(const int nsize);    
  void untrim()		{ sze = rsize; } // sets the array to the maximal allocated size
  
  
  virtual void reset(const T val = 0.0);
  T operator=(const T val) { reset(val); return val; } // set all elements of the vector to val
  
#ifdef DEBUG_MATRIX
  virtual T& operator[](const int i) ; // checks that i is in the proper range and returns x[i]
  virtual T  operator[](const int i) const ; // checks that i is in the proper range and returns x[i]
#else
  virtual T& operator[](const int i) { return x[i]; } // no range checks are performed
  virtual T  operator[](const int i) const   { return x[i]; } // no range checks are performed 
#endif
  T* memory() const { return x ; } // returns the data pointer 

  void width(const int w) { wdth = w; }   // set output columns

  // friend functions
#ifdef ANSI_ISO_CPP
  friend int operator!= <>(const BasicArray<T>&,const BasicArray<T>&);    
  friend int operator== <>(const BasicArray<T>&,const BasicArray<T>&);      
  friend istream& operator>> <>(istream& is, BasicArray<T>& arry);
  friend ostream& operator<< <>(ostream& os, const BasicArray<T>& arry);
#else
  friend int operator!=(const BasicArray<T>&,const BasicArray<T>&);    
  friend int operator==(const BasicArray<T>&,const BasicArray<T>&);      
  friend istream& operator>>(istream& is, BasicArray<T>& arry);
  friend ostream& operator<<(ostream& os, const BasicArray<T>& arry);
#endif


private:
  int rsize;				// the actual size of the array
  int wdth;
  int destruct ;
protected:
  int sze;				// the known size of the array
  T *x;                       // the data pointer
};


typedef BasicArray<int> BasicArray_INT ;            
typedef BasicArray<char> BasicArray_BYTE ;          
typedef BasicArray<double> BasicArray_DOUBLE ;      
typedef BasicArray<float> BasicArray_FLOAT ;      
typedef BasicArray<Complex> BasicArray_COMPLEX ;    
typedef BasicArray<unsigned char> BasicArray_UBYTE ;
typedef BasicArray<Vector2> BasicArray_Vector2 ;
typedef BasicArray<CtrlPnt4D> BasicArray_CtrlPnt4D ;
typedef BasicArray<Point3D> BasicArray_Point3D ;
typedef BasicArray<Color> BasicArray_Color ;
typedef BasicArray<void*> BasicArray_VoidPtr ;
typedef BasicArray<Coordinate> BasicArray_Coordinate ;


#if defined(__sgi ) && !defined(__GNUG__)

// The SGI compiler needs the special definition in the header, not in
// the source file.
// no one knows why, but at least now it works.

void BasicArray<CtrlPnt4D>::resize(const int nsize) {
  int k; 
  
  if ( nsize == rsize ){
    sze = nsize ;
    return;			// nothing to do
  }
  
  if(sze>nsize){
    sze = nsize ;
    return ;
  }

  if((sze<nsize) && (nsize<rsize)){
    for(k=sze;k<nsize;++k)
      x[k] = CtrlPnt4D(0) ;
    sze = nsize ;
    return ;
  }

  CtrlPnt4D *xn ; 
  CtrlPnt4D *p,*pn ;

  xn = new CtrlPnt4D[nsize];
  p = x ;
  pn = xn ;
  
  // we want to have all the data to be in a consecutive memory segment
  float *dn ;

  dn = new float[nsize*4] ;

  for(k=0;k<nsize;++k){
    if(xn[k].created && xn[k].data){
      delete []xn[k].data ;
    }
    xn[k].created = 0; 
    xn[k].data = &dn[k*4] ;
  }
  if(nsize>0)
    xn[0].created = 1 ;

  if ( x )    {
      // copy the old data
      for (k = 0; k < sze; ++k){
	*pn = *p ;
	++pn ; 
	++p ;
      }
      for(k=sze;k<nsize;++k){
	*pn = CtrlPnt4D(0) ;
	++pn ;
      }
      if(sze>0){
	delete []x;
      }
    }
  else
    for (k = 0; k < nsize; ++k){
      *pn = CtrlPnt4D(0) ;
      ++pn ;
    }
  
  rsize = nsize;
  sze = rsize ;
  x = xn;
  
  wdth = rsize + 1; 
}

#endif


#ifdef CPP_SRC_INCLUDE
#include "../src/barray.cc"
#endif

#endif 
