/*=============================================================================
        File: matrix.hh
     Purpose:       
    Revision: $Id: matrix.hh,v 1.2 1998/06/03 14:21:31 lavoie Exp $
  Created by: Philippe Lavoie          (3 Oct, 1996)
 Modified by: 

 Copyright notice:
          Copyright (C) 1996-1998 Philippe Lavoie
 
	  This library is free software; you can redistribute it and/or
	  modify it under the terms of the GNU Library General Public
	  License as published by the Free Software Foundation; either
	  version 2 of the License, or (at your option) any later version.
 
	  This library is distributed in the hope that it will be useful,
	  but WITHOUT ANY WARRANTY; without even the implied warranty of
	  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
	  Library General Public License for more details.
 
	  You should have received a copy of the GNU Library General Public
	  License along with this library; if not, write to the Free
	  Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
=============================================================================*/


#ifndef _Matrix_matrix_h_
#define _Matrix_matrix_h_


#include <iostream.h>
#include "internals.hh"
#include "barray2d.hh"
#include "vector.hh"


// Predefining every friend functions 
// This is required by latest ISO C++ draft

template <class T> class PlMatrix ;
template <class T> PlMatrix<T> operator+(const PlMatrix<T>&,const PlMatrix<T>&);
template <class T> PlMatrix<T> operator-(const PlMatrix<T>&,const PlMatrix<T>&);
template <class T> PlMatrix<T> operator*(const PlMatrix<T>&,const PlMatrix<T>&);
template <class T> PlMatrix<T> operator*(const double,const PlMatrix<T>&);
template <class T> PlMatrix<T> operator*(const Complex&,const PlMatrix<T>&);
template <class T> PlVector<T> operator*(const PlMatrix<T>&,const PlVector<T>&);
template <class T> int operator==(const PlMatrix<T>&,const PlMatrix<T>&);
template <class T> int operator!=(const PlMatrix<T>& a,const PlMatrix<T>& b) ;


/**  **  **  **  **  **  **  **  **  **  **  **  **  **  **  **  **  **  **
        Class: PlMatrix --- A templated matrix class with basic mathematical 
                          operators
  Description: This is a matrix class which has basic mathematical operators 
               and some routines for input/output.
   Created by: Philippe Lavoie (4 Oct. 1996)
  Modified by:
 **  **  **  **  **  **  **  **  **  **  **  **  **  **  **  **  **  **  **/
template<class T>
class PlMatrix : public Basic2DArray<T>
{
public:
  PlMatrix(const int r,const int c) : Basic2DArray<T>(r,c) {}
  PlMatrix() : Basic2DArray<T>() {}
  PlMatrix(const PlMatrix<T>& M) : Basic2DArray<T>(M) {}
  PlMatrix(T* p, const int r, const int c) : Basic2DArray<T>(p,r,c) {}
  //~Matrix() {}	
  
  PlMatrix<T>&	operator=(const PlMatrix<T>&);	
  T operator=(const T v) 
    { reset((T)0); 
      diag(v);
      return v; }
  void submatrix(int i, int j, PlMatrix<T>&); 
  void as(int rw, int cl, PlMatrix<T>&) ;
  PlMatrix<T> get(int rw, int cl, int nr, int nc) const ;
  
  // Mathematical functions
  PlMatrix<T>& operator+=(const PlMatrix<T>&);
  PlMatrix<T>& operator-=(const PlMatrix<T>&);
  PlMatrix<T>& operator+=(double d) ;
  PlMatrix<T>& operator-=(double d) ;
  PlMatrix<T>& operator*=(double d) ;
  PlMatrix<T>& operator/=(double d) ;

#ifdef ANSI_ISO_CPP
  friend PlMatrix<T> operator+ <>(const PlMatrix<T>&,
				const PlMatrix<T>&);
  friend PlMatrix<T> operator- <>(const PlMatrix<T>&,
				  const PlMatrix<T>&);
  friend PlMatrix<T> operator* <>(const PlMatrix<T>&,
				  const PlMatrix<T>&);
  friend PlMatrix<T> operator* <>(const double,
				  const PlMatrix<T>&);
  friend PlMatrix<T> operator* <>(const Complex&,
				  const PlMatrix<T>&);
  friend PlVector<T> operator* <>(const PlMatrix<T>&,
				  const PlVector<T>&);
  friend int operator== <>(const PlMatrix<T>&,
			   const PlMatrix<T>&);
  friend int operator!= <>(const PlMatrix<T>& a,
			   const PlMatrix<T>& b) ;
#else
  friend PlMatrix<T> operator+(const PlMatrix<T>&,
			     const PlMatrix<T>&);
  friend PlMatrix<T> operator-(const PlMatrix<T>&,
			       const PlMatrix<T>&);
  friend PlMatrix<T> operator*(const PlMatrix<T>&,
			       const PlMatrix<T>&);
  friend PlMatrix<T> operator*(const double,
			       const PlMatrix<T>&);
  friend PlMatrix<T> operator*(const Complex&,
			       const PlMatrix<T>&);
  friend PlVector<T> operator*(const PlMatrix<T>&,
			       const PlVector<T>&);
  friend int operator==(const PlMatrix<T>&,
			const PlMatrix<T>&);
  friend int operator!=(const PlMatrix<T>& a,
			const PlMatrix<T>& b) ;
#endif  

  PlMatrix<T> herm() const ;
  PlMatrix<T> transpose() const ;
  T trace() const ;
  
  double norm(void) ;
  void diag(const T fv);
  PlVector<T> getDiag(); 
  
  void qSort() ;
  
  
  // file i/o functions
  int read(char* filename) ; 
  int read(char* filename, int rows, int cols) ;
  int write(char* filename) ; 
  int writeRaw(char* filename) ; 
  
  // calls to LAPACK functions are defined in this class
  friend class LAPACK ;
};

template <class T>
PlMatrix<T> comm( const PlMatrix<T>& a, const PlMatrix<T>& b);

template <class T>
PlMatrix<T> herm( const PlMatrix<T>& a) {
  return a.herm() ;
}

template <class T>
PlMatrix<T> transpose( const PlMatrix<T>& a) {
  return a.transpose() ;
}

template <class T>
T trace( const PlMatrix<T>& a) {
  return a.trace() ;
}

template <class T>
int operator!=(const PlMatrix<T>& a, const PlMatrix<T>& b) {
  return a==b?0:1 ;
}


typedef PlMatrix<int> Matrix_INT ;
typedef PlMatrix<char> Matrix_BYTE ;
typedef PlMatrix<float> Matrix_FLOAT ;
typedef PlMatrix<double> Matrix_DOUBLE ;
typedef PlMatrix<Complex> Matrix_COMPLEX ;
typedef PlMatrix<unsigned char> Matrix_UBYTE ;
typedef PlMatrix<Vector2> Matrix_Vector2 ;
typedef PlMatrix<Point3D> Matrix_Point3D ;
typedef PlMatrix<CtrlPnt4D> Matrix_CtrlPnt4D ;
typedef PlMatrix<Color> Matrix_Color ;

typedef PlMatrix<int> PlMatrix_int ;
typedef PlMatrix<char> PlMatrix_byte ;
typedef PlMatrix<float> PlMatrix_float ;
typedef PlMatrix<double> PlMatrix_double ;
typedef PlMatrix<Complex> PlMatrix_complex ;
typedef PlMatrix<unsigned char> PlMatrix_ubyte ;
typedef PlMatrix<Vector2> PlMatrix_Vector2 ;
typedef PlMatrix<Point3D> PlMatrix_Point3D ;
typedef PlMatrix<CtrlPnt4D> PlMatrix_CtrlPnt4D ;
typedef PlMatrix<Color> PlMatrix_Color ;

#ifdef CPP_SRC_INCLUDE
#include "../src/matrix.cc"
#endif

#endif 
