/*============================================================================
        File: mlapack.hh
     Purpose:       
    Revision: $Id: mlapack.hh,v 1.1.1.1 1998/04/06 21:02:31 lavoie Exp $
  Created by: Philippe Lavoie          (3 Oct, 1996)
 Modified by: 

 Copyright notice:
          Copyright (C) 1996-1997 Philippe Lavoie
 
	  This library is free software; you can redistribute it and/or
	  modify it under the terms of the GNU Library General Public
	  License as published by the Free Software Foundation; either
	  version 2 of the License, or (at your option) any later version.
 
	  This library is distributed in the hope that it will be useful,
	  but WITHOUT ANY WARRANTY; without even the implied warranty of
	  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
	  Library General Public License for more details.
 
	  You should have received a copy of the GNU Library General Public
	  License along with this library; if not, write to the Free
	  Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
=============================================================================*/

#ifndef _Matrix_mlapack_h_
#define _Matrix_mlapack_h_


#include "matrix.hh"

#ifdef WITH_LAPACK
/**  **  **  **  **  **  **  **  **  **  **  **  **  **  **  **  **  **  **
        Class: LAPACK --- a class with a limited interface to LAPACK
  Description: This class is a wrapper for some of the LAPACK routine. 

               The class is already instantiated with the variable 
	       {\tt LAPack}. Thus all calls to the LAPACK functionnality 
	       should be made through that variable. {\em i.e.} use 
	       {\tt LAPack.svd(A,U,S,V) } to call the svd function.

   Created by: Philippe Lavoie (4 Oct. 1996)
  Modified by:
 **  **  **  **  **  **  **  **  **  **  **  **  **  **  **  **  **  **  **/
class LAPACK {
public:
  int last_error ; // the value of the last error message
  int print_error ; // a flag specifying if the error message should be printed
  LAPACK(int printE=0) { print_error = printE ; }

  int svd(const Matrix<double> &A, 
	  Matrix<double> &U, 
	  Matrix<double> &S, 
	  Matrix<double> &V);
  int lse(const Matrix<double> &A, 
	  const Matrix<double> &B, 
	  Matrix<double> &X);
  Matrix_DOUBLE inverse(const Matrix_DOUBLE& A);
};

extern LAPACK LAPack ;

// Please set the following definitions to the way your LAPACK library is set

#ifndef name2
#include <generic.h>
#endif

#define F77NAME(x) name2(x,_)

#define DGETRI F77NAME(dgetri)
#define DGETRF F77NAME(dgetrf)
#define DGESVD F77NAME(dgesvd)
#define DGELS F77NAME(dgels)

extern "C" {
    void DGETRI(long int *, double *, long int *, long int *, double *, long int *, long int *) ;
    void DGETRF(long int *m, long int *n, double *A, long int *lda, long int *ipiv,long int *info) ;
    void DGESVD(char *, char *, long int *, long int *, double *, long int *, double *, double *, long int *, double *, long int *, double *, long int *, long int *) ;
    void DGELS(char *trans, long int *m, long int *n, long int *nrhs, double *A,long int *lda, double *B, long int *ldb, double *work, long int *lwork, long int *info) ;
}

#endif // WITH_LAPACK

#endif 
