/*=============================================================================
        File: vector.hh
     Purpose:       
    Revision: $Id: vector.hh,v 1.1.1.1 1998/04/06 21:02:31 lavoie Exp $
  Created by: Philippe Lavoie          (3 Oct, 1996)
 Modified by: 

 Copyright notice:
          Copyright (C) 1996-1998 Philippe Lavoie
 
	  This library is free software; you can redistribute it and/or
	  modify it under the terms of the GNU Library General Public
	  License as published by the Free Software Foundation; either
	  version 2 of the License, or (at your option) any later version.
 
	  This library is distributed in the hope that it will be useful,
	  but WITHOUT ANY WARRANTY; without even the implied warranty of
	  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
	  Library General Public License for more details.
 
	  You should have received a copy of the GNU Library General Public
	  License along with this library; if not, write to the Free
	  Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
=============================================================================*/

#ifndef _Matrix_vector_h_
#define _Matrix_vector_h_

#include <math.h>
#include "barray.hh"
#include "specialType.hh"

// Predefining every friend functions 
// This is required by latest ISO C++ draft

template <class T> class PlVector ;
template <class T> PlVector<T> operator+(const PlVector<T>&, const PlVector<T>&);
template <class T> PlVector<T> operator-(const PlVector<T>&, const PlVector<T>&);
template <class T> T operator*(const PlVector<T>&,const PlVector<T>&); 

template <class T> PlVector<T> operator*(const PlVector<T>& v, const double d);
template <class T> PlVector<T> operator*(const PlVector<T>& v, const Complex d);
		    
template <class T> PlVector<T> operator*(const double d,const PlVector<T>& v) ;
template <class T> PlVector<T> operator*(const Complex d,const PlVector<T>& v) ;

template <class T> int operator==(const PlVector<T>&,const PlVector<T>&);
template <class T> int operator!=(const PlVector<T>& a,const PlVector<T>& b);



/**  **  **  **  **  **  **  **  **  **  **  **  **  **  **  **  **  **  **
        Class: PlVector --- A template vector class
  Description: A simple vector class with basic linear algebraic vector 
               operators
   Created by: Philippe Lavoie (4 Oct. 1996)
  Modified by:
 **  **  **  **  **  **  **  **  **  **  **  **  **  **  **  **  **  **  **/
template<class T> class PlVector : public BasicArray<T>
{
public:
  const int& rows;
  PlVector() : BasicArray<T>(1), rows(sze) {}
  PlVector(const int r) : BasicArray<T>(r), rows(sze) {}
  PlVector(const PlVector<T>& v) : BasicArray<T>(v), rows(sze)  {}
  PlVector(const BasicArray<T>& v) : BasicArray<T>(v), rows(sze)  {}
  PlVector(T* ap, const int size) : BasicArray<T>(ap,size), rows(sze) {}
  PlVector(BasicList<T>& list) : BasicArray<T>(list), rows(sze) {}

  virtual ~PlVector() {}
  
  PlVector<T>& operator=(const PlVector<T>& v);
  PlVector<T>& operator=(const BasicArray<T>& v);
  
  PlVector<T>& operator+=(const PlVector<T>&);
  PlVector<T>& operator-=(const PlVector<T>&);

  T operator=(const T d);
  void as(int i, const PlVector<T>& b);
  PlVector<T> get(int i, int l);
  
  int minIndex() const ;
  T minimum() const { return operator[](minIndex()) ; }  // returns the minimal value inside the vector

  void qSortStd() ;
  void qSort(int M=7) ; 
  void sortIndex(PlVector<int>& index, int M=7) const;

#ifdef ANSI_ISO_CPP
  friend PlVector<T> operator+ <>(const PlVector<T>&, const PlVector<T>&);
  friend PlVector<T> operator- <>(const PlVector<T>&, const PlVector<T>&);
  friend T operator* <>(const PlVector<T>&,const PlVector<T>&); 
  
  friend PlVector<T> operator* <>(const PlVector<T>& v, const double d);
  friend PlVector<T> operator* <>(const PlVector<T>& v, const Complex d);
  			    
  friend PlVector<T> operator* <>(const double d,const PlVector<T>& v) ;
  friend PlVector<T> operator* <>(const Complex d,const PlVector<T>& v) ;
  
  friend int operator== <>(const PlVector<T>&,const PlVector<T>&);
  friend int operator!= <>(const PlVector<T>& a,const PlVector<T>& b);
#else
  friend PlVector<T> operator+(const PlVector<T>&, const PlVector<T>&);
  friend PlVector<T> operator-(const PlVector<T>&, const PlVector<T>&);
  friend T operator*(const PlVector<T>&,const PlVector<T>&); 
  
  friend PlVector<T> operator*(const PlVector<T>& v, const double d);
  friend PlVector<T> operator*(const PlVector<T>& v, const Complex d);
  			    
  friend PlVector<T> operator*(const double d,const PlVector<T>& v) ;
  friend PlVector<T> operator*(const Complex d,const PlVector<T>& v) ;
  
  friend int operator==(const PlVector<T>&,const PlVector<T>&);
  friend int operator!=(const PlVector<T>& a,const PlVector<T>& b);
#endif
  
};

template<class T> int operator!=(const PlVector<T>& a, const PlVector<T>& b){
  return (a==b)?0:1 ; 
}

template<class T>
PlVector<T>  operator*(const double d, const PlVector<T>& v) {
  return v*d ;
}

template<class T>
PlVector<T>  operator*(const Complex d, const PlVector<T>& v) {
  return v*d ;
}


typedef PlVector<int> Vector_INT ;
typedef PlVector<char> Vector_BYTE ;
typedef PlVector<float> Vector_FLOAT ;
typedef PlVector<double> Vector_DOUBLE ;
typedef PlVector<Complex> Vector_COMPLEX ;
typedef PlVector<unsigned char> Vector_UBYTE ;
typedef PlVector<Vector2> Vector_Vector2 ;
typedef PlVector<CtrlPnt4D> Vector_CtrlPnt4D;
typedef PlVector<Point3D> Vector_Point3D ;
typedef PlVector<Color> Vector_Color ;

typedef PlVector<int> PlVector_int ;
typedef PlVector<char> PlVector_byte ;
typedef PlVector<float> PlVector_float ;
typedef PlVector<double> PlVector_double ;
typedef PlVector<Complex> PlVector_complex ;
typedef PlVector<unsigned char> PlVector_ubyte ;
typedef PlVector<Vector2> PlVector_Vector2 ;
typedef PlVector<CtrlPnt4D> PlVector_CtrlPnt4D;
typedef PlVector<Point3D> PlVector_Point3D ;
typedef PlVector<Color> PlVector_Color ;


#ifdef CPP_SRC_INCLUDE
#include "../src/vector.cc"
#endif


#endif 
