/*=============================================================================
        File: barray.cc
     Purpose:       
    Revision: $Id: barray.cc,v 1.3 1998/06/03 14:33:09 lavoie Exp $
  Created by: Philippe Lavoie          (3 Oct, 1996)
 Modified by: 

 Copyright notice:
          Copyright (C) 1996-1998 Philippe Lavoie
 
	  This library is free software; you can redistribute it and/or
	  modify it under the terms of the GNU Library General Public
	  License as published by the Free Software Foundation; either
	  version 2 of the License, or (at your option) any later version.
 
	  This library is distributed in the hope that it will be useful,
	  but WITHOUT ANY WARRANTY; without even the implied warranty of
	  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
	  Library General Public License for more details.
 
	  You should have received a copy of the GNU Library General Public
	  License along with this library; if not, write to the Free
	  Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
=============================================================================*/


#include "internals.hh"
#include "barray.hh"

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
       Member: BasicArray --- default constructor
  Description: Default constructor
        Input:
       Output: 
 Restrictions:
   Created by: Philippe Lavoie (24 January 1997)
  Modified by:
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
template<class T>
BasicArray<T>::BasicArray() : n(sze),rsize(1), wdth(1),sze(1)
{
  x = new T [1];
  x[0] = (T) 0 ;
  destruct = 1 ;
}

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
       Member: BasicArray --- constructor with size specified
  Description: Constructor with size specified
        Input: ni --> the size of the basic array
       Output:
 Restrictions:
   Created by: Philippe Lavoie (24 January 1997)
  Modified by:
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
template<class T>
BasicArray<T>::BasicArray(const int ni) : n(sze){
  x = 0 ;
  sze = wdth = rsize = 0 ;
  resize(ni) ;
  destruct = 1 ;
}

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
       Member: BasicArray --- copy constructor from a pointer
  Description: The BasicArray is constructed from a pointer and a specified
               size. The desctructor of the class will {\bf not} delete this
	       pointer when it is finished with it. It is up to the function
	       which created it to delete it.
        Input: ap --> a pointer to an array
               size --> the size of the ap array
       Output:
 Restrictions:
   Created by: Philippe Lavoie (24 January 1997)
  Modified by:
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
template<class T>
BasicArray<T>::BasicArray(T* ap, const int size) : n(sze), rsize(size), wdth(size+1), sze(size)
{
  x = ap ;
  destruct = 0 ;
}

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
       Member: BasicArray --- copy constructor
  Description: Copy constructor
        Input: f2 --> the BasicArray to copy
       Output:
 Restrictions:
   Created by: Philippe Lavoie (24 January 1997)
  Modified by:
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
template<class T>
BasicArray<T>::BasicArray(const BasicArray<T>& f2) : n(sze)
{
  rsize = sze = 0 ;
  x = 0 ;
  resize(f2.size());
  T *p2,*p1 ;
  p1 = x-1 ;
  p2 = f2.x - 1 ;
  
  for (int k = 0; k < rsize; ++k)
    *(++p1) = *(++p2) ;
  destruct = 1 ;
}

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
       Member: BasicArray --- constructor from a linked list
  Description: Constructor from a linked list
        Input: list --> the linked list to copy
       Output:
 Restrictions:
   Created by: Philippe Lavoie (24 January 1997)
  Modified by:
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
template<class T>
BasicArray<T>::BasicArray(BasicList<T> &list) : n(sze)
{
  BasicNode<T> *node = list.goToFirst() ;
  rsize = sze = 0 ;
  x = 0 ;
  resize(list.size());

  for (int k = 0; k < rsize; ++k){
    x[k] = *node->data ;
    node = list.goToNext() ;
  }
  destruct = 1 ;
}

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
       Member: ~BasicArray --- destructor
  Description: Destructor
        Input:
       Output:
 Restrictions:
   Created by: Philippe Lavoie (24 January 1997)
  Modified by:
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
template<class T>
BasicArray<T>::~BasicArray()
{
  if(destruct){
    if ( x ) delete []x;
    x = 0;
    rsize = sze = 0;                 
  }
}

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
       Member: operator= --- assignment operator
  Description: The assignment operator
        Input: f2 --> the BasicArray to copy from
       Output: a reference to itself
 Restrictions:
   Created by: Philippe Lavoie (24 January 1997)
  Modified by:
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
template<class T> 
BasicArray<T>& 
BasicArray<T>::operator= (const BasicArray<T>& f2)
{
  if ( this == &f2 )
    return *this;

  resize(f2.size()) ;
  
  // wdth = f2.wdth;
  T *p1,*p2 ;
  p1 = x-1 ;
  p2 = f2.x-1 ;

  for (int k = 0; k < sze; ++k)
    *(++p1) = *(++p2) ;

  return *this;	
}

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
       Member: trim --- shortens the array without destroying the components
  Description: Shortens the array without destroying the components
        Input: nsize --> the new size of the vector
       Output:
 Restrictions:
   Created by: Philippe Lavoie (24 January 1997)
  Modified by:
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
template<class T>
void 
BasicArray<T>::trim(const int nsize)	
{
  if ( nsize >= 0 && nsize <= rsize )
    sze = nsize;
}

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
       Member: reset --- reset all values of the vetor to $val$
  Description: Reset all the values of the vetor to $val$.
        Input: val --> resets the elements of the array to this value
       Output:
 Restrictions:
   Created by: Philippe Lavoie (24 January 1997)
  Modified by:
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
template<class T>
void 
BasicArray<T>::reset(const T val)
{
  T *p1 ;
  p1 = x-1 ;
  for (int k = 0; k <sze; ++k)
    *(++p1) = val;
}

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
       Member: resize --- a non-destructive resize of the vector's length
  Description: Will change the size of the vector to $nsize$. If the size 
               is increased, the old data is still accessible at the same 
	       old index

        Input: nsize --> the new size of the vector
       Output:
 Restrictions:
   Created by: Philippe Lavoie (24 January 1997)
  Modified by:
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
template<class T>
void 
BasicArray<T>::resize(const int nsize)
{
  int k;
  
  if ( nsize == rsize ){
    sze = nsize ;
    return;			// nothing to do
  }
  
  if(sze>nsize){
    sze = nsize ;
    return ;
  }

  if((sze<nsize) && (nsize<rsize)){
    for(k=sze;k<nsize;++k)
      x[k] = (T)0 ;
    sze = nsize ;
    return ;
  }

  T *xn ; 
  T *p,*pn ;
  xn = new T[nsize];
  p = x ;
  pn = xn ;
  
  if ( x )
    {
      // copy the old data
      for (k = 0; k < sze; ++k){
	*pn = *p ;
	++pn ; 
	++p ;
      }
      for(k=sze;k<nsize;++k){
	*pn = (T)0 ;
	++pn ;
      }
      if(destruct)
	delete []x;
    }
  else
    for (k = 0; k < nsize; ++k){
      *pn = (T)0 ;
      ++pn ;
    }
  
  rsize = nsize;
  sze = rsize ;
  x = xn;
  destruct = 1 ;
  wdth = rsize + 1;
}

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
      Routine: operator>> --- stores an input string into a vector
  Description: Stores an input string into a vector
        Input: is --> the input stream
             arry <-- the array initialized by $is$
       Output: $is$ after setting $arry$
 Restrictions:
   Created by: Philippe Lavoie (24 January 1997)
  Modified by:
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
template<class T>
istream&  
operator>> (istream& is, BasicArray<T>& arry)
{
  T new_x;
  int k, kend;

  if ( is.eof() )
    return is;
  
  
  kend = arry.size();
  
  for (k = 0; k < kend; ++k)
	{		
	  is >> new_x;	  
	  // end of file reached, abort the read
	  if ( is.eof() || is.fail() )
	    return is;
	  
	  arry[k] = new_x;		
	}
  
  return is;
	
}

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
      Routine: operator<< --- sends a BasicArray to the ostream
  Description: Sends a BasicArray to the ostream
        Input: os <-> output stream
             arry --> the array to send to the stream
       Output: $os$ with $arry$ piped into it.
 Restrictions:
   Created by: Philippe Lavoie (24 January 1997)
  Modified by:
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
template<class T>
ostream&  
operator<<(ostream& os,const BasicArray<T>& arry)
{
  int iend = arry.size();
  
  for (int i = 0; i < iend;  )
    {
      os << arry[i] ; // << endl;
      if ( (++i % arry.wdth) == 0 )
	os << '\n';
      else
	os << "   "; 
    }
  os << '\n' ;
  //	if ( (i % arry.wdth) != 0 )	os << '\n';
  
  return os;	
}

ostream&
operator<<(ostream& os, const BasicArray<Complex>& arry) {
  int iend = arry.size();
  
  for (int i = 0; i < iend;  )
    {
      os << real(arry[i]) << "+" << imag(arry[i]) << "i" ; // << endl;
      if ( (++i % arry.wdth) == 0 )
	os << '\n';
      else
	os << "   "; 
    }
  os << '\n' ;
  //	if ( (i % arry.wdth) != 0 )	os << '\n';
  
  return os;	
}


/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
      Routine: operator!= --- compares two arrays to see if they're different
  Description: This checks if one of the components of the two arrays are 
               different. A check is made after each comparison to see if 
	       they are the same, this speeds up the calculation if the 
	       arrays have early different components, otherwise the use of 
	       the == operator would yield a faster result.
        Input: a --> the first array to compare
               b --> the second array to compare
       Output: 1 if the arrays are {\em not} identical, 0 otherwise.
 Restrictions:
   Created by: Philippe Lavoie (24 January 1997)
  Modified by:
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
template<class T>
int  
operator!=(const BasicArray<T> &a,const BasicArray<T> &b) 
{
  if ( a.size() != b.size() )
    return 1;
  
  const int sz = a.size();
  int l = 1;
  
  for (int i = 0; i < sz; ++i){
    l = l && ( a[i] == b[i] );
    if(!l) break ;
  }
  
  return (l==0 ? 1 : 0);
  
}

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
      Routine: operator== --- checks if two arrays are identical
  Description: Checks if two arrays are identical
        Input: a --> the first array to compare
               b --> the second array to compare
       Output: 1 if the arrays are identical, 0 otherwise
 Restrictions:
   Created by: Philippe Lavoie (24 January 1997)
  Modified by:
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
template<class T>
int  
operator==(const BasicArray<T> &a,const BasicArray<T> &b) 
{
  if ( a.size() != b.size() )
    return 0;
  
  const int sz = a.size();
  int l = 1;
  
  for (int i = 0; i < sz; ++i)
    l = l && ( a[i] == b[i] );
  
  return l; 
}

#ifdef DEBUG_MATRIX
template <class T>
T& BasicArray<T>::operator[](const int i) {
  if(i<0 || i>=n){
    PlError error("BasicArray<T>::operator[]") ;
    error << "Error Accessing the vector (press C-c to stop)\n" ;
    error << "Bad index " << i << endl ;
    error.fatal() ;
  }
  return x[i] ;
  
}

template <class T>
T  BasicArray<T>::operator[](const int i) const {
  if(i<0 || i>=n){
    PlError error("BasicArray<T>::operator[]") ;
    error << "Error Accessing the vector (press C-c to stop)\n" ;
    error << "Bad index " << i << endl ;
    error.fatal() ;
  }
  return x[i] ;
}

#endif

#if !defined(__sgi ) || defined(__GNUG__)

// Some types need a different handler for certain functions

void BasicArray<CtrlPnt4D>::resize(const int nsize) {
  int k; 
  
  if ( nsize == rsize ){
    sze = nsize ;
    return;			// nothing to do
  }
  
  if(sze>nsize){
    sze = nsize ;
    return ;
  }

  if((sze<nsize) && (nsize<rsize)){
    for(k=sze;k<nsize;++k)
      x[k] = CtrlPnt4D(0) ;
    sze = nsize ;
    return ;
  }

  CtrlPnt4D *xn ; 
  CtrlPnt4D *p,*pn ;

  xn = new CtrlPnt4D[nsize];
  p = x ;
  pn = xn ;
  
  // we want to have all the data to be in a consecutive memory segment
  float *dn ;

  dn = new float[nsize*4] ;

  for(k=0;k<nsize;++k){
    if(xn[k].created && xn[k].data){
      delete []xn[k].data ;
    }
    xn[k].created = 0; 
    xn[k].data = &dn[k*4] ;
  }
  if(nsize>0)
    xn[0].created = 1 ;

  if ( x )    {
      // copy the old data
      for (k = 0; k < sze; ++k){
	*pn = *p ;
	++pn ; 
	++p ;
      }
      for(k=sze;k<nsize;++k){
	*pn = CtrlPnt4D(0) ;
	++pn ;
      }
      if(sze>0 && destruct){
	delete []x;
      }
    }
  else
    for (k = 0; k < nsize; ++k){
      *pn = CtrlPnt4D(0) ;
      ++pn ;
    }
  
  rsize = nsize;
  sze = rsize ;
  x = xn;
  destruct = 1 ;
  
  wdth = rsize + 1; 
}

#endif

#if defined(__GNUG__) && !defined(CPP_SRC_INCLUDE)

// template instantiation (to be used with -fno-implicit-templates

template class BasicArray<int> ;
template class BasicArray<char> ;
template class BasicArray<float> ;
template class BasicArray<double> ;
template class BasicArray<Complex> ;
template class BasicArray<unsigned char> ;
template class BasicArray<Vector2>;
template class BasicArray<CtrlPnt4D>;
template class BasicArray<Point3D>;
template class BasicArray<Color>;
template class BasicArray<Coordinate> ;
template class BasicArray<void*> ;

template int operator!=(const BasicArray<int>&,const BasicArray<int>&); 
template int operator!=(const BasicArray<char>&,const BasicArray<char>&); 
template int operator!=(const BasicArray<float>&,const BasicArray<float>&); 
template int operator!=(const BasicArray<double>&,const BasicArray<double>&); 
template int operator!=(const BasicArray<Complex>&,const BasicArray<Complex>&);
template int operator!=(const BasicArray<unsigned char>&,const BasicArray<unsigned char>&); 
template int operator!=(const BasicArray<Vector2>&,const BasicArray<Vector2>&);template int operator!=(const BasicArray<Color>&,const BasicArray<Color>&); 
template int operator!=(const BasicArray<Point3D>&,const BasicArray<Point3D>&);
template int operator!=(const BasicArray<CtrlPnt4D>&,const BasicArray<CtrlPnt4D>&); 
template int operator!=(const BasicArray<void*>&,const BasicArray<void*>&); 
template int operator!=(const BasicArray<Coordinate>&,const BasicArray<Coordinate>&); 

template int operator==(const BasicArray<int>&,const BasicArray<int>&); 
template int operator==(const BasicArray<char>&,const BasicArray<char>&); 
template int operator==(const BasicArray<float>&,const BasicArray<float>&); 
template int operator==(const BasicArray<double>&,const BasicArray<double>&); 
template int operator==(const BasicArray<Complex>&,const BasicArray<Complex>&);
template int operator==(const BasicArray<Vector2>&,const BasicArray<Vector2>&);
template int operator==(const BasicArray<Color>&,const BasicArray<Color>&); 
template int operator==(const BasicArray<Point3D>&,const BasicArray<Point3D>&);
template int operator==(const BasicArray<CtrlPnt4D>&,const BasicArray<CtrlPnt4D>&); 
template int operator==(const BasicArray<void*>&,const BasicArray<void*>&); 
template int operator==(const BasicArray<Coordinate>&,const BasicArray<Coordinate>&); 

 
template istream& operator>>(istream& is, BasicArray<int>& ary);
template istream& operator>>(istream& is, BasicArray<char>& ary);
template istream& operator>>(istream& is, BasicArray<float>& ary);
template istream& operator>>(istream& is, BasicArray<double>& ary);
#ifdef GNU_GCC_2_8
template class complex<double> ;
template istream& operator>><double>(istream& , Complex& ) ;
#else
template istream& operator>>(istream&, Complex&) ;
#endif
template istream& operator>>(istream& is, BasicArray<Complex>& ary);
template istream& operator>>(istream& is, BasicArray<unsigned char>& ary);
template istream& operator>>(istream& is, BasicArray<Color>& ary);
template istream& operator>>(istream& is, BasicArray<Point3D>& ary);
template istream& operator>>(istream& is, BasicArray<CtrlPnt4D>& ary);
template istream& operator>>(istream& is, BasicArray<Coordinate>& ary);

template ostream& operator<<(ostream& os, const BasicArray<int>& ary);
template ostream& operator<<(ostream& os, const BasicArray<char>& ary);
template ostream& operator<<(ostream& os, const BasicArray<float>& ary);
template ostream& operator<<(ostream& os, const BasicArray<double>& ary);
#ifndef GNU_GCC_2_8
template ostream& operator<<(ostream& os, const BasicArray<Complex>& ary);
#endif
template ostream& operator<<(ostream& os, const BasicArray<unsigned char>& ary);
template ostream& operator<<(ostream& os, const BasicArray<Color>& ary);
template ostream& operator<<(ostream& os, const BasicArray<Point3D>& ary);
template ostream& operator<<(ostream& os, const BasicArray<CtrlPnt4D>& ary);
template ostream& operator<<(ostream& os, const BasicArray<Coordinate>& ary);
template ostream& operator<<(ostream& os, const BasicArray<void*>& ary);



#else

void dumbSPARC_BasicArray(){
  BasicArray<int> a ;
  BasicArray<double> b ;
  BasicArray<char> c2 ; 
  BasicArray<unsigned char> c ;
  BasicArray<Complex> d ;
  BasicArray<CtrlPnt4D> e ;
  BasicArray<Point3D> f ;
  BasicArray<Vector2> g ;
  BasicArray<CtrlPnt4D> h ;
  BasicArray<float> i ;
  BasicArray<Color> j ;
  BasicArray<Coordinate> k ;
  //BasicArray<void*> l ; // I can't get the compiler to accept this line

  a = a ;
  b = b ;
  c = c ;
  d = d ;
  e = e ;
  f = f ;
  g = g ;
  h = h ;
  j = j ;
  c2 = c2 ;
  //l = l  ;

  cout << a << endl;
  cout << b << endl;
  cout << c << endl;
  cout << c2 << endl ;
  cout << d << endl;
  cout << e << endl;
  cout << f << endl;
  cout << g << endl;
  cout << h << endl;
  cout << i << endl;
  cout << j << endl ;
  cout << k ;
  //cout << l ;

  a == a ;
  b == b ;
  c == c ;
  d == d ;
  e == e ;
  f == f ;
  g == g ;
  h == h ;
  i == i ;
  j == j ;
  //l == l ;

  c2 == c2 ; 
  c2 != c2 ;

  a != a ;
  b != b ;
  c != c ;
  d != d ;
  e != e ;
  f != f ;
  g != g ;
  h != h ;
  i != i ;
  j != j ;
  //l != l ;
}

#endif

