/*=============================================================================
        File: barray2d.cc
     Purpose:
    Revision: $Id: barray2d.cc,v 1.2 1998/05/12 18:11:41 lavoie Exp $
  Created by: Philippe Lavoie          (3 Oct, 1996)
 Modified by: 

 Copyright notice:
          Copyright (C) 1996-1997 Philippe Lavoie
 
	  This library is free software; you can redistribute it and/or
	  modify it under the terms of the GNU Library General Public
	  License as published by the Free Software Foundation; either
	  version 2 of the License, or (at your option) any later version.
 
	  This library is distributed in the hope that it will be useful,
	  but WITHOUT ANY WARRANTY; without even the implied warranty of
	  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
	  Library General Public License for more details.
 
	  You should have received a copy of the GNU Library General Public
	  License along with this library; if not, write to the Free
	  Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
=============================================================================*/

#include "internals.hh"
#include "barray2d.hh"

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
       Member: init --- initialize the 2D array
  Description: Initialize the 2D array
        Input:
       Output: 
 Restrictions:
   Created by: Philippe Lavoie (24 January 1997)
  Modified by:
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
template <class T>
void 
Basic2DArray<T>::init(const int r,const int c)
{
  if ( r <= 0  || c <= 0 )
    {
      return ;
      PlError error("Basic2DArray<T>::init");
      error << "bad Basic2DArray<T> size " << r << ", " << c << endl ;
      error.fatal() ;
    }
	
  rz = r;	cz = c;

  m = new T [rz*cz];
  created = 1 ;
#ifdef COLUMN_ORDER
  vm = new T* [cz] ;
#else
  vm = new T* [rz] ;
#endif
  
  T *p1 ;
  int sze = rz*cz ;
  p1 = m-1 ;
  
  int i ;
  for(i = 0; i < sze; i++)
      *(++p1) = (T)0;
#ifdef COLUMN_ORDER
  for(i=0;i<cz;++i)
    vm[i] = &m[i*rz] ;
#else
  for(i=0;i<rz;++i)
    vm[i] = &m[i*cz] ;
#endif
}

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
       Member: Basic2DArray --- constructor  
  Description: Constructs an array of size (1,1).
        Input: 
       Output:
 Restrictions: 
   Created by: Philippe Lavoie (24 January 1997)
  Modified by:
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
template <class T>
Basic2DArray<T>::Basic2DArray() : rows(rz), cols(cz){
  by_columns = 0 ;
  width = 2 ;
  created = 1 ;
  m = 0; 
  vm = 0 ;
  init(); 
}

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
       Member: Basic2DArray --- constructor  with the size specified
  Description: Constructs an array of size (r,c).
        Input: r --> the number of rows 
	       c --> the number of columns
       Output:
 Restrictions: 
   Created by: Philippe Lavoie (24 January 1997)
  Modified by:
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
template <class T>
Basic2DArray<T>::Basic2DArray(const int r, const int c) : rows(rz), cols(cz){
    by_columns = 0 ;
    width = 2 ;
    created = 1 ;
    m = 0; 
    vm = 0 ;
    init(r,c); 
}


/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
       Member: Basic2DArray --- constructor from memory
  Description: Constructs a matrix from memory. No memory is allocated for 
               this new basic2DArray.
        Input: p --> the memory pointer
	       r --> the number of rows 
	       c --> the number of columns
       Output:
 Restrictions: Do not access the basic2DArray once you delete the memory
               segment used by the matrix since it now points to a dead
	       memory location.
   Created by: Philippe Lavoie (24 January 1997)
  Modified by:
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
template <class T>
Basic2DArray<T>::Basic2DArray(T* p, const int r, const int c) : rows(rz), cols(cz){
  created = 0 ;
  rz = r ;
  cz = c ;
  m = p ;
  by_columns = 0 ;
  width = 2 ; 
#ifdef COLUMN_ORDER
  vm = new T* [cz] ;
  for(int i=0;i<cz;++i)
    vm[i] = &m[i*rz] ;
#else
  vm = new T* [rz] ;
  for(int i=0;i<rz;++i)
    vm[i] = &m[i*cz] ;
#endif
}

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
       Member: Basic2DArray --- copy constructor
  Description: Copy constructor
        Input: a --> the Basic2DArray to copy from
       Output:
 Restrictions:
   Created by: Philippe Lavoie (24 January 1997)
  Modified by:
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
template <class T>
Basic2DArray<T>::Basic2DArray(const Basic2DArray<T> &a) : rows(rz), cols(cz)
{
  int i;
  created = 1 ;
  m = 0 ;
  init(a.rows,a.cols);
  by_columns = a.by_columns;
  width = a.width ;
  
  T *p1,*pa ;
  int sz = a.rows*a.cols ;
  p1 = m-1 ;
  pa = a.m-1 ;

  for (i = 0; i < sz ; i++)
      *(++p1) = *(++pa) ;
}

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
       Member: operator= --- assignment operator
  Description: Assignment operator
        Input: a --> the Basic2DArray to copy from
       Output: a reference to itself
 Restrictions:
   Created by: Philippe Lavoie (24 January 1997)
  Modified by:
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
template <class T>
Basic2DArray<T>& Basic2DArray<T>::operator=(const Basic2DArray<T> &a)
{
  int i;
  
  if ( this == &a )
    return *this;
  
  if(rows != a.rows || cols != a.cols){
    resize(a.rows,a.cols) ;
  }

  T *p1,*pa ;
  int sz = a.rows*a.cols ;
  p1 = m-1 ;
  pa = a.m-1 ;

  for (i = 0; i < sz; i++)
      *(++p1) = *(++pa) ;
  
  by_columns = a.by_columns;
  width = a.width ;
  
  return *this;
}

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
       Member: ~Basic2DArray --- destructor
  Description: Destructor
        Input:
       Output:
 Restrictions:
   Created by: Philippe Lavoie (24 January 1997)
  Modified by:
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
template <class T>
Basic2DArray<T>::~Basic2DArray()
{
  if(m && created){
    delete []m;
  }
  if(vm)
    delete []vm ;
}

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
       Member: resize --- a destructive resize of the matrix dimensions
  Description: Changes the matrix dimensions and intialize it to 0.

        Input:  nr --> the new number of rows
                nc --> the new number of columns
       Output:
 Restrictions:
   Created by: Philippe Lavoie (24 January 1997)
  Modified by:
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
template <class T>
void 
Basic2DArray<T>::resize(const int nr,const int nc)
{
  if(rows>1 || cols>1){
    if(m && created)
      delete []m ;
    if(vm)
      delete []vm ;
  }
  else{
    if(m && created)
      delete []m ;
    if(vm)
      delete []vm ;
  }
    
  init(nr, nc);
}

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
       Member: resizeKeep --- a non-destructive resize of the matrix dimensions
  Description: Changes the matrix dimensions in a non-destructive manner.
               The old elements are still accessible at the same coordinates.

	       This is an expensive function computationally wise due to the 
	       copying which needs to be done.

        Input: nr --> the new number of rows
               nc --> the new number of columns
       Output:
 Restrictions:
   Created by: Philippe Lavoie (24 January 1997)
  Modified by:
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
template <class T>
void 
Basic2DArray<T>::resizeKeep(const int nr,const int nc)
{

  if(nr==rz && nc==cz){
    return ;
  }

  T *mn ;
  T *p,*pn ;

  mn = new T[nr*nc] ;
  
  int i,j ;


#ifdef COLUMN_ORDER
  for(j=0;j<minimum(nc,cz);j++){
    p = &m[j*rz] -1;
    pn = &mn[j*nr] -1 ; 
    for(i=0;i<minimum(nr,rz);i++){
      *(++pn) = *(++p) ;
    }
  }
  
  for(j=cz;j<nc;j++){
    pn = &mn[j*nr+rz]-1 ;
    for(i=rz;i<nr;i++)
      *(pn++) = (T)0 ;
  }
#else
  for(i=0;i<minimum(nr,rz);i++){
    p = &m[i*cz] -1;
    pn = &mn[i*nc] -1 ;
    for(j=0;j<minimum(nc,cz);j++){
      *(++pn) = *(++p) ;
    }
  }

  for(i=rz;i<nr;i++){
    pn = &mn[i*nc+cz]-1 ;
    for(j=cz;j<nc;j++)
      *(pn++) = (T)0 ;
  }
#endif

  
  rz = nr ;
  cz = nc ;
  
  if(m && created)
    delete []m ;
  m = mn ;
  if(vm)
    delete []vm ;
#ifdef COLUMN_ORDER
  vm = new T* [cz] ;
  for(i=0;i<cz;++i)
    vm[i] = &m[i*rz] ;
#else
  vm = new T* [rz] ;
  for(i=0;i<rz;++i)
    vm[i] = &m[i*cz] ;
#endif
}

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
       Member: reset --- reset all values of the 2D array to $v$
  Description: Reset all values of the 2D array to $v$
        Input: v --> resets the elements of the 2D array to this value
       Output:
 Restrictions:
   Created by: Philippe Lavoie (24 January 1997)
  Modified by:
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
template <class T>
void 
Basic2DArray<T>::reset(const T v)
{
  T *p1 ;
  p1 = m-1 ;
  int size = rows*cols ;
  for(int i=0;i<size; ++i)
    *(++p1) = v ;
}

#ifdef DEBUG_MATRIX

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
       Member: elem --- accesses an element of a 2D array
  Description: Access an element of a 2D array. If the \verb.DEBUG_MATRIX.
               flag has been set, then a fatal error will be issued if the 
	       element index are not in a valid range.
        Input: i --> the row of the element
               j --> the column of the element
       Output: $vm[i][j]$ or $vm[j][i]$, depending on 
               COLUMN$\underline{ }$ORDER.
 Restrictions:
   Created by: Philippe Lavoie (24 January 1997)
  Modified by:
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
template <class T>
T& 
Basic2DArray<T>::elem(const int i,const int j)
{
  if ( i < 0  || rows <= i || j < 0 || cols <= j )
    {
      PlError error("T& Basic2DArray<T>::elem(int,int)") ;
      if (i < 0 || rows <= i){
	error << "bad first index " << i << endl ;
      }
      if (j < 0 || cols <= j){
	error << "bad second index " << j << endl ;
      }
      error.fatal() ;
    }
#ifdef COLUMN_ORDER
  return vm[j][i] ;
#else
  return vm[i][j] ;
#endif
}

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
       Member: elem --- accesses an element of a 2D array
  Description: Access an element of a 2D array. If the \verb.DEBUG_MATRIX.
               flag has been set, then a fatal error will be issued if the 
	       element index are not in a valid range.
        Input: i --> the row of the element
               j --> the column of the element
       Output: $vm[i][j]$ or $vm[j][i]$, depending on 
               COLUMN$\underline{ }$ORDER.
 Restrictions:
   Created by: Philippe Lavoie (24 January 1997)
  Modified by:
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
template <class T>
T 
Basic2DArray<T>::elem(const int i,const int j) const
{
  if ( i < 0  || rows <= i || j < 0 || cols <= j )
    {
      PlError error("T Basic2DArray<T>::elem(int,int) const") ;
      if (i < 0 || rows <= i){
	error << "bad first index " << i << endl ;
      }
      if (j < 0 || cols <= j){
	error << "bad second index " << j << endl ;
      }
      error.fatal() ;
    }
#ifdef COLUMN_ORDER
  return vm[j][i] ;
#else
  return vm[i][j] ;
#endif
}
#endif

template <class T>
istream& 
operator>> (istream& is,Basic2DArray<T>& a)
{
  int i, j;
  int r = a.rows, c = a.cols;
  
  if ( a.by_columns )
    for (j = 0; j < c; j++)
      for (i = 0; i < r; i++)
	is >>  a.elem(i,j) ;
  else
    for (i = 0; i < r; i++)
      for (j = 0; j < c; j++)
	is >>  a.elem(i,j) ;
  
  return is;	
}

template <class T>
ostream& 
operator<<(ostream& os,const Basic2DArray<T>& a)
{
  int i, j;
  int r = a.rows, c = a.cols;
  
  if ( a.by_columns )
    for (j = 0; j < c; j++)
      {
	for (i = 0; i < r; i++) {
	  os <<  setw(a.width) << a.elem(i,j) << ' ';
	}
	os << '\n';
      }  
  else
    for (i = 0; i < r; i++)
      {
	for (j = 0; j < c; j++){
	  os <<  setw(a.width) << a.elem(i,j) << ' ' ;
	}
	os << '\n';
      }

  return os;
}

ostream& 
operator<<(ostream& os,const Basic2DArray<unsigned char>& a)
{
  int i, j;
  int r = a.rows, c = a.cols;
  
  if ( a.by_columns )
    for (j = 0; j < c; j++)
      {
	for (i = 0; i < r; i++) {
	  os <<  setw(a.width) << (int)a.elem(i,j) << ' ';
	}
	os << '\n';
      }  
  else
    for (i = 0; i < r; i++)
      {
	for (j = 0; j < c; j++){
	  os <<  setw(a.width) << (int)a.elem(i,j) << ' ';
	}
	os << '\n';
      }

  return os;
}

#if !defined(__sgi ) || defined(__GNUG__)

void
Basic2DArray<CtrlPnt4D>::init(const int r,const int c)
{
  if ( r <= 0  || c <= 0 )
    {
      if(m)
	delete []m ;
      if(vm)
	delete []vm ;
      rz = r ;
      cz = c ;
      return ;
      PlError error("Basic2DArray<T>::init");
      error << "bad Basic2DArray<T> size " << r << ", " << c << endl ;
      error.fatal() ;
    }
	
  rz = r;	cz = c;

  m = new CtrlPnt4D [rz*cz];
  created = 1 ;
#ifdef COLUMN_ORDER
  vm = new CtrlPnt4D* [cz] ;
#else
  vm = new CtrlPnt4D* [rz] ;
#endif

  // we want to have all the data in a consecutive memory segment
  // but it has to be in column order instead of row order
  // i.e.
  //   m1 m5 m9  m13
  //   m2 m6 m10 m14
  //   m3 m7 m11 m15
  //   m4 m8 m12 m16
  // This is done so that OpenGL can be used effectively
  // since the data portion will now be in column order.
  // Note that the order of the elements themselves are
  // still defined by the COLUMN_ORDER macro

  float* dn ;
  dn = new float[rz*cz*4] ;

  int i ;
#ifdef COLUMN_ORDER
  for(i=0;i<rz*cz;++i){
    if(m[i].created && m[i].data)
      delete [] m[i].data ;
    m[i].created = 0 ;
    m[i].data = &dn[i*4] ;
  }
#else
  for(i=0;i<rz;++i){
    for(int j=0;j<cz;++j){
      if(m[i*cz+j].created && m[i*cz+j].data)
	delete [] m[i*cz+j].data ;
      m[i*cz+j].created = 0 ;
      m[i*cz+j].data = &dn[(rz*j+i)*4] ;
    }
  }
#endif
  
  CtrlPnt4D *p1 ;
  int sze = rz*cz ;
  p1 = m-1 ;

  if(sze>0)
    m[0].created = 1 ;

  for(i = 0; i < sze; i++)
      *(++p1) = (CtrlPnt4D)0;
#ifdef COLUMN_ORDER
  for(i=0;i<cz;++i)
    vm[i] = &m[i*rz] ;
#else
  for(i=0;i<rz;++i)
    vm[i] = &m[i*cz] ;
#endif
}

void 
Basic2DArray<CtrlPnt4D>::resizeKeep(const int nr,const int nc)
{
  if(nr==rz && nc==cz){
    return ;
  }

  CtrlPnt4D *mn ;
  CtrlPnt4D *p,*pn ;

  pn = 0 ;

  mn = new CtrlPnt4D[nr*nc] ;
  
  // we want to have all the data in a consecutive memory segment
  // but it has to be in column order instead of row order
  // i.e.
  //   m1 m5 m9  m13
  //   m2 m6 m10 m14
  //   m3 m7 m11 m15
  //   m4 m8 m12 m16
  // This is done so that OpenGL can be used effectively
  // since the data is column ordered.
  // Note that the elements themselves are still ordered
  // according to COLUMN_ORDER

  float* dn ;
  dn = new float[nr*nc*4] ;

  int i,j ;
#ifdef COLUMN_ORDER
  for(i=0;i<nr*nc;++i){
    if(mn[i].created && mn[i].data)
      delete [] mn[i].data ;
    mn[i].created = 0 ; 
    mn[i].data = &dn[i*4] ;
  }
  for(j=0;j<minimum(nc,cz);j++){
    p = &m[j*rz] -1;
    pn = &mn[j*nr] -1 ;
    for(i=0;i<minimum(nr,rz);i++){
      *(++pn) = *(++p) ;
    }
  }

  for(j=cz;j<nc;j++)
    pn = &mn[j*nr+rz]-1 ;
    for(i=rz;i<nr;i++){
      *(pn++) = (CtrlPnt4D)0 ;
  }

#else
  for(i=0;i<nr;++i){
    for(j=0;j<nc;++j){
      if(mn[i*nc+j].created && mn[i*nc+j].data)
	delete [] mn[i*nc+j].data ;
      mn[i*nc+j].created = 0 ; 
      mn[i*nc+j].data = &dn[(nr*j+i)*4] ;
    }
  }
  for(i=0;i<minimum(nr,rz);i++){
    p = &m[i*cz] -1;
    pn = &mn[i*nc] -1 ;
    for(j=0;j<minimum(nc,cz);j++){
      *(++pn) = *(++p) ;
    }
  }

  for(i=rz;i<nr;i++){
    pn = &mn[i*nc+cz]-1 ;
    for(j=cz;j<nc;j++)
      *(pn++) = (CtrlPnt4D)0 ;
  }

#endif

  rz = nr ;
  cz = nc ;

  if(m && created){
    delete []m ;
  }
  created = 1 ;
  m = mn ;
  if((nr*nc)>0)
    m[0].created = 1 ;
  if(vm)
    delete []vm ;
#ifdef COLUMN_ORDER
  vm = new CtrlPnt4D* [cz] ;
  for(i=0;i<cz;++i)
    vm[i] = &m[i*rz] ;
#else
  vm = new CtrlPnt4D* [rz] ;
  for(i=0;i<rz;++i)
    vm[i] = &m[i*cz] ;
#endif
}

#endif // not a SGICOMPILER

#if defined(__GNUG__) && !defined(CPP_SRC_INCLUDE)
// Template instantiation for g++ to be used with -fno-implici-templates

template class Basic2DArray<int> ;
template class Basic2DArray<float> ;
template class Basic2DArray<char> ;
template class Basic2DArray<double> ;
template class Basic2DArray<Complex> ;
template class Basic2DArray<unsigned char> ;
template class Basic2DArray<Vector2>;
template class Basic2DArray<Color> ;
template class Basic2DArray<Point3D> ;
template class Basic2DArray<CtrlPnt4D> ;
template class Basic2DArray<Coordinate> ; 

template istream& operator>>(istream& is, Basic2DArray<int>& ary);
template istream& operator>>(istream& is, Basic2DArray<float>& ary);
template istream& operator>>(istream& is, Basic2DArray<char>& ary);
template istream& operator>>(istream& is, Basic2DArray<double>& ary);
template istream& operator>>(istream& is, Basic2DArray<Complex>& ary);
template istream& operator>>(istream& is, Basic2DArray<unsigned char>& ary);
template istream& operator>>(istream& is, Basic2DArray<Color>& ary);
template istream& operator>>(istream& is, Basic2DArray<Point3D>& ary);
template istream& operator>>(istream& is, Basic2DArray<CtrlPnt4D>& ary);
template istream& operator>>(istream& is, Basic2DArray<Coordinate>& ary);

template ostream& operator<<(ostream& os, const Basic2DArray<int>& ary);
template ostream& operator<<(ostream& os, const Basic2DArray<float>& ary);
template ostream& operator<<(ostream& os, const Basic2DArray<char>& ary);
template ostream& operator<<(ostream& os, const Basic2DArray<double>& ary);
template ostream& operator<<(ostream& os, const Basic2DArray<Complex>& ary);
#ifndef GNU_GCC_2_8
template ostream& operator<<(ostream& os, const Basic2DArray<unsigned char>& ary);
#endif
template ostream& operator<<(ostream& os, const Basic2DArray<Color>& ary);
template ostream& operator<<(ostream& os, const Basic2DArray<Point3D>& ary);
template ostream& operator<<(ostream& os, const Basic2DArray<CtrlPnt4D>& ary);
template ostream& operator<<(ostream& os, const Basic2DArray<Coordinate>& ary);

#else


void dumbSPAR_Basic2DArray(){
  Basic2DArray<int> a;
  Basic2DArray<char> b;
  Basic2DArray<double> c;
  Basic2DArray<Complex> d;
  Basic2DArray<unsigned char> e;
  Basic2DArray<Vector2> f;  
  Basic2DArray<Point3D> g ;
  Basic2DArray<CtrlPnt4D> h ;
  Basic2DArray<float> i ;
  Basic2DArray<Color> j ;
  Basic2DArray<Coordinate> k ;

  cout << a << endl ;
  cout << b << endl ;
  cout << c << endl ;
  cout << d << endl ;
  cout << e << endl ;
  cout << f << endl ;
  cout << g << endl ;
  cout << h << endl ;
  cout << i << endl ;
  cout << j << endl ;
  cout << k << endl ;

  a = a ;
  b = b ;
  c = c ;
  d = d ;
  e = e ;
  f = f ;
  g = g ;
  h = h ;
  i = i ;
  j = j ;
}

#endif

