/*=============================================================================
        File: specialType.cc
     Purpose:
    Revision: $Id: specialType.cc,v 1.2 1998/05/19 18:53:57 lavoie Exp $
  Created by:    Philippe Lavoie          (3 Oct, 1996)
 Modified by: 

 Copyright notice:
          Copyright (C) 1996-1997 Philippe Lavoie
 
	  This library is free software; you can redistribute it and/or
	  modify it under the terms of the GNU Library General Public
	  License as published by the Free Software Foundation; either
	  version 2 of the License, or (at your option) any later version.
 
	  This library is distributed in the hope that it will be useful,
	  but WITHOUT ANY WARRANTY; without even the implied warranty of
	  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
	  Library General Public License for more details.
 
	  You should have received a copy of the GNU Library General Public
	  License along with this library; if not, write to the Free
	  Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
=============================================================================*/

#include "specialType.hh"

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
      Routine: operator+ --- the addition operator
  Description: adds two points together
        Input: a --> the first point
               b --> the second point
       Output: the result of $a+b$
 Restrictions:
   Created by: Philippe Lavoie (24 January 1997)
  Modified by:
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
Vector2 operator+(const Vector2& a,const Vector2& b) {
  Vector2 sum(a) ;
  sum += b ;
  return sum ;
}


/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
      Routine: operator- --- The substraction operator
  Description: The substraction operator
        Input: a --> the element to substract from
	       b --> the element to substract
       Output: The substraction of $a-b$
 Restrictions:
   Created by: Philippe Lavoie (24 January 1997)
  Modified by:
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
Vector2 operator-(const Vector2& a,const Vector2& b) {
  Vector2 diff(a) ;
  diff -= b ;
  return diff ;
}


/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
      Routine: operator== --- The equality operator
  Description: The equality operator
        Input: a --> the first element to compare
	       b --> the second element to compare
       Output: 1 if they are equal, 0 otherwise
 Restrictions:
   Created by: Philippe Lavoie (24 January 1997)
  Modified by:
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
int operator==(const Vector2&a, double b) {
  if(a.x == b && a.y == b)
    return 1 ;
  return 0 ;
}

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
      Routine: angle --- the angle between two Vector2
  Description: The angle between two Vector2
        Input: a --> the first vector
	       b --> the second vector
       Output: the angle in radians between $ab$.
 Restrictions:
   Created by: Philippe Lavoie (24 January 1997)
  Modified by:
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
double angle(const Vector2& a,const Vector2& b) {
  if(b==0 || a==0)
    return 0 ;
  return dot(a,b)/abs(a)/abs(b) ;
}

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
      Routine: dot --- The dot product of two Vector2
  Description: The dot product of two Vector2
        Input: a --> the first vector
	       b --> the second vector
       Output: the dot product $a.b$
 Restrictions:
   Created by: Philippe Lavoie (24 January 1997)
  Modified by:
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
double dot(const Vector2& a,const Vector2& b) {
  return a.x*b.x + a.y*b.y ;
}


/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
      Routine: operator* --- multiplies a Vector2 by a double
  Description: Multiplies a Vector2 by a double
        Input: a --> the double value
               b --> the vector to multiply
       Output: the vector $b$ multiplied by $a$.
 Restrictions:
   Created by: Philippe Lavoie (24 January 1997)
  Modified by:
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
Vector2 operator*(const double a,const Vector2& b) {
  Vector2 mul(b) ;
  mul.x *= a ;
  mul.y *= a ;
  return mul ;
};

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
      Routine: operator/ --- division of a Vector2 by a double
  Description: Division of a Vector2 by a double
        Input: a --> the vector
	       b --> the double value
       Output: The vector equal to the elements of a divided by b.
 Restrictions:
   Created by: Philippe Lavoie (24 January 1997)
  Modified by:
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
Vector2 operator/(const Vector2& a,const double b) {
  Vector2 div(a) ;
  div.x /= b ;
  div.y /= b ;
  return div ;
}

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
      Routine: operator<< --- output a Vector2 to the standard ostream
  Description: Output a Vector2 to the standard ostream
        Input: os --> an ostream
               v2 --> the vector to output
       Output: the ostream having v2 added
 Restrictions:
   Created by: Philippe Lavoie (24 January 1997)
  Modified by:
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
ostream& operator<<(ostream& os,const Vector2& v2)
{
  os << v2.x << " " << v2.y << " " ;
  return os;	
}

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
      Routine: operator>> --- input a Vector2 to the standard ostream
  Description: Output a Vector2 to the standard ostream
        Input: os --> an ostream
               v2 --> the vector to output
       Output: the ostream having v2 added
 Restrictions:
   Created by: Philippe Lavoie (24 January 1997)
  Modified by:
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
istream& operator>>(istream& is, Vector2& v2)
{
  is >> v2.x >> v2.y ;
  return is;	
}

// CtrlPnt definitions

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
       Member: operator= --- the assignment operator with a CtrlPnt4D
  Description: The assignment operator with a CtrlPnt4D
        Input: v --> the control point to assign to
       Output: a reference to itself
 Restrictions:
   Created by: Philippe Lavoie (24 January 1997)
  Modified by:
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
CtrlPnt4D& CtrlPnt4D::operator=(const CtrlPnt4D& v){
  data[0] = v.data[0] ; 
  data[1] = v.data[1] ;
  data[2] = v.data[2] ;
  data[3] = v.data[3] ; 
  return *this ;
}

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
       Member: operator+= --- the increment operator on a CtrlPnt4D
  Description: The increment operator on a CtrlPnt4D
        Input: v --> the value to increment the control point with
       Output: A reference to itself.
 Restrictions:
   Created by: Philippe Lavoie (24 January 1997)
  Modified by:
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
CtrlPnt4D& CtrlPnt4D::operator+=(const CtrlPnt4D& v){
  data[0] += v.data[0] ; 
  data[1] += v.data[1] ;
  data[2] += v.data[2] ;
  data[3] += v.data[3] ; 
  return *this ;
}

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
       Member: operator-= --- the decrement operator on a CtrlPnt4D
  Description: The decrement operator on a CtrlPnt4D
        Input: v --> the value of the decrement 
       Output: A reference to itself.
 Restrictions:
   Created by: Philippe Lavoie (24 January 1997)
  Modified by:
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
CtrlPnt4D& CtrlPnt4D::operator-=(const CtrlPnt4D& v){
  data[0] -= v.data[0] ; 
  data[1] -= v.data[1] ;
  data[2] -= v.data[2] ;
  data[3] -= v.data[3] ; 
  return *this ;
}

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
       Member: operator/= --- divide itself by a float
  Description: Divide itself by a float
        Input: v --> the value to divide itself by
       Output: the control point divided by $v$
 Restrictions:
   Created by: Philippe Lavoie (24 January 1997)
  Modified by:
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
CtrlPnt4D& CtrlPnt4D::operator/=(float v){
  data[0] /= v ; 
  data[1] /= v ;
  data[2] /= v ;
  data[3] /= v ; 
  return *this ;
}

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
      Routine: operator+ --- the addition operator for a CtrlPnt4D
  Description: The addition operator for CtrlPnt4D
        Input: a --> the first point
               b --> the second point
       Output: $a+b$
 Restrictions:
   Created by: Philippe Lavoie (24 January 1997)
  Modified by:
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
CtrlPnt4D operator+(const CtrlPnt4D& a,const CtrlPnt4D& b) {
  CtrlPnt4D sum(a) ;
  sum += b ;
  return sum ;
}


/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
      Routine: operator- --- the substraction operator for a CtrlPnt4D
  Description: The substraction operator for a CtrlPnt4D
        Input: a --> the first vector
               b --> the second vector
       Output: $a-b$
 Restrictions:
   Created by: Philippe Lavoie (24 January 1997)
  Modified by:
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
CtrlPnt4D operator-(const CtrlPnt4D& a,const CtrlPnt4D& b) {
  CtrlPnt4D diff(a) ;
  diff -= b ;
  return diff ;
}


/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
      Routine: operator== --- The equality operator with a float
  Description: The equality operator with a float. Verifies if every elements
               of $a$ are equal to $b$.
        Input: a --> the control point
	       b --> the floating point value
       Output: 1 if every elements of $a$ is equal to $b$, 0 otherwise
 Restrictions:
   Created by: Philippe Lavoie (24 January 1997)
  Modified by:
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
int operator==(const CtrlPnt4D& a, float b) {
  int r ;
  r = 1 ;
  for(int i=0;i<4;i++)
    r = r && a.data[i]==b ;
  return r ;
}

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
      Routine: operator!= --- the inequality operator between two points
  Description: The inequality operator between two points
        Input: a --> the first vector
               b --> the second vector
       Output: 1 if they are different, 0 otherwise
 Restrictions: This might be of limited value since comparing two floating
               point numbers doesn't usually give the proper result.
   Created by: Philippe Lavoie (24 January 1997)
  Modified by:
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
int operator!=(const CtrlPnt4D& a, const CtrlPnt4D& b){
  return !(a==b) ;
}

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
      Routine: operator== --- the equality operator between two points
  Description: The equality operator between two points
        Input: a --> the first vector
               b --> the second vector
       Output: 1 if they are equal, 0 otherwise
 Restrictions: This might be of limited value since comparing two floating
               point numbers doesn't usually give the proper result.
   Created by: Philippe Lavoie (24 January 1997)
  Modified by:
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
int operator==(const CtrlPnt4D& a, const CtrlPnt4D& b){
  int r = 1 ;
  for(int i=0;i<4;i++)
    r = r && a.data[i]==b.data[i] ;
  return r ;
}

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
      Routine: angle --- the angle between two control points
  Description: The angle between two control points
        Input: a --> the first vector
               b --> the second vector
       Output: the angle in radians between the two points
 Restrictions:
   Created by: Philippe Lavoie (24 January 1997)
  Modified by:
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
float angle(const CtrlPnt4D& a,const CtrlPnt4D& b) {
  if(b==0 || a==0 )
    return 0 ;
  return dot(a,b)/abs(a)/abs(b) ;
}

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
      Routine: abs2 --- the sum of the square of all the elements of a point
  Description: The sum of the square of all the elements of a point
        Input: a --> the point
       Output: $a_x^2+a_y^2+a_z^2+a_w^2$
 Restrictions:
   Created by: Philippe Lavoie (24 January 1997)
  Modified by:
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
float abs2(const CtrlPnt4D& a){
  double temp = 0 ;
  for(int i=0;i<4;i++)
    temp += a.data[i]*a.data[i] ;
  return temp ;
}

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
      Routine: dot --- The dot product between two points
  Description: The dot product between two points in 4D.
        Input: a --> the first vector
               b --> the second vector
       Output: $a.b$
 Restrictions:
   Created by: Philippe Lavoie (24 January 1997)
  Modified by:
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
float dot(const CtrlPnt4D& a,const CtrlPnt4D& b) {
  double temp = 0 ;
  for(int i=0;i<4;i++)
    temp += a.data[i]*b.data[i] ;
  return temp ;
}


/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
      Routine: operator/ --- Divide a point by a float
  Description: Divide a point by a float
        Input: a --> the point in 4D
               b --> the floating point value
       Output: $a/b$
 Restrictions:
   Created by: Philippe Lavoie (24 January 1997)
  Modified by:
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
CtrlPnt4D operator/(const CtrlPnt4D& a,const float b) {
  CtrlPnt4D div(a) ;
  for(int i=0;i<4;i++)
    div.data[i] /= b ;
  return div ;
}

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
      Routine: project --- projects a point in 4D to a point in 3D
  Description: Projects a point in 4D to a point in 3D
        Input: a --> the point in 4D
       Output: the point in 3D $(a_x/a_w, a_y/a_w, a_z/a_w)$
 Restrictions:
   Created by: Philippe Lavoie (24 January 1997)
  Modified by:
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
Point3D project(const CtrlPnt4D& a) {
  Point3D p ;
  for(int i=0;i<3;i++)
    p.data[i] = a.data[i]/a.data[3] ;
  return p ;
}

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
       Member: move --- translates a point in 4D by a point in 3D
  Description: Translates a point in 4D by a point in 3D. This will only
               modify the $x,y$ and $z$ component of the point in 4D.
	       It will not modify the $w$.
        Input: m --> the point in 3D
       Output: $a'_x = a_x+m_x, a'_y = a_y+m_y, a'_z = a_z+m_z, a'_w = a_w$
 Restrictions:
   Created by: Philippe Lavoie (24 January 1997)
  Modified by:
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
void CtrlPnt4D::move(const Point3D& m) {
  for(int i=0;i<3;i++)
    data[i] += m.data[i]*data[3] ;
}

//ColorF CtrlPnt4D::selectColor(1.0,1.0,1.0) ;
//ColorF CtrlPnt4D::deselectColor(1.0,1.0,0.0) ;


/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
      Routine: operator<< --- the output operator to an ostream
  Description: The output operator to an ostream
        Input:  os --> the ostream
             point --> the point to output
       Output: the ostream with the point 
 Restrictions:
   Created by: Philippe Lavoie (24 January 1997)
  Modified by:
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
ostream& operator<<(ostream& os,const CtrlPnt4D& point)
{
  os << point.x() << " " << point.y() << " " << point.z() << " " << point.w() << " " ;
  return os;	
}

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
      Routine: operator>> --- the input operator from an istream
  Description: Initialize a point in 4D from the input stream.
        Input:  os --> the input stream
             point <-- the point to initialize
       Output: the istream without the point
 Restrictions:
   Created by: Philippe Lavoie (24 January 1997)
  Modified by:
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
istream& operator>>(istream& os, CtrlPnt4D& point){
  float x,y,z,w ;
  os >> x >> y >> z >> w;
  point.data[0] = x ;
  point.data[1] = y ; 
  point.data[2] = z ;
  point.data[3] = w ;
  return os ;
}

// Point3D definitions

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
      Routine: operator+ --- the addition operator with points in 3D
  Description: The addition operator with points in 3D
        Input: a --> the first point in 3D
	       b --> the second point in 3D
       Output: $a+b$
 Restrictions:
   Created by: Philippe Lavoie (24 January 1997)
  Modified by:
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
Point3D operator+(const Point3D& a,const Point3D& b) {
  Point3D sum(a) ;
  sum += b ;
  return sum ;
}


/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
      Routine: operator- --- the subtraction operator with points in 3D
  Description: The subtraction operator with points in 3D
        Input: a --> the first point in 3D
	       b --> the second point in 3D
       Output: $a-b$
 Restrictions:
   Created by: Philippe Lavoie (24 January 1997)
  Modified by:
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
Point3D operator-(const Point3D& a,const Point3D& b) {
  Point3D diff(a) ;
  diff -= b ;
  return diff ;
}


/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
      Routine: operator== --- the equality operator with a float
  Description: Finds if all the elements of the point in 3D are equal to $b$
        Input: a --> the point in 3D
	       b --> the floating point value
       Output: 1 if equal, 0 otherwise
 Restrictions:
   Created by: Philippe Lavoie (24 January 1997)
  Modified by:
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
int operator==(const Point3D&a, float b) {
  if(a.x == b && a.y == b && a.z==b)
    return 1 ;
  return 0 ;
}

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
      Routine: operator!= --- the inequality operator between points in 3D
  Description: The inequality operator between points in 3D.
        Input: a --> the first point in 3D
	       b --> the second point in 3D
       Output: 1 if they are different, 0 otherwise
 Restrictions:
   Created by: Philippe Lavoie (24 January 1997)
  Modified by:
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
int operator!=(const Point3D& a, const Point3D& b){
  if(a.x == b.x && a.y == b.y && a.z == b.z)
    return 0 ;
  else
    return 1 ;
}

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
      Routine: operator== --- the equality operator between points in 3D
  Description: The equality operator between points in 3D.
        Input: a --> the first point in 3D
	       b --> the second point in 3D
       Output: 1 if they are equal, 0 otherwise
 Restrictions:
   Created by: Philippe Lavoie (24 January 1997)
  Modified by:
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
int operator==(const Point3D& a, const Point3D& b){
  if(a.x == b.x && a.y == b.y && a.z == b.z)
    return 1 ;
  else
    return 0 ;
}

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
      Routine: angle --- Finds the angle between two points in 3D
  Description: Finds the angle between two points in 3D
        Input: a --> the first point in 3D
	       b --> the second point in 3D
       Output: The angle in radian between the two points
 Restrictions:
   Created by: Philippe Lavoie (24 January 1997)
  Modified by:
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
float angle(const Point3D& a,const Point3D& b) {
  if(b==0 || a==0 )
    return 0 ;
  return acos(dot(a,b)/abs(a)/abs(b)) ;
}

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
      Routine: dot --- the dot product of two points in 3D
  Description: The dot product of two points in 3D
        Input: a --> the first point in 3D
	       b --> the second point in 3D
       Output: $a.b$
 Restrictions:
   Created by: Philippe Lavoie (24 January 1997)
  Modified by:
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
float dot(const Point3D& a,const Point3D& b) {
  return a.x*b.x + a.y*b.y + a.z*b.z ;
}


/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
      Routine: operator* --- multiplies a point in 3D with a float
  Description: Multiplies a point in 3D with a float
        Input: a --> the floating point value
	       b --> the point in 3D
       Output: $ab$
 Restrictions:
   Created by: Philippe Lavoie (24 January 1997)
  Modified by:
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
Point3D operator*(const float a,const Point3D& b) {
  Point3D mul(b) ;
  mul.x *= a ;
  mul.y *= a ;
  mul.z *= a ;
  return mul ;
};

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
      Routine: operator/ --- Divide a point in 3D by a float
  Description: Divide a point in 3D by a float.
        Input: a --> the point in 3D
	       b --> the floating point value to divide with
       Output: $a/b$
 Restrictions:
   Created by: Philippe Lavoie (24 January 1997)
  Modified by:
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
Point3D operator/(const Point3D& a,const float b) {
  Point3D div(a) ;
  div.x /= b ;
  div.y /= b ;
  div.z /= b ;
  return div ;
}

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
      Routine: operator<< ---  the output operator of a point in 3D 
                               to an ostream
  Description: The output operator of a point in 3D to an ostream .
        Input:  os --> the ostream
             point --> the point to output
       Output: the ostream with the point 
 Restrictions:
   Created by: Philippe Lavoie (24 January 1997)
  Modified by:
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
ostream& operator<<(ostream& os,const Point3D& point)
{
  os << " " << point.x << " " << point.y << " " << point.z << " " ;
  return os;	
}

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
      Routine: operator>> --- the input operator from an istream
  Description: Initialize a point in 3D from the input stream.
        Input:  os --> the input stream
             point <-- the point to initialize
       Output: the istream without the point
 Restrictions:
   Created by: Philippe Lavoie (24 January 1997)
  Modified by:
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
istream& operator>>(istream& os, Point3D& point){
  float x,y,z ;
  os >> x >> y >> z ;
  point.x = x ;
  point.y = y ; 
  point.z = z ;
  return os ;
}


/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
      Routine: operator<< --- the output operator of a color to an ostream
  Description: The output operator of a color to an ostream
        Input: os --> the ostream
                c --> the color to output
       Output: the ostream with the color $c$.
 Restrictions:
   Created by: Philippe Lavoie (24 January 1997)
  Modified by:
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
ostream& operator<<(ostream& os,const Color& c)
{
  os << (int)c.r << " " << (int)c.g << " " << (int)c.b << " " ;
  return os;	
}

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
      Routine: operator>> --- the input operator of a color from an istream
  Description: Initialize a color from an istream
        Input: os --> the input stream
	        c --> the color to initialize
       Output: the istream without the color
 Restrictions:
   Created by: Philippe Lavoie (24 January 1997)
  Modified by:
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
istream& operator>>(istream& os, Color& c){
  os >> c.r >> c.g >> c.b ;
  return os ;
}

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
      Routine: operator<< --- the output operator of a coordinate to an ostream
  Description: The output operator of a coordinate to an ostream
        Input: os --> the ostream
                c --> the coordinate to output
       Output: the ostream with the coordinate $c$.
 Restrictions:
   Created by: Philippe Lavoie (24 January 1997)
  Modified by:
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
ostream& operator<<(ostream& os,const Coordinate& c)
{
  os << c.i << " " << c.j << " " ;
  return os;	
}

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
      Routine: operator>> --- the input operator of a coordinate from an 
                              istream
  Description: Initialize a coordinate from an istream
        Input: os --> the input stream
	        c --> the coordinate to initialize
       Output:the istream without the coordinate
 Restrictions:
   Created by: Philippe Lavoie (24 January 1997)
  Modified by:
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
istream& operator>>(istream& os, Coordinate& c){
  os >> c.i >> c.j ;
  return os ;
}

template <class T>
int compareT(T*a, T* b){
  if(*(a) > *(b))
    return 1 ;
  if(*(a) < *(b))
    return -1 ;
  return 0 ;
}

int compareInt(const void* a, const void* b){
  return compareT((int*)a,(int*)b) ;
}

int compareFloat(const void* a, const void* b){
  return compareT((float*)a,(float*)b) ;
}

int compareDouble(const void* a, const void* b){
  return compareT((float*)a,(float*)b) ;
}

#if defined(__GNUG__) && !defined(CPP_SRC_INCLUDE)


#ifndef GNU_GCC_2_8

template int maximum(int,int) ;
template float maximum(float,float) ;
template double maximum(double,double) ;
template unsigned char maximum(unsigned char,unsigned char) ;
template char maximum(char,char) ;
template char maximum(long,long) ;
template Point3D maximum(Point3D, Point3D) ;
template CtrlPnt4D maximum(CtrlPnt4D, CtrlPnt4D) ;
//template Complex maximum(Complex, Complex) ;
template Coordinate maximum(Coordinate, Coordinate) ;
template Vector2 maximum(Vector2, Vector2) ;

template Point3D maximumRef(const Point3D&, const Point3D&) ;
template CtrlPnt4D maximumRef(const CtrlPnt4D&, const CtrlPnt4D&) ;
//template Complex maximumRef(const Complex&, const Complex&) ;
template Coordinate maximumRef(const Coordinate&, const Coordinate&) ;
template Vector2 maximumRef(const Vector2&, const Vector2&) ;

template int minimum(int,int) ;
template float minimum(float,float) ;
template double minimum(double,double) ;
template unsigned char minimum(unsigned char,unsigned char) ;
template char minimum(char,char) ;
template char minimum(long,long);
template Point3D minimum(Point3D, Point3D) ;
template CtrlPnt4D minimum(CtrlPnt4D, CtrlPnt4D) ;
//template Complex minimum(Complex, Complex) ;
template Coordinate minimum(Coordinate, Coordinate) ;
template Vector2 minimum(Vector2, Vector2) ;

template Point3D minimumRef(const Point3D&, const Point3D&) ;
template CtrlPnt4D minimumRef(const CtrlPnt4D&, const CtrlPnt4D&) ;
//template Complex minimumRef(const Complex&, const Complex&) ;
template Coordinate minimumRef(const Coordinate&, const Coordinate&) ;
template Vector2 minimumRef(const Vector2&, const Vector2&) ;

#endif

template compareT(int*, int*);
template compareT(float*, float*);
template compareT(double*, double*);

template double absolute(double) ;
template float absolute(float) ;
template int absolute(int) ;
template char absolute(char) ;
template long absolute(long) ;

template void boundTo( float&, float, float);

#endif

Color whiteColor(255,255,255);
Color redColor(255,0,0) ;
Color blueColor(0,0,255) ;
Color greenColor(0,255,0) ;
Color yellowColor(255,255,0) ;
Color cyanColor(0,255,255) ;
Color magentaColor(255,0,255);
Color gray80Color(204,204,204) ;
Color gray50Color(127,127,127) ;
Color blackColor(0,0,0) ;

