/*=============================================================================
        File: vector.cc
     Purpose:
    Revision: $Id: vector.cc,v 1.1.1.1 1998/04/06 21:02:32 lavoie Exp $
  Created by:    Philippe Lavoie          (3 Oct, 1996)
 Modified by: 

 Copyright notice:
          Copyright (C) 1996-1998 Philippe Lavoie
 
	  This library is free software; you can redistribute it and/or
	  modify it under the terms of the GNU Library General Public
	  License as published by the Free Software Foundation; either
	  version 2 of the License, or (at your option) any later version.
 
	  This library is distributed in the hope that it will be useful,
	  but WITHOUT ANY WARRANTY; without even the implied warranty of
	  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
	  Library General Public License for more details.
 
	  You should have received a copy of the GNU Library General Public
	  License along with this library; if not, write to the Free
	  Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
=============================================================================*/

#include "internals.hh"

#include "vector.hh"
//#include <stdlib.h>

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
       Member: operator=  --- the assignment operator
  Description: The values of a vector are copied to this one
        Input: b --> the vector to copy
       Output: a reference to itself
 Restrictions:
   Created by: Philippe Lavoie (24 January 1997)
  Modified by:
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
template<class T>
PlVector<T>& PlVector<T>::operator=(const PlVector<T> &b)
{
  if(this==&b)
    return *this ;

  if ( n != b.n)
    {
      resize(b.n) ;
    }

  sze = b.n ;
  T *pa, *pb ;
  pa = x-1 ;
  pb = b.x-1 ;
  for(int i=0;i<n;i++){
    *(++pa) = *(++pb) ;
  }
  return *this;

}

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
       Member: operator=  --- the assignment operator with a BasicArray
  Description: The values of a vector are copied to this one
        Input: b --> the BasicArray to copy
       Output: a reference to itself
 Restrictions:
   Created by: Philippe Lavoie (24 January 1997)
  Modified by:
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
template<class T>
PlVector<T>& PlVector<T>::operator=(const BasicArray<T> &b)
{
  if ( size() != b.size())
    {
      resize(b.size()) ;
    }
  T *ptr ;
  ptr = x - 1 ;
  for(int i=0;i<size();i++)
     *(++ptr) = b[i] ;

  return *this;
}

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
       Member: opeator= --- assigns all the components of the vector to a value
  Description: All the components of the vector are assigned to the value $d$
        Input: d --> the value to assigned the vector to
       Output: the value of $d$
 Restrictions:
   Created by: Philippe Lavoie (24 January 1997)
  Modified by:
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
template<class T>
T PlVector<T>::operator=(const T d)
{
  const int sz = size(); 
  T *ptr ;
  ptr = x-1 ;
  for (int i = 0; i < sz; i++)
    *(++ptr) = d ;

  return d;	
}

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
       Member: operator+= --- the += operator
  Description: Each component of the vector is increased by the components of 
               vector $a$.
        Input: a --> the vector to add to itself
       Output: a reference to itself
 Restrictions: The vectors must have the {\em same} size.
   Created by: Philippe Lavoie (24 January 1997)
  Modified by:
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
template<class T>
PlVector<T>& PlVector<T>::operator+=(const PlVector<T> &a)
{
  if ( a.size() != size())
    {
      PlError error("PlVector<T>::operator+=(PlVector<T>&)");
      error << "PlVector<T> a += PlVector<T> b different sizes, a = " << size() << ", b = " << a.size() ;
      error.fatal() ;
    }
  const int sz = size();
  T *ptr,*aptr ;
  ptr = x-1 ;
  aptr = a.x-1 ;
  for (int i = 0; i < sz; i++)
    *(++ptr) += *(++aptr) ;
  
  return *this;
}

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
       Member: operator-= ---  the -= operator
  Description: Each component of the vector is decreased by the components of vector $a$.
        Input: a --> the vector to substract from itself
       Output: a reference to itself
 Restrictions: The vectors must have the {\em same} size.
   Created by: Philippe Lavoie (24 January 1997)
  Modified by:
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
template<class T>
PlVector<T>& PlVector<T>::operator-=(const PlVector<T> &a)
{
  if ( a.size() != size())
    {
      PlError error("PlVector<T>::operator-=(PlVector<T>&)");
      error << "PlVector<T> a -= PlVector<T> b different sizes, a = " << size() << ", b = " << a.size() ;
      error.fatal() ;
    }
  
  const int sz = size(); 
  T *ptr,*aptr ;
  ptr = x-1 ;
  aptr = a.x-1 ;
  for (int i = 0; i < sz; i++)
    *(++ptr) -= *(++aptr) ;
  
  return *this;
}

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
      Routine: operator+ --- Adds two vectors
  Description: The addition of two vectors
        Input: a --> the first vector to add
               b --> the second vector to add
       Output: the result of $a+b$
 Restrictions: The vectors must have the {\em same} size.
   Created by: Philippe Lavoie (24 January 1997)
  Modified by:
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
template<class T>
PlVector<T> operator+(const PlVector<T>& a, const PlVector<T> &b)	
{
  PlVector<T> sum(a);
  sum += b ;
  
  return sum;
}

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
      Routine: operator- --- Substracts two vectors
  Description: The substraction of two vectors
        Input: a --> the first vector to add
               b --> the second vector to add
       Output: the result of $a-b$
 Restrictions: The vectors must have the {\em same} size.
   Created by: Philippe Lavoie (24 January 1997)
  Modified by:
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
template<class T>
PlVector<T> operator-(const PlVector<T>& a, const PlVector<T> &b)
{
  PlVector<T> diff(a);
  diff -= b ;
  return diff;
}

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
      Routine: operator* --- the multiplicative operator
  Description: Multiplies two vectors
        Input: a --> the first vector
               b --> the second vector to multiply with
       Output: $a*b$
 Restrictions: The vectors must have the {\em same} size.
   Created by: Philippe Lavoie (24 January 1997)
  Modified by:
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
template<class T>
T operator*(const PlVector<T> &a,const PlVector<T> &b)
{
  if ( a.size() != b.size() )
    {
      PlError error("PlVector<T>::operator=(PlVector<T>&)");
      error << "PlVector<T> a = PlVector<T> b different sizes, a = " << a.size() << ", b = " << b.size() ;
      error.fatal() ;
    }

  int i, sz = a.size();
  T prod, zero = (T)0;
  T *aptr,*bptr ;
  aptr = a.x-1 ;
  bptr = b.x-1 ;
  prod = zero ;
  for (i = 0 ; i < sz; i++)
    prod += (*(++aptr)) * (*(++bptr)) ;
  
  return prod;
}

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
      Routine: operator* --- multiplies a vector with a double
  Description: Multiplies all the elements of the vector $v$ with a double $d$.
        Input: v --> the vector to multiply
               d --> multiply the vector by this value
       Output: a vector having value $V \times d$
 Restrictions: 
   Created by: Philippe Lavoie (24 January 1997)
  Modified by:
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
template<class T>
PlVector<T>  operator*(const PlVector<T>& v, const double d)
{
  int i, sz = v.size();
  PlVector<T> b(v.size());
  
  T *aptr,*bptr ;
  aptr = v.x-1 ;
  bptr = b.x-1 ;
  for (i = 0; i < sz; ++i){
    *(++bptr) = (T)(d * (*(++aptr)) ) ;
  }
  
  return b;

}

PlVector<Complex>  operator*(const PlVector<Complex>& v, const double d)
{
  int i, sz = v.size();
  PlVector<Complex> b(v.size());
  
  Complex *aptr,*bptr ;
  aptr = v.x-1 ;
  bptr = b.x-1 ;
  for (i = 0; i < sz; ++i){
    *(++bptr) = (Complex)(d * (*(++aptr)) ) ;
  }
  
  return b;

}


/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
      Routine: operator* --- multiplies the vector with a complex number
  Description: Multiplies a vector with a complex number
        Input: v --> the vector to multiply
               d --> the complex value to multiply $v$ with
       Output: a vector having $v \times d$
 Restrictions: If the vector is not of complex type, then the vector will be multiplied by the real
          part of $d$. 
   Created by: Philippe Lavoie (24 January 1997)
  Modified by:
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
template<class T>
PlVector<T>  operator*(const PlVector<T>& v, const Complex d)
{
  int i, sz = v.size();
  PlVector<T> b = v;
  
  T *bptr ;
  bptr = b.x-1 ;
  for (i = 0; i < sz; i++){
    ++bptr ;
    (*bptr) = (T)(real(d)*(*bptr));
  }
  
  return b;

}

PlVector<Complex>  operator*(const PlVector<Complex>& v, 
			   const Complex d)
{
  int i, sz = v.size();
  PlVector<Complex> b = v;
  
  Complex *bptr=b.x ;
  for (i = 0; i < sz; i++)
    *(++bptr) *= d;
  
  return b;

}

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
      Routine: operator== --- the equality operator
  Description: Checks if a vector is equal to another one
        Input: a --> the first vector to check
               b --> the second vector to check   
       Output: 1if the vectors are equal, 0 otherwise
 Restrictions:
   Created by: Philippe Lavoie (24 January 1997)
  Modified by:
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
template<class T>
int operator==(const PlVector<T> &a,const PlVector<T> &b)
{
  if ( a.size() != b.size() )
    {
      return 0 ;
    }

  int i, sz = a.size();
  int l = 1;
  
  T *aptr,*bptr ;
  aptr = a.x-1 ;
  bptr = b.x-1 ;

  for (i = 0; i < sz; i++)
    l = l && ( *(++aptr) == *(++bptr) );
  
  return l;
}

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
       Member: as --- copies the values of $b$ to the vector starting from 
                      the index $i$.
  Description: The values of the vector $b$ replace the values of the vector 
               starting at the index $i$. 
        Input: i --> the index to start copying from 
               b --> the vector to copy from 
       Output:
 Restrictions: The vector $b$ must fit entirely into the vector when starting 
               at $i$.
   Created by: Philippe Lavoie (24 January 1997)
  Modified by:
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
template<class T> void
PlVector<T>::as(int i, const PlVector<T> &b)
{
  if ( (i + b.rows) > rows )
    {
      PlError error("PlVector<T>::as(int,PlVector<T>)");
      error << "PlVector is too long to fit at i= " << i << endl ;
      error.fatal() ;
    }

  T *aptr,*bptr ;
  aptr = &x[i]-1 ;
  bptr = b.x-1 ;
  for ( int j = 0; j<b.rows; j++)
      *(++aptr) = *(++bptr) ;
}

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
       Member: get --- extract a vector of size $l$ starting at index $i$
  Description: This extracts a vector of size $l$ by copying the values from 
               the vector starting at index $i$.
        Input: i --> the index to start copying from
               l --> the length of the new vector
       Output: A vector representing the range $[i..i+l]$ from this vector
 Restrictions: The vector extracted must fit inside the vector starting at $i$.
   Created by: Philippe Lavoie (24 January 1997)
  Modified by:
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
template<class T> PlVector<T> 
PlVector<T>::get(int i, int l)
{
  if ( (i + l) > rows )
    {
      PlError error("PlVector<T>::get(int,PlVector<T>)");
      error << "PlVector is too long to extract from  i= " << i << "from l=" << l << endl ;
      error.fatal() ;
    }

  PlVector<T> subvec(l) ;
  T *aptr, *bptr ;
  aptr = &x[i] - 1 ;
  bptr = subvec.x -1 ;
  for ( int j = 0; j < l; j++)
    *(++bptr) = *(++aptr) ;
  return subvec ;
}

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
       Member: minIndex --- finds the index of its minimal entry
  Description: Scans the vector to find its minimal value and returns the 
               index of that value.
        Input:
       Output: the index of the minimal component.
 Restrictions:
   Created by: Philippe Lavoie (24 January 1997)
  Modified by:
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
template <class T>
int PlVector<T>::minIndex() const {
  T min = x[0] ;
  int index = 0 ;

  for(int i=1;i<n;i++){
    if(x[i]<=min){
      min = x[i] ;
      index = i ;
    }
  }
  return index ;
}

int PlVector<Color>::minIndex() const {
  PlError error("PlVector<color>::minIndex") ;
  error << "ERROR: you can't get the minIndex of a vector of Colors!\n" ;
  error.fatal() ;
  return 0 ;
}

int PlVector<Complex>::minIndex() const {
  Complex min = x[0] ;
  int index = 0 ;

  Complex *p ;
  p = x-1 ;

  for(int i=1;i<n;i++){
    if(abs(*(++p))<abs(min)){
      min = *p ;
      index = i ;
    }
  }
  return index ;
}

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
       Member: qSortStd --- do a quick sort using the standard C library sort 
                            algorithm
  Description: Performs a quick sort of the vector. The quick sort algorithm 
               used is the one from the standard C library.
        Input: 
       Output: The vector is sorted in ascending order
 Restrictions: Due to the use of the standard library, only certain types can 
               be used with this function: int, float and double.
   Created by: Philippe Lavoie (21 May 1997)
  Modified by:
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
template <class T>
void PlVector<T>::qSortStd(){
  PlError error("PlVector<T>::qSort()");
  error << "qSortStd is not defined for that type.\nPlease defined it in your .C file!";
  error.fatal() ;
}

void PlVector<int>::qSortStd(){
  qsort((char*)memory(),n,sizeof(int),compareInt) ;
}

void PlVector<float>::qSortStd(){
  qsort((char*)memory(),n,sizeof(float),compareFloat) ;
}

void PlVector<double>::qSortStd(){
  qsort((char*)memory(),n,sizeof(float),compareDouble) ;
}

template <class T>
inline void swap(T& a, T& b){
  T temp = a ;
  a = b ;
  b = temp ;
}

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
       Member: qSort --- do a quick sort using an optimized algorithm
  Description: Do a quick sort of the vector using an algorithm based on the 
               one described in "Numerical Recipes in C". You should use this 
	       method over using qSortStd since it is usually faster.
        Input: M --> regions smaller than this value are sorted using the 
	             insertion method, the default value of 7 is suitable for 
		     most cases.
       Output: The vector is sorted in ascending order
 Restrictions: Some types don't have comparison operators and are not 
               supported.
   Created by: Philippe Lavoie (21 May 1997)
  Modified by:
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
template <class T>
void PlVector<T>::qSort(int M){
  const int Nstack=50 ; 
  int i,ir,j,k,l ;
  PlVector<int> istack(Nstack) ;
  int jstack=0;
  T a ;
  T *v1,*v2  ;

  ir = sze-1 ;
  l = 0 ;
  
  while(1){
    if(ir-l<M){ // perform an insertion sort when the array is small enough
      v1 = &x[l] ;
      for(j=l+1;j<=ir;++j){
	a = *(++v1) ;
	v2 = v1 ;
	--v2 ;
	for(i=j-1;i>=0;--i){
	  if(*v2 <= a) break ;
	  *(v2+1) = *v2 ;
	  --v2 ;
	}
	++v2 ;
	*(v2) = a ;
      }
      if(jstack==0) break ;
      ir=istack[--jstack] ;
      l=istack[--jstack] ;
    }
    else{
      k=(l+ir) >> 1 ;
      swap(x[k],x[l+1]) ;
      if(x[l+1] > x[ir]){
	swap(x[l+1],x[ir]) ;
      }
      if(x[l]> x[ir]){
	swap(x[l],x[ir]) ;
      }
      if(x[l+1] > x[l]){
	swap(x[l+1],x[l]) ;
      }
      i=l+1 ;
      j=ir ;
      a=x[l] ;
      v1 = &x[i] ;
      v2 = &x[j] ;
      while(1){
	while(*v1 < a) { ++i ; ++v1 ; }
	while(*v2 > a) { --j ; --v2 ; }
	if(j<i) break ;
	if(*v1 == *v2)  // both are equal to a...
	  break ;
	swap(x[i],x[j]) ;
      }
      x[l] = x[j] ;
      x[j] = a ;
      jstack += 2 ;
      if(jstack>=Nstack){
	istack.resize(istack.n+Nstack) ; // increase the vector size
      }
      if(ir-i+1 >= j-l){
	istack[jstack-1]=ir ;
	istack[jstack-2] = i ;
	ir=j-1 ;
      }
      else{
	istack[jstack-1] = j-1 ;
	istack[jstack-2] = l ;
	l=i ;
      }
    }
  }
}

void PlVector<Complex>::qSort(int Nstack){
  Nstack++ ; // shuts up the warning messages
  PlError error("PlVector<Complex>::qSort()");
  error << "Sorry! The complex quick sort operator isn't working.\n" ;
  error.fatal() ;
}

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
       Member: sortIndex --- generates sorted index vector
  Description: A sorted index vector is generated by this routine. It is based
               on a routine described in "Numercial Recipes in C".
        Input: index <-> The index vector
                   M --> regions smaller than this value are sorted using the 
		         insertion method, the default value of 7 is suitable 
			 for most cases.
       Output: index holds a sorted index vector
 Restrictions: Some types don't have comparison operators and are not
               supported.
   Created by: Philippe Lavoie (21 May 1997)
  Modified by:
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
template <class T>
void PlVector<T>::sortIndex(PlVector<int>& index, int M) const{
  const int Nstack=50 ; 
  int i,ir,j,k,l,indext ;
  PlVector<int> istack(Nstack) ;
  int jstack=0;
  T a ;

  ir = sze-1 ;
  l = 0 ;
  
  index.resize(sze) ;
  for(i=0;i<index.n;++i)
    index[i] = i ;

  while(1){
    if(ir-l<M){ // perform an insertion sort when the array is small enough
      for(j=l+1;j<=ir;++j){
	indext = index[j] ;
	a = x[indext] ;
	for(i=j-1;i>=0;--i){
	  if(x[index[i]] <= a) break ;
	  index[i+1] = index[i] ;
	}
	index[i+1] = indext ;
      }
      if(jstack==0) break ;
      ir=istack[--jstack] ;
      l=istack[--jstack] ;
    }
    else{
      k=(l+ir) >> 1 ;
      swap(index[k],index[l+1]) ;
      if(x[index[l+1]] > x[index[ir]]){
	swap(index[l+1],index[ir]) ;
      }
      if(x[index[l]]> x[index[ir]]){
	swap(index[l],index[ir]) ;
      }
      if(x[index[l+1]] > x[index[l]]){
	swap(index[l+1],index[l]) ;
      }
      i=l+1 ;
      j=ir ;
      indext = index[l] ;
      a=x[indext] ;
      while(1){
	while(x[index[i]] < a) { ++i ; }
	while(x[index[j]] > a) { --j ; }
	if(j<i) break ;
	if(x[index[i]] == x[index[j]])
	  break ;
	swap(index[i],index[j]) ;
      }
      index[l] = index[j] ;
      index[j] = indext ;
      jstack += 2 ;
      if(jstack>=Nstack){
	istack.resize(istack.n+Nstack) ; // increase the vector size
      }
      if(ir-i+1 >= j-l){
	istack[jstack-1]=ir ;
	istack[jstack-2] = i ;
	ir=j-1 ;
      }
      else{
	istack[jstack-1] = j-1 ;
	istack[jstack-2] = l ;
	l=i ;
      }
    }
  }
}

void PlVector<Complex>::sortIndex(PlVector<int>& index, int M) const {
  M = index[0] ; // shuts up the warning messages
  ++M ; // shuts up the warning messages
  PlError error("PlVector<Complex>::sortIndex()");
  error << "Sorry! The complex quick sort operator isn't working.\n" ;
  error.fatal() ;
}

#if defined(__GNUG__) && !defined(CPP_SRC_INCLUDE)

// Template instantation for g++ to be used with -fno-implicit-templates

// Integer instantation

template class PlVector<int> ;

template PlVector<int> operator+(const PlVector<int>&, const PlVector<int>&);
template PlVector<int> operator-(const PlVector<int>&, const PlVector<int>&);
template int operator*(const PlVector<int>&,const PlVector<int>&);
template PlVector<int> operator*(const PlVector<int>& v, const double d);
template PlVector<int> operator*(const PlVector<int>& v, const Complex d);
template int operator==(const PlVector<int>&,const PlVector<int>&);
template int operator!=(const PlVector<int>&,const PlVector<int>&);
template void swap(int&,int&) ;

// Byte instantation

template class PlVector<char> ;

template PlVector<char> operator+(const PlVector<char>&, const PlVector<char>&);
template PlVector<char> operator-(const PlVector<char>&, const PlVector<char>&);
template char operator*(const PlVector<char>&,const PlVector<char>&);
template PlVector<char> operator*(const PlVector<char>& v, const double d);
template PlVector<char> operator*(const PlVector<char>& v, const Complex d);
template int operator==(const PlVector<char>&,const PlVector<char>&);
template int operator!=(const PlVector<char>&,const PlVector<char>&);
template void swap(char&,char&) ;

// Double instantation

template class PlVector<double> ;

template PlVector<double> operator+(const PlVector<double>&, const PlVector<double>&);
template PlVector<double> operator-(const PlVector<double>&, const PlVector<double>&);
template double operator*(const PlVector<double>&,const PlVector<double>&);
template PlVector<double> operator*(const PlVector<double>& v, const double d);
template PlVector<double> operator*(const double d,const PlVector<double>& v);
template PlVector<double> operator*(const PlVector<double>& v, const Complex d);
template int operator==(const PlVector<double>&,const PlVector<double>&);
template int operator!=(const PlVector<double>&,const PlVector<double>&);
template void swap(double&,double&) ;

// Float instantation

template class PlVector<float> ;

template PlVector<float> operator+(const PlVector<float>&, const PlVector<float>&);
template PlVector<float> operator-(const PlVector<float>&, const PlVector<float>&);
template float operator*(const PlVector<float>&,const PlVector<float>&);
template PlVector<float> operator*(const PlVector<float>& v, const double d);
template PlVector<float> operator*(const double d,const PlVector<float>& v);
template PlVector<float> operator*(const PlVector<float>& v, const Complex d);
template int operator==(const PlVector<float>&,const PlVector<float>&);
template int operator!=(const PlVector<float>&,const PlVector<float>&);
template void swap(float&,float&) ;

// Complex instantation

template class PlVector<Complex> ;

template PlVector<Complex> operator+(const PlVector<Complex>&, const PlVector<Complex>&);
template PlVector<Complex> operator-(const PlVector<Complex>&, const PlVector<Complex>&);
template Complex operator*(const PlVector<Complex>&,const PlVector<Complex>&);
#ifndef GNU_GCC_2_8
template PlVector<Complex> operator*(const PlVector<Complex>& v, const double d);
template PlVector<Complex> operator*(const PlVector<Complex>& v, const Complex d);
#endif
template int operator==(const PlVector<Complex>&,const PlVector<Complex>&);
template int operator!=(const PlVector<Complex>&,const PlVector<Complex>&);
template void swap(Complex&,Complex&) ;

// Ubyte instantation

template class PlVector<unsigned char> ;

template PlVector<unsigned char> operator+(const PlVector<unsigned char>&, const PlVector<unsigned char>&);
template PlVector<unsigned char> operator-(const PlVector<unsigned char>&, const PlVector<unsigned char>&);
template unsigned char operator*(const PlVector<unsigned char>&,const PlVector<unsigned char>&);
template PlVector<unsigned char> operator*(const PlVector<unsigned char>& v, const double d);
template PlVector<unsigned char> operator*(const PlVector<unsigned char>& v, const Complex d);
template int operator==(const PlVector<unsigned char>&,const PlVector<unsigned char>&);
template int operator!=(const PlVector<unsigned char>&,const PlVector<unsigned char>&);
template void swap(unsigned char&,unsigned char&) ;

template class PlVector<Vector2> ;

template PlVector<Vector2> operator+(const PlVector<Vector2>&, const PlVector<Vector2>&);
template PlVector<Vector2> operator-(const PlVector<Vector2>&, const PlVector<Vector2>&);
template PlVector<Vector2> operator*(const PlVector<Vector2>& v, const double d);
template int operator==(const PlVector<Vector2>&,const PlVector<Vector2>&);
template int operator!=(const PlVector<Vector2>&,const PlVector<Vector2>&);
template void swap(Vector2&,Vector2&) ;

// Color instantation

template class PlVector<Color> ;

template PlVector<Color> operator+(const PlVector<Color>&, const PlVector<Color>&);
template PlVector<Color> operator-(const PlVector<Color>&, const PlVector<Color>&);
template unsigned char operator*(const PlVector<Color>&,const PlVector<Color>&);
template PlVector<Color> operator*(const PlVector<Color>& v, const double d);
template PlVector<Color> operator*(const PlVector<Color>& v, const Complex d);
template int operator==(const PlVector<Color>&,const PlVector<Color>&);
template int operator!=(const PlVector<Color>&,const PlVector<Color>&);
template void swap(Color&,Color&) ;

// Point3D instantation

template class PlVector<Point3D> ;

template PlVector<Point3D> operator+(const PlVector<Point3D>&, const PlVector<Point3D>&);
template PlVector<Point3D> operator-(const PlVector<Point3D>&, const PlVector<Point3D>&);
template unsigned char operator*(const PlVector<Point3D>&,const PlVector<Point3D>&);
template PlVector<Point3D> operator*(const PlVector<Point3D>& v, const double d);
template PlVector<Point3D> operator*(const PlVector<Point3D>& v, const Complex d);
template int operator==(const PlVector<Point3D>&,const PlVector<Point3D>&);
template int operator!=(const PlVector<Point3D>&,const PlVector<Point3D>&);
template void swap(Point3D&,Point3D&) ;

// CtrlPnt4D instantation

template class PlVector<CtrlPnt4D> ;

template PlVector<CtrlPnt4D> operator+(const PlVector<CtrlPnt4D>&, const PlVector<CtrlPnt4D>&);
template PlVector<CtrlPnt4D> operator-(const PlVector<CtrlPnt4D>&, const PlVector<CtrlPnt4D>&);
template unsigned char operator*(const PlVector<CtrlPnt4D>&,const PlVector<CtrlPnt4D>&);
template PlVector<CtrlPnt4D> operator*(const PlVector<CtrlPnt4D>& v, const double d);
template PlVector<CtrlPnt4D> operator*(const PlVector<CtrlPnt4D>& v, const Complex d);
template int operator==(const PlVector<CtrlPnt4D>&,const PlVector<CtrlPnt4D>&);
template int operator!=(const PlVector<CtrlPnt4D>&,const PlVector<CtrlPnt4D>&);
template void swap(CtrlPnt4D&,CtrlPnt4D&) ;

#else

void  PlVector_dumbSPARC(){
  PlVector<int> a;
  PlVector<unsigned char> b;
  PlVector<double> c1,c2,c3;
  PlVector<Complex> d;
  PlVector<Vector2> e;
  PlVector<CtrlPnt4D> f ;
  PlVector<Point3D> g ;
  PlVector<float> h ;
  PlVector<Color> i ;
  PlVector<char> j ;

  Point3D t ;
  double dd=1 ;
  
  c1 = c2-c3 ;
  c1 = c2+c3 ;
  c1 = dd*c2 ;


  f = f + dd*f ;
  f = f - f ;
  f = f * f ;
  t = project(f[0]) ;

  g = g + g ;
  g = g * g ;
  g = g-g+dd*g ;

  h = h + h + dd*h;
  h = h * h ;

  i = i + i - dd*i ;
  //i = i * i ;

  e = e  ;

  cout << c1 ;
  cout << a ;
  cout << b ;
  cout << d ;
  cout << e ;
  cout << f ;
  cout << g ;
  cout << h ;
  cout << i ;

  a == a ;
  b == b ;
  c1 == c1 ;
  d == d ;
  e == e ;
  f == f ;
  g == g ;
  h == h ;
  i == i ;

  a != a ;
  b != b ;
  c1 != c1 ;
  d != d ;
  e != e ;
  f != f ;
  g != g ;
  h != h ;
  i != i ;

}
#endif
