#include <mlapack.hh>
#include <matrixMat.hh>
#include <stdlib.h>

const double MaxRandom = 32768 ; // 2^15

int main(){

#ifdef WITH_LAPACK
  // generating a random matrix

  Matrix_DOUBLE m(6,6) ;
  int i,j ;
  for(i=0;i<m.rows;++i)
    for(j=0;j<m.cols;++j){
      m(i,j) = double(rand())/MaxRandom*100.0 ;
    }
  
  cout << "m = \n" << m << endl ;

  cout << "m * m(-1) = I\n " << m * LAPack.inverse(m) << endl ;

  cout << "Trying to solve the following equations:\n" ;
  cout << "\t 3a + 9b + 4c -  d + 8e = 15\n" ;
  cout << "\t10a + 9b + 4c + 8d +15e = 115 \n" ;
  cout << "\t 4a + 7b + 5c + 3d + 2e = 54\n" ;
  cout << "\t 9a - 3b + 7c + 5d + 6e = 121\n"; 
  cout << "\t 7a +19b - 4c +40d + 3e = 282\n" ;
  cout << "\t -a +  b +11c + 2d +13e = 90\n" ;
  cout << "We write the equation as A X = B  (where X is the abcde matrix)\n";

  Matrix<double> A,B,X ;

  A.resize(6,5) ;

  double am[30] = {3, 9, 4, -1, 8, 10, 9, 4, 8, 15, 4, 7, 5, 3, 2,9,-3,7,5,6,7,19,-4,40,3,-1,1,11,2,13} ;
  for(i=0;i<6;++i)
    for(j=0;j<5;++j)
      A(i,j) = am[i*5+j] ;

  cout << "A =\n" << A ;
  
  B.resize(6,1) ;
  B(0,0) = 15 ;
  B(1,0) = 115 ;
  B(2,0) = 54 ;
  B(3,0) = 121 ;
  B(4,0) = 282 ;
  B(5,0) = 90 ;
  cout << "B=\n" << B ;

  X.resize(5,1) ;

  LAPack.lse(A,B,X) ;

  cout << "\nX =\n" << X ;
  cout << "The result should be 3, -2, 6, 8 and 1\n" ;
 

  Matrix<double> U,S,V ;
  Matrix<double> A2 ;
  
  LAPack.svd(A,U,S,V) ;

  cout << "SVD decomposition of A yields\n" ;
  cout << "U =\n" << U ;
  cout << "V'=\n" << V ;
  cout << "S=\n" << S ;

  A2 = U*S*V ;
  cout << "USv'=\n" << A2 << endl ;
  cout << "A =\n" << A << endl ;
  cout << "The two matrix should be equal.\n" ;

#endif
  return 0 ;
}
