/*=============================================================================
        File:              curve.hh
     Purpose:       
    Revision:       $Id: curve.hh,v 2.1 1998/05/12 17:43:26 lavoie Exp $
  Created by:    Philippe Lavoie          (3 Oct, 1996)
 Modified by: 

 Copyright notice:
          Copyright (C) 1996-1997 Philippe Lavoie
 
          This library is free software; you can redistribute it and/or
          modify it under the terms of the GNU Library General Public
          License as published by the Free Software Foundation; either
          version 2 of the License, or (at your option) any later version.
 
          This library is distributed in the hope that it will be useful,
          but WITHOUT ANY WARRANTY; without even the implied warranty of
          MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
          Library General Public License for more details.
 
          You should have received a copy of the GNU Library General Public
          License along with this library; if not, write to the Free
          Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
=============================================================================*/
#ifndef _nurbs_curve_h_
#define _nurbs_curve_h_

#include "internals.hh"

#include MatrixLib(vector.hh)

/**  **  **  **  **  **  **  **  **  **  **  **  **  **  **  **  **  **  **
        Class: PlCurve --- An abstract parametric curve class
  Description: This is an abstract class used as a basis for NURBS and 
               HNURBS curves.
   Created by: Philippe Lavoie (4 Oct. 1996)
  Modified by:
 **  **  **  **  **  **  **  **  **  **  **  **  **  **  **  **  **  **  **/
class PlParaCurve{
public:
  PlParaCurve() {;}


  virtual CtrlPnt4D operator()(float u) const =0;
  CtrlPnt4D point4D(float u) const { return operator()(u); }
  Point3D point3D(float u) const { return project(operator()(u)) ; }

  virtual void deriveAt(float u, int, Vector_CtrlPnt4D&) const =0;
  virtual void deriveAt(float u, int, Vector_Point3D&) const =0;
  friend void Cder(float u,const PlParaCurve& c,int d, Vector_CtrlPnt4D& ders) { c.deriveAt(u,d,ders);}
  friend void Cder(float u,const PlParaCurve& c,int d, Vector_Point3D& ders) { c.deriveAt(u,d,ders);}
  
  virtual float minKnot() const = 0 ;
  virtual float maxKnot() const = 0 ;

  virtual float minDist2(const Point3D& p, float& guess,float error=0.0001,float s=0.2,int sep=9,int maxIter=10,float um=-1, float uM=-1) const ;
  virtual Point3D minDistY(float y, float& guessU, float error=0.0001, float s=-1, int sep=9, int maxIter=10, float um=-1, float uM=-1) const ;
  virtual Point3D minDistX(float y, float& guessU, float error=0.0001, float s=-1, int sep=9, int maxIter=10, float um=-1, float uM=-1) const ;
  virtual Point3D minDistZ(float y, float& guessU, float error=0.0001, float s=-1, int sep=9, int maxIter=10, float um=-1, float uM=-1) const ;

  enum CoordinateType { coordX, coordY, coordZ } ;
  virtual float extremum(int findMin, CoordinateType coord, float minDu=0.0001, int sep=9, int maxIter=10, float um=-1, float uM=-1) const ;

};

#endif 

