/*============================================================================
        File: matrixRT.hh
     Purpose: Describes a rotation-translation matrix      	      NURBS
    Revision: $Id: matrixRT.hh,v 2.0 1998/04/15 18:11:11 lavoie Exp $
  Created by: Philippe Lavoie          (3 Oct, 1996)
 Modified by: 

 Copyright notice:
          Copyright (C) 1996-1997 Philippe Lavoie
 
	  This library is free software; you can redistribute it and/or
	  modify it under the terms of the GNU Library General Public
	  License as published by the Free Software Foundation; either
	  version 2 of the License, or (at your option) any later version.
 
	  This library is distributed in the hope that it will be useful,
	  but WITHOUT ANY WARRANTY; without even the implied warranty of
	  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
	  Library General Public License for more details.
 
	  You should have received a copy of the GNU Library General Public
	  License along with this library; if not, write to the Free
	  Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
=============================================================================*/
#ifndef _nurbs_matrixRT_h_
#define _nurbs_matrixRT_h_

#include "curve.hh"
#include MatrixLib(matrix.hh)
// Predefining every friend functions
// This is required by latest ISO C++ draft

template <class T> class MatrixRT ;
template <class T> CtrlPnt4D operator*(const MatrixRT<T>&,const CtrlPnt4D&) ;
template <class T> Point3D operator*(const MatrixRT<T>&,const Point3D&) ;
template <class T> MatrixRT<T> operator*(const MatrixRT<T>&,const MatrixRT<T>&) ;


/**  **  **  **  **  **  **  **  **  **  **  **  **  **  **  **  **  **  **
        Class: MatrixRT --- a matrix for rotation and translation 
			    transformation
  Description: This is a matrix for the rotation, translation and scaling 
               of a point in 3D or 4D.
   Created by: Philippe Lavoie (25 July 1997)
  Modified by:
 **  **  **  **  **  **  **  **  **  **  **  **  **  **  **  **  **  **  **/
template <class T>
class MatrixRT : public PlMatrix<T> {
public:
  MatrixRT(T ax, T ay, T az, T x, T y, T z) ;
  MatrixRT() ;
  MatrixRT(T* p) ;
  
  MatrixRT<T>& rotate(T ax,T ay, T az);
  MatrixRT<T>& rotateXYZ(T ax,T ay, T az);
  MatrixRT<T>& translate(T x, T y, T z) ;
  MatrixRT<T>& scale(T x, T y, T z) ;
  
  MatrixRT<T>& rotateDeg(T ax, T ay, T az) { return rotate(T(ax*M_PI/180.0),T(ay*M_PI/180.0),T(az*M_PI/180.0)) ; } // creates a rotation matrix with the angles in degrees
  MatrixRT<T>& rotateDegXYZ(T ax, T ay, T az) { return rotateXYZ(T(ax*M_PI/180.0),T(ay*M_PI/180.0),T(az*M_PI/180.0)) ; } // creates a rotation matrix with the angle in degrees

  MatrixRT<T>& operator=(const PlMatrix<T>&) ;
  MatrixRT<T>& operator=(const MatrixRT<T>&) ;

#ifdef ANSI_ISO_CPP
  friend CtrlPnt4D operator* <>(const MatrixRT<T>&,const CtrlPnt4D&) ;
  friend Point3D operator* <>(const MatrixRT<T>&,const Point3D&) ;
  friend MatrixRT<T> operator* <>(const MatrixRT<T>&,const MatrixRT<T>&) ;
#else
  friend CtrlPnt4D operator*(const MatrixRT<T>&,const CtrlPnt4D&) ;
  friend Point3D operator*(const MatrixRT<T>&,const Point3D&) ;
  friend MatrixRT<T> operator*(const MatrixRT<T>&,const MatrixRT<T>&) ;
#endif


protected:
  int created ;
};

typedef MatrixRT<float> MatrixRT_FLOAT ;
typedef MatrixRT<double> MatrixRT_DOUBLE ;

#endif
