/*=============================================================================
        File: nurbsGL.H
     Purpose: Describes all OpenGL related classes.
    Revision: $Id: nurbsGL.hh,v 2.8 1998/06/03 18:22:30 lavoie Exp $
  Created by: Philippe Lavoie          (28 September, 1997)
 Modified by: 

 Copyright notice:
          Copyright (C) 1996-1998 Philippe Lavoie
 
          This library is free software; you can redistribute it and/or
          modify it under the terms of the GNU Library General Public
          License as published by the Free Software Foundation; either
          version 2 of the License, or (at your option) any later version.
 
          This library is distributed in the hope that it will be useful,
          but WITHOUT ANY WARRANTY; without even the implied warranty of
          MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
          Library General Public License for more details.
 
          You should have received a copy of the GNU Library General Public
          License along with this library; if not, write to the Free
          Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
=============================================================================*/
#ifndef _nurbs_nurbsGL_h_
#define _nurbs_nurbsGL_h_

#include "internals.hh"

#ifdef WITH_OPENGL

#include <nurbs_sp.hh>
#include <nurbsS_sp.hh>
#include <hnurbsS_sp.hh>
#include <GL/gl.h>
#include <GL/glu.h>

enum PlObjectGLState {hideState=0, objectState, selectedState, currentState, selected_currentState };
//enum PlNurbsGLState { showCurve=0, showCPoints, showBbox } ;

extern Color objectColorDefault ;
extern Color selectColorDefault ;
extern Color currentColorDefault ;

extern Color axisXColorDefault ;
extern Color axisYColorDefault ;
extern Color axisZColorDefault ;

extern Color cpointColorDefault ;
extern Color cpoint0ColorDefault ;
extern Color cPolygonColorDefault ;
extern Color cpointActiveColorDefault ;

extern Color knotColorDefault ;
extern Color knotActiveColorDefault ;

extern PlObjectGLState objectStateDefault ;

extern int pSizeDefault ;

extern const int PLNURBS_FLAGS_AFFECT_ALL ;
extern const int PLNURBS_FLAGS_AFFECT_ACTIVE_ONLY  ;
extern const int PLNURBS_FLAGS_AFFECT_SELECTED_ONLY  ;
extern const int PLNURBS_FLAGS_VIEW_BBOX ; 

extern const int PLNURBS_DISPLAY_NORMAL ;
extern const int PLNURBS_DISPLAY_ISOCURVES ;
extern const int PLNURBS_DISPLAY_SHADED ;
extern const int PLNURBS_DISPLAY_HIDDEN_LINES ; 
extern const int PLNURBS_DISPLAY_TESSELATION ;

extern const int PLNURBSLIST_DELETE_AT_RESET ;
extern const int PLNURBSLIST_KEEP_AT_RESET ;

extern int PlNurbsDisplayMode ;
extern float tessel_tolerance ;

extern CtrlPnt4D dummyCPoint ;

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
      Routine: glColor3f --- calls glColor3f
  Description: Calls glColor3f with the proper arguments
        Input: color --> the color to use	       
       Output: 
 Restrictions: 
   Created by: Philippe Lavoie (23 September 1997)
  Modified by:
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
inline void glColor3f(const Color& color){
  glColor3f(GLfloat(color.r)/255.0,GLfloat(color.g)/255.0,GLfloat(color.b)/255.0) ;
}

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
      Routine: colorToMaterial --- stores the color information
  Description: Stores the color information inside a GLfloat vector.
        Input: color --> the color to use
                   c --> the GLfloat vector
	       alpha --> the alpha value [1.0]
       Output: 
 Restrictions: 
   Created by: Philippe Lavoie (23 September 1997)
  Modified by:
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
inline void colorToMaterial(const Color& color, GLfloat* c, GLfloat alpha=1.0){
  c[0] = GLfloat(color.r)/255.0 ;
  c[1] = GLfloat(color.g)/255.0 ;
  c[2] = GLfloat(color.b)/255.0 ;
  c[3] = alpha ;
}

/**  **  **  **  **  **  **  **  **  **  **  **  **  **  **  **  **  **  **
        Class: RGBAf --- a class to hold rgba floating point values
  Description: This holds the red, green, blue and alpha floting point
               values. These informations are neede by some renderer.
   Created by: Philippe Lavoie (28 September 1997)
  Modified by:
 **  **  **  **  **  **  **  **  **  **  **  **  **  **  **  **  **  **  **/
struct RGBAf {
  float r,g,b,a ;

  RGBAf(){ r=g=b=a=0;}
  RGBAf(const RGBAf& c) { r = c.r ; g =c.g ; b=c.b ; a=c.a; }
  RGBAf(const Color& c) { r = (float)c.r/255.0 ; g = (float)c.g/255.0 ; b = (float)c.b/255.0 ; a = 1 ; }

};

/**  **  **  **  **  **  **  **  **  **  **  **  **  **  **  **  **  **  **
        Class: PlMaterial --- a class for the material properties of
	                      an object.
  Description: This holds the information about the material properties
               of the object. This is a long list, since it has properties
	       which are usefull for more than one rendering method.

	       You should not use this class yet. It is not used by any 
	       other class.
   Created by: Philippe Lavoie (28 September 1997)
  Modified by:
 **  **  **  **  **  **  **  **  **  **  **  **  **  **  **  **  **  **  **/
struct PlMaterial {
  PlMaterial() ; 
  // For OpenGL rendering
  RGBAf frontAmbient, backAmbient ;
  RGBAf frontDiffuse, backDiffuse ;
  RGBAf frontSpecular, backSpecular ;
  RGBAf frontEmission, backEmission ;
  float frontShininess, backShininess ;
  glMaterial() ;
  
  // for POV-RAY rendering 
  // pigment options
  RGBAf pigment ;
  double pigment_transfer ;
  double pigment_transmit ;
  char *pigment_userdefined ;
  // normal options
  double bump,bump_scale ;
  char *normal_userdefined ;
  // finish options
  Color ambient ;
  double diffuse,brilliance,phong,specular,roughness,metallic ;
  Color reflection ;
  double refraction,ior,caustics,fade_distance,fade_power ;
  double irid_thick ;
  Vector_Point3D irid_turbulence ;
  double crand ;

  // for RIB rendering
  char* material ;
};

/**  **  **  **  **  **  **  **  **  **  **  **  **  **  **  **  **  **  **
        Class: PlObjectGL --- The base class for OpenGL objects
  Description: This is the base virtual class for objects which can be
               displayed using OpenGL.

	       The class contains informations regarding the color of
	       the object and the state of the object (selected, active,
	       displayed, hidden). It also associates a color for each 
	       state.

	       Information about local transformations which should be 
	       applied to the objects are also given.
   Created by: Philippe Lavoie (28 September 1997)
  Modified by:
 **  **  **  **  **  **  **  **  **  **  **  **  **  **  **  **  **  **  **/
class PlObjectGL {
public:
  PlObjectGL() ; 
  virtual ~PlObjectGL() ;

  PlObjectGL& operator=(const PlObjectGL& a) ;

  Color objectColor ;
  Color selectColor ;
  Color currentColor ;
  GLfloat *materialColor ;
  

  void setObjectColor(const Color& c) { objectColor = c ; }
  void setSelectColor(const Color& c) { selectColor = c ; }
  void setCurrentColor(const Color& c) { currentColor = c ; }

  void glSelectColor() const { glColor3f(selectColor) ; colorToMaterial(selectColor,materialColor) ;}
  void glObjectColor() const { glColor3f(objectColor) ; colorToMaterial(objectColor,materialColor) ;}
  void glCurrentColor() const { glColor3f(currentColor) ; colorToMaterial(currentColor,materialColor) ;}

  void hideObject() { state = hideState ; }
  void selectObject() { state = selectedState ; }
  void viewObject() { state = objectState ; }
  void currentObject() { state = currentState ; }

  virtual void glColor() const 
    { if(isActive())
        glColor(currentColor) ;
        else{
	  if(isSelected()) glColor(selectColor) ; 
	  else glColor(objectColor) ; }
    }
                             

  void glColor(const Color& c) const {
    colorToMaterial(Color(0,0,0),materialColor) ;
    glMaterialfv(GL_FRONT_AND_BACK,GL_DIFFUSE,materialColor) ;
    colorToMaterial(c,materialColor) ;
    glMaterialfv(GL_FRONT_AND_BACK,GL_EMISSION,materialColor) ;
    //glColor3f(c) ;
  }

  virtual void glObject() const = 0 ;  

  enum ObjectCategory { badType, nurbsType, pointType, vectorType, listType };
  enum ObjectType {badObject,curveObject,surfaceObject,pointObject,cpointObject,cpointPolygonObject,bboxObject,vectorObject,listObject,hSurfObject,hcpointObject,pointListObject,spointObject} ;
  ObjectType type ;
  ObjectCategory category ;


  // object tranformation information
  GLfloat tx,ty,tz ;
  GLfloat rx,ry,rz ;
  GLfloat sx,sy,sz ;

  virtual void glTransform() const ;

  virtual void display() const { if(state) {glPushMatrix() ; glTransform() ; glObject(); glPopMatrix() ;} }
  virtual void displayList()  { if(state) {glPushMatrix() ; glTransform() ; glColor(); glCallList(callListId); glPopMatrix();} }
  virtual void displayName() { ; }
  
  virtual void glNewList() ; 
  int callListId ; // This is assigned by the user, NOT the class

  // Information for the link list node
  virtual PlObjectGL*& previous() { return prev_ ;}
  virtual PlObjectGL*& next() { return next_ ;}
  virtual PlObjectGL* previous() const { return prev_ ;}
  virtual PlObjectGL* next() const { return next_ ;}

  PlObjectGLState getState() const { return state ; }

  virtual void select() { selected = 1 ; }
  virtual void deselect() { selected = 0 ; }
  int isSelected() const { return selected ; }

  virtual void activate() { active = 1 ; }
  virtual void deactivate() { active = 0 ; }
  int isActive() const { return active ; }

  virtual PlObjectGL* copy() { return 0;} 
  virtual void applyTransform() { ;} 

  // I/O functions
  virtual int read(const char* filename) {return 1; }
  virtual int write(const char* filename) const {return 1; }
  virtual int writeRIB(const char* filename) const {return 1; }
  virtual int writePOVRAY(const char* filename) const {return 1; }
  virtual int read(ifstream &fin) {return 1; }
  virtual int write(ofstream &fout) const { return 1; }
  virtual int writeRIB(ofstream &fout) const { return 1; }
  virtual int writePOVRAY(ofstream &fout) const { return 1; }

protected:
  int selected,active ;
  PlObjectGLState state ;  
  PlObjectGL *prev_,*next_ ;

};


/**  **  **  **  **  **  **  **  **  **  **  **  **  **  **  **  **  **  **
        Class: PlObjectRefGL --- a reference object for OpenGL
  Description: This class reference other OpenGL objects.
   Created by: Philippe Lavoie (28 September 1997)
  Modified by:
 **  **  **  **  **  **  **  **  **  **  **  **  **  **  **  **  **  **  **/
class PlObjectRefGL : public PlObjectGL {
public:
  PlObjectRefGL(PlObjectGL* p) : ptr(ptr_) { ptr_ = p ; }

  PlObjectGL* &ptr ;
  void glObject() const { ptr->glObject(); }

protected:
  PlObjectGL* ptr_ ;
};

/**  **  **  **  **  **  **  **  **  **  **  **  **  **  **  **  **  **  **
        Class: PlObjectListGL --- a link list of PlObjectGL
  Description: A linked list of PlObjectGL
   Created by: Philippe Lavoie (28 September 1997)
  Modified by:
 **  **  **  **  **  **  **  **  **  **  **  **  **  **  **  **  **  **  **/
class PlObjectListGL : public PlObjectGL {
 public:
  PlObjectListGL() ;
  virtual ~PlObjectListGL() ;
  
#if defined(__sgi) && !defined(__GNUG__)
  PlObjectGL* &first ;
  PlObjectGL* &last ;
#else
  const PlObjectGL* &first ;
  const PlObjectGL* &last ;
#endif

  PlObjectGL* &current ;

  virtual void glObject() const ;
  virtual void display() const ;
  virtual void displayName() const ;

  void reset() ;
  void setResetMode(int m) { resetMode = m ; }
  void add(PlObjectGL* obj) ;
  PlObjectGL* remove(PlObjectGL* obj) ;
  // void insertAfter(PlObjectGL* obj) ;
  // void insertBefore(PlObjectGL* obj) ;

  void activate() ;
  void deactivate() ;

  void select() ;
  void deselect() ;

  void transformTo(GLfloat x, GLfloat y, GLfloat z, GLfloat a, GLfloat b, GLfloat c, GLfloat sx, GLfloat sy, GLfloat sz, int behavior=PLNURBS_FLAGS_AFFECT_ALL) ;
  void transformBy(GLfloat x, GLfloat y, GLfloat z, GLfloat a, GLfloat b, GLfloat c, GLfloat sx, GLfloat sy, GLfloat sz, int behavior=PLNURBS_FLAGS_AFFECT_ALL) ;

  PlObjectGL* goTo(int a) ;
  PlObjectGL* goToActive(int a) ;

  PlObjectGL* goToNext() ; 
  PlObjectGL* goToPrevious() ; 

  PlObjectGL* goToNextActive() ; 
  PlObjectGL* goToPreviousActive() ; 

  PlObjectGL* jumpToNext() ;
  PlObjectGL* jumpToPrevious() ;

  void setJumpSize(int a) {if(a>0) jumpSize=a ; }

  int size() const { return  n; }

  virtual int read(const char* filename) ;
  virtual int write(const char* filename) const ;
  virtual int writeRIB(const char* filename) const ;
  virtual int writePOVRAY(const char* filename) const ;

  void viewAllObjects() ;
  void hideAllObjects() ;

protected:
  PlObjectGL *first_, *last_ ;
  PlObjectGL *current_ ;
  int jumpSize ;
  int n ;
  int resetMode ;
};

/**  **  **  **  **  **  **  **  **  **  **  **  **  **  **  **  **  **  **
        Class: PlObjectRefListGL --- a link list of PlObjectRefListGL
  Description: A linked list of PlObjectRefListGL
   Created by: Philippe Lavoie (28 September 1997)
  Modified by:
 **  **  **  **  **  **  **  **  **  **  **  **  **  **  **  **  **  **  **/
class PlObjectRefListGL : public PlObjectListGL{
public:
  PlObjectRefListGL() : PlObjectListGL() { resetMode = PLNURBSLIST_KEEP_AT_RESET ; }
  ~PlObjectRefListGL() ;

  void add(PlObjectGL* obj) { PlObjectRefGL *t = new PlObjectRefGL(obj) ; PlObjectListGL::add(t) ; }
  PlObjectGL* remove(PlObjectGL* obj) ;

  void refList(const PlObjectListGL* list, int addOnce=1) ;

};


/**  **  **  **  **  **  **  **  **  **  **  **  **  **  **  **  **  **  **
        Class: PlPointListGL --- a class to hold a list of points.
  Description: A class to hold a list of points.
   Created by: Philippe Lavoie (29 Mars 1998)
  Modified by:
 **  **  **  **  **  **  **  **  **  **  **  **  **  **  **  **  **  **  **/
class PlPointListGL : public PlObjectGL {
public:
  PlPointListGL() :  PlObjectGL() { category = pointType ; type = pointListObject ; psize = pSizeDefault ; }
  PlPointListGL(const PlPointListGL &pl);
  PlPointListGL(const BasicList<Point3D> &l) ;
  virtual void glObject() const ;
  void setPsize(int s) {psize = s ;}

  int read(ifstream &fin) ;
  int write(ofstream &fout) const ; 

  void applyTransform() ;


  PlObjectGL* copy() { PlPointListGL *t = new PlPointListGL(*this) ; return t ; }

  BasicList<Point3D> list ;
protected:
  int psize ;
};


/**  **  **  **  **  **  **  **  **  **  **  **  **  **  **  **  **  **  **
        Class: PlCPointGL --- a class to hold a control point.
  Description: A class to hold a control point.
   Created by: Philippe Lavoie (23 September 1997)
  Modified by:
 **  **  **  **  **  **  **  **  **  **  **  **  **  **  **  **  **  **  **/
class PlCPointGL : public PlObjectGL  {
public:  
  PlCPointGL() : PlObjectGL(), cpoint(dummyCPoint) { type = cpointObject;  psize=pSizeDefault ; xCoord = yCoord = zCoord = wCoord = 1.0 ;}
  PlCPointGL(CtrlPnt4D& cp): PlObjectGL(), cpoint(cp) { type = cpointObject; psize=pSizeDefault ;} 
  ~PlCPointGL() {;}

  virtual void glObject() const;

  virtual void modify(const CtrlPnt4D& v) { cpoint += v ; }
  virtual void modifySym(const CtrlPnt4D &v) ;

  void setPsize(int s) {psize = s ; }

  CtrlPnt4D& point() const { return cpoint ; }

  void setSym(PlCPointGL* sp, float x, float y, float z, float w) { symPoint = sp ; xCoord = x; yCoord = y ; zCoord = z; wCoord = w ; if(symPoint == this) symPoint = 0 ;}

  virtual int row() const { return -1 ; }
  virtual int col() const { return -1 ; }

protected:
  CtrlPnt4D& cpoint ;
  int psize ;
  PlCPointGL *symPoint ;
  float xCoord,yCoord,zCoord,wCoord ;
};

class PlHNurbsSurfaceGL ;

/**  **  **  **  **  **  **  **  **  **  **  **  **  **  **  **  **  **  **
        Class: PlHCPointGL --- a class to hold a control point from a HNURBS
  Description: A class to hold a control point from a HNURBS.
   Created by: Philippe Lavoie (3 November 1997)
  Modified by:
 **  **  **  **  **  **  **  **  **  **  **  **  **  **  **  **  **  **  **/
class PlHCPointGL : public PlCPointGL  {
public:
  PlHCPointGL(CtrlPnt4D& off, int i, int j, PlHNurbsSurfaceGL* hs, int mod): PlCPointGL(), offset(off),s(hs) { type = hcpointObject; psize=pSizeDefault ; i0 = i ; j0 = j ; canModify = mod ;} 
  ~PlHCPointGL() { ; } 

  virtual void glObject() const;

  virtual void modify(const CtrlPnt4D& v) ;

  virtual int row() const { return i0; }
  virtual int col() const { return j0; }

protected:
  CtrlPnt4D& offset ;
  PlHNurbsSurfaceGL *s ;
  int i0,j0 ;
  int canModify ;
};

/**  **  **  **  **  **  **  **  **  **  **  **  **  **  **  **  **  **  **
        Class: PlSPointCurveGL --- a class to hold a control point from a NURBS
  Description: A class to hold a control point from a NURBS curve or surface
   Created by: Philippe Lavoie (11 May 1998)
  Modified by:
 **  **  **  **  **  **  **  **  **  **  **  **  **  **  **  **  **  **  **/
class PlSPointCurveGL : public PlCPointGL  {
public:
  PlSPointCurveGL(int i, PlNurbsCurveSP *c);
  ~PlSPointCurveGL() { ; } 

  virtual void glObject() const;

  virtual void modify(const CtrlPnt4D& v) ; 

  void updateOthers() ; 

  void setStartEnd(PlSPointCurveGL* s, int r) { start = s; rows = r ;  }

  virtual int row() { return k ; }

protected:
  CtrlPnt4D spoint ;
  PlNurbsCurveSP *curve ;
  PlSPointCurveGL *start ; 
  int k ;
  int rows ;
};

/**  **  **  **  **  **  **  **  **  **  **  **  **  **  **  **  **  **  **
        Class: PlSPointSurfaceGL --- a class to hold a surface point 
  Description: A class to hold a control point from a NURBS curve or surface
   Created by: Philippe Lavoie (11 May 1998)
  Modified by:
 **  **  **  **  **  **  **  **  **  **  **  **  **  **  **  **  **  **  **/
class PlSPointSurfaceGL : public PlCPointGL  {
public:
  PlSPointSurfaceGL(int i, int j, PlNurbsSurfaceSP *s, PlObjectListGL *sp);
  ~PlSPointSurfaceGL() { ; } 

  virtual void glObject() const;

  virtual void modify(const CtrlPnt4D& v) ; 

  void updateOthers() ; 

  void setStartEnd(PlSPointSurfaceGL* s, int r, int c=0) { start = s; rows = r ; cols = c ;  }

  virtual int row() { return k ; }
  virtual int col() { return l ; }

protected:
  CtrlPnt4D spoint ;
  PlNurbsSurfaceSP *surface ;
  PlSPointSurfaceGL *start ; 
  PlObjectListGL *spoints ;
  int k,l ;
  int rows,cols ; 
};

/**  **  **  **  **  **  **  **  **  **  **  **  **  **  **  **  **  **  **
        Class: PlSPointHSurfaceGL --- a class to hold a surface point 
  Description: A class to hold a control point from a NURBS curve or surface
   Created by: Philippe Lavoie (11 May 1998)
  Modified by:
 **  **  **  **  **  **  **  **  **  **  **  **  **  **  **  **  **  **  **/
class PlSPointHSurfaceGL : public PlCPointGL  {
public:
  PlSPointHSurfaceGL(int i, int j, PlHNurbsSurfaceSP *s, PlObjectListGL *sp);
  ~PlSPointHSurfaceGL() { ; } 

  virtual void glObject() const;

  virtual void modify(const CtrlPnt4D& v) ; 

  void updateOthers() ; 

  void setStartEnd(PlSPointHSurfaceGL* s, int r, int c=0) { start = s; rows = r ; cols = c ;  }

  virtual int row() { return k ; }
  virtual int col() { return l ; }

protected:
  CtrlPnt4D spoint ;
  PlHNurbsSurfaceSP *surface ;
  PlSPointHSurfaceGL *start ; 
  PlObjectListGL *spoints ;
  int k,l ;
  int rows,cols ; 
};

/**  **  **  **  **  **  **  **  **  **  **  **  **  **  **  **  **  **  **
        Class: PlPointGL --- a class to hold a 3D point
  Description: A class to hold a 3D point
   Created by: Philippe Lavoie (23 September 1997)
  Modified by:
 **  **  **  **  **  **  **  **  **  **  **  **  **  **  **  **  **  **  **/
class PlPointGL : public PlObjectGL  {
public:  
  PlPointGL(const Point3D& p3d): PlObjectGL(), p(p3d) { type = pointObject ; psize=pSizeDefault ;}

  void glObject() const ;

  void modify(const Point3D& v) { p += v ; }
  void set(const Point3D& v) { p = v ; }

  void setPsize(int s) {psize = s ; }

  float x() const { return p.x ; }
  float y() const { return p.y ; }
  float z() const { return p.z ; }
  const Point3D& point() const { return p ; }

  PlObjectGL* copy() { PlPointGL* t = new PlPointGL(p) ; return t ; }

protected:
  Point3D p ;
  int psize ;
};

/**  **  **  **  **  **  **  **  **  **  **  **  **  **  **  **  **  **  **
        Class: PlKnotGL --- a class to hold a knot
  Description: a class to hold a knot
   Created by: Philippe Lavoie (23 September 1997)
  Modified by:
 **  **  **  **  **  **  **  **  **  **  **  **  **  **  **  **  **  **  **/
class PlKnotGL : public PlObjectGL  {
public:  
  PlKnotGL(const Point3D& p3d, int kU, int kV): PlObjectGL(), p(p3d), ku(kU), kv(kV) { type = pointObject ; psize=pSizeDefault ;}

  void glObject() const ;

  void modify(const Point3D& v) { p += v ; }
  void set(const Point3D& v) { p = v ; }

  void setPsize(int s) {psize = s ; }

  float x() const { return p.x ; }
  float y() const { return p.y ; }
  float z() const { return p.z ; }
  const Point3D& point() const { return p ; }

  PlObjectGL* copy() { PlPointGL* t = new PlPointGL(p) ; return t ; }
  
  int knotU() { return ku ; }
  int knotV() { return kv ; }

protected:
  Point3D p ;
  int ku,kv ;
  int psize ;
};

/**  **  **  **  **  **  **  **  **  **  **  **  **  **  **  **  **  **  **
        Class: PlNurbsCpolygonGL --- holds the control polygon for a curve
  Description: Holds the control polygon for a curve
   Created by: Philippe Lavoie (23 September 1997)
  Modified by:
 **  **  **  **  **  **  **  **  **  **  **  **  **  **  **  **  **  **  **/
class PlNurbsCpolygonGL : public PlObjectGL  {
public:  
  PlNurbsCpolygonGL(PlNurbsCurve& c): PlObjectGL(),curve(c) { objectColor = cPolygonColorDefault;}

  void glObject() const;


protected:
  PlNurbsCurve& curve ;
};

/**  **  **  **  **  **  **  **  **  **  **  **  **  **  **  **  **  **  **
        Class: PlNurbsSpolygonGL --- holds the control polygon for a surface 
  Description: Holds the control polygon for a surface 
   Created by: Philippe Lavoie (23 September 1997)
  Modified by:
 **  **  **  **  **  **  **  **  **  **  **  **  **  **  **  **  **  **  **/
class PlNurbsSpolygonGL : public PlObjectGL  {
public:  
  PlNurbsSpolygonGL(PlNurbsSurface& s): PlObjectGL(),surface(s) {objectColor = cPolygonColorDefault;}

  void glObject() const;


protected:
  PlNurbsSurface& surface ;
};

/**  **  **  **  **  **  **  **  **  **  **  **  **  **  **  **  **  **  **
        Class: PlBoundingBoxGL --- holds a bounding box
  Description: Holds a bounding box
   Created by: Philippe Lavoie (23 September 1997)
  Modified by:
 **  **  **  **  **  **  **  **  **  **  **  **  **  **  **  **  **  **  **/
class PlBoundingBoxGL : public PlObjectGL  {
public:  
  PlBoundingBoxGL() ;

  void glObject() const;

  Point3D minP, maxP ;
 
  void setColorXYZ(const Color& colX, const Color& colY, const Color& colZ){
    colorX = colX; colorY = colY; colorZ = colZ ; }

protected:
  Color colorX, colorY, colorZ ;
};

/**  **  **  **  **  **  **  **  **  **  **  **  **  **  **  **  **  **  **
        Class: PlNurbsGL --- a virtual NURBS object class.
  Description: A virtual NURBS object class.
   Created by: Philippe Lavoie (28 September 1997)
  Modified by:
 **  **  **  **  **  **  **  **  **  **  **  **  **  **  **  **  **  **  **/
class PlNurbsGL : public PlObjectGL {
public:
  PlNurbsGL();

  virtual ~PlNurbsGL() { if(polygon) delete polygon ; }


  virtual PlNurbsGL& operator=(const PlNurbsGL& a) ;

  virtual void gluNurbs() const = 0 ;
  
  void setNurbsRenderer(GLUnurbsObj *nobj) { nurbsRenderer = nobj ; }

  void glObject() const;

  virtual void point(float &u, float &v, int pSize, const Color& colorP, int cp_flag=0) const = 0 ;

  virtual void resetAll() { resetBoundingBox(); resetCPoints(); resetPolygon(); resetKnots(); }
  virtual void resetBoundingBox() {} ;
  virtual void resetCPoints() {} ;
  virtual void resetPolygon() {};
  virtual void resetKnots() {} ;
  
  friend PlNurbsGL* readNurbsObject(const char* filename) ;

  Color colorCP, colorCP0, colorPolygon, colorKnot ;


  void setObjectColor(const Color& a, const Color& b, const Color& c){ objectColor=a ; selectColor = b ; currentColor = c ; }
  void setBBoxColor(const Color& a, const Color& b, const Color& c,const Color& d, const Color& e, const Color& f){ bbox.objectColor=a ; bbox.selectColor = b ; bbox.currentColor = c ; bbox.setColorXYZ(d,e,f) ;}
  void setPolygonColor(const Color& a, const Color& b, const Color& c){ polygon->objectColor=a ; polygon->selectColor = b ; polygon->currentColor = c ; }
  void setCPointColor(const Color& a, const Color& b, const Color& c, const Color& d){ colorCP=a ; colorCP0=b ; cpoints.selectColor = c ; cpoints.currentColor = d ; }
  void setKnotsColor(const Color& a, const Color& b, const Color& c){ knots.objectColor = colorKnot =a ; knots.selectColor = b ; knots.currentColor = c ; }

  void viewBBox()    { bbox.viewObject();}
  void viewCPoints() { cpoints.viewAllObjects();}
  void viewCpolygon(){ if(polygon) polygon->viewObject() ; }
  void viewNurbs()   { nurbsState = objectState ; }
  void viewKnots()   { knots.viewObject();}

  void hideBBox()     { bbox.hideObject();}         
  void hideCPoints()  { cpoints.hideAllObjects();}      
  void hideCpolygon() { if(polygon) polygon->hideObject() ; }   
  void hideNurbs()    { nurbsState = hideState ; }
  void hideKnots()    { knots.hideObject() ; }

  void select() { selected = 1 ; bbox.select() ; }
  void deselect() { selected = 0 ; bbox.deselect() ; }

  void activate() { active = 1 ; bbox.activate() ; }
  void deactivate() { active = 0 ; bbox.deactivate() ; }

  virtual void setSym(int set, int uDir, float x, float y, float z, float w) = 0 ;

  PlObjectListGL cpoints;
  PlObjectListGL knots ;
  PlObjectGL *polygon; 
  PlBoundingBoxGL bbox ;

  PlObjectGLState nurbsState ;

  virtual void modifyPoint(float u, float v, float dx, float dy, float dz) = 0 ;

  int editSurfacePoints() const { return editSP ; }
  int editControlPoints() const { return !editSP; }
  int editSurfacePoints(int a) { editSP = a ; return editSP ; }
  int editControlPoints(int a) { editSP = !a ; return !editSP; }

  void setULines(int u) { if(u<=1) u = 2 ; nUlines = u ; }
  void setVLines(int v) { if(v<=2) v = 2 ; nVlines = v ; }
  int ULines() const { return nUlines ; }
  int VLines() const { return nVlines ; }

protected:
  int editSP ;
  GLUnurbsObj *nurbsRenderer;
  int nUlines,nVlines ; 
};


/**  **  **  **  **  **  **  **  **  **  **  **  **  **  **  **  **  **  **
        Class: PlNurbsCurveGL --- a NURBS curve class for OpenGL
  Description: A NURBS curve class for OpenGL
   Created by: Philippe Lavoie (23 September 1997)
  Modified by:
 **  **  **  **  **  **  **  **  **  **  **  **  **  **  **  **  **  **  **/
class PlNurbsCurveGL : public PlNurbsCurveSP, public PlNurbsGL {
public:
  PlNurbsCurveGL() : PlNurbsCurveSP(),PlNurbsGL() { type = curveObject ;  polygon = new PlNurbsCpolygonGL(*this) ;}
  PlNurbsCurveGL(const PlNurbsCurve& nurb): PlNurbsCurveSP(nurb),PlNurbsGL() { type = curveObject ; polygon = new PlNurbsCpolygonGL(*this) ;}
  PlNurbsCurveGL(const PlNurbsCurveGL& nurb): PlNurbsCurveSP((PlNurbsCurve)nurb),PlNurbsGL() { type = curveObject ; polygon = new PlNurbsCpolygonGL(*this) ;}
  PlNurbsCurveGL(const Vector_CtrlPnt4D& P1, const Vector_FLOAT &U1, int degree=3):PlNurbsCurveSP(P1,U1,degree),PlNurbsGL() { type = curveObject ; polygon = new PlNurbsCpolygonGL(*this) ;}
  PlNurbsCurveGL(const Vector_Point3D& P1, const Vector_FLOAT &W, const Vector_FLOAT &U1, int degree=3):PlNurbsCurveSP(P1,W,U1,degree),PlNurbsGL() { type = curveObject ;  polygon = new PlNurbsCpolygonGL(*this) ;}

  void gluNurbs() const;
  void point(float &u, float &v, int pSize, const Color& colorP, int cp_flag=0) const ;

  PlNurbsCurveGL& operator=(const PlNurbsCurveGL& a) ;
  PlNurbsCurveGL& operator=(const PlNurbsCurve& a) ;

  void resetBoundingBox() ;
  void resetCPoints() ;
  void resetPolygon() {}
  void resetKnots() ;

  PlObjectGL* copy() { PlNurbsCurveGL *t = new PlNurbsCurveGL(*this); return t ; }

  int read(const char* filename) { if(PlNurbsCurve::read(filename)){ resetBoundingBox() ; return 1 ; } return 0 ; }
  int read(ifstream &fin) { if(PlNurbsCurve::read(fin)){ resetBoundingBox() ; return 1 ; } return 0 ; }
  int write(const char* filename) const { return PlNurbsCurve::write(filename); }
  int write(ofstream &fout) const { return PlNurbsCurve::write(fout); }

  void applyTransform() ;
  void modifyPoint(float u, float v, float dx, float dy, float dz);


  void setSym(int set, int uDir, float x, float y, float z, float w) { }

};

/**  **  **  **  **  **  **  **  **  **  **  **  **  **  **  **  **  **  **
        Class: PlNurbsSurfaceGL --- a NURBS surface class for OpenGL
  Description: A NURBS surface class for OpenGL
   Created by: Philippe Lavoie (23 September 1997)
  Modified by:
 **  **  **  **  **  **  **  **  **  **  **  **  **  **  **  **  **  **  **/
class PlNurbsSurfaceGL : public PlNurbsSurfaceSP, public PlNurbsGL  {
public:  
  PlNurbsSurfaceGL():PlNurbsSurfaceSP(),PlNurbsGL() { type = surfaceObject ; polygon = new PlNurbsSpolygonGL(*this) ; image = 0 ; }
  PlNurbsSurfaceGL(const PlNurbsSurface& nS):PlNurbsSurfaceSP(nS),PlNurbsGL() { type = surfaceObject ; polygon = new PlNurbsSpolygonGL(*this) ; image = 0 ; }
  PlNurbsSurfaceGL(const PlNurbsSurfaceGL& nS):PlNurbsSurfaceSP((PlNurbsSurface)nS),PlNurbsGL() { type = surfaceObject ; polygon = new PlNurbsSpolygonGL(*this) ; image = 0 ; }
  PlNurbsSurfaceGL(int DegU, int DegV, const Vector_FLOAT& Uk, const Vector_FLOAT& Vk, const Matrix_CtrlPnt4D& Cp):PlNurbsSurfaceSP(DegU,DegV,Uk,Vk,Cp),PlNurbsGL() { type = surfaceObject ; polygon = new PlNurbsSpolygonGL(*this) ; image = 0 ; }
  PlNurbsSurfaceGL(int DegU, int DegV, Vector_FLOAT& Uk, Vector_FLOAT& Vk, Matrix_Point3D& Cp, Matrix_FLOAT& W):PlNurbsSurfaceSP(DegU,DegV,Uk,Vk,Cp,W),PlNurbsGL() { type = surfaceObject ; polygon = new PlNurbsSpolygonGL(*this) ; image = 0 ; }
  ~PlNurbsSurfaceGL() { if(image) delete []image ; }

  void gluNurbs() const;
  void point(float &u, float &v, int pSize, const Color& colorP, int cp_flag=0) const ;

  virtual PlNurbsSurfaceGL& operator=(const PlNurbsSurfaceGL& a);
  virtual PlNurbsSurfaceGL& operator=(const PlNurbsSurface& a);


  void resetBoundingBox() ;
  void resetCPoints() ;
  void resetPolygon() {}
  void resetKnots() ;

  int read(const char* filename) { if(PlNurbsSurface::read(filename)){ resetBoundingBox() ; return 1 ; } return 0 ; }
  int read(ifstream &fin) { if(PlNurbsSurface::read(fin)){ resetBoundingBox() ; return 1 ; } return 0 ; }
  int write(const char* filename) const { return PlNurbsSurface::write(filename); }
  int write(ofstream &fout) const { return PlNurbsSurface::write(fout); }
  int writeRIB(ofstream &fout) const { return PlNurbsSurface::writeRIB(fout); }
  //  int writePOVRAY(ofstream &fout) const { return PlNurbsSurface::writePOVRAY(0.1,fout); }
  int writePOVRAY(ofstream &fout) const { PlNurbsSurface::writeVRML("test.wrl",float(0.05),Color(255,255,255)); return PlNurbsSurface::writePOVRAY(0.1,fout); }

  PlObjectGL* copy() { PlNurbsSurfaceGL *t = new PlNurbsSurfaceGL(*this); return t ; }

  void applyTransform() ;

  void modifyPoint(float u, float v, float dx, float dy, float dz);

  void setImage(GLubyte *img,GLint w, GLint h) ;

  void setSym(int set, int uDir, float x, float y, float z, float w) { }


protected:
  GLubyte *image ;
  GLint imgW, imgH ;
};

/**  **  **  **  **  **  **  **  **  **  **  **  **  **  **  **  **  **  **
        Class: PlHNurbsSurfaceGL --- a HNURBS surface class for OpenGL
  Description: a HNURBS surface class for OpenGL
   Created by: Philippe Lavoie (23 September 1997)
  Modified by:
 **  **  **  **  **  **  **  **  **  **  **  **  **  **  **  **  **  **  **/
class PlHNurbsSurfaceGL : public PlHNurbsSurfaceSP, public PlNurbsGL  {
public:  
  PlHNurbsSurfaceGL() ;
  PlHNurbsSurfaceGL(const PlNurbsSurface& nS);
  PlHNurbsSurfaceGL(const PlHNurbsSurfaceGL& bS) ;
  PlHNurbsSurfaceGL(const PlHNurbsSurfaceGL* bS);
  virtual ~PlHNurbsSurfaceGL() { ; }

  void setLevelOfDetail(int l) { lod = l ; }
  int levelOfDetail() const { return lod; }
  void increaseLevelOfDetail() { ++lod ; if(lod>maxLevel()) lod=maxLevel() ;}
  void decreaseLevelOfDetail() ;
  void highestLevelOfDetail() { lod = maxLevel() ; }
  int maxLevelOfDetail() { return maxLevel() ; }

  void gluNurbs() const;
  void point(float &u, float &v, int pSize, const Color& colorP, int cp_flag=0) const ;

  //virtual PlNurbsSurfaceGL& operator=(const PlNurbsSurfaceGL& a);
  //virtual PlNurbsSurfaceGL& operator=(const PlNurbsSurface& a);

  void resetBoundingBox() ;
  void resetCPoints() ;
  void resetPolygon() ;
  void resetKnots() { }

  int read(const char* filename) { if(PlHNurbsSurface::read(filename)){ updateMaxUV(); resetBoundingBox() ; return 1 ; } return 0 ; }
  int read(ifstream &fin) { if(PlHNurbsSurfaceSP::read(fin)){ updateMaxUV(); resetBoundingBox() ; return 1 ; } return 0 ; }
  int write(const char* filename) const { return PlHNurbsSurface::write(filename); }
  int write(ofstream &fout) const { return PlHNurbsSurface::write(fout); }


  int writeRIB(ofstream &fout) const { return PlNurbsSurface::writeRIB(fout); }
  //  int writePOVRAY(ofstream &fout) const { return PlNurbsSurface::writePOVRAY(0.1,fout); }
  int writePOVRAY(ofstream &fout) const { PlNurbsSurface::writeVRML("test.wrl",float(0.05),Color(255,255,255)); return PlNurbsSurface::writePOVRAY(0.1,fout); }

  //void applyTransform() ;

  void selectBasePatch() { activePatch = this ; }
  void selectNextPatch() ;
  void selectPrevPatch() ;
  void selectHigherLevel() ;
  void selectLowerLevel() ;
  void selectHighestLevel() ;
  int editLevel() { return activePatch->level ; }

  void updateUpToLOD() { updateLevels(lod) ; }

  PlHNurbsSurfaceSP* addLevel() ;

  void applyTransform() ;
  void modifyPoint(float u, float v, float dx, float dy, float dz);
  

  PlObjectGL* copy() { PlHNurbsSurfaceGL *t = new PlHNurbsSurfaceGL(*this); return t ; }

  void setSym(int set, int uDir, float x, float y, float z, float w) ;

  void axis(int i, int j, Point3D& xaxis, Point3D& yaxis, Point3D& zaxis) const { activePatch->PlHNurbsSurface::axis(i,j,xaxis,yaxis,zaxis) ; }

protected:
  int lod ;
  PlHNurbsSurfaceGL *activePatch ;
};

/**  **  **  **  **  **  **  **  **  **  **  **  **  **  **  **  **  **  **
        Class: PlNurbsListGL --- a linked list of PlNurbsGL
  Description: A linked list of PlNurbsGL
   Created by: Philippe Lavoie (28 September 1997)
  Modified by:
 **  **  **  **  **  **  **  **  **  **  **  **  **  **  **  **  **  **  **/
class PlNurbsListGL : public PlObjectListGL {
 public:
  //void add(PlNurbsGL* obj) { PlObjectListGL::add(obj) ; }
  PlNurbsGL* remove(PlNurbsGL* obj) { return (PlNurbsGL*)PlObjectListGL::remove(obj) ; }

  void glObject() const ; 
  void display() const ;

  void resetDisplayFlags(int o, int cp, int p, int b, int k, int behavior=PLNURBS_FLAGS_AFFECT_ALL) ;
};


void initColorsGL() ;

#endif WITH_OPENGL

#endif
