/*=============================================================================
        File: surface.H
     Purpose:       
    Revision: $Id: surface.hh,v 2.2 1998/06/01 19:13:41 lavoie Exp $
  Created by: Philippe Lavoie          (3 Oct, 1996)
 Modified by: 

 Copyright notice:
          Copyright (C) 1996-1997 Philippe Lavoie
 
          This library is free software; you can redistribute it and/or
          modify it under the terms of the GNU Library General Public
          License as published by the Free Software Foundation; either
          version 2 of the License, or (at your option) any later version.
 
          This library is distributed in the hope that it will be useful,
          but WITHOUT ANY WARRANTY; without even the implied warranty of
          MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
          Library General Public License for more details.
 
          You should have received a copy of the GNU Library General Public
          License along with this library; if not, write to the Free
          Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
=============================================================================*/
#ifndef _nurbs_surface_h_
#define _nurbs_surface_h_

#include "curve.hh"

// see in the file curve.H for the definition of MatrixLib

#include MatrixLib(matrix.hh)


/**  **  **  **  **  **  **  **  **  **  **  **  **  **  **  **  **  **  **
        Class: PlParaSurface --- An abstract parametric surface class
  Description: This is an abstract class used as a basis for NURBS and 
               HNURBS surfaces.
   Created by: Philippe Lavoie (4 Oct. 1996)
  Modified by:
 **  **  **  **  **  **  **  **  **  **  **  **  **  **  **  **  **  **  **/
class PlParaSurface{
public:
  PlParaSurface() {;}

  virtual CtrlPnt4D operator()(float u, float v) const =0;
  CtrlPnt4D point4D(float u, float v) const { return operator()(u,v); }
  Point3D point3D(float u, float v) const { return project(operator()(u,v)) ; }

  virtual void deriveAt(float u, float v, int d, Matrix_CtrlPnt4D& skl) const =0;
  virtual void deriveAt(float u, float v, int d, Matrix_Point3D& skl) const =0;
  
  virtual float minDist2(const Point3D& p, float& guessU, float& guessV, float error=0.001,float s=0.2,int sep=9,int maxIter=10,float um=0.0, float uM=1.0, float vm=0.0, float vM=1.0) const ;
  virtual float minDist2b(const Point3D& p, float& guessU, float& guessV, float error=0.001,float s=0.3,int sep=5,int maxIter=10,float um=0.0, float uM=1.0, float vm=0.0, float vM=1.0) const ;
  virtual float minDist2xy(const Point3D& p, float& guessU, float& guessV, float error=0.01,float dU=0.0001, float s=0.3,int sepU=5, int sepV=5, int maxIter=10,float um=0.0, float uM=1.0, float vm=0.0, float vM=1.0) const ;

  int projectOn(const Point3D& p, float& u, float& v, int maxI=100, const float um=0.0, const float uM=1.0, const float vm=0.0, const float vM=1.0) const ;

  enum CoordinateType {coordX, coordY, coordZ} ;
  float extremum(int findMin, CoordinateType coord, float minDu=0.0001, int sepU=5, int sepV=5, int maxIter=10, float um=0.0, float uM=1.0, float vm=0.0, float vM=1.0) const ;


  virtual int writeVRML(const char* filename,const Color& color,int Nu,int Nv, float u_s, float u_e, float v_s, float v_e) const ;
  virtual int writeVRML(const char* filename, const Color& color=whiteColor, int Nu=20, int Nv=20) const=0 ;
};

#endif 

