/*=============================================================================
        File: nurbsGL.cc
     Purpose:       
    Revision: $Id: nurbsGL.cc,v 2.11 1998/06/03 18:22:29 lavoie Exp $
  Created by: Philippe Lavoie          (28 September 1997)
 Modified by: 

 Copyright notice:
          Copyright (C) 1996-1998 Philippe Lavoie
 
          This library is free software; you can redistribute it and/or
          modify it under the terms of the GNU Library General Public
          License as published by the Free Software Foundation; either
          version 2 of the License, or (at your option) any later version.
 
          This library is distributed in the hope that it will be useful,
          but WITHOUT ANY WARRANTY; without even the implied warranty of
          MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
          Library General Public License for more details.
 
          You should have received a copy of the GNU Library General Public
          License along with this library; if not, write to the Free
          Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
=============================================================================*/
#include <nurbsGL.hh>
#include <string.h>

#ifdef WITH_OPENGL

Color objectColorDefault(255,255,255) ;
Color selectColorDefault(0,255,0) ;
Color currentColorDefault(0,255,255) ;

Color axisXColorDefault(255,0,0) ;
Color axisYColorDefault(0,255,0) ;
Color axisZColorDefault(0,0,255) ;

Color cpointColorDefault(255,0,0) ;
Color cpoint0ColorDefault(255,255,255) ;
Color cpointActiveColorDefault(255,0,255) ;
Color cPolygonColorDefault(255,255,0)  ;

Color knotColorDefault(255,255,0) ;
Color knotActiveColorDefault(0,255,255) ;

CtrlPnt4D dummyCPoint ;

int pSizeDefault = 4 ;

PlObjectGLState objectStateDefault = objectState;


const int PLNURBS_FLAGS_AFFECT_ALL = 1 ;
const int PLNURBS_FLAGS_AFFECT_ACTIVE_ONLY = 2 ;
const int PLNURBS_FLAGS_AFFECT_SELECTED_ONLY = 4 ;
const int PLNURBS_FLAGS_VIEW_BBOX = 8 ; // view the bbox even if object is in hidden.

const int PLNURBSLIST_DELETE_AT_RESET = 1;
const int PLNURBSLIST_KEEP_AT_RESET = 0 ;

const int PLNURBS_DISPLAY_NORMAL = 0 ;
const int PLNURBS_DISPLAY_ISOCURVES  = 1 ;
const int PLNURBS_DISPLAY_SHADED = 2 ;
const int PLNURBS_DISPLAY_HIDDEN_LINES = 3 ; 
const int PLNURBS_DISPLAY_TESSELATION = 4 ;

float tessel_tolerance ;

int PlNurbsDisplayMode = 0 ;

void initColorsGL(){
  objectColorDefault = Color(0,0,0) ;
  selectColorDefault = Color(0,255,0) ;
  currentColorDefault = Color(0,255,255) ;
  
  axisXColorDefault = Color(255,0,0) ;
  axisYColorDefault = Color(0,255,0) ;
  axisZColorDefault = Color(0,0,255) ;
  
  cpointColorDefault = Color(255,0,0) ;
  cpoint0ColorDefault = Color(255,255,255) ;
  cPolygonColorDefault = Color(255,255,0)  ;
  
  objectStateDefault = objectState;  
}


/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
       Member: PlObjectGL --- The basic constructor
  Description: It sets default values to the state and the colors. The
               default values are defined by the default variables:
	       objectColorDefault, selectColorDefault, currentColorDefault
	       and objectStateDefault.
        Input: 
       Output: 
 Restrictions: 
   Created by: Philippe Lavoie (20 September 1997)
  Modified by:
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
PlObjectGL::PlObjectGL(){
  prev_ = 0 ;
  next_ = 0 ;
  category = badType ; 
  callListId = 0 ;
  state = objectStateDefault ;
  objectColor = objectColorDefault ;
  selectColor = selectColorDefault ;
  currentColor = currentColorDefault ;
  tx = ty = tz = 0 ;
  rx = ry = rz = 0 ;
  sx = sy = sz = 1 ;
  materialColor = new float[4] ;
  active = selected = 0 ;
}

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
       Member: glTransform --- performs the local transformation
  Description: Performs the local transformation in this order:
               scaling, translation, rotation in x, rotation in y
	       and rotation in z.
        Input: 
       Output: 
 Restrictions: 
   Created by: Philippe Lavoie (20 September 1997)
  Modified by:
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
void PlObjectGL::glTransform() const { 
  glScalef(sx,sy,sz) ; 
  glTranslatef(tx,ty,tz) ; 
  glRotatef(rx,1,0,0) ; 
  glRotatef(ry,0,1,0) ; 
  glRotatef(rz,0,0,1) ; 
}

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
       Member: ~PlObjectGL --- The destructor 
  Description: The destructor
        Input: 
       Output: 
 Restrictions: 
   Created by: Philippe Lavoie (30 September 1997)
  Modified by:
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
PlObjectGL::~PlObjectGL(){
  /*
  if(next_){
    next_->prev_ = prev_ ;
  }
  if(prev_){
    prev_->next_ = next_ ;
  }
  */
  next_ = 0 ;
  prev_ = 0 ;
  if(materialColor)
    delete materialColor ;
}

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
       Member: PlObjectListGL --- The constructor
  Description: By default the reset mode is set to delete all the elements
               from the list (\verb.PLNURBSLIST_DELETE_AT_RESET.). If you 
	       don't want this behavior, call
	       \verb.setResetMode(PLNURBS_KEEP_AT_RESET). 
        Input: 
       Output: 
 Restrictions: 
   Created by: Philippe Lavoie (30 September 1997)
  Modified by:
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
PlObjectListGL::PlObjectListGL():PlObjectGL(),first(first_),last(last_),current(current_){
  first_ = 0 ;
  last_ = 0 ;
  current_ = 0 ;
  n = 0 ;
  resetMode = PLNURBSLIST_DELETE_AT_RESET ;
}

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
       Member: ~PlObjectListGL --- The destructor 
  Description: The destructor
        Input: 
       Output: 
 Restrictions: 
   Created by: Philippe Lavoie (30 September 1997)
  Modified by:
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
PlObjectListGL::~PlObjectListGL(){
  reset() ;
}

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
       Member: ~PlObjectRefListGL --- The destructor 
  Description: The destructor
        Input: 
       Output: 
 Restrictions: 
   Created by: Philippe Lavoie (3 June 1998)
  Modified by:
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
PlObjectRefListGL::~PlObjectRefListGL(){
  reset() ;
}

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
       Member: reset --- deletes all the node of the list
  Description: Deletes all the node of the list. Depending on the reset
               mode, it will also delete the elements.
        Input: 
       Output: 
 Restrictions: 
   Created by: Philippe Lavoie (30 September 1997)
  Modified by:
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
void PlObjectListGL::reset(){
  if(resetMode==PLNURBSLIST_DELETE_AT_RESET){
    PlObjectGL *c ;
    c = first_ ;
    while(c){
      current_ = c ;
      c = current_->next() ;
      delete current_ ;
    }
  }
  first_ = current_ = last_ = 0 ;
  n = 0 ;
}

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
       Member: transformBy --- transforms the elements stored in the list
  Description: Transforms the elements stored in the list {\em by} a certain 
               value if the elements are in a proper state. The behavior 
	       variable specifies which objects should be affected by this 
	       function.
        Input: x --> a translation in the $x$ direction
	       y --> a translation in the $y$ direction
	       z --> a translation in the $z$ direction
	       a --> a rotation around the $x$-axis
	       b --> a rotation around the $y$-axis
	       c --> a rotation around the $z$-axis
	      sx --> a scaling in the direction of the $x$-axis
	      sy --> a scaling in the direction of the $x$-axis
	      sz --> a scaling in the direction of the $x$-axis
        behavior --> specifies which objects ared affected by the function
       Output: 
 Restrictions: 
   Created by: Philippe Lavoie (30 September 1997)
  Modified by:
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
void PlObjectListGL::transformBy(GLfloat x, GLfloat y, GLfloat z, GLfloat a, GLfloat b, GLfloat c,GLfloat sx, GLfloat sy, GLfloat sz, int behavior){
  PlObjectGL *t ;
  t = first_ ;
  while(t){
    if(!(behavior & PLNURBS_FLAGS_AFFECT_ALL)){
      if(behavior & PLNURBS_FLAGS_AFFECT_ACTIVE_ONLY){
	if(!t->isActive()){
	  t = t->next() ;
	  continue ;
	}
      }
      if(behavior & PLNURBS_FLAGS_AFFECT_SELECTED_ONLY){
	if(!t->isSelected()){
	  t = t->next() ;
	  continue ;
	}
      }
    }
    t->tx += x ;
    t->ty += y ;
    t->tz += z ;
    t->rx += a ;
    t->ry += b ;
    t->rz += c ;
    t->sx += sx ;
    t->sy += sy ;
    t->sz += sz ;
    t = t->next() ;
   }
}

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
       Member: transformTo --- transforms the elements stored in the list
  Description: Transforms the elements stored in the list {\em to} a certain 
               value if the elements are in a proper state. The behavior 
	       variable specifies which objects should be affected by this 
	       function.
        Input: x --> a translation in the $x$ direction
	       y --> a translation in the $y$ direction
	       z --> a translation in the $z$ direction
	       a --> a rotation around the $x$-axis
	       b --> a rotation around the $y$-axis
	       c --> a rotation around the $z$-axis
	      sx --> a scaling in the direction of the $x$-axis
	      sy --> a scaling in the direction of the $x$-axis
	      sz --> a scaling in the direction of the $x$-axis
        behavior --> specifies which objects ared affected by the function
       Output: 
 Restrictions: 
   Created by: Philippe Lavoie (30 September 1997)
  Modified by:
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
void PlObjectListGL::transformTo(GLfloat x, GLfloat y, GLfloat z, GLfloat a, GLfloat b, GLfloat c,GLfloat sx, GLfloat sy, GLfloat sz, int behavior){
  PlObjectGL *t ;
  t = first_ ;
  while(t){
    if(!(behavior & PLNURBS_FLAGS_AFFECT_ALL)){
      if(behavior & PLNURBS_FLAGS_AFFECT_ACTIVE_ONLY){
	if(!t->isActive()){
	  t = t->next() ;
	  continue ;
	}
      }
      if(behavior & PLNURBS_FLAGS_AFFECT_SELECTED_ONLY){
	if(!t->isSelected()){
	  t = t->next() ;
	  continue ;
	}
      }
    }
    t->tx = x ;
    t->ty = y ;
    t->tz = z ;
    t->rx = a ;
    t->ry = b ;
    t->rz = c ;
    t->sx = sx ;
    t->sy = sy ;
    t->sz = sz ;
    t = t->next() ;
   }
}

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
       Member: glObject --- displays a control point
  Description: Displays a control point.
        Input: 
       Output: 
 Restrictions: 
   Created by: Philippe Lavoie (30 September 1997)
  Modified by:
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
void PlCPointGL::glObject() const {
  glPointSize(psize) ;
  glBegin(GL_POINTS) ;
  glColor() ;
  glVertex4fv(cpoint.data) ;
  glEnd() ;
}

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
       Member: glObject --- displays a list of points
  Description: Displays a list of points.
        Input: 
       Output: 
 Restrictions: 
   Created by: Philippe Lavoie (29 Mars 1998)
  Modified by:
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
void PlPointListGL::glObject() const {
  BasicNode<Point3D> *node ;
  node = (BasicNode<Point3D>*)list.first ;

  glPointSize(psize) ;
  glBegin(GL_POINTS) ;  
  //for(int i=0;i<list.size();++i){
  for(int i=0;i<minimum(20000,list.size());++i){
    glColor() ;
    //glVertex3fv(&(list.current->data->x)) ;
    glVertex3f(node->data->x,node->data->y,node->data->z) ;
    node = node->next ; 
  }
  glEnd() ;
}


/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
       Member: PlPointListGL --- constructor from a list of points
  Description: Constructor from a list of points
        Input: l --> list of points
       Output: 
 Restrictions: 
   Created by: Philippe Lavoie (30 Mars 1998)
  Modified by:
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
PlPointListGL::PlPointListGL(const BasicList<Point3D> &l): PlObjectGL(){
  category = pointType ;
  type = pointListObject ;
  psize = pSizeDefault ;
  list = l ;
}


/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
       Member: PlPointListGL --- copy constructor
  Description: Copy constructor
        Input: pl --> list to copy
       Output: 
 Restrictions: 
   Created by: Philippe Lavoie (30 Mars 1998)
  Modified by:
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
PlPointListGL::PlPointListGL(const PlPointListGL &pl): PlObjectGL(){
  category = pointType ;
  type = pointListObject ;
  psize = pl.psize ;
  list = pl.list ;
}


/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
       Member: read --- reads a list of points
  Description: Reads a list of points.
        Input: fin --> input file stream
       Output: 
 Restrictions: 
   Created by: Philippe Lavoie (29 Mars 1998)
  Modified by:
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
int PlPointListGL::read(ifstream &fin) {
  list.reset() ;

  char *type ;
  type = new char [3] ;

  if(!fin.read(type,sizeof(char)*3)) { delete []type ; return 0 ;}
  int r1 = strncmp(type,"pt3",3) ;
  
  if(r1!=0){
    delete []type ;
    return 0 ;
  }
  setPsize(2) ;

  while(!fin.eof()){
    int mark = fin.tellg() ; // mark position
    if(!fin.read(type,sizeof(char)*3)) { delete []type ; return 1 ; }
    fin.seekg(mark) ; // restore position
    char* ext ;
    ext = strstr(type,"#") ;
    if(ext) { // this line is a comment
      while( *ext != '\n'){
	fin.read(ext,sizeof(char)) ;
      }
    }
    else { 
      ext = strstr(type,"X") ;
      if(ext){ // end of list of points
	while(*type != 'X')
	  fin.read(type,sizeof(char)) ;
	fin.read(type,sizeof(char));
	delete []type ;
	return 1 ;
      }
      // it should be 3 points as in    x  y  z
      double x, y, z ;
      if(fin >> x >> y >> z){
	Point3D p;
	//float scale = 10.0 ;
	//p = Point3D(x/scale,y/scale,z/scale) ;
	p = Point3D(x,y,z) ;
	/*
	if(list.size()==0)
	  offset = p ;
	p -= offset  ;
	*/
	list.add(p) ;
      }
      else{
	delete []type ;
	return 0 ;
      }
    }
  }
  return 1 ;

}

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
       Member: read --- writes a list of points
  Description: Writes a list of points.
        Input: fin --> output file stream
       Output: 
 Restrictions: 
   Created by: Philippe Lavoie (29 Mars 1998)
  Modified by:
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
int PlPointListGL::write(ofstream &fout) const {
  fout << "pt3" ;
  BasicNode<Point3D> *node ;
  node = (BasicNode<Point3D>*)list.first ;

  for(int i=0;i<list.size();++i){
    fout << *(node->data) << endl ;
    node = node->next ;
  }
  fout << "X\n" ;
  if(!fout)
    return 0 ;
  return 1 ;
}


/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
       Member: glObject --- displays a control point
  Description: Displays a control point.
        Input: 
       Output: 
 Restrictions: 
   Created by: Philippe Lavoie (3 November 1997)
  Modified by:
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
void PlHCPointGL::glObject() const {
  glPointSize(psize) ;
  glBegin(GL_POINTS) ;
  glColor() ;
  glVertex4fv(s->CtrlP(i0,j0).data) ;
  glEnd() ;
}

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
       Member: glObject --- displays a control point
  Description: Displays a control point on the surface of a NURBS curve or a
               NURBS surface.
        Input: 
       Output: 
 Restrictions: 
   Created by: Philippe Lavoie (12 May 1998)
  Modified by:
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
void PlSPointCurveGL::glObject() const {
  glPointSize(psize) ;
  glBegin(GL_POINTS) ;
  glColor() ;
  glVertex4fv(cpoint.data) ;
  glEnd() ;
}

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
       Member: glObject --- displays a control point
  Description: Displays a control point on the surface of a NURBS curve or a
               NURBS surface.
        Input: 
       Output: 
 Restrictions: 
   Created by: Philippe Lavoie (12 May 1998)
  Modified by:
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
void PlSPointSurfaceGL::glObject() const {
  glPointSize(psize) ;
  glBegin(GL_POINTS) ;
  glColor() ;
  glVertex4fv(cpoint.data) ;
  glEnd() ;
}

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
       Member: glObject --- displays a control point
  Description: Displays a control point on the surface of a NURBS curve or a
               NURBS surface.
        Input: 
       Output: 
 Restrictions: 
   Created by: Philippe Lavoie (12 May 1998)
  Modified by:
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
void PlSPointHSurfaceGL::glObject() const {
  glPointSize(psize) ;
  glBegin(GL_POINTS) ;
  glColor() ;
  glVertex4fv(cpoint.data) ;
  glEnd() ;
}

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
       Member: modify --- modifies the offset point.
  Description: modifies the offset point.
        Input: 
       Output: 
 Restrictions: 
   Created by: Philippe Lavoie (3 November 1997)
  Modified by:
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
void PlHCPointGL::modify(const CtrlPnt4D& v) {
  if(canModify){
    offset += v ; 
    s->updateSurface(i0,j0) ;
  }
}

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
       Member: glObject --- displays a point
  Description: Displays a point.
        Input: 
       Output: 
 Restrictions: 
   Created by: Philippe Lavoie (30 September 1997)
  Modified by:
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
void PlPointGL::glObject() const {
  glPointSize(psize) ;
  glBegin(GL_POINTS) ;
  glColor() ;
  glVertex3fv(p.data) ;
  glEnd() ;
}

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
       Member: glObject --- displays a point
  Description: Displays a knot.
        Input: 
       Output: 
 Restrictions: 
   Created by: Philippe Lavoie (30 September 1997)
  Modified by:
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
void PlKnotGL::glObject() const {
  glPointSize(psize) ;
  glBegin(GL_POINTS) ;
  glColor() ;
  glVertex3fv(p.data) ;
  glEnd() ;
}

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
       Member: glObject --- calls glObject for all the elements from the list
  Description: Displays all the elements from the list
        Input: 
       Output: 
 Restrictions: 
   Created by: Philippe Lavoie (30 September 1997)
  Modified by:
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
void PlObjectListGL::glObject() const {
  PlObjectGL *t ;
  t = first_ ;
  while(t){
    if(t->getState())
      t->glObject() ;
    t = t->next() ;
  }
}

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
       Member: display --- displays all the elements from the list
  Description: Displays all the elements from the list
        Input: 
       Output: 
 Restrictions: 
   Created by: Philippe Lavoie (30 September 1997)
  Modified by:
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
void PlObjectListGL::display() const {
  PlObjectGL *t ;
  t = first_ ;
  while(t){
    if(t->getState()){
      glPushMatrix() ;
      t->glTransform() ;
      t->glObject() ;
      glPopMatrix() ;
    }
    t = t->next() ;
  }
}

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
       Member: displayName --- displays and name all the elements from the list
  Description: Displays all the elements from the list
        Input: 
       Output: 
 Restrictions: 
   Created by: Philippe Lavoie (30 September 1997)
  Modified by:
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
void PlObjectListGL::displayName() const {
  PlObjectGL *t ;
  int name = 1 ;
  t = first_ ;
  while(t){
    if(t->getState()){
      glPushMatrix() ;
      t->glTransform() ;
      glLoadName(name) ;
      t->glObject() ;
      glPopMatrix() ;
      ++name ;
    }
    t = t->next() ;
  }
}

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
       Member: glObject --- calls glObject for all the Nurbs Object in the list
  Description: Calls glObject for all the Nurbs Object in the list.
        Input: 
       Output: 
 Restrictions: 
   Created by: Philippe Lavoie (30 September 1997)
  Modified by:
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
void PlNurbsListGL::glObject() const {
  PlObjectGL *t ;
  t = first_ ;
  while(t){
    t->glObject() ;
    t = t->next() ;
  }
}

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
       Member: glObject --- displays all the NURBS object from the list
  Description: Displays all the NURBS object from the list.
        Input: 
       Output: 
 Restrictions: 
   Created by: Philippe Lavoie (30 September 1997)
  Modified by:
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
void PlNurbsListGL::display() const {
  PlObjectGL *t ;
  t = first_ ;
  while(t){
    glPushMatrix() ;
    t->glTransform() ;
    t->glObject() ;
    glPopMatrix() ;
    t = t->next() ;
  }
}

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
       Member: resetDisplayFlags --- Resets the display flags for the elements
  Description: Resets the display flags for the elements in the list. It
               will only resets the flags according to the behavior value.
        Input:  o --> display flag
	       cp --> display the control points
	        p --> display the control polygon
		b --> display the bounding box
		k --> display the knots
	 behavior --> specifies which object are affected by the function
       Output: 
 Restrictions: 
   Created by: Philippe Lavoie (30 September 1997)
  Modified by:
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
void PlNurbsListGL::resetDisplayFlags(int o, int cp, int p, int b, int k, int behavior) {
  PlObjectGL *pnt ;
  pnt = (PlObjectGL*)first_ ;

  PlNurbsGL *t ;
  while(pnt){
    if(pnt->category == nurbsType)
      t = (PlNurbsGL*)pnt ;
    else{
      pnt = pnt->next() ;
      continue ;
    }
      
    if(!(behavior & PLNURBS_FLAGS_AFFECT_ALL)){
      if(behavior & PLNURBS_FLAGS_AFFECT_ACTIVE_ONLY){
	if(!t->isActive()){
	  //t = (PlNurbsGL*)t->next() ;
	  pnt = pnt->next() ;
	  continue ;
	}
      }
      if(behavior & PLNURBS_FLAGS_AFFECT_SELECTED_ONLY){
	if(!t->isSelected()){
	  //t = (PlNurbsGL*)t->next() ;
	  pnt = pnt->next() ;
	continue ;
	}
      }
    }
    if(o){
      if(cp)
	t->viewCPoints() ;
      else
	t->hideCPoints() ;
      if(p)
	t->viewCpolygon() ;
      else
	t->hideCpolygon() ;
      if(b)
	t->viewBBox() ;
      else
	t->hideBBox() ;
      if(k)
	t->viewKnots() ;
      else
	t->hideKnots() ;
      t->viewObject() ;
    }
    else{
      if(behavior & PLNURBS_FLAGS_VIEW_BBOX){
	if(b)
	  t->viewBBox() ;
	else
	  t->hideBBox() ;
      }
      t->hideObject() ;
    }
    
    pnt = pnt->next() ; // (PlNurbsGL*)t->next() ;
  }
}

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
       Member: deactivate --- deactivate all the objects of the list
  Description: Deactivates all the objects of the list.
        Input: 
       Output: 
 Restrictions: 
   Created by: Philippe Lavoie (30 September 1997)
  Modified by:
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
void PlObjectListGL::deactivate() {
  PlObjectGL *t ;
  t = first_ ;
  while(t){
    t->deactivate() ;
    t = t->next() ;
  }
}

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
       Member: activate --- activate all the objects of the list
  Description: Activate all the objects of the list
        Input: 
       Output: 
 Restrictions: 
   Created by: Philippe Lavoie (30 September 1997)
  Modified by:
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
void PlObjectListGL::activate(){
  PlObjectGL *t ;
  t = first_ ;
  while(t){
    t->activate() ;
    t = t->next() ;
  }
}

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
       Member: select --- select all the objects of the list
  Description: Select all the objects of the list
        Input: 
       Output: 
 Restrictions: 
   Created by: Philippe Lavoie (30 September 1997)
  Modified by:
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
void PlObjectListGL::select(){
  PlObjectGL *t ;
  t = first_ ;
  while(t){
    t->select() ;
    t = t->next() ;
  }
}


/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
       Member: deselect --- deselect all the objects of the list
  Description: Deselect all the objects of the list
        Input: 
       Output: 
 Restrictions: 
   Created by: Philippe Lavoie (2 October 1997)
  Modified by:
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
void PlObjectListGL::deselect(){
  PlObjectGL *t ;
  t = first_ ;
  while(t){
    t->deselect() ;
    t = t->next() ;
  }
}

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
       Member: goTo --- moves the current pointer to the $n$th element
  Description: Moves the current pointer to the $n$th element. This must
               be in a valid range otherwise 0 is returned.
        Input: a --> the $a$th element 
       Output: A pointer to the current element or 0 if a was out of range.
 Restrictions: 
   Created by: Philippe Lavoie (30 October 1997)
  Modified by:
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
PlObjectGL* PlObjectListGL::goTo(int a){
  if(a>=n)
    return 0 ;
  current_ = first_ ;
  while(current_ && a>0){
    current_ = current_->next() ;
    --a ;
  }
  return current_ ;
}

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
       Member: goToActive --- moves the current pointer to the $n$th active 
                              element
  Description: Moves the current pointer to the $n$th active element. This 
               must be in a valid range otherwise 0 is returned.
        Input: a --> the $a$th active element 
       Output: A pointer to the current element or 0 if a was out of range.
 Restrictions: 
   Created by: Philippe Lavoie (29 January 1998)
  Modified by:
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
PlObjectGL* PlObjectListGL::goToActive(int a){
  if(a>=n)
    return 0 ;
  current_ = first_ ;
  while(current_ ){
    if(current_->getState())
      --a ;
    if(a>=0)
      current_ = current_->next() ;
    else
      break ;
  }
  return current_ ;
}

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
       Member: goToNext --- move the current pointer to the next element
  Description: Moves the current pointer to the next element. If there are
               no next element, it goes to the first element.
        Input: 
       Output: A pointer to the current element
 Restrictions: 
   Created by: Philippe Lavoie (2 October 1997)
  Modified by:
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
PlObjectGL* PlObjectListGL::goToNext(){
  if(current_->next()){
    current_ = current_->next() ;
  }
  else
    current_ = first_ ;
  return current_ ;
}

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
       Member: goToPrevious --- move the current pointer to the previous one
  Description: Moves the current pointer to the previous element. If there are
               no previous element, it goes to the last element.
        Input: 
       Output: A pointer to the current element
 Restrictions: 
   Created by: Philippe Lavoie (30 September 1997)
  Modified by:
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
PlObjectGL* PlObjectListGL::goToPrevious(){
  if(current_->previous()){
    current_ = current_->previous() ;
  }
  else
    current_ = last_ ;
  return current_ ;
}

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
       Member: goToNextActive --- move the current pointer to the next 
                                  active element
  Description: Moves the current pointer to the next element. If there are
               no next element, it goes to the first element.
        Input: 
       Output: A pointer to the current element
 Restrictions: 
   Created by: Philippe Lavoie (29 January 1998)
  Modified by:
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
PlObjectGL* PlObjectListGL::goToNextActive(){
  PlObjectGL *na ;

  na = 0 ;
  if(current_)
    na = current_->next() ;
  if(!na)
    na = first_ ;

  if(na){
    while(!na->getState()){
      if(!na->next())
	na = first_ ;
      else
	na = na->next() ;
      if(na == current_)
	break ;
    }
  }

  current_ = na ;

  return current_ ;
}

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
       Member: goToPreviousActive --- move the current pointer to the previous 
                                      activeElement
  Description: Moves the current pointer to the previous element. If there are
               no previous element, it goes to the last element.
        Input: 
       Output: A pointer to the current element
 Restrictions: 
   Created by: Philippe Lavoie (29 January 1998)
  Modified by:
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
PlObjectGL* PlObjectListGL::goToPreviousActive(){
  PlObjectGL *na ;

  na = 0 ;
  if(current_)
    na = current_->previous() ;
  if(!na)
    na = last_ ;

  if(na){
    while(!na->getState()){
      if(!na->previous())
	na = last_ ;
      else
	na = na->previous() ;
      if(na == current_)
	break ;
    }
  }
  current_ = na ;

  return current_ ;
}

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
       Member: jumpToNext --- move the current pointer to the next element
  Description: Moves the current pointer to the next element. If there are
               no next element, it goes to the first element.
        Input: 
       Output: A pointer to the current element
 Restrictions: 
   Created by: Philippe Lavoie (2 October 1997)
  Modified by:
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
PlObjectGL* PlObjectListGL::jumpToNext(){
  for(int i=0;i<jumpSize-1;++i)
    goToNext() ;
  return goToNext() ;
}

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
       Member: jumpToPrevious --- move the current pointer to the next element
  Description: Moves the current pointer to the next element. If there are
               no next element, it goes to the first element.
        Input: 
       Output: A pointer to the current element
 Restrictions: 
   Created by: Philippe Lavoie (2 October 1997)
  Modified by:
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
PlObjectGL* PlObjectListGL::jumpToPrevious(){
  for(int i=0;i<jumpSize-1;++i)
    goToPrevious() ;
  return goToPrevious() ;
}

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
       Member: add --- adds an element to the list
  Description: Adds an element to the list.
        Input: obj --> the element to add
       Output: 
 Restrictions: 
   Created by: Philippe Lavoie (30 September 1997)
  Modified by:
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
void PlObjectListGL::add(PlObjectGL* obj){
  if(obj){
    if(!first_){
      first_ = obj ;
    }
    else{
      last_->next() = obj ;
      obj->previous() = last_ ;
    }
    last_ = obj ;
    obj->next() = 0 ; 
    current_ = obj ;
    ++n ;
  }
}

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
       Member: remove --- finds an element and remove it from the list
  Description: Finds an element and delete it from the list. The element
               will {\em not} be deleted. This is up to the calling function.
        Input: obj --> the element to search
       Output: a pointer to obj if it was found in the list, 0 otherwise
 Restrictions: 
   Created by: Philippe Lavoie (30 September 1997)
  Modified by:
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
PlObjectGL* PlObjectListGL::remove(PlObjectGL* obj){
  PlObjectGL* t ;

  if(!obj)
    return 0 ;

  if(current_ == obj){
    t = obj ;
    current_ = 0 ;
    if(t->previous()){
      t->previous()->next() = t->next() ;
      current_ = t->previous() ;
    }
    if(t->next()){
      t->next()->previous() = t->previous() ;
      current_ = t->next() ;
    }
    --n ;
    if(first==t)
      first = t->next() ;
    if(last==t)
      last = t->previous() ;
    return t;
  }

  t = first_ ;
  while(t){
    if(t==obj){
      if(t->previous())
	t->previous()->next() = t->next() ;
      if(t->next())
	t->next()->previous() = t->previous() ;
      --n ;
      if(first==t)
	first = t->next() ;
      if(last==t)
	last = t->previous() ;
      return t;
    }
    else
      t = t->next() ;
  }
  return 0 ;
}

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
       Member: remove --- finds an element and remove it from the list
  Description: Finds an element and delete it from the list. The element
               will {\em not} be deleted. This is up to the calling function.
	       Since this is a reference list, we check if the element is
	       referencing obj. If it is,  we remove it from the list. If 
	       not, we check if obj is one of the element from the list
	       and if so we remove it.
        Input: obj --> the element to remove
       Output: the element from the list or 0 if obj wasn't found.
 Restrictions: 
   Created by: Philippe Lavoie (30 September 1997)
  Modified by:
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
PlObjectGL* PlObjectRefListGL::remove(PlObjectGL* obj){
  PlObjectGL* t ;
  PlObjectRefGL* t2 ;

  if(((PlObjectRefGL*)current_)->ptr == obj){
    t2 = (PlObjectRefGL*)current_ ;
    current_ = 0 ;
    if(t2->previous()){
      t2->previous()->next() = t2->next() ;
      current_ = t2->previous() ;
    }
    if(t2->next()){
      t2->next()->previous() = t2->previous() ;
      current_ = t2->next() ;
    }
    --n ;
    if(first==t2)
      first = t2->next() ;
    if(last==t2)
      last = t2->previous() ;
    return t2;
  }

  t2 = (PlObjectRefGL*)first_ ;
  while(t2){
    if(t2->ptr==obj){
      if(t2->previous())
	t2->previous()->next() = t2->next() ;
      if(t2->next())
	t2->next()->previous() = t2->previous() ;
      --n ;
      if(first==t2)
	first = t2->next() ;
      if(last==t2)
	last = t2->previous() ;
      return t2;
    }
    else
      t2 = (PlObjectRefGL*)t2->next() ;
  }

  if(current_ == obj){
    t = obj ;
    current_ = 0 ;
    if(t->previous()){
      t->previous()->next() = t->next() ;
      current_ = t->previous() ;
    }
    if(t->next()){
      t->next()->previous() = t->previous() ;
      current_ = t->next() ;
    }
    --n ;
    if(first==t)
      first = t->next() ;
    if(last==t)
      last = t->previous() ;
    return t;
  }

  t = first_ ;
  while(t){
    if(t==obj){
      if(t->previous())
	t->previous()->next() = t->next() ;
      if(t->next())
	t->next()->previous() = t->previous() ;
      --n ;
      if(first==t)
	first = t->next() ;
      if(last==t)
	last = t->previous() ;
      return t;
    }
    else
      t = t->next() ;
  }
  return 0 ;
}

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
       Member: refList --- reference all the elements from a list
  Description: Reference all the elements from a list. If the addOnce variable
               is set, it will reference the elements of the list only once. 
	       So if the reference list was already referencing one of the 
	       elements, it will not be added again. If the addOnce variable
	       is not set, the resulting list might reference an element
	       more than once.
        Input: list --> the list to reference
       Output: 
 Restrictions: 
   Created by: Philippe Lavoie (30 September 1997)
  Modified by:
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
void PlObjectRefListGL::refList(const PlObjectListGL* list, int addOnce) {
  PlObjectGL *t ;
  t = (PlObjectGL*)list->first ;
  while(t){
    if(addOnce){
      int addOk ;
      // check if t is already in the list
      PlObjectRefGL *t2 ;
      t2 = (PlObjectRefGL*)first_ ;
      addOk = 1 ;
      while(t2){
	if(t2->ptr == t){
	  addOk = 0 ;
	  break ;
	}
	t2 = (PlObjectRefGL*)t2->next() ;
      }
      if(addOk)
	add(t) ;
    }
    else
      add(t) ;
    t = t->next() ;
  }
}


/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
       Member: glNewList --- generates a default call list
  Description: Generates a default call list. It generates a call list
               from the glObject() call using \verb.GL_COMPILE..
        Input: 
       Output: The call list identification number for the object if the call
               was succesfull, 0 otherwise.
 Restrictions: 
   Created by: Philippe Lavoie (23 September 1997)
  Modified by:
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
void PlObjectGL::glNewList(){
  if(callListId)
    glDeleteLists(callListId,1) ;

  callListId = glGenLists(1) ;
  ::glNewList(callListId,GL_COMPILE) ;
  glObject() ;
  glEndList() ;
}

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
       Member: PlBoundingBoxGL --- an object to represent a bounding box
  Description: This will set the colors for the X,Y and Z axis to the value
               specified by the axisXColorDefault, axisYColorDefault and
	       axisZColorDefault global variables.
	       Use setColorXYZ() if you want to change this default.
        Input: 	       
       Output: 
 Restrictions: 
   Created by: Philippe Lavoie (23 September 1997)
  Modified by:
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
PlBoundingBoxGL::PlBoundingBoxGL():PlObjectGL() {
  objectColor = cpoint0ColorDefault ;
  colorX = axisXColorDefault ;
  colorY = axisYColorDefault ;
  colorZ = axisZColorDefault ;
  type = bboxObject ;
}


/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
       Member: glObject --- generates a bounding box 
  Description: This function generates a bounding box around the NURBS object.
        Input: 	       
       Output: 
 Restrictions: 
   Created by: Philippe Lavoie (23 September 1997)
  Modified by:
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
void PlBoundingBoxGL::glObject() const {
  glBegin(GL_LINES);

  //glColor3f(colorX) ;
  glColor(colorX) ;
  glVertex3f(minP.x,minP.y,minP.z) ;
  glVertex3f(maxP.x,minP.y,minP.z) ;

  //glColor3f(colorY) ;
  glColor(colorY) ;
  glVertex3f(minP.x,minP.y,minP.z) ;
  glVertex3f(minP.x,maxP.y,minP.z) ;

  //glColor3f(colorZ) ;
  glColor(colorZ) ;
  glVertex3f(minP.x,minP.y,minP.z) ;
  glVertex3f(minP.x,minP.y,maxP.z) ;

  glColor() ;
  glVertex3f(minP.x,minP.y,maxP.z) ;  glVertex3f(minP.x,maxP.y,maxP.z) ;
  glVertex3f(minP.x,maxP.y,minP.z) ;  glVertex3f(maxP.x,maxP.y,minP.z) ;
  glVertex3f(minP.x,maxP.y,minP.z) ;  glVertex3f(minP.x,maxP.y,maxP.z) ;
  glVertex3f(minP.x,minP.y,maxP.z) ;  glVertex3f(maxP.x,minP.y,maxP.z) ;
  glVertex3f(minP.x,maxP.y,maxP.z) ;  glVertex3f(maxP.x,maxP.y,maxP.z) ;
  glVertex3f(maxP.x,minP.y,minP.z) ;  glVertex3f(maxP.x,minP.y,maxP.z) ;
  glVertex3f(maxP.x,minP.y,maxP.z) ;  glVertex3f(maxP.x,maxP.y,maxP.z) ;
  glVertex3f(maxP.x,maxP.y,minP.z) ;  glVertex3f(maxP.x,minP.y,minP.z) ;
  glVertex3f(maxP.x,maxP.y,maxP.z) ;  glVertex3f(maxP.x,maxP.y,minP.z) ;

  glEnd() ;
}

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
      Routine: PlNurbsGL --- the default constructor
  Description: Initialized the colors of the objects.
        Input: 
       Output: 
 Restrictions: 
   Created by: Philippe Lavoie (23 September 1997)
  Modified by:
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
PlNurbsGL::PlNurbsGL() : 
  PlObjectGL(), editSP(1), nurbsRenderer(0), nUlines(4), nVlines(4)
{ 
  polygon = 0 ;
  nurbsState = objectState; 
  category = nurbsType ;
  editControlPoints(0) ;
  cpoints.objectColor = cpointColorDefault ;
  cpoints.currentColor = cpointActiveColorDefault ;
  knots.objectColor = knotColorDefault ;
  knots.currentColor = knotActiveColorDefault ;
  cpoints.hideObject() ;
  knots.hideObject() ;
  //resetAll(); 
}


/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
      Routine: glObject --- displays the object if it's not hiding
  Description: Displays the object if it's not hiding.
        Input: 
       Output: 
 Restrictions: 
   Created by: Philippe Lavoie (23 September 1997)
  Modified by:
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
void PlNurbsGL::glObject() const {
  if(getState()){
    if(nurbsState){
      gluNurbs() ;
    }
    if(cpoints.getState())
      cpoints.glObject() ;
    if(polygon->getState())
      polygon->glObject() ;
    if(knots.getState())
      knots.glObject() ;
  }
  if(bbox.getState())
    bbox.glObject() ;
}

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
      Routine: readNurbsObject --- reads a Nurb object from a file
  Description: Reads a NURBS object from a file. The routine can 
               read a Nurbs curve or surface.
        Input: filename --> the name of the file to read
	              o <-- a pointer to a Nurbs Object. If this pointer
		            as any value upon entry, the old entry is
			    deleted first.
       Output: 1 if the Nurbs object was read succesfully, 0 otherwise
 Restrictions: Calling this function without a proper curve initialized might
               result in strange results.
   Created by: Philippe Lavoie (23 September 1997)
  Modified by:
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
PlNurbsGL* readNurbsObject(const char* filename) {
  PlNurbsGL *temp ;
  // guess the type of the curve first, if that doesn't work try all of them
  char* ext ; 
  //ext = strstr(filename,".nca") ;
  //if(ext){
  //  openByType = OPENCURVEARRAY ;      
  //}
  //else{
  ext = strstr(filename,".nc") ;
  if(ext){
    temp = new PlNurbsCurveGL ;
    if(temp->read(filename)){
      temp->resetBoundingBox() ;
      return temp ;
    }
    else
      delete temp ;
  }
  else{
    ext = strstr(filename,".ns") ;
    if(ext){
      temp = new PlNurbsSurfaceGL ;
      if(temp->read(filename)){
	return temp ;
      }
      else
	delete temp ;
    }
  }
  //}
   
  // try all the types 1 by 1
  temp = new PlNurbsCurveGL ;
  if(temp->read(filename)){
    return temp ;
  }
  else
    delete temp ;
  
  temp = new PlNurbsSurfaceGL ;
  if(temp->read(filename)){
    return temp ;
  }
  else
    delete temp ;

  return 0 ;
}

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
       Member: resetBoundingBox --- resets the minP and maxP values of bbox
  Description: Resets the minP and maxP values for the bouding box.
        Input: 	       
       Output: 
 Restrictions: Calling this function without a proper curve initialized might
               result in strange results.
   Created by: Philippe Lavoie (23 September 1997)
  Modified by:
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
void PlNurbsCurveGL::resetBoundingBox(){
  bbox.minP.x = extremum(1,coordX) ;
  bbox.minP.y = extremum(1,coordY) ;
  bbox.minP.z = extremum(1,coordZ) ;
  bbox.maxP.x = extremum(0,coordX) ;
  bbox.maxP.y = extremum(0,coordY) ;
  bbox.maxP.z = extremum(0,coordZ) ;
}

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
       Member: resetBoundingBox --- resets the minP and maxP values of bbox
  Description: Resets the minP and maxP values for the bouding box.
        Input: 	       
       Output: 
 Restrictions: Calling this function without a proper surface initialized might
               result in strange results.
   Created by: Philippe Lavoie (23 September 1997)
  Modified by:
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
void PlNurbsSurfaceGL::resetBoundingBox(){
  bbox.minP.x = extremum(1,coordX) ;
  bbox.minP.y = extremum(1,coordY) ;
  bbox.minP.z = extremum(1,coordZ) ;
  bbox.maxP.x = extremum(0,coordX) ;
  bbox.maxP.y = extremum(0,coordY) ;
  bbox.maxP.z = extremum(0,coordZ) ;
}

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
       Member: resetBoundingBox --- resets the minP and maxP values of bbox
  Description: Resets the minP and maxP values for the bouding box.
        Input: 	       
       Output: 
 Restrictions: Calling this function without a proper surface initialized might
               result in strange results.
   Created by: Philippe Lavoie (23 September 1997)
  Modified by:
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
void PlHNurbsSurfaceGL::resetBoundingBox(){
  bbox.minP.x = extremum(1,coordX) ;
  bbox.minP.y = extremum(1,coordY) ;
  bbox.minP.z = extremum(1,coordZ) ;
  bbox.maxP.x = extremum(0,coordX) ;
  bbox.maxP.y = extremum(0,coordY) ;
  bbox.maxP.z = extremum(0,coordZ) ;
}


/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
       Member: gluNurbs --- creates a nurbs curve for OpenGL
  Description: This function calls between a gluBeginCurve/gluEndCurve
               the proper functions to generate a NURBS curve.
        Input: 	       
       Output: 
 Restrictions: 
   Created by: Philippe Lavoie (23 September 1997)
  Modified by:
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
void PlNurbsCurveGL::gluNurbs() const{
  if(nurbsRenderer){
    GLenum t = GL_MAP1_VERTEX_4 ;
    
    gluBeginCurve(nurbsRenderer) ;
    glColor() ;
    gluNurbsCurve(nurbsRenderer, U.n, U.memory(), 4, P.memory()->data, deg+1, t) ;
    gluEndCurve(nurbsRenderer) ;
    
    /*
      // This code tests the tesselation code for a Nurbs Curve
    BasicList<Point3D> list ;
    list = tesselate(0.01) ;
    BasicNode<Point3D>* node = (BasicNode<Point3D>*)list.first ;
    glBegin(GL_LINE_STRIP) ;
    glColor() ;
    while(node){
      glVertex3fv(node->data->data) ;
      node = node->next ;
    }    
    glEnd() ;
    */
  }
}

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
       Member: applyTransform --- apply the local transformation to the 
                                  curve.
  Description: Apply the local transformation to the curve. This is necessary
               if you want to get the proper position for the control points
	       before doing anymore processing on them.
        Input: 
       Output: 
 Restrictions: 
   Created by: Philippe Lavoie (23 September 1997)
  Modified by:
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
void PlPointListGL::applyTransform(){
  rx *= M_PI/180.0 ;
  ry *= M_PI/180.0 ;
  rz *= M_PI/180.0 ;

  MatrixRT<float> T(rx,ry,rz,tx,ty,tz) ;
  MatrixRT<float> Sx ; 
  Sx.scale(sx,sy,sz) ;

  MatrixRT<float> TM(T*Sx) ;

  BasicNode<Point3D> *node ;
  node = (BasicNode<Point3D>*)list.first ;
  
  while(node){
    *node->data = TM * (*node->data) ;
    node = node->next ;
  }
  
  tx = ty = tz = rx = ry = rz = 0 ;
  sx = sy = sz = 1 ;
}

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
       Member: applyTransform --- apply the local transformation to the 
                                  curve.
  Description: Apply the local transformation to the curve. This is necessary
               if you want to get the proper position for the control points
	       before doing anymore processing on them.
        Input: 
       Output: 
 Restrictions: 
   Created by: Philippe Lavoie (23 September 1997)
  Modified by:
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
void PlNurbsCurveGL::applyTransform(){
  rx *= M_PI/180.0 ;
  ry *= M_PI/180.0 ;
  rz *= M_PI/180.0 ;

  MatrixRT<float> T(rx,ry,rz,tx,ty,tz) ;
  MatrixRT<float> Sx ; 
  Sx.scale(sx,sy,sz) ;

  transform(T*Sx) ;
  
  tx = ty = tz = rx = ry = rz = 0 ;
  sx = sy = sz = 1 ;
  resetAll() ;
}

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
       Member: applyTransform --- apply the local transformation to the 
                                  surface.
  Description: Apply the local transformation to the surface. This is necessary
               if you want to get the proper position for the control points
	       before doing anymore processing on them.
        Input: 
       Output: 
 Restrictions: 
   Created by: Philippe Lavoie (23 September 1997)
  Modified by:
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
void PlNurbsSurfaceGL::applyTransform(){
  rx *= M_PI/180.0 ;
  ry *= M_PI/180.0 ;
  rz *= M_PI/180.0 ;

  MatrixRT<float> T(rx,ry,rz,tx,ty,tz) ;
  MatrixRT<float> Sx ; 
  Sx.scale(sx,sy,sz) ;

  transform(T*Sx) ;
  
  tx = ty = tz = rx = ry = rz = 0 ;
  sx = sy = sz = 1 ;
  resetAll() ;
}

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
       Member: polygon --- draws the polygon joining the control points
  Description: This function calls between a glBegin/glEnd
               the proper functions to represent the polygon joining
	       all the control points of the NURBS curve.
        Input: 
       Output: 
 Restrictions: 
   Created by: Philippe Lavoie (23 September 1997)
  Modified by:
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
void PlNurbsCpolygonGL::glObject() const{

  glBegin(GL_LINE_STRIP);
  glColor() ;
  for(int i=0;i<curve.CtrlP.n;++i)
    glVertex4fv(curve.CtrlP[i].data) ;
  glEnd() ;
}


/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
       Member: polygon --- draws the polygon joining the control points
  Description: This function calls between a glBegin/glEnd
               the proper functions to represent the polygon joining
	       all the control points of the NURBS surface.
        Input: 
       Output: 
 Restrictions: 
   Created by: Philippe Lavoie (23 September 1997)
  Modified by:
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
void PlNurbsSpolygonGL::glObject() const{
  int i,j ;
  for(i=0;i<surface.CtrlP.rows;++i){
    glBegin(GL_LINE_STRIP);
    glColor() ;
    for(j=0;j<surface.CtrlP.cols;++j)
      glVertex4fv(surface.CtrlP(i,j).data) ;
    glEnd() ;
  }
  
  for(j=0;j<surface.CtrlP.cols;++j){
    glBegin(GL_LINE_STRIP);
    glColor() ;
    for(i=0;i<surface.CtrlP.rows;++i)
      glVertex4fv(surface.CtrlP(i,j).data);
    glEnd() ;
  }    
}

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
       Member: resetCPoints --- reset the control point information
  Description: Reset the control points information stored in cpoints.
        Input:  
       Output: 
 Restrictions: 
   Created by: Philippe Lavoie (23 September 1997)
  Modified by:
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
void PlNurbsCurveGL::resetCPoints() {
  PlCPointGL *tempCP ;

  cpoints.reset() ;
  if(P.n>10)
    cpoints.setJumpSize(int(P.n/10.0)) ;
  else
    cpoints.setJumpSize(int(float(P.n)/2.001 + 1)) ;

  for(int i=0;i<P.n;++i){
    if(editControlPoints())
      tempCP = new PlCPointGL(P[i]) ;
    else
      tempCP = new PlSPointCurveGL(i,this) ;
    tempCP->setObjectColor(cpointColorDefault) ;
    tempCP->setCurrentColor(cpointActiveColorDefault) ;
    if(i==0)
      tempCP->setObjectColor(cpoint0ColorDefault) ;
    cpoints.add(tempCP) ;
  }
  
  if(editSurfacePoints()){  
    // we need to setup the start-end data for the surface points
    PlSPointCurveGL *a,*b ;
    a = (PlSPointCurveGL*)cpoints.first ;
    while(a){
      b = a ; 
      int r ;
      r = 0 ;
      while(b->previous() && r<deg+1){
	b = (PlSPointCurveGL*)b->previous() ; 
	++r ; 
      }
      a->setStartEnd(b,deg+r+1) ; 
      a = (PlSPointCurveGL*) a->next() ;
    }
  }

}

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
       Member: resetKnots --- Reset the knots information
  Description: Reset the knot information stored in knots.
        Input:  
       Output: 
 Restrictions: 
   Created by: Philippe Lavoie (23 September 1997)
  Modified by:
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
void PlNurbsCurveGL::resetKnots() {
  PlKnotGL *tempCP ;

  knots.reset() ;
  if(P.n>10)
    knots.setJumpSize(int(P.n/10.0)) ;
  else
    knots.setJumpSize(2) ;

  for(int i=0;i<U.n;++i){
    Point3D p = point3D(U[i]) ;
    // we must represent multiple knots by only 1 point
    // but with a bigger radius (increase radius by 1)
    tempCP = new PlKnotGL(p,i,-1) ;
    tempCP->setObjectColor(knots.objectColor) ;
    tempCP->setCurrentColor(knots.currentColor) ;
    if(i==0)
      tempCP->setObjectColor(cpoint0ColorDefault) ;
    int psize = 4 ;
    int j=i ;
    while(j<U.n-1){
      if(U[j+1]<=U[j]){
	++psize ; ++j ;
      }
      else
	j = U.n ;
    }
    tempCP->setPsize(psize) ;
    knots.add(tempCP) ;
  }
}

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
       Member: resetCPoints --- Reset the control point information
  Description: Reset the control point information stored in cpoints.
        Input:  
       Output: 
 Restrictions: 
   Created by: Philippe Lavoie (23 September 1997)
  Modified by:
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
void PlNurbsSurfaceGL::resetCPoints() {
  PlCPointGL *tempCP ;

  cpoints.reset() ;
  cpoints.setJumpSize(P.cols) ;

  for(int i=0;i<P.rows;++i)
    for(int j=0;j<P.cols;++j){
      if(editControlPoints())
	tempCP = new PlCPointGL(P(i,j)) ;
      else
	tempCP = new PlSPointSurfaceGL(i,j,this,&cpoints) ; 
      tempCP->setObjectColor(cpointColorDefault) ;
      tempCP->setCurrentColor(cpointActiveColorDefault) ;
      if(j==0 && (i!=P.rows-1))
	tempCP->setObjectColor(cpoint0ColorDefault) ;
      cpoints.add(tempCP) ;
    }

  if(editSurfacePoints()){  
    // we need to setup the start-end data for the surface points
    PlSPointSurfaceGL *a,*b ;
    a = (PlSPointSurfaceGL*)cpoints.first ;
    while(a){
      b = a ; 
      int r,c ;
      r = 0 ;
      c = 0 ;
      cpoints.current = (PlObjectGL*)a ;
      while(cpoints.jumpToPrevious() && r<degU){
	if(((PlSPointSurfaceGL*)cpoints.current)->row() < a->row() )
	  b = (PlSPointSurfaceGL*) cpoints.current ;
	else
	  break ;
	++r ;
      }
	
      while(b->previous() && c<degV){
	if(((PlSPointSurfaceGL*)b->previous())->col() < a->col() )
	  b = (PlSPointSurfaceGL*)b->previous() ; 
	else
	  break ;
	++c ; 
      }
      a->setStartEnd(b,degU+r+1,degV+c+1) ; 
      a = (PlSPointSurfaceGL*) a->next() ;
    }
    cpoints.current = (PlObjectGL*)cpoints.first ;
  }
}

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
       Member: resetCPoints --- Reset the control point information
  Description: Reset the control point information stored in cpoints
        Input:  
       Output: 
 Restrictions: 
   Created by: Philippe Lavoie (23 September 1997)
  Modified by:
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
void PlHNurbsSurfaceGL::resetCPoints() {
  PlObjectGL *tempCP ;
  if(activePatch){
    cpoints.reset() ;
    cpoints.setJumpSize(activePatch->CtrlP.cols) ;
    
    for(int i=0;i<activePatch->CtrlP.rows;++i)
      for(int j=0;j<activePatch->CtrlP.cols;++j){
	int mod ;
	mod = 1 ;
	if(i==0 || j==0 
	   || i==activePatch->CtrlP.rows-1 
	   || j==activePatch->CtrlP.cols-1)
	  mod = 0 ;
	mod = 1 ; // if we don't use multi-patches it should be safe
	if(editControlPoints())
	  tempCP = new PlHCPointGL(activePatch->offset(i,j),i,j,activePatch,mod) ;
	else
	  tempCP = new PlSPointHSurfaceGL(i,j,activePatch,&cpoints) ; 
	tempCP->setObjectColor(cpointColorDefault) ;
	tempCP->setCurrentColor(cpointActiveColorDefault) ;
	if(j==0 && (i!=activePatch->CtrlP.rows-1))
	  tempCP->setObjectColor(cpoint0ColorDefault) ;
	cpoints.add(tempCP) ;
      }

    if(editSurfacePoints()){  
      // we need to setup the start-end data for the surface points
      PlSPointSurfaceGL *a,*b ;
      a = (PlSPointSurfaceGL*)cpoints.first ;
      while(a){
	b = a ; 
	int r,c ;
	r = 0 ;
	c = 0 ;
	cpoints.current = (PlObjectGL*)a ;
	while(cpoints.jumpToPrevious() && r<degU){
	  if(((PlSPointSurfaceGL*)cpoints.current)->row() < a->row() )
	    b = (PlSPointSurfaceGL*) cpoints.current ;
	  else
	    break ;
	  ++r ;
	}
	
	while(b->previous() && c<degV){
	  if(((PlSPointSurfaceGL*)b->previous())->col() < a->col() )
	    b = (PlSPointSurfaceGL*)b->previous() ; 
	  else
	    break ;
	  ++c ; 
	}
	a->setStartEnd(b,degU+r+1,degV+c+1) ; 
	a = (PlSPointSurfaceGL*) a->next() ;
      }
      cpoints.current = (PlObjectGL*)cpoints.first ;
    }
  }
}

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
       Member: resetKnots --- reset the knots information
  Description: Reset the knots information stored in knots.
        Input:  
       Output: 
 Restrictions: 
   Created by: Philippe Lavoie (23 September 1997)
  Modified by:
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
void PlNurbsSurfaceGL::resetKnots() {
  PlKnotGL *tempCP ;

  knots.reset() ;
  knots.setJumpSize(P.cols) ;

  PlVector<int> sU,sV ;
  sU.resize(U.n) ;
  sV.resize(V.n) ;

  int i,j ;
  
  sU[0] = 0 ;
  for(i=0;i<U.n-1;++i){
    if(U[i+1]>=U[i]){
      sU[i+1] = sU[i]+1 ;
      sU[i] = 0 ;
    }
    else
      sU[i+1] = 1 ;
  }

  sV[0] = 0 ;
  for(i=0;i<V.n-1;++i){
    if(V[i+1]>=V[i]){
      sV[i+1] = sV[i]+1 ;
      sV[i] = 0 ;
    }
    else
      sV[i+1] = 1 ;
  }

  for(i=0;i<U.n;++i)
    for(j=0;j<V.n;++j){
      //if(sU[i] && sV[i]){
	Point3D p = point3D(U[i],V[j]) ;
	// we must represent multiple knots by only 1 point
	// but with a bigger radius (increase radius by 1)
	tempCP = new PlKnotGL(p,i,j) ;
	tempCP->setObjectColor(knotColorDefault) ;
	tempCP->setCurrentColor(knotActiveColorDefault) ;
	if(j==0 && (i!=P.rows-1))
	  tempCP->setObjectColor(cpoint0ColorDefault) ;
	tempCP->setPsize(3+maximum(sU[i],sV[j])) ;
	knots.add(tempCP) ;
	//}
    }
}

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
       Member: point --- draws a point at the location $C(u)$
  Description: This function calls between a glBegin/glEnd
               the proper functions to represent the point which is
	       at $C(u)$ on the curve.
        Input:     u --> the parametric value
	           v --> a dummy variable so the call is the same as with a
		         NURBS surface.
	       psize --> the size of the control points
	      colorP --> the color of the control points
       Output: 
 Restrictions: 
   Created by: Philippe Lavoie (23 September 1997)
  Modified by:
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
void PlNurbsCurveGL::point(float &u,float &v,int pSize,const Color& colorP, int cp_flag) const {
  if(u<U[0])
    u = U[0] ;
  if(u>U[U.n-1])
    u = U[U.n-1] ;
  glPointSize(pSize) ;
  glBegin(GL_POINTS);
  float color[4] ;
  color[0] = color[1] = color[2] =  0.0 ;
  color[3] = 1.0 ;
  glMaterialfv(GL_FRONT_AND_BACK,GL_DIFFUSE,color) ;
  color[0] = float(colorP.r)/255.0 ;
  color[1] = float(colorP.g)/255.0 ;
  color[2] = float(colorP.b)/255.0 ;
  glMaterialfv(GL_FRONT_AND_BACK,GL_EMISSION,color) ;
  glVertex4fv(point4D(u).data) ;
  glEnd() ;
  v *= 1.0 ; // to shut up the warning messages
  
  if(cp_flag){ // if this is set, only display CP which affect the point
    
  }
}

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
       Member: gluNurbs --- creates a nurbs surface for OpenGL
  Description: This function calls between a gluBeginSurface/gluEndSurface
               the proper functions to generate a NURBS surface.
        Input: 	       
       Output: 
 Restrictions: 
   Created by: Philippe Lavoie (23 September 1997)
  Modified by:
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
void PlNurbsSurfaceGL::gluNurbs() const{
  int i ;
  if(nurbsRenderer){
    switch(PlNurbsDisplayMode){
    case PLNURBS_DISPLAY_ISOCURVES:
      {
      // create iso curves
      PlNurbsCurveGL t ;
      t.setObjectColor(objectColor,selectColor,currentColor) ;
      t.setNurbsRenderer(nurbsRenderer) ;
      t.hideBBox() ;
      t.hideCPoints() ;
      t.hideCpolygon() ;
      t.viewNurbs() ;
      if(isActive()) t.activate() ;
      if(isSelected()) t.select() ;

      int i ;
      for(i=0;i<nUlines;++i){
	float u = U[0] + float(i)/float(nUlines-1) * (U[U.n-1]-U[0]) ; 
	isoCurveU(u,t) ;
	t.glObject() ;
      }

      for(i=0;i<nVlines;++i){
	float v = V[0] + float(i)/float(nVlines-1) * (V[V.n-1]-V[0]) ; 
	isoCurveV(v,t) ;
	t.glObject() ;
      }
    }
      break ;
    case PLNURBS_DISPLAY_SHADED:
      {
	GLenum t = GL_MAP2_VERTEX_4 ; 
	if(image)
	  glShadeModel(GL_FLAT) ;
	else
	  glShadeModel(GL_SMOOTH) ;
	gluNurbsProperty(nurbsRenderer,GLU_DISPLAY_MODE,GLU_FILL) ;

	
    //GLfloat lmodel_ambient[] = { 0.3, 0.3, 0.3, 1.0 };

    //glLightModelfv(GL_LIGHT_MODEL_AMBIENT, lmodel_ambient);

    //glEnable(GL_LIGHTING);


    //glDepthFunc(GL_LESS);
    //glEnable(GL_DEPTH_TEST);
    //glEnable(GL_AUTO_NORMAL);
    
    //colorToMaterial(Color(255,100,255),materialColor) ;
    //glMaterialfv(GL_FRONT_AND_BACK,GL_DIFFUSE,materialColor) ;
	//colorToMaterial(Color(0,0,0),materialColor) ;
	//glMaterialfv(GL_FRONT_AND_BACK,GL_EMISSION,materialColor) ;
    //glColor() ;


    gluBeginSurface(nurbsRenderer) ;
    if(image){
      static GLfloat texpts[2][2][2] = {{{0,0},{0,1}},{{1,0},{1,1}}} ;
      
      glMap2f(GL_MAP2_TEXTURE_COORD_2,0,1,2,2,0,1,4,2,&texpts[0][0][0]) ;
      glEnable(GL_MAP2_TEXTURE_COORD_2) ;
      glMapGrid2f(20,0,1,20,0,1) ;
      glEvalMesh2(GL_FILL,0,20,0,20) ;
      glTexEnvf(GL_TEXTURE_ENV,GL_TEXTURE_ENV_MODE, GL_DECAL) ;
      glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_REPEAT) ;
      glTexParameterf(GL_TEXTURE_2D,GL_TEXTURE_WRAP_T,GL_REPEAT) ;
      glTexParameterf(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER,GL_NEAREST) ;
      glTexParameterf(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,GL_NEAREST) ;
      glTexImage2D(GL_TEXTURE_2D,0,3,imgW,imgH,0,GL_RGB,GL_UNSIGNED_BYTE,image) ;
      glShadeModel(GL_FLAT) ;
      glEnable(GL_TEXTURE_2D) ;
    }
    GLfloat mat_ambient[] = { 0.8, 0.8, 0.8, 1.0 };
    GLfloat mat_diffuse[] = { 0.8, 0.8, 0.8, 1.0 };
    GLfloat mat_specular[] = { 0.8, 0.8, 0.8, 1.0 };
    GLfloat mat_shininess[] = { 50.0 };
    glMaterialfv(GL_FRONT_AND_BACK, GL_AMBIENT, mat_ambient);
    glMaterialfv(GL_FRONT_AND_BACK, GL_DIFFUSE, mat_diffuse);
    glMaterialfv(GL_FRONT_AND_BACK, GL_SPECULAR, mat_specular);
    glMaterialfv(GL_FRONT_AND_BACK, GL_SHININESS, mat_shininess);

    gluNurbsSurface(nurbsRenderer, U.n, U.memory(), V.n, V.memory(),4,4*P.rows, P[0]->data, degU+1, degV+1, t) ;
    gluEndSurface(nurbsRenderer) ;
    glDisable(GL_TEXTURE_2D) ;
      }
      break ;
    case PLNURBS_DISPLAY_HIDDEN_LINES:
      {
	GLenum t = GL_MAP2_VERTEX_4 ; 
	gluNurbsProperty(nurbsRenderer,GLU_DISPLAY_MODE,GLU_FILL) ;

	glDisable(GL_LIGHTING);
	glColor3f(.7f, .7f, .7f);
	
	glEnable(GL_STENCIL_TEST);
	glStencilFunc(GL_ALWAYS, 0, 0); /* clear stencil for this object */

	gluBeginSurface(nurbsRenderer) ;
	//glColor() ;
	gluNurbsSurface(nurbsRenderer, U.n, U.memory(), V.n, V.memory(),4,4*P.rows, P[0]->data, degU+1, degV+1, t) ;
	gluEndSurface(nurbsRenderer) ;

	glEnable(GL_LIGHTING);

	glColorMask(GL_FALSE, GL_FALSE, GL_FALSE, GL_FALSE); /* turn off color */
	glDisable(GL_DEPTH_TEST); /* turn off depth */
	glPolygonMode(GL_FRONT_AND_BACK, GL_LINE);
	glStencilFunc(GL_ALWAYS, 1, 1);
	glStencilOp(GL_KEEP, GL_KEEP, GL_REPLACE);

	gluBeginSurface(nurbsRenderer) ;
	//glColor() ;
	gluNurbsSurface(nurbsRenderer, U.n, U.memory(), V.n, V.memory(),4,4*P.rows, P[0]->data, degU+1, degV+1, t) ;
	gluEndSurface(nurbsRenderer) ;

	glStencilFunc(GL_EQUAL, 1 , 1);
	glEnable(GL_DEPTH_TEST);
	glDepthFunc(GL_LEQUAL);
	glPolygonMode(GL_FRONT_AND_BACK, GL_FILL);
	glColorMask(GL_TRUE, GL_TRUE, GL_TRUE, GL_TRUE);

	gluBeginSurface(nurbsRenderer) ;
	//glColor() ;
	gluNurbsSurface(nurbsRenderer, U.n, U.memory(), V.n, V.memory(),4,4*P.rows, P[0]->data, degU+1, degV+1, t) ;
	gluEndSurface(nurbsRenderer) ;


	/* clean up state */
	glDisable(GL_STENCIL_TEST);
	glDepthFunc(GL_LESS);
      }
      break ;
    case PLNURBS_DISPLAY_TESSELATION:
      {
	BasicList<Point3D> pts ;
	BasicList<int> connect ;
	if(tessel_tolerance<0.001) 
	  tessel_tolerance = 0.001 ;
	tesselate(tessel_tolerance,pts,connect) ;

	glPolygonMode(GL_FRONT_AND_BACK,GL_LINE) ;
	glBegin(GL_TRIANGLES) ;
	glColor() ;
	connect.goToFirst() ;
	pts.goToFirst() ;
	for(i=0;i<connect.size();++i){
	  if(*(connect[i]->data) >= 0)
	    glVertex3fv(pts[*(connect[i]->data)]->data->data) ;
	}
	glEnd() ;
      }
      break ;
    case PLNURBS_DISPLAY_NORMAL:
    default:
      {
      GLenum t = GL_MAP2_VERTEX_4 ; 
      gluNurbsProperty(nurbsRenderer,GLU_DISPLAY_MODE,GLU_OUTLINE_POLYGON) ;
      gluBeginSurface(nurbsRenderer) ;
      glColor() ;
      gluNurbsSurface(nurbsRenderer, U.n, U.memory(), V.n, V.memory(),4,4*P.rows, P[0]->data, degU+1, degV+1, t) ;
      gluEndSurface(nurbsRenderer) ;
      }      
    }
  }
}

/*
void splitAtActiveInU(PlNurbsCurveGL &t, PlNurbsCurveGL &ta, float u, PlHNurbsSurface *active, int lod){
  if(lod >= 0 && lod < active->level)
    return ;
  if(u<active->uS || u >active->uE){
    t.glObject() ;
    return ;
  }
  PlNurbsCurve c ;
  c = t ;
  ta = c ;
  if(active->vS<=c.Knot[0] && active->vE>=c.Knot[c.Knot.n-1]){
    ta.glObject() ;
    return ;
  }
  if(active->vS>c.Knot[0] && active->vE < c.Knot[c.Knot.n-1]){
    // active curve is in the middle
    c.splitAt(active->vS,t,ta) ;
    t.glObject() ;
    c = ta ;
    c.splitAt(active->vE,ta,t) ;
    t.glObject() ;
    ta.glObject() ;
  }
  else{
    if(active->vS>c.Knot[0]) {
      c.splitAt(active->vS,t,ta) ;
      t.glObject() ;
      ta.glObject() ;
    }
    else{
      c.splitAt(active->vE,ta,t) ;
      t.glObject() ;
      ta.glObject() ;
    }
  }
}

void splitAtActiveInV(PlNurbsCurveGL &t, PlNurbsCurveGL &ta, float v, PlHNurbsSurface *active, int lod){
  if(lod >= 0 && lod < active->level)
    return ;
  if(v<active->vS || v >active->vE){
    t.glObject() ;
    return ;
  }
  PlNurbsCurve c ;
  c = t ;
  ta = c ;
  if(active->uS<=c.Knot[0] && active->uE>=c.Knot[c.Knot.n-1]){
    ta.glObject() ;
    return ;
  }
  if(active->uS>c.Knot[0] && active->uE < c.Knot[c.Knot.n-1]){
    // active curve is in the middle
    c.splitAt(active->uS,t,ta) ;
    t.glObject() ;
    c = ta ;
    c.splitAt(active->uE,ta,t) ;
    t.glObject() ;
    ta.glObject() ;
  }
  else{
    if(active->uS>c.Knot[0]) {
      c.splitAt(active->uS,t,ta) ;
      t.glObject() ;
      ta.glObject() ;
    }
    else{
      c.splitAt(active->uE,ta,t) ;
      t.glObject() ;
      ta.glObject() ;
    }
  }
}
*/

void isoCurves(const PlHNurbsSurfaceSP *hs, PlNurbsCurveGL &t, int lod, int nu, int nv){
  if(lod<0)
    lod = hs->maxLevel() ; 

  if(lod>=0 && lod<hs->level){
    return ;
  }

  if(lod == hs->level){
    int i ;
    for(i=0;i<nu+hs->level;++i){
      float u = hs->KnotU[0] + float(i)/float(nu+hs->level-1) * (hs->KnotU[hs->KnotU.n-1] - hs->KnotU[0]) ; 
      hs->isoCurveU(u,t,lod) ; 
      t.glObject() ; 
    }
    for(i=0;i<nv+hs->level;++i){
      float v = hs->KnotV[0] + float(i)/float(nv+hs->level-1) * (hs->KnotV[hs->KnotV.n-1] - hs->KnotV[0]) ; 
      hs->isoCurveV(v,t,lod) ; 
      t.glObject() ; 
    }
    return ; 
  }
 
  PlHNurbsSurfaceSP *child ; 
  child = (PlHNurbsSurfaceSP*) hs->nextLevel() ;
  if(child){
    isoCurves(child,t,lod,nu,nv) ;
  }
  return ; 
}

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
       Member: gluNurbs --- creates a HNURBS surface for OpenGL
  Description: This draws a HNURBS. Presently only isocurves are drawn
               to represent the surface.
        Input: 	       
       Output: 
 Restrictions: 
   Created by: Philippe Lavoie (23 September 1997)
  Modified by:
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
void PlHNurbsSurfaceGL::gluNurbs() const{
  if(nurbsRenderer){

    PlHNurbsSurfaceGL* dispPatch ;
    dispPatch = activePatch ;
    int level = levelOfDetail() ;
    if(level<0) 
      level = maxLevel() ;
    while(dispPatch->level < level)
      dispPatch = (PlHNurbsSurfaceGL*)dispPatch->nextLevel() ;
    
    switch(PlNurbsDisplayMode){
    case PLNURBS_DISPLAY_ISOCURVES:
      {
      // create iso curves
      PlNurbsCurveGL t ;
      t.setObjectColor(objectColor,selectColor,currentColor) ;
      t.setNurbsRenderer(nurbsRenderer) ;
      t.hideBBox() ;
      t.hideCPoints() ;
      t.hideCpolygon() ;
      t.viewNurbs() ;
      if(isActive()) t.activate() ;
      if(isSelected()) t.select() ;

      isoCurves(dispPatch,t,level,nUlines,nVlines) ; 

      if(dispPatch->level != activePatch->level){
	t.setObjectColor(Color(255,255,0),Color(255,255,0),Color(255,255,0));
	isoCurves(activePatch,t,activePatch->level,nUlines,nVlines) ; 
      }

    }
      break ;
    case PLNURBS_DISPLAY_SHADED:
    case PLNURBS_DISPLAY_HIDDEN_LINES:
    case PLNURBS_DISPLAY_TESSELATION:
    case PLNURBS_DISPLAY_NORMAL:
    default:
      {
      GLenum t = GL_MAP2_VERTEX_4 ; 
      gluNurbsProperty(nurbsRenderer,GLU_DISPLAY_MODE,GLU_OUTLINE_POLYGON) ;
      gluBeginSurface(nurbsRenderer) ;
      glColor() ;
      gluNurbsSurface(nurbsRenderer, dispPatch->U.n, dispPatch->U.memory(), dispPatch->V.n, dispPatch->V.memory(),4,4*dispPatch->P.rows, dispPatch->P[0]->data, dispPatch->degU+1, dispPatch->degV+1, t) ;
      gluEndSurface(nurbsRenderer) ;
      }      
    }
  }
}

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
       Member: point --- draws a point at the location $C(u)$
  Description: This function calls between a glBegin/glEnd
               the proper functions to represent the point which is
	       at $S(u,v)$ on the surface.
        Input:     u --> the $U$ parametric value
	           v --> the $V$ parametric value
	       psize --> the size of the control points
	      colorP --> the color of the control points
       Output: 
 Restrictions: 
   Created by: Philippe Lavoie (23 September 1997)
  Modified by:
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
void PlNurbsSurfaceGL::point(float &u, float &v, int pSize,const Color& colorP, int cp_flag) const {
  if(u<U[0])
    u = U[0] ;
  if(u>U[U.n-1])
    u = U[U.n-1] ;
  if(v<V[0])
    v = V[0] ;
  if(v>V[V.n-1])
    v = V[V.n-1] ;
  glPointSize(pSize) ;
  glBegin(GL_POINTS);
  float color[4] ;
  color[0] = color[1] = color[2] =  0.0 ;
  color[3] = 1.0 ;
  glMaterialfv(GL_FRONT_AND_BACK,GL_DIFFUSE,color) ;
  color[0] = float(colorP.r)/255.0 ;
  color[1] = float(colorP.g)/255.0 ;
  color[2] = float(colorP.b)/255.0 ;
  glMaterialfv(GL_FRONT_AND_BACK,GL_EMISSION,color) ;
  glVertex4fv(point4D(u,v).data) ;
  glEnd() ;

  PlNurbsCurveGL t ;
  t.setObjectColor(colorP,colorP,colorP) ;
  t.setNurbsRenderer(nurbsRenderer) ;
  t.hideBBox() ;
  t.hideCPoints() ;
  t.hideCpolygon() ;
  t.viewNurbs() ;
  isoCurveU(u,t) ;
  t.glObject() ;
  isoCurveV(v,t) ;
  t.glObject() ;
}


/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
       Member: point --- draws a point at the location $C(u)$
  Description: This function calls between a glBegin/glEnd
               the proper functions to represent the point which is
	       at $S(u,v)$ on the hierarchical surface.
        Input:     u --> the $U$ parametric value
                   v --> the $V$ parametric value
	       psize --> the size of the control points
	      colorP --> the color of the control points
       Output: 
 Restrictions: 
   Created by: Philippe Lavoie (23 September 1997)
  Modified by:
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
void PlHNurbsSurfaceGL::point(float &u, float &v, int pSize,const Color& colorP, int cp_flag) const {
  if(u<U[0])
    u = U[0] ;
  if(u>U[U.n-1])
    u = U[U.n-1] ;
  if(v<V[0])
    v = V[0] ;
  if(v>V[V.n-1])
    v = V[V.n-1] ;
  glPointSize(pSize) ;
  glBegin(GL_POINTS);
  float color[4] ;
  color[0] = color[1] = color[2] =  0.0 ;
  color[3] = 1.0 ;
  glMaterialfv(GL_FRONT_AND_BACK,GL_DIFFUSE,color) ;
  color[0] = float(colorP.r)/255.0 ;
  color[1] = float(colorP.g)/255.0 ;
  color[2] = float(colorP.b)/255.0 ;
  glMaterialfv(GL_FRONT_AND_BACK,GL_EMISSION,color) ;
  glVertex4fv(pointAt(u,v,levelOfDetail()).data) ;
  glEnd() ;


  PlNurbsCurveGL t ;
  t.setObjectColor(colorP,colorP,colorP) ;
  t.setNurbsRenderer(nurbsRenderer) ;
  t.hideBBox() ;
  t.hideCPoints() ;
  t.hideCpolygon() ;
  t.viewNurbs() ;
  isoCurveU(u,t,levelOfDetail()) ;
  t.glObject() ;
  isoCurveV(v,t,levelOfDetail()) ;
  t.glObject() ;  

  if(cp_flag){ // display only the CP affected by the point
    int spanU, spanV ;
    activePatch->findSpan(u,v,spanU,spanV) ;
    int i,j,n ;
    
    PlObjectGL *pnt ;
    pnt = (PlObjectGL*)cpoints.first ;
    n = 0 ;
    while(pnt){
      i = n / activePatch->CtrlP.cols ;
      j = n % activePatch->CtrlP.cols ;
      if(i<spanU-activePatch->degreeU || i>spanU)
	pnt->hideObject() ;
      else{
	if(j<spanV-activePatch->degreeV || j>spanV)
	  pnt->hideObject() ;
	else
	  pnt->viewObject() ;
      }
      ++n ;
      pnt = pnt->next() ;      
    }
  }
}


/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
       Member: PlMaterial --- default constructor
  Description: This sets the values of all the variables to their default
               values (according to their manual).
        Input: 
       Output: 
 Restrictions: 
   Created by: Philippe Lavoie (13 October 1997)
  Modified by:
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
PlMaterial::PlMaterial(){
  // POV-Ray options
  // finish options
  //double ambient = 0.2 ;
  //double diffuse = 0.6 ;
  //double roughness = 0 ;
}

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
       Member: read --- reads a list of objects
  Description: Reads a list of objects.
        Input: filename --> the name of the file to read from 
       Output: 1 on success, 0 otherwise
 Restrictions: 
   Created by: Philippe Lavoie (13 October 1997)
  Modified by:
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
int PlObjectListGL::read(const char* filename){
  ifstream fin(filename) ;
  if(!fin)
    return 0 ;
  fin.clear() ;
  fin.seekg(0,ios::beg) ;

  char *type ;
  type = new char[3] ;

  PlObjectGL* newObject ;

  while(!fin.eof()){
    // get the first 3 characters to determine the type of the object
    int mark = fin.tellg() ; // mark position
    if(!fin.read(type,sizeof(char)*3)){ delete []type ;  return 1 ;  }
    fin.seekg(mark) ; // restore position

    int ot=0 ; // unknown object
    char* ext ;

    ext = strstr(type,"nc3");
    if(ext) ot = 1 ;
    if(!ot){
      ext = strstr(type,"nc4");
      if(ext) ot = 1 ;
    }
    if(!ot){
      ext = strstr(type,"ns3");
      if(ext) ot = 2 ;
    }
    if(!ot){
      ext = strstr(type,"ns4");
      if(ext) ot = 2 ;
    }
    if(!ot){
      ext = strstr(type,"hns");
      if(ext) ot = 3 ;
    }
    if(!ot){
      ext = strstr(type,"pt3");
      if(ext) ot = 4 ;
    }
    
    
    if(!ot){  delete []type ;  return 1 ;     }
    
    newObject = 0 ;
    switch(ot){
    case 0: break ;
    case 1: // A NURBS curve
      newObject = new PlNurbsCurveGL ;
      if(!newObject->read(fin)) {delete []type ; return 0 ;}
      break ;
    case 2: // A NURBS surface
      newObject = new PlNurbsSurfaceGL ;
      if(!newObject->read(fin)) {delete []type ; return 0 ;}
      break ;
    case 3: // A HNURBS surface
      newObject = new PlHNurbsSurfaceGL ;
      if(!newObject->read(fin)) { delete []type ; return 0 ; }
      ((PlHNurbsSurfaceGL*)newObject)->updateLevels() ;
      ((PlHNurbsSurfaceGL*)newObject)->highestLevelOfDetail() ;      
      break ;
    case 4: // a list of points in 3D
      newObject = new PlPointListGL ;
      if(!newObject->read(fin)) { delete []type ; cerr << "B" ; return 0 ;}
      break ;
    }
    
    if(newObject)
      add(newObject) ;

    //if(fin.fail()){ delete []type ; return 0 ;  } 
  }
  delete []type ;
  return 1 ;
}

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
       Member: write --- writess a list of objects
  Description: Writes a list of objects.
        Input: filename --> the name of the file to read from 
       Output: 1 on success, 0 otherwise
 Restrictions: 
   Created by: Philippe Lavoie (13 October 1997)
  Modified by:
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
int PlObjectListGL::write(const char* filename) const {
  ofstream fout(filename) ;
  if(!fout)
    return 0 ;
  
  PlObjectGL *t ;
  t = (PlObjectGL*)first_ ;
  while(t){
    if(!t->write(fout))
      return 0 ;
    t = t->next() ;
  }
  return fout.good() ;
}

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
       Member: writeRIB --- write a list of object in the RIB format 
  Description: Write a list of object in the RIB format 
        Input: filename --> the name of the file to read from 
       Output: 1 on success, 0 otherwise
 Restrictions: 
   Created by: Philippe Lavoie (13 October 1997)
  Modified by:
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
int PlObjectListGL::writeRIB(const char* filename) const {
  ofstream fout(filename) ;
  if(!fout)
    return 0 ;
  
  PlObjectGL *t ;
  t = (PlObjectGL*)first_ ;
  while(t){
    if(!t->writeRIB(fout))
      return 0;
    t = t->next() ;
  }
  return fout.good() ;
}

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
       Member: writePOVRAY --- write a list of object in the POVRAY format 
  Description: Write a list of object in the POVRAY format
        Input: filename --> the name of the file to read from 
       Output: 1 on success, 0 otherwise
 Restrictions: 
   Created by: Philippe Lavoie (13 October 1997)
  Modified by:
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
int PlObjectListGL::writePOVRAY(const char* filename) const {
  ofstream fout(filename) ;
  if(!fout)
    return 0 ;
  
  PlObjectGL *t ;
  t = (PlObjectGL*)first_ ;
  while(t){
    if(!t->writePOVRAY(fout))
      return 0;
    t = t->next() ;
  }
  return fout.good() ;
}

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
       Member: selectNextPatch --- activates the next patch at the same level
  Description: Activates the next patch at the same level
        Input: 
       Output: 
 Restrictions: 
   Created by: Philippe Lavoie (3 November 1997)
  Modified by:
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
/*
void PlHNurbsSurfaceGL::selectNextPatch(){
  if(activePatch){
    if(activePatch->nextLevel_)
      activePatch = (PlHNurbsSurfaceGL*)activePatch->nextPatch ;
    else
      if(activePatch->base)
	activePatch = (PlHNurbsSurfaceGL*)activePatch->base->firstPatch ;
  }
}
*/
/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
       Member: selectPrevPatch --- activates the previous patch at the same 
                                   level
  Description: Activates the previous patch at the same level
        Input: 
       Output: 
 Restrictions: 
   Created by: Philippe Lavoie (3 November 1997)
  Modified by:
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
/*void PlHNurbsSurfaceGL::selectPrevPatch(){
  if(activePatch){
    if(activePatch->prevPatch)
      activePatch = (PlHNurbsSurfaceGL*)activePatch->prevPatch ;
    else
      if(activePatch->baseLevel()){
	activePatch = (PlHNurbsSurfaceGL*)activePatch->base->lastPatch ;
    }
  }
}
*/
/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
       Member: selectHigherLevel --- activates the patch at a higher level
  Description: Activates the patch at a higher level.
        Input: 
       Output: 
 Restrictions: 
   Created by: Philippe Lavoie (3 November 1997)
  Modified by:
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
void PlHNurbsSurfaceGL::selectHigherLevel() {
  if(activePatch){
    if(activePatch->level < levelOfDetail())
      if(activePatch->nextLevel())
	activePatch = (PlHNurbsSurfaceGL*)activePatch->nextLevel() ;
  }
}

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
       Member: selectHighestLevel --- activates the highest level
  Description: Activates the patch at a higher level.
        Input: 
       Output: 
 Restrictions: 
   Created by: Philippe Lavoie (28 January 1998)
  Modified by:
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
void PlHNurbsSurfaceGL::selectHighestLevel() {
  if(activePatch){
    while(activePatch->level < levelOfDetail())
      if(activePatch->nextLevel())
	activePatch = (PlHNurbsSurfaceGL*)activePatch->nextLevel() ;
  }
}

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
       Member: selectHigherLevel --- activates the patch at a higher level
  Description: Activates the patch at a higher level.
        Input: 
       Output: 
 Restrictions: 
   Created by: Philippe Lavoie (3 November 1997)
  Modified by:
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
void PlHNurbsSurfaceGL::selectLowerLevel() {
  if(activePatch){
    if(activePatch->baseLevel())
      activePatch = (PlHNurbsSurfaceGL*)activePatch->baseLevel() ;
  }
}

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
       Member: selectHigherLevel --- activates the patch at a higher level
  Description: Activates the patch at a higher level.
        Input: 
       Output: 
 Restrictions: 
   Created by: Philippe Lavoie (3 November 1997)
  Modified by:
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
void PlHNurbsSurfaceGL::decreaseLevelOfDetail() {
  --lod ; 
  if(lod<0) lod=0 ; 
  if(activePatch)
    while(activePatch->level > lod)
      activePatch = (PlHNurbsSurfaceGL*)activePatch->baseLevel() ;
}

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
       Member: operator= --- copies another ObjectGL
  Description: Copies another ObjectGL. This will not copy the prev and
               next pointers.
        Input: a <-- the object to copy
       Output: 
 Restrictions: 
   Created by: Philippe Lavoie (6 November 1997)
  Modified by:
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
PlObjectGL& PlObjectGL::operator=(const PlObjectGL &a){
  objectColor = a.objectColor ;
  selectColor = a.selectColor ;
  currentColor = a.currentColor ;
  type = a.type ;
  tx = a.tx ;
  ty = a.ty ;
  tz = a.tz ;
  rx = a.rx ;
  ry = a.ry ;
  rz = a.rz ;
  sx = a.sx ;
  sy = a.sy ;
  sz = a.sz ;
  selected = a.selected ;
  active = a.active ;
  state = a.state ;
  
  return *this ;
}

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
       Member: operator= --- copies another Nurbs object GL
  Description: Copies another Nurbs Curve GL
        Input: a <-- the Nurbs curve to copy
       Output: 
 Restrictions: 
   Created by: Philippe Lavoie (6 November 1997)
  Modified by:
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
PlNurbsGL& PlNurbsGL::operator=(const PlNurbsGL &a){
  this->PlObjectGL::operator=(a) ;
  
  nurbsRenderer = a.nurbsRenderer ;
  cpoints.objectColor = a.cpoints.objectColor ;
  cpoints.currentColor = a.cpoints.currentColor ;
  knots.objectColor = a.knots.objectColor ;
  knots.currentColor = a.knots.currentColor ;
  
  editSP = a.editSP ;

  return *this ;  
}

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
       Member: operator= --- copies another Nurbs Curve GL
  Description: Copies another Nurbs Curve GL
        Input: a <-- the Nurbs curve to copy
       Output: 
 Restrictions: 
   Created by: Philippe Lavoie (6 November 1997)
  Modified by:
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
PlNurbsCurveGL& PlNurbsCurveGL::operator=(const PlNurbsCurveGL &a){
  this->PlNurbsCurveSP::operator=(a) ;
  this->PlNurbsGL::operator=(a) ;
  resetAll() ;
  return *this ;  
}

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
       Member: operator= --- copies another Nurbs Curve
  Description: Copies another Nurbs Curve GL
        Input: a <-- the Nurbs curve to copy
       Output: 
 Restrictions: 
   Created by: Philippe Lavoie (6 November 1997)
  Modified by:
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
PlNurbsCurveGL& PlNurbsCurveGL::operator=(const PlNurbsCurve &a){
  this->PlNurbsCurveSP::operator=(a) ;
  return *this ;
}

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
       Member: operator= --- copies another Nurbs Curve GL
  Description: Copies another Nurbs Curve GL
        Input: a <-- the Nurbs curve to copy
       Output: 
 Restrictions: 
   Created by: Philippe Lavoie (6 November 1997)
  Modified by:
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
PlNurbsSurfaceGL& PlNurbsSurfaceGL::operator=(const PlNurbsSurfaceGL &a){
  this->PlNurbsSurfaceSP::operator=(a) ;
  this->PlNurbsGL::operator=(a) ;
  resetAll() ;
  return *this ;  
}

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
       Member: operator= --- copies another Nurbs Curve
  Description: Copies another Nurbs Curve GL
        Input: a <-- the Nurbs curve to copy
       Output: 
 Restrictions: 
   Created by: Philippe Lavoie (6 November 1997)
  Modified by:
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
PlNurbsSurfaceGL& PlNurbsSurfaceGL::operator=(const PlNurbsSurface &a){
  this->PlNurbsSurfaceSP::operator=(a) ;
  return *this ;
}

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
       Member: modifyPoint --- modifies a point on the curve
  Description: Modifies a point on the curve
        Input:  u --> the u parametric value
	        v --> the v parametric value
	       dx --> the delta value in the $x$-axis direction
	       dy --> the delta value in the $y$-axis direction
	       dz --> the delta value in the $z$-axis direction
       Output: 
 Restrictions: 
   Created by: Philippe Lavoie (7 November 1997)
  Modified by:
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
void PlNurbsCurveGL::modifyPoint(float u, float v, float dx, float dy, float dz){
  Point3D delta(dx,dy,dz) ;
  if(u<Knot[0]) u = Knot[0] ;
  if(u>Knot[Knot.n-1]) u = Knot[Knot.n-1] ;
  movePoint(u,delta) ;
}

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
       Member: modifyPoint --- modifies a point on the surface
  Description: Modifies a point on the surface
        Input:  u --> the u parametric value
	        v --> the v parametric value
	       dx --> the delta value in the $x$-axis direction
	       dy --> the delta value in the $y$-axis direction
	       dz --> the delta value in the $z$-axis direction
       Output: 
 Restrictions: 
   Created by: Philippe Lavoie (7 November 1997)
  Modified by:
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
void PlNurbsSurfaceGL::modifyPoint(float u, float v, float dx, float dy, float dz){
  Point3D delta(dx,dy,dz) ;
  boundTo(u,KnotU[0],KnotU[KnotU.n-1]) ;
  boundTo(v,KnotV[0],KnotV[KnotV.n-1]) ; 
  movePoint(u,v,delta) ;
}

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
       Member: modifyPoint --- modifies a point on the surface
  Description: Modifies a point on the surface
        Input:  u --> the u parametric value
	        v --> the v parametric value
	       dx --> the delta value in the $x$-axis direction
	       dy --> the delta value in the $y$-axis direction
	       dz --> the delta value in the $z$-axis direction
       Output: 
 Restrictions: 
   Created by: Philippe Lavoie (7 November 1997)
  Modified by:
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
void PlHNurbsSurfaceGL::modifyPoint(float u, float v, float dx, float dy, float dz){
  Point3D delta(dx,dy,dz) ;
  boundTo(u,activePatch->KnotU[0],activePatch->KnotU[activePatch->KnotU.n-1]) ;
  boundTo(v,activePatch->KnotV[0],activePatch->KnotV[activePatch->KnotV.n-1]) ;
  activePatch->movePointOffset(u,v,delta) ;
  resetCPoints() ; 
}

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
       Member: applyTransform --- apply the local transformation to the 
                                  surface.
  Description: Apply the local transformation to the surface. This is necessary
               if you want to get the proper position for the control points
	       before doing anymore processing on them.
        Input: 
       Output: 
 Restrictions: 
   Created by: Philippe Lavoie (7 November 1997)
  Modified by:
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
void PlHNurbsSurfaceGL::applyTransform(){
  rx *= M_PI/180.0 ;
  ry *= M_PI/180.0 ;
  rz *= M_PI/180.0 ;

  MatrixRT<float> T(rx,ry,rz,tx,ty,tz) ;
  MatrixRT<float> Sx ; 
  Sx.scale(sx,sy,sz) ;

  transform(T*Sx) ;
  
  tx = ty = tz = rx = ry = rz = 0 ;
  sx = sy = sz = 1 ;

  offset = P ; 
  ++updateN ;
  updateLevels(lod) ; 

  resetAll() ;
}

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
       Member: PlHNurbsSurfaceGL --- default constructor
  Description: 
        Input: 
       Output: 
 Restrictions: 
   Created by: Philippe Lavoie (7 November 1997)
  Modified by:
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
PlHNurbsSurfaceGL::PlHNurbsSurfaceGL():PlHNurbsSurfaceSP(),PlNurbsGL() {
  type = hSurfObject ; 
  polygon = new PlNurbsSpolygonGL(*this) ; 
  lod = maxLevel() ; 
  activePatch = this ; 
}

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
       Member: PlHNurbsSurfaceGL --- constructor from a surface
  Description: 
        Input: nS --> a Nurbs Surface
       Output: 
 Restrictions: 
   Created by: Philippe Lavoie (7 November 1997)
  Modified by:
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
PlHNurbsSurfaceGL::PlHNurbsSurfaceGL(const PlNurbsSurface& nS):PlHNurbsSurfaceSP(nS),PlNurbsGL() { 
  type = hSurfObject ; 
  polygon = new PlNurbsSpolygonGL(*this) ; 
  lod = maxLevel() ; 
  activePatch = this ; 
}

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
       Member: PlHNurbsSurfaceGL --- copy constructor with patch information
  Description: 
        Input: bS --> the object to copy
	       us --> the start of the U parametric patch
	       ue --> the end of the U parametric patch
	       vs --> the start of the V parametric patch
	       ue --> the end of the V parametric patch
       Output: 
 Restrictions: 
   Created by: Philippe Lavoie (7 November 1997)
  Modified by:
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
PlHNurbsSurfaceGL::PlHNurbsSurfaceGL(const PlHNurbsSurfaceGL& bS):PlHNurbsSurfaceSP(bS),PlNurbsGL() { 
  type = hSurfObject ;
  polygon = new PlNurbsSpolygonGL(*this) ; 
  lod = maxLevel() ; 
  activePatch = this ; 
}

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
       Member: PlHNurbsSurfaceGL --- copy constructor
  Description: 
        Input: bS --> a pointer to the object to copy
       Output: 
 Restrictions: 
   Created by: Philippe Lavoie (7 November 1997)
  Modified by:
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
PlHNurbsSurfaceGL::PlHNurbsSurfaceGL(const PlHNurbsSurfaceGL* bS): PlHNurbsSurfaceSP(*bS),PlNurbsGL() {  
  type = hSurfObject ; 
  polygon = new PlNurbsSpolygonGL(*this) ; 
  lod = maxLevel() ; 
  activePatch = this ; 
}

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
       Member: addLevel --- adds a level to the Hierarchical surface.
  Description: Adds a level to the Hierarchical surface.
        Input: 
       Output: 
 Restrictions: 
   Created by: Philippe Lavoie (28 January 1998)
  Modified by:
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
PlHNurbsSurfaceSP* PlHNurbsSurfaceGL::addLevel() {  
  // go to the last level
  if(activePatch){
    activePatch = (PlHNurbsSurfaceGL*)activePatch->lastLevel() ;
    return activePatch->PlHNurbsSurfaceSP::addLevel(1) ;
  }
  return 0 ;
}

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
       Member: resetCPoints --- Reset the control point information
  Description: Reset the control point information stored in cpoints
        Input:  
       Output: 
 Restrictions: 
   Created by: Philippe Lavoie (29 January 1998)
  Modified by:
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
void PlHNurbsSurfaceGL::resetPolygon() {
  int h ;
  h = 0 ; 
  if(polygon){
    h = polygon->getState() ;
    delete polygon ;
  }
  polygon = new PlNurbsSpolygonGL(*activePatch) ;
  if(h)
    viewCpolygon() ;
  else
    hideCpolygon() ;
}

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
       Member: viewAllObjects --- sets all object inside the list to view mode
  Description: Sets all object inside the list to view mode
        Input:  
       Output: 
 Restrictions: 
   Created by: Philippe Lavoie (29 January 1998)
  Modified by:
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
void PlObjectListGL::viewAllObjects(){
  state = objectState ;
  PlObjectGL* obj = first_ ;
  while(obj){
    obj->viewObject() ;
    obj = obj->next() ;
  }
}

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
       Member: hideAllObjects --- sets all object inside the list to hide mode
  Description: Sets all object inside the list to hide mode
        Input:  
       Output: 
 Restrictions: 
   Created by: Philippe Lavoie (29 January 1998)
  Modified by:
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
void PlObjectListGL::hideAllObjects(){
  state = hideState ;
  PlObjectGL* obj = first_ ;
  while(obj){
    obj->hideObject() ;
    obj = obj->next() ;
  }
}

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
       Member: modifySym --- modify a control point and his symmetrical point
  Description: Modify a control point and his symmetrical point
        Input: v --> modify the points by this value
       Output: 
 Restrictions: 
   Created by: Philippe Lavoie (29 January 1998)
  Modified by:
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
void PlCPointGL::modifySym(const CtrlPnt4D &v) {
  modify(v) ;
  if(symPoint){
    CtrlPnt4D v2(v) ;
    v2.x() *= xCoord ;
    v2.y() *= yCoord ;
    v2.z() *= zCoord ;
    //v2.w() *= wCoord ;
    symPoint->modify(v2) ;
  }
}

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
       Member: setSym --- sets the symmetry for the control points
  Description: Sets the symmetry for the control points
        Input: true --> 1 if it should be in symmetrical mode
       Output: 
 Restrictions: 
   Created by: Philippe Lavoie (29 January 1998)
  Modified by:
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
void PlHNurbsSurfaceGL::setSym(int set, int uDir, float x, float y, float z, float w) {
  if(activePatch){
    int r = activePatch->CtrlP.rows ;
    int c = activePatch->CtrlP.cols ;
    if(true){
      if(uDir) { // do it so that (i,j) and (rows-i,j) are symmetrical
	PlObjectGL *pnt = (PlObjectGL*)cpoints.first ;
	int i,j ;
	i = 0 ;
	j = 0 ;
	while(pnt){
	  ((PlCPointGL*)pnt)->setSym((PlCPointGL*)cpoints.goTo((r-1-i)*c+j),x,y,z,w) ;
	  ++j ;
	  if(j>=c){
	    j = 0 ;
	    ++i ;
	  }
	  pnt = pnt->next() ;
	}
      }
      else{  // (i,j) and (i,cols-j) are symmetrical
	PlObjectGL *pnt = (PlObjectGL*)cpoints.first ;
	int i,j ;
	i = 0 ;
	j = 0 ;
	while(pnt){
	  ((PlCPointGL*)pnt)->setSym((PlCPointGL*)cpoints.goTo(i*c+(c-1-j)),x,y,z,w) ;
	  ++j ;
	  if(j>=c){
	    j = 0 ;
	    ++i ;
	  }
	  pnt = pnt->next() ;
	}
      }
    }
    else{
      PlObjectGL *pnt = (PlObjectGL*)cpoints.first ;
      while(pnt){
	((PlCPointGL*)pnt)->setSym((PlCPointGL*)0,float(1),float(1),float(1),float(1)) ;
	pnt = pnt->next() ;
      }
    }
  }
}

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
       Member: setImage --- attach an image to a surface
  Description: Attach an image to a surface. The image must contain the
               red, green and  blue component in succesive GLubyte
	       of data. The image data must be sent row wise.

	       OpenGL only uses images which are $2^n$ pixels in width
	       or height. If the image is not of the proper size, it
	       will be padded to be acceptable by OpenGL. If you want
	       to center the image, you have to provide an already
	       centered image.

	       The image data is reversed so that it shows upside up
	       when mapped to a NURBS surface. The img data is copied
	       into a new vector so it is safe to delete it once it
	       has been passed to this routine.

        Input: img --> a pointer to the image data
	         w --> the width of the image
		 h --> the height of the image
       Output: 
 Restrictions: 
   Created by: Philippe Lavoie (5 February 1998)
  Modified by:
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
void PlNurbsSurfaceGL::setImage(GLubyte *img, GLint w, GLint h) { 
  if(image) { delete []image ; } 
  int hs = 2 ;
  int ws = 2 ;
  while(ws<w)
    ws *= 2 ;
  while(hs<h)
    hs *= 2 ;
  
  image = new GLubyte[hs*ws*3] ;

  imgW = ws ; 
  imgH = hs ;

  if(w<ws || h<hs){// must patch the data
    for(int i=0;i<hs;++i)
      for(int j=0;j<ws;++j){
	if(i<h && j<w){
	  image[(hs-1-i)*ws*3+j*3] =  img[i*w*3+j*3] ;
	  image[(hs-1-i)*ws*3+j*3+1] =  img[i*w*3+j*3+1] ;
	  image[(hs-1-i)*ws*3+j*3+2] =  img[i*w*3+j*3+2];
	}
	else{
	  image[(hs-1-i)*ws*3+j*3] = 0 ;
	  image[(hs-1-i)*ws*3+j*3+1] = 0  ;
	  image[(hs-1-i)*ws*3+j*3+2] = 0 ;
	}
      }
    
  }
  else{ // just reverse the data
    for(int i=0;i<hs;++i)
      for(int j=0;j<ws;++j){
	image[(h-1-i)*w*3+j*3] =  img[i*w*3+j*3] ;
	image[(h-1-i)*w*3+j*3+1] =  img[i*w*3+j*3+1];
	image[(h-1-i)*w*3+j*3+2] = img[i*w*3+j*3+2] ;
      }
  }

}

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
       Member: PlSPointCurveGL --- The constructor for a curve point object
  Description: The constructor for a curve object
        Input: i --> the index of the control point
	       c --> the pointer to the NURBS curve
       Output: 
 Restrictions: 
   Created by: Philippe Lavoie (20 September 1997)
  Modified by:
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
PlSPointCurveGL::PlSPointCurveGL(int i, PlNurbsCurveSP *c): 
  PlCPointGL(spoint), curve(c)
{ 
  if(!c){
    PlError error("PlSPointCurveGL constructor with a curve point") ;
    error << "You must provide a *valid* nurbs curve pointer." ;
    error.fatal() ; 
  }
  type = spointObject; 
  psize=pSizeDefault ; 
  k = i ; 
  cpoint = curve->surfP(i) ; 
  if(!curve->okMax()) { // it wasn't setup properly
    curve->updateMaxU() ; 
  }
  start = 0 ;
  rows = 0 ; 
} 

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
       Member: PlSPointSurfaceGL --- The constructor for a surface point object
  Description: The constructor for a curve object
        Input: i --> the row of the control point
	       j --> the column of the control point
	       c --> the pointer to the NURBS surface
       Output: 
 Restrictions: 
   Created by: Philippe Lavoie (20 September 1997)
  Modified by:
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
PlSPointSurfaceGL::PlSPointSurfaceGL(int i, int j, PlNurbsSurfaceSP *s, PlObjectListGL *sp): 
  PlCPointGL(spoint), surface(s), spoints(sp)
{ 
  if(!s){
    PlError error("PlSPointSurfaceGL constructor with a surface point") ;
    error << "You must provide a *valid* nurbs surface pointer." ;
    error.fatal() ; 
  }
  if(!spoints){
    PlError error("PlSPointSurfaceGL constructor with a surface point") ;
    error << "You must provide a *valid* point list pointer." ;
    error.fatal() ; 
  }
  type = spointObject; 
  psize=pSizeDefault ; 
  k=i ; 
  l=j ; 
  if(!surface->okMax()) // it wasn't setup properly
    surface->updateMaxUV() ; 

  cpoint = surface->surfP(i,j) ; 
} 

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
       Member: PlSPointSurfaceGL --- The constructor for a surface point object
  Description: The constructor for a curve object
        Input: i --> the row of the control point
	       j --> the column of the control point
	       c --> the pointer to the NURBS surface
       Output: 
 Restrictions: 
   Created by: Philippe Lavoie (20 September 1997)
  Modified by:
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
PlSPointHSurfaceGL::PlSPointHSurfaceGL(int i, int j, PlHNurbsSurfaceSP *s, PlObjectListGL *sp): 
  PlCPointGL(spoint), surface(s), spoints(sp)
{ 
  if(!s){
    PlError error("PlSPointSurfaceGL constructor with a surface point") ;
    error << "You must provide a *valid* nurbs surface pointer." ;
    error.fatal() ; 
  }
  if(!spoints){
    PlError error("PlSPointSurfaceGL constructor with a surface point") ;
    error << "You must provide a *valid* point list pointer." ;
    error.fatal() ; 
  }
  type = spointObject; 
  psize=pSizeDefault ; 
  k=i ; 
  l=j ; 
  if(!surface->okMax()) // it wasn't setup properly
    surface->updateMaxUV() ; 

  if(i>surface->CtrlP.rows)
    cerr << " BIG ERROR\n" ;
  if(j>surface->CtrlP.cols)
    cerr << " HUGE ERROR\n" ;

  cpoint = surface->surfP(i,j,surface->level) ; 
} 

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
       Member: updateOthers --- updates the other control points
  Description: Updates the other control points 
        Input: 
       Output: 
 Restrictions: 
   Created by: Philippe Lavoie (12 May 1998)
  Modified by:
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
void PlSPointCurveGL::updateOthers(){
  PlSPointCurveGL *a ;
  a = start ;
  int n = 0 ;
  if(curve)
    while(a && n<rows){
      if(a!=this){
	a->spoint = a->curve->surfP(a->k) ; 
      }
      a = (PlSPointCurveGL*) a->next() ;      
      ++n ; 
    }
}

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
       Member: updateOthers --- updates the other control points
  Description: Updates the other control points 
        Input: 
       Output: 
 Restrictions: 
   Created by: Philippe Lavoie (12 May 1998)
  Modified by:
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
void PlSPointSurfaceGL::updateOthers(){
  PlSPointSurfaceGL *a ;
  a = start ;
  int n = 0 ;
  int c = 0 ;
  if(surface){
    PlObjectGL* saveCurrent = spoints->current ;
    PlObjectGL* rowStart ;
    spoints->current = a ;
    rowStart = a ; 
    int mc = a->col() ;
    while(a && n<rows){
      if(a!=this){
	a->spoint = a->surface->surfP(a->k,a->l) ;
      }
      if(a->next()){
	if(((PlSPointSurfaceGL*)a->next())->col() > mc)
	  a = (PlSPointSurfaceGL*) a->next() ; 
	else
	  c = cols ;
      }
      ++c ;       
      if(c>=cols){
	++n ; 
	if(n==rows)
	  break ; 
	spoints->current = (PlObjectGL*)rowStart ; 
	a = (PlSPointSurfaceGL*) spoints->jumpToNext() ;
	rowStart = a ; 
	if(a->row() == 0)
	  break ; 
	c = 0 ;
      }
    }
    spoints->current = saveCurrent ;
  }
}

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
       Member: updateOthers --- updates the other control points
  Description: Updates the other control points 
        Input: 
       Output: 
 Restrictions: 
   Created by: Philippe Lavoie (12 May 1998)
  Modified by:
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
void PlSPointHSurfaceGL::updateOthers(){
  PlSPointHSurfaceGL *a ;
  a = start ;
  int n = 0 ;
  int c = 0 ;
  if(surface){
    PlObjectGL* saveCurrent = spoints->current ;
    PlObjectGL* rowStart ;
    spoints->current = a ;
    rowStart = a ; 
    int mc = a->col() ;
    while(a && n<rows){
      if(a!=this){
	a->spoint = a->surface->surfP(a->k,a->l,a->surface->level) ;
      }
      if(a->next()){
	if(((PlSPointSurfaceGL*)a->next())->col() > mc)
	  a = (PlSPointHSurfaceGL*) a->next() ; 
	else
	  c = cols ;
      }
      ++c ;       
      if(c>=cols){
	++n ; 
	if(n==rows)
	  break ; 
	spoints->current = (PlObjectGL*)rowStart ; 
	a = (PlSPointHSurfaceGL*) spoints->jumpToNext() ;
	rowStart = a ; 
	if(a->row() == 0)
	  break ; 
	c = 0 ;
      }
    }
    spoints->current = saveCurrent ;
  }
}

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
       Member: modify --- modifies the surface point
  Description: Modifies the surface point
        Input: v --> modifies the point by this value
       Output: 
 Restrictions: 
   Created by: Philippe Lavoie (12 May 1998)
  Modified by:
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
void PlSPointCurveGL::modify(const CtrlPnt4D& v)  
{ 
  if(curve) 
    curve->modSurfCPby(k,v) ; 
  spoint += v ; 
  updateOthers(); 
}

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
       Member: modify --- modifies the surface point
  Description: Modifies the surface point
        Input: v --> modifies the point by this value
       Output: 
 Restrictions: 
   Created by: Philippe Lavoie (12 May 1998)
  Modified by:
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
void PlSPointSurfaceGL::modify(const CtrlPnt4D& v)  
{ 
  if(surface) 
    surface->modSurfCPby(k,l,v) ; 
  spoint += v ; 
  updateOthers(); 
}

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
       Member: modify --- modifies the surface point
  Description: Modifies the surface point
        Input: v --> modifies the point by this value
       Output: 
 Restrictions: 
   Created by: Philippe Lavoie (12 May 1998)
  Modified by:
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
void PlSPointHSurfaceGL::modify(const CtrlPnt4D& v)  
{ 
  if(surface) 
    surface->modSurfCPby(k,l,v) ; 
  spoint = surface->surfP(k,l,surface->level) ; 
  updateOthers(); 
}



#endif


