/*=====================================================================
        File: nurbs_sp.cc
     Purpose:       
    Revision: $Id: nurbs_sp.cc,v 2.2 1998/05/13 00:21:15 lavoie Exp $
  Created by: Philippe Lavoie          (7 May, 1998)
 Modified by: 

 Copyright notice:
          Copyright (C) 1996-1997 Philippe Lavoie
 
          This library is free software; you can redistribute it and/or
          modify it under the terms of the GNU Library General Public
          License as published by the Free Software Foundation; either
          version 2 of the License, or (at your option) any later version.
 
          This library is distributed in the hope that it will be useful,
          but WITHOUT ANY WARRANTY; without even the implied warranty of
          MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
          Library General Public License for more details.
 
          You should have received a copy of the GNU Library General Public
          License along with this library; if not, write to the Free
          Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
=====================================================================*/
#include <nurbs_sp.hh>

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
       Member: updateMaxU --- updates the basis value
  Description: Updates the basis value at which a control point has 
               maximal influence. It also finds where the control point
	       has maximal influence. 
        Input:
       Output:
 Restrictions: The degree of the curve must be of 3 or less.
   Created by: Philippe Lavoie (7 May, 1998)
  Modified by:
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
void PlNurbsCurveSP::updateMaxU() {
  if(deg>3){
    PlError error("PlNurbsCurveSP") ; 
    error << "This class can't handle a curve of degree 4 or higher.\n" ;
    error.fatal() ;
  }
  else{
    maxU.resize(P.n) ;
    maxAt_.resize(P.n) ;
    for(int i=0;i<P.n;++i){
      if(!maxInfluence(i,U,deg,maxAt_[i]))
	cerr << "Problem in maxInfluence U!\n" ;
      if(i>0)
	if(maxAt_[i]<maxAt_[i-1]){
	  PlError error("Error updating maxU");
	  error << "HUGE ERROR!\n" ;
	  error << "Knot = " << U << endl ;
	  error << " i = " << i << endl ;
	  error << " deg = " << deg << endl ;
	  error.fatal() ; 
	}
      maxU[i] = basisFun(maxAt_[i],i,deg) ;
    }
    
  }
}
