/*
 * Project : "MindsEye"
 * Copyright 1996, 1998. All Rights Reserved.
 * Author(s) Frank Meier, M. Kuball
 *
 * $Id: m_core.h,v 1.6 1998/03/19 21:22:31 source Exp $
 *
 *
 * COPYRIGHT
 * =========
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 * 
 * OVERVIEW
 * ========
 *
 */

#include "prisma.h"
#include "3dlib/t_objects.h"
#include <GL/gl.h>
#include <GL/glu.h>

#define T_PRISMA (T_SHAPE + T_EXTERN + 0x1)


G3DPrisma::G3DPrisma( void ) :
  _origin(0,0,0), _e1(0,0,1), _e2(0.5,0,0), _e3(0,1,1)
{
  type_ = T_PRISMA;
  tesselate();
}

G3DPrisma::G3DPrisma( const G3DPrisma &orig ) :
  G3DShape( orig )
{
  _origin = orig._origin;
  _e1     = orig._e1;
  _e2     = orig._e2;
  _e3     = orig._e3;
  tesselate();  //TODO copy
}

G3DPrisma::~G3DPrisma()
{
}


G3DObject*
G3DPrisma::copy( void ) const
{
  return( new G3DPrisma(*this) );
}

Field G3DPrisma::FieldList_[4] =
{ {"origin", s_point3}, {"e1", s_vector3}, {"e2", s_vector3},
  {"e2", s_vector3} };

const Field*
G3DPrisma::getFieldList( void )
{
  return( FieldList_ );
}


uint
G3DPrisma::getParameter( Param *pList )
{
  if( pList )
    {
      pList->point3  = &_origin;
      pList++;
      pList->vector3 = &_e1;
      pList++;
      pList->vector3 = &_e2;
      pList++;
      pList->vector3 = &_e3;
    }
  return( 4 );
}

void
G3DPrisma::getPosition( G3DPoint3d &pos ) const
{
  pos = _origin;
}

const char*
G3DPrisma::GetTypeName( void )
{
  return( "Prisma" );
}

const char*
G3DPrisma::getTypeName( void ) const
{
  return( G3DPrisma::GetTypeName() );
}

void
G3DPrisma::resetBoundingBox( void )
{
  G3DVector3d v = _e1.cross( _e2 );
  v.normalize();
  v *= v.dot( _e3 );

  bbox_.setTo( _origin, v, _e1, _e2 );
  if( parent_ )
    ((G3DShape*)parent_)->resetBoundingBox();


}

void
G3DPrisma::setPosition( const G3DPoint3d &pos )
{
  _origin = pos;
  tesselate();
  resetBoundingBox();
}

void
G3DPrisma::shade( int ) const
{
  static int faces[] = { 0,1,2,3, 1,4,5,2, 0,3,5,4,
			 0,4,4,1, 2,5,5,3 };
  const int *facep = faces;

  glBegin( GL_QUADS );
  for( int i = 0; i < 5; i++ )
    {
      glNormal3dv( (const double*)(_norm[i]) );
      glVertex3dv( (const double*)(_points[*(facep++)]) );
      glVertex3dv( (const double*)(_points[*(facep++)]) );
      glVertex3dv( (const double*)(_points[*(facep++)]) );
      glVertex3dv( (const double*)(_points[*(facep++)]) );
    }
  glEnd();
}

void
G3DPrisma::tesselate()
{
  G3DVector3d  t1, t2, t3;

  _points[0] = _origin;
  _points[1] = t1 = _origin + _e1;
  _points[2] = t1 + _e2;
  _points[3] = _origin + _e2;
  _points[4] = t1 = _origin + _e3;
  _points[5] = t1 + _e2;

  _norm[0] = _e2.cross( _e1 );
  _norm[0].normalize();
  _norm[1] = _e2.cross( _e3 - _e1 );
  _norm[1].normalize();
  _norm[2] = _e3.cross( _e2 );
  _norm[2].normalize();
  _norm[3] = _e1.cross( _e3 );
  _norm[3].normalize();
  _norm[4] = _norm[3] * (-1.0);
}


//=================================END==============================
