/*
 * Project : "MindsEye, 3D library"
 * Copyright 1996, 1997. All Rights Reserved.
 * Author(s): M. Kuball
 *
 * $Id: attribute.cpp,v 1.3 1997/12/14 16:57:56 source Exp $
 *
 * 
 * COPYRIGHT
 * =========
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

#include <GL/gl.h>
#include <GL/glu.h>

#include "a-list.h"
#include "attribute.h"
#include "3dlib/shape.h"


G3DAttributeList::G3DAttributeList( const G3DAttributeList &orig )
{
  G3DAttribute *tmp, *tmp1 = orig.root_, *tmp2;

  root_ = NULL;
  while( tmp1 )
    {
      tmp2 = tmp1;
      while( tmp2 )
	{
	  tmp  = tmp2->copy();
	  if( !root_ )
	    root_ = tmp;
	  tmp2 = tmp2->suc_;
	}
      tmp1 = tmp1->next_;
    }
}


G3DAttributeList::~G3DAttributeList()
{
  //TODO release all attributes
}


void
G3DAttributeList::addGroup( G3DAttribute *attrib )
{
  if( root_ == NULL )
    root_ = attrib;
  else
    {
      attrib->next_ = root_->next_;
      root_->next_  = attrib;
    }
}

G3DAttribute*
G3DAttributeList::findGroup( t_type type )
{
  G3DAttribute *tmp = root_;
  while( tmp )
    {
      if( tmp->type() == type )
	break;
      else
	tmp = tmp->next_;
    }
  return tmp;
}

G3DAListIterator
G3DAttributeList::get( const G3DShape *parent, t_type type,
		       bool inherit ) const
{
  t_type tnum  = type & TA_MASK;

  G3DAttribute *tmp = root_;
  while( tmp )
    {
      if( tmp->type() & type )
	break;
      else
	tmp = tmp->next_;
    }
  if( (tnum != 0) && tmp )
    {  //We'r looking for a special subtype
      while( tmp )
	{
	  if( tmp->tnum() == tnum )
	    break;
          else
	    tmp = tmp->suc_;
	}
    }
  if( (tmp == NULL) && inherit && parent )
    return( parent->getAttribute(type, inherit) );
  else
    return( G3DAListIterator(tmp, parent, type, inherit ) );
}

G3DAttribute*
G3DAttributeList::insert( G3DAttribute *attrib )
{
  G3DAttribute *tmp = findGroup( attrib->type_ & (~TA_MASK) );

  if( tmp == NULL )
    addGroup( attrib );
  else
    {
      t_type type = attrib->tnum();
      while( tmp )
	{
	  if( tmp->tnum() == type )
	    { //TODO test if poly
	      if( tmp->isReference() )
		{
		  replace( tmp, attrib );
		  tmp->Dispose();
		  break;
		}
	      else
		{
		  tmp->setValues( attrib );
		  delete attrib;  //TODO or attrib->Delete() ??
		  return( tmp );
		}
	    }
	  else if( tmp->suc_ )
	    tmp = tmp->suc_;
	  else
	    {
	      tmp->suc_ = attrib;
	      break;
	    }
	}
    }
  return( attrib );
}

void
G3DAttributeList::remove( t_type )
{
  //TODO
}

void
G3DAttributeList::remove( G3DAttribute *attrib )
{
  G3DAttribute *tmp1 = root_, *tmp2 = root_;
  t_type type = attrib->type();

  while( tmp2 )
    {
      if( tmp2->type() == type )
	break;
      tmp1 = tmp2;
      tmp2 = tmp2->next_;
    }

  if( tmp2 == attrib )
    {
      if( tmp2 == root_ )
	{
	  if( tmp2->suc_ )
	    {
	      root_        = tmp2->suc_;
	      root_->next_ = tmp2->next_;
	    }
          else
	    root_ = tmp2->next_;
	}
      else
	{
	  if( tmp2->suc_ )
	    {
	      tmp2->suc_->next_ = tmp1->next_;
	      tmp1->next_       = tmp2->suc_;
	    }
	  else
	    tmp1->next_ = tmp2->next_;
	}	    
    }
  else
    {
      //OK, search in the subgroup
      tmp1 = tmp2->suc_;
      while( tmp1 )
	{
	  if( tmp1 == attrib )
	    {
	      tmp2->suc_ = tmp1->suc_;
	      break;
	    }
	  tmp2 = tmp1;
	  tmp1 = tmp1->suc_;
	}
    }
}

void
G3DAttributeList::replace( G3DAttribute *rem_atr, G3DAttribute *rep_atr )
{
  G3DAttribute *tmp1 = root_, *tmp2 = root_;
  t_type type = rem_atr->type();

  rep_atr->next_ = rem_atr->next_;
  rep_atr->suc_  = rem_atr->suc_;

  while( tmp2 )
    {
      if( tmp2->type() == type )
	break;
      tmp1 = tmp2;
      tmp2 = tmp2->next_;
    }

  if( tmp2 == rem_atr )
    {
      if( rem_atr == root_ )
	root_ = rep_atr;
      else
	tmp1->next_ = rep_atr;
    }
  else
    {
      //OK, search in the subgroup
      tmp1 = tmp2->suc_;
      while( tmp1 )
	{
	  if( tmp1 == rem_atr )
	    {
	      tmp2->suc_ = rep_atr;;
	      break;
	    }
	  tmp2 = tmp1;
	  tmp1 = tmp1->suc_;
	}
    }
}


//------------------------------------------------------------


void
G3DAListIterator::operator++( )
{
  if( marker_ )
    {
      t_type type = type_ & TA_MASK;
      if( type != 0 )
	{
	  while( marker_ )
	    {
	      if( marker_->tnum() == type )
		break;
	      else
		marker_ = marker_->suc_;
	    }
	}
      else
	marker_ = marker_->suc_;
    }
  //TODO consider the next group

  if( (marker_ == NULL) && inherit_ && parent_ )
    *this = parent_->getAttribute( type_ );
}


//==================================END================================
