/*
 * Project : "MindsEye, 3D library"
 * Copyright 1996, 1997. All Rights Reserved.
 * Author(s): M. Kuball
 *
 * $Id: attribute.h,v 1.8 1998/04/06 20:05:55 source Exp $
 *
 * 
 * COPYRIGHT
 * =========
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * OVERVIEW
 * ========
 * basic attribute classes
 *
 */

#ifndef _ATTRIBUTE_LIST_H
#define _ATTRIBUTE_LIST_H

#include <stdlib.h>
#include "3dlib/t_objects.h"

class G3DAttribute;
class G3DShape;
class G3DAListIterator;


/**A dynamic List used to store attributes.
   Attributes are classified by type. A type contains
   attributes of different subtypes. Types are e.g. material,
   constraint or animation.
 */
class G3DAttributeList
{
public:
  G3DAttributeList( ) { root_ = NULL; };
  G3DAttributeList( const G3DAttributeList& );
  ~G3DAttributeList();

  /**Insert an attribute. If an attribute with the type of atrr already exists,
     the values of atr are assigned to the existing one and it's address is
     returned. Otherwise atr is returned.*/
  G3DAttribute*    insert( G3DAttribute *atrr );
  G3DAListIterator get( const G3DShape*, t_type, bool ) const;
  //TODO G3DAttribute get( const char*, G3DShape* ) const;
    ///Remove all attributes which match the given type.
  void             remove( t_type );
    ///Remove the attribute attr from the list. attr is not released.
  void             remove( G3DAttribute *attr );
    ///Replace rem_atr with rep_atr.
  void             replace( G3DAttribute *rem_atr, G3DAttribute *rep_atr );

protected:
    /**A new group is always added after the first group if it
       exists.*/
  void          addGroup( G3DAttribute *attrib );
  G3DAttribute* findGroup( t_type type );

protected:
  G3DAttribute *root_;

};


/**An Object of this type is used to specify the result of a search for
  specific attributes in the AttributeList.
  It's a forward iterator and it can be used only once.
  */
class G3DAListIterator
{
public:
  G3DAListIterator( void )
    { marker_ = NULL; parent_ = NULL; type_ = 0; inherit_ = 0; }
  G3DAListIterator( G3DAttribute *atr, const G3DShape *p, t_type t, bool ih )
    { marker_ = atr; parent_ = p; type_ = t; inherit_ = ih; }

  inline G3DAttribute* operator*() { return marker_; }
  void operator++();

protected:
  G3DAttribute   *marker_;
  const G3DShape *parent_;
  t_type         type_;
  bool           inherit_;
};


#endif
