/*
 * Project : "MindsEye, 3D library"
 * Copyright 1996, 1997. All Rights Reserved.
 * Author(s): M. Kuball
 *
 * $Id: animation.h,v 1.2 1998/05/25 20:23:12 source Exp $
 *
 * 
 * COPYRIGHT
 * =========
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * OVERVIEW
 * ========
 * basic attribute classes
 *
 */

#ifndef __ANIMATION_H_
#define __ANIMATION_H_

#include "attribute.h"


class G3DShape;
class G3DObject;


/**Superclass of all animation specific attributes.
 */
class G3DAnimation : public G3DMonoAttribute
{
public:
  G3DAnimation( float atime ) { start_ = stop_ = atime; };

  virtual void evaluateAt( G3DObject*, float t ) const = 0;
  virtual void saveValueAt( const G3DObject*, float t ) = 0;

protected:
  void clamp( float &t ) const
    {
      if( t < start_ ) t = start_;
      else if( t > stop_ )  t = stop_;
    }
  void expand( float t )
    {
      if( t < start_ ) start_ = t;
      else if( t > stop_ ) stop_ = t;
    }

protected:
  //specify time interval. Values outside are clamped
  float start_, stop_;
};

/*

class G3DAnimationScript
class G3DAnimateRot
class G3DAnimateScale
class G3DAnimateQuality
class G3DAnimateMaterial
class G3DAnimateCtpt Vertex
*/


/**
 */
class G3DAnimationVis : public G3DAnimation
{
public:
  G3DAnimationVis( float atime ) : G3DAnimation(atime)
    { type_ = TA_ANIM_VIS; root_ = NULL; }

  virtual void evaluateAt( G3DObject*, float t ) const;
  virtual void saveValueAt( const G3DObject*, float t );
    ///Return a list with the names and types of the object s parameter.
  virtual const Field* getFieldList( void ) const;
    /**Get a list of params describing the geometry of the object.
       If plist is Null you get just the number of elements.*/
  virtual uint  getParameter( Param* );
    ///Return a Descriptive name of the shape.
  virtual const char* getTypeName( void ) const;

    ///Set the user visible values of the attribute to the values of atr.
  virtual void  setValues( const G3DAttribute *atr );

protected:
  struct tvis {
    float  timep;
    short  visible;
    tvis   *next;
  }  *root_;

  static Field FieldList_[1];

};

#endif
