/*
 * Project : "MindsEye, 3D library"
 * Copyright 1996, 1997. All Rights Reserved.
 * Author(s): M. Kuball
 *
 * $Id: attribute.cpp,v 1.5 1998/07/23 20:15:56 source Exp $
 *
 * 
 * COPYRIGHT
 * =========
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

#include <GL/gl.h>
#include <GL/glu.h>

#include "attribute.h"



G3DAttribute::~G3DAttribute()
{
}

void
G3DAttribute::activate( void ) const
{
}

G3DAttribute*
G3DAttribute::copy( void ) const
{
  return( NULL );
}

const G3DAttribute*
G3DAttribute::getReference( void ) const
{
  return this;
}

bool
G3DAttribute::isInheritable( void )
{
  return false;
}

void
G3DAttribute::setParameter( const Param* )
{
}

void
G3DAttribute::Dispose( void )
{
  --refCount_;
  if( refCount_ <= 0 )
    delete this;
}


//-------------------------------------------------------------------


void
G3DAReference::activate( void ) const
{
  ref_->activate();
}

G3DAttribute*
G3DAReference::copy( void ) const
{
  return( new G3DAReference(ref_) );
}

const Field*
G3DAReference::getFieldList( void ) const
{
  return( ref_->getFieldList() );
}

uint
G3DAReference::getParameter( Param *pList )
{
  return( ref_->getParameter(pList) );
}

const G3DAttribute*
G3DAReference::getReference( void ) const
{
  return ref_;
}

const char*
G3DAReference::getTypeName( void ) const
{
  return( ref_->getTypeName() );
}

int
G3DAReference::isMono( void )
{
  return( ref_->isMono() );
}

void
G3DAReference::Dispose( void )
{
  ref_->Dispose();
  delete this;
}

void
G3DAReference::setValues( const G3DAttribute *src )
{
  ref_->setValues( src );
}


//-------------------------------------------------------------------

G3DAttribute*
G3DAResolution::copy( void ) const
{
  return( new G3DAResolution(*this) );
}

Field G3DAResolution::FieldList_[1] = { {"quality", s_flt} };

const Field*
G3DAResolution::getFieldList( void ) const
{
  return( FieldList_ );
}

uint
G3DAResolution::getParameter( Param *pList )
{
  if( pList )
    {
      pList->flt  = quality_;
    }
  return( 1 );
}

const char*
G3DAResolution::GetTypeName( void )
{
  return( "Resolution" );
}

const char*
G3DAResolution::getTypeName( void ) const
{
  return( G3DAResolution::GetTypeName() );
}

void
G3DAResolution::setValues( const G3DAttribute *src )
{
  if( src->getType() == type_ )
    {
      quality_ = ((G3DAResolution*)src)->quality_;
      //style_   = ((G3DAResolution&)src).style_;
    }
}


//-------------------------------------------------------------------


G3DA_URL::~G3DA_URL()
{
  if( url_ )
    free( url_ );
}

G3DAttribute*
G3DA_URL::copy( void ) const
{
  return( new G3DA_URL(*this) );
}

Field G3DA_URL::FieldList_[1] = { {"url", s_string} };

const Field*
G3DA_URL::getFieldList( void ) const
{
  return( FieldList_ );
}

uint
G3DA_URL::getParameter( Param *pList )
{
  if( pList )
    {
      pList->string  = url_;
    }
  return( 1 );
}

const char*
G3DA_URL::getTypeName( void ) const
{
  return( "URL" );
}


void
G3DA_URL::setURL( const char *url )
{
  if( url_ )
    free( url_ );
  
}

void
G3DA_URL::setValues( const G3DAttribute* )
{
  //TODO
}


//==================================END================================
