/*
 * Project : "MindsEye, 3D library"
 * Copyright 1996, 1997. All Rights Reserved.
 * Author(s): M. Kuball
 *
 * $Id: texture.h,v 1.4 1998/07/23 20:15:57 source Exp $
 *
 * 
 * COPYRIGHT
 * =========
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * OVERVIEW
 * ========
 * basic attribute classes
 *
 */

#ifndef _METAL_H_
#define _METAL_H_

//#include <string.h>
#include "texture.h"



/*
class G3DAOpaqueTexture : public G3DATexture
{
};
*/
/**
 */
/*
class G3DADefaultTexture : public G3DATexture
{
};
*/

/**A Description of the visible properties of a surface
   using the Phong ilumination model.
  */
class G3DAMetal : public G3DATexture
{
public:
  G3DAMetal( void ) :
    acolor_(ac,ac,ac), dcolor_(dc,dc,dc)
    {
      specRef_      = 1.0;
      mirrorEffect_ = 0.0;
      roughness_    = 0.1;
      type_         = TA_METAL;
    };
  G3DAMetal( const G3DAMetal &orig ) :
    acolor_(orig.acolor_), dcolor_(orig.dcolor_)
    {
      specRef_      = orig.specRef_;
      mirrorEffect_ = orig.mirrorEffect_;
      roughness_    = orig.roughness_;
      type_   = TA_METAL;
    };
  ~G3DAMetal();

    ///Set the corresponding OpenGL state variables.
  virtual void activate( void ) const;
  virtual G3DAttribute* copy( void ) const;
    ///Return a list with the names and types of the object s parameter.
  virtual const Field* getFieldList( void ) const;
    /**Get a list of params describing the geometry of the object.
       If plist is Null you get just the number of elements.*/
  virtual uint  getParameter( Param* );
    ///Return a Descriptive name of the shape.
  virtual const char* getTypeName( void ) const;
    ///Return a Descriptive name of the shape.
  static  const char* GetTypeName( void );

    ///
  void getAmbientColor( G3DColor &afc ) const { afc = acolor_; };
    ///
  void getDiffuseColor( G3DColor &dfc ) const { dfc = dcolor_; }
    ///
  float getMirrorEffect( void ) { return mirrorEffect_; }
    ///
  float getRoughness( void ) const { return roughness_; }
    ///
  float getSpecRef( void ) const { return specRef_; }
    ///
  void setAmbientColor( const G3DColor &afc ) { acolor_ = afc; }
    ///
  void setDiffuseColor( const G3DColor &dfc ) { dcolor_ = dfc; }
    ///
  void setMirrorEffect( float me ) { mirrorEffect_ = me; }
    ///
  void setRoughness( float rg ) { roughness_ = rg; }
    ///
  void setSpecRef( float sp ) { specRef_ = sp; }

    ///Set the user visible values of the attribute to the values of atr.
  virtual void  setValues( const G3DAttribute *atr );

protected:
  G3DColor acolor_;  //ambient color
  G3DColor dcolor_;  //diffuse color
  float    specRef_;
  float    roughness_;
  float    mirrorEffect_;

  static Field FieldList_[5];

};


#endif
