/*
 * Project : "MindsEye, 3D library"
 * Copyright 1996, 1997. All Rights Reserved.
 * Author(s): M. Kuball
 *
 * $Id: texture.cpp,v 1.4 1998/07/23 20:15:57 source Exp $
 *
 * 
 * COPYRIGHT
 * =========
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

#include <GL/gl.h>
#include <GL/glu.h>
#include <math.h>
#include "metal.h"


G3DAMetal::~G3DAMetal()
{
}

void
G3DAMetal::activate( void ) const
{
  float scolor[4] = {1,1,1,1};
  glMaterialfv( GL_FRONT, GL_AMBIENT,  (const float*)acolor_ );
  glMaterialfv( GL_FRONT, GL_DIFFUSE,  (const float*)dcolor_ );
  glMaterialfv( GL_FRONT, GL_SPECULAR, scolor );

  float phong;
  phong = log( roughness_ ) * 128 / log(0.00001);
  glMaterialf( GL_FRONT, GL_SHININESS, phong );
  //TODO  glMaterialfv (GL_FRONT, GL_EMISSION, no_mat);
}

G3DAttribute*
G3DAMetal::copy( void ) const
{
  return( new G3DAMetal(*this) );
}

Field G3DAMetal::FieldList_[5] =
{ {"ambient", s_color}, {"color", s_color}, {"specular", s_flt},
  {"roughness", s_flt}, {"mirror_effect", s_flt} };

const Field*
G3DAMetal::getFieldList( void ) const
{
  return( FieldList_ );
}

uint
G3DAMetal::getParameter( Param *pList )
{
  if( pList )
    {
      pList->color = &acolor_;
      pList++;
      pList->color = &dcolor_;
      pList++;
      pList->flt = specRef_;
      pList++;
      pList->flt   = roughness_;
      pList++;
      pList->flt   = mirrorEffect_;
    }
  return( 5 );
}

const char*
G3DAMetal::GetTypeName( void )
{
  return( "MetalFinish" );
}

const char*
G3DAMetal::getTypeName( void ) const
{
  return( G3DAMetal::GetTypeName() );
}

void
G3DAMetal::setValues( const G3DAttribute *src )
{
  if( src->getType() == type_ )
    {
      acolor_       = ((G3DAMetal*)src)->acolor_;
      dcolor_       = ((G3DAMetal*)src)->dcolor_;
      specRef_      = ((G3DAMetal*)src)->specRef_;
      roughness_    = ((G3DAMetal*)src)->roughness_;
      mirrorEffect_ = ((G3DAMetal*)src)->mirrorEffect_;
    }
}


//==================================END================================
