/*
 * Project : "MindsEye, 3D library"
 * Copyright 1996, 1997. All Rights Reserved.
 * Author(s): M. Kuball
 *
 * $Id: texture.cpp,v 1.4 1998/07/23 20:15:57 source Exp $
 *
 * 
 * COPYRIGHT
 * =========
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

#include <GL/gl.h>
#include <GL/glu.h>

#include "texture.h"


bool
G3DATexture::isInheritable( void )
{
  return true;
}


G3DAColor::~G3DAColor()
{
}

void
G3DAColor::activate( void ) const
{
  glMaterialfv( GL_FRONT, GL_AMBIENT,  (const float*)acolor_ );
  glMaterialfv( GL_FRONT, GL_DIFFUSE,  (const float*)dcolor_ );
  glMaterialfv( GL_FRONT, GL_SPECULAR, (const float*)scolor_ );
  glMaterialf( GL_FRONT, GL_SHININESS, sexp_ );
  //TODO  glMaterialfv (GL_FRONT, GL_EMISSION, no_mat);
}

G3DAttribute*
G3DAColor::copy( void ) const
{
  return( new G3DAColor(*this) );
}

Field G3DAColor::FieldList_[5] =
{ {"ambient", s_color}, {"diffuse", s_color}, {"specular", s_color},
  {"spec_exp", s_flt}, {"transparency", s_flt} };

const Field*
G3DAColor::getFieldList( void ) const
{
  return( FieldList_ );
}

uint
G3DAColor::getParameter( Param *pList )
{
  if( pList )
    {
      pList->color = &acolor_;
      pList++;
      pList->color = &dcolor_;
      pList++;
      pList->color = &scolor_;
      pList++;
      pList->flt   = sexp_;
      pList++;
      pList->flt   = transp_;
    }
  return( 5 );
}

const char*
G3DAColor::GetTypeName( void )
{
  return( "Color" );
}

const char*
G3DAColor::getTypeName( void ) const
{
  return( G3DAColor::GetTypeName() );
}

void
G3DAColor::setValues( const G3DAttribute *src )
{
  if( src->getType() == type_ )
    {
      //G3DAColor
      dcolor_ = ((G3DAColor*)src)->dcolor_;
      scolor_ = ((G3DAColor*)src)->scolor_;
      sexp_   = ((G3DAColor*)src)->sexp_;
      transp_ = ((G3DAColor*)src)->transp_;
    }
}


//==================================END================================
