/*
 * Project : "MindsEye, 3D library"
 * Copyright 1996, 1997. All Rights Reserved.
 * Author(s): M. Kuball
 *
 * $Id: texture.h,v 1.4 1998/07/23 20:15:57 source Exp $
 *
 * 
 * COPYRIGHT
 * =========
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * OVERVIEW
 * ========
 * basic attribute classes
 *
 */

#ifndef _TEXTURE_H_
#define _TEXTURE_H_

//#include <string.h>
#include "attribute.h"


/*
simple-exture  pov-texture rib-texture
map-texture proc-texture

bump-texture
displacement shader
*/

const float ac = 0.2;
const float dc = 0.7;
const float sc = 0.0;

class G3DATexture : public G3DMonoAttribute
{
public:
  G3DATexture( void ) {}

  virtual bool isInheritable( void );
};

/*
class G3DAOpaqueTexture : public G3DATexture
{
};
*/
/**
 */
/*
class G3DADefaultTexture : public G3DATexture
{
};
*/

/**A Description of the visible properties of a surface
   using the Phong ilumination model.
  */
class G3DAColor : public G3DATexture
{
public:
  G3DAColor( void ) :
    acolor_(ac,ac,ac), dcolor_(dc,dc,dc), scolor_(sc,sc,sc)
    {
      sexp_   = 0;
      transp_ = 1.0;
      type_   = TA_COLOR;
    };
  G3DAColor( const G3DAColor &orig ) :
    acolor_(orig.acolor_), dcolor_(orig.dcolor_), scolor_(orig.scolor_)
    {
      sexp_   = orig.sexp_;
      transp_ = orig.transp_;
      type_   = TA_COLOR;
    };
  ~G3DAColor();

    ///Set the corresponding OpenGL state variables.
  virtual void activate( void ) const;
  virtual G3DAttribute* copy( void ) const;
    ///Return a list with the names and types of the object s parameter.
  virtual const Field* getFieldList( void ) const;
    /**Get a list of params describing the geometry of the object.
       If plist is Null you get just the number of elements.*/
  virtual uint  getParameter( Param* );
    ///Return a Descriptive name of the shape.
  static  const char* GetTypeName( void );
    ///Return a Descriptive name of the shape.
  virtual const char* getTypeName( void ) const;

    ///
  void getAmbientColor( G3DColor &afc ) const { afc = acolor_; };
    ///
  void getDiffuseColor( G3DColor &dfc ) const { dfc = dcolor_; };
    ///
  void getSpecularColor( G3DColor &spc ) const { spc = dcolor_; };
    ///
  float getSpecularExp( void ) const { return sexp_; };
    ///
  void setAmbientColor( const G3DColor &afc ) { acolor_ = afc; };
    ///
  void setDiffuseColor( const G3DColor &dfc )
    { dcolor_ = dfc; dcolor_.setA( transp_ ); }
    ///
  void setShininess( float sh ) { sexp_ = sh; }
    ///
  void setSpecularColor( const G3DColor &spc ) { scolor_ = spc; };

    ///Set the user visible values of the attribute to the values of atr.
  virtual void  setValues( const G3DAttribute *atr );

protected:
  G3DColor acolor_;  //ambient color
  G3DColor dcolor_;  //diffuse color
  G3DColor scolor_;  //specular color
  float    sexp_;    //specular exponent
  float    transp_;  //transparency;

  static Field FieldList_[5];
//emission
//refraction

};

/**
 */
/*
class G3DAImageMap : public G3DATexture
{
public:
  G3DAImageMap( void );

    ///Set the corresponding OpenGL state variables.
  virtual void activate( void ) const;

protected:
  char *image_;
  char *source_;  //if not null points to a file-name
  enum projection_;

};
*/

//class G3DAPlastic : public G3DAColor
//class G3DAMetal : public G3DAColor
//class G3DATrafo
//class G3DABumpMap
//class G3DAProcedure
//class G3DADisplacement


#endif
