/*
 * Project : "MindsEye, 3D library"
 * Copyright 1996 - 1998. All Rights Reserved.
 * Author(s): M. Kuball
 *
 * $Id: aggregate.h,v 1.8 1998/06/22 20:12:20 source Exp $
 *
 * 
 * COPYRIGHT
 * =========
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * Description
 * ===========
 * classes for building linked lists
 */


#include "include/config.h"
#include <stdlib.h>
#include <assert.h>

#include "aggregate.h"
#include "iterator.h"
#include "shape.h"
#include "light.h"


template<class T>
void G3DDLList<T>::append( T* pElement )
{
  if( !last_ )
    {
      last_ = new G3DListElement<T>( pElement ); 
      root_ = last_;
    }
  else
    {
      G3DListElement<T>* tmp = new G3DListElement<T>(pElement); 
      tmp->prev   = last_;
      last_->next = tmp;
      last_       = tmp;
    }
  incNumber();
}


template<class T>
void G3DDLList<T>::doWithElements(GIterator<T> &it) const
{
  G3DListElement<T>* tmp = root_;
  while( tmp )
    {
      T& tElement = tmp->getElement();
      if( it.exec(tElement) )
	break;
      tmp = tmp->next;
    }
}


template<class T>
int G3DDLList<T>::findElement( T* obj ) const
{ 
  G3DListElement<T>* tmp = root_;
  if( !tmp )
    return( -1 );
  int num = 0;
  do
    if( tmp->getPointer() == obj )
      return( num );
  while( ++num, (tmp = tmp->next) );
  return( -1 );
}


template<class T>
T* G3DDLList<T>::getElement( int pIndex )
{
  assert((0 <= pIndex) && (pIndex < size()));
  G3DListElement<T>* tmp = root_;
  for (int i = 0; i < size(); i++)
    {
      assert(tmp); 
      if (i == pIndex)
	return(tmp->getPointer());
      else
	tmp = tmp->next;
    }
  // never reached because of first assertion:
  return( NULL );
}


template<class T>
void G3DDLList<T>::remove( int pIndex )
{ 
  assert( (0 <= pIndex) && (pIndex < size()) );

  G3DListElement<T>* tmp = root_;
  int i = 0;
  while( tmp )
    {
      if( i == pIndex )
	{
	  if( tmp->prev )
	    tmp->prev->next = tmp->next;
	  else
	    root_ = tmp->next;
	  if (tmp->next)
	    tmp->next->prev = tmp->prev;
	  else
	    last_ = tmp->prev;
	  if( clear_ )
	    delete tmp->getPointer();
	  delete tmp;
	  decNumber();
	  return;
	}
      else tmp = tmp->next;
    }
}

template<class T>
void G3DDLList<T>::remove( T* obj )
{ 
  G3DListElement<T>* tmp = root_;
  while( tmp )
    {
      if( tmp->getPointer() == obj )
	{
	  if( tmp->prev )
	    tmp->prev->next = tmp->next;
	  else
	    root_ = tmp->next;
	  if( tmp->next )
	    tmp->next->prev = tmp->prev;
	  else
	    last_ = tmp->prev;
	  if( clear_ )
	    delete tmp->getPointer();
	  delete tmp;
	  decNumber();
	  return;
	}
      tmp = tmp->next;
    }
}

template<class T>
void G3DDLList<T>::reInitialize(void)
{
  G3DListElement<T> *x, *tmp = root_;

  while( tmp )
    {
      x = tmp;
      if( clear_ )
	delete tmp->getPointer();
      tmp = tmp->next;
      delete x;
    }
  number_ = 0;
  root_   = last_ = NULL;
}

template<class T>
void G3DDLList<T>::setElement(int pIndex, T& obj)
{
  assert(pIndex >= 0);
  if (pIndex >= size())
    append(&obj);
  else
    {
      G3DListElement<T>* tmp = root_;
      for( int i = 0; i < size(); i++ )
	{
	  assert(tmp); 
	  if (i == pIndex)
	    {
	      tmp->setElement( obj );
	      break;
	    }
	  else tmp = tmp->next;
	}
    }
}

template<class T>
G3DDLList<T>& G3DDLList<T>::operator=( const G3DDLList &src )
{
  reInitialize();

  G3DListElement<T> *tmp = src.root_;
  while( tmp )
    {
      append( (T*)((tmp->getPointer())->copy()) );
      tmp = tmp->next;
    }
}

template<class T>
T& G3DDLList<T>::operator[](int pIndex)
{
  assert((0 <= pIndex) && (pIndex < size()));
  G3DListElement<T>* tmp = root_;
  for( int i = 0; i < size(); i++ )
    {
      assert(tmp); 
      if( i == pIndex )
	return( tmp->getElement() );
      else
	tmp = tmp->next;
    }
}
 
#ifdef EXPLICIT_TEMPLATES
#ifdef __GNUG__

template class G3DListElement<G3DShape>;
template class G3DListElement<G3DLight>;
template class G3DListElement<G3DObject>;
template class G3DListElement<G3DWidget>;

template class G3DDLList<G3DShape>;
template class G3DDLList<G3DLight>;
template class G3DDLList<G3DObject>;
template class G3DDLList<G3DWidget>;

#else

void  Vector_dumbSPARC()
{
  G3DDLList<G3DShape> d1;
  G3DDLList<G3DLight> d2;
  G3DDLList<G3DObject> d3;
  G3DDLList<G3DWidget> d4;
}

#endif
#endif  //EXPLICIT_TEMPLATES

//====================================END===============================

