/*
 * Project : "MindsEye, 3D library"
 * Copyright 1996 - 1998. All Rights Reserved.
 * Author(s): M. Kuball
 *
 * $Id: aggregate.h,v 1.9 1998/07/23 20:15:46 source Exp $
 *
 * 
 * COPYRIGHT
 * =========
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * Description
 * ===========
 * classes for building linked lists
 */

#ifndef __AGGREGATE_H_
#define __AGGREGATE_H_


template<class T> class G3DDLList;
template<class T> class GIterator;


/**An element for DL lists consisting of prev and next pointer and
   the pointer to the  storred element.
   Objects of this class can only be created and accesed by the
   class G3DDLList.
*/
template<class T>
class G3DListElement
{
  friend class G3DDLList<T>;

protected:
  G3DListElement( T* entry ): entry_( entry ) { prev = next = NULL; }

  void setElement( T& entry ) { entry_ = &entry; }; 
  T&   getElement( void ) const { return(*entry_); };
  T*   getPointer( void ) const { return(entry_); };

  G3DListElement<T>* prev;
  G3DListElement<T>* next;
  T* entry_;
};


/**A generic dynamic list storing references, not values. 
   In the constructor an automatic deletion of the referenced elements
   may be requested when they are removed from the list. Default: is off.
   This feature is the main reason for not using a STL list class.
*/
template<class T>
class G3DDLList
{
public:
    /// Initialize double-linked list.
  G3DDLList( bool clear = false ):
    root_(NULL), last_(NULL), clear_(clear), number_(0) {};
  ~G3DDLList(void) { reInitialize(); };

    ///Append an element at the end of the list.
  void append( T* element );
    ///
  void doWithElements( GIterator<T>& ) const;
    ///Return the position of elem in the List. Return -1 if not found.
  int  findElement( T *elem ) const;
    ///Return a Pointer to the element at position pos.
  T*   getElement( int pos );
    ///Remove all elements from the list.
  void reInitialize( void );
    ///Remove the element at the given position.
  void remove( int pos );
    ///Remove element elem, start search at beginning of the list.
  void remove( T *elem );
    /**If mode = true delete elements when they are removed from the list.
       Default is false.*/
  void setDeleteMode( bool mode ) { clear_ = mode; };
    ///TODO do we need this?
  void setElement( int, T& );
    ///Return number of elements in the list.
  int  size(void) const { return(number_); };

  G3DDLList& operator=( const G3DDLList& );
  T&         operator[]( int );

protected:
  void incNumber( void ) { number_++; };
  void decNumber( void ) { number_--; };

protected:
  G3DListElement<T>* root_;
  G3DListElement<T>* last_;
  bool               clear_;
  int                number_;
};


#endif
