/*
 * Project : "MindsEye, 3D library"
 * Copyright 1996, 1997. All Rights Reserved.
 * Author(s): M. Kuball
 *
 * $Id: bounding.h,v 1.4 1998/07/23 20:15:47 source Exp $
 *
 * 
 * COPYRIGHT
 * =========
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * OVERVIEW
 * ========
 * basic attribute classes
 *
 */

#ifndef _BOUNDING_H_
#define _BOUNDING_H_

#include "g3d_math.h"


/**A bounding box.
 */
class G3D_BBox 
{
public:
  G3D_BBox( void ) {};
  virtual ~G3D_BBox() {};

    ///Calculate the geometric center of the box.
  virtual void getCenter( G3DPoint3d& ) const = 0;
    ///
  virtual void getMinMax( G3DPoint3d&, G3DPoint3d& ) const = 0;
    ///
  virtual void getParameter( G3DPoint3d&, G3DVector3d&, G3DVector3d&,
			     G3DVector3d& ) const = 0;
    ///
  virtual void getPosition( G3DPoint3d& ) const = 0;
};


class G3D_BBAligned : public G3D_BBox
{
public:
  G3D_BBAligned( void );

    ///make a box include this and bb
  void addBBox( const G3D_BBAligned &bb , const G3DMatrix& );
    ///
  void addPoint( const TVector<double,3>& );
    ///Calculate the geometric center of the box.
  virtual void getCenter( G3DPoint3d& ) const;
    ///
  virtual void getMinMax( G3DPoint3d&, G3DPoint3d& ) const;
    ///
  virtual void getParameter( G3DPoint3d&, G3DVector3d&, G3DVector3d&,
			     G3DVector3d& ) const;
    ///
  virtual void getPosition( G3DPoint3d& ) const;
    ///Invalidate the box.
  void reset( void );
  bool isValid( void ) const;
    ///
  void setTo( const TVector<double,3> &min, const TVector<double,3> &max );
    ///
  void setTo( const G3DPoint3d&, const G3DVector3d&, const G3DVector3d&,
	      const G3DVector3d& );
  bool testForIntersection( const G3DRay &ray, G3DMatrix &m ) const;

protected:
  G3DPoint3d _min, _max;

};

class G3D_BBFree : public G3D_BBox
{
public:
  G3D_BBFree( const TVector<double,3> &orig, const TVector<double,3> &e1,
	      const TVector<double,3> &e2, const TVector<double,3> &e3 ) :
    _origin(orig),_e1(e1),_e2(e2),_e3(e3) {};

    ///make a bbox containing this and e1,e2,e3 parallel to the x,y,z axes
  //G3D_BBox*    alignToAxis( void ) const;
    ///Calculate the geometric center of the box.
  virtual void getCenter( G3DPoint3d& ) const;
    ///
  virtual void getMinMax( G3DPoint3d&, G3DPoint3d& ) const;
    ///
  virtual void getParameter( G3DPoint3d&, G3DVector3d&, G3DVector3d&,
			     G3DVector3d& ) const;
   ///
  void setParameter( const TVector<double,3>&, const TVector<double,3>&,
		     const TVector<double,3>&, const TVector<double,3>& );
    ///
  virtual void getPosition( G3DPoint3d& ) const;

protected:
  G3DPoint3d  _origin;
  G3DVector3d _e1, _e2, _e3;

};


#endif
