/*
 * Project : "MindsEye, 3D library"
 * Copyright 1996, 1997. All Rights Reserved.
 * Author(s): M. Kuball
 *
 * $Id: color.cpp,v 1.1.1.1 1997/11/22 15:59:12 source Exp $
 *
 * 
 * COPYRIGHT
 * =========
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

#include "color.h"


//const G3DColor gBlack();

G3DColor::G3DColor( int r, int g, int b )
{
  setTo( r, g , b );
}

G3DColor::~G3DColor()
{
}


void
G3DColor::add( const G3DColor &sum )
{
  const float *src = sum._channels;
  float *dst = _channels;

  *(dst++) += *(src++);
  *(dst++) += *(src++);
  *(dst++) += *(src++);
  *dst     += *src;
}


void
G3DColor::mult( float op )
{
  float *dst = _channels;

  *(dst++) *= op;
  *(dst++) *= op;
  *dst     *= op;
}

void
G3DColor::mult( const G3DColor &op )
{
  const float *src = op._channels;
  float *dst = _channels;

  *(dst++) *= *(src++);
  *(dst++) *= *(src++);
  *(dst++) *= *(src++);
  *dst     *= *src;
}

void
G3DColor::setTo( int r, int g, int b )
{
  _channels[0] = r / 255.0;
  _channels[1] = g / 255.0;
  _channels[2] = b / 255.0;
  _channels[3] = 1.0;
}

void
G3DColor::sub( const G3DColor &op )
{
  const float *src = op._channels;
  float *dst = _channels;
  
  *(dst++) -= *(src++);
  *(dst++) -= *(src++);
  *(dst++) -= *(src++);
  *dst     -= *src;
}


G3DColor&
G3DColor::operator=( const G3DColor &col )
{
  const float *src = col._channels;
  float *dst = _channels;
  *(dst++) = *(src++);
  *(dst++) = *(src++);
  *(dst++) = *(src++);
  *dst     = *src;
  return( *this );
}

ostream&
operator<<( ostream& os, const G3DColor &col )
{
  const float *src = col._channels;
  for( int i = 0; i < 3; i++ )
    {
      os << *(src++);
      if( i != 2 )
       os << IOSeparator;
    }
  return( os );
}

istream&
operator>>( istream& is, G3DColor &col )
{
  float *dst = col._channels;
  for( int i = 0; i < 3; i++ )
    is >> *(dst++);
  return( is );
};


//==================================END================================
