/*
 * Project : "MindsEye, 3D library"
 * Copyright 1996, 1997, 1998. All Rights Reserved.
 * Author(s): M. Kuball
 *
 * $Id: color.h,v 1.6 1998/06/22 20:12:21 source Exp $
 *
 * 
 * COPYRIGHT
 * =========
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

#ifndef __COLOR_H_
#define __COLOR_H_



#include <iostream.h>
#include "g3d_math.h"


/**A color has R, G, B and A channels. 
   The intensity of each value (?) is measured in SI units W m^-2 sr^-1.
   Therfore the individual values are not bounded from above. TODO
   Clamping (clipping) is not done before recording the value into an image
   (file or buffer).
 */
class G3DColor
{
  friend ostream&  operator<<( ostream&, const G3DColor& );
  friend istream&  operator>>( istream&, G3DColor& );

public:
  G3DColor( void )
    { _channels[0] = _channels[1] = _channels[2] = _channels[3] = 1.0; };
  G3DColor( int r, int g, int b );
  G3DColor( float r, float g, float b )
    {
      _channels[0] = r; _channels[1] = g; _channels[2] = b;
      _channels[3] = 1.0;
    }
  G3DColor( const G3DColor &rSrc )
    {
      _channels[0] = rSrc._channels[0];
      _channels[1] = rSrc._channels[1];
      _channels[2] = rSrc._channels[2];
      _channels[3] = rSrc._channels[3];
    }
  ~G3DColor();

  void  add( const G3DColor& );
  float getA( void ) const { return _channels[3]; };
  float getB( void ) const { return _channels[2]; };
  float getG( void ) const { return _channels[1]; };
  float getR( void ) const { return _channels[0]; };
  int   blue( void ) const { return (int)(_channels[2] * 255); };
  int   green( void ) const { return (int)(_channels[1] * 255); };
  int   red( void ) const { return (int)(_channels[0] * 255); };

  void  mult( float  );
    ///Component wise multiplication
  void  mult( const G3DColor&  );
  void  setA( float a ) { _channels[3] = a; }
  void  setTo( int r, int g, int b );
  void  sub( const G3DColor& );

  G3DColor& operator=( const G3DColor &col );
  operator const float*() const { return _channels; };

protected:
  float _channels[4];

};


/*
  // interface to the film system
  //static setClip( top, bottom )
  //static setRange( int max ) top - bottom -> max
  //ouput to pixmap ??

    //@ManMemo: Create a new color by multiplication w/ a float
    GColorRGB operator*(const float x) const {
	return(GColorRGB(mR * x, mG * x, mB* x));
    }
    ///
    GColorRGB operator/(const float x) const {
	return(GColorRGB(mR / x, mG / x, mB / x));
    }

    //@ManMemo: Create a new color by multiplication w/ a color
    GColorRGB operator*(const GColorRGB& x) const {
	return(GColorRGB(mR * x.mR, mG * x.mG, mB * x.mB));
    }

    //@ManMemo: Create a new color by addition w/ a color
    GColorRGB operator+(const GColorRGB& c) const { 
	return(GColorRGB( mR + c.mR, mG + c.mG, mB + c.mB)); 
    }


    //@ManMemo: Compute the absolute amount
    float abs(void) const { return(sqrt(mR * mR + mG * mG + mB * mB)); }

    //@ManMemo: Clip each wavelength to [0, 1]
    void correct(void);

};
*/

#endif
