/*
 * Project : "MindsEye, math lib"
 * Copyright (c) 1996, 1997. All Rights Reserved.
 * Author(s): M. Kuball
 *
 * $Id: g3dpoint.h,v 1.2 1998/07/23 20:15:58 source Exp $
 *
 *
 * COPYRIGHT
 * ========= 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * OVERVIEW
 * ========
 * 3 dimensional Vector based on  the "temporary base class idiom".
 * See the article in the german magazine iX issue 11/96 p166.
 *
 */


#ifndef __G3D_POINT_H_
#define __G3D_POINT_H_


#include <stdlib.h>
#include <stdio.h>
#include <iostream.h>
#include <math.h>

#include "tvector.h"
#include "g3dvector.h"


/**A 3 dimensional point.
 */
template <class T>
class G3DPoint3 : public G3DVector3<T>
{
public:
    ///
  G3DPoint3( void ) : G3DVector3<T>() {}
    ///
  G3DPoint3( T a ) : G3DVector3<T>(a) {}
    ///
  G3DPoint3( T x, T y, T z ) : G3DVector3<T>(x,y,z) {}
    ///
  G3DPoint3( const T *vec ) : G3DVector3<T>( vec ) {}
    ///TODO is this a copy constructor?
  G3DPoint3( const TVector<T,3> &tv ) : G3DVector3<T>(tv) {}

  inline G3DPoint3<T>&  operator=( const TVector<T,3> &tv );
  inline G3DPoint3<T>&  operator+=( const TVector<T,3> &tv );
    ///
  inline G3DPoint3<T>&  operator*=( T );

};


//Calculate the min and max points of the encolsing axis aligned bounding
//box. The vmin and vmax must be set to the origin.
//template <class T>
extern void minmax( G3DPoint3<double> &vmin, G3DPoint3<double> &vmax,
		    const G3DVector3<double>&, const G3DVector3<double>&,
		    const G3DVector3<double>& );


//===============Methods for class G3DPoint3===========================


template <class T>
inline G3DPoint3<T>& G3DPoint3<T>::operator=( const TVector<T,3> &tv )
{
        T* dest = _data;
  const T* src  = (const T*)tv;

  *(dest++) = *(src++);
  *(dest++) = *(src++);
  *dest     = *src;

  return( *this );
}


template <class T>
inline G3DPoint3<T>& G3DPoint3<T>::operator+=( const TVector<T,3> &tv )
{
        T* sum1 = _data;
  const T* sum2 = (const T*)tv;

  *(sum1++) += *(sum2++);
  *(sum1++) += *(sum2++);
  *sum1     += *sum2;

  return( *this );
}


template <class T>
inline G3DPoint3<T>& G3DPoint3<T>::operator*=( T mult )
{
  T* dest = _data;

  *(dest++) *= mult;
  *(dest++) *= mult;
  *dest     *= mult;

  return( *this );
}


#endif
