/*
 * Project : "MindsEye, 3D library"
 * Copyright 1996, 1997. All Rights Reserved.
 * Author(s): M. Kuball
 *
 * $Id: object.cpp,v 1.7 1998/07/23 20:15:48 source Exp $
 *
 * 
 * COPYRIGHT
 * =========
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */


#include <string.h>
#include <GL/gl.h>

#include "object.h"

char IOSeparator = ' ';

const G3DColor* G3DObject::selectColor_ = NULL;


void
setIOSeparator( char c )
{
  IOSeparator = c;
}


//-------------------------------------------------------------


G3DObject::G3DObject( void ) :
  scale_(1,1,1)
{
  parent_  = NULL;
  name_    = NULL;
  //count_ = 0;
  state_   = visible;
  type_    = 0;
};


G3DObject::G3DObject( G3DObject *parent ) :
  scale_(1,1,1)
{
  parent_  = parent;
  name_    = NULL;
  //_count   = 0;
  state_   = visible;
  type_    = 0;
};


G3DObject::G3DObject( const G3DObject &orig ) :
  trafo_(orig.trafo_), scale_(orig.scale_), origin_(orig.origin_),
  angles_(orig.angles_)
{
  parent_   = NULL;  //TODO
  name_     = NULL;  //TODO
  //_count    = orig._count;
  state_    = orig.state_;
  type_     = orig.type_;
};


G3DObject::~G3DObject()
{
  if( name_ )  //TODO
    free(name_);
}

void
G3DObject::adjust( void )
{
}

void
G3DObject::applyTransform( void )
{
  trafo_.setToIdentity();
  //TODO
  //reset bounding box
}


G3DObject*
G3DObject::copy( void ) const
{
  return NULL;
}


void
G3DObject::doAction( ActionData &action )
{
  switch( action.type )
    {
    case A_HIDE:
      state_ = G3DObjectState( state_ & (~visible) );  break;
    case A_SHOW:
      state_ = G3DObjectState( state_ | visible);      break;
    case A_SELECT:
      state_ = G3DObjectState( state_ |selected );     break;
    case A_DESELECT:
      state_ = G3DObjectState( state_ & (~selected) ); break;
    case A_GENERIC_ALL:
    case A_GENERIC_SHAPES:
    case A_GENERIC_LIGHTS:
      ((GenericAction&)action).evaluate( this );
      break;
    }
}

const G3DVector3d&
G3DObject::getAngles( void ) const
{
  return( angles_ );
}

const Field*
G3DObject::getFieldList( void ) const
{
  return NULL;
}

void
G3DObject::getFunctionList( void ) const
{
}

void
G3DObject::getLocalTrafo( G3DMatrix &m ) const
{
  m = trafo_;
}

const G3DPoint3d&
G3DObject::getOrigin( void ) const
{
  return( origin_ );
}

uint
G3DObject::getParameter( Param* )
{
  return( 0 );
}

void
G3DObject::getPosition( G3DPoint3d &pos ) const
{
  pos = getOrigin();
}

const G3DVector3d&
G3DObject::getScale( void ) const
{
  return( scale_ );
}

const char*
G3DObject::GetTypeName( void )
{
  return( "Object" );
}

const char*
G3DObject::getTypeName( void ) const
{
  return( G3DObject::GetTypeName() );
}

void
G3DObject::getWorldTrafo( G3DMatrix &m, bool locale ) const
{
  if( parent_ )
    parent_->getWorldTrafo( m );

  if( locale )
    m *= trafo_;
}

/*
double
G3DObject::intersect( G3DRay*, G3DObject** )
{
  //TODO
  return( G3D_INFINITY );
}
*/

void
G3DObject::rebuildTrafo( void )
{
  trafo_.setToIdentity();
  trafo_.translate( origin_ );

  if( !angles_.isZero() )
    {
      G3DMatrix rot;
      rot.rotate( angles_ );
      trafo_ *= rot;
    }

  G3DMatrix sc;
  sc.scale( scale_ );
  trafo_ *= sc;
}

void
G3DObject::selectSubObjects( void )
{
}

void
G3DObject::setAngles( const TVector<double,3> &angles )
{
  angles_ = angles;
  rebuildTrafo();
}

void
G3DObject::setName( const char* name )
{
  name_ = strdup( name );  //TODO make a copy
}

void
G3DObject::setOrigin( const TVector<double,3> &orig )
{
  origin_ = orig;
  rebuildTrafo();
}

void
G3DObject::setParameter( const Param* )
{
}

void
G3DObject::setParent( G3DObject *pt )
{
  /*TODO is this realy useful?
  G3DMatrix mg, mo;
  pt->getWorldTrafo( mg );
  mg.invert4();
  getWorldTrafo( mo );  //w.r.t. the old parent (if any)
  mg *= mo;
  mg.getTranslation( origin_ );
  if( !angles_.isZero() )
    {
      origin_ += anchor_ * (-1);
      G3DMatrix rot( axis_, angles_ );
      G3DPoint3d pc;
      pc = rot * anchor_;
      origin_ += pc;
    }
    */
  parent_ = pt;
}

void
G3DObject::setPosition( const G3DPoint3d& )
{
}

void
G3DObject::setLocalTrafo( const G3DMatrix &m )
{
  trafo_ = m;
}

void
G3DObject::setScale( const TVector<double,3> &scale )
{
  scale_ = scale;
  rebuildTrafo();
}

/*
G3DObject& G3DObject::operator=( const G3DObject &a )
{  
  return *this ;
}
*/

//==============================END===================================
