/*
 * Project : "MindsEye, 3D library"
 * Copyright 1996, 1997. All Rights Reserved.
 * Author(s): M. Kuball
 *
 * $Id: s_glshapes.cpp,v 1.13 1998/07/23 20:15:49 source Exp $
 *
 * 
 * COPYRIGHT
 * =========
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

#include <stdlib.h>
#include <stdio.h>
#include <GL/gl.h>
#include <GL/glu.h>

#include "s_glshapes.h"


G3DS_Polygon::G3DS_Polygon( void )
{
  type_      = T_POLYGON;
  _nVertices = 0;
  _vertices = NULL; //new G3DPoint3d[num];
  resetBoundingBox();
  /*
  _vertices[0].setTo(0,0,-1);
  _vertices[1].setTo(0.8,0,-0.4);
  _vertices[2].setTo(0.8,0,0.4);
  _vertices[3].setTo(0,0,1);
  _vertices[4].setTo(-0.8,0,0.4);
  _vertices[5].setTo(-0.8,0,-0.4);
  */
}

G3DS_Polygon::G3DS_Polygon( const G3DS_Polygon &orig ) :
  G3DShape( orig )
{
  _nVertices = orig._nVertices;
  _vertices  = new G3DPoint3d[_nVertices];
  for( uint i = 0; i < _nVertices; i++ )
    {
      _vertices[i] = orig._vertices[i];
    }
  _normal = orig._normal;
  resetBoundingBox();
}


G3DS_Polygon::~G3DS_Polygon()
{
  //TODO what about the G3DPoint3d destructors?
  delete[] _vertices;
}


G3DObject*
G3DS_Polygon::copy( void ) const
{
  return( new G3DS_Polygon(*this) );
}

Field G3DS_Polygon::FieldList_[3] =
{ {"vertices", m_point3}, {"num_vert", s_number},
  {"normal", s_vector3}
};

const Field*
G3DS_Polygon::getFieldList( void ) const
{
  return( FieldList_ );
}

uint
G3DS_Polygon::getParameter( Param *pList )
{
  pList->point3  = _vertices;
  pList++;
  pList->number  = _nVertices;
  pList++;
  pList->vector3 = &_normal;

  return( 3 );
}

void
G3DS_Polygon::getPosition( G3DPoint3d &p ) const
{
  if( _vertices )
    p = *_vertices;
}

const char*
G3DS_Polygon::GetTypeName( void )
{
  return( "Polygon" );
}

const char*
G3DS_Polygon::getTypeName( void ) const
{
  return( G3DS_Polygon::GetTypeName() );
}

/*
double
G3DS_Polygon::intersect( G3DRay *ray, G3DObject **obj )
{
  //BBox test
  //TODO if level == 0
  //return after BBox test

}
*/

void
G3DS_Polygon::resetBoundingBox( void )
{
  if( _vertices ) //TODO
    {
      G3DPoint3d pmin(*_vertices), pmax(*_vertices);
      G3DPoint3d *vert = _vertices+1;

      for( uint i = 1; i < _nVertices; i++ )
	{
	  pmin.min( *vert );
	  pmax.max( *(vert++) );
	}
      bbox_.setTo( pmin, pmax );
    }
  else
    bbox_.reset();  //TODO
}

void
G3DS_Polygon::setNormal( const TVector<double,3> &normal )
{
  _normal = normal;
  _normal.normalize();
}


void
G3DS_Polygon::setParameter( const Param *plist )
{
  if( plist )
    {
      if( _vertices != plist->point3 )
	{
	  delete[] _vertices;
	  _vertices = plist->point3;
	}
      ++plist;
      _nVertices = plist->number;

      resetBoundingBox();
    }
}

void
G3DS_Polygon::setVertex( uint num, const G3DPoint3d &vertex )
{
  //TODO include bbox calculation?
  if( num < _nVertices )
    {
      _vertices[num] = vertex;
      bbox_.addPoint( vertex );
    }
}

void
G3DS_Polygon::setVertexNumber( uint num )
{
  delete[] _vertices;
  if( num > 0 )
    _vertices  = new G3DPoint3d[num];
  else
    _vertices  = NULL;
  _nVertices = num;
  //TODO resetBoundingBox();
  /* Set at least the 1. vertex ?
  _vertices[0].setTo(0,0,-1);
  _vertices[1].setTo(0.8,0,-0.4);
  _vertices[2].setTo(0.8,0,0.4);
  _vertices[3].setTo(0,0,1);
  _vertices[4].setTo(-0.8,0,0.4);
  _vertices[5].setTo(-0.8,0,-0.4);
  */
}



void
G3DS_Polygon::shade( int ) const
{
  glDisable(GL_CULL_FACE);  //TODO
  glBegin( GL_POLYGON );
  glNormal3dv( (const double*)_normal );
  for( uint i = 0; i < _nVertices; i++ )
    {
      glVertex3dv( (const double*)(_vertices[i]) );
    }
  glEnd();
  glEnable(GL_CULL_FACE);
}

//TODO shadeSubObjects
// shade to vertices


//======================================================================


G3DGLQuadStrip::G3DGLQuadStrip( void )
{
  _vertices  = NULL;
  _normals   = NULL;
  _nVertices = 0;
}


G3DGLQuadStrip::G3DGLQuadStrip( uint, const G3DPoint3d* ) // vertices  )
{
  //TODO
}


G3DGLQuadStrip::G3DGLQuadStrip( const G3DGLQuadStrip &orig )
{
  _nVertices = orig._nVertices;
  _vertices  = new G3DPoint3d[_nVertices];
  _normals   = new G3DVector3d[_nVertices];
}


G3DGLQuadStrip::~G3DGLQuadStrip()
{
  delete[] _vertices;  //TODO call destructors ?
  delete[] _normals;
}


void
G3DGLQuadStrip::setStripeNumber( uint num )
{
  num *= 2 ;
  num += 2;
  delete[] _vertices;
  delete[] _normals;
  if( num > 0 )
    {
      _vertices  = new G3DPoint3d[num];
      _normals   = new G3DVector3d[num];
    }
  else
    {
      _vertices = NULL;
      _normals  = NULL;
    }
  _nVertices = num;
}


void
G3DGLQuadStrip::setVertex( uint num, const G3DPoint3d &vertex )
{
  _vertices[num] = vertex;
}


void
G3DGLQuadStrip::setNormal( uint num, const TVector<double,3> &normal )
{
  _normals[num] = normal;
  _normals[num].normalize();
}


void
G3DGLQuadStrip::shade( void ) const
{
  glBegin( GL_QUAD_STRIP );
  for( uint i = 0; i < _nVertices; i++ )
    {
      glNormal3dv( (const double*)(_normals[i]) );
      // TODO TXTR_COORD(tcx, tcy);
      glVertex3dv( (const double*)(_vertices[i]) );
      //glNormal3dv( (const double*)(_normals[i+1]) );
      //glVertex3dv( (const double*)(_vertices[i+1]) );
    }
  glEnd();
}


//---------------------------------------------------------------


G3DGLTriFan::G3DGLTriFan( uint num )
{
  _vertices  = NULL;
  _normals   = NULL;
  setTriangleNumber( num );
}


G3DGLTriFan::G3DGLTriFan( const G3DGLTriFan &orig )
{
  _nVertices = orig._nVertices;
  _vertices  = new G3DVector3d[_nVertices];
  _normals   = new G3DVector3d[_nVertices/2];
}


G3DGLTriFan::~G3DGLTriFan()
{
  delete[] _vertices;  //TODO call destructors ?
  delete[] _normals;
}


void
G3DGLTriFan::setTriangleNumber( uint num )
{
  num += 2;
  delete[] _vertices;
  delete[] _normals;
  if( num > 0 )
    {
      _vertices  = new G3DVector3d[num];
      _normals   = new G3DVector3d[num/2];
    }
  else
    {
      _vertices = NULL;
      _normals  = NULL;
    }
  _nVertices = num;
}


void
G3DGLTriFan::setVertex( uint num, const G3DPoint3d &vertex )
{
  _vertices[num] = vertex;
}


void
G3DGLTriFan::setNormal( uint num, const TVector<double,3> &normal )
{
  _normals[num] = normal;
  _normals[num].normalize();
}


void
G3DGLTriFan::shade( void ) const
{
  glBegin( GL_TRIANGLE_FAN );
  glVertex3dv( (const double*)(_vertices[0]) );
  for( uint i = 1, j = 0; i < _nVertices; i += 2 )
    {
      glNormal3dv( (const double*)(_normals[j++]) );
      // TODO TXTR_COORD(tcx, tcy);
      glVertex3dv( (const double*)(_vertices[i]) );
      glNormal3dv( (const double*)(_normals[j++]) );
      glVertex3dv( (const double*)(_vertices[i+1]) );
    }
  glEnd();
}



//===============================END===============================
