/*
 * Project : "MindsEye, 3D library"
 * Copyright (c) 1995 - 1997. All Rights Reserved.
 * Author(s): P. Lavoie, M. Kuball
 *
 * $Id: s_line.cpp,v 1.16 1998/07/23 20:15:50 source Exp $
 *
 *
 * COPYRIGHT
 * =========
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *
 * OVERVIEW
 * ========
 *
 */

#include <GL/gl.h>
#include <GL/glu.h>
#include "s_line.h"
#include "s_point.h"


GLUnurbsObj *G3DNurbsCurve::NurbsRenderer_ = NULL;
float        G3DNurbsCurve::tesselTolerance_ = 0.1;


G3DNurbsCpolygon::~G3DNurbsCpolygon()
{
}


void
G3DNurbsCpolygon::shade( void ) const
{
  glBegin(GL_LINE_STRIP);
  //glColor() ;
  for( int i = 0; i < curve.CtrlP.n; ++i )
    glVertex4fv( curve.CtrlP[i].data );
  glEnd() ;
}


//--------------------------------------------------------------


void
G3DNurbsCurve::applyTransform()
{
  if( trafo_ )
    {
      /*
      MatrixRT<double> T( (const double*)trafo_ );
  rx *= M_PI/180.0 ;
  ry *= M_PI/180.0 ;
  rz *= M_PI/180.0 ;

  MatrixRT<float> T(rx,ry,rz,tx,ty,tz) ;
  MatrixRT<float> Sx ; 
  Sx.scale(sx,sy,sz) ;

  transform(T*Sx) ;
  
  tx = ty = tz = rx = ry = rz = 0 ;
  sx = sy = sz = 1 ;
  resetBoundingBox() ;
  */
    }
}


void
G3DNurbsCurve::doAction( ActionData &action )
{
  switch( action.type )
    {
    case A_FIND_SELECTION:
      {
	GLuint n, m;
	if( ((SelectData&)action).peekNextNum(n) )
	  {
	    if( n == selName_cpt )
	      {	
		((SelectData&)action).nextNum();
		((SelectData&)action).getNextNum( m );
		((SelectData&)action).setSelection( new G3DCPoint(P[m]) );
		return;
	      }
	  }
	break;
      }
    case A_SELECT:
      resetKnots();
    }

  G3DNurbs::doAction( action );
}

Field G3DNurbsCurve::FieldList_[4] =
{ {"degree", s_number}, {"knot_vec", m_flt}, {"knot_num", s_number},
  {"cpt_vec", m_vctrlPt}
};

const Field*
G3DNurbsCurve::getFieldList( void ) const
{
  return( FieldList_ );
}


uint
G3DNurbsCurve::getParameter( Param *list )
{
  if( list )
    {
      list->number  = deg;
      list++;
      list->fltList = U.memory();
      list++;
      list->number  = U.n;
      list++;
      list->vCtrlPt = &P;
    }
  return( 4 );
}

const char*
G3DNurbsCurve::GetTypeName( void )
{
  return( "NURBS_Curve" );
}

const char*
G3DNurbsCurve::getTypeName( void ) const
{
  return( G3DNurbsCurve::GetTypeName() );
}

int
G3DNurbsCurve::initClass( void )
{
  tesselTolerance_ = 0.1;
  NurbsRenderer_ = gluNewNurbsRenderer();
  if( NurbsRenderer_ )
    {
      gluNurbsProperty( NurbsRenderer_, (GLenum)GLU_SAMPLING_TOLERANCE, 40.0 );
      return( 1 );
    }
  else
    return( 0 );
}

void
G3DNurbsCurve::modifyPoint( float u, float, float dx, float dy,
			    float dz )
{
  Point3D delta( dx, dy, dz );
  if( u < Knot[0] )
    u = Knot[0];
  if( u > Knot[Knot.n-1] )
    u = Knot[Knot.n-1];
  movePoint( u, delta );
}

void
G3DNurbsCurve::point( float &u, float&, int pSize, const G3DColor& colorP,
		      int ) const
{
  if( u < U[0] )
    u = U[0] ;
  if( u > U[U.n-1] )
    u = U[U.n-1] ;

  glPointSize(pSize) ;
  float color[4];
  color[0] = color[1] = color[2] =  0.0 ;
  color[3] = 1.0 ;
  glMaterialfv( GL_FRONT_AND_BACK, GL_DIFFUSE, color );
  //TODO
  //glMaterialfv( GL_FRONT_AND_BACK, GL_EMISSION, (const float*)colorP );

  glBegin(GL_POINTS);
  glVertex4fv( point4D(u).data );
  glEnd() ;
}

void
G3DNurbsCurve::resetBoundingBox( void )
{
  G3DPoint3d min, max;

  min[0] = extremum( 1, coordX );
  min[1] = extremum( 1, coordY );
  min[2] = extremum( 1, coordZ );
  max[0] = extremum( 0, coordX );
  max[1] = extremum( 0, coordY );
  max[2] = extremum( 0, coordZ );

  bbox_.setTo( min, max );
  if( parent_ )
    ((G3DShape*)parent_)->resetBoundingBox();
}

/*
void G3DNurbsCurve::resetCPoints()
{
  G3DCPoint *tempCP ;

  //TODO set a color attribute to the cpoints group
  cpoints.reset();
  
  if( P.n > 10 )
    cpoints.setJumpSize(int(P.n/10.0)) ;
  else
    cpoints.setJumpSize(int(float(P.n)/2.001 + 1)) ;
    

  for( int i = 0; i < P.n; ++i )
    {
      tempCP = new G3DCPoint( P[i] );
      //tempCP->setObjectColor( cpointColorDefault );
      //tempCP->setCurrentColor(cpointActiveColorDefault) ;
      //if( i == 0 )
      //tempCP->setObjectColor(cpoint0ColorDefault) ;
      cpoints.addShape( tempCP );
    }
}
*/

void G3DNurbsCurve::resetKnots()
{
  G3DKnot *tempCP ;

  //TODO set a color attribute to the knots group
  knots.reInitialize();
  /*
  if(P.n>10)
    knots.setJumpSize(int(P.n/10.0)) ;
  else
    knots.setJumpSize(2) ;
    */

  for( int i = 0; i < U.n; ++i )
    {
      //calcualte the point on the curve corresponding to knot i
      Point3D p = point3D( U[i] );
      // we must represent multiple knots by only 1 point
      // but with a bigger radius (increase radius by 1)
      tempCP = new G3DKnot( p, i, -1 );
      //tempCP->setObjectColor( knots.objectColor );
      //tempCP->setCurrentColor( knots.currentColor );
      //if( i == 0 )
      //tempCP->setObjectColor( cpoint0ColorDefault );
      int psize = 4;
      int j = i;
      while( j < (U.n - 1) )
	{
	  if( U[j+1] <= U[j] )
	    {
	      ++psize ; ++j ;
	    }
	  else
	    j = U.n ;
	}
      tempCP->setPsize( psize );
      knots.append( tempCP );
    }
}

void
G3DNurbsCurve::setParameter( const Param *plist )
{
  if( plist )
    {
      deg = plist->number;            ++plist;
      float *knots = plist->fltList;  ++plist;
      Vector_FLOAT u( knots, plist->number );
      ++plist;
      U  = u;
      delete[] knots;

      P   = *plist->vCtrlPt;
      delete plist->vCtrlPt;

      resetBoundingBox();
    }
}

void
G3DNurbsCurve::shade( int ) const
{
  if( NurbsRenderer_ )
    {
      glLineWidth( 2.0 ); 
      GLenum t = GL_MAP1_VERTEX_4;
      gluBeginCurve( NurbsRenderer_ );
      gluNurbsCurve( NurbsRenderer_, U.n, U.memory(), 4, P.memory()->data,
		     deg+1, t );
      gluEndCurve( NurbsRenderer_ );

    /*
      // This code tests the tesselation code for a Nurbs Curve
    BasicList<Point3D> list ;
    list = tesselate(0.01) ;
    BasicNode<Point3D>* node = (BasicNode<Point3D>*)list.first ;
    glBegin(GL_LINE_STRIP) ;
    glColor() ;
    while(node){
      glVertex3fv(node->data->data) ;
      node = node->next ;
    }    
    glEnd() ;
    */
    }
}

void
G3DNurbsCurve::shadeSubObjects( void ) const
{
  glPushName( selName_cpt );
  glPushName( 0 );
  glPointSize( 5 );
  glColor3f(0.2, 0.2, 0.9);
  for( int i = 0; i < P.n; ++i )
    {
      glLoadName( i );
      glBegin( GL_POINTS );
      glVertex4fv( P[i].data );
      glEnd();
    }
  glPopName(); glPopName();
  //set color knots.shade( 0 );
  ((G3DNurbsCpolygon*)polygon)->shade( );
}

G3DNurbsCurve&
G3DNurbsCurve::operator=(const G3DNurbsCurve &a)
{
  this->PlNurbsCurve::operator=( a );
  this->G3DNurbs::operator=( a );
  return *this;
}

G3DNurbsCurve&
G3DNurbsCurve::operator=(const PlNurbsCurve &a)
{
  this->PlNurbsCurve::operator=( a );
  return *this;
}


//===================================END===================================
