/*
 * Project : "MindsEye, 3D library"
 * Copyright (c) 1995 - 1997. All Rights Reserved.
 * Author(s): P. Lavoie, M. Kuball
 *
 * $Id: s_line.h,v 1.17 1998/07/23 20:15:50 source Exp $
 *

 * COPYRIGHT
 * =========
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *
 * OVERVIEW
 * ========
 *
 */

#ifndef __S_LINE_H_
#define __S_LINE_H_

#include <nurbs.hh>
#include <curve.hh>
#include <GL/glu.h>
#include "s_nurbs.h"


//is this just a Polyline, useful for other things, too ???
//class G3DPolyLine
/**This classrepresents the polygon joining
   all the control points of the NURBS curve.
   Created by: Philippe Lavoie (23 September 1997)
   */
class G3DNurbsCpolygon : public G3DObject
{
public:  
  G3DNurbsCpolygon( PlNurbsCurve& c ) : curve(c) {};
  ~G3DNurbsCpolygon();
  /*    objectColor = cPolygonColorDefault;} */

  void shade( void ) const;

protected:
  PlNurbsCurve& curve;

};


/**This class offers an interface to help design OpenGL 
  applications and use this NURBS library at the same time.

  A nonuniform rational B-spline (NURB) curve is a three-dimensional
  projection of a four-dimensional curve. A NURB curve is specified by its
  order, the number of control points used to define it, the control points
  themselves, and the knots used to define it. Attributes may be applied
  only to the entire NURB curve.

  Created by: Philippe Lavoie (23 September 1997)
*/
class G3DNurbsCurve : public PlNurbsCurve, public G3DNurbs
{
public:
    ///
  G3DNurbsCurve( void ) : PlNurbsCurve(), G3DNurbs()
    {
      type_ = T_NURBS_CURVE ;  polygon = new G3DNurbsCpolygon(*this);
    };
    ///
  G3DNurbsCurve(const PlNurbsCurve& nurb) : PlNurbsCurve(nurb), G3DNurbs()
    {
      type_ = T_NURBS_CURVE ; polygon = new G3DNurbsCpolygon(*this);
      resetBoundingBox();
    };
    ///
  G3DNurbsCurve( const Vector_CtrlPnt4D& P1, const Vector_FLOAT &U1,
		 int degree=3 ): PlNurbsCurve(P1,U1,degree), G3DNurbs()
    {
      type_ = T_NURBS_CURVE ; polygon = new G3DNurbsCpolygon(*this);
      resetBoundingBox();
    };
    ///
  G3DNurbsCurve( const Vector_Point3D& P1, const Vector_FLOAT &W,
		 const Vector_FLOAT &U1, int degree=3) :
    PlNurbsCurve(P1,W,U1,degree), G3DNurbs()
    {
      type_ = T_NURBS_CURVE ;  polygon = new G3DNurbsCpolygon(*this);
      resetBoundingBox();
    };
    ///The copy constructor.
  G3DNurbsCurve( const G3DNurbsCurve& nurb ) :
    PlNurbsCurve((PlNurbsCurve)nurb), G3DNurbs( nurb )
    {
      polygon = new G3DNurbsCpolygon(*this);
      resetBoundingBox();
    };

    ///
  virtual void applyTransform();
    ///
  virtual G3DObject* copy( void ) const { return(new G3DNurbsCurve(*this)); };
    ///
  virtual void doAction( ActionData& );
  virtual const Field* getFieldList( void ) const;
    ///
  virtual uint getParameter( Param *list );
    ///Return a Descriptive name of the shape.
  virtual const char* getTypeName( void ) const;
  static  const char* GetTypeName( void );
   ///
  void modifyPoint(float u, float v, float dx, float dy, float dz);
     /**utility function to draw the point C(u) on the curve.
       Input:   u --> the parametric value
                psize --> the size of the control points
	        colorP --> the color of the control points */
  void point( float &u, float &v, int pSize,
	      const G3DColor& colorP, int ) const;
    ///
  virtual void resetBoundingBox( void );
    ///generate the control point information
  //void resetCPoints();
  void resetKnots();
  void resetPolygon() {};
    ///
  virtual void setParameter( const Param* );
    ///
  virtual void shade( int ) const;
    ///
  virtual void shadeSubObjects( void ) const;

    ///Copies curve a into this curve.
  G3DNurbsCurve& operator=( const G3DNurbsCurve& a );
    ///Replace the nurbs data structure of this with a.
  G3DNurbsCurve& operator=( const PlNurbsCurve& a );

    ///Initialise the GLUnurbsObj.
  static int  initClass( void );

private:
  static GLUnurbsObj *NurbsRenderer_;
  static float       tesselTolerance_;
  static Field       FieldList_[4];

};

#endif
