/*
 * Project : "MindsEye, 3D library"
 * Copyright (c) 1995 - 1997. All Rights Reserved.
 * Author(s): M. Kuball
 *
 * $Id: s_mesh.cpp,v 1.2 1997/12/07 17:04:04 source Exp $
 *
 *
 * COPYRIGHT
 * =========
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *
 * OVERVIEW
 * ========
 * Declaration of a mesh primitive
 */

#include "include/config.h"
#include <GL/gl.h>
#include <GL/glu.h>
#include "s_mesh.h"


G3DMesh::G3DMesh( void ) :
  G3DShape()
{
  nVertices_ = 0;
  nFaces_    = 0;
  nNormals_  = 0;
  vertices_  = NULL;
  faces_     = NULL;
  normals_   = NULL;

  type_ = T_QUADMESH;
}

G3DMesh::G3DMesh( uint nvert, uint nfaces, uint nnormals ) :
  G3DShape()
{
  nVertices_ = nvert;
  nFaces_    = nfaces;
  nNormals_  = nnormals;
  vertices_  = new G3DPoint3d[nvert];
  faces_     = new uint[nfaces];
  normals_   = new G3DVector3d[nnormals];

  type_ = T_QUADMESH;
  /*
  for( int i = 0; i < nvert; i++ )
    {
      point->setTo( vert->x, vert->y, vert->z );
      point++;  vert++;
    }
    */
}

G3DMesh::G3DMesh( const G3DMesh &orig ) :
  G3DShape( orig )
{
}

G3DMesh::~G3DMesh()
{
  delete[] vertices_;
  delete[] faces_;
  delete[] normals_;
}


void
G3DMesh::adjust( void )
{
  //calcSmoothNormals();  //TODO only region arund selected point?
  G3DShape::adjust();
}

G3DObject*
G3DMesh::copy( void ) const
{
  return( new G3DMesh(*this) );
}

//TODO I'm still looking for a cleaner and shorter(?) implementation
void
G3DMesh::calcSmoothNormals( void )
{
  //1st calcualte a normal for each face
  //G3DVector3d *normals = new G3DVector3d[nFaces_];
  normals_ = new G3DVector3d[nFaces_];
  G3DVector3d v1, v2, *norm = normals_;
  uint        *face = faces_;
  for( int i = nFaces_; i > 0; --i )
    {
      v1 = vertices_[*(face+1)] - vertices_[*face];
      v2 = vertices_[*(face+3)] - vertices_[*face];
      *norm = v1.cross(v2);
      norm++;
      face += 4;
    }

  //2nd for each vertex avarage the normals of all adjaced faces.
  /*
  fnormals = new G3DVector3d[nfaces];
  fn       = fnormals;
  vert1 = _vertices;
  vert2 = &(_vertices[_cols]);
  for( uint i = 0; i < _rows - 1; i++ )
    {
      for( uint j = 0; j < _cols-1; j++ )
	{
	  v1 = *vert2 - *vert1;
	  v3 = *(vert1++);
	  v2 = *vert2 - *vert1;
	  v3 = v3 - *vert2;
	  n = v3.cross(v1) + v3.cross(v2) + v1.cross(v2);
	  *(fn++) = n.normalize();

	  v1 = *(vert2++);
	  v1 = v1 - *vert2;
	  v3 = *vert1 - *vert2;
	  n = v2.cross(v3) + v2.cross(v1) + v1.cross(v3);
	  *(fn++) = n.normalize();
	}
    }

  G3DVector3d zero(0,0,0);
  G3DVector3d *n1 = _normals;
  for( uint k = _rows*_cols; k > 0; k--)
    *(n1++) = zero;

  fn = fnormals;
  G3DVector3d *n2;
  n1 = _normals;
  n2 = &(_normals[_cols]);
  for( uint l = nfaces; l > 0; l-=2 )
    {
      *(n1++) += *fn;
      *n2     += *fn;
      *n1     += *(fn++);
      *(n2++) += *fn;
      *n1     += *fn;
      *n2     += *(fn++);
    }
  n1 = _normals;
  for( uint m = _rows*_cols; m > 0; m-- )
    {
      n1->normalize();
      n1++;
    }
    */
}

void
G3DMesh::doAction( ActionData &action )
{
  /*
  if( action.type == A_FIND_SELECTION )
    {
      GLuint n, m;
      if( ((SelectData&)action).peekNextNum(n) )
	{
	  if( n == selName_cpt )
	    {
	      ((SelectData&)action).nextNum();
	      ((SelectData&)action).getNextNum( m );
	      ((SelectData&)action).setSelection( new G3DVertex(_vertices[m],
								this) );
	      return;
	    }
	  else if( n == selName_normal )
	    {
	      ((SelectData&)action).nextNum();
	      ((SelectData&)action).getNextNum( m );
	      ((SelectData&)action).setSelection( new G3DNormal(_vertices[m],
								_normals[m],
								this) );
	      return;
	    }
	}
    }
    */
  G3DShape::doAction( action );
}


Field G3DMesh::MeshFields_[7] =
{ {"vertices", m_point3}, {"array_size", s_number},
  {"num_faces", s_number}, {"faces", m_number}, {"array_size", s_number},
  {"normals", m_vector3}, {"array_size", s_number}
};

const Field*
G3DMesh::getFieldList( void ) const
{
  return( MeshFields_ );
}

uint
G3DMesh::getParameter( Param *pList )
{
  pList->point3  = vertices_;
  pList++;
  pList->number  = nVertices_;
  pList++;
  pList->number  = nFaces_;
  pList++;
  pList->numList = faces_;
  pList++;
  pList->number  = nFaces_ * 4;
  pList++;
  pList->vector3 = normals_;
  pList++;
  pList->number  = nNormals_;

  return( 7 );
}

void
G3DMesh::getPosition( G3DPoint3d &pos ) const
{
  pos = vertices_[0];
}

const char*
G3DMesh::GetTypeName( void )
{
  return( "Mesh" );
}

const char*
G3DMesh::getTypeName( void ) const
{
  return( G3DMesh::GetTypeName() );
}

void
G3DMesh::resetBoundingBox( void )
{
  G3DPoint3d *vert = vertices_;
  G3DPoint3d pmin, pmax;

  if( vert )
    {
      pmin = *vert;
      pmax = *(vert++);
      for( int i = nVertices_-1; i > 0; i-- )
	{
	  pmin.min( *vert );
	  pmax.max( *(vert++) );
	}
      bbox_.setTo( pmin, pmax );
    }
  else
    bbox_.reset();

  if( parent_ )
    ((G3DShape*)parent_)->resetBoundingBox();
}

void
G3DMesh::setParameter( const Param *plist )
{
  if( plist )
    {
      if( vertices_ != plist->point3 )
	{
	  delete[] vertices_;
	  vertices_  = plist->point3;
	  nVertices_ = (plist+1)->number;
	}
      plist += 2;
      nFaces_ = plist->number;  plist++;
      if( faces_ != plist->numList )
	{
	  delete[] faces_;
	  faces_  = plist->numList;
	}
      plist += 2;
      if( (normals_ != plist->vector3) || normals_ == NULL )
	{
	  delete[] normals_;
	  if( plist->vector3 == NULL )
	    {
	      normals_  = new G3DVector3d[nFaces_];
	      nNormals_ = nFaces_;  //TODO
	      calcSmoothNormals();
	    }
	  else
	    {
	      normals_  = plist->vector3;
	      nNormals_ = (plist+1)->number;
	    }
	}
      resetBoundingBox();
    }
}

void
G3DMesh::setVertex( uint r, uint c, TVector<G3Dfloat,3> )
{
  //TODO
}

void
G3DMesh::shade( int ) const
{
  uint       *face = faces_;

  //glDisable(GL_CULL_FACE);  //TODO
  glBegin( GL_QUADS );
  for( int i = nFaces_; i > 0; --i )
    {
	  //glNormal3dv( (double*)(_normals[j++]) );
	  // TODO TXTR_COORD(tcx, tcy);
	  // TODO TXTR_COORD(tcx, tcy);
      glVertex3dv( (const double*)(vertices_[*(face++)]) );
      glVertex3dv( (const double*)(vertices_[*(face++)]) );
      glVertex3dv( (const double*)(vertices_[*(face++)]) );
      glVertex3dv( (const double*)(vertices_[*(face++)]) );
    }
  glEnd();
  //glEnable(GL_CULL_FACE);
}

void
G3DMesh::shadeSubObjects( void ) const
{
  /*
  //G3DVector3d *n1, *n2;
  G3DPoint3d  *vert1;
  uint j;

  glPushName( selName_cpt );
  glPushName( 0 );
  vert1 = _vertices;
  glPointSize( 6 );
  glColor3f(0.2, 0.2, 0.9); //TODO get contrasting color from Texture
  for( uint k = 0; k < _rows*_cols; k++ )
    {
      glLoadName( k );
      glBegin( GL_POINTS );
      glVertex3dv( (const double*)(*(vert1++)) );
      glEnd();
    }
  glPopName(); glPopName();
  */
}


//==================================END=================================
