/*
 * Project : "MindsEye, 3D library"
 * Copyright (c) 1995 - 1997. All Rights Reserved.
 * Author(s): M. Kuball
 *
 * $Id: s_mesh.h,v 1.4 1998/07/23 20:15:51 source Exp $
 *
 *
 * COPYRIGHT
 * =========
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *
 * OVERVIEW
 * ========
 * Declaration of mesh primitives
 */

#ifndef _S_MESH_H_
#define _S_MESH_H_

#include "shape.h"


//Mesh in Generic?

/*
class MeshFaceType
{
protected:
  MeshFaceType( void ) { num = 0; vertices = NULL; };
  ~MeshFaceType() { delete vertices; };

  setTo( const MeshFaceType *orig ) {
    num = orig->num;
    vertices = new uint[num];
    uint *dst = vertices;
    const uint *src = orig.vertices;
    for( int i = num; i > 0; i-- )
      *dst++ = *src++;
  }

  uint num;
  uint *vertices;
};
*/

/**A mesh is an object defined by a collection of vertices, faces, and
  contours. Meshes may be used to model polyhedra, grids, and other faceted
  objects. A mesh may have a boundary. The term contour is used here to refer
  to a polygonal hole contained in a single face of a mesh. A mesh face
  (or contour) is a list of vertices that defines a polygonal facet. A face
  (or contour) need not be planar, and a contour and its surrounding face need
  not be coplanar; however, rendering of a mesh having a nonplanar face or
  contour, or having a contour not coplanar with its surrounding face, may
  lead to unexpected results.

  The specification of a mesh includes an array of vertices and an array of
  faces and contours. The vertices of a mesh are indexed by array position;
  these indices are used to specify the faces and contours of that mesh. Faces
  and contours are also indexed by array position; this index does not
  distinguish between faces and contours. Both of these indices are used in the
  specification of child objects.

  Attributes may be attached separately and selectively to the vertices,
  faces, face edges, and corners of a mesh.
  */
class G3DMesh : public G3DShape
{
public:
  G3DMesh( void );
  G3DMesh( uint nvert , uint nfaces, uint nnormals );
  G3DMesh( const G3DMesh& );
  virtual ~G3DMesh();

    ///
  void               adjust( void );
    ///
  virtual G3DObject* copy( void ) const;
    ///
  virtual void       doAction( ActionData& );
    ///Return a list with the names and types of the object s parameter.
  virtual const Field* getFieldList( void ) const;
    /**Copy pointers to the parameter to the list. The order is:
       _rows, _cols, _vertices, _normals.*/
  virtual uint       getParameter( Param* ); 
    ///Get the position in the local coordinate system.
  virtual void       getPosition( G3DPoint3d& ) const;
    ///Return a Descriptive name of the shape.
  virtual const char* getTypeName( void ) const;
  static  const char* GetTypeName( void );
    ///
  //virtual double     intersect( G3DRay *ray, G3DObject **obj );
    ///
  virtual void       resetBoundingBox( void );
  virtual void       setParameter( const Param *pList );
    ///
  void               setVertex( uint r, uint c, TVector<G3Dfloat,3> );
    ///
  virtual void       shade( int ) const;
    ///
  virtual void       shadeSubObjects( void ) const;

  //void optimize(); This may be a time consuming task

protected:
  void calcSmoothNormals( void );

protected:
  G3DPoint3d  *vertices_;
  G3DVector3d *normals_;

  uint *faces_;
//TODO uint *boundary_; ?? more than 1 ??
  uint nVertices_, nFaces_, nNormals_;

  static Field MeshFields_[7];

};

/*
class G3DTriMesh : public G3DMesh
{
};


class G3DQuadMesh : public G3DMesh
{
protected:
  int  _rows, _columns;
  int  _type;  //smooth or flat
  G3DPoint3d  *_vertices;
  G3DVector3d *_normals;
};
*/



#endif
