/*
 * Project : "MindsEye, 3D library"
 * Copyright (c) 1995 - 1997. All Rights Reserved.
 * Author(s): P. Lavoie, M. Kuball
 *
 * $Id: s_nurbs.h,v 1.13 1998/07/23 20:15:51 source Exp $
 *
 *
 * COPYRIGHT
 * =========
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *
 * OVERVIEW
 * ========
 * Declaration of mesh primitives
 */

#ifndef __S_NURBS_H_
#define __S_NURBS_H_

#include "shape.h"
#include "aggregate.h"

//TODO do we need this?
extern const int PLNURBS_FLAGS_AFFECT_ALL;
extern const int PLNURBS_FLAGS_AFFECT_ACTIVE_ONLY;
extern const int PLNURBS_FLAGS_AFFECT_SELECTED_ONLY;
 // view the bbox even if object is in hidden.
extern const int PLNURBS_FLAGS_VIEW_BBOX;

extern const int PLNURBS_DISPLAY_NORMAL;
extern const int PLNURBS_DISPLAY_ISOCURVES;
extern const int PLNURBS_DISPLAY_SHADED;
extern const int PLNURBS_DISPLAY_HIDDEN_LINES;
extern const int PLNURBS_DISPLAY_TESSELATION;

extern int G3DNurbsDisplayMode;


/**This is the abstract base class for NURBS objects.
   The actual implementation of NURBS for curves and surfaces
   are wrapper.
   Created by: Philippe Lavoie (28 September 1997)
*/
class G3DNurbs : public G3DShape
{
public:
    ///
  G3DNurbs( void );
    ///The copy constructor.
  G3DNurbs( const G3DNurbs& );
  virtual ~G3DNurbs();

    ///
  virtual void modifyPoint( float u, float v, float dx, float dy,
			    float dz ) = 0;
    ///Draw the point on the NURBS specified by u and v.
  virtual void point( float &u, float &v, int pSize,
		      const G3DColor& colorP, int cp_flag ) const = 0 ;

  virtual void resetAll()
    {
      resetBoundingBox(); resetCPoints(); resetPolygon(); resetKnots();
    }
  virtual void resetCPoints() {} ;
  virtual void resetPolygon() {};
  virtual void resetKnots() {} ;

  /*
  void setCPointColor(const Color& a, const Color& b, const Color& c, const Color& d) {
    colorCP=a ; colorCP0=b ; cpoints.selectColor = c ; cpoints.currentColor = d ; }
  */

//TODO why public?
public:
  G3DDLList<G3DObject> knots;

protected:
  G3DObject   *polygon; //TODO no pointer! and a widget??

  //G3DGroup    cpoints;
  //TODO G3DColor    colorCP, colorCP0, colorPolygon, colorKnot ;
};



#endif
