/*
 * Project : "MindsEye, 3D library"
 * Copyright (c) 1995 - 1997. All Rights Reserved.
 * Author(s): P. Lavoie, M. Kuball
 *
 * $Id: s_nurbsurf.cpp,v 1.14 1998/07/23 20:15:51 source Exp $
 *
 *
 * COPYRIGHT
 * =========
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *
 * OVERVIEW
 * ========
 *
 */

#include <GL/gl.h>
#include <GL/glu.h>
#include "s_line.h"
#include "s_nurbsurf.h"
#include "s_point.h"

//GLUnurbsObj *G3DHNurbsSurface::NurbsRenderer_   = NULL;
GLUnurbsObj *G3DNurbsSurface::NurbsRenderer_    = NULL;
//float        G3DHNurbsSurface::tesselTolerance_ = 0.1;
float        G3DNurbsSurface::tesselTolerance_  = 0.1;

/*
void
isoCurves( const PlHNurbsSurface *hs, G3DNurbsCurve &t, G3DNurbsCurve &ta,
	   int lod, const PlVector<float> &ub, const PlVector<float> &vb,
	   PlHNurbsSurface* activePatch );
void
splitAtActiveInU( G3DNurbsCurve &t, G3DNurbsCurve &ta, float u,
		  PlHNurbsSurface *active, int lod );
void
splitAtActiveInV( G3DNurbsCurve &t, G3DNurbsCurve &ta, float v,
		  PlHNurbsSurface *active, int lod );
		  */

//--------------------------------------------------------------------
/*
G3DHNurbsSurface::G3DHNurbsSurface() : 
  PlHNurbsSurface(), G3DNurbs()
{
  type_       = T_NURBS_HSURF;
  polygon     = new G3DNurbsSpolygon(*this);
  lod         = maxLevel() ; 
  activePatch = this ; 
}

G3DHNurbsSurface::G3DHNurbsSurface( const G3DHNurbsSurface& bS ) :
  PlHNurbsSurface(bS), G3DNurbs(bS)
{
  polygon     = new G3DNurbsSpolygon(*this);
  lod         = maxLevel() ; 
  activePatch = this ; 
}

G3DHNurbsSurface::G3DHNurbsSurface( const G3DHNurbsSurface* bS, float us,
				    float ue, float vs, float ve) : 
  PlHNurbsSurface(bS,us,ue,vs,ve), G3DNurbs(*bS)
{
  polygon     = new G3DNurbsSpolygon(*this);
  lod         = maxLevel() ; 
  activePatch = this; 
}

G3DHNurbsSurface::G3DHNurbsSurface( const PlNurbsSurface& nS ) : 
  PlHNurbsSurface(nS), G3DNurbs()
{ 
  type_       = T_NURBS_HSURF;
  polygon     = new G3DNurbsSpolygon(*this);
  lod         = maxLevel() ; 
  activePatch = this ; 
}


void
G3DHNurbsSurface::addLevel()
{
  // go to the last level
  if(activePatch){
     while(activePatch->firstPatch)
       activePatch = (G3DHNurbsSurface*)activePatch->firstPatch ;
     activePatch->addPatch(2) ;
  }
}

void
G3DHNurbsSurface::decreaseLevelOfDetail()
{
  --lod ; 
  if(lod<0) lod=0 ; 
  if(activePatch)
    while(activePatch->level > lod)
      activePatch = (G3DHNurbsSurface*)activePatch->base ;
}


int
G3DHNurbsSurface::initClass( void )
{
  tesselTolerance_ = 0.1;
  NurbsRenderer_   = gluNewNurbsRenderer();
  if( NurbsRenderer_ )
    {
      gluNurbsProperty( NurbsRenderer_, (GLenum)GLU_SAMPLING_TOLERANCE, 25.0 );
      return( 1 );
    }
  else
    return( 0 );
}

void
G3DHNurbsSurface::modifyPoint( float u, float v, float dx, float dy,
			       float dz )
{
}


void
G3DHNurbsSurface::point( float &u, float &v, int pSize,
			 const G3DColor& colorP, int cp_flag ) const
{
  if( u < U[0] )
    u = U[0];
  if( u > U[U.n-1] )
    u = U[U.n-1];
  if( v < V[0] )
    v = V[0];
  if( v > V[V.n-1] )
    v = V[V.n-1];

  glPointSize( pSize );
  glBegin( GL_POINTS );
  float color[4];
  color[0] = color[1] = color[2] =  0.0;
  color[3] = 1.0;
  glMaterialfv( GL_FRONT_AND_BACK, GL_DIFFUSE, color );
  //TODO
  //glMaterialfv( GL_FRONT_AND_BACK, GL_EMISSION, (const float*)colorP ) ;
  glVertex4fv( pointAt(u, v, levelOfDetail()).data );
  glEnd();

  //TODO
  PlNurbsCurveGL t ;
  t.setObjectColor(colorP,colorP,colorP) ;
  //t.setNurbsRenderer(nurbsRenderer) ;
  t.hideBBox() ;
  t.hideCPoints() ;
  t.hideCpolygon() ;
  t.viewNurbs() ;
  isoCurveU(u,t,levelOfDetail()) ;
  t.glObject() ;
  isoCurveV(v,t,levelOfDetail()) ;
  t.glObject() ;

  if(cp_flag){ // display only the CP affected by the point
    //TODO
    int spanU, spanV ;
    activePatch->findSpan(u,v,spanU,spanV) ;
    int i,j,n ;
    
    PlObjectGL *pnt ;
    pnt = (PlObjectGL*)cpoints.first ;
    n = 0 ;
    while(pnt){
      i = n / activePatch->CtrlP.cols ;
      j = n % activePatch->CtrlP.cols ;
      //cerr << i << ", " << j << " : " ;
      if(i<spanU-activePatch->degreeU || i>spanU)
	pnt->hideObject() ;
      else{
	if(j<spanV-activePatch->degreeV || j>spanV)
	  pnt->hideObject() ;
	else
	  pnt->viewObject() ;
      }
      ++n ;
      pnt = pnt->next() ;  
    }
  }
}

void
G3DHNurbsSurface::resetBoundingBox()
{
  G3DPoint3d min, max;

  min[0] = extremum( 1, coordX );
  min[1] = extremum( 1, coordY );
  min[2] = extremum( 1, coordZ );
  max[0] = extremum( 0, coordX );
  max[1] = extremum( 0, coordY );
  max[2] = extremum( 0, coordZ );

  delete bbox_;
  bbox_ = new G3D_BBAligned( min, max );
}

void
G3DHNurbsSurface::resetCPoints()
{
  if(activePatch)
    {
      PlCPointGL *tempCP ;

      cpoints.reset() ;
    cpoints.setJumpSize(activePatch->CtrlP.cols) ;
      
      for(int i=0;i<activePatch->CtrlP.rows;++i)
	for(int j=0;j<activePatch->CtrlP.cols;++j)
	  {
	    int mod ;
	    mod = 1 ;
	    if(i==0 || j==0 
	       || i==activePatch->CtrlP.rows-1 
	       || j==activePatch->CtrlP.cols-1)
	      mod = 0 ;
	mod = 1 ; // if we don't use multi-patches it should be safe
	    tempCP = new PlHCPointGL(activePatch->offset(i,j),i,j,activePatch,mod) ;
	    tempCP->setObjectColor(cpointColorDefault) ;
	    tempCP->setCurrentColor(cpointActiveColorDefault) ;
	    if(j==0 && (i!=activePatch->CtrlP.rows-1))
	      tempCP->setObjectColor(cpoint0ColorDefault) ;
	    cpoints.add(tempCP) ;
	  }
    }
}

void
G3DHNurbsSurface::resetPolygon()
{
  int h ;
  if( polygon )
    {
      h = polygon->getState() ;
      delete polygon ;
    }
  polygon = new G3DNurbsSpolygon(*activePatch) ;
  if( h )
    viewCpolygon() ;
  else
    hideCpolygon() ;
}


void
G3DHNurbsSurface::selectHigherLevel()
{
  if( activePatch )
    {
      if( activePatch->level < levelOfDetail() )
	if( activePatch->firstPatch )
	  activePatch = (G3DHNurbsSurface*)activePatch->firstPatch ;
    }
}

void
G3DHNurbsSurface::selectHighestLevel()
{
  if(activePatch){
    while(activePatch->level < levelOfDetail())
      if(activePatch->firstPatch)
	activePatch = (G3DHNurbsSurface*)activePatch->firstPatch ;
  }
}


void
G3DHNurbsSurface::selectLowerLevel()
{
  if( activePatch )
    {
      if( activePatch->base )
	activePatch = (G3DHNurbsSurface*)activePatch->base;
    }
}


void
G3DHNurbsSurface::selectNextPatch()
{
  if(activePatch){
    if(activePatch->nextPatch)
      activePatch = (G3DHNurbsSurface*)activePatch->nextPatch ;
    else
      if(activePatch->base)
	activePatch = (G3DHNurbsSurface*)activePatch->base->firstPatch ;
  }
}


void
G3DHNurbsSurface::selectPrevPatch()
{
  if(activePatch){
    if(activePatch->prevPatch)
      activePatch = (G3DHNurbsSurface*)activePatch->prevPatch ;
    else
      if(activePatch->base){
	activePatch = (G3DHNurbsSurface*)activePatch->base->lastPatch ;
    }
  }
}


void
G3DHNurbsSurface::shade( int ) const
{
  if( NurbsRenderer_ )
    {
    if(0){ // Well this is not necessary anymore with the new method 
      // of creatint a HNURBS surface, I keep it for now but should
      // delete it soon.

      // create iso curves
      G3DNurbsCurve t ;
      G3DNurbsCurve ta ;
      t.setObjectColor(objectColor,selectColor,currentColor) ;
      t.setNurbsRenderer(nurbsRenderer) ;
      ta.setObjectColor(objectColor,selectColor,currentColor) ;
      ta.setNurbsRenderer(nurbsRenderer) ;
      t.hideBBox() ;
      t.hideCPoints() ;
      t.hideCpolygon() ;
      t.viewNurbs() ;
      ta.hideBBox() ;
      ta.hideCPoints() ;
      ta.hideCpolygon() ;
      ta.viewNurbs() ;
      if(isSelected()) { t.select() ; ta.select() ; }
      if(isActive()) ta.activate() ;

      Vector_FLOAT ub,vb ;
      ub.resize(0) ;
      vb.resize(0) ;
      isoCurves(this,t,ta,lod,ub,vb,activePatch) ;
    }

    G3DHNurbsSurface* dispPatch ;
    dispPatch = activePatch ;
    int level = levelOfDetail() ;
    if(level<0) 
      level = maxLevel() ;
    while(dispPatch->level < level)
      dispPatch = (G3DHNurbsSurface*)dispPatch->firstPatch ;
    
    switch(PlNurbsDisplayMode){
    case PLNURBS_DISPLAY_ISOCURVES:
      {
      // create iso curves
      PlNurbsCurveGL t ;
      t.setObjectColor(objectColor,selectColor,currentColor) ;
      t.setNurbsRenderer(nurbsRenderer) ;
      t.hideBBox() ;
      t.hideCPoints() ;
      t.hideCpolygon() ;
      t.viewNurbs() ;
      if(isActive()) t.activate() ;
      if(isSelected()) t.select() ;

      Vector_FLOAT ub ;
      averagingKnots(dispPatch->U,degU,ub) ;

      int i ;
      for(i=0;i<ub.n;++i){
	dispPatch->isoCurveU(ub[i],t,level) ;
	t.glObject() ;
      }
      averagingKnots(dispPatch->V,degV,ub) ;

      for(i=0;i<ub.n;++i){
	dispPatch->isoCurveV(ub[i],t,level) ;
	t.glObject() ;
      }
    }
      break ;
    case PLNURBS_DISPLAY_SHADED:
    case PLNURBS_DISPLAY_HIDDEN_LINES:
    case PLNURBS_DISPLAY_TESSELATION:
    case PLNURBS_DISPLAY_NORMAL:
    default:
      {
      GLenum t = GL_MAP2_VERTEX_4 ; 
      gluNurbsProperty(nurbsRenderer,GLU_DISPLAY_MODE,GLU_OUTLINE_POLYGON) ;
      gluBeginSurface(nurbsRenderer) ;
      glColor() ;
      gluNurbsSurface(nurbsRenderer, dispPatch->U.n, dispPatch->U.memory(), dispPatch->V.n, dispPatch->V.memory(),4,4*dispPatch->P.rows, dispPatch->P[0]->data, dispPatch->degU+1, dispPatch->degV+1, t) ;
      gluEndSurface(nurbsRenderer) ;
      }      
    }
    }
}
*/


//-------------------------------------------------------------------


void
G3DNurbsSpolygon::shade( void ) const
{
  int i, j;

  //TODO glColor() ;
  for( i = 0; i < surface.CtrlP.rows; ++i )
    {
      glBegin( GL_LINE_STRIP );
      for( j = 0; j < surface.CtrlP.cols; ++j )
	glVertex4fv( surface.CtrlP(i, j).data ) ;
      glEnd() ;
    }
  for( j = 0; j < surface.CtrlP.cols; ++j )
    {
      glBegin( GL_LINE_STRIP );
      for( i = 0; i < surface.CtrlP.rows; ++i )
	glVertex4fv( surface.CtrlP(i, j).data );
      glEnd() ;
    }    
}


//--------------------------------------------------------------------


void
G3DNurbsSurface::applyTransform( void )
{
  /*TODO
  rx *= M_PI/180.0 ;
  ry *= M_PI/180.0 ;
  rz *= M_PI/180.0 ;

  MatrixRT<float> T(rx,ry,rz,tx,ty,tz) ;
  MatrixRT<float> Sx ; 
  Sx.scale(sx,sy,sz) ;

  transform(T*Sx) ;
  
  tx = ty = tz = rx = ry = rz = 0 ;
  sx = sy = sz = 1 ;
  resetBoundingBox() ;
  */
}

void
G3DNurbsSurface::doAction( ActionData &action )
{
  switch( action.type )
    {
    case A_FIND_SELECTION:
      {
	GLuint n, u, v;
	if( ((SelectData&)action).peekNextNum(n) )
	  {
	    if( n == selName_cpt )
	      {	
		((SelectData&)action).nextNum();
		((SelectData&)action).getNextNum( u );
		((SelectData&)action).getNextNum( v );
		((SelectData&)action).setSelection( new G3DCPoint(P(u,v)) );
		return;
	      }
	  }
	break;
      }
    case A_SELECT:
      resetKnots();
    }

  G3DNurbs::doAction( action );
}


Field G3DNurbsSurface::FieldList_[7] =
{ {"u_degree", s_number}, {"v_degree", s_number},
  {"knot_vec_u", m_flt}, {"knot_num_u", s_number},
  {"knot_vec_v", m_flt}, {"knot_num_v", s_number},
  {"ctrl_points", m_mctrlPt}
};

const Field*
G3DNurbsSurface::getFieldList( void ) const
{
  return( FieldList_ );
}

uint
G3DNurbsSurface::getParameter( Param *list )
{
  if( list )
    {
      list->number  = degU;
      list++;
      list->number  = degV;
      list++;
      list->fltList = U.memory();
      list++;
      list->number  = U.n;
      list++;
      list->fltList = V.memory();
      list++;
      list->number  = V.n;
      list++;
      list->mCtrlPt = &P;
    }
  return( 7 );
}

const char*
G3DNurbsSurface::GetTypeName( void )
{
  return( "NURBS_surface" );
}

const char*
G3DNurbsSurface::getTypeName( void ) const
{
  return( G3DNurbsSurface::GetTypeName() );
}

int
G3DNurbsSurface::initClass( void )
{
  tesselTolerance_ = 0.1;
  NurbsRenderer_ = gluNewNurbsRenderer();
  if( NurbsRenderer_ )
    {
      gluNurbsProperty( NurbsRenderer_, (GLenum)GLU_SAMPLING_TOLERANCE, 25.0 );
      return( 1 );
    }
  else
    return( 0 );
}

void
G3DNurbsSurface::modifyPoint( float u, float v, float dx,
			      float dy, float dz )
{
  Point3D delta( dx, dy, dz );
  if( u < KnotU[0] )
    u = KnotU[0];
  if( u > KnotU[KnotU.n-1] )
    u = KnotU[KnotU.n-1];
  if( v < KnotV[0] )
    v = KnotV[0];
  if( v > KnotV[KnotV.n-1] )
    v = KnotV[KnotV.n-1];
  movePoint( u, v, delta );
}


void
G3DNurbsSurface::point( float &u, float &v, int pSize,
			const G3DColor& colorP, int ) const
{
  if( u < U[0] )
    u = U[0];
  if( u > U[U.n-1] )
    u = U[U.n-1];
  if( v < V[0] )
    v = V[0];
  if( v > V[V.n-1] )
    v = V[V.n-1];

  glPointSize( pSize );
  float col[4];
  col[0] = col[1] = col[2] = 0;
  col[3] = 1;
  glMaterialfv( GL_FRONT_AND_BACK, GL_DIFFUSE, col );
  //TODO
  //glMaterialfv(GL_FRONT_AND_BACK,GL_EMISSION, (const float*)colorP);

  glBegin( GL_POINTS );
  glVertex4fv( point4D(u, v).data );
  glEnd();

  /* TODO why?
  PlNurbsCurveGL t;
  t.setObjectColor(colorP,colorP,colorP) ;
  //t.setNurbsRenderer(nurbsRenderer) ;
  t.hideBBox() ;
  t.hideCPoints() ;
  t.hideCpolygon() ;
  t.viewNurbs() ;
  isoCurveU(u,t) ;
  t.glObject() ;
  isoCurveV(v,t) ;
  t.glObject() ;
  */
}

void
G3DNurbsSurface::resetBoundingBox()
{
  G3DPoint3d min, max;

  min[0] = extremum( 1, coordX );
  min[1] = extremum( 1, coordY );
  min[2] = extremum( 1, coordZ );
  max[0] = extremum( 0, coordX );
  max[1] = extremum( 0, coordY );
  max[2] = extremum( 0, coordZ );

  bbox_.setTo( min, max );
  if( parent_ )
    ((G3DShape*)parent_)->resetBoundingBox();
}

/*
void
G3DNurbsSurface::resetCPoints()
{
  G3DCPoint *tempCP ;

  cpoints.reset() ;
  cpoints.setJumpSize(P.cols) ;

  for( int i = 0; i < P.rows; ++i )
    for( int j = 0; j < P.cols; ++j )
      {
	tempCP = new G3DCPoint(P(i,j)) ;
	tempCP->setObjectColor(cpointColorDefault) ;
	tempCP->setCurrentColor(cpointActiveColorDefault) ;
	if( (j == 0) && (i != P.rows-1) )
	  tempCP->setObjectColor(cpoint0ColorDefault) ;
	cpoints.add(tempCP) ;
      }
}
*/

void
G3DNurbsSurface::resetKnots( void )
{
  G3DKnot *tempCP ;

  knots.reInitialize();
  //knots.setJumpSize(P.cols) ;

  PlVector<int> sU, sV;
  sU.resize(U.n);
  sV.resize(V.n);

  int i,j ;
  
  sU[0] = 0 ;
  for( i = 0; i < U.n-1; ++i )
    {
      if( U[i+1] >= U[i] )
	{
	  sU[i+1] = sU[i]+1 ;
	  sU[i] = 0 ;
	}
      else
	sU[i+1] = 1 ;
    }

  sV[0] = 0 ;
  for(i=0;i<V.n-1;++i)
    {
      if(V[i+1]>=V[i])
	{
	  sV[i+1] = sV[i]+1 ;
	  sV[i] = 0 ;
	}
      else
	sV[i+1] = 1 ;
    }

  for( i = 0; i < U.n; ++i )
    for( j = 0; j < V.n; ++j )
      {
	//if(sU[i] && sV[i]){
	Point3D p = point3D(U[i],V[j]);
	// we must represent multiple knots by only 1 point
	// but with a bigger radius (increase radius by 1)
	tempCP = new G3DKnot( p, i, j ) ;
	//TODO tempCP->setObjectColor( knotColorDefault );
	//TODO tempCP->setCurrentColor( knotActiveColorDefault );
	//if( j == 0 && (i != P.rows-1) )
	//tempCP->setObjectColor( cpoint0ColorDefault );
	tempCP->setPsize( 3+maximum(sU[i],sV[j]) );
	knots.append( tempCP );
      }
}

void
G3DNurbsSurface::setParameter( const Param *plist )
{
  if( plist )
    {
      degU = plist->number;           ++plist;
      degV = plist->number;           ++plist;

      float *knots = plist->fltList;  ++plist;
      Vector_FLOAT u( knots, plist->number );
      ++plist;
      U  = u;
      delete[] knots;

      knots = plist->fltList;  ++plist;
      Vector_FLOAT v( knots, plist->number );
      ++plist;
      V  = v;
      delete[] knots;

      P   = *plist->mCtrlPt;
      delete plist->mCtrlPt;

      resetBoundingBox();
    }
}

void
G3DNurbsSurface::shade( int num ) const
{
  if( NurbsRenderer_ )
    {
//TODO get Resolution attribute and set the sampling tolerance
      if( num < 0 )
	gluNurbsProperty( NurbsRenderer_, (GLenum)GLU_AUTO_LOAD_MATRIX,
			  GL_FALSE );
      switch( G3DNurbsDisplayMode )
	{
	case 1: //PLNURBS_DISPLAY_ISOCURVES :
	  {
	    // create iso curves
	    /*
	    G3DNurbsCurve t;
	    //t.setObjectColor(objectColor,selectColor,currentColor) ;
	    //t.setNurbsRenderer(nurbsRenderer) ;
	    //t.hideBBox() ;
	    //t.hideCPoints() ;
	    //t.hideCpolygon() ;
	    //t.viewNurbs() ;
	    //if(isActive()) t.activate() ;
	    //if(isSelected()) t.select() ;
	    
	    PlVector_FLOAT ub ;
	    averagingKnots( U, degU, ub );

	    for( int i = 0; i < ub.n; ++i )
	      {
		isoCurveU( ub[i], t ) ;
		t.shade() ;
	      }
	    averagingKnots(V,degV,ub) ;

	    for( int j = 0; j < ub.n; ++j )
	      {
		isoCurveV( ub[j], t ) ;
		t.shade() ;
	      }
	      */
	    break;
	  }
	case 2: //PLNURBS_DISPLAY_SHADED :  //TODO generate texture coords
	  {
	    glEnable( GL_AUTO_NORMAL );
  	    glFrontFace( GL_CCW );
  	    glDisable( GL_CULL_FACE );
	    GLenum t = GL_MAP2_VERTEX_4;
	    gluNurbsProperty( NurbsRenderer_, (GLenum)GLU_DISPLAY_MODE,
			      (GLenum)GLU_FILL );
	    gluBeginSurface( NurbsRenderer_ );
	    gluNurbsSurface( NurbsRenderer_, U.n, U.memory(), V.n, V.memory(),
			     4, 4*P.rows, P[0]->data, degU+1, degV+1, t );
	    gluEndSurface( NurbsRenderer_ );
            glEnable( GL_CULL_FACE );
  	    glFrontFace( GL_CW );
	    glDisable( GL_AUTO_NORMAL );
	    break;
	  }
	case 3: //PLNURBS_DISPLAY_HIDDEN_LINES:
	  {
	    /*
	    GLenum t = (GLenum)GL_MAP2_VERTEX_4; 
	    gluNurbsProperty( NurbsRenderer_, GLU_DISPLAY_MODE, GLU_FILL );
	  
	    glDisable( GL_LIGHTING );
	    glColor3f(.7f, .7f, .7f);
	  
	    glEnable(GL_STENCIL_TEST);
	    glStencilFunc(GL_ALWAYS, 0, 0); // clear stencil for this object
	    gluBeginSurface( NurbsRenderer_ );
	    gluNurbsSurface( NurbsRenderer_, U.n, U.memory(), V.n, V.memory(),
			     4, 4*P.rows, P[0]->data, degU+1, degV+1, t );
	    gluEndSurface( NurbsRenderer_ );
	    glEnable( GL_LIGHTING );

	    glColorMask(GL_FALSE, GL_FALSE, GL_FALSE, GL_FALSE); // turn off color
	    glDisable( GL_DEPTH_TEST );
	    glPolygonMode(GL_FRONT_AND_BACK, GL_LINE);
	    glStencilFunc(GL_ALWAYS, 1, 1);
	    glStencilOp(GL_KEEP, GL_KEEP, GL_REPLACE);
	  
	    gluBeginSurface( NurbsRenderer_ );
	    //glColor() ;
	    gluNurbsSurface( NurbsRenderer_, U.n, U.memory(), V.n, V.memory(),
			     4, 4*P.rows, P[0]->data, degU+1, degV+1, t );
	    gluEndSurface( NurbsRenderer_ );
	  
	    glStencilFunc(GL_EQUAL, 1 , 1);
	    glEnable(GL_DEPTH_TEST);
	    glDepthFunc(GL_LEQUAL);
	    glPolygonMode(GL_FRONT_AND_BACK, GL_FILL);
	    glColorMask(GL_TRUE, GL_TRUE, GL_TRUE, GL_TRUE);
	  
	    gluBeginSurface( NurbsRenderer_ );
	    //TODO glColor() ;
	    gluNurbsSurface( NurbsRenderer_, U.n, U.memory(), V.n, V.memory(),
			     4, 4*P.rows, P[0]->data, degU+1, degV+1, t );
	    gluEndSurface( NurbsRenderer_ );

	    // clean up state
	    glDisable( GL_STENCIL_TEST );
	    glDepthFunc( GL_LESS );
	    */
	    break;
	  }
	case 4:  //PLNURBS_DISPLAY_TESSELATION:
	  {
	    BasicList<Point3D> pts ;
	    BasicList<int> connect ;
	    if( tesselTolerance_ < 0.001) 
	      tesselTolerance_ = 0.001 ;
	    tesselate( tesselTolerance_, pts,connect );
	    
	    glPolygonMode(GL_FRONT_AND_BACK,GL_LINE);
	    glBegin(GL_TRIANGLES) ;
	    //TODO glColor() ;
	    connect.goToFirst() ;
	    pts.goToFirst() ;
	    for( int i = 0; i < connect.size(); ++i )
	      {
		if( *(connect[i]->data) >= 0)
		  glVertex3fv(pts[*(connect[i]->data)]->data->data );
	      }
	    glEnd() ;
	    break;
	  }
	  //case PLNURBS_DISPLAY_NORMAL:
	default:
	  {
	    GLenum t = (GLenum)GL_MAP2_VERTEX_4; 
	    gluNurbsProperty( NurbsRenderer_, (GLenum)GLU_DISPLAY_MODE,
			      (GLenum)GLU_OUTLINE_POLYGON );
	    gluBeginSurface( NurbsRenderer_ );
	    gluNurbsSurface( NurbsRenderer_, U.n, U.memory(), V.n, V.memory(),
			     4, 4*P.rows, P[0]->data, degU+1, degV+1, t );
	    gluEndSurface( NurbsRenderer_ );
	  }
	}
      if( num < 0 )
	gluNurbsProperty( NurbsRenderer_, (GLenum)GLU_AUTO_LOAD_MATRIX,
			  GL_TRUE );
    }
}

void
G3DNurbsSurface::shadeSubObjects( void ) const
{
  glPushName( selName_cpt );
  glPushName( 0 );
  glPointSize( 6 );
  glColor3f(0.2, 0.2, 0.9);

  for( int i = 0; i < P.rows; ++i )
    {
      glLoadName( i );
      glPushName( 0 );
      for( int j = 0; j < P.cols; ++j )
	{
	  glLoadName( j );
	  glBegin( GL_POINTS );
	  glVertex4fv( P(i,j).data );
	  glEnd();
	}
      glPopName();
    }
  glPopName(); glPopName();
  //set color knots.shade( 0 );
  ((G3DNurbsSpolygon*)polygon)->shade( );
}


//---------------------------------------------------------------------

/*
void
isoCurves( const PlHNurbsSurface *hs, G3DNurbsCurve &t, G3DNurbsCurve &ta,
	   int lod, const PlVector<float> &ub, const PlVector<float> &vb,
	   PlHNurbsSurface* activePatch )
{
  if( (lod >= 0) && (lod < hs->level) )
    return ;

  PlVector<float> uchild, vchild ;
  averagingKnots(hs->KnotU,hs->degreeU,uchild) ;
  averagingKnots(hs->KnotV,hs->degreeV,vchild) ;
 
  int i,j ;

  for( i = 0; i < uchild.n; ++i )
    {
      // only draw if the parent didn't already draw the curve
      for( j = 0; j < ub.n; ++j )
	if( ub[j] == uchild[i] )
	  break;
      if( j == ub.n )
	{
	  hs->isoCurveU(uchild[i],t,lod) ;
	  splitAtActiveInU(t,ta,uchild[i],activePatch,lod) ;
	}
    }

  for( i = 0; i < vchild.n; ++i )
    {
      for( j = 0; j < vb.n; ++j )
	if(vb[j] == vchild[i] )
	  break;
      if(j==vb.n)
	{
	  hs->isoCurveV( vchild[i], t, lod );
	  splitAtActiveInV(t,ta,vchild[i],activePatch,lod) ;
	}
    }

  if( lod == hs->level )
    return ;

  // Add details for the patches
  PlHNurbsSurface *child;
  child = hs->firstPatch;
  while(child)
    {
      isoCurves( child, t, ta, lod, uchild, vchild, activePatch ) ;
      child = child->nextPatch ;
    }
}


void
splitAtActiveInU( G3DNurbsCurve &t, G3DNurbsCurve &ta, float u,
		  PlHNurbsSurface *active, int lod )
{
  if( lod >= 0 && lod < active->level )
    return ;
  if( u < active->uS || u >active->uE )
    {
      t.shade( 0 );
      return ;
    }
  PlNurbsCurve c ;
  c = t ;
  ta = c ;
  if(active->vS<=c.Knot[0] && active->vE>=c.Knot[c.Knot.n-1]){
    ta.shade( 0 );
    return ;
  }
  if(active->vS>c.Knot[0] && active->vE < c.Knot[c.Knot.n-1]){
    // active curve is in the middle
    c.splitAt(active->vS,t,ta) ;
    t.shade( 0 );
    c = ta ;
    c.splitAt(active->vE,ta,t) ;
    t.shade( 0 );
    ta.shade( 0 );
  }
  else{
    if(active->vS>c.Knot[0]) {
      c.splitAt(active->vS,t,ta) ;
      t.shade( 0 );
      ta.shade( 0 );
    }
    else{
      c.splitAt(active->vE,ta,t) ;
      t.shade( 0 );
      ta.shade( 0 );
    }
  }
}


void
splitAtActiveInV( G3DNurbsCurve &t, G3DNurbsCurve &ta, float v,
		  PlHNurbsSurface *active, int lod )
{
  if(lod >= 0 && lod < active->level)
    return ;
  if(v<active->vS || v >active->vE)
    {
      t.shade( 0 );
      return ;
    }
  PlNurbsCurve c ;
  c = t ;
  ta = c ;
  if(active->uS<=c.Knot[0] && active->uE>=c.Knot[c.Knot.n-1])
    {
      ta.shade( 0 );
      return ;
    }
  if(active->uS>c.Knot[0] && active->uE < c.Knot[c.Knot.n-1])
    {
      // active curve is in the middle
      c.splitAt(active->uS,t,ta) ;
      t.shade( 0 );
      c = ta ;
      c.splitAt(active->uE,ta,t) ;
      t.shade( 0 );
      ta.shade( 0 );
    }
  else
    {
      if(active->uS>c.Knot[0])
	{
	  c.splitAt(active->uS,t,ta) ;
	  t.shade( 0 );
	  ta.shade( 0 );
	}
      else
	{
	  c.splitAt(active->uE,ta,t) ;
	  t.shade( 0 );
	  ta.shade( 0 );
	} 
    }
}
*/

//=================================END=================================
