/*
 * Project : "MindsEye, 3D library"
 * Copyright (c) 1995 - 1997. All Rights Reserved.
 * Author(s): P. Lavoie, M. Kuball
 *
 * $Id: s_nurbsurf.h,v 1.12 1998/07/23 20:15:52 source Exp $
 *
 *
 * COPYRIGHT
 * =========
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *
 * OVERVIEW
 * ========
 *
 */

#ifndef __S_NURBS_SURFACE_H_
#define __S_NURBS_SURFACE_H_

#include <nurbsS.hh>
#include <hnurbsS.hh>
#include <GL/glu.h>
#include "s_nurbs.h"


/**This class offers an interface to help design OpenGL 
   applications and use this NURBS library at the same time.
   Created by: Philippe Lavoie (23 September 1997)
   This Object is not selectable.
*/
class G3DNurbsSpolygon : public G3DObject
{
public:  
  G3DNurbsSpolygon( PlNurbsSurface& s ) : surface(s)
    {
      //objectColor = cPolygonColorDefault;
    };

  void shade( void ) const;

protected:
  PlNurbsSurface& surface;
};


/**This class offers an interface to help design OpenGL 
   applications and use this NURBS library at the same time.
   Created by: Philippe Lavoie (23 September 1997)
*/
class G3DNurbsSurface : public PlNurbsSurface, public G3DNurbs
{
public:
  G3DNurbsSurface( void ) : PlNurbsSurface(), G3DNurbs()
    {
      type_ = T_NURBS_SURFACE; polygon = new G3DNurbsSpolygon(*this);
    };
  G3DNurbsSurface(const PlNurbsSurface& nS) : PlNurbsSurface(nS), G3DNurbs()
    {
      type_ = T_NURBS_SURFACE; polygon = new G3DNurbsSpolygon(*this);
      resetBoundingBox();
    };
  G3DNurbsSurface( int DegU, int DegV, const Vector_FLOAT& Uk,
		   const Vector_FLOAT& Vk, const Matrix_CtrlPnt4D& Cp ) :
    PlNurbsSurface(DegU,DegV,Uk,Vk,Cp), G3DNurbs()
    {
      type_ = T_NURBS_SURFACE; polygon = new G3DNurbsSpolygon(*this);
      resetBoundingBox();
    };
    ///why this constructor?
  G3DNurbsSurface(int DegU, int DegV, Vector_FLOAT& Uk, Vector_FLOAT& Vk,
		   Matrix_Point3D& Cp, Matrix_FLOAT& W) :
    PlNurbsSurface(DegU,DegV,Uk,Vk,Cp,W), G3DNurbs()
    {
      type_ = T_NURBS_SURFACE; polygon = new G3DNurbsSpolygon(*this);
      resetBoundingBox();
    };
  G3DNurbsSurface(const G3DNurbsSurface& nS) :
    PlNurbsSurface((PlNurbsSurface)nS), G3DNurbs(nS)
    {
      type_ = T_NURBS_SURFACE; polygon = new G3DNurbsSpolygon(*this);
      resetBoundingBox();
    };

    ///
  void applyTransform();
    ///
  virtual G3DObject* copy(void) const { return(new G3DNurbsSurface(*this)); };
    ///
  virtual void doAction( ActionData& );
  virtual const Field* getFieldList( void ) const;
    ///
  virtual uint getParameter( Param *list );
    ///Return a Descriptive name of the shape.
  virtual const char* getTypeName( void ) const;
  static  const char* GetTypeName( void );
    ///
  void modifyPoint(float u, float v, float dx, float dy, float dz);
  /**Draw a point on the Surface at S(u,v).
     Input:  u, v   --> the parametric values
	     psize  --> the size of the control points
	     colorP --> the color of the control points*/
  void point( float &u, float &v, int pSize, const G3DColor&, int ) const;
    ///
  virtual void resetBoundingBox( void );
  //void resetCPoints() ;
  void resetPolygon() {}
  void resetKnots() ;
    ///
  virtual void setParameter( const Param* );
    ///
  virtual void shade( int ) const;
    ///
  virtual void shadeSubObjects( void ) const;

   ///Initialise the GLUnurbsObj.
  static  int  initClass( void);

private:
  static GLUnurbsObj *NurbsRenderer_;
  static float       tesselTolerance_;
  static Field       FieldList_[7];

};


/**  **  **  **  **  **  **  **  **  **  **  **  **  **  **  **  **  **  **
        Class: PlHNurbsSurfaceGL --- an interface to help design with OpenGL 
  Description: This class offers an interface to help design OpenGL 
               applications and use this NURBS library at the same time.
   Created by: Philippe Lavoie (23 September 1997)
  Modified by:
 **  **  **  **  **  **  **  **  **  **  **  **  **  **  **  **  **  **  **/
/*
class G3DHNurbsSurface : public PlHNurbsSurface, public G3DNurbs
{
public:  
  G3DHNurbsSurface( void );
  G3DHNurbsSurface( const G3DHNurbsSurface& bS );
   **copy constructor with patch information
     Input: bS --> the object to copy
	    us --> the start of the U parametric patch
	    ue --> the end of the U parametric patch
            vs --> the start of the V parametric patch
            ue --> the end of the V parametric patch *
  G3DHNurbsSurface( const G3DHNurbsSurface* bS, float us, float ue,
		    float vs, float ve);
  G3DHNurbsSurface(const PlNurbsSurface& nS);

    ///Adds a level to the Hierarchical surface.
  void       addLevel();
  G3DObject* copy( void ) const { return( new G3DHNurbsSurface(*this) ); }
  void       decreaseLevelOfDetail();
  void       highestLevelOfDetail() { lod = maxLevel() ; }
  void       increaseLevelOfDetail()
    { ++lod; if(lod>maxLevel()) lod=maxLevel(); }
  int        levelOfDetail() const { return lod; }
  int        maxLevelOfDetail() { return maxLevel() ; }
  void       modifyPoint(float u, float v, float dx, float dy, float dz);
    ///Draw a point on the surface at coordinates (u,v).
  void       point(float &u, float &v, int pSize, const G3DColor&, int) const;
  virtual void resetBoundingBox( void );
    ///
  void       resetCPoints();
  void       resetPolygon();
  void       resetKnots() {}
  void       setLevelOfDetail(int l) { lod = l ; }

  void selectBasePatch() { activePatch = this; };
    /// Activates the patch at a higher level if available.
  void selectHigherLevel();
    /// Activates the patch at a lower level if available.
  void selectLowerLevel();
    ///Activates the next patch at the same level.
  void selectNextPatch();
    ///Activates the previous patch at the same level.
  void selectPrevPatch();

  void         selectHighestLevel() ;
  virtual void shade( int ) const;
  void         updateUpToLOD() { updateLevels(lod) ; }

   ///Initialise the GLUnurbsObj.
  static  int  initClass( void);

protected:
  G3DHNurbsSurface *activePatch;
  int lod;

private:
  static GLUnurbsObj *NurbsRenderer_;
  static float       tesselTolerance_;


};
*/

#endif
