/*
 * Project : "MindsEye, 3D library"
 * Copyright (c) 1995 - 1997. All Rights Reserved.
 * Author(s): P. Lavoie, M. Kuball
 *
 * $Id: s_point.cpp,v 1.8 1998/06/22 20:12:25 source Exp $
 *
 *
 * COPYRIGHT
 * =========
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *
 * OVERVIEW
 * ========
 *
 */

#include "GL/gl.h"
#include "GL/glu.h"

#include "s_point.h"


CtrlPnt4D dummyCPoint ;
int       pSizeDefault = 4; //TODO class variables?



G3DKnot::G3DKnot( const Point3D& p3d, int kU, int kV ) :
  p(p3d), ku(kU), kv(kV)
{
  type_ = T_KNOT;
  psize = pSizeDefault;
}


void
G3DKnot::shade( void ) const
{
  glPointSize( psize ) ;
  glBegin(GL_POINTS) ;
  //glColor();
  glVertex3fv(p.data) ;
  glEnd();
}


//------------------------------------------------------------------


void
G3DCPoint::doAction( ActionData &action )
{
  if( action.type == A_DESELECT )
    delete this;
  else
    G3DObject::doAction( action );
}


void
G3DCPoint::translate( const TVector<double,3> &tv )
{
  CtrlPnt4D tmp( tv[0], tv[1], tv[2], 0 );
  cpoint += tmp;
}



//---------------------------------------------------------------------


void
G3DVertex::doAction( ActionData &action )
{
  if( action.type == A_DESELECT )
    delete this;  //TODO bad design
  else
    G3DObject::doAction( action );
}

const G3DPoint3d&
G3DVertex::getOrigin( void ) const
{
  return( point_ );
}
/*
void
G3DVertex::getPosition( G3DPoint3d &pos ) const
{
  pos = point_;
}
*/
void
G3DVertex::setOrigin( const TVector<double,3> &tv )
{
  point_ = tv;
}


//------------------------------------------------------------------

G3DNormal::G3DNormal( const G3DPoint3d &base, G3DVector3d &nrm,
		      G3DObject *parent ) :
  G3DObject(parent), base_(base), normal_(nrm)
{
  type_ = T_NORMAL;
};

void
G3DNormal::doAction( ActionData &action )
{
  if( action.type == A_DESELECT )
    delete this;
  else
    G3DObject::doAction( action );
}

void
G3DNormal::getPosition( G3DPoint3d &pos ) const
{
  pos = base_ + normal_;
}

void
G3DNormal::translate( const TVector<double,3> &tv )
{
  normal_ += tv;
}


//------------------------------------------------------------------


/*
G3DPoint::~G3DPoint()
{
}


void 
G3DPoint::shade( int ) const
{
  //glPointSize( psize );
  glBegin( GL_POINTS );
  //glColor();
  glVertex3fv( p.data );
  glEnd() ;
}
*/


//====================================END===============================
