/*
 * Project : "MindsEye, 3D library"
 * Copyright (c) 1995 - 1997. All Rights Reserved.
 * Author(s): P. Lavoie, M. Kuball
 *
 * $Id: s_point.h,v 1.11 1998/06/22 20:12:26 source Exp $
 *
 *
 * COPYRIGHT
 * =========
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *
 * OVERVIEW
 * ========
 *
 */

#ifndef _POINT_H_
#define _POINT_H_

#include <specialType.hh>
#include "object.h"


extern int pSizeDefault;


/**A wrapper object used to control a 4d control Point of a NURBS object.
*/
class G3DCPoint : public G3DObject
{
public:  
  G3DCPoint( CtrlPnt4D &cp ) : cpoint(cp)
    {
      type_ = T_CPOINT;
      psize = pSizeDefault;
    };

    ///
  virtual void doAction( ActionData& );
          void setPsize(int s) { psize = s; };
    ///
  virtual void translate( const TVector<double,3>& );

protected:
  CtrlPnt4D  &cpoint ;
  int        psize ;
};



/* TODO
class PlHCPointGL : public G3DCPoint
{
public:
  PlHCPointGL(CtrlPnt4D& off, int i, int j, PlHNurbsSurfaceGL* hs, int mod): offset(off),s(hs) { type = hcpointObject; psize=pSizeDefault ; i0 = i ; j0 = j ; canModify = mod ;} 
  ~PlHCPointGL() { ; } 

  virtual void glObject() const;

  virtual void modify(const CtrlPnt4D& v) ;

protected:
  CtrlPnt4D& offset ;
  PlHNurbsSurfaceGL *s ;
  int i0,j0 ;
  int canModify ;
};
*/


/**This class offers an interface to help design OpenGL 
  applications and use this NURBS library at the same time.
  Created by: Philippe Lavoie (23 September 1997)
*/
/*
class G3DPoint : public G3DObject
{
public:  
  G3DPoint( const Point3D &p3d ) : p(p3d)
    {
      type_ = T_POINT ; psize = pSizeDefault;
    };
  G3DPoint( const G3DPoint &orig ) : G3DObject(orig), p(orig.p)
    {
      psize = orig.psize;
    };
  virtual ~G3DPoint();

    ///
  virtual G3DObject* copy( void ) const { return( new G3DPoint(p) ); };
    ///
  //void  modify( const Point3D &v ) { p += v; };
    ///
  const Point3D& point() const { return p; };
    ///
  void  set( const Point3D &v ) { p = v; };
    ///
  void  setPsize( int s ) { psize = s; };
    ///
  virtual void shade( int ) const;

  float x() const { return p.x ; }
  float y() const { return p.y ; }
  float z() const { return p.z ; }

protected:
  Point3D p;
  int     psize;
};
*/

/**Objects of this class represent the Points on NURBS curves/surfaces
  corresponding to the values of the knot vector(s).
  */
class G3DKnot : public G3DObject
{
public:  
  G3DKnot( const Point3D& p3d, int kU, int kV );

  //TODO G3DObject* copy( void ) const { return( new G3DPoint(p) ); };
  int   knotU() { return ku; };
  int   knotV() { return kv; };
  void  modify(const Point3D& v) { p += v ; }
  const Point3D& point() const { return p; };
  void  set(const Point3D& v) { p = v ; }
  void  setPsize(int s) { psize = s; };
  void shade( void ) const;

  float x() const { return p.x; };
  float y() const { return p.y; };
  float z() const { return p.z; };

protected:
  Point3D  p;
  int      ku, kv;
  int      psize;
};


/**A wrapper object used to control a normal of a 3d object.
 */
class G3DNormal : public G3DObject
{
public:
    ///Create a handle for normal, situated at the point base.
  G3DNormal( const G3DPoint3d &base, G3DVector3d &normal, G3DObject *parent );
    ///
  virtual void doAction( ActionData& );
    ///
  virtual void getPosition( G3DPoint3d& ) const;
    ///
  virtual void translate( const TVector<double,3>& );

protected:
  const G3DPoint3d  &base_;
  G3DVector3d       &normal_;

};


/**A wrapper object used to control a vertex of a 3d object.
 */
class G3DVertex : public G3DObject
{
public:
  G3DVertex( G3DPoint3d &pt, G3DObject *parent ) : G3DObject(parent),point_(pt)
    { type_ = T_VERTEX; };

    ///
  virtual void doAction( ActionData& );
  virtual const G3DPoint3d& getOrigin( void ) const;
    ///
  //virtual void getPosition( G3DPoint3d& ) const;
    ///
  virtual void setOrigin( const TVector<double,3>& );

protected:
  G3DPoint3d &point_;
};


#endif
