/*
 * Project : "MindsEye, 3D library"
 * Copyright 1996, 1997. All Rights Reserved.
 * Author(s): M. Kuball
 *
 * $Id: s_object.h,v 1.11 1998/01/14 20:06:32 source Exp $
 *
 *
 * COPYRIGHT
 * =========
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

#ifndef __REFERENCE_H_
#define __REFERENCE_H_


#include <stdio.h>
#include "g3d_math.h"
#include "attribute.h"
#include "t_fields.h"
#include "s_object.h"

class ActionData;



/**A reference to a shape object.
 */
class G3DReference : public G3DShape
{
public:
    ///
  G3DReference( const G3DShape* );
  virtual ~G3DReference();

    /**Build an OpenGL display List. Empty function.*/
  virtual void compileList( int lnum );
    ///
  virtual G3DObject* copy( void ) const;
    ///
  virtual void  doAction( ActionData& );
    ///Get an Attribute by its id number.
  G3DAttribute* getAttribute( int );
    ///Get an Attribute by its name.
  G3DAttribute* getAttribute( const char* );
    ///
  virtual const Field* getFieldList( void ) const;
    ///Get a list of params describing the geometry of the object.
  virtual uint  getParameter( Param* );
    ///Get the position in the local coordinate system.
  virtual void  getPosition( G3DPoint3d& ) const;
    ///Return a Descriptive name of the shape.
  virtual const char* getTypeName( void );
    ///
  virtual double intersect( G3DRay *ray, G3DObject** );
    ///
  virtual int    numberOfShapes( void );
    ///
  virtual void   resetBoundingBox( void );
    ///
  void           setAttribute( G3DAttribute* );
    ///TODO more general?
  virtual void   setPosition( const G3DPoint3d& );
    /**Display the shape using OGL functions.
       lnum is the number of active lights.*/
  virtual void   shade( int lnum ) const = 0;
    ///
  virtual void   shadeSubObjects( void ) const; //TODO protected?

protected:
  G3DShape    *peer_;  //needed for aquiring the attributes of the parent

};


#endif
