/*
 * Project : "MindsEye, 3D Library"
 * Copyright (c) 1997. All Rights Reserved.
 * Author(s):   Martin Kuball
 *
 * $Id: s_simple.cpp,v 1.18 1998/07/23 20:15:52 source Exp $
 *
 *
 * COPYRIGHT
 * ========= 
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * OVERVIEW
 * implementation of classes for geometric primitives
 */


#include <stdlib.h>
#include <stdio.h>
#include <GL/gl.h>
#include <GL/glu.h>

#include "s_simple.h"


int G3DS_Box::Faces_[6][4] = {
    {4,5,6,7}, {0,1,2,3}, {1,7,6,2},
    {0,3,5,4}, {2,6,5,3}, {0,4,7,1}
  };


G3DS_Box::G3DS_Box( void ) :
  _origin(0,0,0), _e1(0,1,0),
  _e2(0,0,1), _e3(1,0,0)

{
  type_ = T_BOX;
  tesselate();
}

G3DS_Box::G3DS_Box( const G3DVector3d &orig, const G3DVector3d &e1,
		    const G3DVector3d &e2, const G3DVector3d &e3 ) :
  _origin( orig ), _e1( e1 ), _e2( e2 ), _e3( e3 )
{
  type_ = T_BOX;
  tesselate();
}

G3DS_Box::G3DS_Box( const G3DS_Box &orig ) :
  G3DShape( orig )
{
  _origin = orig._origin;
  _e1     = orig._e1;
  _e2     = orig._e2;
  _e3     = orig._e3;
  tesselate();  //TODO copy
}

G3DS_Box::~G3DS_Box()
{
}


G3DObject*
G3DS_Box::copy( void ) const
{
  return( new G3DS_Box(*this) );
}


Field G3DS_Box::FieldList_[4] =
{ {"origin", s_point3}, {"e1", s_vector3}, {"e2", s_vector3},
  {"e2", s_vector3} };

const Field*
G3DS_Box::getFieldList( void ) const
{
  return( FieldList_ );
}


uint
G3DS_Box::getParameter( Param *pList )
{
  if( pList )
    {
      pList->point3  = &_origin;
      pList++;
      pList->vector3 = &_e1;
      pList++;
      pList->vector3 = &_e2;
      pList++;
      pList->vector3 = &_e3;
    }
  return( 4 );
}

void
G3DS_Box::getPosition( G3DPoint3d &pos ) const
{
  pos = _origin;
}

const char*
G3DS_Box::GetTypeName( void )
{
  return( "Box" );
}

const char*
G3DS_Box::getTypeName( void ) const
{
  return( G3DS_Box::GetTypeName() );
}

double
G3DS_Box::intersect( G3DRay *ray, G3DObject **obj )
{
  /*
  double     tmp[8][2];
  G3DMatrix   *mat = ray->getProjection();
  int         i, *signp, signs[8][2];

  //puts( "inter box" );
  //cout << *mat;
  //cout << "dir: " << ray->_direction << endl;

  signp = &(signs[0][0]);
  for( int j = 0; j < 8; j++ )
    {
      //tmp[j] = 
      mat->getxyrot( _points[j], tmp[j][0], tmp[j][1] );
      cout << " -> " << tmp[j][0] << " " << tmp[j][1] << endl;
      *(signp++) = sign( tmp[j][0] );
      *(signp++) = sign( tmp[j][1] );
    }

  signp = &(signs[0][0]);
  for( int k = 0; k < 16; k++ )
    cout << *(signp++) << " ";
  cout << endl;

  for( i = 0; i < 6; i++ )
    {
      if( signs[Faces_[i][0]][0] + signs[Faces_[i][2]][0] == 0 )
	{
	  if( signs[Faces_[i][1]][1] + signs[Faces_[i][3]][1] == 0 )
	    if( axesTest(tmp, Faces_[i]) ) break;
	}
      else if( signs[Faces_[i][0]][1] + signs[Faces_[i][2]][1] == 0 )
	{
	  if( signs[Faces_[i][1]][0] + signs[Faces_[i][3]][0] == 0 )
	    if( axesTest(tmp, Faces_[i]) ) break;
	}
    }
  if( i < 6 )
    {
      double dist =(_norm[i].dot(_points[Faces_[i][0]] - ray->_startpoint)) 
	     / (_norm[i].dot(ray->_direction));
      cout << " box hit in face " << i << " with: " << dist << endl;
      *obj = this;
      return dist;
    }
  else
    return( G3D_INFINITY );
*/
}

void
G3DS_Box::resetBoundingBox( void )
{
  bbox_.setTo( _origin, _e1, _e2, _e3 );
  if( parent_ )
    ((G3DShape*)parent_)->resetBoundingBox();
}

void
G3DS_Box::setParameter( const Param *pList )
{
  if( pList )
    {
      _origin = *(pList->point3);
      ++pList;
      _e1     = *(pList->vector3);
      ++pList;
      _e2     = *(pList->vector3);
      ++pList;
      _e3     = *(pList->vector3);
      tesselate();
    }
}

void
G3DS_Box::setPosition( const G3DPoint3d &pos )
{
  _origin = pos;
  tesselate();
}

void
G3DS_Box::shade( int ) const
{
  const int *facep = &(Faces_[0][0]);

  glBegin( GL_QUADS );
  for( int i = 0; i < 6; i++ )
    {
      //TODO setSurface()
      //glColor3fv( _diffuse->getColor() );
      //glColor3f( 0.0, 1.0, 0.0  );
      glNormal3dv( (const double*)(_norm[i]) );
      glVertex3dv( (const double*)(_points[*(facep++)]) );
      glVertex3dv( (const double*)(_points[*(facep++)]) );
      glVertex3dv( (const double*)(_points[*(facep++)]) );
      glVertex3dv( (const double*)(_points[*(facep++)]) );
    }
  glEnd();
}

void
G3DS_Box::tesselate()
{
  G3DVector3d  t1, t2, t3;

  _points[0] = _origin;
  _points[1] = t1 = _origin + _e2;
  _points[2] = t2 = (_origin + _e2) + _e3;
  _points[3] = t3 = _origin + _e3;
  _points[4] = _origin + _e1;
  _points[5] = t3 + _e1;
  _points[6] = t2 + _e1;
  _points[7] = t1 + _e1;

  _norm[0] = _e2.cross( _e3 );
  _norm[0].normalize();
  _norm[1] = _norm[0] * (-1.0);
  _norm[2] = _e3.cross( _e1 );
  _norm[2].normalize();
  _norm[3] = _norm[2] * (-1.0);
  _norm[4] = _e1.cross( _e2 );
  _norm[4].normalize();
  _norm[5] = _norm[4] * (-1.0);

  resetBoundingBox();
}


//======================================================================

//TODO list of editable points?

G3DS_Cone::G3DS_Cone( void ) :
  _origin(0,0,0), _orientation(0,0,1)
{
  type_ = T_CONE;
  _botr = 0.5;
  _topr = 0.0;
  tesselate();
}

G3DS_Cone::G3DS_Cone( const G3DS_Cone &orig ) :
  G3DShape( orig ),
  _origin(orig._origin), _orientation(orig._orientation)
{
  _botr = orig._botr;
  _topr = orig._topr;
  tesselate();
}

G3DS_Cone::~G3DS_Cone( void )
{
}


G3DObject*
G3DS_Cone::copy( void ) const
{
  return( new G3DS_Cone(*this) );
}

Field G3DS_Cone::FieldList_[4] =
{ {"origin", s_point3}, {"orientation", s_vector3},
  {"r_bottom", s_dbl}, {"r_top", s_dbl} };

const Field*
G3DS_Cone::getFieldList( void ) const
{
  return( FieldList_ );
}

uint
G3DS_Cone::getParameter( Param *pList )
{
  if( pList )
    {
      pList->point3  = &_origin;
      pList++;
      pList->vector3 = &_orientation;
      pList++;
      pList->dbl     = _botr;
      pList++;
      pList->dbl     = _topr;
    }
  return( 4 );
}

void
G3DS_Cone::getPosition( G3DPoint3d &pos ) const
{
  pos = _origin;
}

const char*
G3DS_Cone::GetTypeName( void )
{
  return( "Cone" );
}

const char*
G3DS_Cone::getTypeName( void ) const
{
  return( G3DS_Cone::GetTypeName() );
}

void
G3DS_Cone::resetBoundingBox( void )
{
  G3DPoint3d  orig;
  G3DVector3d ax1, ax2;
  double      rad;

  rad = max(_botr, _topr);
  ax1 = (_orientation.getOrtho()).normalize() * rad;
  ax2 = _orientation.cross( ax1 );
  ax2.normalize();
  ax2 = ax2 * rad;

  orig = (_origin - ax1) - ax2;
  
  bbox_.setTo( orig, ax1*2, ax2*2, _orientation );
  if( parent_ )
    ((G3DShape*)parent_)->resetBoundingBox();
}

void
G3DS_Cone::setParameter( const Param *pList )
{
  if( pList )
    {
      _origin      = *(pList->point3);
      pList++;
      _orientation = *(pList->vector3);
      pList++;
      _botr        = pList->dbl;
      pList++;
      _topr        = pList->dbl;
      tesselate();
    }
}

void
G3DS_Cone::setPosition( const G3DPoint3d &pos )
{
  _origin = pos;
  tesselate();
}

void
G3DS_Cone::shade( int lnum ) const
{
  //glDisable( GL_CULL_FACE );

  _mantle.shade();
  if( _topr > 0 )
    _top.shade( lnum );  //TODO _cupFlag
  _bottom.shade( lnum );

  //glEnable( GL_CULL_FACE );
}

void
G3DS_Cone::tesselate( void )
{
  G3DVector3d point, norm, nup, major, minor;
  //double      ratio = _major.length() / _minor.length();
  double      ratio = _topr / _botr;
  int         j = 0;
  int         slice = 12;

  major = (_orientation.getOrtho()).normalize() * _botr;
  minor = _orientation.cross( major );
  minor.normalize();
  minor *= _botr;

  nup = _orientation;
  nup.normalize();
  nup *= (_botr - _topr) / _orientation.length();

  //TODO slice = getAttribute( RESOLUTION )->getValue();
  _mantle.setStripeNumber( slice );
  if( _topr > 0 )
    _top.setVertexNumber( slice );
  _bottom.setVertexNumber( slice );
  double da = 2 * M_PI / slice;

  for( int i = 0; i <= slice; i++ )
    {
      norm = (major * cos(i*da)) + (minor * sin(i*da));
      point = norm + _origin;
      _mantle.setVertex( j, point );
      _bottom.setVertex( i, point );

      point = (norm * ratio) + _origin + _orientation;
      norm += nup;
      _mantle.setNormal( j++, norm );
      _mantle.setNormal( j, norm );
      _mantle.setVertex( j++, point );
      _top.setVertex( i, point );
    }
  /*TODO remove this.
  point = _origin + major;
  _mantle.setVertex( j, point );
  point = (major * ratio) + _origin + _orientation;
  major += nup;
  _mantle.setNormal( j++, major );
  _mantle.setNormal( j, major );
  _mantle.setVertex( j, point );
  */
  if( _topr > 0 )
    _top.setNormal( _orientation );
  _bottom.setNormal( _orientation * (-1) ); //TODO major.cross(minor)??

  resetBoundingBox();
}



//======================================================================


G3DS_Cylinder::G3DS_Cylinder( double rad, double ht ) :
  _origin(0,0,0), _orientation(ht,0,0),
  _major(0,rad,0), _minor(0,0,rad)
{
  type_ = T_CYLINDER;
  tesselate();
}


G3DS_Cylinder::G3DS_Cylinder( G3DVector3d &orient, G3DVector3d &mj,
			      G3DVector3d &mi, G3DVector3d &orig ) :
   _origin(orig), _orientation(orient), _major(mj), _minor(mi)
{
  type_ = T_CYLINDER;
  tesselate();
}

G3DS_Cylinder::G3DS_Cylinder( const G3DS_Cylinder &orig ) :
  G3DShape( orig )
{
  _origin      = orig._origin;
  _orientation = orig._orientation;
  _major       = orig._major;
  _minor       = orig._minor;
  tesselate();  //TODO copy
}

G3DS_Cylinder::~G3DS_Cylinder()
{
}


G3DObject*
G3DS_Cylinder::copy( void ) const
{
  return( new G3DS_Cylinder(*this) );
}

Field G3DS_Cylinder::FieldList_[4] =
{ {"origin", s_point3}, {"orientation", s_vector3},
  {"major_axes", s_vector3}, {"minor_axes", s_vector3} };

const Field*
G3DS_Cylinder::getFieldList( void ) const
{
  return( FieldList_ );
}

uint
G3DS_Cylinder::getParameter( Param *pList )
{
  if( pList )
    {
      pList->point3  = &_origin;
      pList++;
      pList->vector3 = &_orientation;
      pList++;
      pList->vector3 = &_major;
      pList++;
      pList->vector3 = &_minor;
    }
  return( 4 );
}

void
G3DS_Cylinder::getPosition( G3DPoint3d &pos ) const
{
  pos = _origin;
}

const char*
G3DS_Cylinder::GetTypeName( void )
{
  return( "Cylinder" );
}

const char*
G3DS_Cylinder::getTypeName( void ) const
{
  return( G3DS_Cylinder::GetTypeName() );
}

double
G3DS_Cylinder::intersect( G3DRay*, G3DObject **obj )
{
  /*
  if( bbTest() )
    {
      //make some complicated Calculations
    }
    */
  *obj = this;
  return( 1 );
  //return( -1.0 );
}

void
G3DS_Cylinder::resetBoundingBox( void )
{
  G3DPoint3d orig;
  orig = (_origin - _major) - _minor;

  bbox_.setTo( orig, _major*2, _minor*2, _orientation );
  if( parent_ )
    ((G3DShape*)parent_)->resetBoundingBox();
}

void
G3DS_Cylinder::setParameter( const Param *pList )
{
  if( pList )
    {
      _origin      = *(pList->point3);
      ++pList;
      _orientation = *(pList->vector3);
      ++pList;
      _major       = *(pList->vector3);
      ++pList;
      _minor       = *(pList->vector3);
      tesselate();
    }
}

void
G3DS_Cylinder::setPosition( const G3DPoint3d &pos )
{
  _origin = pos;
  tesselate();
}

void
G3DS_Cylinder::shade( int lnum ) const
{
  //glDisable( GL_CULL_FACE );

  _mantle.shade();
  _top.shade( lnum );  //TODO _cupFlag
  _bottom.shade( lnum );

  //glEnable( GL_CULL_FACE );
}

void
G3DS_Cylinder::tesselate( void )
{
  G3DVector3d point, norm;
  double      ratio = _major.length() / _minor.length();
  int         j = 0;
  int         slice = 12;

  //slice = getAttribute( RESOLUTION )->getValue();
  _mantle.setStripeNumber( slice );
  _top.setVertexNumber( slice );
  _bottom.setVertexNumber( slice );
  double da = 2 * M_PI / slice;

  //TODO _cupFlag
  for( int i = 0; i < slice; i++ )
    {
      point = _origin + ( (_major * cos(i*da)) + (_minor * sin(i*da)) );
      norm = (_major * (cos( i*da )/ratio))
	 + (_minor * (sin( i*da )*ratio));
      _mantle.setNormal( j, norm );
      _mantle.setVertex( j++, point );
      _bottom.setVertex( i, point );

      point += _orientation;
      _mantle.setNormal( j, norm );
      _mantle.setVertex( j++, point );
      _top.setVertex( i, point );

    }
  point = _origin + _major;
  _mantle.setNormal( j, _major );
  _mantle.setVertex( j++, point );
  point += _orientation;
  _mantle.setNormal( j, _major );
  _mantle.setVertex( j, point );

  _top.setNormal( _orientation );
  _bottom.setNormal( _orientation * (-1) );

  resetBoundingBox();
}


//======================================================================


G3DS_Sphere::G3DS_Sphere( double radius ) :
  _center(0,0,0) //, _r1(radius,0,0), _r2(0,radius,0), _r3(0,0,radius)
{
  type_   = T_SPHERE;
  _mantle = NULL;
  _radius = radius;
  tesselate();
};

G3DS_Sphere::G3DS_Sphere( double radius, const G3DPoint3d &center ) :
  _center(center) //, _r1(radius,0,0), _r2(0,radius,0), _r3(0,0,radius)
{
  type_   = T_SPHERE;
  _mantle = NULL;
  _radius = radius;
  tesselate();
};

G3DS_Sphere::G3DS_Sphere( const G3DS_Sphere &orig ) :
  G3DShape(orig)
{
  _center = orig._center;
  _radius = orig._radius;
  _mantle = NULL;
  tesselate();
}

G3DS_Sphere::~G3DS_Sphere()
{
  delete[] _mantle;
}


G3DObject*
G3DS_Sphere::copy( void ) const
{
  return( new G3DS_Sphere(*this) );
}

Field G3DS_Sphere::FieldList_[2] =
{ {"center", s_point3}, {"radius", s_dbl} };

const Field*
G3DS_Sphere::getFieldList( void ) const
{
  return( FieldList_ );
}

uint
G3DS_Sphere::getParameter( Param *pList )
{
  if( pList )
    {
      pList->point3 = &_center;
      pList++;
      pList->dbl    = _radius;
    }
  return( 2 );
}

void
G3DS_Sphere::getPosition( G3DPoint3d &pos ) const
{
  pos = _center;
}

const char*
G3DS_Sphere::GetTypeName( void )
{
  return( "Sphere" );
}

const char*
G3DS_Sphere::getTypeName( void ) const
{
  return( G3DS_Sphere::GetTypeName() );
}

double
G3DS_Sphere::intersect( G3DRay *ray, G3DObject **obj )
{
  G3DVector3d pp;
  double skpp, skpd, skpd2, root;

  pp = _center - ray->startpoint_;
  skpp = pp.dot( pp );
  skpd = pp.dot( ray->direction_ );
  skpd2 = skpd * skpd;

  root = (skpd2 - skpp) + _radius;
  if( root < 0 )
    return( G3D_INFINITY );     //no intersection with the sphere
  else
    {
      *obj = this;
      if( root == 0 )
	return( skpd );
      else
	{
	  root = sqrt( root );
	  if( skpd >= root )
	    return( skpd - root );
          else
	    return( skpd + root );
	}
    }
}

void
G3DS_Sphere::resetBoundingBox()
{
  G3DVector3d max(_radius, _radius, _radius);

  bbox_.setTo( _center - max, _center + max );
  if( parent_ )
    ((G3DShape*)parent_)->resetBoundingBox();
}

void
G3DS_Sphere::setParameter( const Param *pList )
{
  if( pList )
    {
      _center = *(pList->point3);
      ++pList;
      _radius = pList->dbl;
      tesselate();
    }
}

void
G3DS_Sphere::setPosition( const G3DPoint3d &pos )
{
  _center = pos;
  tesselate();
}

void
G3DS_Sphere::shade( int ) const
{
  for( int i = 0; i < _stacks; i++ )
    _mantle[i].shade();
}


void
G3DS_Sphere::tesselate( void )
{
  double x, y, z, nsign = 1.0;  //TODO based on inside/outside Attribute
  double rho, drho, theta, dtheta;
  G3DGLQuadStrip *strip;
  int k, slices = 8;
  G3DVector3d normal;
  G3DPoint3d  point;

  _stacks = 8;
  delete[] _mantle;
  _mantle = new G3DGLQuadStrip[_stacks];
  //if (qobj->Orientation==GLU_INSIDE)
  //nsign = -1.0;
  //else

   drho = M_PI / (double)_stacks;
   dtheta = 2.0 * M_PI / (double)slices;

  // draw intermediate stacks as quad strips
  for( int i = 0; i < _stacks; i++ )
     {
       strip = &(_mantle[i]);
       strip->setStripeNumber( slices );
       rho = i * drho;
       k = 0;
       for( int j = 0; j <= slices; j++ )
	 {
	   theta = ( (j == slices) ? 0.0 : j * dtheta );

	   x = cos(theta) * sin(rho);  //TODO might already be calculated
	   y = -sin(theta) * sin(rho);
	   z = cos(rho);
	   normal.setTo( x, y, z );
	   strip->setNormal( k, normal * nsign );
	   point.setTo( x, y, z );
	   point *= _radius;
	   point += _center;
	   strip->setVertex( k++, point );

	   x = cos(theta) * sin(rho+drho);
	   y = -sin(theta) * sin(rho+drho);
	   z = cos(rho+drho);
	   normal.setTo( x, y, z );
	   strip->setNormal( k, normal * nsign );
	   point.setTo( x, y, z );
	   point *= _radius;
	   point += _center;
	   strip->setVertex( k++, point );
	 }
     }

  resetBoundingBox();
}

/*TODO
void gluSphere( GLUquadricObj *qobj,
                GLdouble radius, GLint slices, GLint stacks )
{
   GLfloat rho, drho, theta, dtheta;
   GLfloat x, y, z;
   GLfloat s, t, ds, dt;
   GLint i, j, imin, imax;
   GLfloat nsign;

   stacks = 12;
   _mantle = new G3DGLQuads[12];
   //if (qobj->Orientation==GLU_INSIDE)
   //nsign = -1.0;
   //else
   nsign = 1.0;

   drho = M_PI / (GLfloat) stacks;
   dtheta = 2.0 * M_PI / (GLfloat) slices;

   // texturing: s goes from 0.0/0.25/0.5/0.75/1.0 at +y/+x/-y/-x/+y axis
   // t goes from -1.0/+1.0 at z = -radius/+radius (linear along longitudes)
   // cannot use triangle fan on texturing (s coord. at top/bottom tip varies)

   if (!qobj->TextureFlag)
     {
       // draw +Z end as a triangle fan
       glBegin( GL_TRIANGLE_FAN );
       glNormal3f( 0.0, 0.0, 1.0 );
       TXTR_COORD(0.5,1.0);
       glVertex3f( 0.0, 0.0, nsign * radius );
       for (j=0;j<=slices;j++)
	 {
	   theta = (j==slices) ? 0.0 : j * dtheta;
	   x = -sin(theta) * sin(drho);
	   y = cos(theta) * sin(drho);
	   z = nsign * cos(drho);
	   if (normals)  glNormal3f( x*nsign, y*nsign, z*nsign );
	   glVertex3f( x*radius, y*radius, z*radius );
	 }
	 glEnd();
     }

   ds = 1.0 / slices;
   dt = 1.0 / stacks;
   t = 1.0;    // because loop now runs from 0
   if (qobj->TextureFlag) {
     imin = 0;
     imax = stacks;
   }
   else {
        imin = 1;
        imax = stacks-1;
      }

   G3DGLQuads &strip;
   // draw intermediate stacks as quad strips
   for( i = imin; i < imax; i++ )
     {
       strip = _mantle[i];
       strip.setStripeNumber( slices );
       rho = i * drho;
       s = 0.0;
       for( j = 0; j < slices; j++ )
	 {
	   theta = (j==slices) ? 0.0 : j * dtheta;
	   x = -sin(theta) * sin(rho);
	   y = cos(theta) * sin(rho);
	   z = nsign * cos(rho);

	   normal.setTo( x, y, z );
	   strip.setNormal( j, normal * nsign );
	   //TXTR_COORD(s,t);
	   point.setTo( x, y, z );
	   point *= radius;
	   point += _center;
	   strip.setVertex( j, point );

	   x = -sin(theta) * sin(rho+drho);
	   y = cos(theta) * sin(rho+drho);
	   z = nsign * cos(rho+drho);
	   glNormal3f( x*nsign, y*nsign, z*nsign );
	   TXTR_COORD(s,t-dt);
	   s += ds;
	   glVertex3f( x*radius, y*radius, z*radius );
	 }
       t -= dt;
     }

   if (!qobj->TextureFlag)
     {
       // draw -Z end as a triangle fan
       glBegin( GL_TRIANGLE_FAN );
       glNormal3f( 0.0, 0.0, -1.0 );
       TXTR_COORD(0.5,0.0);
       glVertex3f( 0.0, 0.0, -radius*nsign );
       rho = M_PI - drho;
       s = 1.0;
       t = dt;
       for( j = slices; j >= 0; j-- )
	 {
	   theta = (j==slices) ? 0.0 : j * dtheta;
	   x = -sin(theta) * sin(rho);
	   y = cos(theta) * sin(rho);
	   z = nsign * cos(rho);
	   glNormal3f( x*nsign, y*nsign, z*nsign );
	   TXTR_COORD(s,t);
	   s -= ds;
	   glVertex3f( x*radius, y*radius, z*radius );
	 }
     }
}
*/


//==================================END==================================
