/*
 * Project : "MindsEye, 3D library"
 * Copyright 1996, 1997. All Rights Reserved.
 * Author(s): M. Kuball
 *
 * $Id: s_torus.h,v 1.4 1998/07/23 20:15:53 source Exp $
 *
 *
 * COPYRIGHT
 * =========
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

#ifndef __S_TORUS_H__
#define __S_TORUS_H__

#include "shape.h"
#include "s_glshapes.h"


/**.A torus is defined by an origin and 2 radii */
class G3DTorus : public G3DShape
{
public:
  G3DTorus( double r1, double r2 );
  G3DTorus( double r1, double r2, const G3DPoint3d &center );
  G3DTorus( const G3DTorus& );
  virtual ~G3DTorus();

    ///
  virtual G3DObject* copy( void ) const;
    ///Return a list with the names and types of the object s parameter.
  virtual const Field* getFieldList( void ) const;
    /**Copy pointers to the geometric data to the paramter list. The order is:
       center, major radius, minor radius.*/
  virtual uint   getParameter( Param* );
    ///
  virtual void   getPosition( G3DPoint3d& ) const;
    ///Return a Descriptive name of the shape.
  virtual const char* getTypeName( void ) const;
  static  const char* GetTypeName( void );
    ///
  virtual double intersect( G3DRay*, G3DObject** );
    ///
  virtual void   resetBoundingBox( void );
    ///
  virtual void   setParameter( const Param* );
    ///
  virtual void   setPosition( const G3DPoint3d& );
    ///
  virtual void   shade( int ) const;

protected:
  void  tesselate( void );

protected:
  G3DGLQuadStrip *_mantle;
  int            _stacks;

  G3DPoint3d _center;
  double     _rad1, _rad2;

  static Field FieldList_[3];

};


#endif
