/*
 * Project : "MindsEye, 3D library"
 * Copyright 1996, 1997. All Rights Reserved.
 * Author(s): M. Kuball
 *
 * $Id: s_trigrid.h,v 1.6 1998/07/23 20:15:53 source Exp $
 *
 *
 * COPYRIGHT
 * =========
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

#ifndef __S_TRIGRID_H__
#define __S_TRIGRID_H__

#include "shape.h"
#include "s_glshapes.h"


///a box object using OpenGL to draw itself on the screen. <s_shapes.h>
/**Instances of this class represent rectangular meshes of triangles.
 */
class G3DTriGrid : public G3DShape
{
public:
  G3DTriGrid( uint, uint );
  G3DTriGrid( const G3DTriGrid& );
  virtual ~G3DTriGrid();

    ///
  void               adjust( void );
    ///
  virtual G3DObject* copy( void ) const;
    ///
  virtual void       doAction( ActionData& );
    ///Return a list with the names and types of the object s parameter.
  virtual const Field* getFieldList( void ) const;
    /**Copy pointers to the parameter to the list. The order is:
       _rows, _cols, _vertices, _normals.*/
  virtual uint       getParameter( Param* ); 
    ///Get the position in the local coordinate system.
  virtual void       getPosition( G3DPoint3d& ) const;
    ///Return a Descriptive name of the shape.
  virtual const char* getTypeName( void ) const;
  static  const char* GetTypeName( void );
    ///
  virtual double     intersect( G3DRay *ray, G3DObject **obj );
    ///
  virtual void       resetBoundingBox( void );
  virtual void       setParameter( const Param *pList );
    ///
  void               setVertex( uint r, uint c, TVector<G3Dfloat,3> );
    ///
  virtual void       shade( int ) const;
    ///
  virtual void       shadeSubObjects( void ) const;

protected:
    ///
  void calcSmoothNormals( void );

protected:
  uint  _rows, _cols;

  G3DPoint3d  *_vertices;
  G3DVector3d *_normals;

  static Field FieldList_[6];
};


#endif
