/*
 * Project : "MindsEye, 3D library"
 * Copyright (c) 1996, 1997. All Rights Reserved.
 * Author(s): M. Kuball
 *
 * $Id: scene.cpp,v 1.10 1998/06/22 20:12:28 source Exp $
 *
 *
 * COPYRIGHT
 * =========
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */


#include "scene.h"
#include "light.h"
#include "Attributes/texture.h"

const float cb = 0.2;


G3DScene::G3DScene( void ) :
  background_(cb, cb, cb)
{
  type_           = T_SCENE;
  selectChildren_ = true;
  attributes_.insert( new G3DAColor ); //TODO set some default value
  addLight( new G3DAmbientLight );
}

G3DScene::G3DScene( const G3DScene &orig ) :
  G3DGroup( orig ), background_(orig.background_)
{
}


G3DObject*
G3DScene::copy( void ) const
{
  return( new G3DScene(*this) );
}

Field G3DScene::SceneFields_[1] =
{ {"background", s_color} };

const Field*
G3DScene::getFieldList( void ) const
{
  return( SceneFields_ );
}

uint
G3DScene::getParameter( Param *pList )
{
  if( pList )
    {
      pList->color  = &background_;
    }
  return( 1 );
}

const char*
G3DScene::GetTypeName( void )
{
  return( "Scene" );
}

const char*
G3DScene::getTypeName( void ) const
{
  return( G3DScene::GetTypeName() );
}
/*
void
G3DScene::setParameter( const Param *pList )
{
}
*/
void
G3DScene::shade( int lnum ) const
{
  glClearColor( background_.getR(), background_.getG(),
		background_.getB(), background_.getA() );
  glClear( (GLbitfield)GL_COLOR_BUFFER_BIT );
  G3DGroup::shade( lnum );
}


//==================================END==================================
