/*
 * Project : "MindsEye, 3D library"
 * Copyright 1996, 1997. All Rights Reserved.
 * Author(s): M. Kuball
 *
 * $Id: selectGroup.cpp,v 1.6 1998/06/22 20:12:29 source Exp $
 *
 *
 * COPYRIGHT
 * =========
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */


#include "include/config.h"

#include "selectGroup.h"
#include "s_group.h"
#include "iterator.h"
#include "Attributes/attribute.h"


class EraseObjectsIterator : public GIterator<G3DObject>
{
public:
  virtual bool exec( G3DObject& t ) {
    G3DGroup *grp = (G3DGroup*)(t.getParent());
    if( grp )
      grp->removeObject( &t );
    return(false);
  }
};

class SetAttributeIterator : public GIterator<G3DObject>
{
public:
  SetAttributeIterator( G3DAttribute *attr ) { attrib_ = attr; first_ = true; }
  virtual bool exec( G3DObject& t ) {
    if( t.getType() & T_SHAPE )
      {
	if( first_ )
	  {
	    ((G3DShape&)t).setAttribute( attrib_ );
	    first_ = false;
	  }
	else
	  ((G3DShape&)t).setAttribute( new G3DAReference(attrib_) );
      }
    return(false);
  }
protected:
  G3DAttribute *attrib_;
  bool         first_;
};



G3DSelectionGroup::G3DSelectionGroup( void )
{
  type_ = T_SEL_GROUP;
}

G3DSelectionGroup::G3DSelectionGroup( const G3DSelectionGroup &orig ) :
  G3DObject( orig )
{
  //TODO
}


void
G3DSelectionGroup::doAction( ActionData &action )
{
  switch( action.type )
    {
    case A_DESELECT:
      sendToAllObjects(action);
      delete this;
      break;
    }
}

void
G3DSelectionGroup::eraseObjects( void )
{
  EraseObjectsIterator it;
  objList_.doWithElements( it );
  objList_.setDeleteMode( true );
  objList_.reInitialize();
  objList_.setDeleteMode( false );
}

void
G3DSelectionGroup::sendToAllObjects( ActionData &action )
{
  SendActionIterator<G3DObject> it(action);
  objList_.doWithElements( it );
}

void
G3DSelectionGroup::setAttribute( G3DAttribute *attrib )
{
  SetAttributeIterator it( attrib );
  objList_.doWithElements( it );
}


//================================END======================================
