/*
 * Project : "MindsEye, 3D library"
 * Copyright 1996, 1997. All Rights Reserved.
 * Author(s): M. Kuball
 *
 * $Id: shape.cpp,v 1.9 1998/06/22 20:12:29 source Exp $
 *
 * 
 * COPYRIGHT
 * =========
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */


#include "include/config.h"

#include <string.h>
#include <GL/gl.h>

#include "shape.h"
#include "iterator.h"
#include "Attributes/texture.h"


/*
//TODO must be dynamic data structurs
static const char* shapeTypes[] =
{ "Sphere", "Torus", "Cone", "Cylinder", "Box",
  "Point", "Disk", "Nthagon", "Blob", "Polygon" };
 
static const char* groupTypes[] =
{ "Group", "Scene", "Union", "Merge", "Intersection",
  "Difference", "Negation" }; 
*/

class ResetIterator : public GIterator<G3DWidget>
{
public:
  virtual bool exec( G3DWidget& t )
    {
      t.reset();
      return( false );
    }
};


//-------------------------------------------------------------


G3DShape::G3DShape( void ) :
  widgets_(true)
{
  listId_  = 0;
};


G3DShape::G3DShape( const G3DShape &orig ) :
  G3DObject(orig), attributes_(orig.attributes_), widgets_(true)
{
  listId_   = 0;
};


G3DShape::~G3DShape()
{
  if( listId_ > 0 )
   glDeleteLists( listId_, 1 );
}

void
G3DShape::addWidget( G3DWidget *widget )
{
  widgets_.append( widget );
}

void
G3DShape::adjust( void )
{
  resetBoundingBox();
  ResetIterator iter;
  widgets_.doWithElements( iter );
}

void
G3DShape::clDisplay( int lnum )
{
  if( state_ & visible )
    {
      int tmp = listId_;
      listId_ = 0;
      display( lnum );
      listId_ = tmp;
    }
}

void
G3DShape::compileList( int lnum )
{
  if( listId_ == 0)
    listId_ = glGenLists(1);
  if( listId_ > 0 )
    {
      glNewList( listId_, GL_COMPILE);
      clDisplay( lnum );
      glEndList();
    }
}

void
G3DShape::deleteList()
{
  if( listId_ > 0 )
    {
      glDeleteLists( listId_, 1 );
      listId_ = 0;
    }
}

void
G3DShape::display( int num ) const
{
  if( state_ & visible )
    {
      if( !(state_ & selected) && (listId_ > 0) )
	glCallList( listId_ );
      else
	{
	  glPushMatrix();
	  glMultMatrixd( (const double*)trafo_ );

	  G3DAListIterator it = attributes_.get( (G3DShape*)parent_,
						 TA_TEXTURE, true );
	  if( *it )
	    (*it)->activate();
	  shade( num ); //TODO add layered texture support

	  if( state_ & selected )
	    {
	      //glDisable( GL_LIGHTING ); TODO ???
	      //glColor4fv( (float*)_selectColor );
	      shadeSubObjects();
	      GShadingIterator<G3DWidget> sit( 0 );
	      widgets_.doWithElements( sit );
	      //glEnable( GL_LIGHTING );
	    }

	  glPopMatrix();
	  //int error = glGetError();
	}
    }
}


void
G3DShape::doAction( ActionData &action )
{
  switch( action.type )
    {
    case A_SELECT:
      //addWidget( new G3DWidget(this) );
      break;
    case A_DESELECT:
      widgets_.reInitialize();
      break;
    case A_FIND_SELECTION:
      {
	GLuint n, t;
	((SelectData&)action).getNextNum(t);
	if( t == selName_widget )
	  {
	    ((SelectData&)action).getNextNum(n);
	    G3DObject *tmp = widgets_.getElement( n );
	    if( tmp )
	      tmp->doAction( action );
	  }
	else if( t == selName_shape )
	  ((SelectData&)action).setSelection( this );
	break;
      }
    }
  G3DObject::doAction( action );
}


G3DAListIterator
G3DShape::getAttribute( t_type type, bool inherit ) const
{
  return( attributes_.get((G3DShape*)parent_, type, inherit) );
}

G3DAListIterator
G3DShape::getAttribute( const char* ) const
{
  return( G3DAListIterator(NULL, this, 0, false) );
}

void
G3DShape::getPosition( G3DPoint3d &pos ) const
{
  bbox_.getPosition( pos );
}

double
G3DShape::intersect( G3DRay*, G3DObject** )
{
  //TODO
  return( G3D_INFINITY );
}


int
G3DShape::numberOfShapes( void )
{
  return( 0 );
}

void
G3DShape::select( void )  //const G3DRay &ray )
{
  if( state_ & visible )
    {
      //bbox_.intersect( ray, m );
      //TODO bounding box test first?
      glPushName( selName_shape );
      glPushMatrix();
      glMultMatrixd( (const double*)trafo_ );
      shade( -1 );
      glPopMatrix();
      glPopName();
    }
}

void
G3DShape::selectSubObjects( void )
{
  G3DMatrix m;
  getWorldTrafo( m );
  glPushMatrix();
  glMultMatrixd( (const double*)m );

  shadeSubObjects();

  glPushName( selName_widget );
  glPushName( 0 );
  GSelectionIterator<G3DWidget> sit;
  widgets_.doWithElements( sit );
  glPopName();  glPopName();

  glPopMatrix();
}


void
G3DShape::setAttribute( G3DAttribute *attrib )
{
  attributes_.insert( attrib );
  //TOTO ?? if( _type & T_Group )
  //doAction( retraverse );
}

void
G3DShape::setParent( G3DObject *pt )
{
  G3DObject::setParent( pt );
  /*
  G3DAListIterator it = attributes_.get( TA_TEXTURE, NULL );
  if( *it == NULL )
    {
      it = attributes_.get( TA_TEXTURE, this );
      attributes_.insert( new G3DAReference(*it) );
    }
    */
}

void
G3DShape::shadeSubObjects( void ) const
{
}


/*
G3DShape& G3DShape::operator=( const G3DShape &a )
{  
  return *this ;
}
*/

//==============================END===================================
