/*
 * Project : "MindsEye, 3D library"
 * Copyright 1996, 1997. All Rights Reserved.
 * Author(s): M. Kuball
 *
 * $Id: shape.h,v 1.8 1998/06/22 20:12:30 source Exp $
 *
 *
 * COPYRIGHT
 * =========
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

#ifndef __SHAPE_H_
#define __SHAPE_H_


#include "Attributes/a-list.h"
#include "object.h"
#include "widget.h"
#include "aggregate.h"
#include "bounding.h"


/**Base class for all 3d Objects in a scene. An object may be
   Display lists are associated with individual shapes, not with
   groups of shapes.
   If a shape is selected, an associated display list is ignored.
 */
class G3DShape : public G3DObject
{
protected:
    ///
  G3DShape( void );
    ///The widget list is not duplicated!
  G3DShape( const G3DShape& );

public:
  virtual ~G3DShape();

  void    addWidget( G3DWidget* );
    ///The object was changed, adjust its widgets.
  virtual void   adjust( void );
    /**Wrapper for display() used when compiling a display list. It
       calls display() if the object is not selected.*/
  void  clDisplay( int lnum );
    /**Build an OpenGL display List. lnum is the number of active lights.
       It's possible to call the function for individual elements in the
       scene graph.*/
  virtual void compileList( int lnum );
    ///Delete the display list.
  void    deleteList( void );
    /**Display the object on the screen. This will call the shade()
       function to issue the necessary OGL commands.*/
  void          display( int ) const;
    ///
  virtual void  doAction( ActionData& );
    ///Get an Attribute by its id number.
  G3DAListIterator getAttribute( t_type, bool inherit=false ) const;
    ///Get an Attribute by its name.
  G3DAListIterator getAttribute( const char* ) const;
    ///
  const G3D_BBAligned& getBoundingBox( void ) const { return( bbox_ ); }
    ///Return the position of the bounding box.
  virtual void    getPosition( G3DPoint3d& ) const;
    ///
  virtual double  intersect( G3DRay *ray, G3DObject** );
    ///
  virtual int     numberOfShapes( void );
    ///
  virtual void    resetBoundingBox( void ) = 0;
    ///
  void            setAttribute( G3DAttribute* );
    ///
  virtual void    setParent( G3DObject *pt );
    ///Selection using the OGL picking mechanism
  virtual void    select( void );
    ///Select subobjects of an object.
  virtual void    selectSubObjects( void );
    /**Display the shape using OGL functions.
       lnum is the number of active lights.*/
  virtual void    shade( int lnum ) const = 0;
    ///
  virtual void    shadeSubObjects( void ) const; //TODO protected?

protected:
  G3DAttributeList attributes_;
  G3D_BBAligned    bbox_;
  int              listId_;

  G3DDLList<G3DWidget> widgets_;  //TODO use STL class?

};


#endif
