/*
 * Project : "MindsEye, 3D library"
 * Copyright 1996, 1997. All Rights Reserved.
 * Author(s): M. van Velsen, M. Kuball
 *
 * $Id: t_objects.h,v 1.12 1998/07/23 20:15:56 source Exp $
 *
 *
 * COPYRIGHT
 * =========
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * OVERVIEW
 * ========
 * If you want to register a new object type, send a mail to the
 * MindsEye team.
 *
 */

#ifndef __T_OBJECTS_H__
#define __T_OBJECTS_H__

typedef unsigned int t_type;


// basic object types
#define T_SHAPE         0x10000
#define T_LIGHT         0x20000
#define T_SOUND         0x40000
#define T_WIDGET        0x80000
#define T_HANDLE        0x100000


//-------------------------------------------------------------------


// shape objects
#define T_GROUP         0x1000
#define T_S_GROUP       (1 + T_GROUP + T_SHAPE)
#define T_SCENE         (2 + T_GROUP + T_SHAPE)

#define T_BOX		(1 + T_SHAPE)
#define T_CONE		(2 + T_SHAPE)
#define T_CYLINDER	(3 + T_SHAPE)
#define T_SPHERE	(4 + T_SHAPE)
#define T_TORUS		(5 + T_SHAPE)

#define T_POINT		(6 + T_SHAPE)
#define	T_DISK		(7 + T_SHAPE)
//#define NTHAGON		27 // may need to be prisim ?
#define T_BLOB		(8 + T_SHAPE)
#define T_POLYGON       (9 + T_SHAPE)
#define T_KNOT          (11 + T_SHAPE)

/* Meshes */
#define T_MESH          0x2000
#define T_NURBS_CURVE   (T_SHAPE + T_MESH + 1)
#define T_NURBS_SURFACE (T_SHAPE + T_MESH + 2)
#define T_NURBS_HSURF   (T_SHAPE + T_MESH + 3)
#define T_TRIGRID       (T_SHAPE + T_MESH + 4)
#define T_ROT_SURFACE   (T_SHAPE + T_MESH + 5)
#define T_QUADMESH      (T_SHAPE + T_MESH + 10)

/* CVS */
#define T_CVS           0x4000
#define T_UNION		(1 + T_CVS + T_SHAPE)
#define T_INTERSECTION	(2 + T_CVS + T_SHAPE)
#define T_DIFFERENCE	(3 + T_CVS + T_SHAPE)
//#define T_MERGE		(4 + T_CVS + T_SHAPE)
//#define T_NEGATION	(5 + T_GROUP + T_SHAPE)

//base type for plugin objects
#define T_EXTERN        0x8000


//-------------------------------------------------------------------


#define T_SCALE_WIDGET   (1 + T_WIDGET)
#define T_COORD_WIDGET   (2 + T_WIDGET)
#define T_P_LIGHT_WIDGET (3 + T_WIDGET)
#define T_ROTATE_WIDGET  (4 + T_WIDGET)
#define T_ORBIT_WIDGET   (5 + T_WIDGET)

//-------------------------------------------------------------------


#define T_VERTEX        (1 + T_HANDLE)
#define T_NORMAL        (2 + T_HANDLE)
#define T_CPOINT        (3 + T_HANDLE)
#define T_SEL_GROUP     1

//-------------------------------------------------------------------


// Types of lights
#define T_AMBIENT_LIGHT  (0 + T_LIGHT)
#define T_DIRECTIONAL_LIGHT (1 + T_LIGHT)
#define T_POINT_LIGHT	 (2 + T_LIGHT)
#define T_SPOT_LIGHT	 (3 + T_LIGHT)
#define T_AREA_LIGHT	 (4 + T_LIGHT)

//-------------------------------------------------------------------


/* Cameras */
#define T_CAMERA        0x100000
#define T_PROJECTION	T_CAMERA


//-------------------------------------------------------------------


// attributes

#define TA_MASK        0xFFFF

#define TA_TEXTURE     0x10000
#define TA_COLOR       (1 + TA_TEXTURE)
#define TA_IMAP        (2 + TA_TEXTURE)
#define TA_PLASTIC     (3 + TA_TEXTURE)
#define TA_METAL       (4 + TA_TEXTURE)



#define TA_ANIMATION   0x20000
#define TA_ANIM_VIS    (1 | TA_ANIMATION)
#define TA_ANIM_POS    (2 | TA_ANIMATION)


#define TA_CONSTRAINT  0x40000

#define TA_MISC        0x80000
#define TA_RESOLUTION  (TA_MISC | 1)
#define TA_URL         (TA_MISC | 2 )

//#define TA_CENTER      (TA_NO_INHERITANCE + 1)
//#define TA_NAME        (1 + T_ATTRIBUTE)


#define TA_LOCAL_ATTRIBUTES (0x10000 | TA_MISC | TA_ANIMATION | TA_CONSTRAINT)

#endif
