/*
 * Project : "MindsEye"
 * Copyright (c) 1998. All Rights Reserved.
 * Author(s):   Martin Kuball
 * 
 * $Id: widget.cpp,v 1.9 1998/06/07 20:12:21 source Exp $
 *
 *
 * COPYRIGHT
 * =========
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */


#include <stdlib.h>
#include <GL/gl.h>

#include "w_orbit.h"
#include "shape.h"
#include "t_actions.h"



G3DOrbitWidget::G3DOrbitWidget( G3DShape *parent ) :
  G3DWidget( parent )
{
  type_      = T_ORBIT_WIDGET;
  selVertex_ = 0;
  reset();
}


void
G3DOrbitWidget::display( int ) const
{
  const G3DPoint3d  *vert;

  glDisable( GL_LIGHTING );
  glColor3f(1.0, 0, 0);
  glLineWidth( 2 );

  vert = _vertices;
  glPushName( 0 );
  glBegin( GL_LINES );
  for( int i = 0; i < 8; i++ )
    {
      glVertex3dv( (const double*)(*vert++) );
    }
  glEnd();

  glPopName();
  glEnable( GL_LIGHTING );
}

void
G3DOrbitWidget::doAction( ActionData &action )
{
  if( action.type == A_SELECT )
    {
    }
  else if( action.type == A_FIND_SELECTION )
    {
      if( ((SelectData&)action).getNextNum(selVertex_) )
	{
cout << "selected" << endl;
	  ((SelectData&)action).setSelection( this );
	}
     }
}

void
G3DOrbitWidget::reset( void )
{
  G3DVector3d    e1, e2, e3;
  G3DPoint3d     center, origin;
  G3DMatrix      wmat;
  const G3D_BBox *bbox;

  bbox = ((G3DShape*)parent_)->getBoundingBox();
  bbox->getParameter( origin, e1, e2, e3 );
  bbox->getCenter( ocenter_ );
  //calculateVertices( origin, e1, e2, e3 );
  anchor_ = parent_->getAnchor();

  G3DPoint3d *v = _vertices;

  *(v++) = anchor_ - e1;
  *(v++) = anchor_ + e1;
  *(v++) = anchor_ - e2;
  *(v++) = anchor_ + e2;
  *(v++) = anchor_ - e3;
  *(v++) = anchor_ + e3;
  *(v++) = ocenter_;
  *v     = anchor_;

  parent_->getWorldTrafo( wmat );
  /*
  double lmin, sum, sf1, sf2, sf3;
  lmin = sum = e1.length();
  sf1  = (wmat * e1).length();
  if( sf1 > 0 )
    sf1 = lmin / sf1;
  else
    sf1 = 1.0;
  sf2  = (wmat * e2).length();
  if( sf2 > 0 )
    sf2 = e2.length() / sf2;
  else
    sf2 = 1.0;
  sf3  = (wmat * e3).length();
  if( sf3 > 0 )
    sf3 = e3.length() / sf3;
  else
    sf3 = 1.0;
  _unScale.setTo( sf1, sf2, sf3 );
  //TODO cout << "scale: " << _unScale << endl;

  lmin = min( lmin, e2.length() );
  sum += e2.length();
  lmin = min( lmin, e3.length() );
  sum += e3.length();
  sum *= (1.0/30.0);
  lmin *= 0.3;
  lmin = min( sum, lmin );

  e1.normalize();
  e2.normalize();
  e3.normalize();
  _rotaxes = e3 * -1;
  e1 *= lmin;
  e2 *= lmin;
  e3 *= lmin;

  G3DPoint3d  *tripts = _triangles;

  (tripts++)->setTo(0,0,0); // = offset;
  *(tripts++) = e1 * -1;
  *(tripts++) = e2;
  *(tripts++) = e3;
  (tripts++)->setTo(0,0,0); // = offset;
  *(tripts++) = e1;
  *(tripts++) = e2;
  *tripts     = e3 * -1;
  */
}

void
G3DOrbitWidget::select( void )
{
  display();
}

const G3DPoint3d&
G3DOrbitWidget::getOrigin( void ) const
{
  return( anchor_ );
}

void
G3DOrbitWidget::setOrigin( const TVector<double,3> &tv )
{
  anchor_ = tv;
  reset();
}

//================================END====================================
