/*
 * Project : "MindsEye"
 * Copyright (c) 1998. All Rights Reserved.
 * Author(s):   Martin Kuball
 * 
 * $Id: w_rotate.cpp,v 1.1 1998/07/22 21:01:54 source Exp $
 *
 *
 * COPYRIGHT
 * =========
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */


#include <stdlib.h>
#include <GL/gl.h>

#include "w_rotate.h"
#include "shape.h"
#include "t_actions.h"

#define CIRCLE_SCALE 0.75


G3DRotateWidget::G3DRotateWidget( G3DShape *parent ) :
  G3DWidget( parent )
{
  type_      = T_ROTATE_WIDGET;
  selVertex_ = 0;
  reset();
}


void
G3DRotateWidget::display( int ) const
{
  const G3DPoint3d  *vert;

  glDisable( GL_LIGHTING );
  glColor3f(1.0, 0, 0);


  glLineWidth( 2 );
  vert = _vertices;
  glBegin( GL_LINES );
  for( int i = 0; i < 6; ++i )
    {
      glVertex3dv( (const double*)(*vert++) );
    }
  glEnd();
  glPushName( 1 );
  vert = circle1_;
  glBegin( GL_LINE_STRIP );
  for( int i = 0; i < 16; ++i )
    {
      glVertex3dv( (const double*)(*vert++) );
    }
  glVertex3dv( (const double*)circle1_[0] );
  glEnd();
  glLoadName( 2 );
  vert = circle2_;
  glBegin( GL_LINE_STRIP );
  for( int i = 0; i < 16; ++i )
    {
      glVertex3dv( (const double*)(*vert++) );
    }
  glVertex3dv( (const double*)circle2_[0] );
  glEnd();
  glLoadName( 3 );
  vert = circle3_;
  glBegin( GL_LINE_STRIP );
  for( int i = 0; i < 16; ++i )
    {
      glVertex3dv( (const double*)(*vert++) );
    }
  glVertex3dv( (const double*)circle3_[0] );
  glEnd();

  glPopName();
  glEnable( GL_LIGHTING );
}

void
G3DRotateWidget::doAction( ActionData &action )
{
  if( action.type == A_SELECT )
    {
    }
  else if( action.type == A_FIND_SELECTION )
    {
      if( ((SelectData&)action).getNextNum(selVertex_) )
	{
	  ((SelectData&)action).setSelection( this );
	}
     }
}

void
G3DRotateWidget::getAxis( G3DVector3d &axis ) const
{
  G3DVector3d    e1, e2, e3;
  G3DPoint3d     origin;
  const G3D_BBAligned &bbox( ((G3DShape*)parent_)->getBoundingBox() );

  bbox.getParameter( origin, e1, e2, e3 );
  switch( selVertex_ )
    {
    case 1:
      axis = e1; break;
    case 2:
      axis = e2; break;
    case 3:
      axis = e3; break;
    }
}

void
G3DRotateWidget::getCircle( G3DVector3d &e1, G3DVector3d &e2 ) const
{
  G3DVector3d    be1, be2, be3;
  G3DPoint3d     origin;
  const G3D_BBAligned &bbox( ((G3DShape*)parent_)->getBoundingBox() );

  bbox.getParameter( origin, be1, be2, be3 );
  be1 *= CIRCLE_SCALE;
  be2 *= CIRCLE_SCALE;
  be3 *= CIRCLE_SCALE;
  switch( selVertex_ )
    {
    case 1:
      e1 = be2; e2 = be3; break;
    case 2:
      e1 = be1; e2 = be3; break;
    case 3:
      e1 = be1; e2 = be2; break;
    }
}

void
G3DRotateWidget::makeCircle( const G3DPoint3d &o, const G3DVector3d &e1,
			     const G3DVector3d &e2, G3DPoint3d *vert )
{
  double alpha = 0;

  for( int i = 0; i < 16; ++ i )
    {
      *(vert++) = e1*sin(alpha) + e2*cos(alpha) + o;
      alpha += M_PI / 8;
    }
}

void
G3DRotateWidget::reset( void )
{
  G3DPoint3d     center, origin;
  G3DMatrix      wmat;
  const G3D_BBAligned &bbox( ((G3DShape*)parent_)->getBoundingBox() );

  bbox.getParameter( origin, ce1_, ce2_, ce3_ );
  bbox.getCenter( ocenter_ );

  double sc = CIRCLE_SCALE;
  ce1_ *= sc;
  ce2_ *= sc;
  ce3_ *= sc;

  G3DPoint3d *v = _vertices;
  *(v++) = ocenter_ - ce1_;
  *(v++) = ocenter_ + ce1_;
  *(v++) = ocenter_ - ce2_;
  *(v++) = ocenter_ + ce2_;
  *(v++) = ocenter_ - ce3_;
  *(v++) = ocenter_ + ce3_;

  makeCircle( ocenter_, ce1_, ce2_, circle3_ );
  makeCircle( ocenter_, ce2_, ce3_, circle1_ );
  makeCircle( ocenter_, ce3_, ce1_, circle2_ );
}

void
G3DRotateWidget::select( void )
{
  display();
}


//================================END====================================
