/*
 * Project : "MindsEye"
 * Copyright (c) 1998. All Rights Reserved.
 * Author(s):   Martin Kuball
 * 
 * $Id: w_scale.cpp,v 1.2 1998/07/23 20:15:56 source Exp $
 *
 *
 * COPYRIGHT
 * =========
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */


#include <stdlib.h>
#include <GL/gl.h>

#include "w_scale.h"
#include "shape.h"
#include "t_actions.h"

void scale( G3DVector3d &e, double s );


G3DScaleWidget::G3DScaleWidget( G3DShape *parent ) :
  G3DWidget( parent )
{
  type_      = T_SCALE_WIDGET;
  selVertex_ = 0;
  reset();
}

void
G3DScaleWidget::calculateVertices( const G3DPoint3d &origin, const G3DVector3d &e1,
			      const G3DVector3d &e2,    const G3DVector3d &e3 )
{
  G3DPoint3d  *vert;
  G3DPoint3d  p;

  vert    = _vertices;
  *vert++ = p = origin;
  p += e1;
  *vert++ = p;
  p += e2;
  *vert++ = p;
  *vert++ = origin + e2;
  *vert++ = origin;
  *vert++ = p = origin + e3;
  p += e1;
  *vert++ = p;
  p += e2;
  *vert++ = p;

  *vert++ = _vertices[5] + e2;
  *vert++ = _vertices[5];

  *vert++ = _vertices[1];
  *vert++ = _vertices[6];
  *vert++ = _vertices[2];
  *vert++ = _vertices[7];
  *vert++ = _vertices[3];
  *vert++ = _vertices[8];
}

void
G3DScaleWidget::display( int ) const
{
  const G3DPoint3d  *vert;

  glDisable( GL_LIGHTING );
  glColor3f(1.0, 0, 0);

  glPolygonOffset(-2.0, -1);
  glEnable(GL_POLYGON_OFFSET_FILL);

  glPushName( 0 );
  vert = _triangles;
  G3DMatrix  mat;
  G3DMatrix  scaler;
  scaler.scale( _unScale );
  double     angle = 0;
  for( uint k = 1; k < 9; k++ )
    {
      mat.rotate( _rotaxes, angle );
      mat = scaler * mat;
      mat.translate( _vertices[k] );
      glPushMatrix();
      glMultMatrixd( (const double*)mat );
      glLoadName( k );
      glBegin( GL_TRIANGLES );
	  //glNormal3dv( (const double*)(*n1++) );
      glVertex3dv( (const double*)(*vert) );
      glVertex3dv( (const double*)(*(vert+2)) );
      glVertex3dv( (const double*)(*(vert+1)) );
	  //glNormal3dv( (const double*)(*n1++) );
      glVertex3dv( (const double*)(*vert) );
      glVertex3dv( (const double*)(*(vert+1)) );
      glVertex3dv( (const double*)(*(vert+3)) );
	  //glNormal3dv( (const double*)(*n1++) );
      glVertex3dv( (const double*)(*vert) );
      glVertex3dv( (const double*)(*(vert+3)) );
      glVertex3dv( (const double*)(*(vert+2)) );
	  //glNormal3dv( (const double*)(*n1++) );
      glVertex3dv( (const double*)(*(vert+1)) );
      glVertex3dv( (const double*)(*(vert+2)) );
      glVertex3dv( (const double*)(*(vert+3)) );
      glEnd();
      glPopMatrix();
      if( k == 4 )
        {
          vert += 4;
	  angle = 0;
	}
      else
        angle -= M_PI/2;
    }
  glPopName();
  glDisable(GL_POLYGON_OFFSET_FILL);

  glLineWidth( 2 ); //TODO only necessary for a box?
  vert = _vertices;
  glBegin( GL_LINE_STRIP );
  for( int i = 0; i < 10; i++ )
    {
      glVertex3dv( (const double*)(*vert++) );
    }
  glEnd();
  glBegin( GL_LINES );
  for( int j = 0; j < 6; j++ )
    {
      glVertex3dv( (const double*)(*vert++) );
    }
  glEnd();

  glEnable( GL_LIGHTING );
}

void
G3DScaleWidget::doAction( ActionData &action )
{
  if( action.type == A_SELECT )
    {
    }
  else if( action.type == A_FIND_SELECTION )
    {
      if( ((SelectData&)action).getNextNum(selVertex_) )
	{
	  ((SelectData&)action).setSelection( this );
	}
     }
}

void
G3DScaleWidget::getFrameAtHandle(G3DVector3d &e1, G3DVector3d &e2, G3DVector3d &e3)
{
  static int basisV[8][3] = { {1,-1,-1},{1,1,-1},{-1,1,-1},{-1,-1,-1},
			      {-1,-1,1},{1,-1,1},{1,1,1},{-1,1,1} };
  G3DPoint3d     origin;
  const G3D_BBAligned &bbox( ((G3DShape*)parent_)->getBoundingBox() );

  bbox.getParameter( origin, e1, e2, e3 );
  if( e1.isZero() )
    e1 = e2.cross( e3 );
  if( e2.isZero() )
    e2 = e3.cross( e1 );
  if( e3.isZero() )
    e3 = e1.cross( e2 );
  e1 *= basisV[selVertex_][0];
  e2 *= basisV[selVertex_][1];
  e3 *= basisV[selVertex_][2];
}

void
G3DScaleWidget::getHandlePosition( G3DPoint3d &pos ) const
{
  pos = _vertices[selVertex_];
}

void
G3DScaleWidget::getPosition( G3DPoint3d &pos ) const
{
  //Get the number of the vertex opposite of the one at the index
  static short oppVert[9] = {7, 8, 5, 6, 7, 2, 3, 0, 1};

  pos = _vertices[ oppVert[selVertex_] ];
}

void
G3DScaleWidget::reset( void )
{
  G3DVector3d    e1, e2, e3;
  G3DPoint3d     origin;
  G3DMatrix      wmat;
  const G3D_BBAligned &bbox( ((G3DShape*)parent_)->getBoundingBox() );

  bbox.getParameter( origin, e1, e2, e3 );
  calculateVertices( origin, e1, e2, e3 );

  double s1, s2, s3;
  parent_->getWorldTrafo( wmat );
  s1 = (wmat * e1).length();
  if( s1 > 0.2 )
    s1 = 0.1 / s1;
  else
    s1 = 0.5;
  s1 *= e1.length();
  e1.normalize();
  s2 = (wmat * e2).length();
  if( s2 > 0.2 )
    s2 = 0.1 / s2;
  else
    s2 = 0.5;
  s2 *= e2.length();
  e2.normalize();
  s3 = (wmat * e3).length();
  if( s3 > 0.2 )
    s3 = 0.1 / s3;
  else
    s3 = 0.5;
  s3 *= e3.length();
  e3.normalize();
  _rotaxes = e3 * (-1.0);

  G3DPoint3d  *tripts = _triangles;
  (tripts++)->setTo(0,0,0); // = offset;
  *(tripts++) = e1 * -1;
  *(tripts++) = e2;
  *(tripts++) = e3;
  (tripts++)->setTo(0,0,0); // = offset;
  *(tripts++) = e1;
  *(tripts++) = e2;
  *tripts     = e3 * -1;

  _unScale = (e1*s1) + (e2*s2) + (e3*s3);
}

void
scale( G3DVector3d &e, double s )
{
  if( e.length() > FG_EPSILON )
    {
      e *= 0.1 / e.length();
      if( s < 0.2 )
	e *= s / 0.2;
    }
}

void
G3DScaleWidget::select( void )
{
  display();
}

//================================END====================================
