/*
 * vi:ts=4:sw=4:
 *
 * Project : "MindsEye"
 * Copyright 1996. All Rights Reserved.
 * 
 * $RCSfile: m_3ddata.h,v $
 *
 * $Revision: 1.1.1.1 $
 * 
 * $Author: source $ 
 * 
 * $Locker:  $
 * 
 * $State: Exp $
 * 
 * 
 * COPYRIGHT
 * =========
 * 
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 * 
 * AUTHORS
 * ======= 
 *
 * Martin van Velsen,
 * Kent Mein
 *
 * OVERVIEW
 * ========
 *
 *
 */

#ifndef __DATA_3D_H__
#define __DATA_3D_H__

#include "m_nurbs.h"

#define NAME_LEN 256

/* basic building blocks */
#define UNION		 1
#define MERGE		 2 
#define INTERSECTION	 3
#define DIFFERENCE	 4
#define NEGATION	 5
//#define BLOB		 6 // I'm not sure about this one, see primitives

/* Types of lights */
#define POINTLIGHT	11
#define SPOTLIGHT	12
#define AREALIGHT	13

/* Cameras */
#define CAMERA		19

/* Meshes */
#define TMESH		30
#define NURB		31

// POV Primitive defines
#define BOX              1
#define SPHERE           2
#define TORUS            3
#define CYLINDER         4
#define PRISM            5
#define CONE             6
#define DISC             7
#define BLOB             8
#define NTHAGON	         9 // may need to be prisim ?

// This is the base class for a 3-dimensional entity

class Coordinate_3d
{
  public:

   double world [3];
};

// a two-dimensional coordinate is used in those instances where
// an item has to be able to be displayed

class Coordinate_2d
{
  public:

    int screen [2];
};

// A vertex is what we use in a mesh for instance. It's a 3d coordinate
// that can be displayed

class Vertex : public Coordinate_3d, public Coordinate_2d
{
  public:
 
   Vertex *next;
};

class Polygon
{
   Vertex *Vertices [3];    // Let's use triangles
};

class Line // ? nessecary ?
{
   Vertex *Vertices [2];
};

class BoundingVolume
{

  private:

   Coordinate_3d corners [8];
};

class Weight 
{
   public:

    double value;

    Weight *next;
};

class Operation
{
  double origin [3]; // this is a funny trick, the default operation
                     // parameters contain the tranformation from
                     // local axes to world axes ( in other words
                     // these are the local axes )
  Operation *next;
};

class Transform : public Operation
{
 double trans [3];
};

class Scale : public Operation
{
 double scale [3];
};

class Rotate : public Operation
{
 double rotate [3];
};

class Object : public BoundingVolume, public Coordinate_3d
{

  public:

     char name [NAME_LEN];

     Operation *transforms;  // list of subsequent tranformations
     Weight    *wt;          // used for size of splines or radius's... or camera roll
    
     Object *tree_top;
     Object *current_object; // pointer used when adding objects

     Object *next;           // next in line ( not child )
};

class Mesh : public Object
{
  char type; // MESH / NURB

  Vertex *vertex_top;
  Vertex *current_vertex;    

  Polygon *polygon_top;
  Polygon *current_polygon;
}; 

class Primitive : public Object
{
  private:

    char primitive_type;   // see POV primitive defines
};
 
class Camera : public Object
{
  private: 

     Coordinate_3d Target; // eye point is center of object
     float roll;           // rotation around eyepoint in degrees
     float lens;           // 35mm, ....    
};

class Light : public Object
{
  private:

     Coordinate_3d *Target; // only available if spotlight

     char type;             // POINTLIGHT / SPOTLIGHT / AREALIGHT
     float intensity;
     float dispersion;
     float color [3];       // this in favour of OpenGl ligth params 
};

class Frame 
{
   public:

      int id; // could this be key ??? 

      Object *objs;

      Frame *next;
};

class anim 
{
   public:
   
     char *name;
   
     int nr_frames;
     int jiffies;
   
     Frame *frames;
};

#endif

