/*
 * Project : "MindsEye"
 * Copyright 1996. All Rights Reserved.
 * Author(s)  M. van Velsen, M. Kuball
 *
 * $Id: m_messg.h,v 1.2 1998/03/08 17:24:00 source Exp $
 *
 *
 * COPYRIGHT
 * =========
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 * 
 * OVERVIEW
 * ========
 * Defines and support structures for a kernel-message system
 * Messages will be sorted in ranges:
 * 
 * errors   0     -  255  ( kernel )
 * messages 256   -  1024 ( kernel )
 * IO       1025  -  2048 
 * Console  2049  -  3073
 * Data     3074  -  4097
 *
 */

#ifndef __M_KMESSG__
#define __M_KMESSG__

#define ERROR_START			0
#define KMESS_START			256
#define IOMESS_START			1025
#define CONMESS_START			2049
#define DATAMESS_START			3074

//>---------- The Messages :

#define K_MESS_OK			0
#define K_MESS_PANIC			1
#define K_MESS_SHUTDOWN                 2    // maybe we need a message for each signal

#define K_MESS_BLOCK			256
#define K_MESS_RELEASE			257
#define K_MESS_UPDATE			258  // kernel version of QWidget:update();

//>---------- IO Messages

#define K_MESS_LOADSCENE		1025  
#define K_MESS_SAVESCENE		1026
#define K_MESS_IMPORTOBJ                1027
#define K_MESS_EXPORTOBJ                1028
//TODO see line 101 #define K_MESS_NEWSCENE			1050

//#define K_MESS_CREATEVERTEX		1027
//#define K_MESS_CREATEPOLYGON		1028
#define K_MESS_CREATEOBJECT		1029 // update all visible data
#define K_MESS_CREATELINE		1030 // ? nessecary ?

#define K_MESS_CREATEPRISM		1031
#define K_MESS_CREATESPHERE		1032 // includes hemisphere
#define K_MESS_CREATECUBE		1033
#define K_MESS_CREATECYLINDER		1034
#define K_MESS_CREATEPYRAMID		1035
#define K_MESS_CREATETOROID		1036
#define K_MESS_CREATECONE		1037
#define K_MESS_CREATEBLOB		1038

//>---------- Console messages

#define K_MESS_DATAINFO			2049 // summary of 3d data 
#define K_MESS_MODULES			2050 // similar to ps :)
#define K_MESS_STATUS			2051 // report kernel status
#define K_MESS_WHO			2052 // when we are in multi-user mode
#define K_MESS_QUIT			2053 // general quit message, folowed by kernel mess K_MESS_SHUTDOWN
#define K_MESS_HELP			2054 // general call for help

//>---------- 3D Data Specific

#define K_MESS_NEWSCENE			3074

#endif // __M_KMESSG__
