/*
    Project : "MindsEye I/O Module"
    Copyright 1996. All Rights Reserved.

    FILE:         3ds_bin.cpp
    AUTHOR:       Martin van Velsen


    OVERVIEW
    ========
    This is a lib which reads 3d-studio binary files from version 3.0
    and higher (v1.05)
    ( and some great help by Gert van der Spoel )
*/

#include "3ds_bin.h"

FILE *bin3ds; // var for easy access

/*---------------------------------------------------------------------------*/
IO_3DS_Bin_Class::IO_3DS_Bin_Class (void)
{
 current_chunk=0L;
 views_read   =0;
 numb_faces   =0;
 numb_vertices=0;
}
/*---------------------------------------------------------------------------*/
unsigned long IO_3DS_Bin_Class::ReadChunkPointer (void)
{
 unsigned long temp;

 read_unsigned_long (bin3ds,&temp);

 return (temp);
}
/*---------------------------------------------------------------------------*/
unsigned long IO_3DS_Bin_Class::GetChunkPointer (void)
{
 return (ftell (bin3ds)-2); // compensate for the already read Marker
}
/*---------------------------------------------------------------------------*/
void IO_3DS_Bin_Class::ChangeChunkPointer (unsigned long temp_pointer)
{
 fseek (bin3ds,temp_pointer,SEEK_SET);
}
/*---------------------------------------------------------------------------*/
int IO_3DS_Bin_Class::ReadName (void)
{
 unsigned int teller=0;
 unsigned char letter;

 strcpy (temp_name,"Default name");

 letter=readchar (bin3ds);
 if (letter==0) return (-1); // dummy object
 temp_name [teller]=letter;
 teller++;

 do
 {
  letter=readchar (bin3ds);
  temp_name [teller]=letter;
  teller++;
 }
 while ((letter!=0) && (teller<12));

 temp_name [teller-1]=0;

 #ifdef _3DSB_DEBUG_
  printf ("     Found name : %s\n",temp_name);
 #endif
 return (0);
}
/*---------------------------------------------------------------------------*/
int IO_3DS_Bin_Class::ReadLongName (void)
{
 unsigned int teller=0;
 unsigned char letter;

 strcpy (temp_name,"Default name");

 letter=readchar (bin3ds);
 if (letter==0) return (-1); // dummy object
 temp_name [teller]=letter;
 teller++;

 do
 {
  letter=readchar (bin3ds);
  temp_name [teller]=letter;
  teller++;
 }
 while (letter!=0);

 temp_name [teller-1]=0;

 #ifdef _3DSB_DEBUG_
   printf ("Found name : %s\n",temp_name);
 #endif
 return (0);
}
/*---------------------------------------------------------------------------*/
unsigned long IO_3DS_Bin_Class::ReadUnknownChunk (unsigned int chunk_id)
{
 unsigned long current_pointer;
 unsigned long temp_pointer;

 chunk_id=chunk_id;

 current_pointer=GetChunkPointer ();
 temp_pointer   =ReadChunkPointer ();

 ChangeChunkPointer (current_pointer+temp_pointer); 
 // move to the new chunk position
 return (temp_pointer);
}
/*---------------------------------------------------------------------------*/
unsigned long IO_3DS_Bin_Class::ReadRGBColor (void)
{
 float rgb_val [3];

 for (int i=0;i<3;i++)
  fread (&(rgb_val [i]),sizeof (float),1,bin3ds);

 #ifdef _3DSB_DEBUG_
 printf ("     Found Color (RGB) def of: R:%5.2f,G:%5.2f,B:%5.2f\n",
          rgb_val [0],
          rgb_val [1],
          rgb_val [2]);
 #endif

 return (12L);
}
/*---------------------------------------------------------------------------*/
unsigned long IO_3DS_Bin_Class::ReadTrueColor (void)
{
 unsigned char true_c_val [3];

 for (int i=0;i<3;i++)
  true_c_val [i]=readchar (bin3ds);

 #ifdef _3DSB_DEBUG_
 printf ("     Found Color (24bit) def of: R:%d,G:%d,B:%d\n",
          true_c_val [0],
          true_c_val [1],
          true_c_val [2]);
 #endif

 return (3L);
}
/*---------------------------------------------------------------------------*/
unsigned long IO_3DS_Bin_Class::ReadBooleanChunk (unsigned char *boolean)
{
 unsigned long current_pointer;
 unsigned long temp_pointer;

 current_pointer=GetChunkPointer ();
 temp_pointer   =ReadChunkPointer ();

 *boolean=readchar (bin3ds);

 ChangeChunkPointer (current_pointer+temp_pointer); // move to the new chunk position
 return (temp_pointer);
}
/*---------------------------------------------------------------------------*/
unsigned long IO_3DS_Bin_Class::ReadSpotChunk (void)
{
 unsigned long current_pointer;
 unsigned long temp_pointer;
 float target [4];
 float hotspot,falloff;

 current_pointer=GetChunkPointer ();
 temp_pointer   =ReadChunkPointer ();

 fread (&(target [0]),sizeof (float),1,bin3ds);
 fread (&(target [1]),sizeof (float),1,bin3ds);
 fread (&(target [2]),sizeof (float),1,bin3ds);
 fread (&hotspot,sizeof (float),1,bin3ds);
 fread (&falloff,sizeof (float),1,bin3ds);

 #ifdef _3DSB_DEBUG_
 printf ("      The target of the spot is at: X:%5.2f Y:%5.2f Y:%5.2f\n",
          target [0],
          target [1],
          target [2]);
 printf ("      The hotspot of this light is : %5.2f\n",hotspot);
 printf ("      The falloff of this light is : %5.2f\n",falloff);
 #endif

 ChangeChunkPointer (current_pointer+temp_pointer); 
 // move to the new chunk position
 return (temp_pointer);
}
/*---------------------------------------------------------------------------*/
unsigned long IO_3DS_Bin_Class::ReadLightChunk (void)
{
 unsigned char end_found=FALSE,boolean;
 unsigned int temp_int;
 unsigned long current_pointer;
 unsigned long temp_pointer;
 unsigned long counter=6L; // 2 id + 4 pointer
 float light_coors [3];

 current_pointer=GetChunkPointer ();
 temp_pointer   =ReadChunkPointer ();

 fread (&(light_coors [0]),sizeof (float),1,bin3ds);
 fread (&(light_coors [1]),sizeof (float),1,bin3ds);
 fread (&(light_coors [2]),sizeof (float),1,bin3ds);

 #ifdef _3DSB_DEBUG_
 printf ("     Found light at coordinates: X: %5.2f, Y: %5.2f,Z: %5.2f\n",
          light_coors [0],
          light_coors [1],
          light_coors [2]);
 #endif

 while (end_found==FALSE)
 {
   read_unsigned_int (bin3ds,&temp_int);

       switch (temp_int)
       {
        case LIT_UNKNWN01 :
                           #ifdef _3DSB_DEBUG_
                           printf (">>>>> Found Light unknown chunk id of %0X\n",LIT_UNKNWN01);
                           #endif
                           counter+=ReadUnknownChunk (LIT_UNKNWN01);
                           break;
        case LIT_OFF      :
                           #ifdef _3DSB_DEBUG_
                           printf (">>>>> Light is (on/off) chunk: %0X\n",LIT_OFF);
                           #endif
                           counter+=ReadBooleanChunk (&boolean);
                           #ifdef _3DSB_DEBUG_
                           if (boolean==TRUE)
                             printf ("      Light is on\n");
                           else
                             printf ("      Light is off\n");
                           #endif
                           break;
        case LIT_SPOT     :
                           #ifdef _3DSB_DEBUG_
                           printf (">>>>> Light is SpotLight: %0X\n",TRI_VERTEXL);
                           #endif
                           counter+=ReadSpotChunk ();
                           break;
        case COL_RGB      :
                           #ifdef _3DSB_DEBUG_
                           printf (">>>>> Found Color def (RGB) chunk id of %0X\n",temp_int);
                           #endif
                           counter+=ReadRGBColor ();
                           break;
        case COL_TRU      :
                           #ifdef _3DSB_DEBUG_
                           printf (">>>>> Found Color def (24bit) chunk id of %0X\n",temp_int);
                           #endif
                           counter+=ReadTrueColor ();
                           break;
        default           :break;
       }

   counter+=2;
   if (counter>=temp_pointer)
     end_found=TRUE;
 }

 ChangeChunkPointer (current_pointer+temp_pointer); 
 // move to the new chunk position
 return (temp_pointer);
}
/*---------------------------------------------------------------------------*/
unsigned long IO_3DS_Bin_Class::ReadCameraChunk (void)
{
 unsigned long current_pointer;
 unsigned long temp_pointer;
 unsigned int temp_int;
 float camera_eye [3];
 float camera_focus [3];
 float rotation,lens;

 current_pointer=GetChunkPointer ();
 temp_pointer   =ReadChunkPointer ();

 fread (&(camera_eye [0]),sizeof (float),1,bin3ds);
 fread (&(camera_eye [1]),sizeof (float),1,bin3ds);
 fread (&(camera_eye [2]),sizeof (float),1,bin3ds);

 #ifdef _3DSB_DEBUG_
 printf ("     Found Camera viewpoint at coordinates: X: %5.2f, Y: %5.2f,Z: %5.2f\n",
          camera_eye [0],
          camera_eye [1],
          camera_eye [2]);
 #endif

 fread (&(camera_focus [0]),sizeof (float),1,bin3ds);
 fread (&(camera_focus [1]),sizeof (float),1,bin3ds);
 fread (&(camera_focus [2]),sizeof (float),1,bin3ds);

 #ifdef _3DSB_DEBUG_
 printf ("     Found Camera focus coors at coordinates: X: %5.2f, Y: %5.2f,Z: %5.2f\n",
          camera_focus [0],
          camera_focus [1],
          camera_focus [2]);
 #endif

 fread (&rotation,sizeof (float),1,bin3ds);
 fread (&lens,sizeof (float),1,bin3ds);
 #ifdef _3DSB_DEBUG_
 printf ("     Rotation of camera is:  %5.4f\n",rotation);
 printf ("     Lens in used camera is: %5.4fmm\n",lens);
 #endif

 if ((temp_pointer-38)>0) // this means more chunks are to follow
 {
  #ifdef _3DSB_DEBUG_
  printf ("     **** found extra cam chunks ****\n");
  #endif

  read_unsigned_int (bin3ds,&temp_int);

  if (temp_int==CAM_UNKNWN01)
  {
   #ifdef _3DSB_DEBUG_
   printf ("     **** Found cam 1 type ch ****\n");
   #endif
   ReadUnknownChunk (CAM_UNKNWN01);
  }

  read_unsigned_int (bin3ds,&temp_int);
  if (temp_int==CAM_UNKNWN02)
  {
   #ifdef _3DSB_DEBUG_
   printf ("     **** Found cam 2 type ch ****\n");
   #endif
   ReadUnknownChunk (CAM_UNKNWN02);
  }
 }

 ChangeChunkPointer (current_pointer+temp_pointer); 
 // move to the new chunk position
 return (temp_pointer);
}
/*---------------------------------------------------------------------------*/
unsigned long IO_3DS_Bin_Class::ReadVerticesChunk (void)
{
 unsigned long current_pointer;
 unsigned long temp_pointer;
 float vertices [3]; // x,y,z
 unsigned int numb_v;

 current_pointer=GetChunkPointer ();
 temp_pointer   =ReadChunkPointer ();
 read_unsigned_int (bin3ds,&numb_vertices);

 #ifdef _3DSB_DEBUG_
 printf ("      Found (%d) number of vertices\n",numb_vertices);
 #endif

 for (int i=0;i<numb_vertices;i++)
 {
  fread (&(vertices [0]),sizeof (float),1,bin3ds);
  fread (&(vertices [1]),sizeof (float),1,bin3ds);
  fread (&(vertices [2]),sizeof (float),1,bin3ds);

  #ifdef _3DSB_DEBUG_
  printf ("      Vertex nr%4d: X: %5.2f  Y: %5.2f  Z:%5.2f\n",
           i,
           vertices [0],
           vertices [1],
           vertices [2]);
  #endif
 }

 ChangeChunkPointer (current_pointer+temp_pointer); 
 // move to the new chunk position
 return (temp_pointer);
}
/*---------------------------------------------------------------------------*/ 
unsigned long IO_3DS_Bin_Class::ReadSmoothingChunk ()
{
 unsigned long current_pointer;
 unsigned long temp_pointer;
 unsigned long smoothing;

 current_pointer=GetChunkPointer ();
 temp_pointer   =ReadChunkPointer ();

 for (int i=0;i<numb_faces;i++)
 {
  read_unsigned_long (bin3ds,&smoothing);
  smoothing=smoothing; // compiler warnig depressor *>:)
  #ifdef _3DSB_DEBUG_
  printf ("      The smoothing group for face [%5d] is %d\n",i,smoothing);
  #endif
 }

 ChangeChunkPointer (current_pointer+temp_pointer); 
 // move to the new chunk position
 return (temp_pointer);
}
/*---------------------------------------------------------------------------*/
unsigned long IO_3DS_Bin_Class::ReadFacesChunk (void)
{
 unsigned long current_pointer;
 unsigned long temp_pointer;
 unsigned int temp_diff;
 unsigned int faces [6]; // a,b,c,Diff (Diff= AB: BC: CA: )

 current_pointer=GetChunkPointer ();
 temp_pointer   =ReadChunkPointer ();
 read_unsigned_int (bin3ds,&numb_faces);
 #ifdef _3DSB_DEBUG_
 printf ("      Found (%d) number of faces\n",numb_faces);
 #endif

 for (int i=0;i<numb_faces;i++)
 {
  read_unsigned_int (bin3ds,&faces [0]);
  read_unsigned_int (bin3ds,&faces [1]);
  read_unsigned_int (bin3ds,&faces [2]);
  read_unsigned_int (bin3ds,&temp_diff);
  temp_diff=temp_diff & 0x000F;

  faces [3]=(temp_diff & 0x0004) >> 2;
  faces [4]=(temp_diff & 0x0002) >> 1;
  faces [5]=(temp_diff & 0x0001);

  #ifdef _3DSB_DEBUG_
  printf ("      Face nr:%d, A: %d  B: %d  C:%d , AB:%d  BC:%d  CA:%d\n",
           i,
           faces [0],
           faces [1],
           faces [2],
           faces [3],
           faces [4],
           faces [5]);
  #endif
 }
 
 read_unsigned_int (bin3ds,&temp_diff);
 
 if (temp_diff==TRI_SMOOTH)
  ReadSmoothingChunk ();
 #ifdef _3DSB_DEBUG_
 else
  printf ("      No smoothing groups found, assuming autosmooth\n");
 #endif

 ChangeChunkPointer (current_pointer+temp_pointer); 
 // move to the new chunk position
 return (temp_pointer);
}
/*---------------------------------------------------------------------------*/
unsigned long IO_3DS_Bin_Class::ReadTranslationChunk (void)
{
 unsigned long current_pointer;
 unsigned long temp_pointer;
 current_pointer=GetChunkPointer ();
 temp_pointer   =ReadChunkPointer ();

 for (int j=0;j<4;j++)
 {
   for (int i=0;i<3;i++)
    fread (&(trans_mat [j][i]),sizeof (float),1,bin3ds);
 }

 trans_mat [0][3]=0;
 trans_mat [1][3]=0;
 trans_mat [2][3]=0;
 trans_mat [3][3]=1;

 #ifdef _3DSB_DEBUG_
 printf ("     The translation matrix is:\n");
 for (int i=0;i<4;i++)
     printf ("      | %5.2f %5.2f %5.2f %5.2f |\n",
              trans_mat [i][0],
              trans_mat [i][1],
              trans_mat [i][2],
              trans_mat [i][3]);
 #endif

 ChangeChunkPointer (current_pointer+temp_pointer); 
 // move to the new chunk position
 return (temp_pointer);
}
/*---------------------------------------------------------------------------*/
unsigned long IO_3DS_Bin_Class::ReadObjChunk (void)
{
 unsigned char end_found=FALSE,boolean=TRUE;
 unsigned int temp_int;
 unsigned long current_pointer;
 unsigned long temp_pointer;
 unsigned long counter=6L; // 2 id + 4 pointer

 current_pointer=GetChunkPointer ();
 temp_pointer   =ReadChunkPointer ();

 while (end_found==FALSE)
 {
   read_unsigned_int (bin3ds,&temp_int);

       switch (temp_int)
       {
        case TRI_VERTEXL :
                          #ifdef _3DSB_DEBUG_
                          printf (">>>>> Found Object vertices chunk id of %0X\n",
                                  temp_int);
                          #endif
                          counter+=ReadVerticesChunk ();
                          break;
        case TRI_FACEL1  :
                          #ifdef _3DSB_DEBUG_
                          printf (">>>>> Found Object faces (1) chunk id of %0X\n",
                                  temp_int);
                          #endif
                          counter+=ReadFacesChunk ();
                          break;
        case TRI_FACEL2  :
                          #ifdef _3DSB_DEBUG_
                          printf (">>>>> Found Object faces (2) chunk id of %0X\n",
                                  temp_int);
                          #endif
                          counter+=ReadUnknownChunk (temp_int);
                          break;
        case TRI_TRANSL  :
                          #ifdef _3DSB_DEBUG_
                          printf (">>>>> Found Object translation chunk id of %0X\n",
                                  temp_int);
                          #endif
                          counter+=ReadTranslationChunk ();
                          break;
        case TRI_VISIBLE :
                          #ifdef _3DSB_DEBUG_
                          printf (">>>>> Found Object vis/invis chunk id of %0X\n",
                                  temp_int);
                          #endif
                          counter+=ReadBooleanChunk (&boolean);

                          #ifdef _3DSB_DEBUG_
                          if (boolean==TRUE)
                             printf ("      Object is (visible)\n");
                          else
                             printf ("      Object is (not visible)\n");
                          #endif
                          break;
        default:          break;
       }

   counter+=2;
   if (counter>=temp_pointer)
     end_found=TRUE;
 }

 ChangeChunkPointer (current_pointer+temp_pointer); 
 // move to the new chunk position
 return (temp_pointer);
}
/*---------------------------------------------------------------------------*/
unsigned long IO_3DS_Bin_Class::ReadObjectChunk (void)
{
 unsigned char end_found=FALSE;
 unsigned int temp_int;
 unsigned long current_pointer;
 unsigned long temp_pointer;
 unsigned long counter=6L; // 2 id + 4 pointer

 current_pointer=GetChunkPointer ();
 temp_pointer   =ReadChunkPointer ();

 if (ReadName ()==-1)
 {
  #ifdef _3DSB_DEBUG_
  printf (">>>>* Dummy Object found\n");
  #endif
 }

 while (end_found==FALSE)
 {
   read_unsigned_int (bin3ds,&temp_int);

       switch (temp_int)
       {
        case OBJ_UNKNWN01:counter+=ReadUnknownChunk (OBJ_UNKNWN01);break;
        case OBJ_UNKNWN02:counter+=ReadUnknownChunk (OBJ_UNKNWN02);break;
        case OBJ_TRIMESH :
                          #ifdef _3DSB_DEBUG_
                          printf (">>>> Found Obj/Mesh chunk id of %0X\n",
                                  OBJ_TRIMESH);
                          #endif
                          counter+=ReadObjChunk ();
                          break;
        case OBJ_LIGHT   :
                          #ifdef _3DSB_DEBUG_
                          printf (">>>> Found Light chunk id of %0X\n",
                                  OBJ_LIGHT);
                          #endif
                          counter+=ReadLightChunk ();
                          break;
        case OBJ_CAMERA  :
                          #ifdef _3DSB_DEBUG_
                          printf (">>>> Found Camera chunk id of %0X\n",
                                  OBJ_CAMERA);
                          #endif
                          counter+=ReadCameraChunk ();
                          break;
        default:          break;
       }

   counter+=2;
   if (counter>=temp_pointer)
     end_found=TRUE;
 }

 ChangeChunkPointer (current_pointer+temp_pointer); 
 // move to the new chunk position
 return (temp_pointer);
}
/*---------------------------------------------------------------------------*/
unsigned long IO_3DS_Bin_Class::ReadBackgrChunk (void)
{
 unsigned char end_found=FALSE;
 unsigned int temp_int;
 unsigned long current_pointer;
 unsigned long temp_pointer;
 unsigned long counter=6L; // 2 id + 4 pointer

 current_pointer=GetChunkPointer ();
 temp_pointer   =ReadChunkPointer ();

 while (end_found==FALSE)
 {
   read_unsigned_int (bin3ds,&temp_int);
 
       switch (temp_int)
       {
        case COL_RGB :
                      #ifdef _3DSB_DEBUG_
                      printf (">> Found Color def (RGB) chunk id of %0X\n",
                              temp_int);
                      #endif
                      counter+=ReadRGBColor ();
                      break;
        case COL_TRU :
                      #ifdef _3DSB_DEBUG_
                      printf (">> Found Color def (24bit) chunk id of %0X\n",
                              temp_int);
                      #endif
                      counter+=ReadTrueColor ();
                      break;
        default:      break;
       }

   counter+=2;
   if (counter>=temp_pointer)
     end_found=TRUE;
 }

 ChangeChunkPointer (current_pointer+temp_pointer); 
 // move to the new chunk position
 return (temp_pointer);
}
/*---------------------------------------------------------------------------*/
unsigned long IO_3DS_Bin_Class::ReadAmbientChunk (void)
{
 unsigned char end_found=FALSE;
 unsigned int temp_int;
 unsigned long current_pointer;
 unsigned long temp_pointer;
 unsigned long counter=6L; // 2 id + 4 pointer

 current_pointer=GetChunkPointer ();
 temp_pointer   =ReadChunkPointer ();

 while (end_found==FALSE)
 {
   read_unsigned_int (bin3ds,&temp_int);   

       switch (temp_int)
       {
        case COL_RGB :
                      #ifdef _3DSB_DEBUG_
                      printf (">>>> Found Color def (RGB) chunk id of %0X\n",
                              temp_int);
                      #endif
                      counter+=ReadRGBColor ();
                      break;
        case COL_TRU :
                      #ifdef _3DSB_DEBUG_
                      printf (">>>> Found Color def (24bit) chunk id of %0X\n",
                              temp_int);
                      #endif
                      counter+=ReadTrueColor ();
                      break;
        default:      break;
       }

   counter+=2;
   if (counter>=temp_pointer)
     end_found=TRUE;
 }

 ChangeChunkPointer (current_pointer+temp_pointer); 
 // move to the new chunk position
 return (temp_pointer);
}
/*---------------------------------------------------------------------------*/
unsigned long IO_3DS_Bin_Class::FindCameraChunk (void)
{
 long temp_pointer=0L;
 unsigned int temp_int;

 for (int i=0;i<12;i++)
  read_unsigned_int (bin3ds,&temp_int);

 temp_pointer=11L;
 temp_pointer=ReadName ();

 #ifdef _3DSB_DEBUG_
 if (temp_pointer==-1)
   printf (">>>>* No Camera name found\n");
 #endif

 return (temp_pointer);
}
/*---------------------------------------------------------------------------*/
unsigned long IO_3DS_Bin_Class::ReadViewPortChunk (void)
{
 unsigned long current_pointer;
 unsigned long temp_pointer;
 unsigned int port,attribs;
 unsigned int temp_int;

 views_read++;

 current_pointer=GetChunkPointer ();
 temp_pointer   =ReadChunkPointer ();

 read_unsigned_int (bin3ds,&attribs);
 if (attribs==3)
 {
  #ifdef _3DSB_DEBUG_
  printf ("<Snap> active in viewport\n");
  #endif
 }
 if (attribs==5)
 {
  #ifdef _3DSB_DEBUG_
  printf ("<Grid> active in viewport\n");
  #endif
 }

 for (int i=1;i<6;i++) 
 read_unsigned_int (bin3ds,&temp_int);// read 5 ints to get to the viewport

 read_unsigned_int (bin3ds,&port);
 if ((port==0xFFFF) || (port==0))
 {
   FindCameraChunk ();
   port=CAMERA;
 }

 #ifdef _3DSB_DEBUG_
 printf ("Reading [%s] information with id:%d\n",viewports [port],port);
 #endif

 ChangeChunkPointer (current_pointer+temp_pointer); 
 // move to the new chunk position
 return (temp_pointer);
}
/*---------------------------------------------------------------------------*/
unsigned long IO_3DS_Bin_Class::ReadViewChunk (void)
{
 unsigned char end_found=FALSE;
 unsigned int temp_int;
 unsigned long current_pointer;
 unsigned long temp_pointer;
 unsigned long counter=6L;

 current_pointer=GetChunkPointer ();
 temp_pointer   =ReadChunkPointer ();

 while (end_found==FALSE)
 {
   read_unsigned_int (bin3ds,&temp_int);

       switch (temp_int)
       {
        case EDIT_VIEW_P1 :
                           #ifdef _3DSB_DEBUG_
                           printf (">>>> Found Viewport1 chunk id of %0X\n",
                                   temp_int);
                           #endif
                           counter+=ReadViewPortChunk ();
                           break;
        case EDIT_VIEW_P2 :
                           #ifdef _3DSB_DEBUG_
                           printf (">>>> Found Viewport2 (bogus) chunk id of %0X\n",
                                   temp_int);
                           #endif
                           counter+=ReadUnknownChunk (EDIT_VIEW_P2);
                           break;
       case EDIT_VIEW_P3 :
                           #ifdef _3DSB_DEBUG_
                           printf (">>>> Found Viewport chunk id of %0X\n",
                                   temp_int);
                           #endif
                           counter+=ReadViewPortChunk ();
                           break;
        default           :break;
       }

   counter+=2;
   if (counter>=temp_pointer)
     end_found=TRUE;

   if (views_read>3)
     end_found=TRUE;
 }

 ChangeChunkPointer (current_pointer+temp_pointer); 
 // move to the new chunk position
 return (temp_pointer);
}
/*---------------------------------------------------------------------------*/
unsigned long IO_3DS_Bin_Class::ReadMatDefChunk (void)
{
 unsigned long current_pointer;
 unsigned long temp_pointer;

 current_pointer=GetChunkPointer ();
 temp_pointer   =ReadChunkPointer ();

 if (ReadLongName ()==-1)
 {
   #ifdef _3DSB_DEBUG_
   printf (">>>>* No Material name found\n");
   #endif
 }

 ChangeChunkPointer (current_pointer+temp_pointer); 
 // move to the new chunk position
 return (temp_pointer);
}
/*---------------------------------------------------------------------------*/
unsigned long IO_3DS_Bin_Class::ReadMaterialChunk (void)
{
 unsigned char end_found=FALSE;
 unsigned int temp_int;
 unsigned long current_pointer;
 unsigned long temp_pointer;
 unsigned long counter=6L;

 current_pointer=GetChunkPointer ();
 temp_pointer   =ReadChunkPointer ();

 while (end_found==FALSE)
 {
   read_unsigned_int (bin3ds,&temp_int);

       switch (temp_int)
       {
        case MAT_NAME01  :
                          #ifdef _3DSB_DEBUG_
                          printf (">>>> Found Material def chunk id of %0X\n",
                                  temp_int);
                          #endif
                          counter+=ReadMatDefChunk ();
                          break;
        default:break;
       }

   counter+=2;
   if (counter>=temp_pointer)
     end_found=TRUE;
 }

 ChangeChunkPointer (current_pointer+temp_pointer); 
 // move to the new chunk position
 return (temp_pointer);
}
/*---------------------------------------------------------------------------*/
unsigned long IO_3DS_Bin_Class::ReadEditChunk (void)
{
 unsigned char end_found=FALSE;
 unsigned int temp_int;
 unsigned long current_pointer;
 unsigned long temp_pointer;
 unsigned long counter=6L;

 current_pointer=GetChunkPointer ();
 temp_pointer   =ReadChunkPointer ();

 while (end_found==FALSE)
 {
   read_unsigned_int (bin3ds,&temp_int);

       switch (temp_int)
       {
        case EDIT_UNKNW01:counter+=ReadUnknownChunk (EDIT_UNKNW01);break;
        case EDIT_UNKNW02:counter+=ReadUnknownChunk (EDIT_UNKNW02);break;
        case EDIT_UNKNW03:counter+=ReadUnknownChunk (EDIT_UNKNW03);break;
        case EDIT_UNKNW04:counter+=ReadUnknownChunk (EDIT_UNKNW04);break;
        case EDIT_UNKNW05:counter+=ReadUnknownChunk (EDIT_UNKNW05);break;
        case EDIT_UNKNW06:counter+=ReadUnknownChunk (EDIT_UNKNW06);break;
        case EDIT_UNKNW07:counter+=ReadUnknownChunk (EDIT_UNKNW07);break;
        case EDIT_UNKNW08:counter+=ReadUnknownChunk (EDIT_UNKNW08);break;
        case EDIT_UNKNW09:counter+=ReadUnknownChunk (EDIT_UNKNW09);break;
        case EDIT_UNKNW10:counter+=ReadUnknownChunk (EDIT_UNKNW10);break;
        case EDIT_UNKNW11:counter+=ReadUnknownChunk (EDIT_UNKNW11);break;
        case EDIT_UNKNW12:counter+=ReadUnknownChunk (EDIT_UNKNW12);break;
        case EDIT_UNKNW13:counter+=ReadUnknownChunk (EDIT_UNKNW13);break;

        case EDIT_MATERIAL :
                            #ifdef _3DSB_DEBUG_
                            printf (">>> Found Materials chunk id of %0X\n",
                                    temp_int);
                            #endif
                            counter+=ReadMaterialChunk ();
                            break;
        case EDIT_VIEW1    :
                            #ifdef _3DSB_DEBUG_
                            printf (">>> Found View main def chunk id of %0X\n",
                                    temp_int);
                            #endif
                            counter+=ReadViewChunk ();
                            break;
        case EDIT_BACKGR   :
                            #ifdef _3DSB_DEBUG_
                            printf (">>> Found Backgr chunk id of %0X\n",
                                    temp_int);
                            #endif
                            counter+=ReadBackgrChunk ();
                            break;
        case EDIT_AMBIENT  :
                            #ifdef _3DSB_DEBUG_
                            printf (">>> Found Ambient chunk id of %0X\n",
                                    temp_int);
                            #endif
                            counter+=ReadAmbientChunk ();
                            break;
        case EDIT_OBJECT   :
                            #ifdef _3DSB_DEBUG_
                            printf (">>> Found Object chunk id of %0X\n",
                                    temp_int);
                            #endif
                            counter+=ReadObjectChunk ();
                            break;
        default:            break;
       }

   counter+=2;
   if (counter>=temp_pointer)
     end_found=TRUE;
 }

 ChangeChunkPointer (current_pointer+temp_pointer); 
 // move to the new chunk position
 return (temp_pointer);
}
/*---------------------------------------------------------------------------*/
unsigned long IO_3DS_Bin_Class::ReadKeyfChunk (void)
{
 unsigned char end_found=FALSE;
 unsigned int temp_int;
 unsigned long current_pointer;
 unsigned long temp_pointer;
 unsigned long counter=6L;

 current_pointer=GetChunkPointer ();
 temp_pointer   =ReadChunkPointer ();

 while (end_found==FALSE)
 {
   read_unsigned_int (bin3ds,&temp_int);

       switch (temp_int)
       {
        case KEYF_UNKNWN01 :counter+=ReadUnknownChunk (temp_int);break;
        case KEYF_UNKNWN02 :counter+=ReadUnknownChunk (temp_int);break;
        case KEYF_FRAMES   :
                            #ifdef _3DSB_DEBUG_
                            printf (">>> Found Keyframer frames chunk id of %0X\n",
                                    temp_int);
                            #endif
                            counter+=ReadUnknownChunk (temp_int);
                            break;
        case KEYF_OBJDES   :
                            #ifdef _3DSB_DEBUG_
                            printf (">>> Found Keyframer object description chunk id of %0X\n",
                                    temp_int);
                            #endif
                            counter+=ReadUnknownChunk (temp_int);
                            break;
        case EDIT_VIEW1    :
                            #ifdef _3DSB_DEBUG_
                            printf (">>> Found View main def chunk id of %0X\n",
                                    temp_int);
                            #endif
                            counter+=ReadViewChunk ();
                            break;
        default:            break;
       }

   counter+=2;
   if (counter>=temp_pointer)
     end_found=TRUE;
 }

 ChangeChunkPointer (current_pointer+temp_pointer); 
 // move to the new chunk position
 return (temp_pointer);
}
/*---------------------------------------------------------------------------*/
unsigned long IO_3DS_Bin_Class::ReadMainChunk (void)
{
 unsigned char end_found=FALSE;
 unsigned int temp_int;
 unsigned long current_pointer;
 unsigned long temp_pointer;
 unsigned long counter=6L;

 current_pointer=GetChunkPointer ();
 temp_pointer   =ReadChunkPointer ();

 while (end_found==FALSE)
 {
   read_unsigned_int (bin3ds,&temp_int);

       switch (temp_int)
       {
        case KEYF3DS :
                      #ifdef _3DSB_DEBUG_
                      printf (">> Found *Keyframer* chunk id of %0X\n",KEYF3DS);
                      #endif
                      counter+=ReadKeyfChunk ();
                      break;
        case EDIT3DS :
                      #ifdef _3DSB_DEBUG_
                      printf (">> Found *Editor* chunk id of %0X\n",EDIT3DS);
                      #endif
                      counter+=ReadEditChunk ();
                      break;
        default:      break;
       }

   counter+=2;
   if (counter>=temp_pointer)
    end_found=TRUE;
 }

 ChangeChunkPointer (current_pointer+temp_pointer); 
 // move to the new chunk position
 return (temp_pointer);
}
/*---------------------------------------------------------------------------*/
int IO_3DS_Bin_Class::ReadPrimaryChunk (void)
{
 unsigned char version;
 unsigned int temp_int;

 read_unsigned_int (bin3ds,&temp_int);
 if (temp_int==MAIN3DS)
 {
  #ifdef _3DSB_DEBUG_
  printf ("> Found Main chunk id of %0X\n",MAIN3DS);
  #endif
  //>---------- find version number
  fseek (bin3ds,28L,SEEK_SET);
  version=readchar (bin3ds);
  if (version<3)
  {
   #ifdef _3DSB_DEBUG_
   printf ("Sorry this lib can only read 3ds files of version 3.0 and higher\n");
   printf ("The version of the file you want to read is: %d\n",version);
   #endif
   return (1);
  }
  fseek (bin3ds,2,SEEK_SET);
  ReadMainChunk ();
 }
 else
  return (1);

 return (0);
}
/*---------------------------------------------------------------------------*/
int IO_3DS_Bin_Class::check_header (FileClass *tempclass)
{    



  return (FILE_FUNCTIONNOTDEFINED);
}
/*---------------------------------------------------------------------------*/
int IO_3DS_Bin_Class::file_load (FileClass *tempclass)
{
  // we assume the filepointer is valid
  bin3ds=tempclass->filepointer;

  #ifdef _3DSB_DEBUG_
   printf ("\nLoading 3ds binary file : %s\n",tempclass->filename);
  #endif
  while (ReadPrimaryChunk ()==0);

  return (FILE_DONE);
}
/*---------------------------------------------------------------------------*/
int IO_3DS_Bin_Class::file_save (class FileClass *tempclass)
{
 // we assume the filepointer is valid
 bin3ds=tempclass->filepointer;


 return (FILE_FUNCTIONNOTDEFINED);
}
/*---------------------------------------------------------------------------*/