/*
 * Project : "MindsEye, 3D library"
 * Copyright 1996, 1997. All Rights Reserved.
 * Author(s): M. Kuball
 *
 * $Id: ac3d.cpp,v 1.2 1997/12/09 21:32:03 source Exp $
 *
 * $LOG:$
 * 
 * COPYRIGHT
 * =========
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

#include <stdlib.h>

#include "3dlib/s_shapes.h"
#include "3dlib/light.h"
#include "ac3d.h"
#include "ac3dobjects.h"


IO_BaseClass*
NewAC3D( void )
{
  return( new MEIoAc3d );
}


int
MEIoAc3d::check_header( char* )
{
  //must bei AC3Dx where x is a hex number with ther format version
  return( 0 );  //TODO undefined
}


int
MEIoAC3D::exitLoad( int )
{
  return( IO_BaseClass::exitLoad(1) );
}


int
MEIoAC3D::exitSave( int )
{
  _inpf.close();
  return( 1 );
}


int
MEIoAC3D::getFormatInfo( int type )
{
  return( 0 ); //TODO return a FormatInfo object
}


int
MEIoAC3D::initLoad( const char *fname )
{
  char buf[5];
  int result;

  _inpf.open( fname );
  if( _inpf.good() )
    {
      _inpf.getline( buffer, 5 );
      if( strcmp(buffer, "AC3D") == 0 )
	{
	  _version = _inpf.getc();
	  _inpf.ignore();
	  return( READ_SUCCESS );
	}
    }
  return( READ_ERROR );
}


int
MEIoAC3D::initSave( const char *fname, int flag )
{
  return( WRITE_SUCCESS );
}



int
MEIoAC3D::loadObjects( G3DGroup *group )
{
  static char token[40];
  AC3DSurface  *tSurf = NULL;
  AC3DObject   *tmp;
  char cmd;

  nexttok[0] = '\0';

  while( !_inpf.eof() )
    {
      _inpf >> token;
      if( !_inpf )
	break;

      if( strcmp( token, "OBJECT" ) )
	{
	  _inpf >> token;
	  if( strcmp(token, "poly") )
	    {
	      tmp = AC3DPoly( _inpf );
	    }
	  else if( strcmp(token, "group") )  //same as world??
	    {
	      tmp = AC3DPoly( _inpf );
	    }
	  else if( strcmp(token, "world") )
	    {
	      //get number of children
	      tmp = AC3DWorld( _inpf );
	    }
	}
      else if( strcmp(token, "MATERIAL") )
	{
	  material.read( _inpf );
	}
      else
	{ //unknown Object }

	default:
	  tmp = NULL;
	}

      if( tmp )
	if( tmp->isValid() )
	  {
	    //TODO tmp->addAttribute( tSurf->getAttribute() );
	    group->append( tmp->buildG3DObject() );
	    _objCount++;
	  }
        else
	  delete tmp;
    }
  return( 1 );
}


int
MEIoAC3D::saveObjects( void )
{
  return( 0 );
}


//====================================END================================
