/*
 * Project : "MindsEye, 3D library"
 * Copyright 1996, 1997. All Rights Reserved.
 * Author(s): M. Kuball
 *
 * $Id: ac3dobjects.cpp,v 1.2 1997/12/09 21:32:03 source Exp $
 *
 * $LOG:$
 * 
 * COPYRIGHT
 * =========
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

#include <stdlib.h>
#include <stdio.h>

#include "3dlib/generic.h"
#include "3dlib/light.h"
#include "3dlib/s_shapes.h"
#include "3dlib/s_prims.h"
#include "3dlib/shader.h"

#include "ac3dobjects.h"


void
prerror( char *str )
{
  puts( str );
}



AC3DMaterial::read( ifstream &file )
{
  char buf[8];
  file >> name;
  file >> setw(6) >> buf;  //rgb
  file >> dr >> dg >> db;
  file >> setw(6) >> buf;  //amb
  file >> ar >> ag >> ab;
  file >> setw(6) >> buf;  //emmis
  file >> er >> eg >> eb;
  file >> setw(6) >> buf;  //spec
  file >> sr >> sg >> sb;
  file >> setw(6) >> buf;  //shine
  file >> shine;
  file >> setw(6) >> buf;  //trans
  file >> trans;
}


//======================================================================


NFFCamera::NFFCamera( ifstream &inpf, char *nexttok )
{
  int xres, yres;  //TODO values are currently ignored
			  
  inpf >> nexttok;
  if (strcmp(nexttok, "from"))
    prerror("\"from\" expected");
  inpf >> fromx >> fromy >> fromz >> nexttok;
  
  if (strcmp(nexttok, "at"))
    prerror("\"at\" expected");
  inpf >> atx >> aty >> atz >> nexttok;
  
  if (strcmp(nexttok, "up"))
    prerror("\"up\" expected");
  inpf >> upx >> upy >> upz >> nexttok;
  
  if (strcmp(nexttok, "angle"))
    prerror("\"angle\" expected");
  inpf >> zoom >> nexttok;
  
  if (strcmp(nexttok, "hither"))
    prerror("\"hither\" expected");
  inpf >> near >> nexttok;
  
  if (strcmp(nexttok, "resolution"))
    prerror("\"resolution\" expected");
  inpf >> xres >> yres;
  
  nexttok[0] = '\0';
}


G3DGLShader*
NFFCamera::buildG3DCamera( void )
{
  G3DPoint3d  pos( fromx, fromy, fromz );
  G3DVector3d poi( atx, aty, atz );
  G3DVector3d up ( upx, upy, upz );

  G3DGLShader *tmp = new G3DGLShader( near, near*100 );
  poi += (pos * -1.0);
  tmp->setView( pos, poi, up );
  tmp->zoomSet( zoom );

  return( tmp );
}


G3DShadable*
NFFCamera::buildG3DObject( void )
{
  return( NULL );
}


//======================================================================


NFFLight::NFFLight( ifstream &inpf, char *nexttok )
{
  char cmd, *sc;
  /*
  // rectangular light source
  if (inpf.peek() == 'r')
    {   
      inpf >> cmd;
    
      GFloat ax, ay, az;
      GFloat width, height;
      int Wres, Hres;
      int min, max, cand;
	    
      inpf >> lx >> ly >> lz;
      inpf >> ax >> ay >> az;
      inpf >> width >> height;
      inpf >> Wres >> Hres;
      inpf >> min >> max >> cand;
    
      sRectL++;
  }

  // spherical light source
  if( inpf.peek() == 's' )
  {
  inpf >> cmd;
    
    GFloat r;
    int Wres, Hres;
    inpf >> lx >> ly >> lz >> r >>Wres >> Hres ;
    tLight = new GXSphericalLight(GPoint3(lx, ly, lz), r, Wres, Hres);
    sSphL++;
  }
  */  
  // point light source
  //else {
  inpf >> pos.x >> pos.y >> pos.z;
  
  // color of the light source. It's optional so test for it.
  streampos tPos = inpf.tellg();
  inpf >> nexttok;
  cx = strtod(nexttok, &sc);
  if (sc == &(nexttok[0]))
    {
      inpf.seekg(tPos);
      nexttok[0] = '\0';
      cx = cy = cz = 1.0;
    }
  else
    {
      inpf >> cy >> cz;
      nexttok[0] = '\0';
    }
}


G3DShadable*
NFFLight::buildG3DObject( void )
{
  VectorStruct attenuation = {1,0,0};

  G3DPointLight *tmp = new G3DPointLight( &pos, &attenuation );
  tmp->setColor( G3DColor(cx, cy, cz) );
  return( tmp );
}


//======================================================================


NFFPPatch::NFFPPatch( ifstream &inpf )
{
  float x, y, z, nx, ny, nz;
  int   tNum;

  inpf >> tNum;
  _vertices = new G3DPoint3d[tNum];
  _normals  = new G3DVector3d[tNum];
  for (register int i = 0; i < tNum ; i++)
    {
      inpf >> x >> y >> z >> nx >> ny >> nz;
      _vertices[i].setTo(x, y, z);
      _normals[i].setTo(nx, ny, nz);
    }
}

NFFPPatch::~NFFPPatch()
{
  delete [] _vertices;
  delete [] _normals;
}


G3DShadable*
NFFPPatch::buildG3DObject( void )
{
    /*	      
  if (tNum >  3)
    tPoly = new GTSmoothPolygon(tNum, tVerts, tNorms);
  else
    tPoly = new GTSmoothTriangle(tVerts, tNorms);
    */
  return(NULL);
}


//======================================================================


NFFPolygon::NFFPolygon( ifstream &inpf )
{
  float x, y, z;
  int   tNum;

  inpf >> tNum;
  _vertices = new G3DPoint3d[tNum];
  for (register int i = 0; i < tNum ; i++)
    {
      inpf >> x >> y >> z;
      _vertices[i].setTo(x, y, z);
    }
}


NFFPolygon::~NFFPolygon()
{
  delete[] _vertices;
}


G3DShadable*
NFFPolygon::buildG3DObject( void )
{
    /*
  if(tNum > 3)
    tPoly = new GTPolygon(tNum, tVerts);
  else
    tPoly = new GTTriangle(tVerts);
    */
  return(NULL);
}


//===================================END=================================
