/*
 * Project : "MindsEye, 3D library"
 * Copyright 1996, 1997. All Rights Reserved.
 * Author(s): M. van Velsen, M. Kuball
 *
 * $Id: io_base.cpp,v 1.8 1998/07/23 20:16:01 source Exp $
 *
 *
 * COPYRIGHT
 * =========
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * OVERWIE
 * =======
 * I/O lib for reading and writing Quick Draw 3dmf ascii files.
 * It's basically an interface between the Mindseye 3dlib and the
 * 3DMF i/o stuff from apple.
 */


#include "3dlib/light.h"
#include "3dlib/camera.h"
#include "3dlib/scene.h"
#include "3dlib/aggregate.h"
#include "io_base.h"
#include "io_module.h"
#include "kernel/world_obj.h"


IO_BaseClass::~IO_BaseClass()
{
}


int
IO_BaseClass::check_header( char* )
{
  return( 0 );
}


int
IO_BaseClass::exitLoad( int res )
{
  //TODO test res for errors?
  cout << "loaded: " << _objCount << endl;
  if( _world && _lightCount == 0 )
    {  //Add a default light source
      (_world->getScene())->addLight( new G3DPointLight() );
    }
  if( _world && (_world->numberOfCameras() == 0) )
    {
      G3DCamera *camera = new G3DCamera;
      camera->setName( "default Camera (automatically generated)" );
      _world->addCamera( camera );
    }
  return( res );
}


int
IO_BaseClass::file_load( MWorldObject *world, const char *fname )
{
  int res;

  _world = world;
  if( initLoad( fname ) == READ_SUCCESS )
    {
      res = loadObjects( world->getScene() );
      return( exitLoad(res) );
    }
  else
    return( READ_ERROR );
}


int
IO_BaseClass::file_save( MWorldObject *world, const char *fname, int flag )
{
  int res;

  _world = world;
  if( initSave( fname, flag ) == WRITE_SUCCESS )
    {
      res = saveObjects( world->getScene() );
      return( exitSave(res) );
    }
  else
    return( WRITE_ERROR );
}


int
IO_BaseClass::import( G3DGroup *root, const char *fname )
{
  int res;

  _world = NULL;
  if( initLoad( fname ) == READ_SUCCESS )
    {
      res = loadObjects( root );
      return( exitLoad(res) );
    }
  else
    return( READ_ERROR );
}


//===================================END=================================
