/*
 * Project : "MindsEye, 3D library"
 * Copyright 1996, 1997. All Rights Reserved.
 * Author(s): M. Kuball
 *
 * $Id: io_base.h,v 1.8 1998/07/23 20:16:01 source Exp $
 *
 *
 * COPYRIGHT
 * =========
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * OVERWIE
 * =======
 * Definition of the base class of the IO module
 *
 */

#ifndef __IO_BASE_H_
#define __IO_BASE_H_


class MWorldObject;
class G3DGroup;
class G3DScene;


//ME3DFilter
/**Base class for all classes that implement loading and saving
   of a 3D file format.
   */
class IO_BaseClass
{
protected:
    ///
  IO_BaseClass( void )
    {
      _objCount   = 0;
      _lightCount = 0;
    }

public:
  virtual ~IO_BaseClass();

    /// TODO return Version if possible?
  virtual int  check_header( char *fname ) = 0;
    ///
  int  file_load( MWorldObject*, const char *fname );
    ///Save a scene to a file. The flag has additional format information.
  int  file_save( MWorldObject*, const char *fname, int flag );
    /// TODO return FormatInfo
  virtual int  getFormatInfo( int type ) = 0;
    ///Import objects from a file. They will be added as children of root.
  int  import( G3DGroup *root, const char *fname );

protected:
    ///Add default light source and camera if not defined in the loaded file
  virtual int  exitLoad( int );
    ///
  virtual int  exitSave( int ) = 0;
    ///Open the file and test the format.
  virtual int  initLoad( const char *fname ) = 0;
    ///Open the file.
  virtual int  initSave( const char *fname, int ) = 0;
    ///
  virtual int  loadObjects( G3DGroup* ) = 0;
    ///
  virtual int  saveObjects( G3DScene* ) = 0;

protected:
  MWorldObject *_world;
  int  _objCount, _lightCount;
};


#endif
