/*
 * Project : "MindsEye, IO library"
 * Copyright 1996, 1997. All Rights Reserved.
 * Author(s): M. Kuball
 *
 * $Id: io_dialogs.h,v 1.7 1998/07/23 20:16:02 source Exp $
 *
 * COPYRIGHT
 * =========
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

#ifndef __IO_DIALOGS__
#define __IO_DIALOGS__

#include "include/config.h"
#include "io_module.h"
#include "kernel/m_module.h"
#include "kernel/world_list.h"

#define NR_FORMATS     12
#define FORMAT_BWIDTH 250
#define BUTTON_HEIGHT  22

class QListBox;
class QPushButton;
class G3DGroup;
class IO_BaseClass;
class MWorldObject;

typedef IO_BaseClass* IoFactory( void );


/**The class provides an interface for storing and accesing IO filter.
   There are some build-in standarf filter like the MindsEye format and
   nff.
   Objects of this class provide a GUI to access the IO filter to load and
   store 3d data.
 */
class MEIoModule : public MEModule
{
    Q_OBJECT

public:
    ///
  MEIoModule( int, int );
  ~MEIoModule( );

  static  int addFilter( IoFactory* fnt, const char* ename,
			 const char *ftype );
    ///
  static  IO_BaseClass* getIoFilter( int, const char* = NULL );
  static  void  initStandardFilter( void );
    ///Specify if you want to load or save a file.
  void  setAction( int action );
    /**Root must be set if you want to import objects from a file.
       The root will become the parent of all objects in the file.*/
  void  setRoot( G3DGroup *root ) { _root = root; };

signals:
    ///
  void  objectsImported();

private slots:
    ///
  void	activated(int);
    ///
  void  closeIt();
    /**Create an io object of the selected type. Call the
       load or save method based on the value of _action.*/
  void  doIO();
    ///Set the list of loaded scenes.
  void  setWorldList();
    ///
  void useLoaded(int);

protected:
  //QPushButton *button [NR_FORMATS];
  QPushButton   *_doitButton;
    ///
  QListBox      *_worldList;
    ///Load or save.
  int    _action;
    /// number of the selected item in the combo box
  int    _current_type;
  int    _selNum;
    ///
  G3DGroup    *_root;

  //TODO static map<int, IoFactory*,ltint>  *IoFilterList_;
};


#endif
