/*
 * Project : "MindsEye, IO library"
 * Copyright (c) 1996, 1997. All Rights Reserved.
 * Author(s): M. Kuball
 *
 * $Id: nff.cpp,v 1.13 1998/07/23 20:16:14 source Exp $
 *
 *
 * COPYRIGHT
 * =========
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * OVERWIE
 * =======
 * NFF Format i/o module.
 * Some parts of this code are based on ideas taken from the Generic
 * graphics library (G1.6) from Ekkehard Beier.
 *
 */

#include <stdlib.h>

#include "3dlib/light.h"
#include "3dlib/scene.h"
#include "3dlib/Attributes/texture.h"
#include "3dlib/camera.h"
#include "3dlib/actions.h"
#include "nff.h"
#include "nffobjects.h"
#include "kernel/world_list.h"
#include "kernel/world_obj.h"
#include "io/io_module.h"


class NFFOutputShapes : public GIterator<G3DShape>
{
public:
  NFFOutputShapes( MEIoNff *ioObj, ofstream &out ) :
    _outf(out), _mat( NFFMaterial::outp )
    { _ioObj = ioObj; };
  virtual bool exec( G3DShape& );

private:
  Param       _params[12];
  MEIoNff     *_ioObj;
  ofstream    &_outf;
  NFFMaterial _mat;
};


class NFFSaveLightsAction : public GenericAction
{
public:
  NFFSaveLightsAction( ofstream &out ) :
    GenericAction(A_GENERIC_LIGHTS), outf_(out) {};
  virtual EvalState evaluate( G3DObject* );

private:
  Param    params_[12];
  ofstream &outf_;
};


IO_BaseClass*
createNffFilter( void )
{
  return( new MEIoNff );
}


//----------------------------------------------------------------


int
MEIoNff::check_header( char* )
{
  return( 0 );  //TODO undefined
}


int
MEIoNff::exitLoad( int )
{
  _outpf.close();
  return( IO_BaseClass::exitLoad(1) );
}


int
MEIoNff::exitSave( int )
{
  cout << "loaded " << _objCount << " Objects" << endl;
  _inpf.close();
  return( 1 );
}


int
MEIoNff::getFormatInfo( int type )
{
  return( 0 ); //TODO return a FormatInfo object
}


int
MEIoNff::initLoad( const char *fname )
{
  int result;

  _inpf.open( fname );
  if( !_inpf.good() )
    {
      printf("# Open Input failed. Error %ld.\n", (long)result);
      return( READ_ERROR );
    }
  else
    return( READ_SUCCESS );
}

int
MEIoNff::initSave( const char *fname, int )
{
  _outpf.open( fname );
  if( !_outpf.good() )
    {
      puts( "# Open Output failed" );
      return( READ_ERROR );
    }
  else
    {
      MCameraList *clist = _world->getCameras();
      MCameraList::iterator cam = clist->begin();
      while( cam != clist->end() )
	NFFCamera::write( _outpf, *(cam++) );
      return( WRITE_SUCCESS );
    }
}

int
MEIoNff::loadObjects( G3DGroup *group )
{
  static char nexttok[256];
  NFFMaterial tSurf( NFFMaterial::inp );
  NFFObject   *tmp = NULL;
  NFFBox      box;
  NFFCone     cone;
  NFFCurve    curve;
  NFFNurbs    nurbs;
  NFFPPatch   ppatch;
  NFFPolygon  polygon;
  NFFSphere   sphere;
  char cmd;
  int  camNum = 0;

  nexttok[0] = '\0';

  while( !_inpf.eof() )
    {
      if( nexttok[0] )
	{
	  cmd = nexttok[0];
	  nexttok[0] = '\0';
	}
      else
	{
	  _inpf >> cmd;
	  if( !_inpf ) //TODO ??
	    break;
	}
      switch (cmd)
	{
	case 'v':
	  {
	    NFFCamera ncam( _inpf, nexttok );
	    if( _world )
	      {
		G3DCamera *cam = ncam.buildG3DCamera();
		char buf[20];
		sprintf( buf, "Camera %i", ++camNum );
		cam->setName( buf );
		_world->addCamera( cam );
		tmp = NULL;
	      }
	    break;
	  }
	case 'b':    // Background
	  if( _world )
	    {
	      float r, g, b;
	      _inpf >> r >> g >> b;
	      //group->setBackground( G3DColorRGB(r, g, b) );
	    }
	  break;
	case 'l':
	  {
	    NFFLight light( _inpf, nexttok );
	    if( light.isValid() )
	      {
		group->addLight( light.buildG3DLight() );
		_lightCount++;
	      }
	    tmp = NULL;
	    break;
	  }
	case 'f':     // Material
	  {
	    tSurf.read( _inpf );
	    tmp = NULL;  //not an object
	    break;  
	  }
	case 's':
	  {
	    sphere.read( _inpf );
	    tmp = &sphere;
	    break;
	  }
	case 'B':
	  {
	    box.read( _inpf );
	    tmp = &box;
	    break;
	  }
	case 'c':  //cone or cylinder
	  {
	    cone.read( _inpf );
	    tmp = &cone;
	    break;
	  }
	case 'p':
	  {
	    char test = _inpf.peek();
	    if( test == 'p' )
	      {
		_inpf >> cmd;
		ppatch.read( _inpf );
		tmp = &ppatch;
	      }
	    else if( test == 'c' )
	      {
		_inpf >> cmd;
		curve.read( _inpf );
		tmp = &curve;
	      }
	    else if( test == 'n' )
	      {
		_inpf >> cmd;
		nurbs.read( _inpf );
		tmp = &nurbs;
	      }
	    else
	      {
		polygon.read( _inpf );
		tmp = &polygon;
	      }
	    break;
	  }
	case '#':  //Comment
	  {
	    _inpf.getline(nexttok, 256);
	    nexttok[0] = '\0';
	    break;
	  }
	default:
	  tmp = NULL;  //TODO ignore it ?? better report an error
	}

      if( tmp && tmp->isValid() )
	{
	  G3DShape *gobj = tmp->buildG3DShape();
	  gobj->setAttribute( tSurf.buildG3DColor() );
	  group->addShape( gobj );
	  _objCount++;
	}
      tmp = NULL;
    }
  return( 1 );
}

bool
NFFOutputShapes::exec( G3DShape& t )
{
    if( !(t.getType() & T_GROUP) )
      {
	G3DMatrix    m;
	G3DAListIterator atr = t.getAttribute( TA_TEXTURE );
	if( *atr ) //TODO
	  _mat.write( _outf, *atr );
	t.getParameter( _params );
	t.getWorldTrafo( m );
	switch( t.getType() )
	  {
	  case T_BOX:
	    NFFBox::write( _outf, _params, m );
	    break;
	  case T_CYLINDER:
	    NFFCone::write( _outf, _params, m, 1);
	    break;
	  case T_CONE:
	    NFFCone::write( _outf, _params, m, 0);
	    break;
	  case T_POLYGON:
	    NFFPolygon::write( _outf, _params, m );
	    break;
	  case T_SPHERE:
	    NFFSphere::write( _outf, _params, m );
	    break;
	  case T_NURBS_CURVE:
	    NFFCurve::write( _outf, _params, m );
	    break;
	  case T_NURBS_SURFACE:
	  case T_ROT_SURFACE:
	    NFFNurbs::write( _outf, _params, m );
	    break;
	  }
      }
    /*
    else if( t.getType() & T_GROUP )
      _ioObj->saveObjects( (G3DGroup*)&t );
      */
    return( false );
}

EvalState
NFFSaveLightsAction::evaluate( G3DObject *l )
{
  if( l->getType() == T_POINT_LIGHT )
    {
      l->getParameter( params_ );
      outf_ << "l " << l->getOrigin() << ' ' << *(params_[0].color) << endl;
    }
  return( continueEval );
}

int
MEIoNff::saveObjects(  G3DScene *group )
{
  NFFSaveLightsAction laction( _outpf );
  group->doAction( laction );

  //iterate over all elements of the scene
  NFFOutputShapes its( this, _outpf );
  group->doWithShapes( its );

  return( 0 );
}


//====================================END================================
