/*
 * Project : "MindsEye, IO library"
 * Copyright (c)1996, 1997. All Rights Reserved.
 * Author(s): M. Kuball
 *
 * $Id: nffobjects.cpp,v 1.12 1998/07/23 20:16:15 source Exp $
 *
 *
 * COPYRIGHT
 * =========
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * OVERWIE
 * =======
 * implementation of classes representing nff objects.
 * Some parts of this code are based on ideas taken from the Generic
 * graphics library (G1.6) from Ekkehard Beier.
 *
 */

#include <stdlib.h>
#include <stdio.h>


#include "specialType.hh"
#include "3dlib/g3d_math.h"
#include "3dlib/light.h"
#include "3dlib/s_glshapes.h"
#include "3dlib/s_simple.h"
#include "3dlib/s_line.h"
#include "3dlib/s_nurbsurf.h"
#include "3dlib/camera.h"
#include "3dlib/Attributes/texture.h"
#include "3dlib/math/tools.h"

#include "nffobjects.h"


void
prerror( char *str )
{
  puts( str );
}


//======================================================================


/*TODO
NFFBox::NFFBox( G3DS_Box *box )
{
  box->getGeometry();
  x = 0;
}
*/

G3DShape*
NFFBox::buildG3DShape( void )
{
  return( new G3DS_Box( G3DPoint3d(x,y,z1), G3DVector3d(0,y1-y,0),
			G3DVector3d(0,0,z-z1), G3DVector3d(x1-x,0,0) ) );
}

void
NFFBox::read( ifstream &inpf )
{
  inpf >> x >> y >> z >> x1>> y1>> z1;
}

void
NFFBox::write( ofstream &opf, Param *list, G3DMatrix &m )
{
  G3DPoint3d p1( *(list->point3) );
  G3DPoint3d p2( *(list->point3) );

  minmax( p1, p2, *(list[1].vector3), *(list[2].vector3), *(list[3].vector3) );
  opf << "B " << p1 << " " << p2 << endl;
}


//======================================================================


NFFCamera::NFFCamera( ifstream &inpf, char *nexttok )
{
  int xres, yres;  //TODO values are currently ignored
			  
  inpf >> nexttok;
  if( strcmp(nexttok, "from") )
    prerror("\"from\" expected");
  inpf >> fromx >> fromy >> fromz >> nexttok;
  
  if( strcmp(nexttok, "at") )
    prerror("\"at\" expected");
  inpf >> atx >> aty >> atz >> nexttok;
  
  if (strcmp(nexttok, "up"))
    prerror("\"up\" expected");
  inpf >> upx >> upy >> upz >> nexttok;
  
  if (strcmp(nexttok, "angle"))
    prerror("\"angle\" expected");
  inpf >> zoom >> nexttok;
  
  if (strcmp(nexttok, "hither"))
    prerror("\"hither\" expected");
  inpf >> near >> nexttok;
  
  if (strcmp(nexttok, "resolution"))
    prerror("\"resolution\" expected");
  inpf >> xres >> yres;
  
  nexttok[0] = '\0';
}


G3DCamera*
NFFCamera::buildG3DCamera( void )
{
  G3DPoint3d  pos( fromx, fromy, fromz );
  G3DVector3d poi( atx, aty, atz );
  G3DVector3d up ( upx, upy, upz );

  //  poi += (pos * -1.0); 
  // (atx,aty,atz) is the look_at point in world space, no need to tweak it [fm]
  G3DCamera *tmp = new G3DCamera(  pos, poi, up );
  tmp->setFOV( zoom );
/*TODO Frank
!   //  tmp->setZNear(near);  tmp->setZFar(near*100 );
!   //  poi += (pos * -1.0);
!   tmp->setUVN( pos, poi, up );
  */

  return( tmp );
}


void
NFFCamera::write( ofstream &opf, const G3DCamera *cam )
{
  opf << "v"     << endl;
  opf << "from " << cam->getPosition() << endl;
  opf << "at "   << cam->getLookAt() << endl;
  opf << "up "   << cam->getUp() << endl;
  opf << "angle " << cam->getFOV() << endl;
  opf << "hither 1" << endl;
  opf << "resolution 100 100" << endl;
}


//======================================================================


G3DShape*
NFFCone::buildG3DShape( void )
{
  G3DPoint3d  orig ( aX, aY, aZ );
  G3DPoint3d  orig2( bX, bY, bZ );
  G3DVector3d orientation, mao, mio;

  orientation = orig2 - orig;

  if( bR == aR )
    {
      mao = orientation.getOrtho();
      mao.normalize();
      mao *= aR;
      mio = orientation.cross( mao );
      mio.normalize();
      mio *= aR;
      return new G3DS_Cylinder( orientation, mao, mio, orig );
    }
  else
    {
      Param pList[5];
      pList[0].point3  = &orig;
      pList[1].vector3 = &orientation;
      pList[2].dbl     = aR;
      pList[3].dbl     = bR;
      G3DS_Cone *cone  = new G3DS_Cone;
      cone->setParameter( pList );
      return cone;
    }
}

void
NFFCone::read( ifstream &inpf )
{
  inpf >> aX >> aY >> aZ >> aR;
  inpf >> bX >> bY >> bZ >> bR;
}

void
NFFCone::write(  ofstream &opf, Param *list, G3DMatrix &m, int type )
{
  double r1, r2;

  opf << "c" << endl;
  G3DPoint3d p( *(list->point3) );
  opf << (m * p);

  list++;
  p += *(list->vector3);
  list++;
  if( type == 1 )
    r1 = r2 = (list->vector3)->length();
  else
    {
      r1 = list->dbl;
      list++;
      r2 = list->dbl;
    }
  opf << " " << r1 << endl;
  opf << (m * p) << " " << r2 << endl;
}

//======================================================================


NFFCurve::~NFFCurve()
{
  if( _used == 0 )
    delete _curve;
}

G3DShape*
NFFCurve::buildG3DShape( void )
{
  _used = 1;
  return( _curve );
}

void
NFFCurve::read( ifstream &inpf )
{
  int  deg, knum, pnum;
  inpf >> deg >> knum >> pnum;

  Vector_FLOAT     knot( knum );
  Vector_CtrlPnt4D cpoints( pnum );

  inpf >> knot;
  inpf >> cpoints;
  if( _used == 0 )
    delete _curve;
  else
    _used = 0;
  _curve = new G3DNurbsCurve( cpoints, knot, deg );
}

void
NFFCurve::write( ofstream &opf, Param *list, G3DMatrix &m )
{
  Vector_CtrlPnt4D *cpoints;
  float     *knots;
  int       knum, pnum;
  
  opf << "pc" << endl;
  opf << list->flt << " "; list++;   //degree

  knots = list->fltList;  list++;
  knum  = list->number;  list++;
  opf << knum << " ";

  cpoints = list->vCtrlPt;
  pnum    = cpoints->n;
  opf << pnum << endl;
  for( ; knum > 0; knum-- )
    opf << " " << *(knots++);
  opf << endl;

  MatrixRT<float> trafo;
  makeMatrix( trafo, m );
  for( int i = 0; i < pnum; ++i )
    opf << trafo * (*cpoints)[i] << endl;
}


//======================================================================


NFFLight::NFFLight( ifstream &inpf, char *nexttok )
{
  char cmd, *sc;
  /*
  // rectangular light source
  if (inpf.peek() == 'r')
    {   
      inpf >> cmd;
    
      GFloat ax, ay, az;
      GFloat width, height;
      int Wres, Hres;
      int min, max, cand;
	    
      inpf >> lx >> ly >> lz;
      inpf >> ax >> ay >> az;
      inpf >> width >> height;
      inpf >> Wres >> Hres;
      inpf >> min >> max >> cand;
    
      sRectL++;
  }

  // spherical light source
  if( inpf.peek() == 's' )
  {
  inpf >> cmd;
    
    GFloat r;
    int Wres, Hres;
    inpf >> lx >> ly >> lz >> r >>Wres >> Hres ;
    tLight = new GXSphericalLight(GPoint3(lx, ly, lz), r, Wres, Hres);
    sSphL++;
  }
  */  
  // point light source
  //else {
  inpf >> pos;
  
  // color of the light source. It's optional so test for it.
  streampos tPos = inpf.tellg();
  inpf >> nexttok;
  cx = strtod(nexttok, &sc);
  if (sc == &(nexttok[0]))
    {
      inpf.seekg(tPos);
      nexttok[0] = '\0';
      cx = cy = cz = 1.0;
    }
  else
    {
      inpf >> cy >> cz;
      nexttok[0] = '\0';
    }
}


G3DLight*
NFFLight::buildG3DLight( void )
{
  G3DPointLight *tmp = new G3DPointLight;
  tmp->setOrigin( pos );
  tmp->setColor( G3DColor(cx, cy, cz) );
  return( tmp );
}


//======================================================================

NFFNurbs::~NFFNurbs()
{
  if( !used_ )
    delete nurbs_;
}

G3DShape*
NFFNurbs::buildG3DShape( void )
{
  used_ = true;
  return( nurbs_ );
}


void
NFFNurbs::read( ifstream &inpf )
{
  int  udeg, vdeg, uknum, vknum, rows, cols;
  inpf >> udeg >> vdeg >> uknum >> vknum >> rows >> cols;

  Vector_FLOAT     uknot( uknum ), vknot( vknum );
  Matrix_CtrlPnt4D cpoints( rows, cols );

  inpf >> uknot;
  inpf >> vknot;
  inpf >> cpoints;

  if( !used_ )
    delete nurbs_;
  else
    used_ = false;
  nurbs_ = new G3DNurbsSurface( udeg, vdeg, uknot, vknot, cpoints );
}

void
NFFNurbs::write( ofstream &opf, Param *list, G3DMatrix &m )
{
  Matrix_CtrlPnt4D *cpoints;
  float     *uknots, *vknots;
  int       uknum, vknum, upnum, vpnum;
  
  opf << "pn" << endl;
  opf << list->flt << ' '; list++;  //u-degree
  opf << list->flt << ' '; list++;  //v-degree
  uknots  = list->fltList;  list++;
  uknum   = list->number;   list++;
  vknots  = list->fltList;  list++;
  vknum   = list->number;   list++;
  cpoints = list->mCtrlPt;
  upnum   = cpoints->rows;
  vpnum   = cpoints->cols;

  opf << uknum << ' ' << vknum << ' ' << upnum << ' ' << vpnum << endl;

  for( ; uknum > 0; --uknum )
    opf << *(uknots++) << ' ';
  opf << endl;
  for( ; vknum > 0; --vknum )
    opf << *(vknots++) << ' ';
  opf << endl;

  MatrixRT<float> trafo;
  makeMatrix( trafo, m );
  for( int i = 0; i < cpoints->rows; ++i )
    for( int j = 0; j < cpoints->cols; ++j )
      opf << trafo * (*cpoints)(i,j) << endl;  
}


//======================================================================


G3DShape*
NFFObject::buildG3DShape( void )
{
  return( NULL );
}


//======================================================================


NFFPPatch::NFFPPatch( void )
{
  vertices_ = NULL;
  normals_  = new G3DVector3d[10];
  tMax_     = 10;
}

NFFPPatch::~NFFPPatch()
{
  delete[] vertices_;
  delete[] normals_;
}

G3DShape*
NFFPPatch::buildG3DShape( void )
{
    /*	 TODO     
  if (tNum >  3)
    tPoly = new GTSmoothPolygon(tNum, tVerts, tNorms);
  else
    tPoly = new GTSmoothTriangle(tVerts, tNorms);
    */

  G3DS_Polygon *tmp = new G3DS_Polygon;

  Param params[6];
  tmp->getParameter( params );
  params[0].vector3 = vertices_;
  params[1].number  = tNum_;
  G3DVector3d n1, n2;
  n1 = vertices_[0] - vertices_[1];
  n2 = vertices_[2] - vertices_[1];
  *(params[2].vector3) = n2.cross( n1 );
  params[2].vector3->normalize();

  tmp->setParameter( params );
  vertices_ = NULL;
  return( tmp );
}

void
NFFPPatch::read( ifstream &inpf )
{
  inpf >> tNum_;
  if( tNum_ > 10 )
    {
      delete[] normals_;
      normals_  = new G3DVector3d[tNum_];
      tMax_     = tNum_;
    }
  delete[] vertices_;
  vertices_ = new G3DPoint3d[tNum_];
  for( int i = 0; i < tNum_ ; i++ )
    {
      inpf >> vertices_[i] >> normals_[i];
    }
}


//======================================================================


NFFPolygon::NFFPolygon( void )
{
  vertices_ = NULL;
}

NFFPolygon::~NFFPolygon()
{
  delete[] vertices_;
}


G3DShape*
NFFPolygon::buildG3DShape( void )
{
    /*TODO
  if(tNum > 3)
    tPoly = new GTPolygon(tNum, tVerts);
  else
    tPoly = new GTTriangle(tVerts);
    */
  G3DS_Polygon *tmp = new G3DS_Polygon;

  Param params[6];
  tmp->getParameter( params );
  params[0].vector3 = vertices_;
  params[1].number  = tNum_;
  G3DVector3d n1, n2;
  n1 = vertices_[0] - vertices_[1];
  n2 = vertices_[2] - vertices_[1];
  *(params[2].vector3) = n2.cross( n1 );
  params[2].vector3->normalize();

  tmp->setParameter( params );
  vertices_ = NULL;
  return( tmp );
}

void
NFFPolygon::read( ifstream &inpf )
{
  inpf >> tNum_;
  delete[] vertices_;
  vertices_ = new G3DPoint3d[tNum_];
  for( int i = tNum_-1; i >= 0; --i )
    {
      inpf >> vertices_[i];
    }
}

void
NFFPolygon::write(  ofstream &opf, Param *list, G3DMatrix &m )
{
  G3DPoint3d *points;
  uint       num;

  num    = list[1].number;
  points = list->point3;
  points += num-1;

  opf << "p " << num << endl;
  for( ; num > 0; --num )
    {
      opf << (m * *points) << endl;
      points--;
    }
}


//======================================================================


G3DShape*
NFFSphere::buildG3DShape( void )
{
  G3DS_Sphere *sphere = new G3DS_Sphere( r, G3DPoint3d(x, y, z) );
  return( sphere );
}

void
NFFSphere::read( ifstream &inpf )
{
  inpf >> x >> y >> z >> r;
}


void
NFFSphere::write(  ofstream &opf, Param *list, G3DMatrix &m )
{
  G3DPoint3d p( *(list->point3) );
  list++;

  p = m * p;
  opf << "s  " << p;
  opf << " " << list->dbl << endl;
}


//======================================================================


NFFMaterial::NFFMaterial( int iotype )
{
  _orig = 0;
  if( iotype == inp )
    _mat  = new G3DAColor();
  else
    _mat = NULL;
  _last = NULL;
}

NFFMaterial::~NFFMaterial( void )
{
  if( _orig == 0 )
    delete _mat;
}

G3DAttribute*
NFFMaterial::buildG3DColor( void )
{
  if( _orig )
    return( new G3DAReference(_mat) );
  else
    {
      _orig = 1;
      return( _mat );
    }
}

void
NFFMaterial::read( ifstream &inpf )
{
  float r, g, b, kd, ks, shine, t, ior;

  inpf >> r >> g >> b >> kd >> ks >> shine >> t >> ior;
  if( _orig == 0 )
    delete _mat;
  _mat = new G3DAColor;
  _mat->setDiffuseColor( G3DColor(kd*r, kd*g, kd*b) );
  _mat->setSpecularColor( G3DColor(ks*r, ks*g, ks*b) );
  _mat->setShininess( (int)shine );
  _orig = 0;
}

void
NFFMaterial::write(  ofstream &opf, G3DAttribute *mat )
{
  G3DAColor *texture = (G3DAColor*)(mat->getReference());
  if( texture != _last ) //write only if different from last texture
    {
      G3DColor    col;

      _last = texture;
      //TODO use getParameter
      opf << "f ";
      texture->getDiffuseColor( col );
      opf << col << " ";
      texture->getSpecularColor( col );
      opf << col << " " << texture->getSpecularExp()
	  << " 0 0" << endl;
    }
}


//===================================END=================================
