//
// hash.cc
//

#include "stdlib.h"
#include "hash.h"

//
// Allocate data structures for the hash table
//
void
HashTable::init( int l )
{
  typedef void *t_void;

  len   = l;
  adr   = (const void**)(new t_void[l]);
  index = new int[l];
  for( int i = 0; i < l; i++)
    adr[i] = NULL;
}


//
// Free the memory used by the hash table.
//
void HashTable::free()
{
  delete[] adr;
  delete[] index;
}


//
// Add an entry to the hash table using open addressing.
//
void
HashTable::insert( const void* id, int idx)
{
  int	h, inserted;

  inserted = 0;
  h = int(id) % len;
  while( !inserted )
    {
      if( adr[h] == NULL )
	{
	  adr[h]   = id;
	  index[h] = idx;
	  inserted = 1;
	}
      else
	{
	  h++;
	  if (h == len) h = 0; 
	}
    }
}


//
// Find the appropriate entry in the hash table and return the point's index.
//
int HashTable::search( const void *id )
{
  int h, h0;
  
  h = h0 = int(id) % len;
  do {
    if( adr[h] == id )
      return( index[h] );
    else
      {
	h++;
	if( h == h0 )
	  break;
	if( h == len )
	  h = 0;
      }
  } while( 1 );
  return( -1 );
}
