/*
 * Project : "MindsEye, IO library"
 * Copyright (c)1996, 1997. All Rights Reserved.
 * Author(s): M. Kuball
 *
 * $Id: nffobjects.cpp,v 1.7 1998/03/08 17:24:04 source Exp $
 *
 *
 * COPYRIGHT
 * =========
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * OVERWIE
 * =======
 * implementation of classes representing nff objects.
 * Some parts of this code are based on ideas taken from the Generic
 * graphics library (G1.6) from Ekkehard Beier.
 *
 */

#include <stdlib.h>
#include <stdio.h>


#include "specialType.H"
#include "3dlib/g3d_math.h"
#include "3dlib/light.h"
#include "3dlib/s_glshapes.h"
#include "3dlib/s_simple.h"
#include "3dlib/s_line.h"
#include "3dlib/camera.h"

#include "nffobjects.h"


void
prerror( char *str )
{
  puts( str );
}


//======================================================================


/*TODO
NFFBox::NFFBox( G3DS_Box *box )
{
  box->getGeometry();
  x = 0;
}
*/

G3DShape*
NFFBox::buildG3DShape( void )
{
  return( new G3DS_Box( G3DPoint3d(x,y,z1), G3DVector3d(0,y1-y,0),
			G3DVector3d(0,0,z-z1), G3DVector3d(x1-x,0,0) ) );
}

void
NFFBox::read( ifstream &inpf )
{
  inpf >> x >> y >> z >> x1>> y1>> z1;
}

void
minmax( G3DPoint3d &p1, G3DPoint3d &p2 )
{
  double s, t;

  s = p1[0];  t = p2[0];
  if( s > t )
    {
      p1[0] = t;  p2[0] = s;
    }
  s = p1[1];  t = p2[1];
  if( s > t )
    {
      p1[1] = t;  p2[1] = s;
    }
  s = p1[2];  t = p2[2];
  if( s < t )
    {
      p1[2] = t;  p2[2] = s;
    }
}

void
NFFBox::write( ofstream &opf, Param *list, G3DMatrix &m )
{
  G3DPoint3d p1( *(list->point) );
  G3DPoint3d p2( *(list->point) );
  list++;
  p2 += *(list->vector);
  list++;
  p2 += *(list->vector);
  list++;
  p2 += *(list->vector);
  p1 = m * p1;
  p2 = m * p2;
  minmax( p1, p2 ); 
  opf << "B " << p1 << " " << p2 << endl;
}


//======================================================================


NFFCamera::NFFCamera( ifstream &inpf, char *nexttok )
{
  int xres, yres;  //TODO values are currently ignored
			  
  inpf >> nexttok;
  if( strcmp(nexttok, "from") )
    prerror("\"from\" expected");
  inpf >> fromx >> fromy >> fromz >> nexttok;
  
  if( strcmp(nexttok, "at") )
    prerror("\"at\" expected");
  inpf >> atx >> aty >> atz >> nexttok;
  
  if (strcmp(nexttok, "up"))
    prerror("\"up\" expected");
  inpf >> upx >> upy >> upz >> nexttok;
  
  if (strcmp(nexttok, "angle"))
    prerror("\"angle\" expected");
  inpf >> zoom >> nexttok;
  
  if (strcmp(nexttok, "hither"))
    prerror("\"hither\" expected");
  inpf >> near >> nexttok;
  
  if (strcmp(nexttok, "resolution"))
    prerror("\"resolution\" expected");
  inpf >> xres >> yres;
  
  nexttok[0] = '\0';
}


G3DCamera*
NFFCamera::buildG3DCamera( void )
{
  G3DPoint3d  pos( fromx, fromy, fromz );
  G3DVector3d poi( atx, aty, atz );
  G3DVector3d up ( upx, upy, upz );

  //  poi += (pos * -1.0); 
  // (atx,aty,atz) is the look_at point in world space, no need to tweak it [fm]
  G3DCamera *tmp = new G3DCamera(  pos, poi, up );
  tmp->setFOV( zoom );
/*TODO Frank
!   //  tmp->setZNear(near);  tmp->setZFar(near*100 );
!   //  poi += (pos * -1.0);
!   tmp->setUVN( pos, poi, up );
  */

  return( tmp );
}


G3DShape*
NFFCamera::buildG3DShape( void )
{
  return( NULL );
}

void
NFFCamera::write( ofstream &opf, G3DCamera *cam )
{
  opf << "v"     << endl;
  opf << "from " << cam->getPosition() << endl;
  opf << "at "   << cam->getLookAt() << endl;
  opf << "up "   << cam->getUp() << endl;
  opf << "angle " << cam->getFOV() << endl;
  opf << "hither 1" << endl;
  opf << "resolution 100 100" << endl;
}


//======================================================================


G3DShape*
NFFCone::buildG3DShape( void )
{
  G3DPoint3d  orig ( aX, aY, aZ );
  G3DPoint3d  orig2( bX, bY, bZ );
  G3DVector3d orientation, mao, mio;

  orientation = orig2 - orig;

  if( bR == aR )
    {
      mao = orientation.getOrtho();
      mao.normalize();
      mao *= aR;
      mio = orientation.cross( mao );
      mio.normalize();
      mio *= aR;
      return new G3DS_Cylinder( orientation, mao, mio, orig );
    }
  else
    {
      return NULL;
      //tmp = new G3DS_Cone();
    }
}

void
NFFCone::read( ifstream &inpf )
{
  inpf >> aX >> aY >> aZ >> aR;
  inpf >> bX >> bY >> bZ >> bR;
}

void
NFFCone::write(  ofstream &opf, Param *list, G3DMatrix &m, int type )
{
  double r1, r2;

  opf << "c" << endl;
  G3DPoint3d p( *(list->point) );
  opf << (m * p);

  list++;
  p += *(list->vector);
  list++;
  if( type == 1 )
    r1 = r2 = (list->vector)->length();
  else
    {
      r1 = list->dbl;
      list++;
      r2 = list->dbl;
    }
  opf << " " << r1 << endl;
  opf << (m * p) << " " << r2 << endl;
}

//======================================================================


NFFCurve::~NFFCurve()
{
  if( _used == 0 )
    delete _curve;
}

G3DShape*
NFFCurve::buildG3DShape( void )
{
  _used = 1;
  return( _curve );
}


void
NFFCurve::read( ifstream &inpf )
{
  //CtrlPnt4D cpt;
  int       deg, knum, pnum;

  inpf >> deg >> knum >> pnum;
  Vector_FLOAT     knot( knum );
  Vector_CtrlPnt4D cpoints( pnum );

  inpf >> knot;
  inpf >> cpoints;
  /*
  for( int i = 0; i < knum; i++ )
    inpf >> uknot;
  for( int i = 0; i < upoints; i++ )
    {
      inpf >> *(cpt++);
    }
    */
  if( _used == 0 )
    delete _curve;
  else
    _used = 0;
  _curve = new G3DNurbsCurve( cpoints, knot, deg );
}

void
NFFCurve::write( ofstream &opf, Param *list, G3DMatrix &m )
{
  CtrlPnt4D *cpoints;
  float     *knots;
  int       knum, pnum;
  
  opf << "pc" << endl;
  opf << list->flt << " ";  //degree
  list++;
  knum = list->number;  list++;
  opf << knum << " ";
  knots = list->fltList;  list++;
  pnum  = list->number;   list++;
  opf << pnum << endl;
  for( ; knum > 0; knum-- )
    opf << " " << *(knots++);
  opf << endl;
  cpoints = list->ctrlPt;
  for( ; pnum > 0; pnum-- )
    opf << " " << *(cpoints++) << endl;
}


//======================================================================


NFFLight::NFFLight( ifstream &inpf, char *nexttok )
{
  char cmd, *sc;
  /*
  // rectangular light source
  if (inpf.peek() == 'r')
    {   
      inpf >> cmd;
    
      GFloat ax, ay, az;
      GFloat width, height;
      int Wres, Hres;
      int min, max, cand;
	    
      inpf >> lx >> ly >> lz;
      inpf >> ax >> ay >> az;
      inpf >> width >> height;
      inpf >> Wres >> Hres;
      inpf >> min >> max >> cand;
    
      sRectL++;
  }

  // spherical light source
  if( inpf.peek() == 's' )
  {
  inpf >> cmd;
    
    GFloat r;
    int Wres, Hres;
    inpf >> lx >> ly >> lz >> r >>Wres >> Hres ;
    tLight = new GXSphericalLight(GPoint3(lx, ly, lz), r, Wres, Hres);
    sSphL++;
  }
  */  
  // point light source
  //else {
  inpf >> pos.x >> pos.y >> pos.z;
  
  // color of the light source. It's optional so test for it.
  streampos tPos = inpf.tellg();
  inpf >> nexttok;
  cx = strtod(nexttok, &sc);
  if (sc == &(nexttok[0]))
    {
      inpf.seekg(tPos);
      nexttok[0] = '\0';
      cx = cy = cz = 1.0;
    }
  else
    {
      inpf >> cy >> cz;
      nexttok[0] = '\0';
    }
}


G3DShape*
NFFLight::buildG3DShape( void )
{
  return( NULL );
}

G3DLight*
NFFLight::buildG3DLight( void )
{
  VectorStruct attenuation = {1,0,0};

  G3DPointLight *tmp = new G3DPointLight( &pos, &attenuation );
  tmp->setColor( G3DColor(cx, cy, cz) );
  return( tmp );
}


//======================================================================


G3DShape*
NFFNurbs::buildG3DShape( void )
{
  //new G3DNURBSurface()
  return( NULL );
}


void
NFFNurbs::read( ifstream &inpf )
{
  /*
  CtrlPnt4D *cpoints, *cpt;
  int udeg, vdeg, uknum, vknum, upnum, vpnum;

  inpf >> udeg >> vdeg >> uknum >> vknum >> upnum >> vpnum;
  cpt = cpoints = new CtrlPnt4D[pnum];

  for( int i = knum; i > 0; i-- )
    inpf >> uknot;
  for( int i = 0; i < upoints; i++ )
    {
      inpf >> *(cpt++);
    }
    */
}

void
NFFNurbs::write( ofstream &opf, Param *list, G3DMatrix &m )
{
  /*
  CtrlPnt4D *cpoints;
  float     *uknots, *vknots;
  int       uknum, vknum, upnum, vpnum;
  
  opf << "pn" << endl;
  opf << list->flt << " ";  //u-degree
  list++;
  opf << list->flt << " ";  //v-degree
  list++;
  uknum = list->number;  list++;
  vknum = list->number;  list++;
  opf << uknum << ' ' << vknum << ' ';
  uknots = list->fltList;  list++;
  pnum = list->number;
  opf << pnum << endl;
  list++;
  cpoints = list->ctrlPt;
  for( ; knum > 0; knum-- )
    opf << " " << *(knots++);
  opf << endl;
  for( ; pnum > 0; pnum-- )
    opf << " " << *(cpoints++) << endl;
    */
}


//======================================================================


NFFPPatch::NFFPPatch( void )
{
  _vertices = _normals = NULL;
}

NFFPPatch::~NFFPPatch()
{
  if( _vertices && (_tNum > 10) )
    {
      delete [] _vertices;
      delete [] _normals;
    }
}

G3DShape*
NFFPPatch::buildG3DShape( void )
{
    /*	      
  if (tNum >  3)
    tPoly = new GTSmoothPolygon(tNum, tVerts, tNorms);
  else
    tPoly = new GTSmoothTriangle(tVerts, tNorms);
    */
  // TODO for now return Polygon
  G3DS_Polygon *tmp = new G3DS_Polygon(_tNum, _vertices);
  if( _tNum > 10 )
    {
      delete[] _vertices;
      delete[] _normals;
      _vertices = _normals  = NULL;
    }
  return( tmp );
}

void
NFFPPatch::read( ifstream &inpf )
{
  float x, y, z, nx, ny, nz;
  //int   tNum;

  inpf >> _tNum;
  if( _tNum > 10 )
    {
      _vertices = new VectorStruct[_tNum];
      _normals  = new VectorStruct[_tNum];
    }
  else
    { //use object storage
      _vertices = _vert10;
      _normals  = _norm10;
    }
  for (register int i = 0; i < _tNum ; i++)
    {
      inpf >> x >> y >> z >> nx >> ny >> nz;
      _vertices[i].x = x;
      _vertices[i].y = y;
      _vertices[i].z = z;
      //_normals[i].setTo(nx, ny, nz);
    }
}


//======================================================================


NFFPolygon::NFFPolygon( void )
{
  _vertices = NULL;
}

NFFPolygon::~NFFPolygon()
{
  if( _vertices && (_tNum > 10) )
    delete[] _vertices;
}


G3DShape*
NFFPolygon::buildG3DShape( void )
{
    /*TODO
  if(tNum > 3)
    tPoly = new GTPolygon(tNum, tVerts);
  else
    tPoly = new GTTriangle(tVerts);
    */
  G3DS_Polygon *tmp = new G3DS_Polygon( _tNum, _vertices );
  if( _tNum > 10 )
    delete[] _vertices;
  _vertices = NULL;
  return( tmp );
}

void
NFFPolygon::read( ifstream &inpf )
{
  VectorStruct *tmp;

  inpf >> _tNum;
  if( _tNum > 10 )
    _vertices = new VectorStruct[_tNum];
  else
    _vertices = _vert10;
  tmp = &(_vertices[_tNum-1]);
  for( int i = _tNum; i > 0; i-- )
    {
      inpf >> tmp->x >> tmp->y >> tmp->z;
      tmp--;
    }
}

void
NFFPolygon::write(  ofstream &opf, Param *list, G3DMatrix &m )
{
  G3DPoint3d *points;
  uint       num;

  num = list->number;
  list++;
  points = list->point;
  points += num-1;

  opf << "p " << num << endl;
  for( ; num > 0; num-- )
    {
      opf << (m * *points) << endl;
      points--;
    }
}


//======================================================================


G3DShape*
NFFSphere::buildG3DShape( void )
{
  G3DS_Sphere *sphere = new G3DS_Sphere( r, G3DPoint3d(x, y, z) );
  return( sphere );
}

void
NFFSphere::read( ifstream &inpf )
{
  inpf >> x >> y >> z >> r;
}


void
NFFSphere::write(  ofstream &opf, Param *list, G3DMatrix &m )
{
  G3DPoint3d p( *(list->point) );
  list++;

  p = m * p;
  opf << "s  " << p;
  opf << " " << list->dbl << endl;
}


//======================================================================


NFFMaterial::NFFMaterial( int iotype )
{
  _orig = 0;
  if( iotype == inp )
    _mat  = new G3DATexture( G3DColor(0.8, 0.8, 0.8),
			     G3DColor(0, 0, 0), 0 );
  else
    _mat = NULL;
  _last = NULL;
}

NFFMaterial::~NFFMaterial( void )
{
  if( _orig == 0 )
    delete _mat;
}

G3DAttribute*
NFFMaterial::buildG3DTexture( void )
{
  if( _orig )
    return( new G3DAReference(_mat) );
  else
    {
      _orig = 1;
      return( _mat );
    }
}

void
NFFMaterial::read( ifstream &inpf )
{
  float r, g, b, kd, ks, shine, t, ior;

  inpf >> r >> g >> b >> kd >> ks >> shine >> t >> ior;
  if( _orig == 0 )
    delete _mat;
  _mat = new G3DATexture( G3DColor(kd*r, kd*g, kd*b),
	 		  G3DColor(ks*r, ks*g, ks*b), shine );
  /*
  tSurf->setTransparency(t);
  tSurf->setRefraction(ior);
  */
  _orig = 0;
}

void
NFFMaterial::write(  ofstream &opf, G3DAttribute *mat )
{
  G3DATexture *texture = (G3DATexture*)(mat->getReference());
  if( texture != _last )
    {
      G3DColor    col;

      _last = texture;

      opf << "f ";
      texture->getDiffuseColor( col );
      opf << col << " ";
      texture->getSpecularColor( col );
      opf << col << " " << texture->getSpecularExp()
	  << " 0 0" << endl;
    }
}


//===================================END=================================
