/*
 * Project : "MindsEye, IO library"
 * Copyright (c) 1996, 1997. All Rights Reserved.
 * Author(s): M. Kuball
 *
 * $Id: nff.cpp,v 1.7 1998/03/08 17:24:03 source Exp $
 *
 *
 * COPYRIGHT
 * =========
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * OVERWIE
 * =======
 * Povray  Format output filter.
 *
 */

#include "include/config.h"
#include <stdlib.h>

#include "include/version.h"
#include "3dlib/light.h"
#include "3dlib/scene.h"
#include "3dlib/Attributes/texture.h"
#include "3dlib/camera.h"
#include "pov.h"
#include "povobjects.h"

#include "kernel/world_list.h"
#include "kernel/world_obj.h"
//#include "kernel/m_core.h"
#include "io/io_module.h"
#include "hash.h"


class PovOutputShapes : public GIterator<G3DShape>
{
public:
  PovOutputShapes( MEIoPov *ioObj, ostream &os, ostream &tos ) :
    os_(os), tos_(tos), hashTable_(711)
    { ioObj_ = ioObj; index_ = 0; };
  virtual bool exec( G3DShape& );
  void    prepareTexture( G3DAListIterator &mat );

private:
  Param     params_[20];
  MEIoPov   *ioObj_;
  ostream   &os_, &tos_;
  HashTable hashTable_;
  int       index_;
};

class PovOutputLights : public GenericAction 
{
public:
  PovOutputLights( ofstream &ofs ) :
    GenericAction( A_GENERIC_LIGHTS ), ofs_(ofs) {}

  virtual EvalState evaluate( G3DObject *obj )
  {
    writeLight( ofs_, (G3DLight*)obj );
  }

protected:
  ofstream &ofs_;
};



IO_BaseClass*
createPovFilter( void )
{
  return( new MEIoPov );
}


//----------------------------------------------------------------


int
MEIoPov::check_header( char* )
{
  return( 0 );  //TODO undefined
}


int
MEIoPov::exitLoad( int )
{
  return( 0 );
}


int
MEIoPov::exitSave( int )
{
  setIOSeparator();
  //cout << "saved " << _objCount << " Objects" << endl;
  outf_.close();
  return( 1 );
}


int
MEIoPov::getFormatInfo( int type )
{
  return( 0 ); //TODO return a FormatInfo object
}


int
MEIoPov::initLoad( const char* )
{
  return( READ_ERROR );
}

int
MEIoPov::initSave( const char *fname, int )
{
  outf_.open( fname );
  tos_.open( "mindseye.inc" );  //TODO
  if( !outf_.good() )
    {
      puts( "# Open Output failed" );
      return( WRITE_ERROR );
    }
  else
    {
      setIOSeparator( ',' );
      writeHeader();
      writeCamera( );
      return( WRITE_SUCCESS );  //TODO
    }
}


int
MEIoPov::loadObjects( G3DGroup *group )
{
  return( 0 );
}


bool
PovOutputShapes::exec( G3DShape& t )
{
//references per define?

  t.getParameter( params_ );
  switch( t.getType() )
    {
    case T_GROUP:
      break;
    case T_BOX:
      writeBox( os_, params_ );
      break;
    case T_CONE:
      writeCone( os_, params_ );
      break;
    case T_CYLINDER:
      writeCylinder( os_, params_ );
      break;
    case T_SPHERE:
      writeSphere( os_, params_ );
      break;
    case T_TORUS:
      writeTorus( os_, params_ );
      break;
    case T_POLYGON:
      writePolygon( os_, params_ );
      break;
      /*
    case T_NURBS_CURVE:
      PovCurve::write( os_, params_ );
      break;
    case T_NURBS_SURFACE:
      PovNurbs::write( os_, params_ );
      break;
      */
    default:
      return( false );
    }

  G3DAListIterator atr = t.getAttribute( TA_TEXTURE, false );
  prepareTexture( atr );

  G3DMatrix m;
  t.getWorldTrafo( m );
  writePovFormat( os_, m );
  os_ << "}" << endl << endl;  //close the object description

  return( false );
}

void
PovOutputShapes::prepareTexture( G3DAListIterator &mat )
{
  if( *mat == NULL )
    return;

  const G3DAttribute *atr = (*mat)->getReference();
  if( atr->isReferenced() )
    {
      int i = hashTable_.search( atr );
      if( i < 0 )
	{
	  i = index_;
	  tos_ << "#declare ME_TEXTURE_" << i << " =" << endl;
	  writeTexture( tos_, (const G3DATexture*)atr );
	  hashTable_.insert( atr, index_++ );
	}
      os_ << " texture { ME_TEXTURE_" << i << " }" << endl;
    }
  else
    writeTexture( os_, (const G3DATexture*)atr ); 
}


int
MEIoPov::saveObjects(  G3DScene *group )
{
  //TODO save only top level lights?
  PovOutputLights itl( outf_ );
  group->doAction( itl );

  Param plist[2];
  group->getParameter( plist );
  outf_ << "background { color rgb <" << *(plist->color) << "> }" << endl;
  saveRecursive( group );
}

void
MEIoPov::saveRecursive( G3DGroup *group )
{
  outf_ << "union {" << endl;
  PovOutputShapes its( this, outf_, tos_ );
  group->doWithShapes( its );

  G3DAListIterator atr = group->getAttribute( TA_TEXTURE, false );
  writeTexture( outf_, (const G3DATexture*)(*atr) );
  
  outf_ << "}" << endl;
}

void
MEIoPov::writeCamera( void )
{
  G3DCamera *cam = _world->getActiveCamera();

  if( cam )
    {
      outf_ << "camera { " << endl;
      //if( cam->getType() == Perspective
      outf_ << "  up <0,1,0>" << endl
	    << "  right <" << -1.333 << ",0,0>" << endl;
      outf_ << "  location <" << cam->getPosition() << ">" << endl;
      outf_ << "  look_at <" << cam->getLookAt() << ">" << endl;

      //POV expects the angle in x-direction instead in y as OpenGL.
      double angle = deg2rad( cam->getFOV() );
      angle = 2 * atan( cam->getAspectRatio() * tan(angle/2.0) );
      angle = rad2deg( angle );
      if( angle >= 180 )
	angle = 179.9;
      outf_ << "  angle " << angle << endl;
      outf_ << "}" << endl << endl;
    }
}

void
MEIoPov::writeHeader( void )
{
  outf_ << "//" << endl
	<< "//Generated for POV-Ray(tm) 3.0 by MindsEye "
	<< "V" << MAJOR << "." << MINOR << "." << PATCH << endl
	<<  "//" << endl << endl;
  outf_ << "#include \"colors.inc\"" << endl;
  outf_ << "#include \"mindseye.inc\"" << endl << endl << endl;
}


//====================================END================================
