/*
 * Project : "MindsEye, IO library"
 * Copyright (c)1996, 1997. All Rights Reserved.
 * Author(s): M. Kuball
 *
 * $Id: nffobjects.cpp,v 1.7 1998/03/08 17:24:04 source Exp $
 *
 *
 * COPYRIGHT
 * =========
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * OVERWIE
 * =======
 * implementation of classes representing nff objects.
 * Some parts of this code are based on ideas taken from the Generic
 * graphics library (G1.6) from Ekkehard Beier.
 *
 */

#include <stdlib.h>
#include <stdio.h>


#include "specialType.hh"
#include "3dlib/g3d_math.h"
#include "3dlib/light.h"
//#include "3dlib/s_glshapes.h"
#include "3dlib/s_simple.h"
//#include "3dlib/s_line.h"
#include "3dlib/Attributes/texture.h"
#include "3dlib/Attributes/metal.h"

#include "povobjects.h"

const char str1[] = ">, <";
const char str2[] = ">, ";
const char str3[] = ", <";



void
writeBox( ostream &os, Param *list )
{
  G3DPoint3d p1( *(list->point3) );
  G3DPoint3d p2( *(list->point3) );

  minmax( p1, p2, *(list[1].vector3), *(list[2].vector3), *(list[3].vector3) );
  os << "box { <" << p1 << str1 << p2 << '>' << endl;
}

void
writeCone( ostream &os, Param *list )
{
  os << "cone {  <" << *(list->point3) << str2 << list[2].dbl
     << str3 << *(list->point3) + *(list[1].vector3) << str2
     << list[3].dbl << endl;
}

void
writeCylinder( ostream &os, Param *list )
{
  os << "cylinder {  <" << *(list->point3) << str1
     << *(list->point3) + *(list[1].vector3) << str2
     << (list[2].vector3)->length() << endl;
}


/*
void
write( ofstream &opf, Param *list, G3DMatrix &m )
{
  CtrlPnt4D *cpoints;
  float     *knots;
  int       knum, pnum;
  
  opf << "pc" << endl;
  opf << list->flt << " ";  //degree
  list++;
  knum = list->number;  list++;
  opf << knum << " ";
  knots = list->fltList;  list++;
  pnum  = list->number;   list++;
  opf << pnum << endl;
  for( ; knum > 0; knum-- )
    opf << " " << *(knots++);
  opf << endl;
  cpoints = list->ctrlPt;
  for( ; pnum > 0; pnum-- )
    opf << " " << *(cpoints++) << endl;
}
*/

void
writeLight( ostream &os, G3DLight *lt )
{
  if( !lt->isOn() )
    return;

  if( lt->getType() == T_AMBIENT_LIGHT )
    {
      os << "global_settings { ambient_light color rgb<"
	 << lt->getColor() << "> }" << endl << endl;
    }
  else
    {
      os << "light_source {" << endl;
      os << "  <" << lt->getOrigin() << ">" << endl;
      os << "  color rgb<" << lt->getColor() << ">" << endl;

      /*
	switch( lt->getType() )
	{
	case T_POINT_LIGHT:
	os << 
	*/
      os << "}" << endl << endl;
    }
}

/*
void
PovNurbs::write( ofstream &opf, Param *list, G3DMatrix &m )
{
  CtrlPnt4D *cpoints;
  float     *uknots, *vknots;
  int       uknum, vknum, upnum, vpnum;
  
  opf << "pn" << endl;
  opf << list->flt << " ";  //u-degree
  list++;
  opf << list->flt << " ";  //v-degree
  list++;
  uknum = list->number;  list++;
  vknum = list->number;  list++;
  opf << uknum << ' ' << vknum << ' ';
  uknots = list->fltList;  list++;
  pnum = list->number;
  opf << pnum << endl;
  list++;
  cpoints = list->ctrlPt;
  for( ; knum > 0; knum-- )
    opf << " " << *(knots++);
  opf << endl;
  for( ; pnum > 0; pnum-- )
    opf << " " << *(cpoints++) << endl;
}
*/

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
       Member: writePOVRAY --- writes a set of povray bicubic patches
  Description: Writes a set of povray bicubic patches to the ostream.
        Input: patch_type --> the type of the patch
	         flatness --> the flatness level
	      num_u_steps --> the minimum number of triangles in the U 
	                      direction
	      num_v_steps --> the minimum number of triangles in the V 
	                      direction
       Output: an ostream containing the definition of the surface
 Restrictions: POVRAY only accepts rational spline patches. Thus you can't
               have a value other then 1.0 for the weights of your surface.
	       A warning message will be generated if this is the case.

	       POVRAY only deals with surfaces of degree 3. If the surface 
	       as a degree below 3 either in the U or V direction it will be
	       elevated to be at 3 in both directions. If the surface as a
	       degree higher then 3, then the function aborts.
   Created by: Philippe Lavoie (8 October, 1997)
  Modified by:
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
/*
void
writeSurface( ostream &os, Param *plist )
int patch_type, double flatness,
			    int num_u_steps, int num_v_steps)
{
  if( degU > 3 || degV > 3)
    {
	//TODO get a mesh from the obj instead
      cerr << "The degree of the surface is higher than 3!" << endl
	   << "A povrary file can not be generated." << endl;
      return 0;
    }

  PlNurbsSurface S;

  degreeElevate( obj, 3-degU, 3-degV, S );

  PlNurbsSurfaceArray Sa;
  S.decompose(Sa);
  int bad = 0 ; 

  for( int i = 0; i < Sa.n; ++i )
    {
      os << "bicubic_patch {\n\ttype " << patch_type << endl ;
      os << "\tflatness " << flatness << endl ;
      os << "\tu_steps " << num_u_steps << endl ;
      os << "\tv_steps " << num_v_steps << endl ;
      for( int j = 0; j < 4; ++j )
	{
	  for( int k = 0; k < 4; ++k )
	    {
	      Point3D p = project(Sa[i].CtrlP(j,k)) ;
	      if( Sa[i].CtrlP(j,k).w()>1.0001 || Sa[i].CtrlP(j,k).w()<0.9999 )
		bad = 1 ;
	      os << "\t<" << p.x << ", " << p.y << ", " << p.z << '>' ;
	      if( j==3 && k==3 )
		os << endl << '}' << endl << endl;
	      else
		os << ',' << endl;
	    }
	  os << endl;
	}
    }
  if(bad)
    {
      cerr << "Warning: at least one of the control point was not rational\n" ;
      cerr << "The resulting surface will NOT be the same as the one which\n" ;
      cerr << "generated it.\n" ;
    }
  return bad ;
}
*/


void
writePolygon( ostream &os, Param *list )
{
  G3DPoint3d *points;
  int num = list[1].number;

  os << "polygon { " << num+1 << ',' << endl << '<'; 
  points = list->point3;
  for( ; num > 0; num-- )
    os << *(points++) << str1;
  os << *(list->point3) << '>' << endl;
}

void
writeSphere( ostream &os, Param *list )
{
  os << "sphere {  <" << *(list->point3) << str2
     << list[1].dbl << endl;
}

void
writeTexture(  ostream &os, const G3DATexture *atr )
{
  os << "texture {" << endl;
  switch( atr->getType() )
    {
    case TA_COLOR:
      {
	G3DColor col;
	((G3DAColor*)atr)->getDiffuseColor( col );
	os << " pigment{ color rgb<" << col << "> }" << endl;
	((G3DAColor*)atr)->getAmbientColor( col );
	os << " finish { ambient rgb<" << col << "> }" << endl;
	break;
      }
    case TA_METAL:
      {
	G3DColor col;
	((G3DAMetal*)atr)->getDiffuseColor( col );
	os << " pigment{ color rgb<" << col << "> }" << endl;
	((G3DAMetal*)atr)->getAmbientColor( col );
	os << " finish { ambient rgb<" << col << '>' << endl;
	os << "  specular " << ((G3DAMetal*)atr)->getSpecRef() << endl
	   << "  roughness " << ((G3DAMetal*)atr)->getRoughness() << endl
	   << "  metallic" << endl;
	//TODO mirror + diffuse
	os << " }" << endl;
	break;
      }
    }
  os << " }" << endl;
}

void
writeTorus( ostream &os, Param *list )
{
  os << "torus { " << list[1].dbl << ',' << list[2].dbl << endl;
  if( !list->point3->isZero() )
    os << "  translate <" << *(list->point3) << '>' << endl;
}

/*
void
writeTrigrid( ostream &os, Param *list )
{
  os << "mesh {" << endl;
  int num = list[3].number * 2;
  G3DPoint3d  *vert = list[2].point3;
  G3DVector3d *norm = list[4].vector3;

  for( num )
    {
      os << "smooth_triangle{
    }
  pList->number  = _rows;
  pList++;
  pList->number  = _cols;
  pList++;
  pList->point3  = _vertices;
  pList++;
  pList->number  = _rows * _cols;
  pList++;
  pList->vector3 = _normals;
  pList++;
  pList->number  = _rows * _cols;

*list[1].dbl << ',' << *list[2].dbl << endl;
  if( !list->point3->isZero() )
    os << "  translate <" << *(list->point3) << '>' << endl;
}
*/

//===================================END=================================


