#ifndef __IO_RIB_H__
#define __IO_RIB_H__

/*
    Project : "MindsEye I/O Module"
    Copyright 1997. All Rights Reserved.

    FILE:         io_rib.h
    AUTHOR:       Nicholas Yue (nicholas_yue@acm.org)

    $Id: io_rib.h,v 1.7 1998/05/14 20:23:12 source Exp $

    OVERVIEW
    ========
    Class definition for the Renderman Interface Bitstream Format i/o module
*/

#include <fstream.h>
#include <values.h>
#include "io/io_module.h"
#include "io/io_base.h"
#include "3dlib/light.h"

class G3DScene;

/**IO Module for the RenderMan scene description language.
 */
class IO_RIB : public IO_BaseClass
{
public:
  IO_RIB();
  ~IO_RIB();
  // virtual function
  virtual int  check_header( char *fname );
      
  // pure virtual function
  virtual int getFormatInfo(int type);
protected:
  // virtual function
  int  exitLoad(int result);

  // pure virtual function
  virtual int exitSave(int result);
  virtual int initLoad(const char *fname);
  virtual int initSave(const char *fname, int flag);
  virtual int loadObjects(G3DGroup *);
  virtual int saveObjects(G3DScene *);
protected:
  ifstream  _inpf;
  ofstream  _outpf;
private:
  Param _param[12];
  char filename[256];
  void writeCamera(G3DCamera* cameraRef);
};

class IO_RIB_Lights : public GIterator<G3DLight>
{
public:
  IO_RIB_Lights(ofstream& out ) : _outf(out) {};
  virtual bool exec (G3DLight& );
  void resetLightCounter() { _lightCounter = 0; };
  void incrementLightCounter() { _lightCounter++; };
private:
  Param _params[12];
  ofstream& _outf;
  int _lightCounter;
};

class IO_RIB_Geometry : public GIterator<G3DShape>
{
public:
  IO_RIB_Geometry(ofstream& out ) : _outf(out) {};
  virtual bool exec (G3DShape& );
private:
  Param _params[12];
  ofstream& _outf;
  //============================
  void writeBlob( G3DMatrix& m );
  void writeBox( G3DMatrix& m );
  void writeCone( G3DMatrix& m);
  void writeCylinder( G3DMatrix& m);
  void writeDifference( G3DMatrix& m);
  void writeDisk( G3DMatrix& m);
  void writeIntersection( G3DMatrix& m);
  void writeNurbsCurve( G3DMatrix& m );
  void writeNurbsSurface( G3DMatrix& m );
  void writePoint( G3DMatrix& m );
  void writePolygon( G3DMatrix& m );
  void writeSphere( G3DMatrix& m );
  void writeTorus( G3DMatrix& m );
  void writeUnion( G3DMatrix& m );
  //============================
  void minmax(const float* input,unsigned int size, float& min,float& max);
  void writeTransformation( G3DMatrix& m );
  void writeShader();
};

#endif
