/*
 * Project : "MindsEye, 3D library"
 * Copyright 1996, 1997. All Rights Reserved.
 * Author(s): M. Kuball
 *
 * $Id: $
 *
 * $LOG:$
 * 
 * COPYRIGHT
 * =========
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

#include <stdlib.h>

#include "3dlib/s_shapes.h"
#include "3dlib/light.h"
#include "vrml.h"
#include "vrmlobjects.h"


IO_BaseClass*
NewVRML1( void )
{
  return( new MEIoVRML1 );
}


int
MEIoVRML1::check_header( char* )
{
  //must be #VRML V1.0 ascii
  return( 0 );  //TODO undefined
}


int
MEIoVRML1::exitLoad( int )
{
  return( IO_BaseClass::exitLoad(1) );
}


int
MEIoVRML1::exitSave( int )
{
  _inpf.close();
  return( 1 );
}


int
MEIoVRML1::getFormatInfo( int type )
{
  return( 0 ); //TODO return a FormatInfo object
}


int
MEIoVRML1::initLoad( const char *fname )
{
  char buffer[40];
  int  result;

  _inpf.open( fname );
  _inpf.setf( ios::skipws );
  if( _inpf.good() )
    {
      _inpf.getline( buffer, 40 );
      if( strcmp(buffer, VRML_1_HEADER) == 0 )
	return( READ_SUCCESS );
    }
  return( READ_ERROR );
}


int
MEIoVRML1::initSave( const char *fname, int flag )
{
  _outpf.open( fname );
  if( !_outpf.good() )
    {
      puts( "# Open Output failed" );
      return( READ_ERROR );
    }
  else
    {
      _outpf << VRML_HEADER << endl;
      return( WRITE_SUCCESS );
    }
}



int
MEIoVRML1::loadObjects( G3DGroup *group )
{
  static char token[256];
  AC3DSurface  *tSurf = NULL;
  AC3DObject   *tmp;
  char cmd;

/* if the first object is a group, don't creat one.
  The scene group already exists
*/

  nexttok[0] = '\0';

  read name
  switch on type
  
  while( !_inpf.eof() )
    {
      _inpf >> token;
      if( !_inpf )
	break;

      if( strcmp( token, "Separator" ) )
	{
	  _inpf >> token;
	  if( strcmp(token, "world") )
	    {
	      tmp = AC3DWorld( _inpf );
	    }
	  else if( strcmp(token, "poly") )
	    {
	      tmp = AC3DPoly( _inpf );
	    }
	  else if( strcmp(token, "group") )  //same as world??
	    {
	      tmp = AC3DPoly( _inpf );
	    }
	}
      else if( strcmp(token, "DirectionalLight") )
	{
	}
      else
	{ //unknown Object }

      switch (cmd)
	{
	case 'v':
	  {
	    tmp = new AC3DCamera( _inpf, nexttok );
	    _world->addShader( ((AC3DCamera*)tmp)->buildG3DCamera() );
	    delete tmp;
	    tmp = NULL;
	    break;
	  }
	case 'b':    // Background
	  {
	    float r, g, b;
	    _inpf >> r >> g >> b;
	    //group->setBackground( G3DColorRGB(r, g, b) );
	    break;
	  }
	case 'l':
	  {
	    tmp = new AC3DLight( _inpf, nexttok );
	    if( tmp->isValid() )
	      {
		group->append( tmp->buildG3DObject() );
		_lightCount++;
	      }
	    delete tmp;
	    tmp = NULL;
	    break;
	  }
	case 'f':     // Surface
	  {
	    tSurf = new AC3DSurface( _inpf );
	    break;  
	  }
	case 's':
	  {
	    tmp = new AC3DSphere( _inpf );
	    break;
	  }
	case 'B':
	  {
	    //tmp = new AC3DBox( _inpf );
	    break;
	  }
	case 'c':  //cone or cylinder
	  {
	    tmp = new AC3DCone( _inpf );
	    break;
	  }
	case 'p':
	  {
	    if( _inpf.peek() == 'p' )
	      {
		_inpf >> cmd;
		tmp = new AC3DPPatch( _inpf );
	      }
	    else
	      tmp = new AC3DPolygon( _inpf );
	    break;
	  }
	case '#':  //Comment
	  {
	    _inpf.getline(nexttok, 256);
	    nexttok[0] = '\0';
	    break;
	  }
	default:
	  tmp = NULL;
	}

      if( tmp )
	if( tmp->isValid() )
	  {
	    //TODO tmp->addAttribute( tSurf->getAttribute() );
	    group->append( tmp->buildG3DObject() );
	    _objCount++;
	  }
        else
	  delete tmp;
    }
  return( 1 );
}


int
MEIoVRML1::saveObjects( G3DScene *group )
{
  //iterate over all elements of the scene
  VRMLOutIterator it( this, _outpf );
  group->doWithShapes( it );
   return( 0 );
}


//====================================END================================
