/*
 * Project : "MindsEye, 3D library"
 * Copyright 1996, 1997. All Rights Reserved.
 * Author(s): M. Kuball
 *
 * $Id: ac3dobjects.cpp,v 1.1.1.1 1997/11/22 15:59:16 source Exp $
 *
 * $LOG:$
 * 
 * COPYRIGHT
 * =========
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

#include <stdlib.h>
#include <stdio.h>

#include "3dlib/generic.h"
#include "3dlib/light.h"
#include "3dlib/s_shapes.h"
#include "3dlib/s_prims.h"
#include "3dlib/shader.h"

#include "nffobjects.h"


void
prerror( char *str )
{
  puts( str );
}


/*
NFFBox::NFFBox()
{
  GFloat x, y, z, x1,y1,z1;
  inpf >> x >> y >> z >> x1>> y1>> z1;
  tPrim = new GTBox(GPoint3(x, y, z), GPoint3(x1,y1,z1));
  if(tText)
    tPrim->setSurface(tText);
  else
    tPrim->setSurface((tSurf) ? tSurf : &gDummySurface);
  
  rSC.append(tPrim);
}


*/

//======================================================================


NFFCamera::NFFCamera( ifstream &inpf, char *nexttok )
{
  int xres, yres;  //TODO values are currently ignored
			  
  inpf >> nexttok;
  if (strcmp(nexttok, "from"))
    prerror("\"from\" expected");
  inpf >> fromx >> fromy >> fromz >> nexttok;
  
  if (strcmp(nexttok, "at"))
    prerror("\"at\" expected");
  inpf >> atx >> aty >> atz >> nexttok;
  
  if (strcmp(nexttok, "up"))
    prerror("\"up\" expected");
  inpf >> upx >> upy >> upz >> nexttok;
  
  if (strcmp(nexttok, "angle"))
    prerror("\"angle\" expected");
  inpf >> zoom >> nexttok;
  
  if (strcmp(nexttok, "hither"))
    prerror("\"hither\" expected");
  inpf >> near >> nexttok;
  
  if (strcmp(nexttok, "resolution"))
    prerror("\"resolution\" expected");
  inpf >> xres >> yres;
  
  nexttok[0] = '\0';
}


G3DGLShader*
NFFCamera::buildG3DCamera( void )
{
  G3DPoint3d  pos( fromx, fromy, fromz );
  G3DVector3d poi( atx, aty, atz );
  G3DVector3d up ( upx, upy, upz );

  G3DGLShader *tmp = new G3DGLShader( near, near*100 );
  poi += (pos * -1.0);
  tmp->setView( pos, poi, up );
  tmp->zoomSet( zoom );

  return( tmp );
}


G3DShadable*
NFFCamera::buildG3DObject( void )
{
  return( NULL );
}


//======================================================================


NFFCone::NFFCone( ifstream &inpf )
{
  inpf >> aX >> aY >> aZ >> aR;
  inpf >> bX >> bY >> bZ >> bR;
}


G3DShadable*
NFFCone::buildG3DObject( void )
{
  G3DPoint3d  orig ( aX, aY, aZ );
  G3DPoint3d  orig2( bX, bY, bZ );
  G3DVector3d orientation, mao, mio;

  orientation = orig2 - orig;

  if( bR == aR )
    {
      mao = orientation.getOrtho();
      mao.normalize();
      mao *= aR;
      mio = mao.cross( orientation );
      mio.normalize();
      mio *= aR;
      return new G3DS_Cylinder( orientation, mao, mio, orig );
    }
  else
    {
      return NULL;
      //tmp = new G3DS_Cone();
    }
}


//======================================================================


NFFLight::NFFLight( ifstream &inpf, char *nexttok )
{
  char cmd, *sc;
  /*
  // rectangular light source
  if (inpf.peek() == 'r')
    {   
      inpf >> cmd;
    
      GFloat ax, ay, az;
      GFloat width, height;
      int Wres, Hres;
      int min, max, cand;
	    
      inpf >> lx >> ly >> lz;
      inpf >> ax >> ay >> az;
      inpf >> width >> height;
      inpf >> Wres >> Hres;
      inpf >> min >> max >> cand;
    
      sRectL++;
  }

  // spherical light source
  if( inpf.peek() == 's' )
  {
  inpf >> cmd;
    
    GFloat r;
    int Wres, Hres;
    inpf >> lx >> ly >> lz >> r >>Wres >> Hres ;
    tLight = new GXSphericalLight(GPoint3(lx, ly, lz), r, Wres, Hres);
    sSphL++;
  }
  */  
  // point light source
  //else {
  inpf >> pos.x >> pos.y >> pos.z;
  
  // color of the light source. It's optional so test for it.
  streampos tPos = inpf.tellg();
  inpf >> nexttok;
  cx = strtod(nexttok, &sc);
  if (sc == &(nexttok[0]))
    {
      inpf.seekg(tPos);
      nexttok[0] = '\0';
      cx = cy = cz = 1.0;
    }
  else
    {
      inpf >> cy >> cz;
      nexttok[0] = '\0';
    }
}


G3DShadable*
NFFLight::buildG3DObject( void )
{
  VectorStruct attenuation = {1,0,0};

  G3DPointLight *tmp = new G3DPointLight( &pos, &attenuation );
  tmp->setColor( G3DColor(cx, cy, cz) );
  return( tmp );
}


//======================================================================


NFFPPatch::NFFPPatch( ifstream &inpf )
{
  float x, y, z, nx, ny, nz;
  int   tNum;

  inpf >> tNum;
  _vertices = new G3DPoint3d[tNum];
  _normals  = new G3DVector3d[tNum];
  for (register int i = 0; i < tNum ; i++)
    {
      inpf >> x >> y >> z >> nx >> ny >> nz;
      _vertices[i].setTo(x, y, z);
      _normals[i].setTo(nx, ny, nz);
    }
}

NFFPPatch::~NFFPPatch()
{
  delete [] _vertices;
  delete [] _normals;
}


G3DShadable*
NFFPPatch::buildG3DObject( void )
{
    /*	      
  if (tNum >  3)
    tPoly = new GTSmoothPolygon(tNum, tVerts, tNorms);
  else
    tPoly = new GTSmoothTriangle(tVerts, tNorms);
    */
  return(NULL);
}


//======================================================================


NFFPolygon::NFFPolygon( ifstream &inpf )
{
  float x, y, z;
  int   tNum;

  inpf >> tNum;
  _vertices = new G3DPoint3d[tNum];
  for (register int i = 0; i < tNum ; i++)
    {
      inpf >> x >> y >> z;
      _vertices[i].setTo(x, y, z);
    }
}


NFFPolygon::~NFFPolygon()
{
  delete[] _vertices;
}


G3DShadable*
NFFPolygon::buildG3DObject( void )
{
    /*
  if(tNum > 3)
    tPoly = new GTPolygon(tNum, tVerts);
  else
    tPoly = new GTTriangle(tVerts);
    */
  return(NULL);
}


//======================================================================


NFFSphere::NFFSphere( ifstream &inpf )
{
  //// sphere
			  
  inpf >> x >> y >> z >> r;
  //GTSphere *_pSph = new GTSphere(GPoint3(x, y, z), r);
  
  //rSC.append(_pSph);
  
  //sSph++;
}


G3DShadable*
NFFSphere::buildG3DObject( void )
{
  return( NULL );
}


//======================================================================


NFFSurface::NFFSurface( ifstream &inpf )
{
  inpf >> r >> g >> b >> kd >> ks >> shine >> t >> ior;
  /*
  tSurf = new GSurface();
  tSurf->setAmbient(GColorRGB(0, 0, 0));
  tSurf->setDiffuse(GColorRGB(kd*r, kd*g, kd*b));
  tSurf->setSpecular(GColorRGB(ks*r, ks*g, ks*b));
  tSurf->setShininess(shine);
  tSurf->setTransparency(t);
  tSurf->setRefraction(ior);
  */
}


//===================================END=================================
