/*
 * vi:ts=4:sw=4:
 *
 * Project : "Config Manager"
 * Copyright 1997. All Rights Reserved.
 * 
 * $RCSfile: m_config.cpp,v $
 *
 * $Revision: 1.1.1.1 $
 * 
 * $Author: source $ 
 * 
 * $Locker:  $
 * 
 * $State: Exp $
 * 
 * 
 * COPYRIGHT
 * =========
 * 
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 * 
 * OVERVIEW
 * ========
 *
 *
 */
 
#include "m_config.h"
#include "parser.h"

char compose_str [PATH_MAX];

/*---------------------------------------------------------------------------*/
ConfigClass::ConfigClass ()
{
 config_status =CONFIG_OK;

 parent        =NULL;
 plugin_top    =NULL;
 plugin_current=NULL;
 next          =NULL;
 
 home_string   =NULL;
 program_name  =NULL;
 file_name     =NULL;

 floating_group=NULL;
 floating_entry=NULL;
 
 file_pointer  =NULL;

 group_top     =NULL;
 group_current =NULL;

 default_int   =DEFAULT_INT;
 default_float =DEFAULT_FLOAT;
 default_string=NULL;

 if (create_homedir ()==1)
  config_status=NO_HOMEDIR;
 else
 {
   if (parent==NULL)
    parse_confclass=this; 
 }
}
/*---------------------------------------------------------------------------*/
ConfigClass::ConfigClass (int argc,char **argv)
{
 config_status =CONFIG_OK;

 parent         =NULL;
 plugin_top     =NULL;
 plugin_current =NULL;
 next           =NULL;

 home_string    =NULL;
 program_name   =NULL;
 file_name      =NULL;

 floating_group =NULL;
 floating_entry =NULL;

 if (create_homedir ()==1)
  config_status=NO_HOMEDIR;
 else
 {
   argc           =argc;

   if (argv [0]!=NULL)
   {
     program_name=(char *) malloc (strlen (argv [0])+1);
     strcpy (program_name,argv [0]);
   }
 
   file_name      =(char *) malloc (strlen (program_name)+4); 
   compose_str [0]=0;
   sprintf (compose_str,".%src",program_name);           

   file_name      =create_conffile (compose_str); 
   if (file_name==NULL)
     config_status=ERROR_CREATEDIR;

   file_pointer   =NULL;

   group_top      =NULL;
   group_current  =NULL;

   default_int    =DEFAULT_INT;
   default_float  =DEFAULT_FLOAT;
   default_string =NULL;

   default_string =(char *) malloc (strlen (DEFAULT_STRING)+1);
   strcpy (default_string,DEFAULT_STRING);

   if (parent==NULL)
    parse_confclass=this;

   read_config (); // make sure we've got the config read before entries are
                   // added
 }
}
/*---------------------------------------------------------------------------*/
ConfigClass::ConfigClass (char *config_file)
{
 config_status =CONFIG_OK;

 parent        =NULL;
 plugin_top    =NULL;
 plugin_current=NULL;
 next          =NULL;

 home_string   =NULL;
 program_name  =NULL;
 file_name     =NULL;

 floating_group=NULL;
 floating_entry=NULL;

 if (create_homedir ()==1)
  config_status=NO_HOMEDIR;
 else
 {

   file_name     =create_conffile (config_file); 
   if (file_name==NULL)
     config_status=ERROR_CREATEDIR;  

   file_pointer  =NULL;

   group_top     =NULL;
   group_current =NULL;

   default_int   =DEFAULT_INT;
   default_float =DEFAULT_FLOAT;
   default_string=NULL;

   default_string=(char *) malloc (strlen (DEFAULT_STRING)+1);
   strcpy (default_string,DEFAULT_STRING);

   if (parent==NULL)
    parse_confclass=this;

   read_config (); // make sure we've got the config read before entries are
                   // added
 }
}
/*---------------------------------------------------------------------------*/
ConfigClass::ConfigClass (char *config_file,char *prog_name)
{
 config_status =CONFIG_OK;

 parent        =NULL;
 plugin_top    =NULL;
 plugin_current=NULL;
 next          =NULL;

 home_string   =NULL;
 program_name  =NULL;
 file_name     =NULL;

 floating_group=NULL;
 floating_entry=NULL;

 if (create_homedir ()==1)
  config_status=NO_HOMEDIR;
 else
 {
 
   if (prog_name!=NULL)
   {
     program_name=(char *) malloc (strlen (prog_name)+1);
     strcpy (program_name,prog_name);
   }

   file_name     =create_conffile (config_file); 
   if (file_name==NULL)
     config_status=ERROR_CREATEDIR;

   file_pointer  =NULL;
  
   group_top     =NULL;
   group_current =NULL;

   default_int   =DEFAULT_INT;
   default_float =DEFAULT_FLOAT;
   default_string=NULL;

   default_string=(char *) malloc (strlen (DEFAULT_STRING)+1);
   strcpy (default_string,DEFAULT_STRING);

   if (parent==NULL)
    parse_confclass=this;

   read_config (); // make sure we've got the config read before entries are
                   // added
 }
}
/*---------------------------------------------------------------------------*/
ConfigClass::ConfigClass (ConfigClass *new_parent)
{
 config_status =CONFIG_OK;

 parent        =new_parent;
 plugin_top    =NULL;
 plugin_current=NULL;
 next          =NULL;

 if (parent!=NULL)
  parent->add_child (this);

 home_string   =NULL; 
 program_name  =NULL;
 file_name     =NULL;

 floating_group=NULL;
 floating_entry=NULL;
 
 file_pointer  =NULL;

 group_top     =NULL;
 group_current =NULL;

 default_int   =DEFAULT_INT;
 default_float =DEFAULT_FLOAT;
 default_string=NULL;

 if (create_homedir ()==1)
  config_status=NO_HOMEDIR;

}
/*---------------------------------------------------------------------------*/
ConfigClass::~ConfigClass ()
{
 if (parent!=NULL) // clone data structure and give the copy to the parent
 {
 
 }
 else
 {
  // first write config
  write_config ();

  // then delete the entire structure
  clear_data ();
 }
}
/*---------------------------------------------------------------------------*/
void ConfigClass::clear_data (void)
{
/*
 struct group_entry *temp_group=group_top;
 struct conf_entry  *temp_entry;

 while (temp_group!=NULL)
 {
  temp_entry=temp_group->var_top;

  if (temp_group->group_name!=NULL)
   free (temp_group->group_name);
 
  while (temp_entry!=NULL)
  {  
   if (temp_entry->var_name!=NULL)
    free (temp_entry->var_name);

   if (temp_entry->var_types!=NULL)
    free (temp_entry->var_types);

   if (temp_entry->var_array!=NULL)
   {
     for (int i=0;i<temp_entry->nr_entries;i++)
      free (temp_entry->var_array [i]);
  
     free (temp_entry->var_array);
   }

   temp_entry=temp_entry->next;
  }

  temp_group=temp_group->next;
 }
*/
}
/*---------------------------------------------------------------------------*/
int ConfigClass::create_homedir (void)
{
 // get the user's homedir location
 
 home_string=getenv ("HOME");
 if (home_string==NULL)
 {
  fprintf (stderr,"Unable to find home-directory\n");
  return (1);
 }

 return (0);
}
/*---------------------------------------------------------------------------*/
int ConfigClass::create_confdir (char *target_dir)
{
 int dmode=0664;

 time_t time_struct;

 time (&time_struct);

 // create ~/.configfiles/

 if (mkdir (target_dir,dmode)!=0)
  return (1);
 else
 {
  // create readme file
  strcat (target_dir,"/README");
  FILE *readme=fopen (target_dir,"w");
  if (readme!=NULL)
  {
   fprintf (readme,"# This directory contains config files created by:\n");
   fprintf (readme,"# ConfigManager\n");
   fprintf (readme,"# author: Martin van Velsen <vvelsen@ronix.ptf.hro.nl>\n");
   fprintf (readme,"# \n");
   fprintf (readme,"# This file was automatically created on: %s",
                   ctime (&time_struct));

   fclose (readme);
  }
  else
   return (1);
 }

 return (0);
}
/*---------------------------------------------------------------------------*/

char dummy_str [PATH_MAX];

/*---------------------------------------------------------------------------*/
char *ConfigClass::create_conffile (char *target_file) 
{
 // first see if the config dir exists

 dummy_str [0]=0;
 sprintf (dummy_str,
          "%s/.configfiles",
          home_string);

 struct DIR *checkdir;

 checkdir=opendir (dummy_str);
 if (checkdir==NULL)
 {
  if (create_confdir (dummy_str)!=0)
   return (NULL);
 }

 // create directory name again
 dummy_str [0]=0;
 sprintf (dummy_str,
          "%s/.configfiles/%s",
          home_string,
          target_file);

 // add filename and create copy
  
 char *return_string=(char *) malloc (strlen (dummy_str)+1);
 strcpy (return_string,dummy_str);

 return (return_string); // might be buggy and I don't mean the car
}
/*---------------------------------------------------------------------------*/
void ConfigClass::add_child (ConfigClass *child)
{
 if (plugin_top==NULL)
 {
  plugin_top=new ConfigClass ();
  plugin_current=plugin_top; 
 }
 else
 {
  plugin_current->next=new ConfigClass ();
  plugin_current=plugin_current->next;
 }

 plugin_current=child;
}
/*---------------------------------------------------------------------------*/
struct conf_entry *ConfigClass::find_entry (char *target)
{
 struct conf_entry *temp_entry=floating_group->var_top;

 while (temp_entry!=NULL)
 {
  if (strcmp (temp_entry->var_name,target)==0)
    return (temp_entry);

  temp_entry=temp_entry->next;
 }

 return (NULL); // not found
}
/*---------------------------------------------------------------------------*/
struct group_entry *ConfigClass::find_group (char *target)
{
 struct group_entry *temp_group=group_top;

 while (temp_group!=NULL)
 {
  if (strcmp (temp_group->group_name,target)==0)
   return (temp_group);

  temp_group=temp_group->next;
 }

 return (NULL); // not found
}
/*---------------------------------------------------------------------------*/
void ConfigClass::insert_group (char *new_name)
{
  struct group_entry *target=find_group (new_name);

  if (target==NULL)  
  {
    if (group_top==NULL)
    {
     group_top          =new group_entry;
     group_current      =group_top;
     group_current->next=NULL;  
    }
    else
    {
     group_current->next=new group_entry;
     group_current      =group_current->next;
     group_current->next=NULL;
    }
 
    // now fill in the blanks
    group_current->group_name=(char *) malloc (strlen (new_name)+1);
    strcpy (group_current->group_name,new_name);

    group_current->var_top    =NULL;
    group_current->var_current=NULL;

    floating_group=group_current;
 }
 else
 {
  // set active group to "target"
  floating_group=target;
 } 
}
/*---------------------------------------------------------------------------*/
void ConfigClass::insert_entry (char *name)
{
 struct conf_entry *target=find_entry (name);
  
 if (target==NULL)
 {
    //>------ make sure we have a group to insert to, if not create one

    if (group_top==NULL)
    {
     group_top                 =new group_entry;
     group_current             =group_top;
     group_current->next       =NULL;  

     group_current->group_name =(char *) malloc (strlen ("main")+1);
     strcpy (group_current->group_name,"main");

     group_current->var_top    =NULL;
     group_current->var_current=NULL;
    }

    //>------ create a new entry in the current group

    if (group_current->var_top==NULL)
    {
     group_current->var_top          =new conf_entry;
     group_current->var_current      =group_current->var_top;
     group_current->var_current->next=NULL;
    }
    else
    {
     group_current->var_current->next=new conf_entry;
     group_current->var_current      =group_current->var_current->next;
     group_current->var_current->next=NULL;
    }

    floating_entry=group_current->var_current;
 }
 else
 {
  floating_entry=target;
 }

 floating_entry->var_name    =(char *) malloc (MAX_VARS+1);
 strcpy (floating_entry->var_name,name);

 floating_entry->var_types   =(char *) malloc (MAX_VARS+1);
 floating_entry->nr_entries=0;

 for (int i=0;i<MAX_VARS;i++)
  floating_entry->var_types [i]=0;

 for (int i=0;i<MAX_VARS;i++)
  floating_entry->var_array [i]=NULL;
}
/*---------------------------------------------------------------------------*/
void ConfigClass::insert_entry (char *name,
                                char *var_string,...)
{
 int    i=0;
 int    temp_i;
 float  temp_f;
 char  *temp_s;

 // first some general checking :

 if (strlen (var_string)>MAX_VARS)
  var_string [strlen (var_string)-1]=0;
 
 struct conf_entry *target=find_entry (name);

 if (target==NULL)
 {
    //>------ make sure we have a group to insert to, if not create one
    if (group_top==NULL)
    {
     group_top                 =new group_entry;
     group_current             =group_top;
     group_current->next       =NULL;  

     group_current->group_name =(char *) malloc (strlen ("main")+1);
     strcpy (group_current->group_name,"main");
 
     group_current->var_top    =NULL;
     group_current->var_current=NULL;
    }

    //>------ create a new entry in the current group

    if (group_current->var_top==NULL)
    {
     group_current->var_top          =new conf_entry;
     group_current->var_current      =group_current->var_top;
     group_current->var_top->next    =NULL;
    }
    else
    {
     group_current->var_current->next=new conf_entry;
     group_current->var_current      =group_current->var_current->next;
     group_current->var_current->next=NULL;
    }

    floating_entry=group_current->var_current;

    floating_entry->var_name         =(char *) malloc (MAX_VARS+1);
    floating_entry->var_types        =(char *) malloc (MAX_VARS+1);
    floating_entry->nr_entries       =strlen (var_string);

    strcpy (floating_entry->var_types,var_string);
    strcpy (floating_entry->var_name,name);

    for (int j=0;j<MAX_VARS;j++)
      floating_entry->var_array [j]=NULL;    
 }
 else
 {
  floating_entry=target;
 }

 va_list ap;
 va_start (ap,var_string);

 i=0;
 // now try to match the new entry with the existing one
 while (*var_string)
 {
   if (floating_entry->var_array [i]==NULL) // pfff it's a new entry
   {      
      floating_entry->var_array [i]=(void *) va_arg (ap,void *);
   }
   else                                     // damn we'll probably have to match
   {
      if (floating_entry->var_types [i]==*var_string)
      {
        switch (floating_entry->var_types [i])
        {
         case 'i':
                  temp_i=*((int *) floating_entry->var_array [i]);
                  free (floating_entry->var_array [i]);                   
                  floating_entry->var_array [i]=(void *) va_arg (ap,int *);
                  *((int *) floating_entry->var_array [i])=temp_i;    
 
                  break;

         case 'f':
                  temp_f=*((float *) floating_entry->var_array [i]);
                  free (floating_entry->var_array [i]);                   
                  floating_entry->var_array [i]=(void *) va_arg (ap,int *);
                  *((float *) floating_entry->var_array [i])=temp_f;    
 
                  break;
 
         case 's':temp_s=(char *) floating_entry->var_array [i];
                   
                  break;
        }      
      }
      else
      {
       // we'll have to find a new place for this one
      }  
   }

   var_string++;

   i++; 
 }

 va_end (ap);
}
/*---------------------------------------------------------------------------*/
void ConfigClass::insert_entry_to_group (char *name,
                                         char *group,
                                         char *var_string,...)
{
 name=name;
 group=group;
 var_string=var_string;

}
/*---------------------------------------------------------------------------*/
void ConfigClass::set_defaults (int def1,float def2,char *def3)
{
 default_int  =def1;
 default_float=def2;

 if (default_string!=NULL)
  free (default_string);

 default_string=(char *) malloc (strlen (def3)+1);
   strcpy (default_string,def3); 
}
/*---------------------------------------------------------------------------*/
int ConfigClass::read_config (void)
{
 if ((config_status!=CONFIG_OK) && (config_status!=NO_CONFIG))
  return (1);

 file_pointer=fopen (file_name,"r");
 if (file_pointer==NULL)
 {
  // we'll leave the next fprintf out because this could be the very first time
  // that someone is running the program

  // fprintf (stderr,"No config file found with specified name\n");
  return (NO_CONFIG);      
 }
 
 //>----------- make sure the data-structure is clean

 //clear_data ();

 //>----------- now read the damn thing :)

 if (file_pointer!=NULL)
 {
  #ifdef __CREATE_DEBUG__
    printf ("parsing....\n");
  #endif

  parse_a_file (this,file_pointer);
 }

 fclose (file_pointer); 
 return (0);
}
/*---------------------------------------------------------------------------*/
int ConfigClass::write_entries (FILE *temp_file,struct conf_entry *temp_entry)
{
 int i;

 while (temp_entry!=NULL)
 {
  fprintf (temp_file,"\n<%s> ",temp_entry->var_name);

  for (i=0;i<temp_entry->nr_entries;i++)
  {
   switch (temp_entry->var_types [i])
   {
    case 'i': 
              fprintf (temp_file," %d",*((int *) (temp_entry->var_array [i])));
              break;
    case 'f': 
              fprintf (temp_file," %f",*((float *) (temp_entry->var_array [i])));
              break;
    case 's': 
              fprintf (temp_file," \"%s\"",(char *) temp_entry->var_array [i]);
              break;
   }
  }

  fprintf (temp_file,"\n");

  temp_entry=temp_entry->next;
 }

 return (0);
}
/*---------------------------------------------------------------------------*/
int ConfigClass::write_config (void)
{
 time_t time_struct;

 time (&time_struct);

 if ((config_status!=CONFIG_OK) && (config_status!=NO_CONFIG))
  return (1);

 file_pointer=fopen (file_name,"w");
 if (file_pointer==NULL)
 {
  fprintf (stderr,
           "Unable to open config file with specified name\n [%s]\n",
           compose_str);
  return (ERROR_WRITING);      
 }

 fprintf (file_pointer,"\n\n");
 fprintf (file_pointer,"# This file was automatically generated, please do not edit\n");
 fprintf (file_pointer,"# by hand unless you know what you're doing\n");
 fprintf (file_pointer,"#\n");
 fprintf (file_pointer,"# File created by [ConfigManager] <ver> %d.%d <on> %s",
                          VERSION_MAJOR,
                          VERSION_MINOR,
                          ctime (&time_struct));
 fprintf (file_pointer,"# Copyright (C) 1997 Martin van Velsen, vvelsen@ronix.ptf.hro.nl\n");
 fprintf (file_pointer,"#\n");

 if (program_name!=NULL)
   fprintf (file_pointer,
            "# This config file belongs to the (%s) package\n",
            program_name);

 fprintf (file_pointer,"#\n");
 fprintf (file_pointer,"\n");

 // now traverse the group/entry/variable tree

 struct group_entry *temp_group=group_top;

 while (temp_group!=NULL)
 {
  if (strcmp (temp_group->group_name,"ConfigManager")!=0)
  {
    fprintf (file_pointer,"\n#--------------------------------------------------------\n\n");
    fprintf (file_pointer,"[%s]\n\n",temp_group->group_name);
  
    if (temp_group->var_top!=NULL)
      write_entries (file_pointer,
                     temp_group->var_top);
  }

  temp_group=temp_group->next;
 }

 fprintf (file_pointer,"\n#--------------------------------------------------------\n"); 
 fprintf (file_pointer,"# end of config file \n\n");

 fclose (file_pointer); 
 return (0);
}
/*---------------------------------------------------------------------------*/
void ConfigClass::create_group (char *token)
{ 
 insert_group (token);

 #ifdef __CREATE_DEBUG__
   printf ("Creating group [%s]\n",token);
 #endif
}
/*---------------------------------------------------------------------------*/
void ConfigClass::create_var   (char *token)
{
 insert_entry (token);

 #ifdef __CREATE_DEBUG__
   printf ("Creating variable [%s]\n",token);
 #endif
}
/*---------------------------------------------------------------------------*/
void ConfigClass::add_integer  (int token)
{
 int *temp_int=(int *) malloc (sizeof (int));
 *temp_int=token;

 group_current->var_current->var_types [group_current->var_current->nr_entries]  ='i';
 group_current->var_current->var_types [group_current->var_current->nr_entries+1]=0;
 group_current->var_current->var_array [group_current->var_current->nr_entries]  =(void *) temp_int;

 #ifdef __CREATE_DEBUG__
   printf ("Added int [%d]\n",*((int *) group_current->var_current->var_array [group_current->var_current->nr_entries]));
 #endif

 group_current->var_current->nr_entries++;
}
/*---------------------------------------------------------------------------*/
void ConfigClass::add_float    (float token)
{
 float *temp_float=(float *) malloc (sizeof (float));
 *temp_float=token;

 group_current->var_current->var_types [group_current->var_current->nr_entries]  ='f';
 group_current->var_current->var_types [group_current->var_current->nr_entries+1]=0;
 group_current->var_current->var_array [group_current->var_current->nr_entries]  =(void *) temp_float;

 #ifdef __CREATE_DEBUG__
   printf ("Added float [%f]\n",*((float *) group_current->var_current->var_array [group_current->var_current->nr_entries]));
 #endif

 group_current->var_current->nr_entries++;
}
/*---------------------------------------------------------------------------*/
void ConfigClass::add_string   (char *token)
{
 char *temp_string=(char *) malloc (strlen (token)+1);
 strcpy (temp_string,token);

 group_current->var_current->var_types [group_current->var_current->nr_entries]  ='s';
 group_current->var_current->var_types [group_current->var_current->nr_entries+1]=0;
 group_current->var_current->var_array [group_current->var_current->nr_entries]  =(void *) temp_string;

 #ifdef __CREATE_DEBUG__
   printf ("Added string [%s]\n",(char *) group_current->var_current->var_array [group_current->var_current->nr_entries]);
 #endif

 group_current->var_current->nr_entries++;
}
/*---------------------------------------------------------------------------*/  
  