/*
 * Project : "MindsEye"
 * Copyright (c) 1996, 1997. All Rights Reserved.
 * Author(s):  F. Meier, M. Kuball
 * 
 * $Id: init3d.cpp,v 1.2 1998/07/23 20:16:16 source Exp $
 *
 *
 * COPYRIGHT
 * ========= 
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

#include "3dlib/light.h"
#include "3dlib/s_group.h"
#include "3dlib/scene.h"
#include "3dlib/s_simple.h"
#include "3dlib/s_trigrid.h"
#include "3dlib/s_line.h"
#include "3dlib/s_mesh.h"
#include "3dlib/s_nurbsurf.h"
#include "3dlib/s_nurbsrot.h"
#include "3dlib/s_torus.h"
#include "3dlib/Attributes/attribute.h"
#include "3dlib/Attributes/texture.h"

#include "nurbsedit/shaper.h"

#include "m_core.h"
#include "init3d.h"

//G3DShape* instead?
G3DObject* createBox( void );
G3DObject* createCone( void );
G3DObject* createCurve( void );
G3DObject* createCurve2( void );
G3DObject* createCylinder( void );
G3DObject* createGroup( void );
G3DObject* createMesh( void );
G3DObject* createNurbsSurface( void );
G3DObject* createRotSurface( void );
G3DObject* createScene( void );
G3DObject* createSphere( void );
G3DObject* createRotSurface( void );
G3DObject* createTorus( void );
G3DObject* createTrigrid( void );

G3DObject* createPointLight( void );
G3DObject* createAmbientLight( void );


G3DAttribute* createAResolution( void );
G3DAttribute* createAColor( void );

static const char *ObjClassList[] = 
{ "simple",
  "nurbs",
  "user defined",
  "special",
  "lights",
};

static const char *str1 = "curve_from_shaper";



void
init3dlib( MCore *kernel )
{
  //G3DObject::init();
  G3DNurbsCurve::initClass();
  G3DNurbsSurface::initClass();

  kernel->addObjectFactory(G3DS_Box::GetTypeName(),      createBox);
  kernel->addObjectFactory(G3DS_Cone::GetTypeName(),     createCone);
  kernel->addObjectFactory(G3DS_Cylinder::GetTypeName(), createCylinder);
  kernel->addObjectFactory(G3DMesh::GetTypeName(),       createMesh);
  kernel->addObjectFactory(G3DNurbsCurve::GetTypeName(), createCurve);
  kernel->addObjectFactory( str1,                        createCurve2);
  kernel->addObjectFactory(G3DNurbsSurface::GetTypeName(), createNurbsSurface);
  kernel->addObjectFactory(G3DGroup::GetTypeName(),      createGroup);
  kernel->addObjectFactory(G3DRotationSurface::GetTypeName(),createRotSurface);
  kernel->addObjectFactory(G3DScene::GetTypeName(),      createScene);
  kernel->addObjectFactory(G3DS_Sphere::GetTypeName(),   createSphere);
  kernel->addObjectFactory(G3DTorus::GetTypeName(),      createTorus);
  kernel->addObjectFactory(G3DTriGrid::GetTypeName(),    createTrigrid);

  kernel->addObjectFactory(G3DPointLight::GetTypeName(), createPointLight);
  kernel->addObjectFactory(G3DAmbientLight::GetTypeName(), createAmbientLight);

  kernel->addAttributeFactory( G3DAResolution::GetTypeName(),
			       createAResolution );
  kernel->addAttributeFactory( G3DAColor::GetTypeName(), createAColor );

  for( int i = 0; i < 5; ++i )
    Kernel->addObjectClass( ObjClassList[i] );

  const char *cn = ObjClassList[0];
  Kernel->addObjectName( cn, "Box", G3DS_Box::GetTypeName() );
  Kernel->addObjectName( cn, "Cone", G3DS_Cone::GetTypeName() );
  Kernel->addObjectName( cn, "Cylinder", G3DS_Cylinder::GetTypeName() );
  Kernel->addObjectName( cn, "Sphere", G3DS_Sphere::GetTypeName() );
  Kernel->addObjectName( cn, "Torus", G3DTorus::GetTypeName() );
  Kernel->addObjectName( cn, "Trigrid", G3DTriGrid::GetTypeName() );

  cn = ObjClassList[1];
  Kernel->addObjectName( cn, "curve from 2dshaper", str1 );

  cn = ObjClassList[4];
  Kernel->addObjectName( cn, "Point Light", G3DPointLight::GetTypeName() );
}

G3DObject*
createBox( void )
{
  return( new G3DS_Box );
}

G3DObject*
createCone( void )
{
  return( new G3DS_Cone );
}

G3DObject*
createCurve( void )
{
  return( new G3DNurbsCurve );
}

G3DObject*
createCurve2( void )
{
  ME2DShaper *tmp = (ME2DShaper*)(Kernel->getModule(MODULE_SHAPER));
  if( tmp )
    return( tmp->getObject() );  //TODO or via emit signal
  else
    return( NULL );
}

G3DObject* createCylinder( void )
{
  return( new G3DS_Cylinder(0.5, 1) );
}

G3DObject*
createGroup( void )
{
  return( new G3DGroup );
}

G3DObject*
createMesh( void )
{
  return( new G3DMesh );
}

G3DObject*
createNurbsSurface( void )
{
  return( new G3DNurbsSurface );
}

G3DObject*
createRotSurface( void )
{
  return( new G3DRotationSurface );
}

G3DObject*
createScene( void )
{
  return( new G3DScene );
}

G3DObject*
createSphere( void )
{
  return( new G3DS_Sphere(0.5) );
}

G3DObject*
createTorus( void )
{
  return( new G3DTorus(0.8, 0.1) );
}

G3DObject*
createTrigrid( void )
{
  return( new G3DTriGrid(12, 12) );
}


//----------------------------------------------------------------------


G3DObject*
createAmbientLight( void )
{
  return( new G3DAmbientLight );
}

G3DObject*
createPointLight( void )
{
  return( new G3DPointLight );
}


//----------------------------------------------------------------------


G3DAttribute*
createAResolution( void )
{
  return( new G3DAResolution );
}

G3DAttribute*
createAColor( void )
{
  return( new G3DAColor );
}


//------------------------------
/*
getWidgets( propList )
{
  propList->add( T_POINT_LIGHT, cPointLightInfo );
}


cPointLightInfo()
{
  return new MEPointLightInfo;
}
*/


//====================================END=================================
