/*
 * Project : "MindsEye"
 * Copyright 1996, 1998. All Rights Reserved.
 * Author(s): M. van Velsen
 *
 * $Id: m_console.cpp,v 1.4 1998/07/23 20:16:16 source Exp $
 *
 *
 * COPYRIGHT
 * ========= 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *
 * OVERVIEW
 * ========
 * The purpose of the console is straightforward, you can give direct 
 * commands to the kernel to either get something done directly or to
 * get some info very quickly. The console might also be your last 
 * resort to kill certain modules :) Oh and of course it's a great
 * debugging feature :) :)
 *
 */

#ifdef HAVE_CONFIG_H
#include "include/config.h"
#endif	 // HAVE_CONFIG_H

#include <qmenubar.h>
#include <qpopmenu.h>
#include <qkeycode.h>

#ifdef _HAVE_SIOD_
#include <siod.h>
#endif

#include "m_console.h"
#include "m_core.h"
#include "include/m_vars.h"
#include "include/m_messg.h"  // this is where the console commands are defined


#define NR_COMMANDS 6

static char print_text[PATH_MAX]; //TODO
static char *commands [NR_COMMANDS] =
{
  "datainfo",
  "modules",
  "status",
  "who",
  "quit",
  "help"
};


/*---------------------------------------------------------------------------*/
MKernelConsole::MKernelConsole( QWidget *parent,
                                const char *name) : QWidget (parent,name)
{
  scrollb = 100; // start with 100 lines

  setMinimumSize( 320, 300 );

  view_box = new QBoxLayout( this, QBoxLayout::TopToBottom, 3, 3 );

#ifdef WITH_CONSOLE_MENU
  QPopupMenu *file = new QPopupMenu;
  CHECK_PTR( file );
  file->insertItem( "load",  this, SLOT(toggleLoad()), CTRL+Key_L );
  file->insertItem( "save",  this, SLOT(toggleSave()), CTRL+Key_S );
  file->insertSeparator();
  file->insertItem( "close", qApp, SLOT(quit()), CTRL+Key_C );

  QPopupMenu *tools = new QPopupMenu;
  CHECK_PTR( tools );
  tools->insertItem( "modeler", this, SLOT(toggleModeler()) );
  tools->insertItem( "tree list", this, SLOT(toggleTreeList()) );
  //tools->insertItem( "world list", this, SLOT(dummy()) );

  QMenuBar *menu = new QMenuBar( this );
  CHECK_PTR( menu );
  menu->setFrameStyle( QFrame::WinPanel | QFrame::Raised );
  menu->insertItem( "&File", file );
  menu->insertItem( "&Tools", tools );
  view_box->setMenuBar( menu );
#endif

  QBoxLayout *gm_01 = new QBoxLayout (QBoxLayout::LeftToRight);
  QBoxLayout *gm_02 = new QBoxLayout (QBoxLayout::LeftToRight);

  view_box->addLayout (gm_01,100);
  view_box->addLayout (gm_02,0);

  console_view = new QMultiLineEdit (this,0);
  console_view->setReadOnly (TRUE);
 
  console_prompt = new QLineEdit (this,0);
  console_prompt->setMinimumSize (75,20);
  console_prompt->setFrame (FALSE);

  console_input = new QLineEdit (this,0);
  console_input->setMinimumSize (200,20);
  console_input->setFrame (FALSE);

  connect (console_input,
	   SIGNAL (returnPressed()),
	   this,
	   SLOT (interpret()));

  gm_01->addWidget (console_view,0);
  gm_02->addWidget (console_prompt,0);
  gm_02->addWidget (console_input,100);

  view_box->activate (); 

  setCaption ("MindsEye Console");
  prompt ();
}

MKernelConsole::~MKernelConsole ()
{
}
/*---------------------------------------------------------------------------*/
//TODO move this code into an extra class (default minimal scripting)
void MKernelConsole::execute( int com_nr )
{
  int con_mess = com_nr + CONMESS_START;

  switch (con_mess)
    {
    case K_MESS_DATAINFO :
      break;

    case K_MESS_MODULES  :
      break;

    case K_MESS_STATUS   :
      m_puts( "No status available, yet" );
      break;

    case K_MESS_WHO      :
      break;

    case K_MESS_QUIT     :
      break;

    case K_MESS_HELP     :
      m_puts( "No scripting support available." );
      m_puts( "Valid console commands are:" );  
      for( int i = 0; i < NR_COMMANDS; i++ )
	m_printf( "- %s", commands[i] ); 
      break;

    default              :
      m_puts( "Whoops error in parsing command" );
      break;
    }
}
/*---------------------------------------------------------------------------*/
void
MKernelConsole::initScripting( void )
{
#ifdef _HAVE_SIOD_
/*TODO this should be implemented as a plugin
  at the moment we are using siod as a testbed */
  iobufflen_ = 1000;
  char *sargv[4];

  sargv[0] = argv[0];
  sargv[1] = "-v0";
  sargv[2] = "-g0";  //mark and sweep garbage collection
  sargv[3] = "-h10000:10";
  siod_init( 4, sargv );

  iobuff_ = new char[iobufflen_];
#endif
}

void MKernelConsole::interpret( void )
{
  char *command_line = (char *) console_input->text ();

#ifdef _HAVE_SIOD_
  sprintf( iobuff_, "(*catch 'errobj (begin %s))", command_line );
  int retval = repl_c_string( iobuff_, 0, 0, iobufflen_ );
  m_puts( iobuff_ );
  //m_printf( "retval = %d\n%s\n", retval, iobuff );
#else

  //TODO if each word is considered as a token, arguments are impossible
  char   *token = strtok( command_line, " " );
  double d1, d2, d3, e1, e2, e3;

  if (token != NULL)
    {
      parse_token (token);
      token = strtok (NULL," ");   
      
      while (token != NULL)
	{
	  parse_token (token);
	  token = strtok (NULL," ");   
	}
    }
#endif
  prompt ();
}
/*---------------------------------------------------------------------------*/
/* This is the console's function of printf, use it just like the            */
/* real thing.                                                               */
/*---------------------------------------------------------------------------*/
void MKernelConsole::m_printf (const char *print_string, ...)
{
  va_list ap;

  va_start (ap,print_string);
 
  print_text [0] = 0;
  vsprintf (print_text,print_string,ap);

 //>----------- check for '\n'

  if (print_text [strlen (print_text)-1]=='\n')
    print_text [strlen (print_text)-1]=0;

  if (console_view!=NULL)
    {
      if (console_view->numLines ()>scrollb)
	console_view->removeLine (0);

      console_view->insertLine (print_text,-1);
    }
 
  va_end (ap);
  console_view->update ();
}
/*---------------------------------------------------------------------------*/
/* This is the console's function of puts, use it just like the              */
/* real thing.                                                               */
/*---------------------------------------------------------------------------*/
void MKernelConsole::m_puts (const char *print_string)
{
 print_text [0]=0;
 sprintf (print_text,print_string);

 //>----------- check for '\n'

 if (print_text [strlen (print_text)-2]=='\n')
  print_text [strlen (print_text)-2]=0;

 if (console_view!=NULL)
 {
    if (console_view->numLines ()>scrollb)
       console_view->removeLine (0);

    console_view->insertLine (print_text,-1);
    console_view->update ();
 }
}
/*---------------------------------------------------------------------------*/
void MKernelConsole::parse_token (char *token)
{
  bool found = FALSE;

  for (int i = 0; i < NR_COMMANDS; i++)
    {
      if (strcmp (commands [i],token) == 0)
	{
	  m_printf ("Executing: %s",commands [i]);   
	  execute (i);
	  found = TRUE;   
	}
    }

  if (found == FALSE)
    m_printf ("Unknown command: %s",token); 
}
/*---------------------------------------------------------------------------*/
void MKernelConsole::prompt (void)
{
  console_prompt->setText ("MindsEye $>");
  console_input->setText ("");
}
/*---------------------------------------------------------------------------*/

void MKernelConsole::toggleAbout()
{
  Kernel->toggle_module (MODULE_ABOUT);
}

void MKernelConsole::toggleLoad() 
{
  Kernel->toggle_module (MODULE_LOAD);
}

void MKernelConsole::toggleModeler()
{
  Kernel->toggle_module (MODULE_MODEL);
}

void MKernelConsole::toggleSave() 
{
  Kernel->toggle_module (MODULE_SAVE); 
}

void MKernelConsole::toggleTreeList()
{
  Kernel->toggle_module (MODULE_TREEDLG);
}

//====================================END=================================
