/*
 * Project : MindsEye
 * Copyright 1996, 1998. All Rights Reserved.
 * Author(s) M. van Velsen
 *
 * $Id: m_console.h,v 1.4 1998/07/23 20:16:17 source Exp $
 *
 * 
 * COPYRIGHT
 * =========
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 * 
 * OVERVIEW
 * ========
 *
 */

#ifndef __CONSOLE_H__
#define __CONSOLE_H__

#include "include/config.h"

#include <qlined.h>
#include <qmlined.h>
#include <qwidget.h>
#include <qlayout.h>

#include <stdio.h>
#include <stdlib.h>
#include <stdarg.h>
#include <string.h>


/** An instance of this class is used as display for text messages from
    the kernel.
  The purpose of the console is straightforward, you can give direct 
  commands to the kernel to either get something done directly or to
  get some info very quickly. The console might also be your last 
  resort to kill certain modules :) Oh and of course it's a great
  debugging feature :) :)
  */
class MKernelConsole : public QWidget
{  
  Q_OBJECT
     
public:
    ///
  MKernelConsole( QWidget*, const char* );
  ~MKernelConsole();
     
    /**This is the console's function of printf, use it just like the
       real thing. */
  void m_printf (const char *, ...);

public slots:
    ///Call a scripting language?
  void interpret ();
    ///
  void m_puts    (const char *);

  void toggleAbout();
  void toggleLoad();
  void toggleModeler();
  void toggleSave();
  void toggleTreeList();

protected:
     ///
  void execute( int );
    ///
  void initScripting( void );
     ///
  void parse_token( char* );
     ///
  void prompt( void );

private:
  int   scrollb;
   
  QMultiLineEdit *console_view;
  QLineEdit      *console_prompt;
  QLineEdit      *console_input;
  QBoxLayout     *view_box;

#ifdef _HAVE_SIOD_
  char           *iobuff_;
  long           iobufflen_;
#endif

};


#endif
