/*
 * Project : "MindsEye"
 * Copyright 1996. All Rights Reserved.
 * Author(s) M. van Velsen
 *
 * $Id: m_core.h,v 1.11 1998/07/23 20:16:17 source Exp $
 *
 *
 * COPYRIGHT
 * =========
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 * 
 * OVERVIEW
 * ========
 *
 */

#ifndef __MINDSEYE_CORE__
#define __MINDSEYE_CORE__

#include <stdio.h>
#include <stdlib.h>

#include <qobject.h>
#include <qapp.h>

#include "icons/mk_icons.h"
#include "icons/mm_icons.h"

//#include "prop_dlg.h"
#include "m_message.h"
#include "world_list.h"
#include "init3d.h"
#define red stl_red
#define black stl_black
#include <map.h>
#undef red
#undef black
#include "include/m_vars.h"


class   MEModule;
class   MKernelWidget;
class   MKernelConsole;
class   G3DShape;
class   PropertyDialog;
class   MWorldObject;
class   MEPluginManager;
class   MERenderQueue;
class   TMatWidget;

typedef char*          CreateFunc( void );
typedef G3DAttribute*  AcreateFunc( void );
typedef G3DObject*     OcreateFunc( void );
//typedef MEPropertyDlg* PcreateFunc( void );

//TODO move into plugin stuff??
/**This struct bundles the information needed by the kernel
   (MCore) to manage a module.
   */
struct module 
{
  int                load_type;       // LOAD_ON_CALL / LOAD_AND_STAY
  MEModule           *the_module;     // this is a pointer to the actual module
  struct module_info *the_module_inf; // this is what the modules hand back
  int                toggled;         // active/not-active  hidden/shown
};


/** There is only one instance of class MCore during a MindsEye session
    (singleton design pattern).
    It is created on startup in main(). This instance sets up the
    environment for all other objects in the system:
      it loads resources, ...

    MCore starts two windows which let the user interact with MindsEye
    and open other tools to do the real work (modeling, animation).
  */
class MCore : public QObject, public MMessageHandler
{
   Q_OBJECT

public:
   ~MCore (void);

   virtual void  addAttributeFactory( const char*, AcreateFunc* );
   virtual void  addAttributeWidget( TMatWidget* );
     /**Add a new property Dialog to the list of proptery dialogs:
        key is the type of the G3DObject.*/
   //virtual void  addObjectWidget( const char*, PropertyDialog *dlg );

   virtual void  addObjectClass( const char* );
   virtual void  addObjectFactory( const char*, OcreateFunc* );
     ///Add an object with name n of type t to class c.
   virtual void  addObjectName( const char *c, const char *n, const char *t );
   virtual void  addRenderQueue( MERenderQueue* );
     ///
   virtual void  addWorld( void* );
   G3DAttribute* createAttribute( const char* );
   G3DObject*    createObject( const char* );
   vector<void*>& getAttributeWidgets( void );
     ///Return a pointer to the module. Return NULL if module is not loaded.
   MEModule*     getModule( int id );
     /**Return a list of object classes. For example simple, NURBS, lights.
        Last entry is 0. The list must be deleted by the caller.*/
   const char**  getObjClassList( void );
     /**Return a list with descriptions of objects for the given class.
        Last entry is 0. The list must be deleted by the caller.*/
   const char**  getObjList( const char *cname );
   const char*   getObjTypeName( const char *cname, const char *desc );
   vector<void*>& getRenderQueues( void );
   MWorldList&   getWorldList( void ) { return worldList_; }

     ///This was a protected function ???
   void process_message( struct message* );
     ///Start processing messages and events.
   int  run            (void);

     ///Return the kernel object. Initialize the object if called the 1st time.
   static MCore* loadKernel( int, char** );

signals:
   void worldListChanged();

public slots:
     ///
   void toggle_module( int );
     ///
   void close_module( int );

private:
     //Use loadKernel to create the kernel object.
   MCore( int, char ** );
     //
   void load_module( int );
     /**The setup function can be compared to the rc.local init script.
        You can place all experimental setup stuff here. The console is
	already running at this point so you can do:
	the_console->m_printf (".....                */
   int setup          (void);
     ///QT releated init stuff
   int setup_app      (int,char **);
     ///
   int setup_chooser  (void);
     ///
   int setup_console  (void);
     ///QT signals
   int setup_sns      (void);
     //
   void shutdown( void );

private:
   QApplication     *mindseye_app;
   QFont            *app_font;
   MKernelConsole   *the_console;
   MKernelWidget    *kernel_widget;   
   QPixmap          *background;

   MWorldList       worldList_;
   MEPluginManager  *pluginMgr_;

   struct ltstr
   {
     bool operator()( const char* s1, const char* s2 ) const
     { return( strcmp(s1, s2) < 0 ); }
   };
   typedef map<const char*, const char*, ltstr> ObjList;

   map<const char*, CreateFunc*, ltstr>  attributeFactories_;
   map<const char*, CreateFunc*, ltstr>  objectFactories_;
   map<const char*, ObjList*, ltstr>     objClassList_;
   //map<const char*, CreateFunc*, ltstr>  attributeWidgets_;
   vector<void*>                         attributeWidgets_;
   vector<void*>                         renderQueues_;
   //vector<void*>  propertyDialogs_;
   //map<const char*, CreateFunc*, ltstr>  propertyDialogs_; objectWidgets_

          bool      kernel_blocked; //TODO a semaphore
   struct module    modules[NR_MODULES];  //TODO use a dynamic list?

};


//A global reference to the one and only kernel
extern MCore *Kernel;


#endif // __MINDSEYE_CORE__
