/*
 * Project : "MindsEye"
 * Copyright 1996. All Rights Reserved.
 * Author(s) M. van Velsen
 * 
 * $Id: m_kmap.cpp,v 1.5 1998/07/23 20:16:18 source Exp $
 * 
 * 
 * COPYRIGHT
 * =========
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 * 
 * OVERVIEW
 * ========
 * The is the main view of the MindsEye kernel, this widgets signals
 * if modules have to be loaded and executed.
 */

#ifdef HAVE_CONFIG_H
#include "include/config.h"
#endif	 // HAVE_CONFIG_H

#include <qapp.h>
#include <qpushbt.h>
#include <qpixmap.h>
#include <qbttngrp.h>

#include "m_kmap.h"
#ifdef WITH_KMAP_ICONS
#include "icons/mk_icons.h"
#endif
#include "include/m_vars.h"


MKernelWidget::MKernelWidget( QWidget *parent=0, const char *name=0 ) :
  QWidget(parent, name)
{
  setCaption ("MindsEye Kernel");
  init();

#ifdef WITH_KMAP_ICONS
  resize( 343,185 );
  setMinimumSize( 343,185 );
  setMaximumSize( 343,185 );
#else
  setMinimumSize( 96,298 );
  setMaximumSize( 96,298 );
  resize( 96,298 );
#endif
}

void
MKernelWidget::init( void )
{
  QButtonGroup *bgroup = new QButtonGroup;
  connect( bgroup, SIGNAL(clicked(int)), this, SLOT(toggleModule(int)) );

  QPushButton* pbtn;

#ifdef WITH_KMAP_ICONS
  QPixmap my_pix;

  pbtn = new QPushButton( this, "PushButton_1" );
  my_pix.loadFromData(load_bmp_data,load_bmp_len);
  pbtn->setPixmap( my_pix );
  pbtn->setGeometry( 45, 160, my_pix.width (),my_pix.height ());
  bgroup->insert( pbtn, MODULE_LOAD  );

  pbtn = new QPushButton( this, "PushButton_2" );
  my_pix.loadFromData(save_bmp_data,save_bmp_len);
  pbtn->setPixmap( my_pix );
  pbtn->setGeometry( 150, 160, my_pix.width (),my_pix.height ());
  bgroup->insert( pbtn, MODULE_SAVE  );

  pbtn = new QPushButton( this, "PushButton_3" );
  my_pix.loadFromData(modeler_bmp_data,modeler_bmp_len);
  pbtn->setPixmap( my_pix );
  pbtn->setGeometry( 45, 80, my_pix.width (),my_pix.height ());
  bgroup->insert( pbtn, MODULE_MODEL  );

  pbtn = new QPushButton( this, "PushButton_4" );
  my_pix.loadFromData(d_tree_bmp_data,d_tree_bmp_len);
  pbtn->setPixmap( my_pix );
  pbtn->setGeometry( 1, 1, my_pix.width (),my_pix.height ());
  bgroup->insert( pbtn, MODULE_TREEDLG  );

  pbtn = new QPushButton( this, "PushButton_5" );
  my_pix.loadFromData(material_bmp_data,material_bmp_len);
  //pbtn->setPixmap( my_pix );
  pbtn->setText( "Shaper" );
  pbtn->setGeometry( 45, 120, my_pix.width (),my_pix.height ());
  bgroup->insert( pbtn, MODULE_SHAPER );

  pbtn = new QPushButton( this, "PushButton_6" );
  my_pix.loadFromData(material_bmp_data,material_bmp_len);
  pbtn->setPixmap( my_pix );
  pbtn->setGeometry( 150, 80, my_pix.width (),my_pix.height ());
  bgroup->insert( pbtn, MODULE_MATERIAL );

  pbtn = new QPushButton( this, "PushButton_7" );
  my_pix.loadFromData(viewer_bmp_data,viewer_bmp_len);
  pbtn->setPixmap( my_pix );
  pbtn->setGeometry( 150, 120, my_pix.width (),my_pix.height ());
  bgroup->insert( pbtn, MODULE_VIEWER );

  pbtn = new QPushButton( this, "PushButton_8" );
  my_pix.loadFromData(console_bmp_data,console_bmp_len);
  pbtn->setPixmap( my_pix );
  pbtn->setGeometry( 1, 41, my_pix.width (),my_pix.height ());
  bgroup->insert( pbtn, MODULE_CONSOLE );

  pbtn = new QPushButton( this, "PushButton_9" );
  my_pix.loadFromData(config_bmp_data,config_bmp_len);
  pbtn->setPixmap( my_pix );
  pbtn->setGeometry( 1, 80, my_pix.width (),my_pix.height ());
  bgroup->insert( pbtn, MODULE_CONFIG );

  pbtn = new QPushButton( this, "PushButton_10" );
  my_pix.loadFromData(title_bmp_data,title_bmp_len);
  pbtn->setPixmap( my_pix );
  pbtn->setGeometry( 45, 1, my_pix.width (),my_pix.height ());
  bgroup->insert( pbtn, MODULE_ABOUT );

  pbtn = new QPushButton( this, "PushButton_11" );
  my_pix.loadFromData(help_bmp_data,help_bmp_len);
  pbtn->setPixmap( my_pix );
  pbtn->setGeometry( 256, 160, my_pix.width (),my_pix.height ());
  bgroup->insert( pbtn, MODULE_MODEL  );

  pbtn = new QPushButton( this, "PushButton_12" );
  my_pix.loadFromData(quit_bmp_data,quit_bmp_len);
  pbtn->setPixmap( my_pix );
  pbtn->setGeometry( 1, 160, my_pix.width (),my_pix.height ());
  connect( pbtn, SIGNAL (clicked()), qApp, SLOT (quit()) );

#else

  pbtn = new QPushButton( this, "PushButton_1" );
  pbtn->setGeometry( 0, 0, 96, 24 );
  pbtn->setText( "Load" );
  bgroup->insert( pbtn, MODULE_LOAD  );

  pbtn = new QPushButton( this, "PushButton_2" );
  pbtn->setGeometry( 0, 24, 96, 24 );
  pbtn->setText( "Save" );
  bgroup->insert( pbtn, MODULE_SAVE  );

  pbtn = new QPushButton( this, "PushButton_3" );
  pbtn->setGeometry( 0, 50, 96, 24 );
  pbtn->setText( "Modeler" );
  bgroup->insert( pbtn, MODULE_MODEL  );

  pbtn = new QPushButton( this, "PushButton_4" );
  pbtn->setGeometry( 0, 74, 96, 24 );
  pbtn->setText( "Hierarchy Editor" );
  bgroup->insert( pbtn, MODULE_TREEDLG  );

  pbtn = new QPushButton( this, "PushButton_5" );
  pbtn->setGeometry( 0, 98, 96, 24 );
  pbtn->setText( "Shaper" );
  bgroup->insert( pbtn, MODULE_SHAPER );

  pbtn = new QPushButton( this, "PushButton_6" );
  pbtn->setGeometry( 0, 124, 96, 24 );
  pbtn->setText( "Materials" );
  bgroup->insert( pbtn, MODULE_MATERIAL );

  pbtn = new QPushButton( this, "PushButton_7" );
  pbtn->setGeometry( 0, 148, 96, 24 );
  pbtn->setText( "Viewer" );
  bgroup->insert( pbtn, MODULE_VIEWER );

  pbtn = new QPushButton( this, "PushButton_8" );
  pbtn->setGeometry( 0, 174, 96, 24 );
  pbtn->setText( "Console" );
  bgroup->insert( pbtn, MODULE_CONSOLE );

  pbtn = new QPushButton( this, "PushButton_9" );
  pbtn->setGeometry( 0, 198, 96, 24 );
  pbtn->setText( "Preferences" );
  bgroup->insert( pbtn, MODULE_CONFIG );

  pbtn = new QPushButton( this, "PushButton_10" );
  pbtn->setGeometry( 0, 224, 96, 24 );
  pbtn->setText( "About" );
  bgroup->insert( pbtn, MODULE_ABOUT );

  pbtn = new QPushButton( this, "PushButton_11" );
  pbtn->setGeometry( 0, 248, 96, 24 );
  pbtn->setText( "Help" );
  bgroup->insert( pbtn, MODULE_MODEL  );

  pbtn = new QPushButton( this, "PushButton_12" );
  pbtn->setGeometry( 0, 274, 96, 24 );
  pbtn->setText( "Quit" );
  connect( pbtn, SIGNAL (clicked()), qApp, SLOT (quit()) );
#endif
}

void
MKernelWidget::toggleModule( int id )
{
  emit toggle_module( id );
}

 
//===================================END=================================
