/*
 * vi:ts=4:sw=4:
 *
 * Project : "MindsEye"
 * Copyright 1996. All Rights Reserved.
 * Author(s) M. van Velsen
 *
 * $Id: m_message.h,v 1.2 1997/12/21 20:18:45 source Exp $
 *
 * 
 * COPYRIGHT
 * =========
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 * 
 * OVERVIEW
 * ========
 * The class MMessageHandler is a base class for all class who which to
 * receive messages. This of course includes the kernel itself.
 * You inherit from MMessageHandler and overload the function: 
 *  process_message (MMessage *);
 * 
 * NOTE:
 * The function "process_message" should examine the whole message
 * because the function "remove_message" will be called after it
 */

#ifndef __M_KMESSAGE__
#define __M_KMESSAGE__

#include <string.h>
#include <stdlib.h>
#include <stdio.h>
#include <qobject.h>


//>---------- General support defines

#define DEFAULT_QSIZE	15

struct message 
{
  int message_id;       // see defines above
  
  int from_module;      // module id
  int to_module;        // to module id
  
  char *contents;       // the actual message
  int mess_length;      // message length
  
  struct message *next; // next in queue
};



class MMessageServer : public QObject
{  
    Q_OBJECT
  
 public:
     MMessageServer (int q_size=DEFAULT_QSIZE);
     ~MMessageServer ();
     
     void receive_message (struct message *);
     
 signals:
     void console_output (const char *);

 private: 
     int queue_size;
     struct message *queue_top;
     struct message *queue_bottom;     
     
};



class MMessageHandler
{
  public:
  
    MMessageHandler (void);
    ~MMessageHandler ();

    void handle_message (struct message *);

  protected:
     struct message *create_message (int,int,int,char *);
     void            post_message (struct message *);    
     virtual void    process_message (struct message *);
     void            remove_message (struct message *);
 
};

/// this is a pointer to the actual message server
extern MMessageServer *message_system;

#endif // __M_KMESSAGE__
