/*
 * vi:ts=4:sw=4:
 *
 * Project : "MindsEye, core library"
 * Copyright 1996. All Rights Reserved.
 * Author(s) M. van Velsen, M. Kuball
 *
 * $Id: m_module.h,v 1.4 1998/06/07 20:12:28 source Exp $
 * 
 * 
 * COPYRIGHT
 * =========
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 * 
 * OVERVIEW
 * ========
 * This Widget is the basis for main modules. It contains certain
 * basic characteristics nessecary for interwidget communication 
 * with the MindsEye kernel
 * 
 * The widget contains extra code for dynamic loading management. For 
 * Example: When a user closes a window it depends on how the module
 * was loaded to determine it's behaviour. There are two types of module
 * loading:
 *  
 *   - Load on call
 *   - Load on call and stay 
 *    
 * When a user closes a window which is "Load on call" the window will hide
 * and the object will be destroyed. On the other hand when a user closes a
 * window the code will stay in memory. If the kernel get's a request to
 * load this module then it won't actually load it but just ->show () it
 * again.     
 *
 * NOTE:
 *
 * Each module should be capable of loading plugins. The plugins
 * do not make notice of the fact that a module is resident or
 * volatile. Plugins will be unloaded even if they are loaded
 * resident. 
 *
 * REMINDER: split the module code into: abstract code and widget code
 */


#ifndef __M_MODULE_H__ 
#define __M_MODULE_H__ 

#include <qwidget.h>
#include <qframe.h>
#include "m_message.h"

#define LOAD_ON_CALL   0
#define LOAD_AND_STAY  1


//class MKernelConsole;
//class MCore;


struct module_info
{
  int  major_version;
  int  minor_version;
  int  patch_level;
  char module_name [25]; //TODO this is dangerous
};


//TODO add plugin management??
//TODO include MMessageHandler stuff ?

/**The base class of all Modules.
 */
class MEModule :  public QWidget, public MMessageHandler
{
   Q_OBJECT

public:
   MEModule( int, int, const char *name=0 );
   virtual ~MEModule ();

   virtual struct module_info *get_module_info( void );  //TODO necessary?
   //TODO virtual void  hide( );
   //virtual void show( );

signals:
   void close_me(int);

protected:
   ///
   virtual void  closeEvent( QCloseEvent* );

protected:
   int id;
   //int load_type; //TODO do we need this?

   struct info_pack   *core_pack;
   struct module_info *m_info;

};


#endif
