#include "appapp.h"

#include <assert.h>

const char *path[] = { "./plugins", NULL };

void App::loadDlls()
{
  const char **dir = path;
   
  while (*dir) {
    mDllDir.loadDllsFromDir( *dir, NULL );
    dir++;
  }

  cout << "DLLs loaded: " << mDllDir.numDllsLoaded() << endl;

  mDllDir.print();

  createCategories();
}

////////////////////

SuperClassEntry App::sSClassEntries[] = {
  SuperClassEntry(CAMERA_CLASS_ID),
  SuperClassEntry(LIGHT_CLASS_ID),
  SuperClassEntry(GEOMOBJ_CLASS_ID),
  SuperClassEntry(SHAPE_CLASS_ID),
  SuperClassEntry(HELPER_CLASS_ID),
};

int App::sNumSClassEntries = sizeof(App::sSClassEntries)/sizeof(SuperClassEntry);

////////////////////

CategoryList *App::getCategoryList(SClassID superClassID) const
{
  for (int i=0; i<sNumSClassEntries; i++) 
    if (sSClassEntries[i].superClassID() == superClassID)
      return sSClassEntries[i].getCategoryList();
  return 0;
}
     
void App::createCategories()
{
  ClassDirectory &cd = mDllDir.getClassDir();

  for (int i=0; i<sNumSClassEntries; i++)
  {
    SubClassList *scl = cd.getClassList( sSClassEntries[i].superClassID() );
    if (!scl) {
      cerr << "[App::createCategories] no subclasses for super class ID: " << sSClassEntries[i].superClassID() << endl;
      continue;
    }

    CategoryList *tCatList = new CategoryList;
    
    for (int k=0; k<scl->numClasses(); k++) 
    {
      ClassEntry &ce = (*scl)[ k ];

      //      cerr << "                      (" << ce.className() << ", " << ce.category() << endl;

      if (ce.isPublic()) {
	assert(strcmp(ce.category(), "") != 0);
	tCatList->addClassInfo( ce.category(), ce.classInfo() );
      }
    }    

    sSClassEntries[i].setCategoryList( tCatList );
  }
}

////////////////////////////////////////////////////////////////////////////////

void printCategories(App *app)
{
  SClassID superClasses[] = { CAMERA_CLASS_ID, LIGHT_CLASS_ID, GEOMOBJ_CLASS_ID, HELPER_CLASS_ID };
  int numSClasses = 4;

  cout << "___________________________________________________________" << endl;
  for (int i=0; i<numSClasses; i++) {
    CategoryList *cl = app->getCategoryList(superClasses[i]);
    cout << "[" << i << "] ";
    if (!cl) 
      cout << "no categories found." << endl;
    else {
      cout << "sclass ID: " << superClasses[i] << endl;
      for (int j=0; j<cl->numCategories(); j++) 
      {
	Category *ca = (*cl)[j];
	cout << "  category: " << ca->categoryName() << endl;
	for (int k=0; k<ca->numClassInfos(); k++) 
        {
	  ClassInfo *ci = (*ca)[k];
	  cout << "    " << ci->className() << endl;
	}
      }
    }
  }
  cout << "___________________________________________________________" << endl;
      
}

void printClassDir(App* app)
{
  ClassDirectory &cd = app->mDllDir.getClassDir();

  cout << "====================================== ClassDirectory" << endl;
  cout << " num super classes: " << cd.numClasses() << endl;
  for (int i=0; i<cd.numClasses(); i++) {
    SubClassList &scl = cd[i];
    cout << "  sclass ID: " << scl.superClassID() << endl;

    for (int j=0; j<scl.numClasses(); j++) {
      ClassEntry &ce = scl[j];
      cout << "    class: " << ce.className() << endl;
    }
  }
  
  cout << "====================================== ClassDirectory" << endl;
}

