/*
 * Project : "MindsEye"
 * Copyright 1996. All Rights Reserved.
 * Author(s) Frank Meier
 *
 * $Id: m_core.h,v 1.6 1998/03/19 21:22:31 source Exp $
 *
 *
 * COPYRIGHT
 * =========
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 * 
 * OVERVIEW
 * ========
 *
 */

#ifndef __CLASSINFO_H__
#define __CLASSINFO_H__

#include "eotypes.h"
#include "eoclasses.h"

class Load;
class Save;

////////////////////////////////////////

/**Used for?
 */
class ClassID
{
public:
  ClassID(unsigned, unsigned);    // built-in classes have 'bb==0'
  ClassID();

  unsigned partA() const { return aa; };
  unsigned partB() const { return bb; };

  bool operator== (const ClassID&) const;
  bool operator!= (const ClassID&) const;

protected:
  unsigned aa,bb;
};

typedef unsigned  SClassID;

////////////////////

inline ClassID::ClassID(unsigned a,unsigned b) : aa(a), bb(b) {}

inline ClassID::ClassID() : aa(0xffffffff), bb(0xffffffff) {}

inline bool ClassID::operator==(const ClassID& cid) const
  { return (aa==cid.aa && bb==cid.bb); }

inline bool ClassID::operator!=(const ClassID& cid) const
  { return (aa!=cid.aa || bb!=cid.bb); }

////////////////////////////////////////


/**Base class of all plugins
 */
class ClassInfo
{
public:
  virtual ~ClassInfo() {}

  virtual void *create(bool loading=false) =0;
  virtual bool isPublic() =0;

    /// "" empty string is 'standard' category
  virtual const char  *category() =0;
  virtual ClassID      classID() =0;
  virtual const char  *className() =0;
  virtual IOResult    load(Load *) { return IO_OK; }
    /// for class data only, not instance data
  virtual bool        needsToSave() { return false; }
  virtual IOResult    save(Save *) { return IO_OK; }
  virtual SClassID    superClassID() =0;

  //  virtual int beginCreate(Interface *) { return 0; }
  //  virtual int endCreate(Interface *) { return 0; }
  //  virtual bool OkToCreate(Interface *i) { return true; }  
};

#endif

