#include "dll.h"

#include <sys/types.h>
#include <dirent.h>
#include <dlfcn.h>
#include <assert.h>

////////////////////////////////////////////////////////////////////////////////

unsigned getEhnoVersion() { return EHNO_VERSION; }   // application version

//////////////////////////////////////////////////////////////////////

void *createInstance(SClassID superClassID, ClassID classID)
{
  ClassInfo *ci;
  /*  if ((ci = theApp.getDllDir().getClassDir().findClass(superClassID,classID)) == NULL) {
    cerr << "[createInstance] no such class" << endl;
    assert(0);
  } */
  return (void *)ci->create();
}


////////// SubClassList ////////////////////////////////////////////////////

SubClassList::SubClassList(SClassID superClassID)
  : mSuperClassID(superClassID)
{
}

SClassID SubClassList::superClassID()
{
  return mSuperClassID;
}

int SubClassList::findClass(ClassID classID)
{
  for (unsigned i=0; i<mClassEntries.size(); i++)
    if (mClassEntries[i]->classID() == classID)
      return i;

  return -1;
}

int SubClassList::findClass(const char *className)
{
  for (unsigned i=0; i<mClassEntries.size(); i++)
    if (strcmp(mClassEntries[i]->className(),className) == 0)
      return i;

  return -1;
}

int SubClassList::numClasses()     // int numClasses(type) type: public,private,both
{
  return mClassEntries.size();
}

/* 
int SubClassList::getFirst()       // dito. (type)
{
  if (!numClasses())
    return -1;
  
  mIterator=0;
  return mIterator;
}
 
int SubClassList::getNext() // dito.
{
  if (mIterator < numClasses())
    return mIterator++;
  else
    return -1;
}
*/

bool SubClassList::addClass(ClassInfo *ci, int dllNum, int index)
{
  int ceIndex = findClass(ci->classID());

  if (ceIndex > -1) {
    cerr << "[SubClassList::addClass] already have this: \"" << ci->className() << "\"" << endl;
    return false;
  }

  ClassEntry *nce = new ClassEntry(ci,dllNum,index);

  mClassEntries.reserve(mClassEntries.size()+1);
  mClassEntries.push_back(nce);
  
  return true;
}
 
ClassEntry& SubClassList::operator[] (int i)
{
  return * mClassEntries[i];
}

////////// ClassDirectory //////////////////////////////////////////////////

ClassDirectory::ClassDirectory()
{
}

SubClassList *ClassDirectory::getClassList(SClassID superClassID)
{
  for (unsigned i=0; i<mClassList.size(); i++)
    if (mClassList[i]->superClassID() == superClassID)
      return mClassList[i];

  return 0;

  /*  for (vector<SubClassList *>::iterator subcl=mClassList.begin(); subcl!=mClassList.end(); subcl++)
    if ((*subcl)->superClassID() == superClassID)
      return (*subcl);

  return NULL; */
}

ClassInfo *ClassDirectory::findClass(SClassID superClassID, ClassID classID)
{
  return findClassEntry(superClassID,classID)->classInfo();
}

ClassEntry *ClassDirectory::findClassEntry(SClassID superClassID, ClassID classID)
{
  SubClassList *scl = getClassList(superClassID);
  if (scl == NULL)
    return NULL;
  int ceIndex = scl->findClass(classID);
  ClassEntry *ce = & (*scl)[ ceIndex ];
  assert(ce);
  return ce;
}
  
SubClassList *ClassDirectory::addSuperClass(SClassID superClassID)
{
  SubClassList *nScl = getClassList(superClassID); 

  if (!nScl) {
    nScl = new SubClassList(superClassID);   // we don't have this one. create new SubClassList

    mClassList.reserve(mClassList.size()+1);
    mClassList.push_back( nScl );
  }

  return nScl;
}

bool ClassDirectory::addClass(ClassInfo *ci, int dllNum, int index)
{
  SubClassList *tScl = addSuperClass(ci->superClassID());
  return tScl->addClass(ci, dllNum, index);
}

int ClassDirectory::numClasses() const
{
  return mClassList.size();
}

SubClassList& ClassDirectory::operator[] (int i)
{
  return *(mClassList[i]);
}

////////// ClassEntry ///////////////////////////////////////////////

ClassEntry::ClassEntry(ClassInfo *ci, int dllNum, int index)
  : mClassInfo(ci),
    mDllIndex(dllNum),
    mIndexInDll(index)    
{
}

ClassInfo *ClassEntry::classInfo()
{
  return mClassInfo;
}

bool ClassEntry::isPublic() const
{
  return mClassInfo->isPublic();
}

ClassID ClassEntry::classID() const
{
  return mClassInfo->classID();
}

const char *ClassEntry::className()
{
  return mClassInfo->className();
}

const char *ClassEntry::category()
{
  return mClassInfo->category();
}

int ClassEntry::dllNumber()
{
  // index into master dll list
  return mDllIndex;
}

////////// DllInfo //////////////////////////////////////////////////
DllInfo::DllInfo(void *handle,
		 const char *dllName,
		 const char *(*fDesc)(),
		 int (*fNumClasses)(),
		 ClassInfo *(*fClassInfo)(int))
  : mHandle(handle), 
    mDllFilename(dllName), 
    libDescription(fDesc), 
    libNumClasses(fNumClasses), 
    libClassInfo(fClassInfo) 
{
}

const char *DllInfo::getDescription()
{
  return libDescription();
}

int DllInfo::getNumClasses()
{
  return libNumClasses();
}

ClassInfo *DllInfo::operator[] (int i)
{
  return libClassInfo(i);
}

void DllInfo::free()
{
  dlclose(mHandle);
}

bool DllInfo::operator== (const DllInfo&)
{
  assert(0);         // there is no simple way to do this, 
  return false;      // comparing all libClassInfo(..)->classIDs() may not be safe enough
                     // see the end of loadDll() for preventing multiple loads of the same dll
}


////////////////////////////////////////////////////////////////////////////////

DllDir::~DllDir()
{
  unloadAllDlls();
}

int DllDir::numDllsLoaded() const
{
  return mDlls.size();
}

DllInfo& DllDir::operator[] (int i)
{
  return *(mDlls[i]);
}


void DllDir::loadDll(const char *pDllName)
{
  void *handle;
  
  handle = dlopen(pDllName, RTLD_LAZY);
  if (!handle) {
    cerr << "[DllDir::loadDll] dlopen " << dlerror() << endl;
    return;
  }
  
  const char *error;

  void (*libInit)();
  int (*libNumClasses)();
  ClassInfo *(*libClassInfo)(int);
  const char * (*libDesc)();
  unsigned (*libVersion)();

  libInit = (void (*)()) dlsym(handle, "libInit__Fv");
  if ((error = dlerror()) != NULL) {
    cerr << "[dlsym] libInit: " << error << endl;
    dlclose(handle);
    return;
  }

  libNumClasses = (int (*)()) dlsym(handle, "libNumClasses__Fv");
  if ((error = dlerror()) != NULL) {
    cerr << "[dlsym] libNumClasses: " << error << endl;
    dlclose(handle);
    return;
  }

  libClassInfo = (ClassInfo *(*)(int)) dlsym(handle, "libClassInfo__Fi");
  if ((error = dlerror()) != NULL) {
    cerr << "[dlsym] libClassInfo: " << error << endl;
    dlclose(handle);
    return;
  }

  libVersion = (unsigned (*)()) dlsym(handle, "libVersion__Fv");
  if ((error = dlerror()) != NULL) {
    cerr << "[dlsym] libVersion: " << error << endl;
    dlclose(handle);
    return;
  }

  libDesc = (const char *(*)()) dlsym(handle, "libDescription__Fv");
  if ((error = dlerror()) != NULL) {
    cerr << "[dlsym] libDesc: " << error << endl;
    dlclose(handle);
    return;
  }  

  //////////

  if (libVersion() != getEhnoVersion()) {
    cerr << "[DllDir::loadDll] plug-in version mismatch: \"" << pDllName << "\"" << endl;
    return;
  }

  if (libNumClasses() == 0) {
    cerr << "[DllDir::loadDll] plug-in contains no classes: \"" << pDllName << "\"" << endl;
    return;
  }

  libInit();

  ////////////////////////////////////////

  int dllNum = mDlls.size();       // index of this DLL in master list
  bool keepDll=true;
  int thereWasAnError=-1;          // this holds the index of the libClassInfo(index) that produced the error(if any)

  for (int i=0; i<libNumClasses(); i++)
    if (!mClassDir.addClass(libClassInfo(i), dllNum, i)) {
      thereWasAnError=i;
      break;
    }

  if (thereWasAnError >= 0) 
  {
    cerr << "[DllDir::loadDll] cannot (completely) load dll: \"" << pDllName << "\"" << endl;

    if (thereWasAnError == 0)
      keepDll=false;       // error on libClassInfo(0) so we skip this DLL
    else
      ;                    // oops, we've load part of it, so we need to keep the DLL
  }

  if (keepDll) {
    DllInfo *dllInfo = new DllInfo(handle, pDllName, libDesc, libNumClasses, libClassInfo);
      
    mDlls.reserve(mDlls.size()+1);
    mDlls.push_back(dllInfo);      
  } else 
    dlclose(handle);
}

bool DllDir::loadDllsFromDir(const char *pDirName, const char *pWildcard)
{
  DIR *dir;
  struct dirent *de;

  char dllFile[1024];
  
  if ((dir = opendir(pDirName)) == NULL) 
  {
    perror("[DllDir::loadDllsFromDir] opendir");  
    closedir(dir);
    return false;
  }

  while ((de = readdir(dir))) {
    if ((strcmp(de->d_name, ".") == 0) ||
	(strcmp(de->d_name, "..") == 0))
      continue;
				      
    //    cerr << "plugin: " << de->d_name << endl;
    strcpy(dllFile, pDirName);
    strcat(dllFile, "/");
    strcat(dllFile, de->d_name);
    cerr << "plugin: " << dllFile << endl;
    loadDll(dllFile);    // HIIIIIIER "pDirName + de.d_name"
  }
 
  closedir(dir);

  return true;
}

void DllDir::unloadAllDlls()
{
  for (vector<DllInfo *>::iterator dll=mDlls.begin(); dll!=mDlls.end(); dll++) {
    (*dll)->free();
    delete (*dll);   
  }
}

ClassDirectory& DllDir::getClassDir()
{
  return mClassDir;
}

void DllDir::print()
{
  cout << "-- DllDir ---------------------------------------------" << endl;
  for (unsigned i=0; i<mDlls.size(); i++) 
  {
    DllInfo& dllInfo = (*this)[i];

    cout << " DLL " 
	 << "num classes: " << dllInfo.getNumClasses() << ""
	 << " desc: \"" << dllInfo.getDescription() << "\"" << endl;	 
  }
  cout << "-- " << mDlls.size() << " DLL(s) ----------------------" << endl;
}




