#ifndef __EODLL_H__
#define __EODLL_H__

#include <std/string.h>
#include <vector>

#include "classinfo.h"


class DllInfo
{
public:
  DllInfo(void *,
	  const char *,
	  const char *(*)(),
	  int (*)(),
	  ClassInfo *(*)(int));

  const char *getDescription();
  int         getNumClasses();
  ClassInfo  *operator[] (int);

  void        free();

  bool        operator== (const DllInfo&);

protected:
  void    *mHandle;
  string   mDllFilename;

  const char * (*libDescription)();
  int          (*libNumClasses)();
  ClassInfo  * (*libClassInfo)(int);
};


////////////////////////////////////////


class ClassEntry
{
public:
  ClassEntry(ClassInfo *,int,int);

  ClassInfo *classInfo();

  bool isPublic() const;
  ClassID classID() const;
  const char *className();
  const char *category();

  int dllNumber();   // index into master dll list

protected:
  ClassInfo  *mClassInfo;
  int         mDllIndex;
  int         mIndexInDll; // index of class within dll(libClassInfo(index))
};

class SubClassList
{
public:
  SubClassList(SClassID);

  SClassID superClassID();

  int findClass(ClassID);
  int findClass(const char *);

  bool addClass(ClassInfo *ci, int dllNum, int index);

  int numClasses();     
  ClassEntry& operator[] (int);
protected:
  SClassID             mSuperClassID;
  vector<ClassEntry *> mClassEntries;

  int   mIterator;
};

class ClassDirectory
{
public:
  ClassDirectory();

  SubClassList *getClassList(SClassID);
  ClassInfo    *findClass(SClassID, ClassID);
  ClassEntry   *findClassEntry(SClassID, ClassID);
  
  SubClassList *addSuperClass(SClassID);
  bool          addClass(ClassInfo *, int dllNum, int index);

  int           numClasses() const;
  SubClassList&	operator[] (int);
private:
  vector<SubClassList *> mClassList;
};


class DllDir
{
public:
  ~DllDir();

  int       numDllsLoaded() const;
  DllInfo&  operator[] (int);

  bool      loadDllsFromDir(const char *dir, const char *wildcard);
  void      unloadAllDlls();

  ClassDirectory&  getClassDir();

  void print();

protected:
  void      loadDll(const char *);

protected:
  vector<DllInfo *> mDlls;
  ClassDirectory    mClassDir;
};


#endif
