/*
 * Project : "MindsEye"
 * Copyright 1996, 1998. All Rights Reserved.
 * Author(s) Frank Meier, M. Kuball
 *
 * $Id: m_core.h,v 1.6 1998/03/19 21:22:31 source Exp $
 *
 *
 * COPYRIGHT
 * =========
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 * 
 * OVERVIEW
 * ========
 *
 */

#include "classinfo.h"

class G3DObject;

G3DObject* createTest( void );

//Define the entry points for the plugin loader

int
libInit( void )
{
  //  Kernel->addObjectCreator( G3DPrisma::GetTypeName(), createPrisma );
}

int
libNumClasses( void )
{
  return( 1 );
}

const char*
libDesc( void )
{
  return( "An example plugin" );
}

unsigned
libVersion( void )
{
  return EHNO_VERSION;
}


//---------------------------------------------------------------

#include "3dlib/shape.h"

class G3DPrisma : public G3DShape
{
public:
  G3DPrisma( void );
  G3DPrisma( const G3DPrisma& );
  virtual ~G3DPrisma();

    ///
  virtual G3DObject* copy( void ) const;
    ///Return a list with the names and types of the object s parameter.
  virtual const Field* getFieldList( void );
    /**Copy pointers to the geometric data to the list. The order is:
       origin, e1, e2, e3*/
  virtual uint   getParameter( Param* );
    ///
  virtual void   getPosition( G3DPoint3d& ) const;
    ///Return a Descriptive name of the shape.
  virtual const char* getTypeName( void ) const;
  static  const char* GetTypeName( void );
    ///
  virtual void   resetBoundingBox( void );
    ///
  virtual void   setPosition( const G3DPoint3d& );
    ///
  virtual void   shade( int ) const;

protected:
  void  tesselate( void );

protected:
  G3DPoint3d  _origin;
  G3DVector3d _e1, _e2, _e3;  //Position + geometry of the prisma

  G3DVector3d _points[6]; //TODO points
  G3DVector3d _norm[5];

  static Field FieldList_[4];

};

G3DObject*
createPrisma( void )
{
  return( new G3DPrisma );
}


//-------------------------------------------------------------------


#include <GL/gl.h>
#include <GL/glu.h>


G3DPrisma::G3DPrisma( void ) :
  _origin(0,0,0), _e1(0,0,1), _e2(0.5,0,0), _e3(0,1,1)
{
  type_ = T_PRISMA;  //TODO
  tesselate();
}

G3DPrisma::G3DPrisma( const G3DPrisma &orig ) :
  G3DShape( orig )
{
  _origin = orig._origin;
  _e1     = orig._e1;
  _e2     = orig._e2;
  _e3     = orig._e3;
  tesselate();  //TODO copy
}

G3DPrisma::~G3DPrisma()
{
}


G3DObject*
G3DPrisma::copy( void ) const
{
  return( new G3DPrisma(*this) );
}

Field G3DPrisma::FieldList_[4] =
{ {"origin", s_point}, {"e1", s_vector}, {"e2", s_vector}, {"e2", s_vector} };

const Field*
G3DPrisma::getFieldList( void )
{
  return( FieldList_ );
}


uint
G3DPrisma::getParameter( Param *pList )
{
  if( pList )
    {
      pList->point  = &_origin;
      pList++;
      pList->vector = &_e1;
      pList++;
      pList->vector = &_e2;
      pList++;
      pList->vector = &_e3;
    }
  return( 4 );
}

void
G3DPrisma::getPosition( G3DPoint3d &pos ) const
{
  pos = _origin;
}

const char*
G3DPrisma::GetTypeName( void )
{
  return( "Prisma" );
}

const char*
G3DPrisma::getTypeName( void ) const
{
  return( G3DPrisma::GetTypeName() );
}

void
G3DPrisma::resetBoundingBox( void )
{
  G3DVector3d v = _e1.cross( _e2 );
  v.normalize();
  v *= v.dot( _e3 );

  delete bbox_;
  bbox_ = new G3D_BBFree( _origin, v, _e1, _e2 );
  if( parent_ )
    ((G3DShape*)parent_)->resetBoundingBox();
}

void
G3DPrisma::setPosition( const G3DPoint3d &pos )
{
  _origin = pos;
  tesselate();
  resetBoundingBox();
}

void
G3DPrisma::shade( int ) const
{
  static int faces[] = { 0,1,2,3, 1,4,5,2, 0,3,5,4,
			 0,4,4,1, 2,5,5,3 };
  const int *facep = faces;

  glBegin( GL_QUADS );
  for( int i = 0; i < 5; i++ )
    {
      glNormal3dv( (const double*)(_norm[i]) );
      glVertex3dv( (const double*)(_points[*(facep++)]) );
      glVertex3dv( (const double*)(_points[*(facep++)]) );
      glVertex3dv( (const double*)(_points[*(facep++)]) );
      glVertex3dv( (const double*)(_points[*(facep++)]) );
    }
  glEnd();
}

void
G3DPrisma::tesselate()
{
  G3DVector3d  t1, t2, t3;

  _points[0] = _origin;
  _points[1] = t1 = _origin + _e1;
  _points[2] = t1 + _e2;
  _points[3] = _origin + _e2;
  _points[4] = t1 = _origin + _e3;
  _points[5] = t1 + _e2;

  _norm[0] = _e2.cross( _e1 );
  _norm[0].normalize();
  _norm[1] = _e2.cross( _e3 - _e1 );
  _norm[1].normalize();
  _norm[2] = _e3.cross( _e2 );
  _norm[2].normalize();
  _norm[3] = _e1.cross( _e3 );
  _norm[3].normalize();
  _norm[4] = _norm[3] * (-1.0);
}



//=================================END==============================
