/*
 * Project : "MindsEye"
 * Copyright (c) 1996 - 1998. All Rights Reserved.
 * Author(s) M. Kuball
 *
 * $Id: $
 * 
 * 
 * COPYRIGHT
 * =========
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 * 
 * OVERVIEW
 * ========
 *
 */

#include "include/config.h"
#include <qlabel.h>
#include <qmlined.h>
#include "dialogs/color/colorpicker.h"
#include "3dlib/scene.h"
#include "3dlib/t_fields.h"
#include "prop_dlg.h"


PropertyDialog::~PropertyDialog()
{
}

void
PropertyDialog::addPages( void )
{
}

void
PropertyDialog::hideWindow( void )
{
  hide();
}


//TODO group properties??
//--------------------------------------------------------------------


ObjectProperties::ObjectProperties( QWidget *parent, const char *name ) :
  PropertyDialog(parent, name)
{
  setMinimumSize( 250, 200 );
  setMaximumSize( 500, 400 );
  resize( 310, 220 );

  setApplyButton();

  _obj = NULL;
  //_gpage = new QWidget( this, "Geometry" );
  //QLabel *l = new QLabel( "List of Geometric values", _gpage, NULL );
  //l->setGeometry( 10,10, 100, 15 );
  //addTab( _gpage, "Geometry" );

  _apage = new QWidget( this, "Attributes" );
}

void
ObjectProperties::addPages( void )
{
  addTab( _apage, "Attributes" );
}

void
ObjectProperties::setObject( G3DObject *obj )
{
  _obj = obj;
//TODO init _apage

  show();
}


//--------------------------------------------------------------------


ScenePropertiesDlg::ScenePropertiesDlg( G3DScene *scene ) :
  PropertyDialog(NULL, NULL)
{
  setCaption( "Scene properties" );

  scene_ = scene;

  QWidget *page;
  dpage_ = new MESceneInfo( this );
  addTab( dpage_, "Description" );

  apage_ = new QWidget( this, "Attributes" );
  addTab( apage_, "Attributes" );

  connect( this, SIGNAL(applyButtonPressed()), SLOT(applyValues()) );

  setMinimumSize( 200, 150 );
  setMaximumSize( 500, 400 );
  resize( 200, 150 );
}


void
ScenePropertiesDlg::applyValues()
{
  dpage_->applyValues( scene_ );
}


MESceneInfo::MESceneInfo( QWidget *parent ) :
  QWidget( parent, "SceneInfo" )
{
    //This one is used to add some informative text to the scene
  infoEditor_ = new QMultiLineEdit( this, NULL );
  infoEditor_->setGeometry( 5, 5, 150, 60 );
  infoEditor_->setText( "Enter a description" );

  colButton_ = new QPushButton( "Color", this );
  connect( colButton_, SIGNAL(clicked()), SLOT(setBgColor()) );
  colButton_->setGeometry( 5, 65, 60, 25 );
}

void
MESceneInfo::applyValues( G3DScene *scene )
{
  Param params[2];
  scene->getParameter( params );
  *(params->color) = bgColor_; 
  //TODO info text
}

void
MESceneInfo::setBgColor()
{
  QColor col;

  MEColorPicker::colorFromUser( &col );
  colButton_->setPalette( QPalette(col) );
  bgColor_.setTo( col.red(), col.green(), col.blue() );
}


//=====================================END================================
