/*
 * Project : "MindsEye, 3D library"
 * Copyright 1996, 1997. All Rights Reserved.
 * Author(s): M. Kuball
 *
 * $Id: world_obj.h,v 1.14 1998/07/23 20:16:18 source Exp $
 *
 *
 * COPYRIGHT
 * =========
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

#ifndef _WORLD_OBJ_H_
#define _WORLD_OBJ_H_

#include <qobject.h>
#include "3dlib/camera.h"
#include "world_list.h"

class G3DObject;
class G3DShape;
class G3DGroup;
class G3DScene;
class G3DLight;
class MWorldObject;
class ActionData;


/** Objects of this class bundles all the information necessary
    to view and edit a scene. I.e. one or more shader objects and the scene
    itself.  TODO
      scene exactly 1
      selection none | 1 object | group
      cameras >= 1
    It supplies the editor with methods to manipulate the scene.

    You can add dependents to a world object via the QT
    SIGNAL SLOT mechanism.

    If a modul selects an object in the scene it tells the world via the

         selectObject()

    function. The world object will then emit the signal objectSelected.
    To inform a world object that a shape was added to or deleted from
    the scene use the function

         update()

    The world object will then emit the signal sceneChanged.

    TODO
    The world object stores additional information about the scene:
      original type of the file
      change log
      statistic information
    */
class MWorldObject : public QObject
{
  Q_OBJECT

public:
    ///Create an empty world (named scratch).
  MWorldObject( void );
    ///
  MWorldObject( G3DScene *scene, MCameraList *camera );
  ~MWorldObject();

    ///Add the camera to the camera list.
  void         addCamera( G3DCamera* );
    ///Add the light to the active group.
  void         addLight( G3DLight* );
    ///Add the object to the active group.
  void         addObject( G3DObject* );
    ///Add the shape to the active group.
  void         addShape( G3DShape* );
    ///
  G3DObject*   addToSelection( G3DObject* );
    ///
  void         deleteObject( G3DObject* );
    ///
  void         doWithScene( ActionData& );
    ///
  void         doWithSelection( ActionData& );
    ///
  G3DCamera*   getActiveCamera( void ) { return &activeCamera_; };
    ///
  int          getFlag() { return(flag_); };
    ///Get the name of the associated scene.
  const char*  getName() const;
    ///
  G3DScene*    getScene() { return(scene_); };
    ///
  MCameraList* getCameras() { return(cameraList_); };
    ///Get the currently selected object.
  G3DObject*   getSelection( void ) { return(selection_); };
    ///If > 1 objects are selected, move them into a new group.
  void         groupSelectedObjects( void );
    ///Return number of cameras.
  uint         numberOfCameras( void );
    /**Set the obj to selected mode and send signal objectSelected
      to all dependents.*/
  void         selectObject( G3DObject *obj );
    ///
  void         setActiveCamera( const G3DCamera * );
    ///The specified group will be the parent of all inserted objects.
  void         setActiveGroup( G3DGroup *grp );
    ///
  void         setFlag(int flag ) { flag_ = flag; };
    /**Send signal objectChanged to all dependents.*/
  void         update( void );
    /**Send signal sceneChanged to all dependents. The group
       specifies the part of the scene that changed.*/
  void         update( G3DGroup* );

signals:
  void         decorateObject( G3DObject* );
    ///
  void         objectChanged();
    ///
  void         objectSelected(G3DObject*);
    ///
  void         sceneChanged(G3DGroup*);

protected:
    ///Delete a Light from the scene
  void           deleteLight( G3DLight* );
    ///Delete an Object from the scene
  void           deleteShape( G3DShape* );

protected:
  G3DScene    *scene_;
  G3DGroup    *activeGroup_;
  G3DObject   *selection_;
  MCameraList *cameraList_;
  G3DCamera   activeCamera_;
    ///TODO not used yet maybe set if changes not saved
  int         flag_;

};

//TODO use a STL list?
//typedef G3DDLList<MWorldObject> MWorldObjectList;

#endif
