/*
 * Project : "MindsEye"
 * Copyright 1996 - 1998. All Rights Reserved.
 * Author(s): M. van Velsen, M. Kuball
 *
 * $Id: m_mat.cpp,v 1.4 1998/06/22 20:12:41 source Exp $
 *
 *
 * COPYRIGHT
 * =========
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 * 
 * OVERVIEW
 * ========
 *
 */

#ifdef HAVE_CONFIG_H
#include "include/config.h"
#endif	 // HAVE_CONFIG_H

#include <qbitmap.h>
#include <qlayout.h>
//#include <qgrpbox.h>
#include <qlabel.h>
#include <qcombo.h>
#include <qpushbt.h>

#include "3dlib/Attributes/texture.h"
#include "m_mat.h"
#include "mat_widget.h"
#include "kernel/m_core.h"
#include "include/cursors.h"


/*---------------------------------------------------------------------------*/
MEMaterialEditor::MEMaterialEditor( int module_id, int load_t ) :
  MEModule(module_id, load_t, "MaterialWidget")
{
  createPreviews();
  label_ = new QLabel( "Choose a material type", this );
  label_->setAlignment( AlignHCenter|AlignVCenter );
  label_->setFixedSize( label_->sizeHint() );
  selType_ = new QComboBox( this );
  selType_->setMinimumSize( 40, 24 );
  connect( selType_, SIGNAL(activated(int)), SLOT(createLayout(int)) );

  closeButton_ = new QPushButton( "Close", this, "Close" );
  //quitButton->setText( "Cancel" );
  closeButton_->setMinimumSize( 80, 25 );
  connect( closeButton_, SIGNAL(clicked()), SLOT(closeIt()) );

  vector<void*> &widgets = Kernel->getAttributeWidgets();
  for( int i = 0; i < widgets.size(); ++i )
    {
      selType_->insertItem( ((TMatWidget*)widgets[i])->name );
    }

  gmMain_    = NULL;
  gmh1_      = NULL;
  gmh2_      = NULL;
  matWidget_ = NULL;
  createLayout( 0 );
  active_ = 0;

  setCaption ("MindsEye Materials editor");
  resize( 450, 450 );
}

MEMaterialEditor::~MEMaterialEditor ()
{

}

/*---------------------------------------------------------------------------*/

void
MEMaterialEditor::closeIt()
{
  close();
}

void
MEMaterialEditor::createLayout( int id )
{
  delete matWidget_;
  delete gmh1_;
  delete gmh2_;
  delete gmMain_;

  gmMain_ = new QVBoxLayout( this, 4, 4 );
  gmMain_->addWidget( previewFrame_, 0 );

  gmh1_ = new QHBoxLayout();
  gmMain_->addLayout( gmh1_, 0 );

  gmh1_->addWidget( label_, 1 );
  gmh1_->addStretch( 2 );
  gmh1_->addWidget( selType_, 100 );

  vector<void*> &widgets = Kernel->getAttributeWidgets();
  MWCreator *mwFunc;
  mwFunc = ((TMatWidget*)widgets[id])->factory;
  matWidget_ = mwFunc( this );
  matWidget_->setMinimumSize( 10, matWidget_->height() );
  matWidget_->show();
  connect( matWidget_, SIGNAL(changed()), SLOT(setMaterial()) );
  gmMain_->addWidget( matWidget_, 0 );

  gmMain_->addStretch( 100 );

  gmh2_ = new QHBoxLayout();
  gmMain_->addLayout( gmh2_, 0 );
  gmh2_->addStretch( 100 );
  gmh2_->addWidget( closeButton_, 0 );

  gmMain_->activate();
  //TODO what about a resize?
  update();
}

void
MEMaterialEditor::createPreviews( void )
{
 //>------- create cursors for the viewports
  //QBitmap *cb1 = new QBitmap(CURSOR_X, CURSOR_Y, hand_open_cur_bits, FALSE);
  //QBitmap *cm1 = new QBitmap(CURSOR_X, CURSOR_Y, hand_open_msk_bits, FALSE);
  //QCursor *hand_cursor = new QCursor (*cb1,*cm1);

  previewFrame_ = new QFrame( this, 0 );
  previewFrame_->setFrameStyle( QFrame::WinPanel | QFrame::Sunken );
  previewFrame_->setMinimumSize( 0, 106 );

  QGridLayout *gm = new QGridLayout( previewFrame_, 2, 5, 4, 4 );
  gm->addRowSpacing( 0, 70 );
  gm->addRowSpacing( 1, 20 );

  QLineEdit *name;
  for( int i = 0; i < 5; ++i )
    {
      preView_[i] = new MEMaterialViewport( i, previewFrame_ );
      connect( preView_[i], SIGNAL(activated(int)),
	       SLOT(setActivePreview(int)) );
      //preView[i]->setCursor( *hand_cursor );
      gm->addWidget( preView_[i], 0, i );

      name = new QLineEdit( previewFrame_ );
      name->setText( "Material" );  //TODO
      gm->addWidget( name, 1, i );
    }
}

const G3DATexture*
MEMaterialEditor::getMaterial( int i ) const
{
  if( (i < 0) || (i > 4) )
    i = 0;
  return( preView_[i]->getMaterial() );
}

struct module_info*
MEMaterialEditor::get_module_info (void)
{
  // enter this module's specific information and give it back to the
  // kernel
  strcpy (m_info->module_name,"Material Editor");
  return (m_info);
}

void
MEMaterialEditor::setActivePreview( int num )
{
  if( num != active_ )
    {
      //TODO if material type different, change widget
      matWidget_->setMaterial( (preView_[num])->getMaterial() );
    }
  active_ = num;
}

void
MEMaterialEditor::setMaterial()
{
  preView_[active_]->setMaterial( matWidget_->getMaterial() );
}


//====================================END===================================
