/*
 * Project : MindsEye, material editor
 * Copyright (c) 1996 - 1998
 * Author(s): M. van Velsen, M. Kuball
 * 
 * $Id: m_matv.cpp,v 1.2 1998/06/22 20:12:42 source Exp $
 *
 * 
 * COPYRIGHT
 * =========
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 * 
 * OVERVIEW
 * ========
 *
 */

#ifdef HAVE_CONFIG_H
#include "include/config.h"
#endif	 // HAVE_CONFIG_H

#include "3dlib/Attributes/texture.h"
#include "m_matv.h"

//The quad object is created by the first invocation of the constructor
//and kept until the termination of the program.
GLUquadricObj *MEMaterialViewport::QuadObj_ = NULL;


/*---------------------------------------------------------------------------*/
MEMaterialViewport::MEMaterialViewport( int i, QWidget *parent,
					const char *name ) :
  QGLWidget(parent, name)
{
  num_      = i;
  material_ = new G3DAColor;
  glClearColor (0.0, 0.1, 0.1, 0.0);
  if( QuadObj_ == NULL )
    {
      QuadObj_ = gluNewQuadric();
      gluQuadricDrawStyle (QuadObj_, (GLenum)GLU_FILL);
      gluQuadricNormals   (QuadObj_, (GLenum)GLU_SMOOTH);
    }
}

MEMaterialViewport::~MEMaterialViewport ()
{
  delete material_;
}
/*---------------------------------------------------------------------------*/

void
MEMaterialViewport::auxSolidSphere( GLdouble radius ) const
{
  gluSphere( QuadObj_, radius, 32, 32 );
}

const G3DATexture*
MEMaterialViewport::getMaterial( void ) const
{
  return( material_ );
}

void
MEMaterialViewport::init( int width, int height )
{
  glViewport    (0, 0, width, height);
  glMatrixMode  (GL_PROJECTION);
  glLoadIdentity();

  if (width <= (height * 2))
    glOrtho ( -6.0, 6.0, 
	      -3.0*((GLfloat)height*2)/(GLfloat)width, 
	      3.0*((GLfloat)height*2)/(GLfloat)width, 
	      -10.0, 10.0);
  else
    glOrtho ( -6.0*(GLfloat)width/((GLfloat)height*2), 
	      6.0*(GLfloat)width/((GLfloat)height*2), 
	      -3.0, 3.0, 
	      -10.0, 10.0);

  glMatrixMode(GL_MODELVIEW);
}

void
MEMaterialViewport::mouseReleaseEvent( QMouseEvent* )
{
  emit activated( num_ );
}

void
MEMaterialViewport::resizeGL( int width, int height )
{ 
  init( width, height );
}

void
MEMaterialViewport::paintGL()
{
  GLfloat position[]       = { 0.0, 3.0, 2.0, 0.0 };
  GLfloat lmodel_ambient[] = { 0.4, 0.4, 0.4, 1.0 };

  glDisable( GL_DEPTH_TEST );
  glEnable( GL_CULL_FACE );
  glEnable (GL_LIGHTING);
  glEnable (GL_LIGHT0);

  glLightfv( GL_LIGHT0, GL_POSITION, position );
  glLightModelfv( GL_LIGHT_MODEL_AMBIENT, lmodel_ambient );

  glClear( GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT );
  glPushMatrix();
  glTranslatef (0.0, 0.0, 0.0);
  material_->activate();
  auxSolidSphere( 4.5 );
  glPopMatrix();
}

void
MEMaterialViewport::setMaterial( const G3DATexture *mat )
{
  delete material_;
  material_ = (G3DATexture*)(mat->copy());
  updateGL();
}


//====================================END====================================
