
#ifndef _QGL_PIXMAP_H
#define _QGL_PIXMAP_H

#include <qpixmpap.h>

/**This class provieds an off-screen OpenGL render buffer.
   no double Buffering is supported.
   */
class QGLPixmap : public QPixmap
{
    Q_OBJECT

public:
    QGLPixmap( int w, int h );

    void	makeCurrent();
    bool	isValid() const;

    const QGLFormat  &format() const;
    void	setFormat( const QGLFormat & );

    const QGLContext *context() const;
    void	setContext( QGLContext * );

public slots:
    void	updateGL();

protected:
    virtual void paintGL();
    virtual void resizeGL( int w, int h );

    void	paintEvent( QPaintEvent * );
    void	resizeEvent( QResizeEvent * );

private:
    void	gl_init( QGLContext * );
    QGLContext *glcx;

private:	// Disabled copy constructor and operator=
    QGLWidget( const QGLWidget & ) {}
    QGLWidget &operator=( const QGLWidget & ) { return *this; }
};



//
// QGLWidget inline functions
//

inline bool QGLWidget::isValid() const
{
    return glcx->isValid();
}

inline void QGLWidget::makeCurrent()
{
    glcx->makeCurrent();
}

inline const QGLFormat &QGLWidget::format() const
{
    return glcx->format();
}

inline const QGLContext *QGLWidget::context() const
{
    return glcx;
}

inline void QGLWidget::updateGL()
{
    repaint( FALSE );
}


#endif
