/*
 * Project : "MindsEye, NURBS editor"
 * Copyright 1996, 1997. All Rights Reserved.
 * Author(s): P. Lavoie, M. Kuball
 *
 * $Id: animWidget.cpp,v 1.5 1998/06/11 17:07:18 source Exp $
 *
 * COPYRIGHT
 * =========
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

//#define red qt_red
//#define black qt_black
#include <algo.h>

#include <qchkbox.h>
#include <qlined.h>
#include <qpopmenu.h>
#include <qmsgbox.h>

#include "3dlib/shape.h"
#include "3dlib/scene.h"
#include "3dlib/actions.h"
#include "3dlib/Attributes/attribute.h"
#include "3dlib/Attributes/animation.h"
#include "3dlib/Attributes/a-list.h"
#include "kernel/world_obj.h"
#include "animWidget.h"
#include "modeler.h"



class SaveValueAction : public GenericAction
{
public:
  SaveValueAction( float atime ) :
    GenericAction(A_GENERIC_ALL), time_(atime) {}
  virtual EvalState evaluate( G3DObject* );

protected:
  const float time_;
};

class SetTimeAction : public GenericAction
{
public:
  SetTimeAction( float atime ) :
    GenericAction(A_GENERIC_ALL), time_(atime) {}
  virtual EvalState evaluate( G3DObject* );

protected:
  const float time_;
};


//==============================================


AnimationWidget::AnimationWidget( MModeler *pv, QWidget *parent,
					  const char *name )
  : QFrame(parent,name)
{
  setMinimumSize( 120, 24 );
  setFrameStyle( WinPanel | Raised );
  parentView_ = pv;
  time_       = 0;

  timeField_ = new QLineEdit( this );
    timeField_->setMaxLength( 16 );
    timeField_->setText( "0.0" );
    timeField_->setGeometry( 2, 2, 40, 20 );
    //timeField_->setMinimumSize( 40, 20 );
    connect( timeField_, SIGNAL(returnPressed()), this, SLOT(setTime()) );

  keyframeToggle_ = new QCheckBox( " Keyframe", this );
    keyframeToggle_->setGeometry( 50, 2, 70, 20 );
    connect( keyframeToggle_, SIGNAL(toggled(bool)),
	     this, SLOT(setKeyframe(bool)) );

  connect( pv, SIGNAL(changed()), this, SLOT(getAnimationData()) );
  connect( pv, SIGNAL(createPopup(G3DObject*,QPopupMenu*)),
	   this, SLOT(createPopup(G3DObject*,QPopupMenu*)) );
  getAnimationData();
}

void
AnimationWidget::activate()
{
  //popup a dialog: select the attributes you wish to animate
  //animationDlg_->show( _world->getSelection() );
  //TODO add animation attribute
  MWorldObject *world;
  world = parentView_->getWorld();
  G3DObject *sel = world->getSelection();
  if( sel->getType() & T_SHAPE )
    {
      G3DAListIterator it = ((G3DShape*)sel)->getAttribute( TA_ANIMATION );
      if( *it )
	{
	  if( QMessageBox::information( 0, "MindsEye", 
"The object already has animation attributes.\nDo you want to reset them?",
					QMessageBox::Yes, QMessageBox::No )
	      == QMessageBox::No )
	    return;
	}
      ((G3DShape*)sel)->setAttribute( new G3DAnimationVis(time_) );
    }
}

void
AnimationWidget::createPopup( G3DObject *obj, QPopupMenu *menu )
{
 //TODO animation of/off
  if( obj->getType() & T_SHAPE )
    menu->insertItem( "animate", this, SLOT(activate()) );
}

void
AnimationWidget::getAnimationData()
{
  MWorldObject *world;
  world = parentView_->getWorld();
  //if( world )
  //TODO get the Animation Context of the scene
}

void
AnimationWidget::setKeyframe( bool flag )
{
  if( flag == TRUE )
    {
      keyList_.insert( time_ );
    }
  else
    {
      keyList_.erase( time_ );
      //TODO remove this time frame in all Attributes?
    }
}


EvalState
SaveValueAction::evaluate( G3DObject *obj )
{
  if( obj->getType() & T_SHAPE )
    {
      G3DShape         *shape = (G3DShape*)obj;
      G3DAListIterator attr   = shape->getAttribute( TA_ANIMATION );
      while( *attr != NULL )
	{
	  ((G3DAnimation*)(*attr))->saveValueAt( shape, time_ );
	  ++attr;
	}
    }
  return( continueEval );
}

EvalState
SetTimeAction::evaluate( G3DObject *obj )
{
  if( obj->getType() & T_SHAPE )
    {
      G3DShape         *shape = (G3DShape*)obj;
      G3DAListIterator attr   = shape->getAttribute( TA_ANIMATION );
      while( *attr )
	{
	  ((G3DAnimation*)*attr)->evaluateAt( shape, time_ );
	  ++attr;
	}
    }
  return( continueEval );
}

void
AnimationWidget::setTime( )
{
  float t;
  t = atof( timeField_->text() );

  MWorldObject *world;
  world = parentView_->getWorld();
  if( ::find(keyList_.begin(),keyList_.end(),time_) != keyList_.end() )
    {
      SaveValueAction action1( time_ );
      world->doWithScene( action1 );
    }

  SetTimeAction action2( t );
  world->doWithScene( action2 );
 
  //TODO don't forget the camera!
  time_ = t;
  if( ::find(keyList_.begin(),keyList_.end(),time_) != keyList_.end() )
    keyframeToggle_->setChecked( TRUE );
  else
    keyframeToggle_->setChecked( FALSE );

  parentView_->updateView();
}

//===================================END=================================
