/*
 * Project : "MindsEye, modeler"
 * Copyright 1996, 1997. All Rights Reserved.
 * Author(s): P. Lavoie, M. Kuball
 *
 * $Id: materialTool.cpp,v 1.1 1998/06/11 17:07:45 source Exp $
 *
 * COPYRIGHT
 * =========
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

#include "include/config.h"

#include <qpushbt.h>
#include <qbttngrp.h>

#include "kernel/m_core.h"
#include "kernel/world_obj.h"
#include "material/m_mat.h"
#include "3dlib/shape.h"
#include "3dlib/selectGroup.h"
#include "3dlib/Attributes/texture.h"
#include "modeler.h"
#include "materialTool.h"


MaterialToolWidget::MaterialToolWidget( MModeler *modeler, QWidget *parent,
					const char *name )
  : QFrame(parent,name)
{
  //TODO connect( pv, SIGNAL(createPopup(G3DObject*,QPopupMenu*)),
  //	   this, SLOT(createPopup(G3DObject*,QPopupMenu*)) );

  setMinimumSize( 102, 24 );
  setFrameStyle( WinPanel | Raised );
  parentView_ = modeler;

  QButtonGroup *bgroup = new QButtonGroup;
  connect( bgroup, SIGNAL(clicked(int)), this, SLOT(setMaterial(int)) );

  QPushButton *btmp;
  btmp = new QPushButton( "m1", this );
    btmp->setGeometry( 2, 2, 20, 20 );
    bgroup->insert( btmp );
  btmp = new QPushButton( "m2", this );
    btmp->setGeometry( 22, 2, 20, 20 );
    bgroup->insert( btmp );
  btmp = new QPushButton( "m3", this );
    btmp->setGeometry( 42, 2, 20, 20 );
    bgroup->insert( btmp );
  btmp = new QPushButton( "m4", this );
    btmp->setGeometry( 62, 2, 20, 20 );
    bgroup->insert( btmp );
  btmp = new QPushButton( "m5", this );
    btmp->setGeometry( 82, 2, 20, 20 );
    bgroup->insert( btmp );
}


void
MaterialToolWidget::setMaterial( int nr )
{
  MWorldObject *world = parentView_->getWorld();
  G3DObject    *sel   = world->getSelection();
  MEMaterialEditor *matedit =
    (MEMaterialEditor*)Kernel->getModule( MODULE_MATERIAL );

  if( sel && matedit )
    {
      G3DATexture *mat = (G3DATexture*)((matedit->getMaterial(nr))->copy());
      
      if( sel->getType() & T_SHAPE )
	{
	  ((G3DShape*)sel)->setAttribute( mat );
	  world->update( NULL );
	}
      else if( sel->getType() & T_SEL_GROUP )
	{
	  ((G3DSelectionGroup*)sel)->setAttribute( mat );
	  world->update( NULL );
	}
    }
}


//===================================END=================================
