/*
 * Project : "MindsEye"
 * Copyright (c) 1996, 1997. All Rights Reserved.
 * Author(s) M. van Velsen, M. Kuball
 *
 * $Id: modeler.cpp,v 1.3 1998/07/23 20:16:19 source Exp $
 *
 *
 * COPYRIGHT
 * =========
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 * 
 * OVERVIEW
 * ========
 *
 */

#include "include/config.h"

#include <qmenubar.h>
#include <qpopmenu.h>
#include <qkeycode.h>

#include "include/m_messg.h"
#include "vlayout/m_viewm.h"
#include "vlayout/m_viewl.h"
#include "vlayout/tcontroler.h"
#include "vlayout/grid.h"
#include "vlayout/m_vmenu.h"
#include "kernel/world_obj.h"
#include "kernel/m_core.h"
#include "kernel/prop_dlg.h"
#include "3dlib/s_group.h"
#include "3dlib/t_actions.h"
#include "3dlib/w_rotate.h"
#include "3dlib/w_scale.h"
#include "3dlib/w_coord.h"
#include "nurbsedit/toolbox.h"
#include "nurbsedit/infoWidget.h"
#include "dialogs/grid/GridDialog.h"
#include "dialogs/object/coneInfo.h"
#include "dialogs/object/torusInfo.h"
#include "dialogs/output/renderDlg.h"
#include "dialogs/light/light_info.h"
#include "modeler.h"
#include "editTools.h"
#include "commHistView.h"
#include "objectList.h"
#include "animWidget.h"
#include "materialTool.h"


class ShowLightsAction : public GenericAction
{
public:
  ShowLightsAction( int a ) :
    GenericAction(A_GENERIC_LIGHTS), action(a) {}
  virtual EvalState evaluate( G3DObject* );
protected:
  ActionData action;
};

EvalState
ShowLightsAction::evaluate( G3DObject *l )
{
  l->doAction( action );
  return( continueEval );
}


/*---------------------------------------------------------------------------*/
MModeler::MModeler( int module_id, int load_t ) :
  MEModule(module_id, load_t, "MModeler")
{
  setCaption ("MindsEye Modeler");
  setMaximumSize( 4000, 4000 );
  setMinimumSize( 480,400 );

  _editFrame  = new MViewPortLayout( this );
  _editFrame->setModel( this );
  _caching    = DirectMode;
  _angleSnap  = _gridSnap = 0;
  _newObject  = NULL;
  world_      = NULL;
  showLights_ = false;

  QMenuBar *menu = makeMenuBar();

  // ------------------------------------------------------

  QBoxLayout *gm = new QVBoxLayout( this, 1, 1 );
  gm->setMenuBar( menu );
  QBoxLayout *gm_01 = new QHBoxLayout();
  gm->addLayout( gm_01, 100 );

  gm_01->addWidget( _editFrame, 100 );
  QBoxLayout *gm_02 = new QVBoxLayout();
  gm_01->addLayout( gm_02, 1 );

  EditToolsWidget *ebox = new EditToolsWidget( this, this, NULL );
    gm_02->addWidget( ebox, 1 );
  MaterialToolWidget *mbox = new MaterialToolWidget( this, this, NULL );
    gm_02->addWidget( mbox, 1 );
  ObjectListWidget *obox = new ObjectListWidget( this, this, NULL );
  //connect( tbox, SIGNAL(newObject(G3DObject*)), SLOT() );
    gm_02->addWidget( obox, 1 );
  CommandHistoryView *cbox = new CommandHistoryView( this, this, NULL );
    gm_02->addWidget( cbox, 1 );
  ToolboxWidget *nurbsbox = new ToolboxWidget( this, this, NULL );
    gm_02->addWidget( nurbsbox, 1 );
  AnimationWidget *animw = new AnimationWidget( this, this, NULL );
    gm_02->addWidget( animw, 1 );

  gm_02->addStretch( 100 );

  _mgrBar = new MViewPortManager( this, _editFrame, this );
  gm->addWidget( _mgrBar, 0 );

  gm->activate ();

  // ------------------------------------------------------
//TODO this should go into the kernel
  _curveProperties   = new InfoCurveWidget;
  _coneProperties    = new InfoConeWidget;
  _defaultProperties = new ObjectProperties( NULL, NULL );
  _torusProperties   = new InfoTorusWidget;

  //create the default controler
  _rControler = new RotateControler;
  connect( _rControler, SIGNAL(setInfoText(const char*)),
	   _mgrBar, SLOT(setCoordinateText(const char*)) );  //TODO
  _sControler = new ScaleControler;
  connect( _sControler, SIGNAL(setInfoText(const char*)),
	   _mgrBar, SLOT(setCoordinateText(const char*)) );
  _mControler = new MoveControler;
  connect( _mControler, SIGNAL(setInfoText(const char*)),
	   _mgrBar, SLOT(setCoordinateText(const char*)) );
  _spaceGrid  = new SpaceGrid;
  _angleGrid  = new AngleGrid;
  _gridDialog = new GridDialog;

  G3DObject::setSelectColor( new G3DColor(1, 0, 0) );
  majorMode_ = editing;
  minorMode_ = scaling;
  renderDlg_ = new MERenderDialog;

  connect( Kernel, SIGNAL(worldListChanged()), SLOT(setWorldList()) );
  sceneMenuCheck_ = 0;
  setWorldList();
}

MModeler::~MModeler ()
{
}

/*---------------------------------------------------------------------------*/

void
MModeler::applyTrafo( )
{
  if( world_->getSelection() )
    (world_->getSelection())->applyTransform();
}

void
MModeler::closeIt()
{
  close();  //TODO or hide?
}

void
MModeler::decorateObject( G3DObject *obj )
{
  if( obj->getType() & T_SHAPE )
    {
      G3DShape *sh = (G3DShape*)obj;
      if( minorMode_ == scaling )
	{
	  sh->addWidget( new G3DScaleWidget(sh) );
	  sh->addWidget( new G3DCoordinateWidget(sh) );
	}
      else if( minorMode_ == rotating )
	{
	  sh->addWidget( new G3DRotateWidget(sh) );
	  sh->addWidget( new G3DCoordinateWidget(sh) );
	}
    }
}

void
MModeler::deleteObject()
{
  world_->deleteObject( NULL );  //delete selected object
}

void
MModeler::exportObjects()
{
}

TransformControler*
MModeler::getControlerFor( G3DObject *obj )
{
  TransformControler *ctr = NULL;

  if( obj->getType() & T_SHAPE )
    {
      //if( minorMode_ == scaling )
      ctr =  _mControler;
      //else if( minorMode_ == rotating )
	//ctr = _rControler;
    }
  else if( obj->getType() & T_WIDGET )
    {
      if( obj->getType() == T_SCALE_WIDGET )
	ctr = _sControler;
      else if( obj->getType() == T_ROTATE_WIDGET )
        ctr = _rControler;
    }
  else if( obj->getType() & T_HANDLE )
    ctr = _mControler;
  else //if( obj->getType() & T_LIGHT )
    ctr = _mControler;

  return( ctr );
}

struct module_info*
MModeler::get_module_info( void )
{
 // enter this module's specific information and give it back to the
 // kernel
 strcpy( m_info->module_name, "Modeler module" );
 return( m_info );
}

G3DObject*
MModeler::getNewObject( void )
{
  if( (majorMode_ == creating) && (_newObject != NULL) )
    {
      G3DObject *tmp;
      tmp = _newObject;
      _newObject = tmp->copy();
      world_->addObject( tmp );
      return( tmp );
    }
  else
    return( NULL );
}

QPopupMenu*
MModeler::getPopupMenuFor( G3DObject *obj )
{
  QPopupMenu *pop = new MObjectPopup( obj, this );
  emit createPopup( obj, pop );
  return( pop );
}


void
MModeler::gridSettings()
{
  _gridDialog->setGrid( (SpaceGrid*)_spaceGrid );
  _gridDialog->show();
}


void
MModeler::hideObject()
{
  ActionData action(A_HIDE);
  world_->doWithSelection( action );
}

void
MModeler::importObjects()
{
  message msg;

  G3DGroup *group = new G3DGroup;
  world_->addShape( group );
  msg.message_id = K_MESS_IMPORTOBJ;
  msg.contents   = (char*)group;
  Kernel->process_message( &msg );
}

void
MModeler::loadScene()
{
  message msg;

  msg.message_id = K_MESS_LOADSCENE;
  Kernel->process_message( &msg );
}

QMenuBar*
MModeler::makeMenuBar( void )
{
  QPopupMenu *file = new QPopupMenu;
  CHECK_PTR( file );
  file->insertItem( "new",  this, SLOT(newScene()), CTRL+Key_N );
  file->insertItem( "load", this, SLOT(loadScene()), CTRL+Key_L );
  file->insertItem( "save", this, SLOT(saveScene()), CTRL+Key_S );
  file->insertSeparator();
  file->insertItem( "import", this, SLOT(importObjects()), CTRL+Key_I );
  file->insertItem( "save Selection", this,SLOT(exportObjects()), CTRL+Key_E );
  file->insertSeparator();
  file->insertItem( "close", this, SLOT(closeIt()), CTRL+Key_C );

  QPopupMenu *edit = new QPopupMenu;
  CHECK_PTR( edit );
  edit->insertItem( "undo",  this, SLOT(newScene()), CTRL+Key_U );
  edit->insertItem( "redo", this, SLOT(newScene()), CTRL+Key_R );
  edit->insertSeparator();
  edit->insertItem( "delete", this, SLOT(deleteObject()), CTRL+Key_D );
  edit->insertSeparator();
  //show a dialog to edit the scene propierties like background, etc.
  edit->insertItem( "scene properties", this, SLOT(sceneProperties()) );

  menuDisplay_ = new QPopupMenu;
  CHECK_PTR( menuDisplay_ );
  menuDisplay_->insertItem( "Wireframe", this, SLOT(setViewMode1()) );
  menuDisplay_->insertItem( "Flat Shading", this, SLOT(setViewMode2()) );
  menuDisplay_->insertItem( "Smooth Shading", this, SLOT(setViewMode3()) );
  menuDisplay_->insertItem( "Texturing", this, SLOT(setViewMode4()) );
  menuDisplay_->insertSeparator();
  menuDisplay_->insertItem( "caching", this, SLOT(toggleCaching()) );
  menuDisplay_->insertSeparator();
  menuLtg_ =
    menuDisplay_->insertItem( "show light sources", this, SLOT(showLights()) );
  menuDisplay_->insertSeparator();
  menuDisplay_->insertItem( "grid settings...", this, SLOT(gridSettings()) );

  QPopupMenu *render = new QPopupMenu;
  CHECK_PTR( render );
  render->insertItem( "render...", this, SLOT(renderOptions()) );
  render->insertItem( "render current", this, SLOT(renderCurrent()) );

  worldsMenu_ = new QPopupMenu;
  connect( worldsMenu_, SIGNAL(activated(int)), SLOT(showWorld(int)) );

  QMenuBar *menu = new QMenuBar( this );
  menu->setFrameStyle( QFrame::WinPanel | QFrame::Raised );
  CHECK_PTR( menu );
  menu->insertItem( "&File", file );
  menu->insertItem( "&Edit", edit );
  menu->insertItem( "&Options", menuDisplay_ );
  menu->insertItem( "&Rendering", render );
  menu->insertItem( "&Scenes", worldsMenu_ );

  return( menu );
}

void
MModeler::newScene()
{
  message msg;

  msg.message_id = K_MESS_NEWSCENE;
  Kernel->process_message( &msg );
}

void
MModeler::objProperties()
{
  G3DObject *obj;

  obj = world_->getSelection();
  //TODO _kernel->showPropertyDialog( obj );
  if( obj )
    {
      if( obj->getType() == T_NURBS_CURVE )
	_curveProperties->setObject( obj );
      else if( obj->getType() == T_CONE )
        _coneProperties->setObject( obj );
      else if( obj->getType() == T_TORUS )
        _torusProperties->setObject( obj );
      else if( obj->getType() == T_POINT_LIGHT )
	{
	  MEPointLightInfo *tmp = new MEPointLightInfo( obj );
	  tmp->show();
	}
      else if( !(obj->getType() & T_SEL_GROUP) )
	{
	  _defaultProperties->setObject( obj );
	  _defaultProperties->show();
	}
      //TODO else give information about sel-group??
    }
}

void
MModeler::pmChildAccess()
{ //TODO use dynamic_cast??
  world_->setActiveGroup( (G3DGroup*)(world_->getSelection()) );
  ActionData action(A_TOGGLE_CHILD_ACCESS);
  world_->doWithSelection( action );
}

void
MModeler::pmGroup()
{
  world_->groupSelectedObjects();
}

void
MModeler::pmSelectParent()
{
  G3DObject *tmp = world_->getSelection();
  if( tmp )
    world_->selectObject( tmp->getParent() );
}

void
MModeler::pmUngroup()
{
  //world_->groupSelectedObjects();
}


void
MModeler::renderOptions()
{
  world_->setActiveCamera( _editFrame->getActiveCamera() );
  renderDlg_->setWorld( world_ );
  renderDlg_->show(); //TODO scene->copy()
}

void
MModeler::renderCurrent()
{
  world_->setActiveCamera( _editFrame->getActiveCamera() );
  renderDlg_->renderCurrent( world_ );
}

void
MModeler::saveScene()
{
  message msg;

  msg.message_id = K_MESS_SAVESCENE;
  Kernel->process_message( &msg );
}

void
MModeler::sceneProperties()
{
  ScenePropertiesDlg *dlg = new ScenePropertiesDlg( world_->getScene() );
  dlg->show();
}

void
MModeler::setMajorMode( EditMode mode )
{
  majorMode_ = mode;
}

void
MModeler::setMinorMode( EditMode mode )
{
  minorMode_ = mode;
}

void
MModeler::setNewObject( G3DObject *obj )
{
  delete _newObject;
  _newObject = obj;
}

void
MModeler::setWorldList()
{
  MWorldList &worlds = Kernel->getWorldList();

  worldsMenu_->clear();
  bool gotIt = false;
  MWorldList::iterator wld = worlds.begin();
  while( wld != worlds.end() )
    {
      if( *wld == world_ )
	gotIt = true;
      worldsMenu_->insertItem( (*(wld++))->getName() );
    }
  if( !gotIt )
    showWorld( 0 );
}

void
MModeler::showLights()
{
  if( showLights_ )
    {
      ShowLightsAction iter( A_REM_SHAPE );
      world_->doWithScene( iter );
      menuDisplay_->changeItem ( "Show light sources", menuLtg_ );
    }
  else
    {
      ShowLightsAction iter( A_ADD_SHAPE );
      world_->doWithScene( iter );
      menuDisplay_->changeItem ( "Hide light sources", menuLtg_ );
      showLights_ = true;
    }
}

void
MModeler::showObject()
{
  ActionData action(A_SHOW);
  world_->doWithSelection( action );
}

void
MModeler::showWorld( int id )
{
  if( world_ )
    disconnect( world_ );

  worldsMenu_->setItemChecked( sceneMenuCheck_, FALSE );
  sceneMenuCheck_ = id;
  worldsMenu_->setItemChecked( sceneMenuCheck_, TRUE );

  MWorldList &worlds = Kernel->getWorldList();
  //TODO if
  world_ = worlds[id];
  if( world_ )
    {
      setCaption( world_->getName() );
      connect( world_, SIGNAL(decorateObject(G3DObject*)),
	       SLOT(decorateObject(G3DObject*)) );
      emit changed();  //TODO ???
    }
}

void
MModeler::takeFocus()
{
  G3DObject *obj = world_->getSelection();
  if( obj )
    {
      G3DPoint3d pos;
      obj->getPosition( pos );
      (_editFrame->getActiveCamera())->setLookAt( pos );
    }
}

void
MModeler::toggleAngleSnap( void )
{
  if( _angleSnap )
    {
      _rControler->setGrid( NULL );
      _angleSnap = 0;
    }
  else
    {
      _rControler->setGrid( _angleGrid );
      _angleSnap =1;
    }
}

void
MModeler::toggleCaching()
{
  if( _caching == DirectMode )
    _caching = DisplayListMode;
  else
    _caching = DirectMode;
  _editFrame->setCaching( _caching );
}

void
MModeler::toggleGridSnap( void )
{
  if( _gridSnap )
    {
      _mControler->setGrid( NULL );
      _gridSnap = 0;
    }
  else
    {
      _mControler->setGrid( _spaceGrid );
      _gridSnap =1;
    }
}

void
MModeler::setViewMode1()
{
}

void
MModeler::setViewMode2()
{
}

void
MModeler::setViewMode3()
{
}

void
MModeler::setViewMode4()
{
}

void
MModeler::updateView( void )
{
  _editFrame->viewChanged();
}


//======================================END============================
