/*
 * Project : "MindsEye"
 * Copyright (c) 1996, 1997. All Rights Reserved.
 * Author(s) M. van Velsen, M. Kuball
 *
 * $Id: modeler.h,v 1.3 1998/07/23 20:16:19 source Exp $
 * 
 * 
 * COPYRIGHT
 * =========
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 * 
 * OVERVIEW
 * ========
 *
 */


#ifndef __MMODELER_H__
#define __MMODELER_H__

#include <qwidget.h>
#include "kernel/m_module.h"

class MViewPortManager;
class MViewPortLayout;
class MWorldObject;
class ToolboxWidget;
class InfoCurveWidget;
class InfoConeWidget;
class InfoTorusWidget;
class ObjectProperties;
class QMenuBar;
class QPopupMenu;
class G3DObject;
class TransformControler;
class MoveControler;
class RotateControler;
class ScaleControler;
class GridDialog;
class Grid;
class MERenderDialog;


/**A window featuring all the necessary components for 3d modelling.
 */
class MModeler : public MEModule
{
  Q_OBJECT

public:
  enum EditMode { creating, editing, scaling, rotating, orbiting };

public:
    ///
  MModeler( int module_id, int load_t );
  ~MModeler();

    ///
  struct module_info* get_module_info( void );
    /**Get a controler depending on the object's type and the current
       modeling mode.*/
  TransformControler* getControlerFor( G3DObject *obj );
    /**Add _newObject to the scene graph.*/
  G3DObject*    getNewObject( void );
    ///
  QPopupMenu*   getPopupMenuFor( G3DObject* );
    ///
  MWorldObject* getWorld( void ) { return( world_ ); };
    ///
  void   setMajorMode( EditMode );
    ///
  void   setMinorMode( EditMode );
    ///
  void   updateView();

signals:
  void changed();
  void createPopup( G3DObject*, QPopupMenu* );

public slots:
  void applyTrafo();
  void decorateObject(G3DObject*);
    ///
  void deleteObject();
  void hideObject();
  void objProperties();
  void pmChildAccess();
  void pmGroup();
  void pmSelectParent();
  void pmUngroup();
    /**A copy of obj will be added to the scene when the left mouse button
       is pressed. If obj is NULL the obj under the mouse is selected.*/
  void setNewObject( G3DObject* );
  void showObject();
  void takeFocus();

protected:
  QMenuBar* makeMenuBar( void );

protected slots:
    ///
  void closeIt();
    ///Save the selected Objects.
  void exportObjects();
    ///Show a dialog to set the grid parameter.
  void gridSettings();
    /**Load objects from a file and add them to the scene. The new objects
       will be added as a subgroup to the scene. */
  void importObjects();
    ///Delegate this to the kernel
  void loadScene();
    ///Replace the current scene with a new empty one.
  void newScene();
    ///
  void renderOptions();
    ///
  void renderCurrent();
    ///Delegate this to the kernel.
  void saveScene();
  void sceneProperties();
  void setViewMode1();
  void setViewMode2();
  void setViewMode3();
  void setViewMode4();
  void setWorldList();
  void showLights();
  void showWorld(int);
    ///
  void toggleAngleSnap();
    ///
  void toggleCaching();
    ///
  void toggleGridSnap();

private:
  MViewPortLayout  *_editFrame;
  MViewPortManager *_mgrBar;
  MWorldObject     *world_;
  short            _caching;
  bool             showLights_;

  G3DObject        *_newObject;

  InfoCurveWidget    *_curveProperties;
  InfoConeWidget     *_coneProperties;
  InfoTorusWidget    *_torusProperties;
  ObjectProperties   *_defaultProperties;

  GridDialog         *_gridDialog;
  MERenderDialog     *renderDlg_;
  QPopupMenu         *menuDisplay_;
  QPopupMenu         *worldsMenu_;
  int                menuLtg_, sceneMenuCheck_;

  MoveControler      *_mControler;
  RotateControler    *_rControler;
  ScaleControler     *_sControler;

  EditMode  majorMode_, minorMode_;
  short     _angleSnap, _gridSnap;
  Grid      *_angleGrid, *_spaceGrid;

};


#endif // __MMODELER_H__
