/*
 * Project : "MindsEye, NURBS editor"
 * Copyright 1996, 1997. All Rights Reserved.
 * Author(s): P. Lavoie, M. Kuball
 *
 * $Id: objectList.cpp,v 1.5 1998/07/23 20:16:19 source Exp $
 *
 * COPYRIGHT
 * =========
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

#include <qpushbt.h>
#include <qbttngrp.h>
#include <qcombo.h>
#include <qlistbox.h>

#include "3dlib/s_simple.h"
#include "3dlib/s_torus.h"
#include "3dlib/s_glshapes.h"
#include "3dlib/s_trigrid.h"
#include "3dlib/light.h"
#include "nurbsedit/shaper.h"
#include "kernel/m_core.h"
#include "objectList.h"
#include "modeler.h"


//TODO get from Kernel
/*
const char *SimpleList[] =
{ "Box", "Cone", "Cylinder", "Sphere", "Torus",
  "Trigrid", NULL };

const char *NurbsList[] =
{ "NURBS_Curve", "Circle",
  "Plane", "Sphere",
   NULL };

const char *LightList[] =
{ "Pointlight", "DirectionalLight", "Spotlight", NULL };
*/
 

ObjectListWidget::ObjectListWidget( MModeler *model, QWidget *parent,
			      const char *name )
  : QFrame(parent,name)
{
  setMinimumSize( 130, 100 );
  setFrameStyle( WinPanel | Raised );

  _model = model;

  const char **objClassList;
  objClassList = Kernel->getObjClassList();
  _objClasses  = new QComboBox( this );
    _objClasses->setGeometry( 2, 2, 126, 22 );
    _objClasses->insertStrList( objClassList );
    connect( _objClasses, SIGNAL(activated(int)), SLOT(useObjClass(int)) );
  delete[] objClassList;

  _objTypes = new QListBox( this );
    _objTypes->setGeometry( 2, 28, 126, 70 );
    connect( _objTypes, SIGNAL(highlighted(int)), SLOT(useObjType(int)) );

  useObjClass( 0 );
}


void
ObjectListWidget::useObjClass( int nr )
{
  _objTypes->setAutoUpdate( FALSE );
  _objTypes->clear();

  cname_ = _objClasses->text( nr );
  const char **list = Kernel->getObjList( cname_ );
  _objTypes->insertStrList( list );
  delete[] list;
  /*
  if( nr == 0 )
    _objTypes->insertStrList( SimpleList );
  else if( nr == 1 )
    _objTypes->insertStrList( NurbsList );
  else if( nr == 4 )
    _objTypes->insertStrList( LightList );
    */

  _objTypes->setAutoUpdate( TRUE );
  _objTypes->repaint();
}

void
ObjectListWidget::useObjType( int nr )
{
  //_typeNum = nr;
  G3DObject *obj = NULL;

  const char *tname = Kernel->getObjTypeName( cname_, _objTypes->text(nr) );
  if( tname != NULL )
    obj = Kernel->createObject( tname );

  _model->setNewObject( obj );
}


//===================================END=================================
