/*
 * Project : "MindsEye, 3D library"
 * Copyright (c) 1995 - 1997. All Rights Reserved.
 * Author(s): P. Lavoie, M. Kuball
 *
 * $Id: InterpWidget.cpp,v 1.8 1998/06/22 20:12:44 source Exp $
 *
 * COPYRIGHT
 * =========
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *
 * OVERVIEW
 * ========
 *
 */

#include <qpainter.h>
#include <qpixmap.h>
#include <qlabel.h>
#include <qbttngrp.h>
#include <qradiobt.h>
#include <qpushbt.h>
#include <qlined.h>
#include <qpopmenu.h>

#include "InterpWidget.h"
#include "3dlib/math/tools.h"


void
MELineObject::draw( QPainter *p, float scale )
{
  if( pl_.size() > 1 )
    {
      int x, y, x_last, y_last;
      QPen pen( black );

      p->setPen( pen );
      x_last = int( pl_[0].x * scale );
      y_last = -int( pl_[0].y * scale );
      for( int i = 1; i < pl_.size(); ++i )
	{
	  x = int( pl_[i].x * scale );
	  y = -int( pl_[i].y * scale );
	  //p->drawPoint( x, y );
	  //p->setPen( *pen2 );
	  p->drawLine( x_last, y_last, x, y );
	  x_last = x;  y_last = y;
	}
    }

  if( curve_ )
    {
      //QColor        color;
      //color.setRgb( 255-20, 20, 0 );
      QPen pen( red );
      p->setPen( pen );

      Point3D p1 = curve_->point3D( 0 );
      for( int j = 1; j <= 64; ++j )
	{
	  Point3D p2 = curve_->point3D(float(j)/float(64));
	  p->drawLine( int(p1.x*scale), -int(p1.y*scale),
		       int(p2.x*scale), -int(p2.y*scale) );
	  p1 = p2;
	}
    }
}


//-------------------------------------------------------------------

MEDrawingWidget::MEDrawingWidget( QWidget *parent, const char *name) :
  QFrame(parent,name)
{
  //setMinimumSize(100,100) ;
  setFrameStyle( WinPanel | Sunken );
  setLineWidth( 2 );

  //preview = new PlNurbsListGL ;
  stepx_     = stepy_ = 0.04;
  scale_     = 200;
  editMode_  = EditCurve;
  gridType_  = Cartesian;
  mousePt_.z = 0;
  active_    = NULL;

  clearCanvas();
  makePopup();
}

MEDrawingWidget::~MEDrawingWidget()
{
}


void
MEDrawingWidget::addCurve( G3DNurbsCurve *c )
{
  MELineObject *obj = new MELineObject( c );
  objects_.insert( objects_.begin(), obj );

  selected_ = -1;
  mod_curve = 0;
  repaint() ;
}

void
MEDrawingWidget::clearCanvas()  //TODO removeObjects
{
  objects_.erase( objects_.begin(), objects_.end() );

  selected_ = -1;
  mod_curve = 0;
  repaint();
}

void
MEDrawingWidget::computeCurve()
{
  if( active_ )
    emit computeCurve( active_ );
  repaint();
}

void
MEDrawingWidget::deleteCurve()
{
  if( active_ )
    active_->deleteCurve();  //TODO if no polyline, delete line object
  repaint();
}

void
MEDrawingWidget::drawContents( QPainter *p )
{
  //QPen     pen1(black);
  //QPen     pen2(green);
  //pen2.setWidth( 3 );

  p->translate( offset_.x(), offset_.y() );

  //drawGrid( p );

  for( int i = 0; i < objects_.size(); ++i )
    {
      MELineObject *obj = objects_[i];
      obj->draw( p, scale_ );
    }
}

void
MEDrawingWidget::drawCrossWire( QPainter &p, int x, int y )
{
  if( gridType_ == Cartesian )
    {
      //p.setPen( DotLine );
      p.drawLine( -width()/2, y, width()/2, y );
      p.drawLine( x, height()/2, x, -height()/2 );
      //p.setPen( SolidLine );
    }
  else if( gridType_ == Polar )
    {
      int r = int( sqrt((x*x) + (y*y)) );
      p.drawEllipse( -r, -r, 2*r, 2*r );
      //alte lschen
      //p->drawLine();
      //p->drawCircle();
    }
}

void
MEDrawingWidget::drawGrid( QPainter *p )
{
//wie macht qtarch das?
  QRect rect = contentsRect();
  int   l, r, t, b;

  l = rect.left() + 10;
  r = rect.right();
  t = rect.top();
  b = rect.bottom();
  /*
  while( l < r )
    {
      p->drawLine( l, t, l, b );
      l += 10;
    }
    */
}

G3DObject*
MEDrawingWidget::getObject( void )
{
  if( active_ )
    return( active_->curve()->copy() );
  else
    return NULL;
}

void
MEDrawingWidget::makePopup()
{
  popup_ = new QPopupMenu;
  popup_->insertItem( "compute curve", this, SLOT(computeCurve()) );
  popup_->insertItem( "delete curve",  this, SLOT(deleteCurve()) );
}

void
MEDrawingWidget::mouseMoveEvent( QMouseEvent * mouse )
{
  float   x   = float( mouse->x() - offset_.x() ) / scale_ ;
  float   y   = float(offset_.y() - mouse->y()) / scale_ ;
  round( x, y );
  if( editMode_ == AppendPoint )  //there is at least 1 point
    {
      if( (mousePt_.x != x) || (mousePt_.y != y) )
	{
	  Point3D endpt;

	  active_->getEndPoint( endpt );
	  QPainter p( this );
	  p.translate( offset_.x(), offset_.y() );
	  p.setRasterOp( XorROP );
	  p.setPen( cyan );
	  //restore pixels under the old line
	  int mx =  int(mousePt_.x * scale_);
	  int my = -int(mousePt_.y * scale_);
	  drawCrossWire( p, mx, my );
	  p.drawLine( int(endpt.x*scale_), -int(endpt.y*scale_), mx, my );

	  mousePt_.x = x;
	  mousePt_.y = y;
	  //draw new line
	  mx =  int(mousePt_.x * scale_);
	  my = -int(mousePt_.y * scale_);
	  p.drawLine( int(endpt.x*scale_), -int(endpt.y*scale_), mx, my );
	  drawCrossWire( p, mx, my );
	}
    }
    /*
else if( editMode_ == EditCurve )
{
  if( selected )
}
else if( editMode_ == FreeHand ) -> stop nach releaseEvent
  if( mouse->state() == LeftButton )
    {
      Point3D &p1 = points_[points_.size()-1];
      float   x   = float(mouse->x())/width() ;
      float   y   = float(height()-mouse->y())/height() ;

      if( (p1.x == x) && (p1.y == y) )
	return ;

      Point3D *p = new Point3D;
      p->x = x ;
      p->y = y ;
      p->z = 0 ;
      
      points_.append( p );
      emit newPoint3D( p );
      repaint() ;
    }
  else if( mouse->state() == MidButton )
    {
      if( selected_ >= 0)
	{
	  float x = float(mouse->x())/width();
	  float y = float(height() - mouse->y())/height();
	  float dx = x - mouseX_;
	  float dy = y - mouseY_;
	  mouseX_ = x ;
	  mouseY_ = y ;
	  //PlNurbsCurveGL* c ;
	  //c = (PlNurbsCurveGL*)preview->last ;
	  if( ncurve_ )
	    {
	      CtrlPnt4D p ;      
	      p.x() = x ;
	      p.y() = y ;
	      p.z() = 0 ;
	      p.w() = 1 ;
	      ncurve_->modCP( selected_, p );
	      repaint() ;
	      mod_curve = 1;
	    }
	}
    }
    */
}

void
MEDrawingWidget::mousePressEvent( QMouseEvent * mouse )
{
  if( mouse->button() == LeftButton )
    {
      if( editMode_ == AddPolyline )
        {
	  float x = float( mouse->x() - offset_.x() ) / scale_;
	  float y = float( offset_.y() - mouse->y() ) / scale_;
	  round( x, y );

	  Point3D endpt( x, y, 0 );
	  active_->appendPoint( endpt );
	  mousePt_.x = x;
	  mousePt_.y = y;

	  editMode_ = AppendPoint;

	  QPainter p( this );
	  p.setPen( cyan );
	  p.translate( offset_.x(), offset_.y() );
	  p.setRasterOp( XorROP );
	  drawCrossWire( p, int(x*scale_), -int(y*scale_) );
	}
      else if( editMode_ == AppendPoint )
	{
	  Point3D endpt;
	  active_->getEndPoint( endpt );
	  if( (mousePt_.x != endpt.x) || (mousePt_.y != endpt.y) )
	    {
	      QPainter p( this );
	      p.setRasterOp( XorROP );
	      p.translate( offset_.x(), offset_.y() );
	      int x = int(mousePt_.x*scale_);
	      int y = -int(mousePt_.y*scale_);
	      //drawCrossWire( p, x, y );
	      p.setRasterOp( CopyROP );
	      p.drawLine( int(endpt.x*scale_), -int(endpt.y*scale_), x, y );
	      active_->appendPoint( mousePt_ );
	    }
	}
    }
  else if( mouse->button() == MidButton )
    {
      if( editMode_ == AppendPoint )
	{
	  //TODO if points.size < 2 delete
	  editMode_ = EditCurve;
	  setMouseTracking( FALSE );
	  repaint();
	}
      else if( editMode_ == EditCurve )
	{
	  //select a control point
	  float x = float(mouse->x()) / scale_;
	  float y = float(height()-mouse->y()) / scale_;
	}
	/*
      mouseX_ = x ;
      mouseY_ = y ;

      Point3D p ;
      p.x = x ;
      p.y = y ;
      p.z = 0 ;

      float dx = float(1)/float(width()) ;
      float dy = float(1)/float(height()) ;

      dx *= 3.0 ;
      dy *= 3.0 ;

      //PlNurbsCurveGL* c ;
      //c = (PlNurbsCurveGL*)preview->last ;
      selected_ = -1 ;
      for( int i = 0; i < ncurve_->CtrlP.n; ++i )
	{
	  Point3D cp = project(ncurve_->CtrlP[i]);
	  if( absolute(cp.x-p.x) < dx && 
	      absolute(cp.y-p.y) < dy)
	    {
	      selected_ = i;
	      break;
	    }
	}
	*/
    }
  else if( mouse->button() == RightButton )
    {
      QPoint new_coors = mapToGlobal(mouse->pos());
      popup_->move( new_coors.x(), new_coors.y() );
      popup_->show();
    }
}

void
MEDrawingWidget::resizeEvent( QResizeEvent *e )
{
  QFrame::resizeEvent( e );
  offset_.setX( width()/2 );
  offset_.setY( height()/2 );
}

void
MEDrawingWidget::round( float &x, float &y )
{
  if( gridType_ == Cartesian )
    {
      int tmp;
      if( x >= 0 )
	tmp = int((x / stepx_) + 0.5);
      else
	tmp = int((x / stepx_) - 0.5);
      x  = tmp * stepx_;
      if( y >= 0 )
	tmp = int((y / stepy_) + 0.5);
      else
	tmp = int((y / stepy_) - 0.5);
      y  = tmp * stepy_;
      emit showCoordinates( x, y );
    }
  else if( gridType_ == Polar )
    {
      double radius = sqrt((x*x) + (y*y));
      double angle  = acos( x / radius );

      int irad = int((radius / stepx_) + 0.5);
      radius   = irad * stepx_;
      if( radius < stepx_ )
	radius = stepx_;

      int iang;
      if( y < 0 )
	angle = 2*M_PI - angle;
      if( angle >= 0 )
	iang = int((angle / stepy_) + 0.5);
      else
	iang = int((angle / stepy_) - 0.5);
      angle  = iang * stepy_;

      x = float(radius * cos( angle ));
      y = float(radius * sin( angle ));
      emit showCoordinates( radius, rad2deg(angle) );
    }
}

void
MEDrawingWidget::setEditMode( TEditMode mode )
{
  //TODO finish previous mode
  editMode_ = mode;
  if( editMode_ == AddPolyline )
    {
      MELineObject *obj = new MELineObject;
      objects_.insert( objects_.end(), obj );
      active_ = obj;
      setMouseTracking( TRUE );
    }
}

void
MEDrawingWidget::setGridMode( TGridMode mode, float step_a, float step_b )
{
  //TODO finish previous mode
  gridType_ = mode;

  stepx_ = step_a;
  stepy_ = step_b;
}


//====================================END==========================
