/*
 * Project : "MindsEye, 3D library"
 * Copyright (c) 1995 - 1997. All Rights Reserved.
 * Author(s): P. Lavoie, M. Kuball
 *
 * $Id: InterpWidget.h,v 1.7 1998/06/22 20:12:45 source Exp $
 *
 * COPYRIGHT
 * =========
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *
 * OVERVIEW
 * ========
 *
 */

#ifndef _ne_InterpWidget_h_
#define _ne_IntergWidget_h_

#include <qframe.h>
#include <qpoint.h>

#include "3dlib/aggregate.h"
#include "3dlib/s_point.h"
#include "3dlib/s_line.h"
#include "grid2d.h"

#include <vector>

enum TEditMode { AddPolyline, AppendPoint, FreeHand, EditCurve };

class QPainter;
class QPopupMenu;


/**Objects of this class represent an object in the 2D shaper.
 */
class MELineObject
{
public:
  MELineObject( void ) { curve_ = NULL; }
  MELineObject( G3DNurbsCurve *c) { curve_ = c; }

  void  appendPoint( const Point3D &p ) { pl_.insert(pl_.end(), p); }
  G3DNurbsCurve* curve( void )
    { if( curve_ == NULL ) curve_ = new G3DNurbsCurve; return curve_; }
  void  deleteCurve( void ) { delete curve_;  curve_ = NULL; }
  void  draw( QPainter*, float );
  void  getEndPoint( Point3D &p )
    { if( pl_.size() > 0 )  p = pl_[pl_.size()-1]; }
  vector<Point3D>& pointList( void ) { return pl_; }
  int   numberOfPoints( void ) { return pl_.size(); }

protected:
  vector<Point3D> pl_;
  G3DNurbsCurve   *curve_;
};


/**A canvas widget to draw a NURBS curve. This is used by 
   CurveInterpWidget.
   */
class MEDrawingWidget: public QFrame
{
  Q_OBJECT

public:
  MEDrawingWidget( QWidget *parent=0, const char *name=0 );
  ~MEDrawingWidget();

  void  addCurve( G3DNurbsCurve* );
  G3DObject* getObject( void );
  int   modCurve() const { return mod_curve; }
  void  setEditMode( TEditMode );

signals:
  void computeCurve( MELineObject* );
  void newPoint3D( Point3D* );
  void showCoordinates( float, float );

public slots:
  void clearCanvas();
  void setGridMode( TGridMode, float, float );

protected:
  void drawContents( QPainter* );
  void drawCrossWire( QPainter &p, int, int );
  void drawGrid( QPainter *p );
  void makePopup( void );
  void mouseMoveEvent ( QMouseEvent *);
  void mousePressEvent ( QMouseEvent *);
  void resizeEvent( QResizeEvent* );
  void round( float &x, float &y );

protected slots:
  void computeCurve();
  void deleteCurve();

protected:
  vector<MELineObject*> objects_;
  MELineObject          *active_;

  int   selected_;
  int   mod_curve;
  float stepx_, stepy_;
  float scale_;
  float mouseX_, mouseY_;

  TGridMode  gridType_;
  TEditMode  editMode_;
  QPopupMenu *popup_;

  Point3D    mousePt_;
  QPoint     offset_;

};


#endif
