/*
 * Project : "MindsEye, NURBS editor"
 * Copyright 1996, 1997. All Rights Reserved.
 * Author(s): P. Lavoie, M. Kuball
 *
 * $Id: SorWidget.cpp,v 1.5 1998/06/07 20:12:33 source Exp $
 *
 *
 * COPYRIGHT
 * =========
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

#include <stdio.h>

#include <qbttngrp.h> 
#include <qpushbt.h>
#include <qradiobt.h>
#include <qpixmap.h>
#include <qkeycode.h>
#include <qlabel.h>
#include <qlined.h>

#include "3dlib/t_objects.h"
#include "3dlib/s_line.h"
#include "3dlib/s_nurbsrot.h"
#include "3dlib/math/tools.h"
#include "kernel/world_obj.h"
#include "angleWidget.h"
#include "SorWidget.h"


SurfOfRevWidget::SurfOfRevWidget( QWidget *parent, const char *name) :
  QWidget(parent,name)
{
  //initMetaObject() ;
  setMinimumSize(300,200) ;
  setMaximumSize(300,200) ;

  _model = NULL;
  QLabel *angleL = new QLabel("Revolution of", this);
  angleL->move(10,5) ;
  
  a = 2.0*M_PI ;

  editA = new QLineEdit(this) ;
    editA->setGeometry(10,115,45,20) ;
    editA->setText("360.0") ;
  aW = new AngleWidget(this) ;
    aW->setGeometry(10,32,70,70) ;
    aW->setAngleDeg(360) ;
  QLabel *labelDegree = new QLabel("degree",this) ;
    labelDegree->setGeometry(60,115,42,20);
  QLabel *labelAround = new QLabel("around",this) ;
    labelAround->setGeometry(100,10,40,20) ;

  connect( editA, SIGNAL(textChanged(const char*)),
	   SLOT(editAngle(const char*)) );
  connect(aW,SIGNAL(angleChanged(double)),SLOT(changeAngle(double))) ;

  QLabel *labelSx,*labelSy,*labelSz,*labelTx,*labelTy,*labelTz ;

  editSx = new QLineEdit(this) ; editSx->setGeometry(240,20,50,20);
  editSy = new QLineEdit(this) ; editSy->setGeometry(240,40,50,20);
  editSz = new QLineEdit(this) ; editSz->setGeometry(240,60,50,20);

  editTx = new QLineEdit(this) ; editTx->setGeometry(240,100,50,20);
  editTy = new QLineEdit(this) ; editTy->setGeometry(240,120,50,20);
  editTz = new QLineEdit(this) ; editTz->setGeometry(240,140,50,20);

  sx = sy = sz = tx = 0 ;
  ty = tz = 1 ;

  editSx->setText("0") ;
  editSy->setText("0") ;
  editSz->setText("0") ;

  editTx->setText("0") ;
  editTy->setText("1") ;
  editTz->setText("1") ;

  labelSx = new QLabel("origin (x)",this) ;
    labelSx->setGeometry(180,20,60,20); 
  labelSy = new QLabel("origin (y)",this) ;
    labelSy->setGeometry(180,40,60,20); 
  labelSz = new QLabel("origin (z)",this) ;
    labelSz->setGeometry(180,60,60,20); 
  labelTx = new QLabel("tangent(x)",this) ;
    labelTx->setGeometry(180,100,60,20);
  labelTy = new QLabel("tangent(y)",this) ;
    labelTy->setGeometry(180,120,60,20);
  labelTz = new QLabel("tangent(z)",this) ;
    labelTz->setGeometry(180,140,60,20);

  connect(editSx,SIGNAL(textChanged(const char*)),
	  this, SLOT(editSX(const char*))) ;
  connect(editSy,SIGNAL(textChanged(const char*)),
	this, SLOT(editSY(const char*))) ;
  connect(editSz,SIGNAL(textChanged(const char*)),
	this, SLOT(editSZ(const char*))) ;
  connect(editTx,SIGNAL(textChanged(const char*)),
	this, SLOT(editTX(const char*))) ;
  connect(editTy,SIGNAL(textChanged(const char*)),
	this, SLOT(editTY(const char*))) ;
  connect(editTz,SIGNAL(textChanged(const char*)),
	this, SLOT(editTZ(const char*))) ;

  QButtonGroup *axisGroup = new QButtonGroup(this) ;
    axisGroup->setGeometry(100,32,60,80) ;
    axisGroup->setFrameStyle(QFrame::NoFrame) ;
    connect(axisGroup,SIGNAL(clicked(int)),SLOT(setRotationAxis(int))) ;

  QRadioButton *Xaxis = new QRadioButton("X axis",axisGroup) ;
  QRadioButton *Yaxis = new QRadioButton("Y axis",axisGroup) ;
  QRadioButton *Zaxis = new QRadioButton("Z axis",axisGroup) ;
  QRadioButton *Oaxis = new QRadioButton("Custom axis",axisGroup) ;

  Xaxis->setGeometry(5,5,55,20) ;
  Yaxis->setGeometry(5,25,55,20) ;
  Zaxis->setGeometry(5,45,55,20) ;
  Oaxis->setGeometry(5,65,55,20) ;

  Yaxis->setChecked(TRUE) ;
  axis_mode = 1 ;
  S = Point3D(0,0,0) ;
  T = Point3D(0,1,0) ;

  QPushButton *okBtn = new QPushButton("Ok",this) ;
    okBtn->setGeometry( 10, 170, 60, 22 );
    connect( okBtn, SIGNAL(clicked()), SLOT(sor()) );

  QPushButton *cancelBtn = new QPushButton("Cancel",this);
    cancelBtn->setGeometry( 80, 170, 60, 22 );
    connect(cancelBtn,SIGNAL(clicked()),SLOT(hideWindow())) ;

}


void
SurfOfRevWidget::changeAngle( double ang )
{
  a = ang ;
  char newText[6] ;
  sprintf(newText,"%3f",a*180.0/M_PI);
  editA->setText( newText ) ;
}

void
SurfOfRevWidget::editAngle(const char* c)
{
  double ang  = atof(c) ;
  aW->setAngleDeg(ang) ;
}

void
SurfOfRevWidget::editSX(const char* c)
{
  sx = atof(c) ;
}

void
SurfOfRevWidget::editSY(const char* c)
{
  sy = atof(c) ;
}

void
SurfOfRevWidget::editSZ(const char* c)
{
  sz = atof(c) ;
}

void
SurfOfRevWidget::editTX(const char* c)
{
  tx = atof(c) ;
}

void
SurfOfRevWidget::editTY(const char* c)
{
  ty = atof(c) ;
}

void SurfOfRevWidget::editTZ(const char* c)
{
  tz = atof(c) ;
}

void
SurfOfRevWidget::hideWindow()
{
  hide();
}

void
SurfOfRevWidget::setRotationAxis(int axis)
{
  axis_mode = axis ;
}

void
SurfOfRevWidget::show( MWorldObject *world )
{
  _model = world;
  QWidget::show();
}

void
SurfOfRevWidget::sor()
{
  S = Point3D( 0, 0, 0);
  switch( axis_mode )
    {
    case 0:
      T = Point3D(1,0,0) ; break ;
    case 1:
      T = Point3D(0,1,0) ; break ;
    case 2:
      T = Point3D(0,0,1) ; break ;
    case 3:
      S = Point3D(sx,sy,sz);
      T = Point3D(tx,ty,tz);
      break;
    }
  if( !(T.x == 0 && T.y == 0 && T.z == 0) && a != 0 )
    {
      surfaceOfRevolution();
    }
  hide() ;
}

/*TODO What happens to the selected curve? world->replace( x with y ); */
void
SurfOfRevWidget::surfaceOfRevolution( void )
{
  G3DObject *obj;
  if( _model )
    {
      obj = _model->getSelection();
      if( obj && (obj->getType() == T_NURBS_CURVE) )
	{
	  PlNurbsCurve    tmpc;

	  tmpc = *(G3DNurbsCurve*)obj;
	  MatrixRT<float> trafo;
	  G3DMatrix       tmp;
	  obj->getWorldTrafo( tmp );
	  makeMatrix( trafo, tmp );
	  tmpc.transform( trafo );

	  G3DNurbsSurface *s = new G3DNurbsSurface;
	  s->makeFromRevolution( tmpc, S, T, a );
	  //G3DRotationSurface *s = new G3DRotationSurface( tmpc, S, T, a );
	  //s->makeRot();

	  _model->addObject( s );
	}
    }
}


//==================================END==============================
