/*
 * Project : "MindsEye, NURBS editor"
 * Copyright 1996, 1997. All Rights Reserved.
 * Author(s): P. Lavoie, M. Kuball
 *
 * $Id: SkinSurf.cpp,v 1.1 1997/11/22 19:11:54 source Exp $
 *
 *
 * COPYRIGHT
 * =========
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

#include <stdio.h>
#include <stdlib.h>
#include <qbttngrp.h> 
#include <qpushbt.h>
#include <qradiobt.h>
#include <qlabel.h>
#include <qlined.h>

#include "3dlib/t_objects.h"
#include "3dlib/s_nurbsurf.h"
#include "3dlib/s_line.h"
#include "3dlib/selectGroup.h"
#include "3dlib/math/tools.h"
#include "kernel/world_obj.h"
#include "SweepWidget.h"


static char *l1Text = "To generate a sweep surface, you need to select\n"
                      "two curves. The 1st one is the profile and the 2nd\n"
                      "one will be the path.";


SweptSurfWidget::SweptSurfWidget( QWidget *parent, const char *name ) :
  QWidget(parent,name)
{
  //initMetaObject() ;
  setMinimumSize( 270, 200 );
  setMaximumSize( 270, 200 );

  // since the information in label2 is important, I want it to be a 
  // different colors than the others.
  QColor label2col( 0, 0, 255 );

  QColorGroup colGroup = colorGroup();
  QColorGroup newColGroup( colGroup.foreground(), colGroup.background(),
			   colGroup.light(), colGroup.dark(), colGroup.mid(),
			   label2col, colGroup.base() );
  QPalette newPalette( newColGroup, newColGroup, newColGroup );

  QLabel *label1 = new QLabel(this) ;
    label1->setText( l1Text );
    label1->setGeometry(10,5,260,55) ;
  QLabel *label2 = new QLabel(this) ;
    label2->setPalette(newPalette) ;
    label2->setText("Make sure the profile is in the X-Z plane!");
    label2->setGeometry(10,55,260,20) ;
  QLabel *label3 = new QLabel(this) ;
    label3->setText("You might need to modify the following options\n"
		    "To obtain a proper result.");
    label3->setGeometry(10,75,280,40) ;

  QButtonGroup *yzBtns = new QButtonGroup(this) ;

  yzBtns->setFrameStyle(QFrame::NoFrame) ;
  yzBtns->setGeometry(25,115,20,30) ;

  QPushButton *btnY = new QPushButton( yzBtns );
  QPushButton *btnZ = new QPushButton( yzBtns );

  btnY->setGeometry(1,0,10,10) ;
  btnZ->setGeometry(1,15,10,10) ;
  btnY->setToggleButton(TRUE) ;
  btnZ->setToggleButton(TRUE) ;

  QLabel *labelY = new QLabel("rotate around Y-axis",this) ;
  QLabel *labelZ = new QLabel("inverse the Z rotation",this) ;
  labelY->setGeometry(40,110,150,15) ;
  labelZ->setGeometry(40,125,150,15) ;

  connect(yzBtns,SIGNAL(clicked(int)),SLOT(editAngles(int))) ;

  aY = 0 ;
  aZ = 0 ;

  QLineEdit *editK = new QLineEdit(this) ;
  editK->setGeometry(220,145,35,20) ;
  editK->setText("0") ;

  connect( editK,SIGNAL(textChanged(const char*)),
	   SLOT(editMinK(const char*)) );

  QLabel* labelK = new QLabel("Specify the minimal number of sections:",this) ;
  labelK->setGeometry( 10, 145, 205, 20 );

  QPushButton *okBtn = new QPushButton("Ok",this) ;
  QPushButton *cancelBtn = new QPushButton("Cancel",this) ;
  
  okBtn->setGeometry(50,175,60,22);
  cancelBtn->setGeometry(150,175,60,22);

  connect(okBtn,SIGNAL(clicked()),SLOT(sweepSurface())) ;
  connect(cancelBtn,SIGNAL(clicked()),SLOT(hideWindow())) ;
}

void
SweptSurfWidget::editAngles(int a)
{
  switch(a){
  case 0: aY = !aY ; break ;
  case 1: aZ = !aZ ; break ;
  }
}

void
SweptSurfWidget::editMinK( const char* d )
{
  double a = atof(d);
  if( a>= 0 )
    minK = int(a);
}

void
SweptSurfWidget::hideWindow()
{
  hide() ;
}

void
SweptSurfWidget::show( MWorldObject *model )
{
  _model = model;
  QWidget::show();
}

void
SweptSurfWidget::sweepSurface()
{

  //emit genSweep(minK,aY,aZ) ;
  // void NurbsObjectWidget::sweptSurface(int k, int ay, int az){

  G3DObject *obj = _model->getSelection();
  if( obj && (obj->getType() == T_SEL_GROUP) )
    {
      G3DSelectionGroup *grp = (G3DSelectionGroup*)obj;
      if( grp->numberOfObjects() != 2 )
	return;
      if(  T_NURBS_CURVE != (grp->getObject(0))->getType() )
        return;
      if( (grp->getObject(1))->getType() != T_NURBS_CURVE )
        return;

      // the 1st selected curve is the profile, 
      PlNurbsCurve profile( *(G3DNurbsCurve*)(grp->getObject(0)) );
      // the 2nd selected curve is the path
      PlNurbsCurve path( *(G3DNurbsCurve*)(grp->getObject(1)) );

      MatrixRT<float> trafo;
      G3DMatrix       tobj;
      (grp->getObject(0))->getLocalTrafo( tobj );
      makeMatrix( trafo, tobj );
      profile.transform( trafo );
      (grp->getObject(1))->getLocalTrafo( tobj );
      makeMatrix( trafo, tobj );
      path.transform( trafo );

      G3DNurbsSurface *s = new G3DNurbsSurface;
      if( minK <= 1 )
	minK = path.CtrlP.n;
      s->sweep( path, profile, minK, aY, aZ );

      _model->addObject( s );
    }
  hide() ;
}


//==================================END==============================
