/*
 * Project : "MindsEye, 3D library"
 * Copyright (c) 1995 - 1997. All Rights Reserved.
 * Author(s): P. Lavoie, M. Kuball
 *
 * $Id: InterpWidget.h,v 1.1.1.1 1997/11/22 15:59:27 source Exp $
 *
 * COPYRIGHT
 * =========
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *
 * OVERVIEW
 * ========
 *
 */


#include <qpainter.h>
#include <iostream.h>
#include <qpixmap.h>

#include "angleWidget.h"


inline double absolute(double a)
{
  return (a < 0) ? -a : a ;
}



AngleWidget::AngleWidget(QWidget *parent, const char *name) :
  QWidget(parent,name)
{
  //initMetaObject();
  angle  = 0;
  angleS = 0;
  angleE = 2.0 * M_PI;  //full circle
  setMinimumSize( 20, 20 );
}


void AngleWidget::mouseMoveEvent(QMouseEvent *mouse)
{
  if(mouse->state() == LeftButton)
    {
      double x  = double(mouse->x() - width()/2) ;
      double y  = double(height()/2 - mouse->y()) ;
      double a  = atan2(y,x) ;
      double a2 = angle ;

      if(a < 0)
	a += 2.0*M_PI ;
      while(a2 > 2.0*M_PI)
	a2 -= 2.0*M_PI ;
      if(a2 < 0)
	a2 += 2.0*M_PI ;
      if(absolute(a-a2) > M_PI)
	{
	  if(a < a2)
	    a += 2.0*M_PI ;
          else
	    a2 += 2.0*M_PI ;
	}
      setAngleRad(angle+(a-a2)) ;
    }
}


void AngleWidget::mousePressEvent(QMouseEvent *mouse)
{
  if(mouse->button() == LeftButton)
    {
      double x  = double(mouse->x() - width()/2) ;
      double y  = double(height()/2 - mouse->y()) ;
      double a  = atan2(y,x) ;
      double a2 = angle ;

      if(a < 0)
	a += 2.0*M_PI ;
      while(a2 > 2.0*M_PI)
	a2 -= 2.0*M_PI ;
      if(a2 < 0)
	a2 += 2.0*M_PI ;
      if(absolute(a-a2) > M_PI)
	{
	  if(a < a2)
	    a += 2.0*M_PI ;
          else
	    a2 += 2.0*M_PI ;
	}
      setAngleRad(angle+(a-a2)) ;
    }
}

void AngleWidget::paintEvent(QPaintEvent *)
{
  QPainter p;
  QBrush   brush(white) ;
  QPen     pen(SolidLine) ;

  pen.setWidth(1) ;  
  p.begin(this) ;
  p.setBrush(brush) ;

  pen.setColor(white) ;
  p.setPen(pen) ;
  p.drawPie( 1,1,width()-2,height()-2,int(angleS*180/M_PI*16.0),
	     int((angleE-angleS)*180/M_PI*16.0) );
  pen.setColor(black) ;
  p.setPen(pen) ;
  p.drawArc( 1,1,width()-2,height()-2,int(angleS*180/M_PI*16.0),
	     int((angleE-angleS)*180/M_PI*16.0) );
  // set the (0,0) to the middle of the window
  p.translate(width()/2,height()/2) ; 
  p.drawLine(0,height()/2-4,0,height()/2) ;
  p.drawLine(0,-(height()/2-4),0,-height()/2) ;
  p.drawLine(width()/2,0,width()/2-4,0) ;
  p.drawLine(-width()/2,0,-(width()/2-4),0) ;

  pen.setColor(blue) ;
  p.setPen(pen) ;

  double radius = double(width()/2-1) ;

  p.drawLine( 0, 0, int(cos(angle)*radius), -int(sin(angle)*radius) );

  p.end() ;
}

void  AngleWidget::setAngleDeg(double a)
{
  setAngleRad(a/180.0*M_PI) ;
}

void AngleWidget::setAngleRad(double a)
{
  if( a != angle )
    {
      if(a < angleS)
	a = angleS ;
      if(a > angleE)
	a = angleE ;
      angle = a ;
      repaint() ;
      emit angleChanged( a );
    }
}

void AngleWidget::setRange(double as, double ae)
{
  if(ae > as)
    {
      angleS = as ;
      angleE = ae ;
    }
  else
    {
      angleE = as ;
      angleS = ae ; 
    }
}


//=====================================END===============================
