/*
 * Project : "MindsEye, 3D library"
 * Copyright (c) 1995 - 1997. All Rights Reserved.
 * Author(s): P. Lavoie, M. Kuball
 *
 * $Id: InterpWidget.h,v 1.1.1.1 1997/11/22 15:59:27 source Exp $
 *
 * COPYRIGHT
 * =========
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *
 * OVERVIEW
 * ========
 *
 */

#include <stdio.h>
#include <qlined.h>
#include <qlabel.h>
#include <qbttngrp.h>
#include <qradiobt.h>
#include <qpushbt.h>
#include <iostream.h>

#include "3dlib/s_line.h"
#include "circleWidget.h"
//#include "angleWidget.h"



GenerateCircleWidget::GenerateCircleWidget( QWidget *parent,
					    const char *name ) :
  QWidget(parent, name) 
{
  //initMetaObject() ;
  //hide() ;
  radius = 1 ;
  as     = 0 ;
  ae     = 2.0*M_PI ; 
  orientation = 0 ;

  setMinimumSize(210,195) ;
  setMaximumSize(210,195) ;

  QLabel *title =
    new QLabel("Create a circle from a start \nto an end angle.",this);
  title->setGeometry(10,5,190,35) ;

  asW = new AngleWidget(this) ;
  asW->setGeometry(10,45,60,60) ;
  asW->setAngleDeg( 0 ) ;
  aeW = new AngleWidget(this) ;
  aeW->setGeometry(80,45,60,60) ;
  aeW->setAngleDeg( 360 ) ;

  editS = new QLineEdit(this) ;
  editS->setGeometry(10,125,60,20) ;
  editS->setText("0.0") ;
  QLabel *sL = new QLabel("Start", this) ;
  sL->setGeometry(10,110,70,15) ;

  editE = new QLineEdit(this) ;
  editE->setGeometry(80,125,60,20) ;
  editE->setText("360.0") ; //TODO 2*PI??
  QLabel *eL = new QLabel("End", this) ;
  eL->setGeometry(80,110,60,15) ;

  editR = new QLineEdit(this) ;
  editR->setText("1.0") ;
  editR->setGeometry( 150, 125, 50, 20) ;
  QLabel *rL = new QLabel("Radius",this) ;
  rL->setGeometry(150,110, 50, 15) ;

  connect( asW, SIGNAL(angleChanged(double)),
	   this, SLOT(changeStartAngle(double)) );
  connect( aeW, SIGNAL(angleChanged(double)),
	   this, SLOT(changeEndAngle(double)) );
  connect( editS, SIGNAL(textChanged(const char*)),
	   this, SLOT(editStartAngle(const char *)) );
  connect( editE, SIGNAL(textChanged(const char*)),
	   this, SLOT(editEndAngle(const char *)) );
  connect( editR, SIGNAL(textChanged(const char*)),
	   this, SLOT(editRadius(const char *)) );

  QButtonGroup* planeGroup ;
  planeGroup = new QButtonGroup("Plane",this) ;
  planeGroup->setExclusive(TRUE) ;
  planeGroup->setGeometry( 155, 42, 50, 60) ;
  connect(planeGroup, SIGNAL(clicked(int)), SLOT(planeMode(int))) ;

  QRadioButton *planeXY = new QRadioButton("X-Y",planeGroup) ;
  QRadioButton *planeXZ = new QRadioButton("X-Z",planeGroup) ;
  QRadioButton *planeYZ = new QRadioButton("Y-Z",planeGroup) ;
  
  planeXY->setGeometry(0,0,50,20) ;
  planeXZ->setGeometry(0,20,50,20) ;
  planeYZ->setGeometry(0,40,50,20) ;
  planeXY->setChecked(TRUE) ;
 
  QPushButton *okButton     = new QPushButton("Create",this) ;
  QPushButton *cancelButton = new QPushButton("Cancel",this) ;
  QPushButton *resetButton  = new QPushButton("Reset",this) ;
 
  connect(okButton,     SIGNAL(clicked()), SLOT(addCircle())) ;
  connect(cancelButton, SIGNAL(clicked()), SLOT(hideWindow())) ;
  connect(resetButton,  SIGNAL(clicked()), SLOT(resetValues())) ;

  okButton->setGeometry(5, 160, 60, 25) ;
  resetButton->setGeometry(75, 160, 60, 25) ;
  cancelButton->setGeometry(145, 160, 60, 25) ;
}


void GenerateCircleWidget::addCircle()
{
  G3DNurbsCurve *curve;
  Point3D       x_axis, y_axis ;

  if( radius <= 0.0 )
    radius = 1.0 ;

  char newText[12] ;
  sprintf(newText, "%.3f", radius) ;
  editR->setText( newText ) ;

  switch(orientation)
    {
    case 0: x_axis = Point3D(1,0,0) ; y_axis=Point3D(0,1,0) ; break ;
    case 1: x_axis = Point3D(1,0,0) ; y_axis=Point3D(0,0,1) ; break ;
    case 2: 
    default:
      x_axis = Point3D(0,1,0) ; y_axis=Point3D(0,0,1) ; break ;
    }
  curve = new G3DNurbsCurve ;
  curve->makeCircle(Point3D(0,0,0),x_axis,y_axis,float(radius),as,ae) ;
  emit newNurbs( (G3DObject*)curve );

  hide() ;
}

void GenerateCircleWidget::changeStartAngle( double a )
{
  as = a ;
  char newText[12] ;
  sprintf( newText, "%.4f", a*180.0/M_PI );
  editS->setText( newText ) ;
}

void GenerateCircleWidget::changeEndAngle(double a)
{
  ae = a ;
  char newText[12] ;
  sprintf( newText, "%.4f", a*180.0/M_PI );
  editE->setText( newText ) ;
}

void GenerateCircleWidget::editStartAngle(const char* c)
{
  double a  = atof(c) ;
  asW->setAngleDeg(a) ;
}

void GenerateCircleWidget::editEndAngle(const char* c)
{
  double a  = atof(c) ;
  aeW->setAngleDeg(a) ;
}

void GenerateCircleWidget::editRadius(const char* c)
{
  radius = atof(c) ;
}

void GenerateCircleWidget::hideWindow()
{
  hide() ;
}

void GenerateCircleWidget::planeMode(int a)
{
  switch(a){
  case 0: orientation = 0 ; break ;
  case 1: orientation = 1 ; break ;
  case 2: orientation = 2 ; break ;
  }
}

void GenerateCircleWidget::resetValues()
{
  radius = 1.0 ;
  as = 0 ;
  ae = 2.0*M_PI ;
  asW->setAngleRad(as) ;
  aeW->setAngleRad(ae) ;
  char newText[12] ;
  sprintf(newText,"%.3f",radius) ;
  editR->setText(newText) ;
}


//================================END===================================
