/*
 * Project : "MindsEye, NURBS editor"
 * Copyright 1996, 1997. All Rights Reserved.
 * Author(s): P. Lavoie, M. Kuball
 *
 * $Id: infoWidget.cpp,v 1.4 1998/03/30 20:23:47 source Exp $
 *
 * COPYRIGHT
 * =========
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * OVERVIEW
 * ========
 * The info widgets are now child classes of class PropertyDialog.
 * The methos curveInfo and surfaceInfo are changed to the
 * virtual function setObject().
 */

#include <qpushbt.h>
#include <qstring.h>
#include <qkeycode.h>
#include <qlistbox.h>
#include <qlabel.h>

#include "3dlib/s_line.h"
#include "3dlib/s_nurbsurf.h"
#include "infoWidget.h"


InfoCurveWidget::InfoCurveWidget(QWidget *parent, const char *name) :
  ObjectProperties(parent, name)
{
  //initMetaObject() ;

  _gpage = new QWidget( this, "Curve" );

  QLabel *label2 = new QLabel("Degree:", _gpage) ;
  label2->setGeometry(10,10,50,15) ;
  degreeLabel = new QLabel( _gpage ) ;
  degreeLabel->setGeometry( 65, 10, 10, 15 );

  QLabel *label3 = new QLabel("Control points:", _gpage) ;
  label3->setGeometry( 10, 30, 150, 15 );
  QLabel *label4 = new QLabel("Knot vector:", _gpage) ;
  label4->setGeometry( 200, 30, 80, 15 );

  cpBox       = new QListBox( _gpage );
  cpBox->setGeometry( 5, 45, 180, 90 );
  knotBox     = new QListBox( _gpage );
  knotBox->setGeometry( 190, 45, 100, 90) ;
 
  QPushButton *increaseBtn = new QPushButton("increase", _gpage) ;
  increaseBtn->setGeometry( 85, 8, 60, 20 );
  connect( increaseBtn, SIGNAL(clicked()), SLOT(increaseDegree()) ) ;

  /*
  QPushButton *okBtn = new QPushButton( "Ok", _gpage );
  okBtn->setGeometry( 140, 288, 60, 20 );
  connect( okBtn, SIGNAL(clicked()), SLOT(hideWindow()) ) ;
  */

  addTab( _gpage, "Curve" );
  ObjectProperties::addPages();
}


void
InfoCurveWidget::setObject( G3DObject *c )
{
  G3DNurbsCurve *curve = (G3DNurbsCurve*)c;
  QString       txt;

  txt.sprintf( "%d", curve->degree );
  degreeLabel->setText( txt );
  cpBox->clear() ;
  for( int i = 0; i < curve->CtrlP.n; ++i )
    {
      QString cpTxt ;
      cpTxt.sprintf("%3d : %3.3f %3.3f %3.3f %3.3f",
		    i, curve->CtrlP[i].x(), curve->CtrlP[i].y(),
		    curve->CtrlP[i].z(), curve->CtrlP[i].w()) ;
      cpBox->insertItem(cpTxt) ;
    }

  knotBox->clear() ;
  for( int j = 0; j < curve->Knot.n; ++j )
    {
      QString knotTxt ;  
      knotTxt.sprintf("%3d : %2.4f", j, curve->Knot[j]) ;
      knotBox->insertItem( knotTxt );
    }

  ObjectProperties::setObject( c ); //TODO
}


void
InfoCurveWidget::increaseDegree()
{
  if( _obj )
    {
      ((G3DNurbsCurve*)_obj)->degreeElevate( 1 );
      ((G3DNurbsCurve*)_obj)->resetAll();
    }
  setObject( _obj );
}


//-----------------------------------------------------------------


InfoSurfaceWidget::InfoSurfaceWidget(QWidget *parent, const char *name) :
  ObjectProperties(parent, name)
{
  QWidget *page;
  page = new QWidget( this, "Surface" );

  QLabel *label2 = new QLabel("Degree U:", page) ;
  label2->setGeometry(45,40,50,15) ;
  QLabel *label5 = new QLabel("Degree V:", page) ;
  label5->setGeometry(45,60,50,15) ;
  QLabel *label3 = new QLabel("Control points:", page) ;
  label3->setGeometry(10,80,150,15) ;
  QLabel *label4 = new QLabel("Knot vector U", page) ;
  label4->setGeometry(20,190,80,15) ;
  QLabel *label6 = new QLabel("Knot vector V", page) ;
  label6->setGeometry(160,190,80,15) ;

  degreeULabel = new QLabel(page) ;
  degreeVLabel = new QLabel(page) ;
  cpBox = new QListBox(page) ;
  knotUbox = new QListBox(page) ;
  knotVbox = new QListBox(page) ;
  
  degreeULabel->setGeometry(100,40,30,15) ;
  degreeVLabel->setGeometry(100,60,30,15) ;
  cpBox->setGeometry(110,80,225,100) ;
  knotUbox->setGeometry(20,210,120,80) ;
  knotVbox->setGeometry(160,210,120,80) ;

  QPushButton *increaseUBtn = new QPushButton("increase",page) ;
  increaseUBtn->setGeometry(130,38,60,19) ;
  connect(increaseUBtn,SIGNAL(clicked()),SLOT(increaseDegreeU())) ;

  QPushButton *increaseVBtn = new QPushButton("increase",page) ;
  increaseVBtn->setGeometry(130,58,60,19) ;
  connect(increaseVBtn,SIGNAL(clicked()),SLOT(increaseDegreeV())) ;

  /*
  QPushButton *okBtn = new QPushButton("Ok",page) ;
  okBtn->setGeometry(140,308,60,20) ;
  connect(okBtn,SIGNAL(clicked()),SLOT(hideWindow())) ;
  */

  addTab( page, "Surface" );
  ObjectProperties::addPages();
}


void
InfoSurfaceWidget::setObject( G3DObject* s )
{
  QString txt ;
  G3DNurbsSurface *surface = (G3DNurbsSurface*)s;
  txt.sprintf("%d",surface->degreeU) ;
  degreeULabel->setText(txt) ;
  txt.sprintf("%d",surface->degreeV) ;
  degreeVLabel->setText(txt) ;

  QString knotUtxt, knotVtxt, cpTxt ;
  int i ;
  
  cpBox->clear() ;
  for( i = 0; i < surface->CtrlP.rows; ++i )
    for( int j = 0; j < surface->CtrlP.cols; ++j )
      {
	cpTxt.sprintf("%3d,%3d : %3.3f %3.3f %3.3f %3.3f",
		      i,j,surface->CtrlP(i,j).x(),surface->CtrlP(i,j).y(),
		      surface->CtrlP(i,j).z(),surface->CtrlP(i,j).w()) ;
	cpBox->insertItem(cpTxt) ;
      }

  knotUbox->clear() ;
  for( i = 0; i < surface->KnotU.n; ++i)
    {
      knotUtxt.sprintf("%3d : %2.3f",i,surface->KnotU[i]) ;
      knotUbox->insertItem(knotUtxt) ;
    }

  knotVbox->clear() ;
  for( i = 0; i < surface->KnotV.n; ++i)
    {
      knotVtxt.sprintf("%3d : %2.3f",i,surface->KnotV[i]) ;
      knotVbox->insertItem(knotVtxt) ;
    }

  ObjectProperties::setObject( s ); //TODO
}


void
InfoSurfaceWidget::increaseDegreeU()
{
  if( _obj )
    {
      G3DNurbsSurface ns( *((G3DNurbsSurface*)_obj) );
      degreeElevateU(ns,1,*((G3DNurbsSurface*)_obj) );
      ((G3DNurbsSurface*)_obj)->resetAll();
    }
  setObject( _obj );
}


void
InfoSurfaceWidget::increaseDegreeV()
{
  if( _obj )
    {
      G3DNurbsSurface ns(*((G3DNurbsSurface*)_obj) );
      degreeElevateV(ns,1,*((G3DNurbsSurface*)_obj) );
      ((G3DNurbsSurface*)_obj)->resetAll();
    }
  setObject( _obj );
}


//====================================END==============================
