/*
 * Project : "MindsEye, 3D library"
 * Copyright (c) 1995 - 1997. All Rights Reserved.
 * Author(s): P. Lavoie, M. Kuball
 *
 * $Id: InterpWidget.cpp,v 1.5 1998/03/30 20:23:46 source Exp $
 *
 * COPYRIGHT
 * =========
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *
 * OVERVIEW
 * ========
 *
 */

#include <qpainter.h>
//#include <qpixmap.h>
#include <qlabel.h>
#include <qbttngrp.h>
#include <qradiobt.h>
#include <qpushbt.h>
#include <qlined.h>
#include <qlayout.h>
#include <qmenubar.h>
#include <qpopmenu.h>
#include <qkeycode.h>
#include <qcombo.h>

#include "InterpWidget.h"
#include "shaper.h"
#include "grid2d.h"
#include "3dlib/math/tools.h"


static const char *InterpolationMethods[3] =
{ "Interpolation", "Least squares", "Approximation" };

enum { Interpolate = 0, LeastSquares, Approximate };


ME2DShaper::ME2DShaper( int module_id, int load_t ) :
  MEModule(module_id, load_t, "ME2DShaper")
{
  interp_mode = Interpolate;
 
  QBoxLayout *gm = new QVBoxLayout( this, 0, 2 );
  gm->setMenuBar( makeMenuBar() );
  gm->addWidget( makeToolbar(), 0 );

  drawArea = new MEDrawingWidget( this );
  connect( drawArea, SIGNAL(computeCurve(MELineObject*)),
	   SLOT(computeCurve(MELineObject*)) );
  connect( drawArea, SIGNAL(showCoordinates(float,float)),
	   SLOT( showCoordinates(float,float)) );
  gm->addWidget( drawArea, 100 );

  previewLabel =  new QLabel( this );
  previewLabel->setMinimumSize( 100, 20 );
  gm->addWidget( previewLabel, 0 );

  gm->activate();
  resetValues();

  setMinimumSize( 200, 200 );
  resize( 400, 400 );
}

ME2DShaper::~ME2DShaper()
{
}


void
ME2DShaper::addCurve()
{
  drawArea->setEditMode( AddPolyline );
}

void
ME2DShaper::addDrawing()
{
  drawArea->setEditMode( FreeHand );
}

void
ME2DShaper::computeCurve( MELineObject *line )
{
  int n = line->numberOfPoints();
  modified = 1 ;

  if( n > 1 )
    {
      if( degree >= n )
	{
	  cerr << "There are not enough points for the degree specified "
	       << degree << endl;
	  return;
	}
      if( (interp_mode==1) && (nCP < degree+1) )
	{
	  cerr << "Not enough points specified for least squares approximation"
	       << endl;
	  return ;
	}
      if( (interp_mode==2) && (error <= 0) )
	{
	  cerr << "Error while inputing the allowed error for approximation"
	       << endl << "Using 1.0 pixel instead." << endl;
	  error = 1;
	}

      Vector_Point3D pts( n );
      vector<Point3D> &points = line->pointList();
      int i = 0, s = 1;

      for( int j = 0; j < points.size(); ++j )
	{
	  pts[i] = points[j];
	  if( i > 0 )
	    {
	      if( pts[i] == pts[i-1] )
		++s;
	      else
		s == 1;
	      if( s <= degree )
		++i ;
	    }
          else
	    ++i ; 
	}
      pts.resize(i) ;

      float err = error / 200.0;
      if( nCP > pts.n )
	nCP = pts.n ;

      switch( interp_mode )
	{
	case Interpolate:
	  line->curve()->globalInterp(pts,degree) ; break ;
	case LeastSquares:
	  line->curve()->leastSquares(pts,degree,nCP) ; break ;
	case Approximate:
	  line->curve()->globalApproxErrBnd(pts,degree,err) ; break ;
	}
      modified = 0 ;

      char txt[100];
      sprintf( txt, "From %d points it generates %d c.p.",
	       pts.n, line->curve()->CtrlP.n );
      previewLabel->setText( txt );
    }
}

void ME2DShaper::editDegree(const char* txt)
{ 
  degree = (int)atof( txt ); 
  modify();
}

void ME2DShaper::editError(const char* txt)
{
  error = (float)atof( txt ); 
  modify();
}

void ME2DShaper::editNofCP(const char* txt)
{
  nCP = (int)atof( txt ); 
  modify();
}

G3DObject*
ME2DShaper::getObject( void )
{
  return drawArea->getObject();
/*
  //if( modified )
    computeCurve();
*/
}

void
ME2DShaper::hideWindow()
{ 
  hide(); 
}

void
ME2DShaper::interpMode( int a )
{
  interp_mode = a;
  modify();
}

QMenuBar*
ME2DShaper::makeMenuBar( void )
{
  QPopupMenu *file = new QPopupMenu;
  CHECK_PTR( file );
  //file->insertItem( "load", this, SLOT(loadScene()), CTRL+Key_L );
  //file->insertItem( "save", this, SLOT(saveScene()), CTRL+Key_S );
  //file->insertItem( "save Selection", this,SLOT(exportObjects()), CTRL+Key_E );
  //file->insertSeparator();
  file->insertItem( "close", this, SLOT(hideWindow()), CTRL+Key_C );

  QPopupMenu *edit = new QPopupMenu;
  CHECK_PTR( edit );
  edit->insertItem( "add polyline", this, SLOT(addCurve()) );
  edit->insertItem( "add drawing", this, SLOT(addDrawing()) );
  //edit->insertItem( "undo",  this, SLOT(newScene()), CTRL+Key_U );
  //edit->insertItem( "scene properties", this, SLOT(sceneProperties()) );


  QPopupMenu *options = new QPopupMenu;
  CHECK_PTR( options );
  options->insertItem( "show Control Pts" );
  
  options->insertItem( "grid settings...", this, SLOT(gridDialog()) );

  QMenuBar *menu = new QMenuBar( this );
  menu->setFrameStyle( QFrame::WinPanel | QFrame::Raised );
  CHECK_PTR( menu );
  menu->insertItem( "&File", file );
  menu->insertItem( "&Edit", edit );
  menu->insertItem( "&Options", options );

  return( menu );
}

QFrame*
ME2DShaper::makeToolbar()
{
  QFrame *toolbar = new QFrame( this );
    toolbar->setMinimumSize( 26, 100 );
    toolbar->setFrameStyle( QFrame::Panel | QFrame::Raised );
    toolbar->setLineWidth( 2 );

  QBoxLayout *gm = new QHBoxLayout( toolbar, 3, 0 );
  QPushButton *clearBtn = new QPushButton( "Clear", toolbar );
    clearBtn->setMinimumSize( 50, 20 );
    connect( clearBtn,   SIGNAL(clicked()), SLOT(resetValues())) ;
    gm->addWidget( clearBtn, 0 );
  QPushButton *previewBtn = new QPushButton( "Preview", toolbar );
    previewBtn->setMinimumSize( 50, 20 );
    connect( previewBtn, SIGNAL(clicked()), SLOT(preview())) ;
    gm->addWidget( previewBtn, 0 );

  gm->addSpacing( 10 );

  QComboBox *cbox = new QComboBox( toolbar );
    cbox->setMinimumSize( 100, 20 );
    cbox->insertStrList( InterpolationMethods, 3 );
    gm->addWidget( cbox, 0 );
    connect( cbox, SIGNAL(activated(int)), SLOT(interpMode(int)));

  gm->addStretch( 100 );
  gm->activate();
  return( toolbar );

    /*
  editDeg = new QLineEdit( toolbar ) ;
    editDeg->setMinimumSize( 40, 20 );
    gm->addWidget( editDeg, 0 );
    connect( editDeg, SIGNAL(textChanged(const char*)),
	     this, SLOT(editDegree(const char *)) );

  QLabel *degL = new QLabel("degree", btnBox) ;
  degL->setGeometry(5,15,150,20) ;

  QLabel *cpL = new QLabel("Number of control points\nfor least squares approx",btnBox);
  cpL->setGeometry(5,130,150,35) ;
  editNCP = new QLineEdit(btnBox) ;
  editNCP->setGeometry(155,140,30,20) ;

  QLabel *errL = new QLabel("Accepted pixel error\nfor approximation",btnBox) ;
  errL->setGeometry(5,160,130,35);
  editErr = new QLineEdit(btnBox);
  editErr->setGeometry(135,170,50,20);

  connect( editNCP,SIGNAL(textChanged(const char*)),
	   this, SLOT(editNofCP(const char *)) );
  connect( editErr,SIGNAL(textChanged(const char*)),
	   this, SLOT(editError(const char *)) );
	   */
}

void
ME2DShaper::modify()
{ 
  modified = 1;
}

void
ME2DShaper::preview()
{
  /*
  if( modified )
    computeCurve() ;
  if( !modified )
    drawArea->addCurve( (G3DNurbsCurve*)curve->copy() );
    */
}

void
ME2DShaper::resetValues()
{
  degree = 3 ;
  nCP    = 8 ;
  error  = 1.5 ;

  modify() ;

  /*
  char txt[64] ;
  sprintf( txt, "%d", degree );
  editDeg->setText(txt) ;
  sprintf(txt,"%d",nCP) ;
  editNCP->setText(txt) ;
  sprintf( txt, "%.3f", error );
  editErr->setText( txt );

  previewLabel->setText( " " );
  */
  drawArea->clearCanvas();
}

void
ME2DShaper::gridDialog()
{
  ME2DGridDialog *gridDlg = new ME2DGridDialog();
  connect( gridDlg, SIGNAL(setGridMode(TGridMode,float,float)),
	   drawArea, SLOT(setGridMode(TGridMode,float,float)) );
  gridDlg->show();
}

void
ME2DShaper::setObject( G3DObject* )
{
}

void
ME2DShaper::showCoordinates( float x, float y )
{
  char text[50];

  sprintf( text, "(%.4g,  %.4g)", x, y );
  previewLabel->setText( text );
}


//====================================END==========================
