/*
 * Project : "MindsEye, gui widgets"
 * Copyright (c) 1995 - 1998. All Rights Reserved.
 * Author(s): P. Lavoie, M. Kuball
 *
 * $Id: InterpWidget.h,v 1.4 1998/03/30 20:23:47 source Exp $
 *
 * COPYRIGHT
 * =========
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *
 * OVERVIEW
 * ========
 *
 */

#ifndef _ne_InterpWidget_h_
#define _ne_IntergWidget_h_

#include <qwidget.h>

#include "3dlib/s_point.h"
#include "kernel/m_module.h"

class QLabel;
class QLineEdit;
class QMenuBar;
class QFrame;
class G3DObject;
class MEDrawingWidget;
class MELineObject;


/**A Widget to draw a flat 3d NURBS curve. The curve is build from
  points enterd by the user using some interpolation technique.
 */
class ME2DShaper: public MEModule
{
  Q_OBJECT

public:
    ///
  ME2DShaper( int module_id, int load_t );
  ~ME2DShaper() ;

  G3DObject* getObject( void );
    ///Let the user edit the object.
  void       setObject( G3DObject* );

signals:
  void newNurbs( G3DObject* );

public slots:
  void addCurve() ;
  void addDrawing() ;
  void computeCurve( MELineObject* );
  void editDegree(const char*) ;
  void editNofCP(const char*) ;
  void editError(const char*) ;
  void gridDialog();
  void interpMode(int);
  void hideWindow();
  void resetValues();            
  void preview();
  void modify();
  void showCoordinates(float, float);

protected:
  //void resizeEvent( QResizeEvent* );
  QMenuBar* makeMenuBar( void );
  QFrame*   makeToolbar();

private:
  float error;
    ///Interpolation mode
  int   interp_mode;
  int   degree, nCP;
  int   modified;

  MEDrawingWidget *drawArea;
  QLabel          *previewLabel;
  QLineEdit       *editDeg,*editNCP,*editErr;

};


#endif
