/*
 * Project : "MindsEye, NURBS editor"
 * Copyright 1996, 1997. All Rights Reserved.
 * Author(s): P. Lavoie, M. Kuball
 *
 * $Id: toolbox.cpp,v 1.13 1998/06/22 20:12:45 source Exp $
 *
 * COPYRIGHT
 * =========
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

#include <stdio.h>

#include <qpushbt.h>
#include <qradiobt.h>
#include <qpixmap.h>
#include <qkeycode.h>
#include <qbttngrp.h>
#include <qlabel.h>
#include <qlayout.h>

#include "modeler/modeler.h"
#include "toolbox.h"
#include "shaper.h"
#include "circleWidget.h"
#include "SkinSurf.h"
#include "SorWidget.h"
#include "SweepWidget.h"
#include "ExtrudeWidget.h"
#include "kernel/m_core.h"


//curve, points, polygon, wight,
//axis, center, reset, boundingbox
static char *vgNames[] =
{ "C", "curve", "P", "curveP", "L", "curvePP", "R", "info",
  "A", "axis",  "C", NULL,     "R", "reset",   "B", "bbox" };

//transformation, locale
static char *edNames[] =
{ "T", "transformation", "L", "globe", "CP", "moveCP", "Scale", NULL };


ToolboxWidget::ToolboxWidget( QWidget *model, QWidget *parent,
			      const char *name )
  : QFrame(parent,name)
{
  //initMetaObject() ;
  setMinimumSize(120,45) ;
  setFrameStyle( WinPanel | Raised );
  _model = (MModeler*)model;

  QBoxLayout  *layout = new QVBoxLayout( this, 4 );
  QPixmap     pixmap;
  QPushButton *btmp;

  genSurfGroup = new QButtonGroup("Generate surface",this) ;
    genSurfGroup->setMinimumSize( 120, 45 );
    genSurfGroup->setFrameStyle( NoFrame );
    connect( genSurfGroup, SIGNAL(clicked(int)), SLOT(genSurf(int)) );
    layout->addWidget( genSurfGroup );
  btmp = new QPushButton("SOR", genSurfGroup);
    pixmap.load("pixmap/sor.pnm");
    btmp->setPixmap(pixmap) ;
    btmp->setGeometry( 0, 15, 30, 30 );
  btmp = new QPushButton("Skin", genSurfGroup) ;
    btmp->setGeometry(30,15,30,30) ;
  btmp = new QPushButton("Sweep", genSurfGroup) ;
    btmp->setGeometry(60,15,30,30) ;
  btmp = new QPushButton("Extrude", genSurfGroup) ;
    btmp->setGeometry(90,15,30,30) ;

    /*
  genCurveGroup = new QButtonGroup("Generate curve",this) ;
    genCurveGroup->setMinimumSize( 120, 45 );
    genCurveGroup->setFrameStyle( NoFrame );
    connect( genCurveGroup, SIGNAL(clicked(int)), SLOT(genCurve(int)) );
    layout->addWidget( genCurveGroup );
  btmp = new QPushButton( "circle", genCurveGroup );
    pixmap.load("pixmap/circle.pnm");
    btmp->setPixmap(pixmap) ;
    btmp->setGeometry( 0, 15, 30, 30 );
  btmp = new QPushButton( "draw", genCurveGroup );
    btmp->setGeometry(30,15,30,30);
    pixmap.load("pixmap/curve.pnm");
    btmp->setPixmap(pixmap);
    */
  /*
  knotGroup = new QButtonGroup("Knot operations",this) ;
  knotGroup->setMinimumSize( 120, 50 );
  knotGroup->setFrameStyle( NoFrame );
  connect( knotGroup, SIGNAL(clicked(int)), SLOT(knotOp(int)) );
  layout->addWidget( knotGroup );
  btmp = new QPushButton("view",knotGroup) ;
  btmp->setGeometry(0,15,30,30) ;
  btmp->setToggleButton(TRUE) ;
  btmp = new QPushButton("insert",knotGroup) ;
  btmp->setGeometry(30,15,30,30) ;
  btmp = new QPushButton("delete",knotGroup) ;
  btmp->setGeometry(60,15,30,30) ;

  //New group
  paramGroup = new QButtonGroup( "Parametric info", this );
  paramGroup->setMinimumSize( 120, 50 );
  paramGroup->setFrameStyle( NoFrame );
  //connect(paramGroup,SIGNAL(clicked(int)),SLOT(paramOp(int)) );
  layout->addWidget( paramGroup );
  QPushButton *selectUVBtn = new QPushButton("Select",paramGroup) ;
  QPushButton *editUVBtn = new QPushButton("Edit",paramGroup) ;
  QPushButton *selUBtn = new QPushButton("u",paramGroup) ;
  QPushButton *selVBtn = new QPushButton("v",paramGroup) ;
  QPushButton *setUVBtn = new QPushButton("Set",paramGroup) ;

  selectUVBtn->setGeometry(0,15,40,15) ;
  editUVBtn->setGeometry(0,30,40,15) ;
  selUBtn->setGeometry(40,15,15,15) ;
  selVBtn->setGeometry(40,30,15,15) ;
  setUVBtn->setGeometry(65,15,30,30) ;

  selectUVBtn->setToggleButton(TRUE) ;
  editUVBtn->setToggleButton(TRUE) ;
  selUBtn->setToggleButton(TRUE) ;
  selVBtn->setToggleButton(TRUE) ;
  selUBtn->setOn(TRUE) ;
  selVBtn->setOn(TRUE) ;
  useU = 1 ;
  useV = 1 ;
  */
  /*
  uLabel = new QLabel(paramGroup) ;
  vLabel = new QLabel(paramGroup) ;
  uLabel->setGeometry(62,15,38,15) ;
  vLabel->setGeometry(62,30,38,15) ;
  setU(0) ;
  setV(0) ;
  */
  //connect(nurbsO,SIGNAL(newU(float)),SLOT(setU(float)));
  //connect(nurbsO,SIGNAL(newV(float)),SLOT(setV(float)));


  layout->activate();

  //------------make some help windows---------------
  /*
  interpWindow = new CurveInterpWidget;
  connect( interpWindow, SIGNAL(newNurbs(G3DObject*)),
	   model, SLOT(setNewObject(G3DObject*)) );
	   */
  circleWindow = new GenerateCircleWidget;
  connect( circleWindow, SIGNAL(newNurbs(G3DObject*)),
           model, SLOT(setNewObject(G3DObject*)) );
  skinS = new SkinSurfWidget;
  surfOfR = new SurfOfRevWidget;
  sweepS = new SweptSurfWidget;
  extrudeS = new ExtrudeSurfWidget;

  /*
  prefW = new PreferenceWidget ;
  connect( prefW, SIGNAL(changeSurfaceView(int)), SLOT(setSurfaceView(int)) );
  setParam = new SetParamWidget ;
  connect(setParam,SIGNAL(newUV(float,float)),SLOT(setUV(float,float))) ;

  */
}


ToolboxWidget::~ToolboxWidget()
{
  delete circleWindow;
  delete extrudeS;
  //delete interpWindow;
  delete skinS;
  delete surfOfR;
  delete sweepS;
}


/* TODO
void NurbsWidget::adjustEditBtn(NurbsEditMode mode){
  if(presentMode != mode){
    switch(presentMode){
    case edit_location: editTransformation->setOn(FALSE) ; break ;
    case edit_scale: editScale->setOn(FALSE) ; break ;
    case edit_cp: editCP->setOn(FALSE) ; break ;
    case edit_knots: nurbsO->setDisplayKnots(0) ;
      viewKnotBtn->setOn(FALSE) ; break ;
    case edit_param: selectUVBtn->setOn(FALSE) ; break ;
    case edit_point: editUVBtn->setOn(FALSE) ; break ;
      break ;
    default:
       ;
    }
    switch(mode){
    case edit_location: editTransformation->setOn(TRUE) ; break ;
    case edit_scale: editScale->setOn(TRUE) ; break ;
    case edit_cp: editCP->setOn(TRUE) ; break ;
    case edit_knots: viewKnotBtn->setOn(TRUE) ; break ;
    case edit_param: selectUVBtn->setOn(TRUE) ; break ;
    case edit_point: editUVBtn->setOn(TRUE) ; break ;
      break ;
    default:
       ;
    }
  }
  presentMode = mode ;
}
*/

void ToolboxWidget::affectMode( int )
{
  /* TODO
  int affect_flag = 0 ;
  switch(a)
    {
    case 0:
      affect_flag = PLNURBS_FLAGS_AFFECT_ACTIVE_ONLY ; break ;
    case 1:
      affect_flag = PLNURBS_FLAGS_AFFECT_SELECTED_ONLY ; break ;
    case 2:
      affect_flag = PLNURBS_FLAGS_AFFECT_ALL ; break ;
    default:
      ;
    }
    */
  //nurbsO->setAffectFlag(affect_flag) ;
  //nurbsO->setTransformFlag(affect_flag) ;
  //nurbsO->updateDisplayFlags() ;
  
  //nurbsO->updateGL() ;
}

void ToolboxWidget::drawCurve()
{
  //interpWindow->show() ;
}

void ToolboxWidget::editMode(int a)
{
  /*
  NurbsEditMode current_mode = nurbsO->editMode() ;
  switch(a)
    {
    case 1: break ;
    case 0: 
      if(current_mode == edit_location)
	nurbsO->setEditMode(no_edit) ; 
      else
	nurbsO->setEditMode(edit_location) ; 
      break ;
    case 2: 
      if(current_mode == edit_cp)
	nurbsO->setEditMode(no_edit) ; 
      else
	nurbsO->setEditMode(edit_cp) ; 
      break ;
    case 3: 
      if(current_mode == edit_scale)
	nurbsO->setEditMode(no_edit) ; 
      else
	nurbsO->setEditMode(edit_scale) ; 
      break ;
    default:
      ;
    }
  nurbsO->updateGL() ;
  */
}


void ToolboxWidget::genCurve(int c)
{
  switch(c)
    {
    case 0:
      circleWindow->show();
      break;
    case 1:
      {
	ME2DShaper *tmp = (ME2DShaper*)(Kernel->getModule( MODULE_SHAPER ));
	if( tmp )
	  {
	    G3DObject *obj = tmp ->getObject();
	    if( obj )
	      _model->setNewObject( obj );
	  }
	break;
      }
    }
}

void
ToolboxWidget::genSurf(int s)
{
  switch(s)
    {
    case 0:
      surfOfR->show( _model->getWorld() );  break;
    case 1:
      skinS->show( _model->getWorld() );    break;
    case 2:
      sweepS->show( _model->getWorld() );   break;
    case 3:
      extrudeS->show( _model->getWorld() ); break;
    }
}

void
ToolboxWidget::knotOp(int o)
{
  /*
  switch(o)
    {
    case 0: 
      if(nurbsO->editMode() == edit_knots)
	{
	  nurbsO->setDisplayKnots(0) ; 
	  nurbsO->setEditMode(no_edit) ;
	}
      else
	{
	  nurbsO->setDisplayKnots(1) ; 
	  nurbsO->setEditMode(edit_knots) ; 
	}
      break ;
    case 1:
      nurbsO->insertKnot(paramU,paramV,useU,useV) ;  //TODO no break???
    case 2:
      nurbsO->deleteKnot() ;
    }
  nurbsO->updateDisplayFlags() ;
  nurbsO->updateGL() ;
  */
}

/*
int ToolboxWidget::load(const char* filename)
{
  if(!nurbsO->load(filename,1)){  
    cerr << "Couldn't load nurbs objects from " << filename << endl ;
    return 0 ;
  }
  return 1 ;
}
*/


void
ToolboxWidget::makeBtnGroup( QButtonGroup *btnGroup, int num, char **names )
{
  QPushButton *btmp;
  QPixmap pixmap;
  char    pname[40];
  int     x = 0, y = 15;

  for( int i = 0; i < num; i++ )
    {
      btmp = new QPushButton( *names++, btnGroup );
      if( *names )
	{
	  sprintf( pname, "pixmap/%s.pnm", *names++ );
	  pixmap.load( pname );
	  btmp->setPixmap( pixmap );
	}
      btmp->setGeometry( x, y, 30, 30 );
      if( i == 0 )
	btmp->setOn(TRUE);
      if( x < 90 )
	x += 30;
      else
	{
	  x = 0; y += 30;
	}
      btmp->setToggleButton( TRUE );
    }
}

/*
void NurbsWidget::paramOp(int o){
  switch(o){
  case 0: 
    if(nurbsO->editMode() == edit_param)
      nurbsO->setEditMode(no_edit) ; 
    else
      nurbsO->setEditMode(edit_param) ; 
    nurbsO->updateGL() ;
    break ;
  case 1: 
    if(nurbsO->editMode() == edit_point)
      nurbsO->setEditMode(no_edit) ; 
    else
      nurbsO->setEditMode(edit_point) ; 
    nurbsO->updateGL() ;
    break ;
  case 2: useU = !useU; break ;
  case 3: useV = !useV; break ;
  case 4: setParam->show() ; break ;// set the parameters value
  }
}
*/

void ToolboxWidget::preferenceWindow()
{
  //prefW->show() ;
}


void ToolboxWidget::selectMode(int a)
{
  /*
  switch(a)
    {
    case 0: nurbsO->selectAll() ; break ;
    case 1: nurbsO->selectNone() ; break ;
    }
  nurbsO->updateGL() ;
  */
}


void ToolboxWidget::setU(float u)
{
  paramU = u ;
  char txt[10] ;
  sprintf(txt,"%2.4f", paramU) ;
  uLabel->setText(txt) ;
}

void ToolboxWidget::setV(float v){
  paramV = v ;
  char txt[10] ;
  sprintf(txt,"%2.4f",paramV) ;
  vLabel->setText(txt) ;
}

void ToolboxWidget::setUV(float u, float v)
{
  //nurbsO->setUV(u,v) ;
}


void ToolboxWidget::setSurfaceView(int a)
{
  /*TODO
  switch(a)
    {
    case 0:
      G3DNurbsDisplayMode = PLNURBS_DISPLAY_NORMAL ; break ;
    case 1:
      G3DNurbsDisplayMode = PLNURBS_DISPLAY_ISOCURVES ; break ;
    }
    */
}

void ToolboxWidget::viewMode(int a)
{
  switch(a)
    {
    case 0:
      //nurbsO->setDisplayObject(!nurbsO->displayObject()) ;
      break ;
      //case 1: nurbsO->setDisplayPoints(!nurbsO->displayPoints()) ;  break ;
      //case 2: nurbsO->setDisplayPolygon(!nurbsO->displayPolygon());   break ;
    case 3:
      emit info(); break;
      //_model->info(); break; // weight
      //case 4: nurbsO->setDisplayAxis(!nurbsO->displayAxis()) ; break ;
      //case 5: nurbsO->setDisplayCenter(!nurbsO->displayCenter()) ; break ;
      //case 6: nurbsO->resetView() ; break ;
      //case 7: nurbsO->setDisplayBBox(!nurbsO->displayBBox()) ; break ;
    default:
      ;
    }
  //nurbsO->updateDisplayFlags();
  //nurbsO->updateGL();
}

/*
int ToolboxWidget::save(const char* filename,int saveMode){
  return nurbsO->save(filename,saveMode) ;
}
*/



/*
PreferenceWidget::PreferenceWidget( QWidget *parent, const char *name )
  : QWidget(parent,name)
{
  initMetaObject() ;
  setMinimumSize(300,200) ;
  setMaximumSize(300,200) ;
  hide() ;

  QLabel *surfaceLabel = new QLabel( "View surface as", this );

  surfaceLabel->move(10,30) ;

  QButtonGroup *surfaceView = new QButtonGroup(this) ;

  surfaceView->setFrameStyle(QFrame::NoFrame) ;
  surfaceView->setGeometry(110,30,90,60) ;

  QRadioButton *normalS = new QRadioButton("normal",surfaceView) ;
  QRadioButton *isocurveS = new QRadioButton("iso curves",surfaceView) ;

  normalS->setGeometry(5,5,80,15);
  isocurveS->setGeometry(5,20,80,15) ;

  PlNurbsDisplayMode = PLNURBS_DISPLAY_ISOCURVES ;
  isocurveS->setChecked(TRUE) ;

  QPushButton *okButton = new QPushButton("Set",this) ;
  okButton->setGeometry(100,170,50,25) ;

  //TODO connect with SIGNAL?
  connect(surfaceView,SIGNAL(clicked(int)),SLOT(newSurfaceView(int))) ;
  connect(okButton,SIGNAL(clicked()), SLOT(hideWindow())) ;
}

void PreferenceWidget::hideWindow()
{
  hide() ;
}

void PreferenceWidget::newSurfaceView(int a)
{
  emit changeSurfaceView(a) ;
}


*/
