/*
 * Project : "MindsEye, render queue"
 * Copyright (c) 1998. All Rights Reserved.
 * Author(s) M. Kuball
 *
 * $Id: renderDlg.cpp,v 1.3 1998/05/26 20:28:02 source Exp $
 *
 *
 * COPYRIGHT
 * =========
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 * 
 * OVERVIEW
 * ========
 *
 */

#include "include/config.h"
#include <unistd.h>

#include "pov_queue.h"
#include "renderjob.h"
#include "io/io_dialogs.h"
#include "io/io_base.h"

bool saveScene( MWorldObject* );


MEPovrayQueue::~MEPovrayQueue( void )
{
  delete[] command_;
}


bool
MEPovrayQueue::install( )
{
  //search for programm and include files
  //ret true if found
  //create ini-file with queue defaults
}

/*
  +UF               Turn field rendering on
  -UF               Turn field rendering off
  +UO               Set odd field flag on
  -UO               Set odd field flag off
By default, field rendering is not used. Setting Field_Render  =on  or  using
+UF will cause alternate frames in an animation to be only the  even  or  odd
fields of an animation. By default,  the  first  frame  is  the  even  field,
followed by the odd field. You can have POV-Ray render the odd field first by
specifying Odd_Field =on, or by using the +UO switch.

  +Fxn               Sets file output on; sets format 'x', depth 'n'
  +FN New PNG (portable network graphics) format
PNG allows you to  optionally  specify  the  output  bit
depth from 5 to 16 bits for each of the red, green, and blue  colors,  giving
from 15 to 48 bits of color information per pixel. The default  output  depth
for all formats is 8 bits/color
*/

bool
MEPovrayQueue::prepareJob( MERenderJob *job )
{
  if( job->getStartTime() != job->getStopTime() )
    job->copyScene();
  //TODO else
  //return( saveScene(job->getWorld()) );
  return( true );
}

void
MEPovrayQueue::renderFrame( MERenderJob *job )
{
  if( command_ == NULL )
    exit( 1 );

  if( saveScene(job->getWorld()) )
    {
      char w[16], h[16];
      sprintf( w, "+W%i", job->getWidth() );
      sprintf( h, "+H%i", job->getHeight() );
      execl( "/usr/bin/nice", "/usr/bin/nice", command_,
	     "+I/tmp/tmp.pov", w, h, "+Ome.ppm", NULL );
    }
  else
    exit( 1 );

/* e.g. RIB
  while( frame <  )
    {
      if( *host )
        fork
        execute( "rlogin" );
       frame++;
   }
  execute();
*/
}

bool
saveScene( MWorldObject *world )
{
  IO_BaseClass *filter = MEIoModule::getIoFilter( IO_FORMAT_POV, NULL );
  if( filter )
    filter->file_save( world, "/tmp/tmp.pov", 0 );
  else
    return( false );
  return( true );
}

void
MEPovrayQueue::setCommand( const char *cmd )
{
  delete[] command_;
  command_ = new char[ strlen(cmd)+1 ];
  strcpy( command_, cmd );
}

void
MEPovrayQueue::showOptionDialog( void )
{
  //TODO
}


//==================================END===================================
