/*
 * Project : "MindsEye"
 * Copyright (c) 1997. All Rights Reserved.
 * Atuhor(s): M. Kuball
 *
 * $Id: scene_dlg.cpp,v 1.12 1998/07/23 20:16:20 source Exp $
 *
 *
 * COPYRIGHT
 * ========= 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 * 
 * OVERVIEW
 * ========
 * A dialog  for viewing and editing the scene graph. It's based on
 * ktreelisttest from Keith Brown.
 *
 */


//#include <qchkbox.h>
//#include <qgrpbox.h>
#include <qlined.h>
#include <qpushbt.h>
#include <qlayout.h>
#include <qlabel.h>
#include <qpopmenu.h>
#include <qcombo.h>

#include "3dlib/shape.h"
#include "3dlib/scene.h"
#include "3dlib/camera.h"
#include "3dlib/aggregate.h"
#include "3dlib/iterator.h"
#include "kernel/world_obj.h"
#include "kernel/m_core.h"

#include "mtreelist.h"
#include "treeitem.h"
#include "scene_dlg.h"
#include "scene_iterator.h"


MSceneGraphDlg::MSceneGraphDlg( int module_id, int load_t ) :
  MEModule(module_id, load_t, "MSceneGraphDialog" )
{
  setMinimumSize( 200, 300 );

  _selectedItem = NULL;

  QVBoxLayout *gm    = new QVBoxLayout( this, 6 );
  QHBoxLayout *gm_00 = new QHBoxLayout;
  QHBoxLayout *gm_01 = new QHBoxLayout;
  QHBoxLayout *gm_02 = new QHBoxLayout;
  QHBoxLayout *gm_03 = new QHBoxLayout;
  gm->addLayout( gm_03, 3 );
  gm->addLayout( gm_00,100 );
  gm->addLayout( gm_01, 3 );
  gm->addLayout( gm_02, 3 );
  //gm->addSpacing( 5 );

  QLabel *lb = new QLabel( "Scene:", this );
    lb->setAlignment( AlignHCenter|AlignVCenter );
    lb->setFixedSize( lb->sizeHint() );
    gm_03->addWidget( lb, 1 );
  worldSelector_ = new QComboBox( this );
    worldSelector_->setMinimumSize( 40, 20 );
    connect( worldSelector_, SIGNAL(highlighted(int)), SLOT(setWorld(int)) );
    gm_03->addWidget( worldSelector_, 100 );

  _outlineTree = new MTreeList( makePopup(), this );
    _outlineTree->setFrameStyle( 51 );
    _outlineTree->setLineWidth( 2 );
    _outlineTree->setMinimumSize( 100, 200 );
    //_outlineTree->setExpandLevel( 1 );
    //TODO _outlineTree->setTreeDrawing(FALSE);
    //_outlineTree->setExpandButtonDrawing(FALSE);
    connect( _outlineTree, SIGNAL(highlighted(int)), SLOT(highlighted(int)) );
    connect( _outlineTree, SIGNAL(selected(int)), SLOT(selected(int)) );
    connect( _outlineTree, SIGNAL(itemDown()), SLOT(itemDown()) );
    gm_00->addWidget( _outlineTree, 100 );

  QLabel *label = new QLabel("Name:", this);
    label->setAlignment( AlignHCenter|AlignVCenter );
    label->setFixedSize( label->sizeHint() );
    gm_01->addWidget( label, 1 );
  _nameEdit = new QLineEdit( this ); //TODO use this for value editing
    _nameEdit->setMinimumSize( 40, 20 );
    gm_01->addWidget( _nameEdit, 100 );
  QPushButton *pbtn = new QPushButton( "set", this );
    pbtn->setMinimumSize( 28, 20 );
    gm_01->addWidget( pbtn, 1 );
    connect( pbtn, SIGNAL(clicked()), SLOT(changeName()) );
  QPushButton *fbtn = new QPushButton( "find", this );
    fbtn->setMinimumSize( 30, 20 );
    gm_01->addWidget( fbtn, 1 );
    //connect( pbtn, SIGNAL(clicked()), SLOT(changeName()) );

  //gm_01->addSpacing( 5 );

  pbtn = new QPushButton( "Close", this );
  pbtn->setMinimumSize( 65, 22 );
  connect( pbtn, SIGNAL(clicked()), this, SLOT(closeIt()) );

  gm_02->addWidget( pbtn, 0 );
  gm_02->addStretch( 100 );

  gm->activate ();
  resize( 200, 300 );

  world_ = NULL;
  setWorldList();
  connect( Kernel, SIGNAL(worldListChanged()), SLOT(setWorldList()) );
}


void
MSceneGraphDlg::addSubItem()
{
  //TODO only add as child if a group otherwise add to parent
  //get currentItem
  //go up while not group

  MTreeListItem *tmp = (MTreeListItem*)(_outlineTree->getCurrentItem());
  /*
  if( tmp && tmp->isGroup() )
    {
      parent = tmp->getGroup();
      parent->append( new MShapeItem(new G3DGroup) );
    }
  _nameEdit->setText("");
  */
}


void MSceneGraphDlg::changeName()
{
  if( _nameEdit->text() && _selectedItem )
    {
      G3DObject *obj = _selectedItem->getObject();
      if( obj )
	{
	  obj->setName( _nameEdit->text() );
	  QString str( obj->getTypeName() );
	  str.append( ": " );
	  str.append( _nameEdit->text() );
	  _outlineTree->changeItem( str, 0, _outlineTree->currentItem() );
	}

       //MTreeListItem *tmp = (MTreeListItem*)( _outlineTree->itemAt(i) );
       //if( tmp != _selectedItem )
      //item = _outlineTree->currentItem();
      //item->setName();
    }
}

void
MSceneGraphDlg::closeIt()
{
  close( );
}

void
MSceneGraphDlg::collapse()
{
  int expansion = _outlineTree->expandLevel();
  if(expansion > 0) {
    _outlineTree->setExpandLevel(--expansion);
    char num[8];
    sprintf(num, "%d", expansion);
    //expandLevelLabel->setText(num);
  }
}

void
MSceneGraphDlg::expand()
{
  int expansion = _outlineTree->expandLevel();
  _outlineTree->setExpandLevel(++expansion);
  char num[8];
  sprintf(num, "%d", expansion);
  //expandLevelLabel->setText(num);
}
 
void
MSceneGraphDlg::highlighted( int i )
{
  MTreeListItem *tmp = (MTreeListItem*)( _outlineTree->itemAt(i) );
  if( tmp != _selectedItem )
    {
      G3DObject *obj = tmp->getObject();
      if( obj )
	{
	  _selectedItem = tmp;
	  _nameEdit->setText( obj->getName() );
	}
      else
	_selectedItem = NULL;
    }
}

void
MSceneGraphDlg::itemDown()
{
  int index = _outlineTree->currentItem();
  if( index >= 0 )
    _outlineTree->lowerItem(index);
}

// indents the current item one level, splitting it into a new branch
void MSceneGraphDlg::itemIn()
{
  _outlineTree->split( _outlineTree->currentItem() );
}

// outdents the current item one level, making it a sibling of its parent
void MSceneGraphDlg::itemOut()
{
  _outlineTree->join( _outlineTree->currentItem() );
}

// moves the current item up one slot in its branch
void MSceneGraphDlg::itemUp()
{
  //TODO reorder the objects in the scene graph
  int index = _outlineTree->currentItem();
  if( index >= 0 )
    _outlineTree->raiseItem(index);
}

QPopupMenu*
MSceneGraphDlg::makePopup( void )
{
  QPopupMenu *popup = new QPopupMenu;
  CHECK_PTR( popup );

  popup->insertItem( "delete",  this, SLOT(removeItem()));
  popup->insertItem( "add",     this, SLOT(addSubItem()));
  popup->insertItem( "inspect", this, SLOT(showProperties()));
  popup->insertSeparator();
  popup->insertItem( "set Focus", this, SLOT(setFocus()) );
  popup->insertItem( "show" );
  return( popup );
}

void
MSceneGraphDlg::objectSelected(G3DObject*)
{
  cout << "objectSelected called" << endl;
}

// removes the current item from the tree
void
MSceneGraphDlg::removeItem()
{
  MTreeListItem *tmp = (MTreeListItem*)(_outlineTree->getCurrentItem());
  if( tmp )
    {
      tmp->remove( world_ );
      _outlineTree->removeItem( _outlineTree->currentItem() );
    }
}

void
MSceneGraphDlg::selected( int i )
{
  MTreeListItem *tmp = (MTreeListItem*)( _outlineTree->itemAt(i) );
  if( tmp )
    {
      //if( _selectedItem )
	//_selectedItem->selectOff( _world );
      tmp->selectOn( world_ );
      _selectedItem = tmp;  //TODO
    }
}

void
MSceneGraphDlg::setFocus( )
{
  /*
  MTreeListItem *tmp = (MTreeListItem*)(_outlineTree->currentItem());
  tmp->object->getPosition();
  _world->getActiveCamera()
  cam->setFocus( pos );
  */
}

void
MSceneGraphDlg::setWorldList( )
{
  MWorldList &worlds = Kernel->getWorldList();
  worldSelector_->clear();

  bool gotIt = false;
  MWorldList::iterator wld = worlds.begin();
  while( wld != worlds.end() )
    {
      if( *wld == world_ )
	gotIt = true;
      worldSelector_->insertItem( (*(wld++))->getName() );
    }
  if( !gotIt )
    setWorld( 0 );
}

void
MSceneGraphDlg::setWorld( int i )
{
  if( world_ )
    disconnect( world_ );

  MWorldList &worlds = Kernel->getWorldList();
  world_ = worlds[i];
  scene_ = world_->getScene();

  connect( world_, SIGNAL(objectSelected(G3DObject*)),
	   SLOT(objectSelected(G3DObject*)) );
  connect( world_, SIGNAL(sceneChanged(G3DGroup*)),
	   SLOT(updateScene(G3DGroup*)) );

  //TODO  ?? _worldSelector->repaint();
  updateScene( NULL );
}

void
MSceneGraphDlg::showProperties( )
{
//get Property dialog from Kernel
}

// toggles expand button drawing on/off
void MSceneGraphDlg::toggleExpandButtons(bool on)
{
  _outlineTree->setExpandButtonDrawing( on );
}

// toggles tree branch drawing on/off
void MSceneGraphDlg::toggleTree( bool on )
{
  _outlineTree->setTreeDrawing( on );
}

void
MSceneGraphDlg::updateScene( G3DGroup* )
{
  _outlineTree->setAutoUpdate( FALSE );
  _outlineTree->clear();

  MShapeItem *tmp = new MShapeItem( scene_ );
  _outlineTree->insertItem( tmp, -1 );
  trace( scene_, tmp );

  _outlineTree->setAutoUpdate( TRUE );
  repaint();
}


void
trace( G3DShape *group, MShapeItem *start )
{
  //MShapeItem *tmpa = new MShapeItem( "Attributes" );
  //start->appendChild( tmpa );

  if( ((G3DGroup*)group)->numberOfLights() > 0 )
    {
      MShapeItem *tmpl = new MShapeItem( "Lights" );
      start->appendChild( tmpl );
      InfoIteratorL lit( tmpl );
      ((G3DGroup*)group)->doWithLights( lit );
    }

  /*TODO ?
  MShapeItem *tmps = new MShapeItem( "Shapes" );
  start->appendChild( tmps );
  */
  InfoIteratorS sit( start );
  ((G3DGroup*)group)->doWithShapes( sit );
}


//==================================END==================================
