/*
 * Project : "MindsEye"
 * Copyright (c) 1997. All Rights Reserved.
 * Atuhor(s): M. Kuball
 *
 * $Id: treeitem.cpp,v 1.10 1998/07/23 20:16:22 source Exp $
 *
 *
 * COPYRIGHT
 * ========= 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 * 
 * OVERVIEW
 * ========
 * A TreeList Item for viewing and editing the scene graph. It's based on
 * ktreelist from Keith Brown.
 *
 */

#include <stdio.h>
#include <qstring.h>

#include "3dlib/shape.h"
#include "3dlib/light.h"
#include "3dlib/scene.h"
#include "3dlib/camera.h"
#include "3dlib/aggregate.h"
#include "kernel/world_obj.h"
#include "treeitem.h"



MLightItem::MLightItem ( G3DLight *light )
{
  _gLight = light;
  if( light->getName() )
    {
      QString str( light->getTypeName() );
      str.append( ": " );
      str.append( light->getName() );
      setText( str );
    }
  else
    setText( light->getTypeName() );
}

G3DObject*
MLightItem::getObject( void )
{
  return( _gLight );
}

void
MLightItem::remove( MWorldObject *world )
{
  world->deleteObject( _gLight );
}

void
MLightItem::selectOff(  MWorldObject* )
{
}

void
MLightItem::selectOn(  MWorldObject* )
{
  //TODO
}


//---------------------------------------------------------------------


MShapeItem::MShapeItem ( G3DShape *obj )
{
  _gObject = obj;
  if( obj->getName() )
    {
      QString str( obj->getTypeName() );
      str.append( ": " );
      str.append( obj->getName() );
      setText( str );
    }
  else
    setText( obj->getTypeName() );
}

MShapeItem::MShapeItem ( const char *text )
{
  _gObject = NULL;
  setText( text );
}

G3DObject*
MShapeItem::getObject( void )
{
  return( _gObject );
}

void
MShapeItem::remove( MWorldObject *world )
{
  world->deleteObject( _gObject );
}

void
MShapeItem::selectOff(  MWorldObject* )
{
}

void
MShapeItem::selectOn( MWorldObject *world )
{
  world->selectObject( _gObject );
}


//======================================END==================================
