/*
 * Project : "MindsEye"
 * Copyright (d) 1996, 1997. All Rights Reserved.
 * Author(s): M. van Velsen, M. Kuball
 *
 * $Id: treeitem.h,v 1.6 1998/07/23 20:16:22 source Exp $
 * 
 * COPYRIGHT
 * =========
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * OVERVIEW
 * ========
 *
 *
 */


#ifndef __DATA_TREE_H
#define __DATA_TREE_H

#include <stdio.h>
#include <qwindow.h>
#include <qpixmap.h>
#include <qpainter.h>
#include <qapp.h>

#include "ktreelist.h"

//TODO #include "include/m_vars.h"


//>------ define node types
#define NODE_CHILD        0
#define NODE_NEXT         1

//>------ size defines
#define NODE_WIDTH       33
#define NODE_OFFSET      22
#define NODE_HEIGHT      15

#define X_BORDER          5
#define Y_BORDER          5

#define Y_DISTANCE        5

//>------ defines if the node should display all that lies beneath
#define NODE_CONTRACTED   0
#define NODE_EXPANDED     1

//>------ will be removed later
//#define TRUE              0
//#define FALSE             1

#define ND_PLUS           0
#define ND_MIN            1
#define ND_NUL            2 

//>------ node types

#define NODE_CAMERA       3
#define NODE_LIGHT        4
#define NODE_OBJECT       5  // mesh
#define NODE_OBJECT_PRIM  6  // primitive
#define NODE_SCENE        7
#define NODE_PATH         8  
#define NODE_SOUND        9
#define NODE_ANIM        10 
#define NODE_MUSIC       11
#define NODE_MATERIAL    12

class G3DShape;
class G3DLight;
class MWorldObject;

/**Virtual base Class for the tree list items used in MindsEye.
  Te class is used to define a common protocol for all items.
  */
class MTreeListItem : public KTreeListItem
{
public:
  virtual G3DObject* getObject( void ) = 0;
  virtual void remove( MWorldObject* ) = 0;
  virtual void selectOff( MWorldObject* ) = 0;
  virtual void selectOn( MWorldObject* )  = 0;
};


/**A node class representing a G3DLight.
 */
class MLightItem : public MTreeListItem
{
public:
  ///Create a node representing the given G3DObject.
  MLightItem( G3DLight* );

  virtual G3DObject* getObject( void );
  virtual void remove( MWorldObject* );
  ///Create a dummy node with text.
  virtual void selectOff( MWorldObject* );
  virtual void selectOn( MWorldObject* );

private:
  G3DLight  *_gLight;
};


/**A node class representing either a G3DObject.
 */
class MShapeItem : public MTreeListItem
{
public:
  ///Create a node representing the given G3DObject.
  MShapeItem( G3DShape* );

  virtual G3DObject* getObject( void );
  virtual void remove( MWorldObject* );
  ///Create a dummy node with text.
  MShapeItem( const char *text );
  virtual void selectOff( MWorldObject* );
  virtual void selectOn( MWorldObject* );

  //edit();
  //remove();
  //update();

private:
  char _nodeType;      // object / path / sound / anim;
  //union {
  G3DShape *_gObject;
  //G3DLight  *_gLight;
  //};
};

//GeometryNode
//AttributeNode

#endif 
