/*
 * Project : MindsEye
 * Copyright (c) 1996, 1997, 1998
 * Author(s): M. Kuball, F. Meier
 * 
 * $Id: m_viewgl.cpp,v 1.29 1998/07/23 20:16:23 source Exp $
 *
 *
 * COPYRIGHT
 * =========
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *   shapeList_.append( obj );

 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * OVERVIEW
 * ========
 *
 */

#ifdef HAVE_CONFIG_H
#include "include/config.h"
#endif	 // HAVE_CONFIG_H

#include <qkeycode.h>
#include <qevent.h>

#include "3dlib/camera.h"
#include "cam_controler.h"


void 
MECameraControler::cameraInteraction( QMouseEvent *e, int mbtn )
{
  if( camera_->isParallelMode() )
    {
      if( mbtn == LeftButton )
	tuckCamera( e );
      else
	fovCamera( e );
    }
  else
    {
      if( mbtn == LeftButton )       // orbit!
	orbitCamera( e );
      else   // tuck!
	tuckCamera( e );
    }
}

void
MECameraControler::dollyCamera(QMouseEvent *e)
{
  int yDiff = e->y() - _oldy;

  if( yDiff != 0 )
    {
      camera_->dolly( yDiff * 0.01 );
      _oldx = e->x(); _oldy = e->y();
    }
}

void 
MECameraControler::fovCamera( QMouseEvent *e )
{
  int yDiff = e->y() - _oldy;

  if( yDiff != 0)
    { 
      if( camera_->isParallelMode() )
	camera_->setParallelScale( _old_value + yDiff*0.05 );
      else
	// add some amount to original fov value
	camera_->setFOV( _old_value + yDiff * 0.5 );
    }
}

void
MECameraControler::move( QMouseEvent *e, int mbtn )
{
  if( camera_ )
    {
      switch( mode_ )
	{
	case AllInOne:
	  cameraInteraction( e, mbtn );
	  break;
	case Dolly:
	  dollyCamera(e);
	  break;
	case Orbit:
	  orbitCamera(e);
	  break;
	case Tuck:
	  tuckCamera(e);
	  break;
	case Zoom:
	  fovCamera(e);
	  break;
	case Roll:
	  rollCamera(e);
	  break;
	case Rotate:
	  rotateCamera(e);
	  break;
	}
    }
}

void 
MECameraControler::orbitCamera( QMouseEvent *e )
{
  int    xDiff, yDiff;
  double x, y;

  xDiff = e->x() - _oldx;
  yDiff = e->y() - _oldy;

  if( xDiff != 0 )
    {
      x = xDiff * 0.5; // 0.5 degrees per pixel
      camera_->azimuth( deg2rad(x) );
    }
  if( yDiff != 0 )
    {
      y = yDiff * 0.5;
      camera_->elevation( deg2rad(y) );
    }
  _oldx = e->x();
  _oldy = e->y();
}

void 
MECameraControler::rollCamera (QMouseEvent *e )
{
  int xDiff = e->x() - _oldx;
  if( xDiff != 0 )
    { 
      double x = -xDiff * 0.5;
      camera_->roll( deg2rad(x) );
      _oldx = e->x(); _oldy = e->y();
    }
}

void 
MECameraControler::rotateCamera( QMouseEvent *e )
{
  bool   doUpdate=FALSE;
  double x, y;
  int    xDiff, yDiff;

  xDiff = e->x() - _oldx;
  yDiff = e->y() - _oldy;

  if (xDiff != 0) {
    x = xDiff * 0.5;
    camera_->yaw(deg2rad(x));
    doUpdate=TRUE;
  }
  if (yDiff != 0) {	 
    y = yDiff * 0.5;
    camera_->pitch(deg2rad(y));
    doUpdate=TRUE;
  }
  if (doUpdate) {
    _oldx = e->x(); _oldy = e->y();
  }
}

void
MECameraControler::setMode( int m )
{
  mode_ = m;
}

void
MECameraControler::start( G3DCamera *cam, QMouseEvent *e )
{
  _oldx = e->x();
  _oldy = e->y();

  camera_ = cam;
  if( cam->isParallelMode() )
    _old_value = camera_->getParallelScale();
  else
    _old_value = camera_->getFOV();
}

void
MECameraControler::tuckCamera( QMouseEvent *e )
{
  double scale  = 1.0/50.0;
  if( camera_->isParallelMode() )
    {
      int xDiff = e->x() - _oldx;
      int yDiff = e->y() - _oldy;

      double xpos,ypos;
      camera_->getWindowCenter( xpos, ypos );
      xpos +=  xDiff * scale;
      ypos += -yDiff * scale;
      camera_->setWindowCenter( xpos, ypos );
    }
  else
    {
      double delx, dely = 0.0, delz = 0.0;

      delx = double( _oldx - e->x() ) * scale;
      if( e->state() & ShiftButton )
	delz = (double)(_oldy - e->y()) * scale;
      else
	dely = (double)(e->y() - _oldy) * scale;
      camera_->tuck( delx, dely, delz );
    }

  _oldx = e->x();
  _oldy = e->y();
}


//================================EMD====================================
