/*
 * Project : MindsEye
 * Copyright (c) 1996, 1997, 1998
 * Author(s): M. Kuball, M. van Velsen, F. Meier
 * 
 * $Id: m_viewgl.cpp,v 1.29 1998/07/23 20:16:23 source Exp $
 *
 *
 * COPYRIGHT
 * =========
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *   shapeList_.append( obj );

 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * OVERVIEW
 * ========
 * a widget which uses OpenGL to display a scene
 *   
 * MvV:
 *
 * Ok I've been playing with these sources to get a certain look ( hope
 * you don't mind Martin K ) I will add a color config dialog for the
 * backgr and gridline colors etc. 
 *
 * The basic sources were exellent because they showed me good way to
 * do 2d editing and scrolling without losing the notion of where you
 * are. This is done by the 3d lines which go into the Z direction. 
 * GREAT !!!   
 *
 */

#ifdef HAVE_CONFIG_H
#include "include/config.h"
#endif	 // HAVE_CONFIG_H
#include <stdio.h>

//include <stdlib.h>
#include <qkeycode.h>
#include <qpopmenu.h>

#include "include/cursors.h"
#include "3dlib/object.h"
#include "3dlib/color.h"
#include "3dlib/camera.h"
#include "3dlib/scene.h"
#include "3dlib/actions.h"
#include "kernel/world_obj.h"
#include "kernel/world_list.h"
#include "modeler/modeler.h"
#include "m_vmenu.h"
#include "m_vdef.h"
#include "m_viewp.h"
#include "tcontroler.h"
#include "cam_controler.h"
#include "grid.h"

//TODO ? GLuint MGLPort::selectBuffer[SELECTSIZE];


/*---------------------------------------------------------------------------*/

MGLPort::MGLPort( QCursor *new_hand, QPopupMenu *dpopup,
		  QWidget *parent, const char *name ) :
  QGLWidget(parent, name)
{
  setFocusPolicy( QWidget::ClickFocus ); // we accept keyboard focus

  world_    = NULL;
  scene_    = new G3DScene;
  camera_   = new G3DCamera;
  camera_->setName( "default camera" );

  _selectColor  = new G3DColor( 1, 0, 0 );
  _selected     = NULL;
  _active       = NULL;
  _defaultPopup = dpopup;
  tControler_   = NULL; //new MoveControler();
  hand_cursor   = new_hand;
  setCursor( *hand_cursor );

  display_type   = FLAT;
  see_grid       = TRUE;
  see_axes       = TRUE;

  opMode_       = ViewMode;
  dispListMode_ = DirectMode;
  camControler_ = new MECameraControler;
}

MGLPort::~MGLPort()
{
  delete _selectColor;
  //TODO delete hand_cursor?
}

/*---------------------------------------------------------------------------*/

void
MGLPort::centerCamera( void )
{
  camera_->setUVN( camera_->getPosition(), G3DPoint3d(0,0,0),
		   G3DVector3d(0,1,0) );
}

void
MGLPort::change_display (int new_dtype)
{
  display_type = new_dtype;
  updateGL();
}

void
MGLPort::drawAxes ( void )
{
  //TODO compile the stuff
  //axisId = glGenLists(1) ;
  //glNewList(axisId,GL_COMPILE);
  //glEndList();
  if( see_axes == TRUE )
    {
      //TODO cout << "axes" << endl;
      glLineWidth( 1 );
      glDisable( GL_LIGHTING );
      glBegin( GL_LINES );

      glColor3f( 1.0, 1.0, 1.0 );
      glVertex3f( -100, 0, 0 );
      glVertex3f( 0, 0, 0 );
      glColor3f( 1.0, 0.0, 0.0 );
      glVertex3f( 0, 0, 0 );
      glVertex3f( 100, 0, 0 );

      glColor3f( 0, 1.0, 0.0 );
      glVertex3f( 0, 0, 0 );
      glVertex3f( 0, 100, 0 );
      glColor3f( 1.0, 1.0, 1.0 );
      glVertex3f( 0, -100, 0 );
      glVertex3f( 0, 0, 0 );

      glColor3f( 0, 0.0, 1.0 );
      glVertex3f( 0, 0, 100 );
      glVertex3f( 0, 0, 0 );
      glColor3f( 1.0, 1.0, 1.0 );
      glVertex3f( 0, 0, 0 );
      glVertex3f( 0, 0, -100 );

      glEnd();
    }
}

//TODO make this more configurable
void
MGLPort::drawGrid( void )
{
  if( see_grid == TRUE )
    {
      float width = 20;
      G3DVector3d step1, step2;
      G3DPoint3d  x1, x2, y1, y2;

      switch( camera_->getViewMode() )
	{
	case G3DCamera::Perspective:
	case G3DCamera::Top:
	case G3DCamera::Bottom:
	  step1.setTo(1, 0, 0);  //TODO step?
	  step2.setTo(0, 0, 1);
	  x1.setTo( -width, 0,  width );
	  x2.setTo( -width, 0, -width );
	  y1.setTo( -width, 0, -width );
	  y2.setTo(  width, 0, -width );
	  break;
	case G3DCamera::Left:
	case G3DCamera::Right:
	  step1.setTo( 0, 1, 0 );
	  step2.setTo( 0, 0, 1 );
	  x1.setTo( 0, -width, width);
	  x2.setTo( 0, -width, -width);
	  y1.setTo( 0, -width, -width);
	  y2.setTo( 0,  width, -width);
	  break;
	case G3DCamera::Front:
	case G3DCamera::Back:
	  step1.setTo( 1, 0, 0 );
	  step2.setTo( 0, 1, 0 );
	  x1.setTo( -width, -width, 0 );
	  x2.setTo( -width,  width, 0 );
	  y1.setTo( -width, -width, 0 );
	  y2.setTo(  width, -width, 0 );
	  break;
	}
      glDisable( GL_LIGHTING );
      glColor3f ( 0.8, 0.8, 0.8 );
      glLineWidth( 1 );
      glBegin( GL_LINES );
      for( int i = 40; i >= 0; i-- )
	{
	  if( i != 20 )
            {
	      glVertex3dv( (const double*)x1 );
	      glVertex3dv( (const double*)x2 );
	      glVertex3dv( (const double*)y1 );
	      glVertex3dv( (const double*)y2 );
            }
	  x1 += step1; x2 += step1;
	  y1 += step2; y2 += step2;
	}
      glEnd();
    }
}

/*---------------------------------------------------------------------------*/

void
MGLPort::initPicking(QMouseEvent *e, GLsizei bs, GLuint buf[] )
{
  GLint         viewport[4];
  G3DMatrix     tM;

  initShader();
  glGetIntegerv(GL_VIEWPORT, viewport);

  glSelectBuffer( bs, buf );
  glRenderMode( GL_SELECT );
  glInitNames();

  glMatrixMode( GL_PROJECTION );
  glPushMatrix();

  glLoadIdentity();
  gluPickMatrix( (GLdouble)e->x(), 
		 (GLdouble)(viewport[3] - e->y()), 
		 4,4,            // pick rectangle in pixels
		 viewport);
  camera_->getProjectionMatrix( tM );
  glMultMatrixd( (const double *)tM );
  
  glMatrixMode( GL_MODELVIEW );
  glPushMatrix();
  camera_->getViewMatrix( tM ); 
  glLoadMatrixd( (const double *)tM );
}

G3DObject*
MGLPort::pickObject(QMouseEvent *e, G3DGroup *elements )
{
  const GLsizei buf_size = 200; 
  GLuint        buf[buf_size];

  initPicking( e, buf_size, buf );
  elements->select();
  glPopMatrix();

  glMatrixMode(GL_PROJECTION);
  glPopMatrix();
  glFlush();

  int hits = (int)glRenderMode(GL_RENDER);
  return examineHitBuffer( hits, buf, elements );
}

G3DObject*
MGLPort::pickSubObject(QMouseEvent *e, G3DObject *obj )
{
  const GLsizei buf_size = 100; 
  GLuint        buf[buf_size];

  initPicking( e, buf_size, buf );
  obj->selectSubObjects();
  glPopMatrix();

  glMatrixMode(GL_PROJECTION);
  glPopMatrix();
  glFlush();

  int hits = (int)glRenderMode(GL_RENDER);
  return examineHitBuffer( hits, buf, obj );
}

G3DObject*
MGLPort::examineHitBuffer( int hits, GLuint *ptr, G3DObject *obj )
{
  GLuint num_names, depth = (GLuint)-1, *picked = NULL;

  //printf("hits: %i\n", hits );
  for( int i = 0; i < hits; ++i )
    {
      //printf( "num %u, a: %u b: %u\n", *ptr, *(ptr+3), *(ptr+4) );
      num_names = *ptr++;
      //printf( "min %u, max %u\n", *ptr, *(ptr+1) );
      if( num_names && (*ptr < depth) )
	{
	  depth  = *ptr ;
	  picked = ptr-1;
	}
      ptr += num_names + 2;
    }

  if( picked )
    {
      //printf("anz: %i\n", (int)(*picked) );
      SelectData sdata( picked );
      obj->doAction( sdata );
      return( sdata.getSelection() );
    }
  else
    return( NULL );
}

G3DObject*
MGLPort::findElement( int x, int y, int mode, G3DGroup *start )
{
  if( scene_ )
    {
      /* old [fm]
	double aspect, top, right, dist;

      aspect = width() / height();
      top    = tan( deg2rad(camera_->zoomGet())/2 ) * _znear;
      right  = top * aspect;

      // the window coordinates x and y must be transformed into the
      // intervals [-right, right] and [-top, top]
      G3DRay *ray = camera_->buildRay( ((2*x/width())-1.0)*right,
				       (1.0-(2*y/height()))*top );
      */
      G3DRay ray;
      camera_->buildRay(x,y, ray); // the buildRay() method now takes pixel locations! [fm]
                                  
      G3DObject *obj = NULL;
      double dist;
      if( start )
	dist = start->intersect( &ray, &obj ); //( iter );
      else
	dist = scene_->intersect( &ray, &obj );

      if( dist < G3D_INFINITY )
	return( obj );
    }

  return( NULL );
}

/*---------------------------------------------------------------------------*/

G3DCamera*
MGLPort::getCamera( void )
{
  return( camera_ );
}

void
MGLPort::initShader( void )
{
/* TODO set the quality of the output:
	HINT( enum target, enum hint )
  target = PERSPECTIVE_CORRECTION_HINT; FONG_HINT, POLYGON_SMOOTH_HINT
  hint = DONT_CARE, NICEST, FASTEST
  enable / disable POLYGON_SMOOTH (aliasing)
  // TODO must be set in MGLPort?? glEnable( GL_DITHER );
  //  glDisable( GL_DITHER );
*/

  if( display_type == WIRE )
    {
      glPolygonMode( GL_FRONT_AND_BACK, GL_LINE );
      glShadeModel( GL_FLAT );
      glDisable( GL_DEPTH_TEST );
      glDisable( GL_CULL_FACE );
    }
  else  //FLAT and SMOOTH
    {
      glClearDepth( 1.0 );  //TODO use near value? copy from a buffer
      glClear( (GLbitfield)GL_DEPTH_BUFFER_BIT );
      glDepthFunc( GL_LESS );
      glEnable( GL_DEPTH_TEST );
      glEnable( GL_CULL_FACE );
      glCullFace( GL_BACK );

      glPolygonMode( GL_FRONT_AND_BACK, GL_FILL );
      if( display_type == FLAT )
	glShadeModel( GL_FLAT );
      else
	glShadeModel (GL_SMOOTH);
    }

  glEnable( GL_NORMALIZE );  //TODO realy? maybe it slows things down?
  //clockwise orientation of polygons
  glFrontFace( GL_CW ); //TODO is this the best choice?

  /*
  //TODO should be GL_DIFFUSE only ???
  glColorMaterial( GL_FRONT_AND_BACK, GL_AMBIENT_AND_DIFFUSE );
#ifdef SUNOPENGL 
  glMaterialfv( GL_FRONT_AND_BACK, GL_AMBIENT_AND_DIFFUSE, black ) ;
#endif
  glEnable( GL_COLOR_MATERIAL );
  */
}

/*---------------------------------------------------------------------------*/

void
MGLPort::focusInEvent ( QFocusEvent * )
{
}

void
MGLPort::focusOutEvent ( QFocusEvent * )
{
}

void
MGLPort::keyPressEvent( QKeyEvent* e )
{
  int sign = -1;

  switch( e->key() )
    {
    case Key_Delete:
      world_->deleteObject( NULL );   //delete selected object
      break;

    case Key_1:
      camControler_->setMode( MECameraControler::AllInOne );
    case Key_2:
      camControler_->setMode( MECameraControler::Pan );
    case Key_3:
      camControler_->setMode( MECameraControler::Orbit );
    case Key_4:
      camControler_->setMode( MECameraControler::Tuck );
    case Key_5:
       camControler_->setMode( MECameraControler::Rotate );
    case Key_6:
       camControler_->setMode( MECameraControler::Zoom );
    case Key_7:
       camControler_->setMode( MECameraControler::Roll );

    /* TODO use keys for exact movement
    case Key_Left:
      camera_->translate( 1,0,0 );
      break;
    case Key_Up:
      camera_->translate( 0,-1,0 );
      break;
    case Key_Right:
      camera_->translate( -1,0,0 );
      break;
    case Key_Down:
      camera_->translate( 0,1,0 );
      break;
    */

    case Key_Plus:
      sign = 1;
    case Key_Minus:
      if( _selected )
	{
	  ScaleControler scnt;  //TODO not connected
	  scnt.setSelectedObject( _selected );
	  scnt.transform( sign*0.2, sign*0.2, sign*0.2 );
	}
      else
	camera_->setFOV( camera_->getFOV() + (sign*2) );
      break;
    default:
      e->ignore();
      return;
    }
  updateGL();
}

void
MGLPort::mouseMoveEvent( QMouseEvent * e )
{
  if ( opMode_ == ViewMode )
    {
      camControler_->move( e, _mouse_button );
    }
  else if( opMode_ == ObjectMode )
    {
      if( _active && tControler_ ) //TODO test for controler is enough?
	tControler_->transform( e, _mouse_button );
    }
  updateGL();
}

void
MGLPort::mousePressEvent( QMouseEvent * e )
{
  _mouse_button = e->button();  // save mouse click state

  if( _mouse_button != RightButton )
    {
      if( opMode_ == ViewMode )
	camControler_->start( camera_, e );
      else   //( opMode_ == ObjectMode )
	{       
	  //TODO if( _collectPointMode )
	  G3DObject *sub = NULL;
	  G3DObject *tmp = _model->getNewObject();
	  if( tmp == NULL )
	    { //try to select an object in the scene
	      if( e->state() & ShiftButton  )
		{
		  sub = pickObject( e, scene_ ); //does not select widgets,...
		  if( sub )
		    {
		      _active = _selected = world_->addToSelection( sub );
		      setControler( e );
		      updateGL();
		    }
		    return; //???
		}
	      if( e->state() & ControlButton )
		{
		  _selected = pickObject( e, scene_ ); //does not select widget
		  if( _selected )
		    {
		      _selected = _selected->copy();
		      world_->addObject( _selected );
		    }
		}
	      else
		{
		  if( _selected && (_selected->getType() != T_SEL_GROUP) )
		    sub = pickSubObject( e, _selected );
		  if( sub == NULL )
		    _selected = pickObject( e, scene_ );
		}
	    }
          else  //we have a new object
	    {
	      _selected = tmp;
	      G3DPoint3d pnt( camera_->getLookAt() );
	      camera_->unproject( e->x(), e->y(), pnt );
	      _selected->setOrigin( pnt );
	    }

	  if( sub )
	    //TODO if( _active != _selected ) delete _active;
	    _active = sub;
          else
	    {
	      world_->selectObject( _selected );
	      _active = _selected;
	    }
	  if( _active )
	    {
	      //_selected->setSelectColor( _selectColor );
	      setControler( e );
	    }
          else
	    updateDisplayList();

	  updateGL();
	}
    }
  else
    {
      QPopupMenu *popup;
      QPoint     new_coors = mapToGlobal(e->pos());
      if( (opMode_ == ViewMode) || (_selected == NULL) )
	popup = _defaultPopup;
      else
	popup = _model->getPopupMenuFor( _selected );	//TODO active?
      if( popup != NULL )
	{
	  popup->move( new_coors.x(), new_coors.y() );
	  popup->show();
	}
    }
}

void
MGLPort::mouseReleaseEvent( QMouseEvent * e )
{
  if( _active )
    {
      //TODO cout << "release" << endl;
      if( tControler_ )
	{
	  tControler_->stop( world_ );
	  tControler_ = NULL;
	}
      if( _selected && (_selected->getType() & T_SHAPE) )
        {
	  _selected->adjust();
          //updateGL();
        }
      //save command
	//TODO update other viewPorts
	emit viewChanged();
    }
}

/*---------------------------------------------------------------------------*/

void
MGLPort::objectSelected( G3DObject *obj )
{
  if( obj != _selected )
    {
      _selected = obj;
      _active   = NULL;
      updateGL();
    }
}

void
MGLPort::paintGL()
{
  initShader();
  camera_->render(); // setup projection and modelview matrices

  if( scene_ )
    {
      resetLights();
      scene_->display( 1 ); //1 = First available OGL light number
    }

  //Must be after rendering of the scene, cause the scene sets the background
  drawGrid ();
  drawAxes ();

  //TODO write errors to console?
  int error;
  while (1)
    {
      error = glGetError();
      if (error == GL_NO_ERROR)
	break;
      fprintf (stderr, "GL error: %s\n", gluErrorString((GLenum)error));
    }
}

void
MGLPort::resetLights( void )
{
  // reset light sources:
  for( int i = 0; i < 8; i++ )
    glDisable( (GLenum)(GL_LIGHT0 + i) );

  //TODO make this configurable?
  //glLightModelfi(GL_LIGHT_MODEL_LOCAL_VIEWER, 1 );
  //glLightModelfi(GL_LIGHT_MODEL_TWO_SIDE, 1 );

  glEnable(GL_LIGHTING);
}

void
MGLPort::resizeGL( int width, int height )
{
  camera_->setWindowSize( width, height );

  //TODO adjust move buffers
}

void
MGLPort::setCameraType( G3DCamera::TViewMode type )
{
  camera_->setViewMode( type );
  /*TODO
  if( type == G3DCamera::Perspective )
      action_type = CAM_ALL_IN_ONE;
  else
      action_type = CAM_PARALLEL_ALL;
      */
}

void
MGLPort::setControler( QMouseEvent *e )
{
  tControler_ = _model->getControlerFor( _active );
  if( tControler_ )
    {
      tControler_->setSelectedObject( _active );
      tControler_->setCoordSys( camera_ );
      tControler_->start( e );
    }
}

void
MGLPort::setDisplayListMode( int mode )
{
  dispListMode_ = mode;
  if( mode == DirectMode )
    {
      DeleteListAction action;
      scene_->doAction( action );
    }
  else if( mode == DisplayListMode )
    updateDisplayList();
}

void
MGLPort::setOperationMode( int mode )
{
  opMode_ = mode;

  switch( opMode_ )
    {
    case ViewMode:
      setCursor( *hand_cursor );
      break;
    case ObjectMode:
      setCursor( arrowCursor );
      break;
    }
}

void
MGLPort::setWorld( MWorldObject *world )
{
  if( world )
    {
      if( world_ )
	  disconnect( world_ );
      world_  = world;
      delete camera_;
      camera_ = new G3DCamera( *(world->getCameras())->front() );
      camera_->setName( "temporary Camera" );
      camera_->setWindowSize( width(), height() );

      scene_  = world->getScene();
      connect( world, SIGNAL(objectSelected(G3DObject*)),
	       this, SLOT(objectSelected(G3DObject*)) );
      connect( world, SIGNAL(sceneChanged(G3DGroup*)),
	       this, SLOT(updateGL()) );
      updateDisplayList();
      updateGL();
    }
}

void
MGLPort::show_axes (bool yes_or_no)
{
  see_axes = yes_or_no;
  update ();
}

void
MGLPort::show_grid (bool yes_or_no)
{
  see_grid = yes_or_no;
  update ();
}

void
MGLPort::updateDisplayList()
{
  if( dispListMode_ == DisplayListMode )
    {
      cout << "compiling display list" << endl;
      initShader();
      scene_->compileList( 0 );
    }
}


//================================EMD====================================
