/*
 * Project : "MindsEye"
 * Copyright 1996 - 1997. All Rights Reserved.
 * Author(s) M. van Velsen, M. Kuball
 *
 * $Id: m_viewl.cpp,v 1.12 1998/07/23 20:16:23 source Exp $
 * 
 * 
 * COPYRIGHT
 * =========
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 * 
 * OVERVIEW
 * ========
 *
 *
 */

#include "include/config.h"

#include <qbitmap.h>
#include <qcursor.h>
#include <qkeycode.h>
#include <qlayout.h>

#include "3dlib/camera.h"
#include "3dlib/aggregate.h"
#include "3dlib/scene.h"
#include "kernel/world_obj.h"
#include "include/cursors.h"
#include "dialogs/camera/m_cameradlg.h"
#include "modeler/modeler.h"
#include "m_vmenu.h"
#include "m_viewl.h"



MViewPortLayout::MViewPortLayout( QWidget *parent, 
                                  const char *name ) : QFrame(parent, name)
{
  makePopups();

  //>------- create cursors for the viewports
  QBitmap *cb1 = new QBitmap(CURSOR_X, CURSOR_Y, hand_open_cur_bits, FALSE);
  QBitmap *cm1 = new QBitmap(CURSOR_X, CURSOR_Y, hand_open_msk_bits, FALSE);
  hand_cursor  = new QCursor( *cb1, *cm1 );

  init_views();
  re_layout (LAYOUT_A); //let's start with our LightWave "Modeler" view layout
}

MViewPortLayout::~MViewPortLayout (void)
{
  delete layout_popup;
  delete hand_cursor;
}


void
MViewPortLayout::centerCamera()
{
  activeView_->glPort_.centerCamera();
}

void
MViewPortLayout::changeDisplayAll( int new_dtype )
{
  view1->glPort_.change_display( new_dtype );
  view2->glPort_.change_display( new_dtype );
  view3->glPort_.change_display( new_dtype );
  view4->glPort_.change_display( new_dtype );
}

void
MViewPortLayout::changeDisplayLocal( int new_dtype )
{
  activeView_->glPort_.change_display( new_dtype );
}

G3DCamera*
MViewPortLayout::getActiveCamera( void )
{
  return( activeView_->glPort_.getCamera() );
}

/*---------------------------------------------------------------------------*/
void
MViewPortLayout::setMainInteractionMode( int m )
{
  view1->glPort_.setOperationMode( m );
  view2->glPort_.setOperationMode( m );
  view3->glPort_.setOperationMode( m );
  view4->glPort_.setOperationMode( m );
}

/*---------------------------------------------------------------------------*/

void
MViewPortLayout::init_views( void )
{
  view1 = new MViewPort( hand_cursor, layout_popup, this );
  view2 = new MViewPort( hand_cursor, layout_popup, this );
  view3 = new MViewPort( hand_cursor, layout_popup, this );
  view4 = new MViewPort( hand_cursor, layout_popup, this );

  connect( view1, SIGNAL( activateView(MViewPort*)),
	   this, SLOT(setActiveView(MViewPort*)) );
  connect( &(view1->glPort_), SIGNAL(viewChanged()),
	   this, SLOT(viewChanged()) );
  connect( view2, SIGNAL( activateView(MViewPort*)),
	   this, SLOT(setActiveView(MViewPort*)) );
  connect( &(view2->glPort_), SIGNAL(viewChanged()),
	   this, SLOT(viewChanged()) );
  connect( view3, SIGNAL( activateView(MViewPort*)),
	   this, SLOT(setActiveView(MViewPort*)) );
  connect( &(view3->glPort_), SIGNAL(viewChanged()),
	   this, SLOT(viewChanged()) );
  connect( view4, SIGNAL( activateView(MViewPort*)),
	   this, SLOT(setActiveView(MViewPort*)) );
  connect( &(view4->glPort_), SIGNAL(viewChanged()),
	   this, SLOT(viewChanged()) );

  gm = gm_00 = gm_01 = NULL;
}

void
MViewPortLayout::keyPressEvent( QKeyEvent *e )
{
  setCursor( waitCursor );  //TODO

  switch( e->key() )
    {  
    case Key_A:
      re_layout (LAYOUT_A);  break;
    case Key_B:
      re_layout (LAYOUT_B);  break;
    case Key_C:
      re_layout (LAYOUT_C);  break;  //TODO ctr-c does not work!
    case Key_D:
      re_layout (LAYOUT_D);  break;
    case Key_E:
      re_layout (LAYOUT_E);  break;
    case Key_F:
      re_layout (LAYOUT_F);  break;
    case Key_G:
      re_layout (LAYOUT_G);  break;
    case Key_H:
      re_layout (LAYOUT_H);  break;
    case Key_I:
      re_layout (LAYOUT_I);  break;
    case Key_J:
      re_layout (LAYOUT_J);  break;
    case Key_K:
      re_layout (LAYOUT_K);  break;
    case Key_L:
      re_layout (LAYOUT_L);  break;
    case Key_M:
      re_layout (LAYOUT_M);  break;
    case Key_N:
      re_layout (LAYOUT_N);  break;
    default:
      e->ignore();
    }

  setCursor( arrowCursor );
}

void
MViewPortLayout::makePopups()
{
  layout_popup = new MViewportPopup( this );
  connect( layout_popup,
	   SIGNAL(change_display (int)),
	   this,
	   SLOT(changeDisplayLocal(int)) );
  connect( layout_popup,
	   SIGNAL(changeCameraType(int)),
	   this,
	   SLOT(setCameraType(int)) );
  connect( layout_popup,
	   SIGNAL(centerCamera()),
	   this,
	   SLOT(centerCamera()) );
}

void
MViewPortLayout::re_layout( int new_layout )
{
  int viewNum = 4;

  view1->hide ();
  view2->hide ();
  view3->hide ();
  view4->hide ();

  delete gm_01;
  delete gm_00;
  delete gm;
  gm_01 = gm_00 = NULL;

    //layout = new_layout;
  switch( new_layout )
    {
    case LAYOUT_A :
      viewNum = 1;
      gm      = new  QBoxLayout( this, QBoxLayout::TopToBottom, 1, 1 );
      gm->addWidget( view1, 100 );
      break;

    case LAYOUT_B :
      gm    = new QBoxLayout( this, QBoxLayout::TopToBottom, 1, 1 );
      gm_00 = new QBoxLayout (QBoxLayout::LeftToRight);
      gm_01 = new QBoxLayout (QBoxLayout::LeftToRight);

      gm->addLayout (gm_00,100);
      gm->addLayout (gm_01,100);

      gm_00->addWidget (view1,0);
      gm_00->addWidget (view2,0);
      gm_01->addWidget (view3,0);
      gm_01->addWidget (view4,0);
      break;
 
    case LAYOUT_C :
      viewNum = 2;
      gm    = new QBoxLayout( this, QBoxLayout::TopToBottom, 1, 1 );
      //gm_00 = new QBoxLayout (QBoxLayout::LeftToRight);
      //gm_01 = new QBoxLayout (QBoxLayout::LeftToRight);

      //gm->addLayout (gm_00,100);
      //gm->addLayout (gm_01,100);
      gm->addWidget( view1, 100 );
      gm->addWidget( view2, 100 );
      break;

    case LAYOUT_D :
      viewNum = 2;
      gm    = new QBoxLayout( this, QBoxLayout::LeftToRight, 1, 1 );
      gm_00 = new QBoxLayout (QBoxLayout::TopToBottom);
      gm_01 = new QBoxLayout (QBoxLayout::TopToBottom);

      gm->addLayout (gm_00,100);
      gm->addLayout (gm_01,100);
      gm_00->addWidget (view1,0);
      gm_01->addWidget (view2,0);
      break;

    case LAYOUT_E:
      viewNum = 3;
      gm    = new QBoxLayout (this, QBoxLayout::LeftToRight, 1, 1 );
      gm_00 = new QBoxLayout (QBoxLayout::TopToBottom);
      gm_01 = new QBoxLayout (QBoxLayout::TopToBottom);

      gm->addLayout (gm_00,100);
      gm->addLayout (gm_01,100);
      gm_00->addWidget (view1,0);
      gm_00->addWidget (view2,0);
      gm_01->addWidget (view3,0);
      break;

    case LAYOUT_F :
      viewNum = 3;
      gm    = new QBoxLayout (this, QBoxLayout::TopToBottom, 1, 1 );
      gm_00 = new QBoxLayout (QBoxLayout::LeftToRight);
      gm_01 = new QBoxLayout (QBoxLayout::LeftToRight);

      gm->addLayout (gm_00,100);
      gm->addLayout (gm_01,100);
      gm_00->addWidget (view1,0);
      gm_01->addWidget (view2,0);
      gm_01->addWidget (view3,0);
      break;

    case LAYOUT_G :
      viewNum = 3;
      gm    = new QBoxLayout (this, QBoxLayout::LeftToRight, 1, 1 );
      gm_00 = new QBoxLayout (QBoxLayout::TopToBottom);
      gm_01 = new QBoxLayout (QBoxLayout::TopToBottom);

      gm->addLayout (gm_00,100);
      gm->addLayout (gm_01,100);
      gm_00->addWidget (view1,0);
      gm_01->addWidget (view2,0);
      gm_01->addWidget (view3,0);
      break;

    case LAYOUT_H :
      viewNum = 3;
      gm    = new QBoxLayout (this, QBoxLayout::TopToBottom, 1, 1 );
      gm_00 = new QBoxLayout (QBoxLayout::LeftToRight);
      gm_01 = new QBoxLayout (QBoxLayout::LeftToRight);

      gm->addLayout (gm_00,100);
      gm->addLayout (gm_01,100);
      gm_00->addWidget (view1,0);
      gm_00->addWidget (view2,0);
      gm_01->addWidget (view3,0);
      break;

    case LAYOUT_I :
      gm    = new QBoxLayout (this, QBoxLayout::LeftToRight, 1, 1 );
      gm_00 = new QBoxLayout (QBoxLayout::TopToBottom);

      gm->addLayout (gm_00,50);
      gm_00->addWidget (view1,0);
      gm_00->addWidget (view2,0);
      gm_00->addWidget (view3,0);
      gm->addWidget( view4, 100 );
      break;

    case LAYOUT_J :
      gm    = new QBoxLayout (this,QBoxLayout::TopToBottom, 1, 1 );
      gm_00 = new QBoxLayout (QBoxLayout::LeftToRight);
      gm_01 = new QBoxLayout (QBoxLayout::LeftToRight);

      gm->addLayout (gm_00,100);
      gm->addLayout (gm_01,50);
      gm_00->addWidget (view1,0);
      gm_01->addWidget (view2,0);
      gm_01->addWidget (view3,0);
      gm_01->addWidget (view4,0);
      break;

    case LAYOUT_K :
      gm    = new QBoxLayout (this, QBoxLayout::LeftToRight, 1, 1 );
      gm_00 = new QBoxLayout (QBoxLayout::TopToBottom);
      gm_01 = new QBoxLayout (QBoxLayout::TopToBottom);

      gm->addLayout (gm_00,100);
      gm->addLayout (gm_01,50);
      gm_00->addWidget (view1,0);
      gm_01->addWidget (view2,0);
      gm_01->addWidget (view3,0);
      gm_01->addWidget (view4,0);
      break;

    case LAYOUT_L :
      gm    = new QBoxLayout (this, QBoxLayout::TopToBottom, 1, 1 );
      gm_00 = new QBoxLayout (QBoxLayout::LeftToRight);
      gm_01 = new QBoxLayout (QBoxLayout::LeftToRight);

      gm->addLayout (gm_00,50);
      gm->addLayout (gm_01,100);
      gm_00->addWidget (view1,0);
      gm_00->addWidget (view2,0);
      gm_00->addWidget (view3,0);
      gm_01->addWidget (view4,0);
      break;

    case LAYOUT_M :
      viewNum = 2;
      gm    = new QBoxLayout (this, QBoxLayout::TopToBottom, 1, 1 );
      gm_00 = new QBoxLayout (QBoxLayout::LeftToRight);
      gm_01 = new QBoxLayout (QBoxLayout::LeftToRight);

      gm->addLayout (gm_00,50);
      gm->addLayout (gm_01,100);
      gm_00->addWidget (view1,0);
      gm_01->addWidget (view2,0);
      break;

    case LAYOUT_N :
      gm    = new QBoxLayout (this, QBoxLayout::LeftToRight, 1, 1 );
      gm_00 = new QBoxLayout (QBoxLayout::TopToBottom);
      gm_01 = new QBoxLayout (QBoxLayout::TopToBottom);

      gm->addLayout (gm_00,50);
      gm->addLayout (gm_01,100);
      gm_00->addWidget (view1,0);
      gm_01->addWidget (view2,0);
      break;
    }

  view1->show();
  if( viewNum > 1 )
    view2->show ();
  if( viewNum > 2 )
    view3->show ();
  if( viewNum > 3 )
    view4->show ();

  gm->activate ();
  update();
}
/*---------------------------------------------------------------------------*/
void
MViewPortLayout::setActiveView( MViewPort *active )
{
  activeView_ = active;
}

void
MViewPortLayout::setCaching( int mode )
{
  //TODO this is a per scene thing
  activeView_->glPort_.setDisplayListMode( mode );
}

void
MViewPortLayout::setCameraType( int mode )
{
  activeView_->glPort_.setCameraType( (G3DCamera::TViewMode)mode );
}

void
MViewPortLayout::setModel( MModeler *model )
{
  parentView_ = model;
  connect( model, SIGNAL(changed()), this, SLOT(getWorld()) );
  cout << "setModel" << endl;

  (view1->glPort_).setModel( model );
  (view2->glPort_).setModel( model );
  (view3->glPort_).setModel( model );
  (view4->glPort_).setModel( model );
}

void
MViewPortLayout::toggle_axes( bool val )
{
  view1->glPort_.show_axes (val);
  view2->glPort_.show_axes (val);
  view3->glPort_.show_axes (val);
  view4->glPort_.show_axes (val);
}

int
MViewPortLayout::toggle_caching( void )
{//TODO
  return( 0 );
}

void
MViewPortLayout::toggle_grid( bool val )
{
  view1->glPort_.show_grid (val);
  view2->glPort_.show_grid (val);
  view3->glPort_.show_grid (val);
  view4->glPort_.show_grid (val);
}

void
MViewPortLayout::getWorld( )
{
  MWorldObject *world = parentView_->getWorld();
  (view1->glPort_).setWorld( world );
  (view2->glPort_).setWorld( world );
  (view3->glPort_).setWorld( world );
  (view4->glPort_).setWorld( world );
}

void
MViewPortLayout::viewChanged()
{
  (view1->glPort_).updateGL();
  (view2->glPort_).updateGL();
  (view3->glPort_).updateGL();
  (view4->glPort_).updateGL();
}


//==================================END===================================
