/*
 * vi:ts=4:sw=4:
 *
 * Project : "MindsEye"
 * Copyright 1996. All Rights Reserved.
 * Author(s) M. van Velsen, M. Kuball
 *
 * $Id: m_viewl.h,v 1.11 1998/07/23 20:16:23 source Exp $
 *
 * 
 * COPYRIGHT
 * =========
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 * 
 * OVERVIEW
 * ========
 *
 *
 */
 
#ifndef __MVIEWPORT_L_H__
#define __MVIEWPORT_L_H__

#include <qframe.h>
#include "m_vdef.h"
#include "m_viewp.h"


#define LAYOUT_FIRST -1
#define LAYOUT_A      0
#define LAYOUT_B      1
#define LAYOUT_C      2 
#define LAYOUT_D      3
#define LAYOUT_E      4
#define LAYOUT_F      5
#define LAYOUT_G      6
#define LAYOUT_H      7
#define LAYOUT_I      8
#define LAYOUT_J      9
#define LAYOUT_K     10
#define LAYOUT_L     11
#define LAYOUT_M     12
#define LAYOUT_N     13


class QBoxLayout;
class QCursor;
class MViewportPopup;
class MModeler;
class G3DCamera;

/**A geometry manager for a quasi automatic layout of the
   3d view ports.
   The class knows about 14 predifined layouts which can be selected
   via a special dialog.
 */
class MViewPortLayout : public QFrame
{
    Q_OBJECT

public:
    ///
  MViewPortLayout( QWidget *parent=0, 
		   const char *name=0 );
  ~MViewPortLayout( );

    ///Get the current layout type.
  //int  get_layout (void) { return (layout); };
     ///
  G3DCamera* getActiveCamera( void );
    ///
  void re_layout (int); //TODO protected?
    ///Use the first viewPort to set the caching
  void setCaching( int mode );
    ///view or object
  void setMainInteractionMode( int );
    ///
  void setModel( MModeler* );
    ///Toggle the show axes flag on all 4 MGLPorts
  void toggle_axes    (bool);
    /**Toggle the caching of shapes in display lists.
       Returns the current mode.*/
  int  toggle_caching( void );
    ///Toggle the show grid flag on all 4 MGLPorts
  void toggle_grid    (bool);

public slots:
    ///
  void centerCamera();
    ///Change the display mode of all viewports.
  void changeDisplayAll( int );
    ///Change the display mode of the active viewport.
  void changeDisplayLocal( int );
    ///
  void setCameraType( int );
    ///
  void viewChanged();

protected:
    ///Initialize the 4 viewports.
  void init_views( void );
    ///
  void keyPressEvent( QKeyEvent* );
    ///Create some popup menus.
  void makePopups( void );

protected slots: //TODO maybe public
  void getWorld();
  void setActiveView( MViewPort* );

private:
   // int layout;
      ///
    MViewportPopup *layout_popup;
      /**The hand cursor is used by the MGLPorts in camera operation mode.
	 The variable allows for central management in this class.*/
    QCursor        *hand_cursor;
 
    QBoxLayout *gm;    // master layout
    QBoxLayout *gm_00; // row 1
    QBoxLayout *gm_01; // row 2
    
    MGLPort   *viewPort1, *viewPort2, *viewPort3, *viewPort4;
    MViewPort *view1;
    MViewPort *view2;
    MViewPort *view3;
    MViewPort *view4;
    MViewPort *activeView_;
    MModeler  *parentView_;
};


#endif // __MVIEWPORT_L_H__
