/*
 * vi:ts=4:sw=4:
 *
 * Project : "MindsEye"
 * Copyright 1996 - 1997. All Rights Reserved.
 * Author(s) M. van Velsen, M. Kuball
 *
 * $Id: m_viewm.cpp,v 1.14 1998/06/22 20:12:47 source Exp $
 * 
 * 
 * COPYRIGHT
 * =========
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 * 
 * OVERVIEW
 * ========
 *
 *
 */

#include "include/config.h"
#include <stdio.h>  //TODO remove this

#include <qpixmap.h>

#include "3dlib/camera.h"
#include "3dlib/aggregate.h"
#include "3dlib/scene.h"
#include "include/cursors.h"
#include "dialogs/camera/m_cameradlg.h"
#include "icons/md_icons.h"
#include "modeler/modeler.h"
#include "kernel/world_obj.h"
#include "m_viewm.h"
#include "m_viewl.h"

#include "m_vdef.h"
#include "m_viewp.h"


MViewPortManager::MViewPortManager( MModeler *pv, MViewPortLayout *viewLayout,
				    QWidget *parent, const char *name ) :
  QFrame(parent, name)
{
  setMinimumSize( 140, BUTTON_HIGHT+2 );
  setMaximumSize( 5000, BUTTON_HIGHT+2 );
  setFrameStyle( QFrame::Panel | QFrame::Raised );

  viewDialog_   = NULL;
  cameraDialog_ = NULL;
  viewLayout_   = viewLayout;
  parentView_   = pv;
        
  QBoxLayout *gm = new QHBoxLayout( this, 1, 1 );

  //>------- mouse coordinates
  coorLabel_ = new QLabel( this );
  coorLabel_->setFrameStyle (QFrame::WinPanel | QFrame::Sunken);
  coorLabel_->setBackgroundColor (darkGray);
  coorLabel_->setAlignment (AlignLeft | AlignVCenter);
  coorLabel_->setMinimumSize( 200, BUTTON_HIGHT );
  coorLabel_->move( 1, 1 ); 
  label_text[0] = '\0';
  coorLabel_->setText( label_text );
  gm->addWidget( coorLabel_, 1 );

  infoField_ = new QLabel( this );
  infoField_->setFrameStyle( QFrame::WinPanel | QFrame::Sunken );
  infoField_->setBackgroundColor( darkGray );
  infoField_->setAlignment( AlignLeft | AlignVCenter );
  infoField_->move( 1, 1 ); 
  infoField_->setMinimumSize( 150, BUTTON_HIGHT );
  gm->addWidget( infoField_, 100 );

  create_viewbuttons( gm );

  gm->activate ();

}

MViewPortManager::~MViewPortManager (void)
{
}

/*---------------------------------------------------------------------------*/

void
MViewPortManager::cameraDialog()
{
  if( cameraDialog_ == NULL )
    {  //TODO pass parentView to cameraDialog
      cameraDialog_ = new MCameraDialog( 0, 0 );
      //if( world_ )
      cameraDialog_->setCameraList( (parentView_->getWorld())->getCameras() );
      connect( cameraDialog_, SIGNAL(closeing()), this, SLOT(cameraDialog()) );
      cameraDialog_->show();
    }
  else
    cameraDialog_ = NULL;
}
/*---------------------------------------------------------------------------*/
void
MViewPortManager::change_layout()
{
  if( viewDialog_ == NULL ) //TODO set to Null when dialog is closed ( DONE )
    {
      viewDialog_ = new MViewportDialog( -1, 0, 0 );
	//TODO viewLayout_->get_layout(), 0, 0 );
      connect( viewDialog_, SIGNAL(do_layout(int)), 
	       viewLayout_, SLOT(re_layout(int)) );
      viewDialog_->show();
    }
}
/*---------------------------------------------------------------------------*/
void
MViewPortManager::create_viewbuttons( QBoxLayout *gm_01 )
{
  QPixmap *views_pixmap = new QPixmap ();

  views_pixmap->loadFromData (views_bmp_data, views_bmp_len);
  QPushButton *viewbtn = new QPushButton (this, 0);
  viewbtn->setPixmap (*views_pixmap);
  viewbtn->setMinimumSize( views_pixmap->width ()+4, BUTTON_HIGHT );
  QToolTip::add (viewbtn, "viewport configuration");
  connect (viewbtn,
	   SIGNAL (clicked ()),
	   this,
	   SLOT (change_layout ()));
  gm_01->addWidget( viewbtn, 0 );

  //>-------

  QPushButton *select = new QPushButton (this,0);
  select->setText( "Cam" );
  select->setMinimumSize( 30,BUTTON_HIGHT );
  QToolTip::add (select,"Camera dialog");
  connect( select, SIGNAL(clicked()), SLOT(cameraDialog()) );
  gm_01->addWidget (select,0);

  //>-------

  QPixmap *pan_pixmap = new QPixmap ();
  pan_pixmap->loadFromData (pan_bmp_data,pan_bmp_len);

  QPushButton *pan = new QPushButton (this,0);
  pan->setPixmap (*pan_pixmap);
  pan->setMinimumSize( pan_pixmap->width ()+4, BUTTON_HIGHT );
  pan->setToggleButton (TRUE);
  pan->setOn( TRUE );
  connect( pan,  SIGNAL (toggled (bool)),
	   this, SLOT (goto_moving (bool)));
  QToolTip::add (pan, "camera mode");
  gm_01->addWidget( pan, 0 );

  //>-------

  QPixmap *select_pixmap = new QPixmap ();
  select_pixmap->loadFromData (select_bmp_data,select_bmp_len);

  select = new QPushButton (this,0);
  select->setPixmap (*select_pixmap);
  select->setMinimumSize( select_pixmap->width ()+4, BUTTON_HIGHT );
  select->setToggleButton (TRUE);
  QToolTip::add (select,"object mode");
  connect (select,SIGNAL (toggled (bool)),
	   this,SLOT (goto_selecting (bool)));
  gm_01->addWidget( select, 0 );

  //>-------

  QButtonGroup *tmp_group = new QButtonGroup( NULL, "Dummy" );
  tmp_group->setExclusive( TRUE );
  tmp_group->insert( pan );
  tmp_group->insert( select );
}
/*---------------------------------------------------------------------------*/
void MViewPortManager::goto_moving (bool on)
{
  if (on == TRUE)
    viewLayout_->setMainInteractionMode( ViewMode );
}
/*---------------------------------------------------------------------------*/
void MViewPortManager::goto_selecting (bool on)
{
  if (on == TRUE)
    viewLayout_->setMainInteractionMode( ObjectMode );
}
/*---------------------------------------------------------------------------*/
void
MViewPortManager::setCoordinateText( const char *text )
{
  strncpy( label_text, text, 60 );
  label_text[59] = '\0';
  coorLabel_->setText( label_text );
}

/*---------------------------------------------------------------------------*/
void MViewPortManager::toggle_axes (bool on)
{
  viewLayout_->toggle_axes (on); 
}
/*---------------------------------------------------------------------------*/
void MViewPortManager::toggle_grid (bool on)
{
  viewLayout_->toggle_grid (on); 
}

/* TODO do we need this?
void
MViewPortManager::update( )
{
  world_ = world;
  if( cameraDialog_ && world_ )
    cameraDialog_->setCameraList( world_->getCameras() );
}
*/

//==================================END======================================
