/*
 * Project : "MindsEye"
 * Copyright 1996. All Rights Reserved.
 * Author(s) M. van Velsen
 *
 * $Id: m_vmenu.cpp,v 1.12 1998/07/23 20:16:24 source Exp $
 *
 *
 * COPYRIGHT
 * =========
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * OVERVIEW
 * ========
 *
 */

#ifdef HAVE_CONFIG_H
#include "include/config.h"
#endif	 // HAVE_CONFIG_H

#include "m_vdef.h"
#include "m_vmenu.h"
#include "3dlib/object.h"
#include "3dlib/t_objects.h"
#include "3dlib/camera.h"


MObjectPopup::MObjectPopup( G3DObject *obj, QWidget *modeler ) :
  QPopupMenu()
{
  //set some default items
  insertItem( "properties", modeler, SLOT(objProperties()) );
  insertItem( "delete", modeler, SLOT(deleteObject()) );
  //insertItem( "apply trafo", modeler, SLOT(applyTrafo()) );
  insertItem( "take focus", modeler, SLOT(takeFocus()) );
  insertItem( "select parent", modeler, SLOT(pmSelectParent()) );
  insertSeparator();
  insertItem( "hide", modeler, SLOT(hideObject()) );
  insertItem( "show", modeler, SLOT(showObject()) );
  if( obj->getType() == T_SEL_GROUP )
    {
      insertSeparator();
      insertItem( "group", modeler, SLOT(pmGroup()) );
    }
  else if( obj->getType() & T_GROUP )
    {
      insertSeparator();
      insertItem( "expand", modeler, SLOT(pmUngroup()) );
      insertItem( "toggle child access", modeler, SLOT(pmChildAccess()) );
    }

  //insertItem( "show normals", OBJ_NORMALS );
  /* TODO
  if( obj )
    {
      obj->getFunctionList();
    }
    */
}


/*---------------------------------------------------------------------------*/


MViewportPopup::MViewportPopup( QWidget *parent ) :
  QPopupMenu()
{
  render_popup = new QPopupMenu;
  CHECK_PTR(render_popup);
  render_popup->insertItem( "Wireframe", this, SLOT(render_wire()) );
  render_popup->insertItem( "Flat",      this, SLOT(render_flat()) );
  render_popup->insertItem( "Shaded",    this, SLOT(render_smooth()) );
  render_popup->insertItem( "Textured",  this, SLOT(render_smooth()) );

  QPopupMenu *parallel_popup = new QPopupMenu;
  connect( parallel_popup, SIGNAL(activated(int)),
	   this, SLOT(setParallel(int)) );
  parallel_popup->insertItem( "Top",    G3DCamera::Top );
  parallel_popup->insertItem( "Bottom", G3DCamera::Bottom);
  parallel_popup->insertItem( "Left",   G3DCamera::Left );
  parallel_popup->insertItem( "Right",  G3DCamera::Right );
  parallel_popup->insertItem( "Front",  G3DCamera::Front );
  parallel_popup->insertItem( "Back",   G3DCamera::Back );

  viewport_popup = new QPopupMenu;
  CHECK_PTR(viewport_popup);
  viewport_popup->insertItem( "perspective", this, SLOT(setPerspective()) );
  viewport_popup->insertItem( "parallel", parallel_popup );

  //TODO insertItem ("Redraw", this, SLOT(redraw()));
  //insertSeparator ();
  insertItem( "Camera type",  viewport_popup,0 );
  insertItem( "Display type", render_popup, 0 );
  insertSeparator();
  insertItem( "Center",this, SLOT(center()) );

  setCheckable (FALSE);
}

void
MViewportPopup::center()
{
  emit centerCamera();
}

void
MViewportPopup::redraw()
{
  emit redraw_viewport ();
}

void
MViewportPopup::setParallel( int id )  //TODO use unnamed enum
{
  emit changeCameraType( id );
}

void
MViewportPopup::setPerspective()
{
  emit changeCameraType( G3DCamera::Perspective );
}

void
MViewportPopup::render_flat()
{
  emit change_display (FLAT);
}

void
MViewportPopup::render_smooth()
{
  emit change_display (SMOOTH);
}

void
MViewportPopup::render_wire()
{
  emit change_display (WIRE);
}


//================================END===================================
