/*
 * Project : "MindsEye"
 * Copyright (c) 1996, 1997 by Martin Kuball. All Rights Reserved.
 * Author(s): M. Kuball, M. van Velsen, F. Meier
 * 
 * $Id: rcontroler.cpp,v 1.2 1998/07/23 20:16:24 source Exp $
 *
 *
 * COPYRIGHT
 * =========
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * OVERVIEW
 * ========
 *
 */

#include "include/config.h"
#include <stdio.h>
//include <stdlib.h>
#include <qkeycode.h>
#include <qwidget.h>

#include "3dlib/w_rotate.h"
#include "3dlib/camera.h"
#include "tcontroler.h"
#include "grid.h"



void
RotateControler::start( QMouseEvent *e )
{
  if( _selected && _selected->getType() == T_ROTATE_WIDGET )
    {
      G3DRotateWidget *wid = (G3DRotateWidget*)_selected;
      G3DPoint3d  center( wid->getCenter() );
      G3DVector3d axis;
      G3DMatrix   lmat, gmat;

      wid->getAxis( axis );  //based on the user selection
      wid->getLocalTrafo( lmat );
      wid->getWorldTrafo( gmat );
      gcenter_ = gmat * center;
      lcenter_ = center;
      lcenter_ = lmat * lcenter_;
      axis_    = lmat * axis;
      gaxis_   = gmat * axis;

      //cout << "ax: " << axis_  << "c: " << lcenter_ << endl;
      mouse_   = gcenter_;
      camera_->unproject( e->x(), e->y(), mouse_ );

      G3DVector3d e1, e2;
      wid->getCircle( e1, e2 );
      e1 = gmat * e1;
      e2 = gmat * e2;
      initAngle( mouse_, e1, e2 );
      origin_  = _selected->getOrigin();
      
      G3DVector3d angles( wid->getAngles() );
      mrot_.setToIdentity();
      mrot_.rotate( angles );

      emit setInfoText( "a = 0" );
    }
}

void
RotateControler::stop( MWorldObject* )
{
}

void
RotateControler::transform( QMouseEvent *e, int )
{
  doRotate( e->x(), e->y() );
}

void
RotateControler::initAngle( G3DVector3d &mouse, G3DVector3d &e1,
			    G3DVector3d &e2 )
{
  const G3DVector3d &up( camera_->getUp() );
  const G3DVector3d &r( camera_->getRight() );
  length_ = e1.length();
  e1 = up * e1.dot(up) + r * e1.dot(r);
  e2 = up * e2.dot(up) + r * e2.dot(r);
  if( (e1.cross( e2 )).length() < FG_EPSILON )
    {
      colinear_ = true;
      if( e1.length() > FG_EPSILON )
	{
	  radius_ = e1;
	}
      else
	radius_ = e2;
      radius_.normalize();

      //G3DVector3d v(mouse - gcenter_);
      angle0_ = (mouse - gcenter_).dot(radius_) / length_;
      if( angle0_ >= 1.0 )
	angle0_ = 0;
      else if( angle0_ <= -1.0 )
	angle0_ = M_PI;
      else
	angle0_ = acos( angle0_ );
      //cout << "  " << rad2deg(angle0_) << endl;
    }
  else
    colinear_ = false;
}

double
RotateControler::calcAngle( G3DVector3d &mouse )
{
  G3DVector3d v1(mouse - gcenter_);
  double      angle;

  if( colinear_ )
    {
      double r = v1.dot(radius_) / length_;
      if( r >= 1.0 )
	angle = 0.0;
      else if( r <= -1.0 )
	angle = M_PI;
      else
	angle = acos( r );
      angle = angle - angle0_;
    }
  else
    {
      G3DVector3d v2( mouse_ - gcenter_ );

      double ca = v1.dot( v2 ) / (v1.length() * v2.length());
      angle = acos( ca );
      if( (v2.cross( v1 )).dot(gaxis_) < 0 )
	angle = 2*M_PI - angle;
    }

  return( angle );
}

void
RotateControler::doRotate( int x, int y )
{
  G3DPoint3d mouse( gcenter_ );
  double     angle;

  camera_->unproject( x, y, mouse );
  angle = calcAngle( mouse );

  //grid for angle or angles;

  G3DVector3d angles, scale;
  G3DVector3d lcenter(lcenter_ * (-1));  
  G3DMatrix   rot;
  rot.rotate( axis_, angle );
  lcenter = rot * lcenter;

  rot *= mrot_;
  rot.getAngles( angles );
  _selected->setAngles( angles );
  //cout << rot;
  //cout << "a---------- " << angles << endl;
  _selected->setOrigin( (lcenter + lcenter_) + origin_ );
  _selected->getLocalTrafo( rot );
  //cout << rot << endl;

  char buffer[20];
  sprintf( buffer, "a = %.3f", rad2deg(angle) );
  emit setInfoText( buffer );
}


//================================END===================================
