/*
 * Project : "MindsEye"
 * Copyright (c) 1996, 1997 by Martin Kuball. All Rights Reserved.
 * Author(s): M. Kuball, M. van Velsen, F. Meier
 * 
 * $Id: scontroler.cpp,v 1.5 1998/07/23 20:16:24 source Exp $
 *
 *
 * COPYRIGHT
 * =========
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * OVERVIEW
 * ========
 *
 */

#include "include/config.h"
#include <stdio.h>
//include <stdlib.h>
#include <qkeycode.h>
#include <qwidget.h>

#include "3dlib/object.h"
#include "3dlib/camera.h"
#include "3dlib/w_scale.h"
#include "tcontroler.h"
#include "grid.h"


void
ScaleControler::start( QMouseEvent *e )
{
  _oldx = e->x();
  _oldy = e->y();
  _oldz = _oldy;

  if( _selected && (_selected->getType() == T_SCALE_WIDGET) )
    {
      G3DMatrix map2global;
      _selected->getWorldTrafo( map2global );
      _selected->getPosition( lfixed_ );
      ((G3DScaleWidget*)_selected)->getHandlePosition( mousepos_ );
      svector_   = mousepos_ - lfixed_;
      gfixed_    = map2global * lfixed_;
      mousepos_  = map2global * mousepos_;

      map2local_ = map2global;
      map2local_.invert3();

      _selected->getLocalTrafo( mlocal_ );
      scale_  = _selected->getScale();
      transl_ = _selected->getOrigin();

      char buffer[40];
      sprintf( buffer, "x: %.3f y: %.3f z: %.3f",
	       scale_.x(),scale_.y(),scale_.z() );
      emit setInfoText( buffer );
    }
}

void
ScaleControler::stop( MWorldObject* )
{
}

void
ScaleControler::transform( QMouseEvent *e, int button )
{
  if( button == LeftButton )
    doScale( e->x(), e->y(), _oldz );
  else
    doScale( e->x(), _oldy, e->y() );
//TODO if alt button, _selected->addWidget( new ConstraintW )
}

void
ScaleControler::transform( float delx, float dely, float delz )
{
  if( _selected )
    {
      G3DVector3d scale( _selected->getScale() );
      G3DVector3d transl(_selected->getOrigin() );
      G3DVector3d delta( delx, dely, delz );
      G3DPoint3d  pos;

      _selected->getLocalTrafo( mlocal_ );
      _selected->getPosition( pos );

      G3DMatrix scaler;
      scaler.scale( (scale + delta).cwDivide(scale) );
      scaler = mlocal_ * scaler;
      scale += delta;

      _selected->setScale( scale );
      _selected->setOrigin( (mlocal_*pos) - (scaler*pos) + transl );

      char buffer[40];
      sprintf( buffer, "x: %.4f y: %.4f z: %.4f",
	       scale.x(),scale.y(),scale.z() );
      emit setInfoText( buffer );
    }
}

void
ScaleControler::doScale( int x, int y, int z )
{
  if( _selected )
    {
      camera_->unproject( x, y, mousepos_ );
  
      G3DVector3d bound(0.01, 0.01, 0.01); //don't make the object smaller
      G3DVector3d svector( mousepos_ - gfixed_ );

      svector = map2local_ * svector;
      svector.cwDivide( svector_ );
      svector.max( bound );

      G3DMatrix scaler;
      scaler.scale( svector );
      scaler = mlocal_ * scaler;
      svector.cwMultiply( scale_ );  //TODO grid

      _selected->setScale( svector );
      _selected->setOrigin( (mlocal_*lfixed_) - (scaler*lfixed_)
				 + transl_ );

      char buffer[40];
      sprintf( buffer, "x: %.3f y: %.3f z: %.3f",
	       svector.x(),svector.y(),svector.z() );
      emit setInfoText( buffer );
    }
}


//================================END===================================
